;These programs handle changes to the rois.
;Revision History
; 980729 ASB Removed code from analyze_roi.pro
; 980811 ASB Added code to change sensitivity of some buttons.
; 980915 ASB Added common initcsi and removed all system variables.
; 980916 ASB Fixed bug in stat window updating.
; 981009 ASB Added code to compute ROI statistices from time domain
;            analyzed data.
; 990302 ASB Fixed bugs in handling of noise.
;            Added code for time domain fitting.
;            Improved code for writing roi analysis in files.
; 990406 ASB Added code to display water map and values.
; 990914 ASB Corrected bug in stattitles for td results.
; 990915 ASB Moved initialization of stattitles to common_list.
;            Changed logic for updating stats window during toggle
;            between TD and FD.
; 990924 ASB Fix bug in display of ROIs on guide images.
; 000525 ASB Enlarged stats to include water calculated 6 ways.

pro asb_defroi_update
;This procedure updates the statistics
COMMON posdata
COMMON defroi
COMMON mets
COMMON roistats
COMMON noise
COMMON initcsi
COMMON mrsidata
COMMON ctable

widget_control,s.tf_w,get_uvalue=flag
pixnoise=rmsnoise*sqrt(float(2*ir+1))
noise=pixnoise/sqrt(count)

;IF s.ustat EQ 0 AND s.smode EQ 0 THEN RETURN
IF count NE 0 THEN BEGIN
;Make test spectra button sensitive
;    widget_control,s.run_ids(1),sensitive=1

    widget_control,s.but_ids(2),sensitive=1 ;make statistics button window sensitive

    IF flag EQ 1 THEN BEGIN ;Time domain
        stats.cho_av=total(cho_td(t32))/count
        stats.cho_sd=sqrt((total(cho_td(t32)^2)-count*stats.cho_av^2)/(count-1))

        stats.cre_av=total(cre_td(t32))/count
        stats.cre_sd=sqrt((total(cre_td(t32)^2)-count*stats.cre_av^2)/(count-1))

        stats.naa_av=total(naa_td(t32))/count
        stats.naa_sd=sqrt((total(naa_td(t32)^2)-count*stats.naa_av^2)/(count-1))

        stats.lac_av=0.
        stats.lac_sd=0.

        stats.ino_av=0.
        stats.ino_sd=0.

; Update pixel list
        xx=t32 mod pe_dim(0)
        yy=t32 / pe_dim(0)
        kk=0
        statstring1=string(format='(i2,",",i2,7f10.2)',$
                     xx(kk),yy(kk),cho_td(t32(kk)),xm(xx(kk),yy(kk),12),$
                     cre_td(t32(kk)),xm(xx(kk),yy(kk),11),$
                     naa_td(t32(kk)),xm(xx(kk),yy(kk),10),$
                     xm(xx(kk),yy(kk),13) )
        FOR kk = 1,count-1 DO BEGIN
            statstring1=statstring1+string(byte(10))+$
              string(format='(i2,",",i2,7f10.2)',$
                     xx(kk),yy(kk),cho_td(t32(kk)),xm(xx(kk),yy(kk),12),$
                     cre_td(t32(kk)),xm(xx(kk),yy(kk),11),$
                     naa_td(t32(kk)),xm(xx(kk),yy(kk),10),$
                     xm(xx(kk),yy(kk),13) )
        ENDFOR
; Update roi summary
        statstring2=string(format='("ROI Av",f9.2,5f10.2)',$
                           stats.cho_av,0.0,stats.cre_av,0.0,$
                           stats.naa_av,0.0)+$
          string(byte(10))+string(format='("ROI SD",f9.2,5f10.2)',$
                                  stats.cho_sd,0.0,$
                                  stats.cre_sd,0.0,$
                                  stats.naa_sd,0.0)
    ENDIF ELSE BEGIN ;frequency domain
        sumpeaks

        stats.cho_av=total(cho(t32))/count
        stats.cho_sd=sqrt((total(cho(t32)^2)-count*stats.cho_av^2)/(count-1))

        stats.cre_av=total(cre(t32))/count
        stats.cre_sd=sqrt((total(cre(t32)^2)-count*stats.cre_av^2)/(count-1))

        stats.naa_av=total(naa(t32))/count
        stats.naa_sd=sqrt((total(naa(t32)^2)-count*stats.naa_av^2)/(count-1))

        stats.lac_av=total(lac(t32))/count
        stats.lac_sd=sqrt((total(lac(t32)^2)-count*stats.lac_av^2)/(count-1))

        stats.ino_av=total(ino(t32))/count
        stats.ino_sd=sqrt((total(ino(t32)^2)-count*stats.ino_av^2)/(count-1))

        npix=n_elements(naa)
        stats.h2oI1_av=total(h2o(t32))/count
        stats.h2oI1_sd=sqrt((total(h2o(t32)^2)-count*stats.h2oI1_av^2)/(count-1))

        stats.h2oI2_av=total(h2o(t32+npix))/count
        stats.h2oI2_sd=sqrt((total(h2o(t32+npix)^2)-count*stats.h2oI2_av^2)/(count-1))

        stats.h2oM1_av=total(h2o(t32+2*npix))/count
        stats.h2oM1_sd=sqrt((total(h2o(t32+2*npix)^2)-count*stats.h2oM1_av^2)/(count-1))

        stats.h2oM2_av=total(h2o(t32+3*npix))/count
        stats.h2oM2_sd=sqrt((total(h2o(t32+3*npix)^2)-count*stats.h2oM2_av^2)/(count-1))

        stats.h2oP1_av=total(h2o(t32+4*npix))/count
        stats.h2oP1_sd=sqrt((total(h2o(t32+4*npix)^2)-count*stats.h2oP1_av^2)/(count-1))

        stats.h2oP2_av=total(h2o(t32+5*npix))/count
        stats.h2oP2_sd=sqrt((total(h2o(t32+5*npix)^2)-count*stats.h2oP2_av^2)/(count-1))

; Update pixel list
        xx=t32 mod pe_dim(0)
        yy=t32 / pe_dim(0)
        statstring1=string(format='(i2,",",i2,12f10.2)',$
                           xx(0),yy(0),cho(t32(0)),cre(t32(0)),$
                           naa(t32(0)),lac(t32(0)),ino(t32(0)),$
                           h2o(t32(0)),h2o(t32(0)+npix),h2o(t32(0)+2*npix),$
                           h2o(t32(0)+3*npix),h2o(t32(0)+4*npix),h2o(t32(0)+5*npix),$
                           pixnoise)
        FOR kk = 1,count-1 DO BEGIN
            statstring1=statstring1+string(byte(10))+$
              string(format='(i2,",",i2,12f10.2)',$
                     xx(kk),yy(kk),cho(t32(kk)),cre(t32(kk)),naa(t32(kk)),$
                     lac(t32(kk)),ino(t32(kk)),$
                     h2o(t32(kk)),h2o(t32(kk)+npix),h2o(t32(kk)+2*npix),$
                     h2o(t32(kk)+3*npix),h2o(t32(kk)+4*npix),h2o(t32(kk)+5*npix),$
                     pixnoise)
                     
        ENDFOR
; Update roi summary
        statstring2=string(format='("ROI Av",f9.2,10f10.2)',$
                           stats.cho_av,stats.cre_av,$
                           stats.naa_av,stats.lac_av,stats.ino_av,$
                           stats.h2oI1_av,stats.h2oI2_av,stats.h2oM1_av,$
                           stats.h2oM2_av,stats.h2oP1_av,stats.h2oP2_av,$
                           noise)$
          +string(byte(10))+string(format='("ROI SD",f9.2,10f10.2)',stats.cho_sd,$
                                   stats.cre_sd,stats.naa_sd,$
                                   stats.lac_sd,stats.ino_sd,$
                                   stats.h2oI1_sd,stats.h2oI2_sd,$
                                   stats.h2oM1_sd,stats.h2oM2_sd,$
                                   stats.h2oP1_sd,stats.h2oP2_sd)
                           
        
    ENDELSE                     ;Frequency domain

    IF s.smode EQ 1 THEN update_specplot ;Update spectrum plot if necessary

    IF xregistered('mkroistats') THEN BEGIN ;Redraw statistics window
        widget_control,width_w,map=1-flag
        widget_control,stats_w,tlb_set_title=stattitles(flag)
        widget_control,pixvalues_w,set_value=statstring1
        widget_control,num_w,set_value=count
        widget_control,roivalues_w,set_value=statstring2
        widget_control,ctitles_w,set_value=' '+stattitles(flag+2)
    ENDIF
ENDIF ELSE BEGIN
    widget_control,s.but_ids(2),sensitive=0 ;make statitstics button window insensitive
    IF xregistered('mkroistats') THEN widget_control,stats_w,/destroy
    count=0
    widget_control, s.num1_w, set_value=0
    
    stats.cre_av=0.
    stats.cre_sd=0.
    stats.cho_sd=0.
    stats.cho_av=0.
    stats.cho_sd=0.
    stats.naa_av=0.
    stats.naa_sd=0.
    stats.lac_av=0.
    stats.lac_sd=0.
    stats.ino_av=0.
    stats.ino_sd=0.
ENDELSE

END

;This program defines a new ROI
pro asb_defroi_nmode,old=old
COMMON posdata
COMMON defroi
COMMON roistats

; Set new mode... Save old roi by concatenating it with s.subs.
n = s.npts
IF (s.mode NE 2) AND (n LE 2) THEN n = 0  ;must have 3 pnts for polygon
s.amode = 0

widget_control, s.subs, GET_UVALUE=t ;Prev roi pnts
IF n GE 1 THEN begin       ;Old region to save?
        asb_defroi_draw, -1, FILL=3
        widget_control, s.xy_pnts, GET_UVALUE=xy  ;Get old ROI
        xy = polyfillv(xy(0,0:n-1),xy(1,0:n-1),s.image_size(0), s.image_size(1))
        ;Concatenate s and xy
    IF total(t) EQ -1 THEN t=xy else t=[t,xy]
    widget_control, s.subs, SET_UVALUE=t

ENDIF

IF total(t) NE -1 THEN BEGIN  ;t sums to -1 only if no roi exists
;sensitize Clear All and Save ROI
    widget_control,s.optbut_ids(1),sensitive=1 ;Clear all
    widget_control,s.save_ids(3),sensitive=1 ;Save ROI

;Compute number of pixels in ROI
    dum256=intarr(256,256)
    dum256(t)=1
;add code for shift?
    dum32=reform(dum256,8,32,8,32)
    dum32=total(total(dum32,1),2)
    widget_control,s.thresh_w,get_value=thresh
    t32=where(dum32 GE thresh(0)*64/100,count) ;s.npts is # of spectroscopy voxels
;    widget_control,s.num_w,set_uvalue=index,set_value=count
    widget_control,s.num1_w,set_value=count
    asb_defroi_draw, -1,fill=3
;Compute roi statistics
    asb_defroi_update
ENDIF ELSE clear_all
;widget_control, s.subs, set_UVALUE=t
              ;Old region to save

IF NOT keyword_set(old) THEN s.npts=0

END

PRO asb_DEFROI_DRAW, i, FILL = fill
; Draw the outline (or polygon if FILL is set) 
; of the region or the ith segment if i < 0.
; Use the XOR drawing mode.
COMMON defroi
COMMON dsettings
COMMON roistats
COMMON posdata
COMMON ctable

n = s.npts
IF n LT 1 AND NOT keyword_set(fill) THEN RETURN
wins=[s.gwins,s.mwins]
nw=n_elements(wins)
DEVICE, SET_GRAPHICS=6          ;Xor drawing mode   
col = 192
;WHILE col LT !d.table_size DO col = col + col
widget_control, s.xy_pnts, GET_UVALUE=xy,/hourglass  ;Get ROI
xsave = !x.s & ysave = !y.s     ;Set scaling to pixel coords
;  p = float([!d.x_size, !d.y_size])
p=float([256,256])
f = 0. / p
q = 1. / p
!x.s = [f(0), q(0)]
!y.s = [f(1), q(1)]

IF s.mode EQ 2 THEN BEGIN       ;Point mode?
    IF i LT 0 THEN BEGIN
        i = 0 & i1 = n-1
    ENDIF ELSE i1 = i
    FOR j = i, i1 DO BEGIN
        polyfill, xy(0,j) + [0, 1, 1,0], xy(1,j) + [0,0,1,1], COLOR=col
        FOR kk = 0,n_elements(s.gwins)-1 DO BEGIN
            wset,s.gwins(kk)
            polyfill, xy(0,j) + [0, 1, 1,0], xy(1,j) + [0,0,1,1], COLOR=col
        ENDFOR
        wset,s.gwins(0)
    ENDFOR
ENDIF ELSE BEGIN            ;Polygon/circle/rect
    IF n GE 2 THEN BEGIN
        IF i LT 0 THEN BEGIN
            FOR kk = 0,nw-1 DO BEGIN
                wset,wins(kk)
                plots, COLOR=col, xy(*, 0:n-1)+.5 ;All of it?
            ENDFOR
        ENDIF ELSE BEGIN
            FOR kk = 0,nw-1 DO BEGIN
                wset,wins(kk)
                plots, COLOR=col, xy(*, i:i+1)+.5 ;One segment
            ENDFOR
        ENDELSE
    ENDIF
ENDELSE
IF keyword_set(fill) OR s.mode EQ 2 THEN BEGIN
;fill=1, do mets only, fill=2 do mris only, fill=3 do all images
    IF NOT keyword_set(fill) THEN fill=1
    IF fill GE 2 THEN BEGIN ;fill rois on guide images
        widget_control,s.subs,get_uvalue=t
        FOR kk = 0,n_elements(s.gwins)-1 DO BEGIN
            wset,s.gwins(kk)
            DEVICE, SET_GRAPHICS=3 ;Copy mode   
            DEVICE, copy = [0,0, !d.x_size, !d.y_size, 0, 0, $
                            s.gbacking(kk)]
            IF dispflag THEN BEGIN
                dumarr=backup(*,*,kk+n_elements(s.mwins))            
            ENDIF ELSE BEGIN
                dumarr=tvrd()
            ENDELSE
            IF (size(t))(0) EQ 1 THEN dumarr(t)=coffset1+dumarr(t)
            tv,dumarr
        ENDFOR
    ENDIF

    IF (fill MOD 2) EQ 1 THEN BEGIN ;fill rois on metabolite maps
;        widget_control,s.num_w,get_uvalue=index
        dumarr=intarr(32,32)
        IF (size(t32))(0) EQ 1 THEN dumarr(t32)=1
        dumarr=rebin(dumarr,256,256,/samp)
        t=where(dumarr EQ 1)
        FOR kk = 0,n_elements(s.mwins)-1 DO BEGIN
            wset,s.mwins(kk)
            DEVICE, SET_GRAPHICS=3 ;Copy mode   
            DEVICE, copy = [0,0, !d.x_size, !d.y_size, 0, 0, $
                            s.mbacking(kk)]
            IF dispflag THEN BEGIN
                dumarr=backup(*,*,kk)            
            ENDIF ELSE BEGIN
                dumarr=tvrd()
            ENDELSE

            IF (size(t))(0) EQ 1 THEN BEGIN
                IF tmode THEN BEGIN
                    dumarr(t)=coffset1+dumarr(t)
                ENDIF ELSE BEGIN
                    dumarr(t)=dumarr(t)-coffset1
                ENDELSE
            ENDIF
            tv,dumarr
        ENDFOR
    ENDIF
ENDIF
wset,s.gwins(0)
    

!x.s = xsave & !y.s = ysave
widget_control, s.xy_pnts, SET_UVALUE=xy  ;Set ROI   
DEVICE, SET_GRAPHICS=3          ;Copy mode   
END   
