FUNCTION auto_mp_pick 

;  910605  GSS  Added lactate offset.  Fixed choline and creatine offsets.
;  911204  JRA  Start beefing it up to do a better automatic job
;               to reduce the time needed in peak editing.
;  911227  JRA  roughly optimized on nov2991nv/P34304.dry 
;  920404  HHS  modified from auto_pick.pro
;		now will not attempt to pick peaks outside of ROI
;		resulting in speed up peak picking by factor of 2
;		ROI determined from info.pro which queries users
;		ROI stored in system variables
;  920916  JRA  Modified for picking 2T omega csi datsets (short te)
;                Very rough optimization on ja0723n
; 930329   JRA modified for picking signa multiple slice data sets.
; 970718   ASB Added code to read 3dcsi data.   
; 980407   Qin Added IDL win to show:  Plot Peak Position Histogram:
; 980915   ASB Replaced !csi_dat with fd_dat in COMMON freqdomain.
; 980915   ASB Added COMMON initcsi and removed system variables.
; 980916   ASB Removed reference to !spectro and !process.
; 980916   ASB Changed name of COMMON freqdomain to mrsidata.
; 981117   ASB Removed reference to s.psd_w
; 981120   ASB Added code so program doesn't bomb when no valid lines
;              are found.
; 990624   ASB Added COMMON mets, changed chemical shifts to elements
;              in common block, to agree with chemical shifts used in
;              other subroutines.
; 990626   ASB Increased values of min_water_snr and max_naa snr.
;              Added code for debugging.
;              Removed print statements.
 
COMMON mrsidata
COMMON defroi
COMMON initcsi
COMMON mets

; Constants
num_pts = spc_len
center_freq = spec_freq 
pt_per_hz = float(num_pts/sweep_width)

;
; Good assignment factors
field_guess_fac = 2.00
max_in_range_fac = 2.0
loc_max_fac = 2.0
found_cho_fac =2.0 
found_cre_fac = 2.0
found_wat_fac = 2.0
found_lac_fac = 1.0
found_fat_fac = 1.0

; Mistaken assignment factors
too_low_fac = 0.5
too_high_fac = 0.5
too_broad_fac = 0.5
too_narrow_fac = 1.0
too_close_fac = 0.5


; assignment criteria 
;  Most likely point assuming water suppression dx is adjusted
most_likely = long((num_pts/2.0) + (pt_per_hz*center_Freq*(h2o_shift-naa_shift)))
;print, 'Most Likely Point = ', most_likely

;  Number of points below a local maximum
loc_max_def = 4 

;  Range over which to make a noise evaluation
noise_left = 0
noise_right = 32

;  Minimum Approximate peak SNRs for defining they are there 
min_cho_snr = 4.0
min_cre_snr = 4.0
min_wat_snr = 100.0
min_naa_snr = 2.0
min_lac_snr = 2.0
min_tot_snr = 4.0
min_fat_snr = 35.0

;  Maximum peak SNRs for establishing incorrect assignments
max_naa_snr = 100.0

;  Maximum peak widths for establishing incorrect assignments
max_naa_width = 7               ; points 
min_naa_width = 2 

;  maximum range over which NAA will be found
;  nominally 2.5 ppm to 1.5 ppm 
left = fix((num_pts/2.0)+((h2o_shift-2.5)*center_freq*pt_per_hz)) ; points
right = fix((num_pts/2.0)+((h2o_shift-1.5)*center_freq*pt_per_hz)) ; points 

;  offsets from naa in pts   
cho_offset = (naa_shift - cho_shift)*center_freq*pt_per_hz 
cre_offset = (naa_shift - cre_shift)*center_freq*pt_per_hz 
lac_offset = (((naa_shift - lac_shift)*center_freq)+ 3.5)*pt_per_hz 
wat_offset = (naa_shift - h2o_shift)*center_freq*pt_per_hz 
fat_offset = (naa_shift - 1.20)*center_freq*pt_per_hz 

;   min distances for discriminating against assignments to end
min_dist=5

naa_pos = lonarr( sp_dim(0), sp_dim(1) )
;	now set the naa_pos outside of ROI+3 voxels to -1
portion = 1
x_ctr = FIX(sp_dim(0) / 2)
y_ctr = FIX(sp_dim(1) / 2)
x_len = FIX(x_ctr * portion + 2)
y_len = FIX(y_ctr * portion + 2)
IF (x_len LT x_ctr) AND (y_len LT y_ctr) THEN BEGIN
                                ;	Don't bother peak picking outside of ROI+2 voxel
    naa_pos(0:x_ctr-x_len-1,*) = -1
    naa_pos(x_ctr+x_len:sp_dim(0)-1,*) = -1
    naa_pos(*,0:y_ctr-y_len-1) = -1
    naa_pos(*,y_ctr+y_len:sp_dim(1)-1) = -1
                                ;	but we will save the left-posterior corner for background check
    naa_pos(0:2,0:2) = 0
ENDIF

n = right - left + 1

score = fltarr(num_pts)         ; score for the entire spectrum to make indexing easy
lines_found = 0

; Sort through spectra in CSI data set
FOR j = 0, sp_dim(1)-1 DO BEGIN
    FOR i = 0, sp_dim(0)-1 DO BEGIN            
        IF naa_pos(i,j) GE 0 THEN BEGIN ; peak pick only if pos not -1
            score(0:num_pts-1) = 1.0
            score(most_likely) = field_guess_fac

                                ;load the spectrum
            cur_spec = abs( fd_dat( 0:num_pts-1, i, j ) )

                                ;Determine characteristic RMS noise
            noise = 0.0
            FOR k = noise_left, noise_right DO $ 
              noise = noise + (cur_spec(k)*cur_spec(k))
            noise = sqrt(noise/(noise_right-noise_left))
                                ;print,"noise = ", noise


                                ; eliminate the spectra that are all noise
            IF max(cur_spec(left:right)) LT min_tot_snr*noise THEN BEGIN
                best_point = -1
;                print,"....... all noise "
            ENDIF ELSE BEGIN    ; determine the score matrix over range
                best_point=-1
                max_score=0.0
                
                                ; count through the points 
                zhood = fltarr(loc_max_def+1)
                FOR k = right, left, -1 DO BEGIN
                                ; Identify local maxima
;                IF i EQ 16 AND j EQ 14 THEN BEGIN
;                    tst=1
;                    stop
;                ENDIF ELSE tst=0
                    tst=0
                    zhood = cur_spec(k-(loc_max_def/2):k+(loc_max_def/2))
                    IF cur_spec(k) EQ max(zhood) $
                      AND cur_spec(k) GT 10*min_naa_snr THEN BEGIN 
                        score(k) = score(k) * loc_max_fac
                                ;discriminate against peaks that are too broad
                                ; or too narrow
                        IF tst THEN print,k,' loc_max_fac',score(k)
                        width = 0
                                ;  print,"k=",k
                        REPEAT BEGIN
                            width = width + 1 
                            IF width GT k THEN GOTO, bagdout 
                        ENDREP UNTIL cur_spec(k-width) LE 0.5*cur_spec(k)
                                ;   print,"OK-OK"
                        bagdout:IF width GT max_naa_width THEN $
                          score(k) = score(k) * too_broad_fac
                        IF width LT min_naa_width THEN BEGIN
                            score(k) = score(k) * too_narrow_fac
                            IF tst THEN print,k,' too_narrow_fac',score(k)
                        ENDIF
                    ENDIF       ;found local max
                    
                                ; Search for concurring assigments for other peaks
                    IF cur_spec(k+fat_offset)/noise GT min_fat_snr THEN BEGIN
                        score(k) = score(k) * found_fat_fac
                        IF tst THEN print,k,' found_fat_fac',score(k)
                        zhood = cur_spec(k+fat_offset - (loc_max_def/2) : $
                                         k + fat_offset + (loc_max_def/2))
                        IF cur_spec(k+fat_offset) EQ max(zhood) THEN BEGIN
                            score(k) = score(k) * found_fat_fac
                            score(k) = score(k) * found_fat_fac
                        ENDIF
                    ENDIF       ; fat concurrence

                    IF cur_spec(k+cre_offset)/noise GT min_cre_snr THEN BEGIN
                        score(k) = score(k) * found_cre_fac
                        IF tst THEN print,k,' found_cre_fac',score(k)
                    ENDIF
                    IF cur_spec(k+lac_offset)/noise GT min_lac_snr THEN BEGIN
                        score(k) = score(k) * found_lac_fac
                        IF tst THEN print,k,' found_lac_fac',score(k)
                    ENDIF
                    IF cur_spec(k+wat_offset)/noise GT min_wat_snr THEN BEGIN
                        score(k) = score(k) * found_wat_fac
                        IF tst THEN print,k,' found_wat_fac',score(k)
                    ENDIF
                    IF cur_spec(k+cho_offset)/noise GT min_cho_snr THEN BEGIN 
                        score(k) = score(k) * found_cho_fac
                        IF tst THEN print,k,' found_cho_fac',score(k)
                        zhood = cur_spec(k+cho_offset - (loc_max_def/2) : $
                                         k + cho_offset + (loc_max_def/2))
                        IF cur_spec(k+cho_offset) EQ max(zhood) THEN BEGIN
                            score(k) = score(k) * found_cho_fac
                            IF tst THEN print,k,' found_cho_fac',score(k)
                        ENDIF
                        IF cur_spec(k+cre_offset) GT min_cre_snr THEN BEGIN
                            score(k) = score(k) * found_cho_fac
                            IF tst THEN print,k,' found_cho_fac1',score(k)
                        ENDIF
                    ENDIF       ; choline concurrence

                                ; Give some weight to the max point in range
                    IF cur_spec(k) EQ max(cur_spec(left:right)) THEN BEGIN
                        score(k) = score(k) * max_in_range_fac
                        IF tst THEN print,k,' max_in_range_fac',score(k)
                    ENDIF
                                ; discriminate against peaks that are too big
                    IF cur_spec(k)/noise GT max_naa_snr THEN BEGIN
                        score(k) = score(k) * too_high_fac
                        IF tst THEN print,k,' too_high_fac',score(k)
                    ENDIF
                                ; discriminate against peaks that are too small
                    IF cur_spec(k)/noise LT min_naa_snr THEN BEGIN
                        score(k) = score(k) * too_high_fac
                        IF tst THEN print,k,' too_high_fac1',score(k)
                    ENDIF

                                ;discriminate against peaks that are too close to edge
                    IF (right - k) LE min_dist THEN BEGIN
                        score(k)=score(k)*too_close_fac  
                        IF tst THEN print,k,' too_close_fac',score(k)
                    ENDIF

                    IF (k - left) LE min_dist THEN BEGIN
                        score(k)=score(k)*too_close_fac  
                        IF tst THEN print,k,' too_close_fac1',score(k)
                    ENDIF

                                ; give more weight to taller peaks
                                ;  score(k) = score(k) * sig_height_fac * cur_spec(k)/noise 

                                ; see if this is the best point
                    IF score(k) GT max_score THEN BEGIN
                        max_score = score(k)
                        best_point = k
                    ENDIF

                ENDFOR          ; assign score over range (k loop)

                lines_found = lines_found + 1

                
                                ; trap for un realistic assignments
                IF max_score LE 3.0 THEN BEGIN
                    best_point = -1
                    lines_found = lines_found -1
                ENDIF

                IF best_point EQ left THEN BEGIN
                    best_point = -1
                    lines_found =lines_found -1
                ENDIF

                IF best_point EQ right THEN BEGIN
                    best_point = -1
                    lines_found =lines_found -1
                ENDIF

            ENDELSE             ; determine score vector over range

;            print,"Spec # -- ",i,j,"    naa_pos= ",best_point, $
;              "   Best Score = ",Max(score)
            naa_pos(i,j) = best_point
        END                     ; end peak eval
    ENDFOR                      ; i loop
ENDFOR                          ; j loop

valid = where( naa_pos NE -1 )
IF valid(0) NE -1 THEN BEGIN
    mean_naa_pos = float( total( naa_pos(valid) ) ) / float(lines_found)
    std_naa_pos = STDEV(float(naa_pos(valid)))

;    print, lines_found, ' valid naa lines found.'
;    print, 'Mean naa position = ', mean_naa_pos

    !p.multi = [0,1,2]

;*****  Plot Peak Scattergram:

    amp = 0
    pos = 0

    FOR j = 0, sp_dim(1)-1 DO BEGIN
        FOR i = 0, sp_dim(0)-1 DO BEGIN
            IF ( naa_pos(i,j) NE -1.0 ) THEN BEGIN
                amp = [ amp, abs(fd_dat(naa_pos(i,j),i,j)) ]
                pos = [ pos, naa_pos(i,j) ]
            END
        END
    END

    amp = amp(1:*)
    pos = pos(1:*)
    window,8
    wset, 8
    plot_io, pos, amp, psym=1, xstyle=1, xran=[left, right], $
      xtit='NAA Peak Position', ytit='NAA Peak Intensity', $
      tit = 'Scatter Plot of NAA Peak Instnsity vs. Position'

;*****  Plot Peak Position Histogram:

;window,1
    wset, 8
    plot, indgen( n )+left, xstyle=1, $
      histogram( naa_pos(valid), min=left, max=right ), $
      tit = 'Histogram of Valid NAA Line Positions', $
      xtit = 'NAA Peak Position', ytit='Occurences'

    !p.multi = 0
ENDIF ELSE BEGIN
    mean_naa_pos = 0
    std_naa_pos = 0

;    print, 'No valid lines found.'
ENDELSE

RETURN, naa_pos

END
