;This program computes the convolution of an image with the point spread
;function of Jeff Duyn's mpsci sequence. The input parameters are:
; image   a 256x256 image to be convolved. For segmentation analysis
;         this image should be a binary image of a particular class.
; apd     the apodization to use. The current version of analyze_roi
;         uses apd=2.
;The function returns the convolution of the psf with the input image.
;The convolution is computed using the Fourier transform, and the
;result is truncated to 32x32 before the transform back to image
;space. If the input is a binary image of a particular class, the
;output represents the contribution of that class to the signal in the
;csi map at the corresponding position.

function convolve_seg,image,apd
xres=32
xsize=256
yres=xres
ysize=256
;Take FT of class image
result=shift(image,xsize/2,ysize/2)
result=fft(result,-1)
;shift position of filter
psf=mkpsf(2,xres,xsize=256,type=4)
psf=shift(psf,-xsize/2,-xsize/2)
filter=float(fft(psf,1))
;apply filter
result=result*filter
;Truncate result 
result=shift(result,xres/2,yres/2)
result=result(0:xres-1,0:yres-1)
result=shift(result,-xres/2,-yres/2)
result=float(fft(result,1))
result=shift(result,xres/2,xres/2)
RETURN,result
END
