;  6/13/91  @(#)csift.pro	1.8  In Vivo NMR Research Center, NIH

;  910301  GSS  Added keyword '/no_alt' to remove shift in x axis
;               because of alternating +/- pulses.
;  910303  GSS  Added the keyword, '/no_shift' to 
;               indicate that the echo max. should not be moved to the 
;               start of the array.
;  910304  GSS  Removed ref. to '!echo_loc'
;  910304a GSS  Added keywords, '/spatial' and '/spectral'
;               Modified shifting code to perform the shift on a per FID
;               basis, to avoid array allocation problems.
;  910419  GSS  Moved Echo shifting into spectral FFT conditional.
;  910521  GSS  Added shift of phase-encode origin to FFT origin.
;  910613  GSS  Changed behavior wrt '!echo_pt' - no change to value 
;               after transform.
;  980915  ASB  Replaced !csi_dat with fd_dat in COMMON freqdomain.
;  980915  ASB  Added COMMON initcsi and removed system variables.
;  980916  ASB  Changed name of COMMON freqdomain to mrsidata.
;  980921  ASB  Added keywords twindow and swindow.
;               Changed order of ft.
;               Removed keywords spatial and spectral.
;               Stores hybrid transform in td_dat.
;  991005  ASB  Added keyword t1window for processing first echo.
;               Removed keyword no_echo_shift.

PRO csift, swindow=swindow,twindow=twindow,t1window=t1window,no_alt=no_alt, $
           no_pe_shift=no_pe_shift
COMMON mrsidata
COMMON initcsi
COMMON defaults
;debug
;etlold=etl
;etl=1
;*****  Set post-FFT shifts allong all axes:

t_shift = spc_len/2
y_shift = (sp_dim(1) / 2)       ;original

IF keyword_set( no_alt ) THEN x_shift = sp_dim(0) / 2 $
ELSE x_shift = 0                ;original

; Spatial FFT:


                                ;*****  Do Spatial FFTs:

IF NOT keyword_set( no_pe_shift ) THEN $
  print, 'Doing Spatial FFTs and PE shifts...' $
ELSE $
  print, 'Doing Spatial FFTs...'

print, format='( "+", 30("-"), "+" )'
interval = spc_len/32
IF n_elements(swindow) NE 0 THEN BEGIN
    window=replicate(1,spc_len)#swindow
    td_dat=td_dat*window
    IF etl EQ 2 THEN td_dat1=td_dat1*window
ENDIF

FOR i = 0, spc_len-1 DO BEGIN
    IF (i MOD interval) EQ 0 THEN print, format='( $, "*" )'

    slice = reform( td_dat( i, *,*) )

    IF NOT keyword_set( no_pe_shift ) THEN $
      slice = shift( slice, pe_dim(0)/2, pe_dim(1)/2 )

    slice = shift( fft(slice, 1), x_shift, y_shift )

    td_dat(i,*,*)=slice
;    td_dat(i, 0, 0) = reform( slice, 1, sp_dim(0), sp_dim(1) )


    IF etl EQ 2 THEN BEGIN
        slice = reform( td_dat1( i, *,*) )
        
        IF NOT keyword_set( no_pe_shift ) THEN $
          slice = shift( slice, pe_dim(0)/2, pe_dim(1)/2 )
        
        slice = shift( fft(slice, 1), x_shift, y_shift )
        
        td_dat1(i,*,*)=slice
;        td_dat1(i, 0, 0) = reform( slice, 1, sp_dim(0), sp_dim(1) )
    ENDIF
END
print


; Spectral FFT:

;Apply apodization
IF n_elements(twindow) NE 0 THEN BEGIN
    window = twindow#replicate(1,pe_dim(0)*pe_dim(1))
    fd_dat=td_dat*window
    IF etl EQ 2 THEN BEGIN
        window = t1window#replicate(1,pe_dim(0)*pe_dim(1))
        fd_dat1=td_dat1*window
    ENDIF
ENDIF ELSE BEGIN
    fd_dat=td_dat
    IF etl EQ 2 THEN fd_dat1=td_dat1
ENDELSE
                                ;*****  Shift echo max. to beginning of array:

print, 'Shifting echo to data start...'
print, format='( "+", ' + string(pe_dim(1)-2) + '("-"), "+" )'

                                ;  Shift the echo max. of each FID individually:
FOR j = 0, pe_dim(1)-1 DO BEGIN
    print, format='( $, "*" )'
    FOR i = 0, pe_dim(0)-1 DO BEGIN
        fd_dat(0,i,j) = shift( fd_dat(*,i,j), -echo_pt )
        IF etl EQ 2 THEN BEGIN
            fd_dat1(0,i,j) = shift( fd_dat1(*,i,j), -echo_pt1 )
        ENDIF
    END
END


                                ;*****  Do Spectral FFTs:
print, 'Doing Spectral FFTs...'
print, format='( "+", ' + string(pe_dim(1)-2) + '("-"), "+" )'

FOR j = 0, pe_dim(1)-1 DO BEGIN
    print, format='( $, "*" )'
    FOR i = 0, pe_dim(0)-1 DO BEGIN
        fd_dat(0,i,j) = shift(fft(fd_dat(*,i,j),-1), t_shift)
        IF etl EQ 2 THEN BEGIN
            fd_dat1(0,i,j) = shift(fft(fd_dat1(*,i,j),-1), t_shift)
        END
    END
END
print
;debug
;etl=etlold
RETURN
END
