PRO csift4x, no_alt=no_alt, $
           no_echo_shift=no_echo_shift, no_pe_shift=no_pe_shift, $
           spatial=spatial, spectral=spectral

;  6/13/91  @(#)csift.pro	1.8  In Vivo NMR Research Center, NIH

;  910301  GSS  Added keyword '/no_alt' to remove shift in x axis
;               because of alternating +/- pulses.
;  910303  GSS  Added the keyword, '/no_shift' to 
;               indicate that the echo max. should not be moved to the 
;               start of the array.
;  910304  GSS  Removed ref. to '!echo_loc'
;  910304a GSS  Added keywords, '/spatial' and '/spectral'
;               Modified shifting code to perform the shift on a per FID
;               basis, to avoid array allocation problems.
;  910419  GSS  Moved Echo shifting into spectral FFT conditional.
;  910521  GSS  Added shift of phase-encode origin to FFT origin.
;  910613  GSS  Changed behavior wrt '!echo_pt' - no change to value 
;               after transform.
;  960126  ASB  Changed direction of spatial fft for use with doanalyze
;  980915  ASB  Replaced !csi_dat with fd_dat in COMMON freqdomain.
;  980915  ASB  Added COMMON initcsi and removed system variables.
;  980916  ASB  Changed name of COMMON freqdomain to mrsidata.
;*****  If neither '/spatial' or '/spectral' is set then do both
COMMON mrsidata
COMMON initcsi
IF (NOT keyword_set( spatial )) AND (NOT keyword_set( spectral )) THEN BEGIN
   spatial = 1
   spectral = 1
END

;*****  Set post-FFT shifts allong all axes:

t_shift = spc_len/2
y_shift = (sp_dim(1) / 2) ;original

IF keyword_set( no_alt ) THEN x_shift = sp_dim(0) / 2 $
                         ELSE x_shift = 0 ;original
                        
; Spectral FFT:

IF keyword_set( spectral ) THEN BEGIN

   ;*****  Shift echo max. to beginning of array:
   
   IF (NOT keyword_set ( no_echo_shift )) AND (echo_pt GT 0) THEN BEGIN
   
      print, 'Shifting echo to data start...'
      print, format='( "+", ' + string(pe_dim(1)-2) + '("-"), "+" )'

      ;  Shift the echo max. of each FID individually:
      FOR j = 0, pe_dim(1)-1 DO BEGIN
         print, format='( $, "*" )'
         FOR i = 0, pe_dim(0)-1 DO BEGIN
            fd_dat(0,i,j) = shift( fd_dat(*,i,j), -echo_pt )
         END
      END
      print

   END

   ;*****  Do Spectral FFTs:

   print, 'Doing Spectral FFTs...'
   print, format='( "+", ' + string(pe_dim(1)-2) + '("-"), "+" )'

   FOR j = 0, pe_dim(1)-1 DO BEGIN
      print, format='( $, "*" )'
      FOR i = 0, pe_dim(0)-1 DO BEGIN
         fd_dat(0,i,j) = shift(fft(fd_dat(*,i,j),-1), t_shift)
      END
   END
   print

END

; Spatial FFT:

IF keyword_set( spatial ) THEN BEGIN

   ;*****  Do Spatial FFTs:

   IF NOT keyword_set( no_pe_shift ) THEN $
      print, 'Doing Spatial FFTs and PE shifts...' $
   ELSE $
      print, 'Doing Spatial FFTs...'

   print, format='( "+", 30("-"), "+" )'
   interval = spc_len/32

   FOR i = 0, spc_len-1 DO BEGIN
      if (i MOD interval) EQ 0 THEN print, format='( $, "*" )'

      slice = reform( fd_dat( i, *,*) )

      IF NOT keyword_set( no_pe_shift ) THEN $
         slice = shift( slice, pe_dim(0)/2, pe_dim(1)/2 )

      slice = shift( fft(slice, 1), x_shift, y_shift )
      slice = rotate(slice,5) ; added 960126, moved 960131

      fd_dat(i, 0, 0) = reform( slice, 1, sp_dim(0), sp_dim(1) )

   END
   print

END

RETURN
END
