;This program does the time domain analysis.
;Revision History
; 980925 asb First version copied from code by JW van der Veen.
; 990302 asb Added common blocks defroi, noise.
;            Added code to sensitize button save button.
;            Changed code to compute eerror matrix for fit routines.
;            Changed calling sequence in calls to fitexp.
;            Added code to read location of complex_svd.so from
;            environment variable SPECHOME.
; 990429 asb Added code to permit user to abort analysis.
pro do_tdomain
COMMON initcsi
COMMON mrsidata
COMMON posdata
COMMON td_widget
COMMON defroi
COMMON noise

;Make arrays to hold results
dry_dat=complexarr(spc_len,pe_dim(0),pe_dim(1))
fit_dat=dry_dat

;Make bo map from naa positions
ppm_to_point = spec_freq * spc_len/ sweep_width
naa_offset = (4.75-2.023) * ppm_to_point
f_map=-(2.*(positions-naa_offset)/spc_len-1.0)*!pi

mask=where(positions NE -1)
w=bytarr(pe_dim(0),pe_dim(1))
w(mask)=1

; Throw away points 0 and 255
istart=1
iend=fid_len-2
ndata=iend-istart+1

;Time axis
t= (dindgen(2*ndata) - echo_pt) / double(sweep_width)
;Frequency axis
f=((findgen(1024)/1024.)-0.5)*double(sweep_width)

xm=dblarr(pe_dim(0),pe_dim(1),14)  ; store fit results and amplitude uncertainties

ti= dindgen(ndata)

;******* 1*SVD water supp + FIR + NAA initial value from SVD *************************
Kmax=20 
nfilt=Kmax*2+1           ; number of FIR filter coeff.

fdata = dcomplexarr(ndata-2*Kmax)
fndata=n_elements(fdata)

fir1=dblarr(nfilt)
freqr=0.22*!dpi ; cutoff freq in fraction of max freq

;*********** Hamming window FIR
FOR i=0,nfilt-1 DO BEGIN
  temp=double(i-nfilt/2)
  IF temp EQ 0. THEN fir1(i)=freqr/!dpi ELSE fir1(i)=sin(freqr*temp)/(temp*!dpi)
  fir1(i)=fir1(i)*(0.54-0.46*cos(2.*!dpi*i/double(nfilt-1)))
ENDFOR
fsum = total(fir1)

;*********** offset window FIR

firoff=dcomplexarr(nfilt)

;Array of times for NL fit below.  I don't understand why points are
;missing, or why array is so long.
tin=[t(Kmax:ndata-Kmax-1),t(ndata+Kmax:2*ndata-Kmax-1)]

;************ SVD vars
nterms=100                  ; number of SVD poles
nsvd=fix(ndata-nterms-1)    ; settings for svd prog. 
msvd=fix(nterms)            ; settings for svd prog.
ipsvd=fix(1)                ; settings for svd prog.

AC=dcomplexarr(nsvd,nterms+ipsvd)
UC=dcomplexarr(nsvd,nterms)
VC=dcomplexarr(nterms, nterms)
SV=dblarr(nterms)
WP=dblarr(nterms,nterms)
xb=dblarr(nterms)           ; xb xc and xt are scratch matrices
xc=dblarr(nterms)           ; for the SVD prog.
xt=dblarr(nterms)

yfit=dcomplexarr(ndata,20)     ; 20 roots max for watersignal
ws_freq=double(50.)          ; Dataproc. water suppression interval (+- Hz)

ti= dindgen(ndata) 

;Stuff for monitor window
nspec=total(w)
nn=0
curwin=!d.window                ;save current window
td_widget                       ;make monitor widget
wset,tdplot                     ;plot in draw widget in monitor window
widget_control,tot_w,set_value=nspec
tstart=systime(1)
foo=strmid(systime(),11,8)
widget_control,sttime_w,set_value=foo
widget_control,cttime_w,set_value=foo
chour=0l
cmin=0l
csec=0l
reads,strmid(foo,0,2),chour
reads,strmid(foo,3,2),cmin
reads,strmid(foo,6,2),csec

;Initialize varibles to hold execution times
svd1_t=double(0)
root_t=double(0)
svd2_t=double(0)
dry_t=double(0)
fit_t=double(0)

FOR i1=0,pe_dim(0)-1 DO BEGIN
    FOR i2=0,pe_dim(1)-1 DO BEGIN
;Check for interrupt
        ev=widget_event(quit_w,/nowait)
        IF ev.id NE 0 THEN BEGIN
            widget_control,quit_w,get_uvalue=top
            widget_control,top,/destroy
            RETURN
        ENDIF
        IF (w(i1,i2) GT 0) THEN BEGIN
;update status widgets
;update current pixel widget            
            title=string( format='(I2,",",I2)',i1,i2)
            widget_control,cur_w,set_value=title

;update current time widget
            foo=strmid(systime(),11,8)
            widget_control,cttime_w,set_value=foo
            
;Compute and display expected time of completion of calculation
            t1=systime(1)
            widget_control,active_w,set_value=0
            IF nn NE 0 THEN BEGIN
                dtime=long(t1-tstart)*nspec/nn
                dtime=dtime+csec+60*(cmin+60*chour)

                hour=strtrim(long(dtime/3600l),2)
                IF strlen(hour) EQ 1 THEN hour='0'+hour

                min=strtrim(long((dtime-hour*3600l)/60l),2)
                IF strlen(min) EQ 1 THEN min='0'+min

                sec=strtrim(long(dtime-(hour*3600l+min*60l)),2)
                IF strlen(sec) EQ 1 THEN sec='0'+sec
                
                widget_control,ectime_w,set_value=hour+':'+min+':'+sec
            ENDIF

            widget_control,com_w,set_value=nspec-nn
            widget_control,nfit_w,set_value=nn
            nn=nn+1

;Solve equations to remove water peak (Prony's method?)
;begin fitting
            test=reform(td_dat(istart:iend,i1,i2))
            maxtest=max(abs(test))/10. ; normalize for algorithmic stability
            test=test/maxtest

            offset=f_map(i1,i2)

;********** remove nr 1

            FOR i=0,nterms DO AC(0:ndata-nterms-2,i)= test(i+1:ndata-1-(nterms-i))

            spechome=getenv("SPECHOME")
            dret=call_external(spechome+'/complex_svd.so',$
                               'complex_svd',nsvd,msvd,ipsvd,AC,SV,UC,VC,xb,xc,xt)

            WP(*,*)=0.0
            ix=where(sv GT max(sv/10000.0))
            wp(ix,ix)=1./sv(ix)
;cpredic contains the coefficients of the characteristic
;polynomial. The roots of cpredic are the frequencies present in the fit.
            cpredic=VC # WP # reform(AC(0:nterms-1,nterms+ipsvd-1))

;Update performance values
            t2=systime(1)
            svd1_t=svd1_t+t2-t1
            widget_control,svd1_w,set_value=svd1_t
            widget_control,dry_w,set_value=svd1_t+root_t+svd2_t
            widget_control,active_w,set_value=1

            l_freq=(-ws_freq*2.*!dpi/double(sweep_width))-offset ; water supp. range
            r_freq=( ws_freq*2.*!dpi/double(sweep_width))-offset ; water supp. range

; find roots of cpredic in this range
;Why is extra coefficient -1 added to cpredic?
            croots=jwrooter([cpredic,dcomplex(-1.0,0.0)],l_freq,r_freq,/polish) 
            ix = where(abs(croots) ne 0.)

            parm=alog(croots(ix)) ;Complex frequencies
            nroots=n_elements(parm) ;Number of roots

;Update performance values
            t3=systime(1)
            root_t=root_t+t3-t2
            widget_control,root_w,set_value=root_t
            widget_control,dry_w,set_value=svd1_t+root_t+svd2_t
            widget_control,active_w,set_value=2

            yfit(*,*)=0.

; reconstruct damped sinusoids from roots
;Why not call complex_svd again?
            FOR i=0,nroots-1 DO yfit(*,i)=exp( parm(i)*ti ) 
            AA=dblarr(ndata*2,nroots*2)
            AA(0    :ndata  -1,0     :nroots  -1)=double(yfit(*,0:nroots-1))
            AA(0    :ndata  -1,nroots:nroots*2-1)=-imaginary(yfit(*,0:nroots-1))
            AA(ndata:ndata*2-1,0     :nroots  -1)=imaginary(yfit(*,0:nroots-1))
            AA(ndata:ndata*2-1,nroots:nroots*2-1)=double(yfit(*,0:nroots-1))

            svdc,AA,w2,u2,v2,/column,/double
            amp=svsol(u2,w2,v2,[double(test),imaginary(test)],/column,/double)

            tyfit=AA#amp
            cyfit=dcomplex( tyfit(0:ndata-1),tyfit(ndata:ndata*2-1) ) 
            testx=test-cyfit    ; subtract fitted water signal

;***************************Apply FIR filter ***************************************************
            FOR i=0,nfilt-1 DO firoff(i)= exp( dcomplex(0.,1.)* $
                 ( (offset-160.*2.*!dpi/double(sweep_width)))*double(i-nfilt/2) )
            fdata(*)=0.
            FOR count=Kmax, ndata-Kmax-1 DO $
              fdata(count-Kmax) = total( fir1*firoff * testx(count-Kmax:count+Kmax) ) / $
              fsum

;******************************** renormalisation after subtraction of water signal*********************

            mult2=10./max(abs(fdata))
            fdata=fdata*mult2
            maxtest=maxtest/mult2

;Update performance values
            t4=systime(1)
            svd2_t=svd2_t+t4-t3
            widget_control,svd2_w,set_value=svd2_t
            widget_control,dry_w,set_value=svd1_t+root_t+svd2_t
            widget_control,active_w,set_value=3

;Compute and store spectrum
            dry_dat(0:fndata-1,i1,i2)=fdata*maxtest
            dry_dat(*,i1,i2)=shift(dry_dat(*,i1,i2),-fndata/2)
            dry_dat(*,i1,i2)=shift(fft(dry_dat(*,i1,i2)),spc_len/2)

; ******************************* find naa position *********************

            bnaastart=double(8.)
            wnaastart=double(sweep_width)*offset

;******************************** NL fit ********************************
;First guesses for amplitudes
            funct, 11,tin,[1.,0.,0.,0.,0.,0.,bnaastart,wnaastart] , xfit
            A11 = ( transpose([double(fdata),imaginary(fdata)])#xfit ) / $
              ( transpose(xfit)#xfit )

            funct, 11,tin,[0.,1.,0.,0.,0.,0.,bnaastart,wnaastart] , xfit
            A12 = ( transpose([double(fdata),imaginary(fdata)])#xfit ) / $
              ( transpose(xfit)#xfit )

            funct, 11,tin,[0.,0.,1.,0.,0.,0.,bnaastart,wnaastart] , xfit
            A21 = ( transpose([double(fdata),imaginary(fdata)])#xfit ) / $
              ( transpose(xfit)#xfit )

            funct, 11,tin,[0.,0.,0.,1.,0.,0.,bnaastart,wnaastart] , xfit
            A22 = ( transpose([double(fdata),imaginary(fdata)])#xfit ) / $
              ( transpose(xfit)#xfit )

            funct, 11,tin,[0.,0.,0.,0.,1.,0.,bnaastart,wnaastart] , xfit
            A31 = ( transpose([double(fdata),imaginary(fdata)])#xfit ) / $
              ( transpose(xfit)#xfit )

            funct, 11,tin,[0.,0.,0.,0.,0.,1.,bnaastart,wnaastart] , xfit
            A32 = ( transpose([double(fdata),imaginary(fdata)])#xfit ) / $
              ( transpose(xfit)#xfit )

            pin=[a11,a12,a21,a22,a31,a32,bnaastart,wnaastart]
;**** fit with all freq. linked
            pout=fitexp(11,tin,[double(fdata),imaginary(fdata)],pin) 

            pin=[pout,pout(7),pout(7)]
;**** fit with all freq. free

            tdnoise=rmsnoise*sqrt(float(spc_len)^2/fid_len)
            xm(i1,i2,0:9)=fitexp(10,tin,[double(fdata),imaginary(fdata)],pin,$
                               tdnoise/maxtest,sigma,chisqr) 
;sigma2 contains curvature matrix
;Compute standard error for metabolite magnitudes, assuming different
;metabolites are uncorrelated (no justified for CHO-CRE)
            funct, 10,tin, reform(xm(i1,i2,0:9)), xfit

            cyfit=dcomplex( xfit(0:fndata-1),xfit(fndata:fndata*2-1) )
            xm(i1,i2,0:5)=xm(i1,i2,0:5)*maxtest ; forgot about this, correct ampl.

            FOR kk = 0,2 DO BEGIN
                xm(i1,i2,10+kk)=sqrt(sigma(2*kk)^2+sigma(2*kk+1)^2)*maxtest
            ENDFOR
;Store chisqr
            xm(i1,i2,13)=chisqr

;Compute and store spectrum of best fit
            tmp=complexarr(spc_len)
            tmp(0)=cyfit*maxtest
            tmp=shift(tmp,-fndata/2)
            fit_dat(*,i1,i2)=shift(fft(tmp),spc_len/2)

            t5=systime(1)
            fit_t=fit_t+t5-t4
            widget_control,fit_w,set_value=fit_t
            widget_control,active_w,set_value=0
        ENDIF
    ENDFOR
ENDFOR


;Change label of quit widget to quit from abort
widget_control,quit_w,set_value='Quit'
;update current time widget
foo=strmid(systime(),11,8)
widget_control,cttime_w,set_value=foo
widget_control,ectime_w,set_value=foo
            
;Compute and display expected time of completion of calculation
t1=systime(1)

widget_control,com_w,set_value=nspec-nn
widget_control,nfit_w,set_value=nn
nn=nn+1

;Make metabolite maps from time domain results
naa_td=sqrt(xm(*,*,0)^2+xm(*,*,1)^2)
cho_td=sqrt(xm(*,*,2)^2+xm(*,*,3)^2)
cre_td=sqrt(xm(*,*,4)^2+xm(*,*,5)^2)
wset,curwin

;display time domain maps and spectra
tdflag=1
IF n_elements(width_w) EQ 1 THEN IF widget_info(width_w,/valid)$
  THEN widget_control,width_w,map=0
widget_control,s.tf_w,set_value=s.tf_labels(1),set_uvalue=1
init_mets
clear_roi

;if roi exists, display it
widget_control,s.num1_w,get_value=count
;IF count GT 0 THEN asb_defroi_draw,fill=1
IF n_elements(stats_w) EQ 1 THEN IF widget_info(stats_w,/valid) EQ 1 $
  THEN asb_defroi_update

;sensitize save button
widget_control,s.save_ids(2),sensitive=1
END
