;This procedure creates a dialog box for choosing settings
;Revision History
; 05Sep96 ASB First version, only changes color table
; 25Feb98 ASB Added code to choose filters and filter parameters
; 27Feb98 ASB Debugged code to choose filters and filter parameters
; 13Apr98 Qin Added a "Set Color Table" button using xloadct( so
;             that contrust is available) , removed the old ones
; 14Apr98 Qin Added "q_naa_max" input_label for input of sys. var
;             naa_scale_max.
;  1May98 QIN Added code to set jhd flag.
; 15May98 ASB Changed code for handling color tables.
;             Added Comments
; 20Jul98 ASB Removed superfluous code.
;             Added code to update plots immediately when limits are changed.
; 21Jul98 ASB Split Settings into recon_settings and display_settings.
;             Added code to handle range of values in spectrum plot
;             and metabolite maps.
; 22Jul98 ASB Added keywords in call to init_mets.
; 16Sep98 ASB Replaced system variables with COMMON initcsi.
; 18Sep98 ASB Changed initial value of maxspec_w to max_spec_value.
; 18Sep98 ASB Changed title of window to 'Display Settings'
; 14Oct98 ASB Added buttons to control display in window 4.
;             Added buttons to set minimum and maximum freqs in map
; 18Nov98 ASB Removed calls to widget_control,s.max and s.min
;             Removed keywords max and min from calls to update_window.
;  7Apr99 ASB Added code for controling the display of the water map in
;             window 4.
; 11Aug99 ASB Added buttons bswap_w to permit byteswapping the mris.
;             Added COMMON posdata.
;  8Dec99 ASB Added code to permit user to choose which maps to display.
;             Added code to display water maps.
;             Added code to display raw or hybrid data in spectrum window.
;             Added code to display real, imaginary, magnitude or
;             phase spectrum.
; 25May00 ASB Changed names of water maps in pulldown menus.

; This program handles events returned by the display_settings widgets.
pro dsettings_event,event
COMMON dsettings
COMMON defaults
COMMON defroi
COMMON update
COMMON initcsi
COMMON posdata
CASE event.id OF 
                                ;Set color table
    setcolort1_w: BEGIN ;MRI color table
        table1=event.value       
        IF table1 EQ table2 THEN BEGIN
            widget_control,setcolort3_w,sensitive=1
            tmode=1
            tvlcolor,table1,table3
        ENDIF ELSE BEGIN
            widget_control,setcolort3_w,sensitive=0
            tvlcolor,table1,table2
            tmode=0
        ENDELSE
;Redraw MRIs
        FOR k = 0,n_elements(s.gwins)-1 DO BEGIN
            update_window,k
        ENDFOR
;Redraw metabolites
        init_mets,/replot_only
;Redraw ROIs
        asb_defroi_draw,0,fill=3
    END

    setcolort2_w: BEGIN ;Metabolite color table
        table2=event.value       
        IF table1 EQ table2 THEN BEGIN
            widget_control,setcolort3_w,/sensitive
            tmode=1
            tvlcolor,table1,table3
        ENDIF ELSE BEGIN
            widget_control,setcolort3_w,sensitive=0
            tvlcolor,table1,table2
            tmode=0
        ENDELSE
;Redraw MRIs
        FOR k = 0,n_elements(s.gwins)-1 DO BEGIN
            update_window,k
        ENDFOR
;Redraw metabolites
        init_mets,/replot_only
;Redraw ROIs
        asb_defroi_draw,0, fill=3
    END

    setcolort3_w: BEGIN ;ROIs if tab1=tab2
        table3=event.value       
        tvlcolor,table1,table3
;Redraw MRIs
        FOR k = 0,n_elements(s.gwins)-1 DO BEGIN
            update_window,k
        ENDFOR
;Redraw metabolites
        init_mets,/replot_only
;Redraw rois
        asb_defroi_draw,0, fill=3
    END

    lfspec_w: BEGIN
        widget_control,lfspec_w,get_value=tmp
        last_ppm_lf=tmp
        update_specplot
    END

    rtspec_w: BEGIN
        widget_control,rtspec_w,get_value=tmp
        last_ppm_rt=tmp
        update_specplot
    END

; Change widowing for mris
    min_w(0) : BEGIN
        widget_control,min_w(0),get_value=pmin
        min_v(0)=pmin(0)
        update_window,0
        mm=s.mode
        s.mode=0
        asb_DEFROI_DRAW,fill=2
        s.mode=mm
    END
        
    max_w(0) : BEGIN
        widget_control,max_w(0),get_value=pmax
        max_v(0)=pmax(0)
        update_window,0
        mm=s.mode
        s.mode=0
        asb_DEFROI_DRAW,fill=2
        s.mode=mm
    END
        
; Change widowing for mris
    min_w(1) : BEGIN
        widget_control,min_w(1),get_value=pmin
        min_v(1)=pmin(0)
        update_window,1
        mm=s.mode
        s.mode=0
        asb_DEFROI_DRAW,fill=2
        s.mode=mm
    END
        
    max_w(1) : BEGIN
        widget_control,max_w(1),get_value=pmax
        max_v(1)=pmax(0)
        update_window,1
        mm=s.mode
        s.mode=0
        asb_DEFROI_DRAW,fill=2
        s.mode=mm
    END
        
; Change widowing for mris
    min_w(2) : BEGIN
        widget_control,min_w(2),get_value=pmin
        min_v(2)=pmin(0)
        update_window,2
        mm=s.mode
        s.mode=0
        asb_DEFROI_DRAW,fill=2
        s.mode=mm
    END
        
    max_w(2) : BEGIN
        widget_control,max_w(2),get_value=pmax
        max_v(2)=pmax(0)
        update_window,2
        mm=s.mode
        s.mode=0
        asb_DEFROI_DRAW,fill=2
        s.mode=mm
    END
        
;byteswap mris
    bswap_w(0) : BEGIN
        tmp=mri(*,*,0)
        byteorder,tmp
        mri(*,*,0)=tmp
        update_window,0
        bswap(0)=(bswap(0)+1) mod 2
        asb_defroi_draw,fill=2
    END

    bswap_w(1) : BEGIN
        tmp=mri(*,*,1)
        byteorder,tmp
        mri(*,*,1)=tmp
        update_window,1
        bswap(1)=(bswap(1)+1) mod 2
        asb_defroi_draw,fill=2
    END

    bswap_w(2) : BEGIN
        tmp=mri(*,*,2)
        byteorder,tmp
        mri(*,*,2)=tmp
        update_window,2
        bswap(2)=(bswap(2)+1) mod 2
        asb_defroi_draw,fill=2
    END


    mxspec_w: BEGIN
        widget_control,mxspec_w,get_value=tmp
        max_spec_val=tmp
        update_specplot
    END

    max_cre_w: BEGIN
        widget_control,max_cre_w,get_value=max_cre
        init_mets,/replot_only,met=1
        asb_defroi_draw,0,/fill
    END

    max_cho_w: BEGIN
        widget_control,max_cho_w,get_value=max_cho
        init_mets,/replot_only,met=2
        asb_defroi_draw,0,/fill
    END

    max_naa_w: BEGIN
        widget_control,max_naa_w,get_value=max_naa
        init_mets,/replot_only,met=0
        asb_defroi_draw,0,/fill
    END

    max_lac_w: BEGIN
        widget_control,max_lac_w,get_value=max_lac
        init_mets,/replot_only,met=3
        asb_defroi_draw,0,/fill
    END
        
    max_ino_w: BEGIN
        widget_control,max_ino_w,get_value=max_ino
        init_mets,/replot_only,met=4
        asb_defroi_draw,0,/fill
    END
        
    min_water_w: BEGIN
        widget_control,min_water_w,get_value=min_water
        init_mets,/replot_only,met=5
        asb_defroi_draw,0,/fill
    END
        
    max_water_w: BEGIN
        widget_control,max_water_w,get_value=max_water
        init_mets,/replot_only,met=5
        asb_defroi_draw,0,/fill
    END
        
    min_b0_w: BEGIN
        widget_control,min_b0_w,get_value=min_b0
        init_mets,/replot_only,met=6
        asb_defroi_draw,0,/fill
    END
        
    max_b0_w: BEGIN
        widget_control,max_b0_w,get_value=max_b0
        init_mets,/replot_only,met=6
        asb_defroi_draw,0,/fill
    END
        
    pdmenu1_w:pdmenu_win,event
    pdmenu2_w:pdmenu_win,event
    pdmenu3_w:pdmenu_win,event
    pdmenu4_w:pdmenu_win,event

    pdmenuet_w:pdmenu_win,event
    pdmenudom_w:pdmenu_win,event
    pdmenucomp_w:pdmenu_win,event

    dsetquit_w:BEGIN     ;Save neccesary info and quit
        widget_control,dsetbase_w,/destroy
    END 

ENDCASE
END

pro pdmenu_win,event
COMMON defroi
COMMON dsettings

CASE Event.Value OF 
;Window1
    'Window 1.NAA': BEGIN
        widget_control,win1l_w,set_value=mettitles(0)
;Draw map
        s.mets(0)=0
        init_mets,metabolite=0
        asb_defroi_draw,0,/fill
    END

    'Window 1.CRE': BEGIN
        widget_control,win1l_w,set_value=mettitles(1)
;Draw map
        s.mets(0)=1
        init_mets,metabolite=1
        asb_defroi_draw,0,/fill
  END

    'Window 1.CHO': BEGIN
        widget_control,win1l_w,set_value=mettitles(2)
;Draw map
        s.mets(0)=2
        init_mets,metabolite=2
        asb_defroi_draw,0,/fill
    END

    'Window 1.LAC': BEGIN
        widget_control,win1l_w,set_value=mettitles(3)
;Draw map
        s.mets(0)=3
        init_mets,metabolite=3
        asb_defroi_draw,0,/fill
    END

    'Window 1.INO': BEGIN
        widget_control,win1l_w,set_value=mettitles(4)
;Draw map
        s.mets(0)=4
        init_mets,metabolite=4
        asb_defroi_draw,0,/fill
    END

    'Window 1.Water I1': BEGIN
        widget_control,win1l_w,set_value=mettitles(5)
;Draw map
        s.mets(0)=5
        init_mets,metabolite=5
        asb_defroi_draw,0,/fill
    END

    'Window 1.Water I2': BEGIN
        widget_control,win1l_w,set_value=mettitles(6)
;Draw map
        s.mets(0)=6
        init_mets,metabolite=6
        asb_defroi_draw,0,/fill
    END

    'Window 1.Water M1': BEGIN
        widget_control,win1l_w,set_value=mettitles(7)
;Draw map
        s.mets(0)=7
        init_mets,metabolite=7
        asb_defroi_draw,0,/fill
    END

    'Window 1.Water M2': BEGIN
        widget_control,win1l_w,set_value=mettitles(8)
;Draw map
        s.mets(0)=8
        init_mets,metabolite=8
        asb_defroi_draw,0,/fill
    END

    'Window 1.Water P1': BEGIN
        widget_control,win1l_w,set_value=mettitles(9)
;Draw map
        s.mets(0)=9
        init_mets,metabolite=9
        asb_defroi_draw,0,/fill
    END

    'Window 1.Water P2': BEGIN
        widget_control,win1l_w,set_value=mettitles(10)
;Draw map
        s.mets(0)=10
        init_mets,metabolite=10
        asb_defroi_draw,0,/fill
    END

;Window2
    'Window 2.NAA': BEGIN
        widget_control,win2l_w,set_value=mettitles(0)
;Draw map
        s.mets(1)=0
        init_mets,metabolite=0
        asb_defroi_draw,0,/fill
    END

    'Window 2.CRE': BEGIN
        widget_control,win2l_w,set_value=mettitles(1)
;Draw map
        s.mets(1)=1
        init_mets,metabolite=1
        asb_defroi_draw,0,/fill
  END

    'Window 2.CHO': BEGIN
        widget_control,win2l_w,set_value=mettitles(2)
;Draw map
        s.mets(1)=2
        init_mets,metabolite=2
        asb_defroi_draw,0,/fill
    END

    'Window 2.LAC': BEGIN
        widget_control,win2l_w,set_value=mettitles(3)
;Draw map
        s.mets(1)=3
        init_mets,metabolite=3
        asb_defroi_draw,0,/fill
    END

    'Window 2.INO': BEGIN
        widget_control,win2l_w,set_value=mettitles(4)
;Draw map
        s.mets(1)=4
        init_mets,metabolite=4
        asb_defroi_draw,0,/fill
    END

    'Window 2.Water I1': BEGIN
        widget_control,win2l_w,set_value=mettitles(5)
;Draw map
        s.mets(1)=5
        init_mets,metabolite=5
        asb_defroi_draw,0,/fill
    END

    'Window 2.Water I2': BEGIN
        widget_control,win2l_w,set_value=mettitles(6)
;Draw map
        s.mets(1)=6
        init_mets,metabolite=6
        asb_defroi_draw,0,/fill
    END

    'Window 2.Water M1': BEGIN
        widget_control,win2l_w,set_value=mettitles(7)
;Draw map
        s.mets(1)=7
        init_mets,metabolite=7
        asb_defroi_draw,0,/fill
    END

    'Window 2.Water M2': BEGIN
        widget_control,win2l_w,set_value=mettitles(8)
;Draw map
        s.mets(1)=8
        init_mets,metabolite=8
        asb_defroi_draw,0,/fill
    END

    'Window 2.Water P1': BEGIN
        widget_control,win2l_w,set_value=mettitles(9)
;Draw map
        s.mets(1)=9
        init_mets,metabolite=9
        asb_defroi_draw,0,/fill
    END

    'Window 2.Water P2': BEGIN
        widget_control,win2l_w,set_value=mettitles(10)
;Draw map
        s.mets(1)=10
        init_mets,metabolite=10
        asb_defroi_draw,0,/fill
    END

;Window3
    'Window 3.NAA': BEGIN
        widget_control,win3l_w,set_value=mettitles(0)
;Draw map
        s.mets(2)=0
        init_mets,metabolite=0
        asb_defroi_draw,0,/fill
    END

    'Window 3.CRE': BEGIN
        widget_control,win3l_w,set_value=mettitles(1)
;Draw map
        s.mets(2)=1
        init_mets,metabolite=1
        asb_defroi_draw,0,/fill
  END

    'Window 3.CHO': BEGIN
        widget_control,win3l_w,set_value=mettitles(2)
;Draw map
        s.mets(2)=2
        init_mets,metabolite=2
        asb_defroi_draw,0,/fill
    END

    'Window 3.LAC': BEGIN
        widget_control,win3l_w,set_value=mettitles(3)
;Draw map
        s.mets(2)=3
        init_mets,metabolite=3
        asb_defroi_draw,0,/fill
    END

    'Window 3.INO': BEGIN
        widget_control,win3l_w,set_value=mettitles(4)
;Draw map
        s.mets(2)=4
        init_mets,metabolite=4
        asb_defroi_draw,0,/fill
    END

    'Window 3.Water I1': BEGIN
        widget_control,win3l_w,set_value=mettitles(5)
;Draw map
        s.mets(2)=5
        init_mets,metabolite=5
        asb_defroi_draw,0,/fill
    END

    'Window 3.Water I2': BEGIN
        widget_control,win3l_w,set_value=mettitles(6)
;Draw map
        s.mets(2)=6
        init_mets,metabolite=6
        asb_defroi_draw,0,/fill
    END

    'Window 3.Water M1': BEGIN
        widget_control,win3l_w,set_value=mettitles(7)
;Draw map
        s.mets(2)=7
        init_mets,metabolite=7
        asb_defroi_draw,0,/fill
    END

    'Window 3.Water M2': BEGIN
        widget_control,win3l_w,set_value=mettitles(8)
;Draw map
        s.mets(2)=8
        init_mets,metabolite=8
        asb_defroi_draw,0,/fill
    END

    'Window 3.Water P1': BEGIN
        widget_control,win3l_w,set_value=mettitles(9)
;Draw map
        s.mets(2)=9
        init_mets,metabolite=9
        asb_defroi_draw,0,/fill
    END

    'Window 3.Water P2': BEGIN
        widget_control,win3l_w,set_value=mettitles(10)
;Draw map
        s.mets(2)=10
        init_mets,metabolite=10
        asb_defroi_draw,0,/fill
    END

;Window4
    'Window 4.NAA': BEGIN
        widget_control,win4l_w,set_value=mettitles(0)
;Draw map
        s.mets(3)=0
        init_mets,metabolite=0
        asb_defroi_draw,0,/fill
    END

    'Window 4.CRE': BEGIN
        widget_control,win4l_w,set_value=mettitles(1)
;Draw map
        s.mets(3)=1
        init_mets,metabolite=1
        asb_defroi_draw,0,/fill
  END

    'Window 4.CHO': BEGIN
        widget_control,win4l_w,set_value=mettitles(2)
;Draw map
        s.mets(3)=2
        init_mets,metabolite=2
        asb_defroi_draw,0,/fill
    END

    'Window 4.LAC': BEGIN
        widget_control,win4l_w,set_value=mettitles(3)
;Draw map
        s.mets(3)=3
        init_mets,metabolite=3
        asb_defroi_draw,0,/fill
    END

    'Window 4.INO': BEGIN
        widget_control,win4l_w,set_value=mettitles(4)
;Draw map
        s.mets(3)=4
        init_mets,metabolite=4
        asb_defroi_draw,0,/fill
    END

    'Window 4.Water I1': BEGIN
        widget_control,win4l_w,set_value=mettitles(5)
;Draw map
        s.mets(3)=5
        init_mets,metabolite=5
        asb_defroi_draw,0,/fill
    END

    'Window 4.Water I2': BEGIN
        widget_control,win4l_w,set_value=mettitles(6)
;Draw map
        s.mets(3)=6
        init_mets,metabolite=6
        asb_defroi_draw,0,/fill
    END

    'Window 4.Water M1': BEGIN
        widget_control,win4l_w,set_value=mettitles(7)
;Draw map
        s.mets(3)=7
        init_mets,metabolite=7
        asb_defroi_draw,0,/fill
    END

    'Window 4.Water M2': BEGIN
        widget_control,win4l_w,set_value=mettitles(8)
;Draw map
        s.mets(3)=8
        init_mets,metabolite=8
        asb_defroi_draw,0,/fill
    END

    'Window 4.Water P1': BEGIN
        widget_control,win4l_w,set_value=mettitles(9)
;Draw map
        s.mets(3)=9
        init_mets,metabolite=9
        asb_defroi_draw,0,/fill
    END

    'Window 4.Water P2': BEGIN
        widget_control,win4l_w,set_value=mettitles(10)
;Draw map
        s.mets(3)=10
        init_mets,metabolite=10
        asb_defroi_draw,0,/fill
    END

    'Echo Time.Long ':BEGIN
        widget_control,winet_w,set_value=spectitles(0)
        plotparms(0)=0
    END

    'Echo Time.Short':BEGIN
        widget_control,winet_w,set_value=spectitles(1)
        plotparms(0)=1
    END

    'Domain.Raw      ':BEGIN
        widget_control,windom_w,set_value=spectitles(2)
        plotparms(1)=0
    END

    'Domain.Time     ':BEGIN
        widget_control,windom_w,set_value=spectitles(3)
        plotparms(1)=1
    END

    'Domain.Frequency':BEGIN
        widget_control,windom_w,set_value=spectitles(4)
        plotparms(1)=2
    END

    'Component.Magnitude':BEGIN
        widget_control,wincomp_w,set_value=spectitles(5)
        plotparms(2)=0
    END

    'Component.Phase    ':BEGIN
        widget_control,wincomp_w,set_value=spectitles(6)
        plotparms(2)=1
    END

    'Component.Real     ':BEGIN
        widget_control,wincomp_w,set_value=spectitles(7)
        plotparms(2)=2
    END

    'Component.Imaginary':BEGIN
        widget_control,wincomp_w,set_value=spectitles(8)
        plotparms(2)=3
    END

ENDCASE

END

;This subroutine creates the settings widget.
pro display_settings
COMMON dsettings
COMMON defaults
COMMON defroi
COMMON initcsi

cts=['Greyscale','Blue-Red','BGR','Bluscale']
dsetbase_w=widget_base(column=1,title='Display Settings') ; Base widget
                                ;Select metabolite map color table 
setcolort2_w=cw_bgroup(dsetbase_w,label_left='Metabolite Map Color Table',cts,$
                       set_value=table2,/exclusive,row=1,/no_release)
                                ;Select MIR color table 
setcolort1_w=cw_bgroup(dsetbase_w,label_left='MRI Color Table',cts,$
                       set_value=table1,/exclusive,row=1,/no_release)
                                ;Select ROI color table
setcolort3_w=cw_bgroup(dsetbase_w,label_left='ROI Color Table',cts,$
                       set_value=table3,/exclusive,row=1,/no_release)
IF table1 NE table2 THEN widget_control,setcolort3_w,sensitive=0

foo_w=widget_base(dsetbase_w,/row)
spectitle_w=widget_label(foo_w,value='Spectrum Window:')
specl_w=widget_base(dsetbase_w,row=1)

;Echo menu
menu_desc_w = [ $
                { CW_PDMENU_S,       3, 'Echo Time' }, $ ;   0
                { CW_PDMENU_S,       0, 'Long ' }, $ ;       1
                { CW_PDMENU_S,       2, 'Short' } $ ;        2
               ]

pdmenuet_w = cw_pdmenu( specl_w, menu_desc_w, /return_full_name)
winet_w = widget_label( specl_w, value=spectitles(plotparms(0)))

;Domain menu
menu_desc_w = [ $
                { CW_PDMENU_S,       3, 'Domain' }, $ ;           0
                { CW_PDMENU_S,       0, 'Raw      ' }, $ ;        1
                { CW_PDMENU_S,       0, 'Time     ' }, $ ;        2
                { CW_PDMENU_S,       2, 'Frequency' } $ ;         3
               ]

pdmenudom_w = cw_pdmenu( specl_w, menu_desc_w, /return_full_name)
windom_w = widget_label( specl_w, value=spectitles(plotparms(1)+2))

;Component menu
menu_desc_w = [ $
                { CW_PDMENU_S,       3, 'Component' }, $ ;       0
                { CW_PDMENU_S,       0, 'Magnitude' }, $ ;       1
                { CW_PDMENU_S,       0, 'Phase    ' }, $ ;       2
                { CW_PDMENU_S,       0, 'Real     ' }, $ ;       3
                { CW_PDMENU_S,       2, 'Imaginary' } $ ;        4
               ]

pdmenucomp_w = cw_pdmenu( specl_w, menu_desc_w, /return_full_name)
wincomp_w = widget_label( specl_w, value=spectitles(plotparms(2)+5))

row1_w=widget_base(dsetbase_w,row=1)
lfspec_w=cw_field(row1_w,title="Left frequency limit (ppm):",$
                value=last_ppm_lf,/return_events,/float)
rtspec_w=cw_field(row1_w,title="Right frequency limit (ppm):",$
                value=last_ppm_rt,/return_events,/float)

mxspec_w=cw_field(dsetbase_w,title="Maximum value of spectrum to plot:",$
                  value=max_spec_val,/float,/return_events)

row2_w=widget_base(dsetbase_w,row=1)
min_w(0) = cw_field(row2_w,title='MRI 1: Min %',value=min_v(0),/return,xsize=4,/int)
max_w(0) = cw_field(row2_w,title='Max %',value=max_v(0),/return,xsize=4,/int)
bswap_w(0) = cw_bgroup(row2_w,["No","Yes"],label_left="Byteswap:",row=1,$
                       /exclusive,set_value=bswap(0),/no_release)
                 
row3_w=widget_base(dsetbase_w,row=1)
min_w(1) = cw_field(row3_w,title='MRI 2: Min %',value=min_v(1),/return,xsize=4,/int)
max_w(1) = cw_field(row3_w,title='Max %',value=max_v(1),/return,xsize=4,/int)
bswap_w(1) = cw_bgroup(row3_w,["No","Yes"],label_left="Byteswap:",row=1,$
                       /exclusive,set_value=bswap(1),/no_release)
                 
row4_w=widget_base(dsetbase_w,row=1)
min_w(2) = cw_field(row4_w,title='MRI 3: Min %',value=min_v(2),/return,xsize=4,/int)
max_w(2) = cw_field(row4_w,title='Max %',value=max_v(2),/return,xsize=4,/int)
bswap_w(2) = cw_bgroup(row4_w,["No","Yes"],label_left="Byteswap:",row=1,$
                       /exclusive,set_value=bswap(2),/no_release)
                 
row31_w=widget_base(dsetbase_w,column=1)

win_w=widget_base(row31_w,row=1)
menu_desc_w = [ $
                { CW_PDMENU_S,       3, 'Window 1' }, $ ;        0
                { CW_PDMENU_S,       0, 'NAA' }, $ ;        1
                { CW_PDMENU_S,       0, 'CRE' }, $ ;        2
                { CW_PDMENU_S,       0, 'CHO' }, $ ;        3
                { CW_PDMENU_S,       0, 'LAC' }, $ ;        4
                { CW_PDMENU_S,       0, 'INO' }, $ ;        5
                { CW_PDMENU_S,       0, 'Water I1' }, $ ;    6
                { CW_PDMENU_S,       0, 'Water I2' }, $ ;    7
                { CW_PDMENU_S,       0, 'Water M1' }, $ ;    8
                { CW_PDMENU_S,       0, 'Water M2' }, $ ;    9
                { CW_PDMENU_S,       0, 'Water P1' }, $ ;    10
                { CW_PDMENU_S,       2, 'Water P2' } $ ;     11
                ]

pdmenu1_w = cw_pdmenu( win_w, menu_desc_w, /return_full_name)
win1l_w = widget_label( win_w, value=mettitles(s.mets(0)))


menu_desc_w = [ $
                { CW_PDMENU_S,       3, 'Window 2' }, $ ;        0
                { CW_PDMENU_S,       0, 'NAA' }, $ ;        1
                { CW_PDMENU_S,       0, 'CRE' }, $ ;        2
                { CW_PDMENU_S,       0, 'CHO' }, $ ;        3
                { CW_PDMENU_S,       0, 'LAC' }, $ ;        4
                { CW_PDMENU_S,       0, 'INO' }, $ ;        5
                { CW_PDMENU_S,       0, 'Water I1' }, $ ;    6
                { CW_PDMENU_S,       0, 'Water I2' }, $ ;    7
                { CW_PDMENU_S,       0, 'Water M1' }, $ ;    8
                { CW_PDMENU_S,       0, 'Water M2' }, $ ;    9
                { CW_PDMENU_S,       0, 'Water P1' }, $ ;    10
                { CW_PDMENU_S,       2, 'Water P2' } $ ;     11
                 ]

pdmenu2_w = cw_pdmenu( win_w, menu_desc_w, /return_full_name)
win2l_w = widget_label( win_w, value=mettitles(s.mets(1)))


menu_desc_w = [ $
                { CW_PDMENU_S,       3, 'Window 3' }, $ ;        0
                { CW_PDMENU_S,       0, 'NAA' }, $ ;        1
                { CW_PDMENU_S,       0, 'CRE' }, $ ;        2
                { CW_PDMENU_S,       0, 'CHO' }, $ ;        3
                { CW_PDMENU_S,       0, 'LAC' }, $ ;        4
                { CW_PDMENU_S,       0, 'INO' }, $ ;        5
                { CW_PDMENU_S,       0, 'Water I1' }, $ ;    6
                { CW_PDMENU_S,       0, 'Water I2' }, $ ;    7
                { CW_PDMENU_S,       0, 'Water M1' }, $ ;    8
                { CW_PDMENU_S,       0, 'Water M2' }, $ ;    9
                { CW_PDMENU_S,       0, 'Water P1' }, $ ;    10
                { CW_PDMENU_S,       2, 'Water P2' } $ ;     11
                 ]

pdmenu3_w = cw_pdmenu( win_w, menu_desc_w, /return_full_name)
win3l_w = widget_label( win_w, value=mettitles(s.mets(2)))


menu_desc_w = [ $
                { CW_PDMENU_S,       3, 'Window 4' }, $ ;        0
                { CW_PDMENU_S,       0, 'NAA' }, $ ;        1
                { CW_PDMENU_S,       0, 'CRE' }, $ ;        2
                { CW_PDMENU_S,       0, 'CHO' }, $ ;        3
                { CW_PDMENU_S,       0, 'LAC' }, $ ;        4
                { CW_PDMENU_S,       0, 'INO' }, $ ;        5
                { CW_PDMENU_S,       0, 'Water I1' }, $ ;    6
                { CW_PDMENU_S,       0, 'Water I2' }, $ ;    7
                { CW_PDMENU_S,       0, 'Water M1' }, $ ;    8
                { CW_PDMENU_S,       0, 'Water M2' }, $ ;    9
                { CW_PDMENU_S,       0, 'Water P1' }, $ ;    10
                { CW_PDMENU_S,       2, 'Water P2' } $ ;     11
                ]

pdmenu4_w = cw_pdmenu( win_w, menu_desc_w, /return_full_name)
win4l_w = widget_label( win_w, value=mettitles(s.mets(3)))


max_cre_w=cw_field(row31_w,title="Maximum CRE value in map:",value=max_cre,$
                   /float,/return)
max_cho_w=cw_field(row31_w,title="Maximum CHO value in map:",value=max_cho,$
                   /float,/return)
max_naa_w=cw_field(row31_w,title="Maximum NAA value in map:",value=max_naa,$
                   /float,/return)

CASE s.mets(3) OF
    3:max_lac_w=cw_field(row31_w,title="Maximum LAC value in map:",value=max_lac,$
                   /float,/return)

    4:max_ino_w=cw_field(row31_w,title="Maximum INO value in map:",value=max_ino,$
                   /float,/return)

    5:BEGIN
        min_water_w=cw_field(row31_w,title="Minimum WATER value in map:",value=min_water,$
                   /float,/return)

        max_water_w=cw_field(row31_w,title="Maximum WATER value in map:",value=max_water,$
                   /float,/return)
    END

    6: BEGIN
        min_b0_w=cw_field(row31_w,title="Minimum frequency shift in B0 map (ppm):",$
                          value=min_b0,$
                          /float,/return)

        max_b0_w=cw_field(row31_w,title="Maximum frequency shift in B0 map (ppm):",$
                          value=max_b0,$
                          /float,/return)
    END

    ELSE:
ENDCASE

dsetquit_w=widget_button(dsetbase_w,value='Quit') ;Quit widget

widget_control,dsetbase_w,/realize ;Realize widgets

xmanager,'display_settings',dsetbase_w,event='dsettings_event',/modal ;Register widgets

END

