;The functions in this file extract the roots of a complex polynomial.
;The input parameters for jwrooter are:
;a      list of coefficients of the polynomial
;lowb   lower bound on imaginary part of root
;highb  upper bound on imaginary part of root
;polish flag to polish roots
;REVISION HISTORY
; 981009 ASB First SCCS version. Original code by JWV.


function laguer,a,m,x,its

EPSS=double(1.7e-7)
MR=8
MT=30
MAXIT=MT*MR

frac=dblarr(MR+1)
frac(0)=0.0
frac(1)=0.5
frac(2)=0.25
frac(3)=0.75
frac(4)=0.13
frac(5)=0.38
frac(6)=0.62
frac(7)=0.88
frac(8)=1.0

for iter=0,MAXIT-1 do begin
  its=iter
  b=a(m)
  err=abs(b)
  d=dcomplex(0.,0.)
  f=dcomplex(0.,0.)
  abx=abs(x)
  for j=m-1,0,-1 do begin
    f=x*f+d
    d=x*d+b
    b=x*b+a(j)
    err=abs(b)+abx*err
  endfor
  err=err*EPSS

;Estimate of roundoff error in evaluating polynomal

  if (abs(b) le err) then return,1
  g=d/b
  g2=g*g
  h=g2-(2.0*(f/b))
  sq=sqrt( double(m-1)*(double(m)*h-g2) )
  gp=g+sq
  gm=g-sq
  abp=abs(gp)
  abm=abs(gm)
  if (abp lt abm) then gp=gm
  
  if (max([abp,abm]) gt 0.0) then begin
    dx = dcomplex(double(m),0.)/gp
  endif else begin
    dx = 0.05*dcomplex(cos(double(iter)),sin(double(iter))) ;jw
  endelse
  dxnorm=fix(abs(dx))
  if (dxnorm gt 0) then dx=dx/double(dxnorm)
  x1=x-dx
  if (double(x) eq double(x1) and imaginary(x) eq imaginary(x1)) then return,1
  if ( iter-fix(iter/MT)*MT ne 0) then x=x1 else x=x-frac(iter/MT)*dx
endfor
print,'too many iterations in laguer'
return,0
end


function jwrooter,a,lowb,highb,polish=polish

m=n_elements(a)-1
EPS=double(2.e-6)
MAXM=200
MINr =20

roots=dcomplexarr(m)
ad=dcomplexarr(MAXM)
angle=double((highb+lowb)/2.)
for j=0,m do ad(j)=a(j)

lastroot=m-1
for j=m,1,-1 do begin

  x=0.99*dcomplex (cos(angle),sin(angle))     ; starting value for poles on unit circle 1,0
  error=laguer(ad,j,x,its)
  rfreq=imaginary(alog(x))
  minr=minr-1
  if ( ((rfreq gt highb) or (rfreq lt lowb)) ) then goto,endrooting
;and (minr lt 1)
  roots(j-1)=x
  lastroot=j-1
  b=ad(j)
  for jj=j-1,0,-1 do begin
    c=ad(jj)
    ad(jj)=b
    b=c+x*b
  endfor
endfor

endrooting: if (keyword_set(polish)) then for j=m,lastroot+1,-1 do error=laguer(a,m,roots(j-1),its)

for j=2,m do begin
  x=roots(j-1)
  i=j-1
  for i=j-1,1,-1 do begin
    if (double(roots(i-1)) le double(x)) then goto,exitloop
    roots(i)=roots(i-1)
  endfor
exitloop: roots(i)=x
endfor
return,roots
end
