;This file contains routines for eliminating spectra outside the brain
;on the basis of the pixel values in a magnitude image.
;Revision History
; 980915 ASB Replaced system variables with common initcsi.
; 981014 ASB Added code to choose image to use for mask.
;            Added code to use water map.
; 000602 ASB Fixed overflow bug for initial plot. 
; 000602 ASB Fixed bug in thresholding.
pro replot,mmri
COMMON mask
COMMON initcsi
COMMON ctable

widget_control,mask_s.thresh_w,get_value=thresh
widget_control, mask_s.pvalue_w, get_value=line
widget_control, mask_s.draw_w, get_value=window_num
wset,window_num
device,set_graphic=3
widget_control, mask_s.rc_w, get_value=rc
IF rc EQ 0 THEN BEGIN
    plot,mmri(*,line),back=0,color=coffset2+6
ENDIF ELSE BEGIN
    plot,mmri(line,*),back=0,color=coffset2+6
ENDELSE
oplot,[0,256],[thresh,thresh],color=coffset2+6
END

;This file contains programs that create a modal dialog box for
;choosing a threshold on an mri image and using the image as a mask
;for excluding spectra.
;Revision History
;09Aug96 ASB First SCCS version
;10Mar98 ASB Removed reference to particular map size.
pro mask_event,event
COMMON posdata
COMMON mask
COMMON initcsi
COMMON mrsidata
widget_control,mask_s.guide_w,get_value=value
IF value EQ 0 THEN BEGIN
    mmri=rebin(sqrt(total(abs(td_dat)^2,1)),256,256)
ENDIF ELSE BEGIN
    mmri=mri(*,*,value-1)    
ENDELSE
replot,mmri*100./float(max(mmri))

thresh1=50.
CASE event.id OF
    mask_s.draw_w:BEGIN
        IF event.press EQ 1 THEN BEGIN
            widget_control, mask_s.draw_w, get_value=window_num
            wset,window_num
            result=convert_coord(event.x,event.y,/device,/to_data)
            widget_control,mask_s.thresh_w,set_value=result(1)
            replot,mmri*100./float(max(mmri))
        END
    END
    mask_s.guide_w:BEGIN
        IF event.value GT 0 THEN mmri=mri(*,*,event.value-1) $
        ELSE mmri=rebin(sqrt(total(abs(td_dat)^2,1)),256,256)
        replot,mmri*100./float(max(mmri))
    END

    mask_s.thresh_w:replot,mmri*100./float(max(mmri))
    mask_s.rc_w:replot,mmri*100./float(max(mmri))
    mask_s.pvalue_w:BEGIN
        widget_control, mask_s.pvalue_w,get_value=value
        value=fix(value)
        widget_control, mask_s.pvalue_w,set_value=strtrim(string(value),2)
        replot,mmri*100./float(max(mmri))
    END
    mask_s.execute_w:BEGIN
        widget_control,mask_s.thresh_w,get_value=value
        index=where(mmri*100. GT float(value(0))*max(mmri))
        dummy=intarr(256,256)
        dummy(index)=1
        nx=256/pe_dim(0)
        ny=256/pe_dim(1)
        dummy=reform(dummy,nx,pe_dim(0),ny,pe_dim(1))
        dummy=total(total(dummy,1),2)
        widget_control,mask_s.pix_w,get_value=thresh1
        IF thresh1(0) EQ 100 THEN thresh1(0) = 99
        index=where(dummy LE thresh1(0)*nx*ny/100)
        positions(index)=-1
        widget_control,mask_s.base_w,/destroy
        init_mets
    END
    mask_s.quit_w: BEGIN
        widget_control,mask_s.base_w,/destroy
    END
ENDCASE
END



pro mask
COMMON posdata
COMMON mask
COMMON mrsidata

line=128
thresh=15
thresh1=50

;Only one copy can exist at a time; test to see if mask is running
IF xregistered('mask') NE 0 THEN return
base_w=widget_base(/column,title='Mask')
draw_w=widget_draw(base_w,/button,xsize=480,ysize=300,/retain)
val=0
val1=0
val2=0
val3=0
IF total(mri(*,*,0)) NE 0 THEN BEGIN
    val=1
    val1=1
ENDIF
IF total(mri(*,*,2)) NE 0 THEN BEGIN
    val=3
    val3=1
ENDIF
IF total(mri(*,*,1)) NE 0 THEN BEGIN
    val=2
    val2=1
ENDIF
guide_w=cw_bgroup(base_w,['Water map','MRI 1','MRI 2','MRI 3'],$
                  /exclusive,label_left='Image:',set_value=val,$
                  ids=gids)
thresh_w=cw_field(base_w,title='Threshold',value=thresh,/int,/return)
plot_w=widget_base(base_w,/row)
rc_w=cw_bgroup(plot_w,['Row','Column'],/row,/return_name,/exclusive,set_value=0)
pvalue_w=widget_text(plot_w,/edit,value=strtrim(string(line),2))
pix_w=cw_field(base_w,title='MRI fraction threshold',value=thresh1,/int)
execute_w=widget_button(base_w,value='Execute')
quit_w=widget_button(base_w,value='Quit')
widget_control,base_w,/realize
;Initialize plot
mask_s={base_w:base_w,$
        draw_w:draw_w,$
        guide_w:guide_w,$
        gids:gids,$
        thresh_w:thresh_w,$
        rc_w:rc_w,$
        pvalue_w:pvalue_w,$
        execute_w:execute_w,$
        pix_w:pix_w,$
        quit_w:quit_w}

wat=sqrt(total(td_dat^2,1))
IF total(wat) EQ 0 THEN BEGIN
    xyouts,10,200,'You must first load the MRSI data.',/dev,charsize=2
    widget_control,draw_w,sensitive=0
    widget_control,thresh_w,sensitive=0
    widget_control,rc_w,sensitive=0
    widget_control,pvalue_w,sensitive=0
    widget_control,execute_w,sensitive=0
    widget_control,pix_w,sensitive=0
    FOR kk=0,3 DO widget_control,gids(kk),sensitive=0
ENDIF ELSE IF val EQ 0 THEN BEGIN
    replot,rebin(wat*100./float(max(wat)),256,256)
    FOR kk=0,2 DO BEGIN
        IF total(mri(*,*,kk)) EQ 0 THEN widget_control,gids(kk+1),sensitive=0
    ENDFOR
ENDIF ELSE BEGIN
    replot,mri(*,*,val-1)*100./float(max(mri(*,*,val-1)))
    IF val1 EQ 0 THEN widget_control,gids(1),sensitive=0
    IF val2 EQ 0 THEN widget_control,gids(2),sensitive=0
    IF val3 EQ 0 THEN widget_control,gids(3),sensitive=0
ENDELSE

xmanager,'mask',base_w
END
