;This program reads the raw data header of files acquired with
;mpcsi56.
;Revision History
; 980724 asb First version.
; 980724 asb Changed strings in value assignments to reflect cw_field widgets.
; 980916 asb Replaced mri_file argument with COMMON initcsi.
; 980916 asb Replaced system variables, added COMMON header.
; 981009 asb Added code to read data type.
;            Added code to read spectrometer frequency.
;            Added keyword lx to read headers from lx files.
;            Added code to identify PSD as lx if image header is blank.
; 990921 asb Added code to read new rh cv's.
;            Fixed bug in reading header info for multiple files.
; 991122 asb Changed code that read OVS parameters.
;            Added double echo stuff.
;            Added shim stuff.
;            Added initialization of paxis, echo_pt, echo_pt1.
pro read_raw_header,lx=lx
COMMON head_inf
COMMON initcsi
COMMON header
COMMON defroi
COMMON mets

examoff=36872l                  ;offset to exam header

seriesoff=examoff+1024
imageoff=seriesoff+1020
IF keyword_set(lx) THEN imageoff=imageoff+8

openr,unit,csi_file,/get        ;open file

point_lun,unit,imageoff+308
foo=bytarr(33)
readu,unit,foo
study.PSD=string(foo)

IF study.PSD EQ '' OR n_elements(LX) NE 0 THEN BEGIN
    version=5
    seriesoff=seriesoff+16
    examoff=examoff+4
    study.PSD='mpcsilx'
ENDIF ELSE version=0

;Read date and time
point_lun,unit,16
foo=bytarr(10)
readu,unit,foo
study.date=string(foo)

foo=bytarr(8)
readu,unit,foo
study.time=string(foo)

;Read data type
ptsize=0
point_lun,unit,82
readu,unit,ptsize
IF bsflag THEN byteorder,ptsize

;read rhuser0-rhuser19
point_lun,unit,216
rh1b=fltarr(20)
readu,unit,rh1b
IF bsflag THEN byteorder,rh1b,/ftoxdr
ovs.thick = rh1b(0)  
ovs.b1 = rh1b(1)
ovs.b2 = rh1b(2)  
ovs.b3 = rh1b(3)  
ovs.b4 = rh1b(4)  
ovs.t1 = rh1b(5)  
ovs.t2 = rh1b(6)  
ovs.t3 = rh1b(7)  
ovs.t4 = rh1b(8) 
xres=rh1b(10)
yres=rh1b(11)
ovs.angle = -180.*atan(rh1b(15)/rh1b(14))/!pi

;Read rhuser20-rhuser48
rhoffset=20
point_lun,unit,1000
rh2b=fltarr(29)
readu,unit,rh2b
IF bsflag THEN byteorder,rh2b,/ftoxdr
rh1b=[rh1b,rh2b]

echomode=rh1b(20)
acq_delay=rh1b(21)
navflag=rh1b(22)
shimflag=rh1b(23)
specpts=rh1b(24)

shim.x1=rh1b(25)
shim.x2=rh1b(26)
shim.x3=rh1b(27)
shim.x4=rh1b(28)

shim.y1=rh1b(29)
shim.y2=rh1b(30)
shim.y3=rh1b(31)
shim.y4=rh1b(32)

shim.z1=rh1b(33)
shim.z2=rh1b(34)
shim.z3=rh1b(35)
shim.z4=rh1b(36)

shim.flag=rh1b(37)
shim.stepx=rh1b(38)
shim.stepy=rh1b(39)
shim.stepz=rh1b(40)
shim.nstep=rh1b(41)

;read spectrometer frequency
point_lun,unit,440
spec_freq=0l
readu,unit,spec_freq
IF bsflag THEN byteorder,spec_freq,/lswap
spec_freq=float(spec_freq)/1.e7

;Read from exam header
point_lun,unit,examoff+10
point_lun,-unit,ploc
print,'Point location = ',examoff+10,ploc
foo=bytarr(33)
readu,unit,foo
study.site=string(foo)

point_lun,unit,examoff+84
foo=bytarr(13)
readu,unit,foo
patient.ID=string(foo)

foo=bytarr(25)
readu,unit,foo
patient.name=string(foo)

foo=0
readu,unit,foo
IF bsflag THEN byteorder,foo
patient.age=foo

readu,unit,foo
readu,unit,foo
IF foo EQ 2 THEN patient.sex='F' $
ELSE patient.sex='M'

point_lun,unit,examoff+134
foo=bytarr(61)
readu,unit,foo
patient.Hx=string(foo)

point_lun,unit,examoff+245
foo=bytarr(33)
readu,unit,foo
patient.radiologist=string(foo)

point_lun,unit,examoff+282
foo=bytarr(23)
readu,unit,foo
study.desc=string(foo)

point_lun,unit,seriesoff+92
foo=bytarr(25)
readu,unit,foo
study.routine=string(foo)

close,unit                      ;close file

;New stuff 11/18
tsamp=1000.
sweep_width=1.e6/tsamp
ppm_offset=h2o_shift
freq_axis = ((-1*indgen(spc_len))/float(spc_len)+0.5)*sweep_width
paxis=freq_axis/spec_freq + ppm_offset

te=272000
echo_pt=(0.5*te-acq_delay)/tsamp
echo_pt1=0

IF widget_info(inf_head.labelq_w,/valid) THEN BEGIN
    widget_control,inf_head.labelq_w,set_value=study.site
    widget_control,inf_head.label0_w,set_value=patient.name
    widget_control,inf_head.label1_w,set_value=patient.ID
    widget_control,inf_head.label2_w,set_value=patient.age
    widget_control,inf_head.label3_w,set_value=patient.sex
    widget_control,inf_head.label4_w,set_value=patient.Hx
    widget_control,inf_head.label5_w,set_value=study.date
    widget_control,inf_head.label6_w,set_value=study.time
    widget_control,inf_head.label7_w,set_value=study.PSD
    widget_control,inf_head.label8_w,set_value=study.routine
    widget_control,inf_head.label9_w,set_value=study.desc
    widget_control,inf_head.labe3_w,$
      set_value=STRING(FORMAT='(I4,I4,I4,I4,I4,I4,I4,I4)',$
                       ovs.b1,ovs.b2,ovs.b3,ovs.b4,ovs.t1,ovs.t2,ovs.t3,ovs.t4)
    widget_control,inf_head.labe4_w,set_value=ovs.angle    
    widget_control,inf_head.labe5_w,$
      set_value=STRING(FORMAT='(I4,I4,I4)',aps.xshim,aps.yshim,aps.zshim)    
    widget_control,inf_head.labe6_w,$
      set_value=STRING(FORMAT='(I5,I5)',aps.ws,aps.sat) 
    widget_control,inf_head.labe7_w,set_value=ptsize
ENDIF

END
