;This program reads the mrsi data and calls the reconstruction programs.
;Revision History
;09Aug96 ASB First SCCS version.
;18Jul97 ASB Added code to read 3dcsi data.
;27Feb98 ASB Added code to permit user to choose apodization.
;23Jul98 ASB Added call to comp_noise.
;            Added call to clear_all.
;11Aug98 ASB Added call to name_sumfile.
;14Sep98 ASB Made copy of input array into fd_data in common block freqdomain.
;980915  ASB Added COMMON initcsi and removed system variables.
;980916  ASB Changed name of COMMON freqdomain to mrsidata.
;980918  ASB Added COMMON dsettings.
;980921  ASB Now use rixxx to read duyn data.
;            Added call to recon settings.
;            Added code to initialize csi_dat.
;            Added code to read lx files.
;990303  ASB Removed reference to s.psd_w.
;            Added time domain flag.
;            Added some comments.
;            Removed some commented out code.
;            Added ELSE to CASE statement to prevent crashes.
;991118  ASB Added code to handle FID and off-centered echo.
;000602  ASB Added code to erase old metabolite maps and postitions.

pro readandrecon
COMMON defroi
COMMON defaults
COMMON mrsidata
COMMON initcsi
COMMON dsettings
COMMON posdata

recon_settings

;Revert to frequency domain
tdflag=0
widget_control,s.tf_w,set_value=s.tf_labels(0),set_uvalue=0

clear_all

IF version EQ 6 THEN BEGIN
;reinitialize parameters for Elfar Adalsteinsson's sequence
    sweep_width=float(326*2)
    freq_axis = ((-1*indgen(spc_len))/float(spc_len)+0.5)*sweep_width
    ppm_offset=4.69
    paxis(*)= freq_axis/spec_freq + ppm_offset

    mn=(spc_len-1)*(last_ppm_lf-paxis(0))/(paxis(spc_len-1)-paxis(0))
    mx=(spc_len-1)*(last_ppm_rt-paxis(0))/(paxis(spc_len-1)-paxis(0))
    widget_control,s.slider_w,set_slider_max=mx,set_slider_min=mn

    widget_control,s.dfilel_w,set_uvalue='old',get_value=csifile,/hourglass

    data=intarr(2,256,pe_dim(0),pe_dim(1))
    openr,unit,csifile,/get
    readu,unit,data
    IF bsflag THEN byteorder,data
    free_lun,unit
    csi_dat=complexarr(spc_len, pe_dim(0), pe_dim(1))
    FOR kk=0,pe_dim(0)-1 DO FOR ll=0,pe_dim(1)-1 DO $
      csi_dat(256:*,kk,ll)=$
      rotate(complex(data(0,*,kk,pe_dim(1)-1-ll),data(1,*,kk,pe_dim(1)-1-ll)),2)

ENDIF ELSE BEGIN

;reinitialize parameters for Jeff Duyn's sequence

    sweep_width=1000.0

    freq_axis = ((-1*indgen(spc_len))/float(spc_len)+0.5)*sweep_width
    ppm_offset=4.75
    paxis(*)= freq_axis/spec_freq + ppm_offset

    mn=(spc_len-1)*(last_ppm_lf-paxis(0))/(paxis(spc_len-1)-paxis(0))
    mx=(spc_len-1)*(last_ppm_rt-paxis(0))/(paxis(spc_len-1)-paxis(0))
    widget_control,s.slider_w,set_slider_max=mx,set_slider_min=mn

    csi_dat=complexarr(spc_len, pe_dim(0), pe_dim(1))

    widget_control,s.dfilel_w,set_uvalue='old',get_value=csifile,/hourglass

; Find out version of input file
    CASE version OF
        0:BEGIN                 ;55
            rixxx,csifile,mpcsi=cur_slice
            FOR i = 1,pe_dim(1)-1,2 DO csi_dat(*,*,i)=-csi_dat(*,*,i)
;            widget_control,s.run_ids(0),sensitive=0
                                ; flip odd y phase encodes
        END

        1:BEGIN                 ;5xb
            CASE cur_slice OF
                1:slice=1
                2:slice=3
                3:slice=2
                4:slice=4
            ENDCASE
            riks,mpcsi=slice,csifile ;read raw data file
            FOR i = 1,pe_dim(1)-1,2 DO csi_dat(*,*,i)=-csi_dat(*,*,i)
        END

        2:BEGIN                 ;5xa
            CASE cur_slice OF
                1:slice=1
                2:slice=3
                3:slice=2
                4:slice=4
            ENDCASE
            riks,mpsci=slice,csifile ;read raw data file
            FOR i = 1,pe_dim(1)-1,2 DO csi_dat(*,*,i)=-csi_dat(*,*,i)
        END
        3:BEGIN                 ;5x
            CASE cur_slice OF
                1:slice=1
                2:slice=3
                3:slice=2
                4:slice=4
            ENDCASE
            riks,mpcsi=slice,csifile ;read raw data file
            FOR i = 1,pe_dim(1)-1,2 DO csi_dat(*,*,i)=-csi_dat(*,*,i)
        END
        4:BEGIN                 ;4x
            ri4x,mpcsi=cur_slice,csifile ;read raw data file
        END

        5:BEGIN                 ;LX
            rixxx,csifile,mpcsi=cur_slice,/lx
            FOR i = 1,pe_dim(1)-1,2 DO BEGIN
                csi_dat(*,*,i)=-csi_dat(*,*,i)
                csi_dat1(*,*,i)=-csi_dat1(*,*,i)
            ENDFOR
;            widget_control,s.run_ids(0),sensitive=0
                                ; flip odd y phase encodes
        END

        ELSE:

    ENDCASE

;Compute spatial window                                
    CASE filtsym OF
        0:type=0
        1:type=4
    ENDCASE

    apd=[sapd,sapd_defs(*,sapd)]
    swindow=mkfilter(apd,pe_dim(0),type=type)
    swindow=reform(swindow,pe_dim(0)*pe_dim(1))

;Compute temporal window

    apd=[tapd,tapd_defs(*,tapd)]
    twindow=mkfilter(apd,fid_len,xsize=spc_len) ;Temporal window

    IF etl EQ 2 THEN BEGIN ;Temporal window for FID
        apd=[tapd1,tapd1_defs(*,tapd1)]
        t1window=mkfilter(apd,fid_len,xsize=spc_len,type=2) 
    ENDIF

;Apply apodization
    IF version NE 0 AND version NE 5 THEN BEGIN
;Compute total window
        window=twindow#swindow
        td_dat=csi_dat*window
    ENDIF ELSE BEGIN
        td_dat=csi_dat
        IF etl EQ 2 THEN BEGIN
            td_dat1=csi_dat1
        ENDIF
    ENDELSE
;do fft's                                
    CASE version OF
        0:csift,swindow=swindow,twindow=twindow
        1:csift
        2:csift5xa
        3:csift5x
        4:csift4x
        5:csift,swindow=swindow,twindow=twindow,t1window=t1window
    ENDCASE
ENDELSE
;compute noise
comp_noise

;Reinitialize metabolite maps
positions(*)=-1
sumpeaks
init_mets

name_sumfile

RETURN
END
