PRO ri4x, filename, $
          file_names=file_names, $
          no_mri=no_mri,  $
          bin=bin, $
          mri=mri, $
          v45l=v45l, $
          v45i=v45i, $
          mpcsi=mpcsi, $
          no_dc=no_dc, $
          dummy_scans=d_scans, $
          extension=extension, $
          ver=ver
;
;  7/2/92  @(#)ri.pro	1.14  In Vivo NMR Research Center, NIH`
;
;  4/2/91  @(#)ri.pro	1.11 - Patched 12/6/91, 5/20/92
;  911106  GSS  Patch!  Not tracked with SCCS
;               Added keywords 'v45i' and 'v45l' to read
;               SIGNA 4.x integer or long data files
;  920520  GSS  Patch!  Not tracked with SCCS
;               Changed 'init_dir' to '/home'
;  920701  GSS  Returned to SCCS as delta 1.12
;
; This function reads in a CSI raw data file collected on the Signa and
; and transfered to the Sun via binary FTP.
;
; The Parameters are:
;   filename - sting containg the name of the data-file
;
; This program was modeled on a CSI image file reading program
; written by Jim Pekar (December, 1988).
;
;  130789  GSS  First writing.
;  101089  GSS  Adapted early signa image reading program.
;  891111  GSS  512 freq. points
;  891111  GSS  1024 freq. points
;  891112  GSS  8 initial background scans split out
;  891112  GSS  Made more versatle
;  891230  GSS  Make use of system variables defined in 'initcsi'
;               Do offset subtraction everytime
;  900101  GSS  Put output into system variable '!csi_dat'
;  900108  GSS  Added support for zero-filling.
;  900110  GSS  Added reading of proton image and adding correct
;               extension automatically.
;               Added keyword parameter 'file_names' to contain a
;               two element vector containing two strings:
;                  [ csi_file_name, mri_file_name ]
;               to be used instead of the automatically generated 
;               names.  A null 'mri_file_name' prevents the
;               reading of the image file.
;               Added keyword parameter 'no_mri' to prevent the
;               reading of the mri file.
;  900801  GSS  Modified for reading semi-monster 32x32 1024 point
;               data sets.
;  910125  GSS  Added '/bin' keyword.  When present, this will read
;               in an array stored with "wrbblk".
;  910205  GSS  Changed '/bin' reading code to read one FID at a time
;               into !csi_dat.  This *greatly* reduces the transient 
;               memory requirements!  Also, added support for reading
;               into a zero-filled !csi_dat array.
;               This fix should really be applied to the normal file
;               reading code, too.
;  910210  GSS  Added code to save header info. and extract some 
;               fields into a structure.  Also, changed the default
;               image read keyword from '/no_mri' to '/mri'.  No
;               image will be read unless the '/mri' keyword is present.
;               The '/no_mri' keyword is retained for compatibility.
;  910225  GSS  Added 'v4.5' keyword for 32 bit data with 31744 byte
;               headers from Signa 4.5
;  910227  GSS  Added a "bunch 'o stuff" from the orignal 'ri' and 
;               renamed 'ri32x32' to 'ri'.
;  910228  GSS  Added a call to 'select_file' if no name is specified
;               on the command line.
;  910301  GSS  Added keyword 'dummy_scans'.  Fixed code to add
;               dummy scans into header length for 3.5 files.
;  910301  GSS  Added keyword, 'extension' to allow the user to spcify
;               filename extension to expect.  This should be generalized
;               to allow any arbitrary wildcard expression.  But not now.
;  910402  GSS  Fixed reading of SIGNA 4.5+ data-files.  Previously had not
;               correctly evaluated dummy-scans.
;               KLUDGE:  Now assumes one "baseline frame" for SIGNA 4.5 type
;                        data files.
;               Also, moved zrray zeroing to the point just before the data
;               are read.
;  980915  ASB  Added COMMON initcsi and removed system variables.
;  980920  ASB  Removed references to file_type.
COMMON initcsi
IF strpos(foo(0),'i686') EQ 0  THEN bsflag = 1 ELSE bsflag = 0
;*******   Display SCCS Version   *******
IF keyword_set( ver ) THEN BEGIN
    print, 'Current SCCS Version for "ri": 1.14'
    RETURN
END

;*******   Create Filename   *******

IF keyword_set( extension ) THEN BEGIN
    extension = strtrim( string( extension ), 2)
ENDIF ELSE BEGIN
                                ;  Set default extensions depending on many things...
    IF keyword_set( bin ) THEN BEGIN
        extension = '.2dcsib'
    ENDIF ELSE IF keyword_set( mpcsi ) THEN BEGIN
        extension = '.mpcsi'
    ENDIF ELSE BEGIN
        extension = '.2dcsi'
    ENDELSE
ENDELSE

ext_len = strlen( extension )

IF NOT keyword_set( file_names ) THEN BEGIN

                                ;*****  No 'filenames=' keyword.

    IF n_params() EQ 0 THEN BEGIN

                                ;*****  No command line filename:

        init_dir = '/home'

    END ELSE BEGIN 

                                ;  Check for a directory name on the command line:
        spawn, 'if ( -d "'+filename+'" ) echo "dir"', dir_test

        IF ( dir_test(0) EQ 'dir' ) THEN BEGIN
            init_dir = filename ;  Use Command line name for data directory
        END ELSE BEGIN
            init_dir = ''       ;  Command line contains a file name
        END
    END

                                ;  Get a file-name using 'select_file', if not from cmd. line
    IF ( init_dir NE '' ) THEN BEGIN
                                ;  Get the file name using 'select_file':
        REPEAT BEGIN
            filename = pickfile( dir=init_dir )
            ext = strmid( filename, strlen(filename)-ext_len, ext_len)
            IF ext NE extension THEN BEGIN
                print, 'Invalid filename extension.'
                spawn, 'dirname '+filename, init_dir
                init_dir = init_dir(0)
            END
        END UNTIL ( ext EQ extension )

                                ;  Remove the '.2dcsi' extension
        filename = strmid( filename, 0, strlen(filename)-ext_len )
    END

    print, 'Base Datafile name "', filename, '".'

                                ;*****  Now, reassemble the filename:
    csi_file = filename+extension
    
END ELSE BEGIN

                                ;*****  'filenames=' keyword supplied.

    csi_file = file_names(0)
    proton_file = file_names(1)

END

IF keyword_set( mri ) THEN proton_file = filename+'.mri' $
ELSE proton_file = ''

;*******  Prepare File for Reading   *******

get_lun,unit                    ;get logical unit number
openr,unit,csi_file             ;open file

raw_len = fid_len*2


nul_fid = intarr( raw_len )
raw_fid = intarr( raw_len )
data_size = 2L
print, 'Reading integer (16 bit) data.'
;raw_header = bytarr(39940L)
raw_header = bytarr(27648L)
print, 'SIGNA 4.5 Header =', n_elements(raw_header), ' bytes'

index = indgen( fid_len ) * 2

                                ;*******   Read Raw Integer Data from File   *******

                                ;*** Clear Data Array ***
print, 'Zeroing data array...'
csi_dat(*) = complex(0,0)

readu, unit, raw_header		;read header

IF keyword_set( mpcsi ) THEN BEGIN
                                ; Skip past 'mpcsi' number of slices:
    print, 'Skipping to slice #', mpcsi
    FOR i = 1, (pe_dim(0)*pe_dim(1)+1)*(mpcsi-1)  DO BEGIN
        readu, unit, raw_fid
    ENDFOR
ENDIF
FOR i = 0, raw_len-1 DO BEGIN
    nul_fid(i)=0
ENDFOR

readu, unit, raw_fid

print, 'Reading and converting datafile...'
print, format='( "+", ' + string(pe_dim(1)-2) + '("-"), "+" )'
ymax=float(pe_dim(1)-1.0)/2.0
xmax=float(pe_dim(0)-1.0)/2.0
idm1=0
idm2=0
FOR j = 0, pe_dim(1)-1 DO BEGIN
    rady=(float(j)-ymax)*(float(j)-ymax)/(ymax*ymax)
    print, format='( $, "*" )'
    FOR i = 0, pe_dim(0)-1 DO BEGIN
        idm2=idm2+1
        radius=rady+((float(i)-xmax)*(float(i)-xmax)/(xmax*xmax))
        readu, unit, raw_fid
        IF bsflag THEN byteorder,rawfid
        IF radius LT 1.01 THEN BEGIN
            idm1=idm1+1
            csi_dat( 0, i, j ) = complex( raw_fid(index), raw_fid(index+1) )
        ENDIF
                                ; ELSE BEGIN
                                ;       csi_dat( 0, i, j ) = complex( nul_fid(index), nul_fid(index+1) )
                                ;    ENDELSE
    END
END
frac=float(idm1)/float(idm2)
print
print, 'riks fraction =', frac
print, 'pe_dim(0),pe_dim(1),raw_len', pe_dim(0),pe_dim(1),raw_len 
print
close,unit                      ;close file
free_lun,unit			;free logical unit number

                                ;*******  Extract Selected Header Fields:

print, 'Extracting fields from header...'
study  =  6 * 256*2
series =  8 * 256*2
image  = 10 * 256*2

;*******   Do DC Offset Subtraction   *******

IF  NOT keyword_set( no_dc ) THEN BEGIN
    print,  'Calculate DC Offset...'
    start = fid_len - 100
    stop = fid_len -1
    dc = total(csi_dat(start:stop,*,*))/n_elements(csi_dat(start:stop,*,*))
    
    print,  'DC Offset Correction...'
    FOR j = 0, pe_dim(1)-1 DO BEGIN
        rady=(float(j)-ymax)*(float(j)-ymax)/(ymax*ymax)
        FOR i = 0, pe_dim(0)-1 DO BEGIN
            radius=rady+((float(i)-xmax)*(float(i)-xmax)/(xmax*xmax))
            IF radius LT 1.01 THEN BEGIN
                csi_dat(0, i, j) = csi_dat(0:fid_len-1, i, j) - dc
            ENDIF
        END
    END 
END       

print,'riks called'
RETURN

END
