;This program reads the raw data for Jan Willem's Time domain analysis
;program.
;Revision History
; 980920 JWV Original version entered into SCCS.
; 980920 ASB Removed references to !file_type.
;            Added COMMON initcsi.
;            Removed commented out lines.
;            Removed superfluous code for generating filename.
;            Added COMMON header.
;            Removed logic for FAST acquisition.
; 980921 ASB Renormalized 32 bit data.
;            Replaced keywords with COMMON defaults.
; 990922 ASB Rewrote logic to remove bugs and improve improve handling
;            of short echo data.
; 991005 ASB Program now save both echoes in 2 echo acquisition.
PRO rixxx, filename, $
           mpcsi=mpcsi,$
           lx=lx

COMMON initcsi
COMMON header
COMMON mrsidata
COMMON defaults

headlen=39940
nksppts=772
IF keyword_set(lx) THEN headlen=headlen+44
nslices=4
sliceorder=[0,2,1,3]
;sliceorder=[3,1,0,2]
;sliceorder=[1,3,2,4]

;*******   Create Filename   *******

extension = '.mpcsi'
csi_file = filename+extension

openr,unit,csi_file,/get        ;open file
point_lun,unit,headlen

raw_len = fid_len*2
IF ptsize EQ 4 THEN BEGIN       ;32 bit data
    csi_dat = dcomplex(csi_dat)
    IF etl EQ 2 THEN csi_dat1=csi_dat
    data = lonarr(2,fid_len,etl,nslices,nksppts)
    print, 'Reading integer (32 bit) data.'
ENDIF ELSE BEGIN                ;16 bit data
    csi_dat = complex(csi_dat)
    IF etl EQ 2 THEN csi_dat1=csi_dat
    data = intarr(2,fid_len,etl,nslices,nksppts)
    print, 'Reading integer (16 bit) data.'
ENDELSE

readu,unit,data                 ;read data

print,'bsflag = ',bsflag
IF ptsize EQ 4 THEN BEGIN
    IF bsflag THEN byteorder,data,/lswap
    data=dcomplex(data(0,*,*,*,*),data(1,*,*,*,*)) ;make complex data
ENDIF ELSE BEGIN
    IF bsflag THEN byteorder,data
    data=complex(data(0,*,*,*,*),data(1,*,*,*,*)) ;make complex data
ENDELSE
data=reform(data)

;make mask
ymax=float(pe_dim(1)-1.0)/2.0
xmax=float(pe_dim(0)-1.0)/2.0

xxx=(((findgen(pe_dim(0))-xmax)/xmax)^2)#replicate(1,pe_dim(1))
yyy=replicate(1,pe_dim(0))#(((findgen(pe_dim(1))-ymax)/ymax)^2)
rad=xxx+yyy
mask=replicate(1,raw_len)#reform(rad,n_elements(rad))
IF raw_len GT fid_len THEN mask(fid_len:raw_len-1,*)=2
mask=where(mask LT 1.01)

IF etl EQ 2 THEN BEGIN
;Load measured points into csi_dat
    csi_dat(mask)=data(*,1,sliceorder(mpcsi-1),*) ;Second Echo

    csi_dat1(mask)=data(*,0,sliceorder(mpcsi-1),*) ;FID or first echo
;Correct for lack of 180 degree phase shift in some FIDs
    factx=(-1)^indgen(pe_dim(0)) mod 2
    facty=(-1)^indgen(pe_dim(1)) mod 2
    fact=reform(factx#facty,pe_dim(0)*pe_dim(1))
    fact=replicate(1,spc_len)#fact
    fact=reform(fact,spc_len,pe_dim(0),pe_dim(1))
    csi_dat1=csi_dat1*fact
ENDIF ELSE csi_dat(mask)=data(*,sliceorder(mpcsi-1),*)

IF ptsize EQ 4 THEN BEGIN
    csi_dat=csi_dat/2l^16
    IF etl EQ 2 THEN csi_dat1=csi_dat1/2l^16
ENDIF

close,unit                      ;close file
free_lun,unit			;free logical unit number

;*******  Extract Selected Header Fields:

print, 'Extracting fields from header...'

RETURN

END
