;This procedure creates a dialog box for choosing settings.
;Revision History
; 05Sep96 ASB First version, only changes color table
; 25Feb98 ASB Added code to choose filters and filter parameters
; 27Feb98 ASB Debugged code to choose filters and filter parameters
; 13Apr98 Qin Added a "Set Color Table" button using xloadct( so
;             that contrust is available) , removed the old ones
; 14Apr98 Qin Added "q_naa_max" input_label for input of sys. var
;             naa_scale_max.
;  1May98 QIN Added code to set jhd flag.
; 15May98 ASB Changed code for handling color tables.
;             Added Comments
; 20Jul98 ASB Removed superfluous code.
;             Added code to update plots immediately when limits are changed.
; 21Jul98 ASB Split Settings into recon_settings and display_settings.
;  6Aug98 ASB Moved initialization to common_list.pro.
; 18Sep98 ASB Added time domain processing option.
; 21Sep98 ASB Added echo number widgets.
;  2Oct98 ASB Removed time domain processing option.
;  9Oct98 ASB Changed label on 'Quit' button to 'OK'.
;             Added code to read LX data files.
;  3Mar99 ASB Debugged code for adding or removing echo train length,
;             echo and skip widgets.
;             Added button for EPI acquisition.
;  5Oct99 ASB Added code for defining temporal window for FID.
; 22Dec99 ASB Added code to handle two echoes.              
; 18May00 ASB Added cosine^2 to filter list.
;  2Jun00 ASB Removed vestigial skip and echo widgets.
;             Fixed bugs in pulldown menus.
; This program handles events returned by the rsettings widgets.

pro rsettings_event,event
COMMON rsettings
COMMON defaults
COMMON defroi

CASE event.id OF 
                                ;Set csi version
    setftype_w: BEGIN
        widget_control,event.id,get_value=version
        parent=widget_info(event.id,/parent)
        sib1=widget_info(event.id,/sibling)
        IF version EQ 0 OR version EQ 5 THEN BEGIN
            IF sib1 EQ 0 THEN BEGIN
                etl_w=cw_field(parent,title='Echo train length',/integer,xsize=1,$
                               value=etl,/return)
            ENDIF
        ENDIF ELSE IF sib1 NE 0 THEN BEGIN
            widget_control,sib1,/destroy
        ENDIF
    END
    
    etl_w: BEGIN
        widget_control,event.id,get_value=foo
        IF foo NE 1 AND foo NE 2 THEN BEGIN ;ignore if etl ne 1 or 2
            widget_control,event.id,set_value=etl
            RETURN
        ENDIF ELSE etl=foo
        IF etl EQ 1 THEN BEGIN
            widget_control,row6_w,/destroy
        ENDIF ELSE BEGIN
            fidtwin
        ENDELSE
    END

    setspfilt_w:                ;Change spatial filter type

    spdmenu_w:filter_event,event ;Change spatial filter
    
    tpdmenu_w:filter_event,event ;Change temporal filter

    tpd1menu_w:filter_event,event ;Change temporal filter
    
    rsetquit_w:BEGIN            ;Save neccesary info and quit
        widget_control,setspfilt_w,get_value=filtsym
        IF sapd GT 3 THEN BEGIN
            widget_control,sprm1value_w,get_value=tmp
            sapd_defs(0,sapd)=tmp
        ENDIF
        IF sapd EQ 5 THEN BEGIN
            widget_control,sprm2value_w,get_value=tmp
            sapd_defs(1,sapd)=tmp
        ENDIF
        IF tapd GT 3 THEN BEGIN
            widget_control,tprm1value_w,get_value=tmp
            tapd_defs(0,tapd)=tmp
        ENDIF
        IF tapd EQ 5 THEN BEGIN
            widget_control,tprm2value_w,get_value=tmp
            tapd_defs(1,tapd)=tmp
        ENDIF
        IF etl EQ 2 THEN BEGIN
            IF tapd1 GT 3 THEN BEGIN
                widget_control,tprm11value_w,get_value=tmp
                tapd1_defs(0,tapd1)=tmp
            ENDIF
            IF tapd1 EQ 5 THEN BEGIN
                widget_control,tprm12value_w,get_value=tmp
                tapd1_defs(1,tapd1)=tmp
            ENDIF
        ENDIF
        widget_control,rsetbase_w,/destroy
    END 

    ELSE: set_apd,event         ;Change filter parameter
ENDCASE
END

;This program handles events generated by the pull down menus
pro filter_event,event
COMMON rsettings
COMMON defaults
CASE event.value OF
    'Spatial filter type.Rectangular':BEGIN
                                ;Change filter type name
        widget_control,sfilter_w,set_value=titles(0)
                                ;Show only parameters that are used
        widget_control,sprm1base_w,map=0
        widget_control,sprm2base_w,map=0
        sapd=0
    END
    'Spatial filter type.Bartlett   ':BEGIN
                                ;Change filter type name
        widget_control,sfilter_w,set_value=titles(1)
                                ;Show only parameters that are used
        widget_control,sprm1base_w,map=0
        widget_control,sprm2base_w,map=0
        sapd=1
    END
    'Spatial filter type.Hann       ':BEGIN
                                ;Change filter type name
        widget_control,sfilter_w,set_value=titles(2)
                                ;Show only parameters that are used
        widget_control,sprm1base_w,map=0
        widget_control,sprm2base_w,map=0
        sapd=2
    END
    'Spatial filter type.Hamming    ':BEGIN
                                ;Change filter type name
        widget_control,sfilter_w,set_value=titles(3)
                                ;Show only parameters that are used
        widget_control,sprm1base_w,map=0
        widget_control,sprm2base_w,map=0
        sapd=3
    END
    'Spatial filter type.Gaussian   ':BEGIN
                                ;Change filter type name
        widget_control,sfilter_w,set_value=titles(4)
                                ;Set value of parameter
        widget_control,sprm1value_w,/map,set_value=strtrim(string(sapd_defs(0,4)),2)
                                ;Set name of parameter
        widget_control,sprm1label_w,set_value='Beta:'
                                ;Show only parameter that is used
        widget_control,sprm1base_w,/map
        widget_control,sprm2base_w,map=0        
        sapd=4
    END
    'Spatial filter type.Fermi      ':BEGIN
                                ;Change filter type name
        widget_control,sfilter_w,set_value=titles(5)
                                ;Set values of parameters
        widget_control,sprm1value_w,/map,set_value=strtrim(string(sapd_defs(0,5)),2)
        widget_control,sprm2value_w,/map,set_value=strtrim(string(sapd_defs(1,5)),2)
                                ;Set names of parameters
        widget_control,sprm1label_w,set_value='Mu:'
        widget_control,sprm2label_w,set_value='Beta:'
                                ;Show both parameters
        widget_control,sprm1base_w,/map
        widget_control,sprm2base_w,/map
        sapd=5
    END
    'Spatial filter type.Chebyshev  ':BEGIN
                                ;Change filter type name
        widget_control,sfilter_w,set_value=titles(6)
                                ;Set value of parameter
        widget_control,sprm1value_w,/map,set_value=strtrim(string(sapd_defs(0,6)),2)
                                ;Set name of parameter
        widget_control,sprm1label_w,set_value='Beta:'
                                ;Show only parameter that is used
        widget_control,sprm1base_w,/map
        widget_control,sprm2base_w,map=0
        sapd=6
    END
    'Spatial filter type.Exponential':BEGIN
                                ;Change filter type name
        widget_control,sfilter_w,set_value=titles(7)
                                ;Set value of parameter
        widget_control,sprm1value_w,/map,set_value=strtrim(string(sapd_defs(0,7)),2)
                                ;Set name of parameter
        widget_control,sprm1label_w,set_value='Alpha:'
                                ;Show only parameter that is used
        widget_control,sprm1base_w,/map
        widget_control,sprm2base_w,map=0
        sapd=7
    END
    'Spatial filter type.Cosine     ':BEGIN
                                ;Change filter type name
        widget_control,sfilter_w,set_value=titles(8)
                                ;Set value of parameter
        widget_control,sprm1value_w,/map,set_value=strtrim(string(sapd_defs(0,8)),2)
                                ;Set name of parameter
        widget_control,sprm1label_w,set_value='Plateau width:'        
                                ;Show only parameter that is used
        widget_control,sprm1base_w,/map
        widget_control,sprm2base_w,map=0
        sapd=8
    END
    'Spatial filter type.Cosine^2   ':BEGIN
                                ;Change filter type name
        widget_control,sfilter_w,set_value=titles(9)
                                ;Set value of parameter
        widget_control,sprm1value_w,/map,set_value=strtrim(string(sapd_defs(0,9)),2)
                                ;Set name of parameter
        widget_control,sprm1label_w,set_value='Plateau width:'        
                                ;Show only parameter that is used
        widget_control,sprm1base_w,/map
        widget_control,sprm2base_w,map=0
        sapd=9
    END
    'Temporal filter type.Rectangular':BEGIN
                                ;Change filter type name
        widget_control,tfilter_w,set_value=titles(0)
                                ;Show only parameters that are used
        widget_control,tprm1base_w,map=0
        widget_control,tprm2base_w,map=0
        tapd=0
    END
    'Temporal filter type.Bartlett   ':BEGIN
                                ;Change filter type name
        widget_control,tfilter_w,set_value=titles(1)
                                ;Show only parameters that are used
        widget_control,tprm1base_w,map=0
        widget_control,tprm2base_w,map=0
        tapd=1
    END
    'Temporal filter type.Hann       ':BEGIN
                                ;Change filter type name
        widget_control,tfilter_w,set_value=titles(2)
                                ;Show only parameters that are used
        widget_control,tprm1base_w,map=0
        widget_control,tprm2base_w,map=0
        tapd=2
    END
    'Temporal filter type.Hamming    ':BEGIN
                                ;Change filter type name 
        widget_control,tfilter_w,set_value=titles(3)
                                ;Show only parameters that are used
        widget_control,tprm1base_w,map=0
        widget_control,tprm2base_w,map=0
        tapd=3
    END
    'Temporal filter type.Gaussian   ':BEGIN
                                ;Change filter type name
        widget_control,tfilter_w,set_value=titles(4)
                                ;Set value of parameter
        widget_control,tprm1value_w,/map,set_value=strtrim(string(tapd_defs(0,4)),2)
                                ;Set name of parameter
        widget_control,tprm1label_w,set_value='Beta:'
                                ;Show only parameter that is used
        widget_control,tprm1base_w,/map
        widget_control,tprm2base_w,map=0
        tapd=4
    END
    'Temporal filter type.Fermi      ':BEGIN
                                ;Change filter type name
        widget_control,tfilter_w,set_value=titles(5)
                                ;Set values of parameters
        widget_control,tprm1value_w,/map,set_value=strtrim(string(tapd_defs(0,5)),2)
        widget_control,tprm2value_w,/map,set_value=strtrim(string(tapd_defs(1,5)),2)
                                ;Set names of parameters
        widget_control,tprm1label_w,set_value='Mu:'
        widget_control,tprm2label_w,set_value='Beta:'
                                ;Show both parameters
        widget_control,tprm1base_w,/map
        widget_control,tprm2base_w,/map
        tapd=5
    END
    'Temporal filter type.Chebyshev  ':BEGIN
                                ;Change filter type name
        widget_control,tfilter_w,set_value=titles(6)
                                ;Set value of parameter
        widget_control,tprm1value_w,/map,set_value=strtrim(string(tapd_defs(0,6)),2)
                                ;Set name of parameter
        widget_control,tprm1label_w,set_value='Beta:'
                                ;Show only parameter that is used
        widget_control,tprm1base_w,/map
        widget_control,tprm2base_w,map=0
        tapd=6
    END
    'Temporal filter type.Exponential':BEGIN
                                ;Change filter type name
        widget_control,tfilter_w,set_value=titles(7)
                                ;Set value of parameter
        widget_control,tprm1value_w,/map,set_value=strtrim(string(tapd_defs(0,7)),2)
                                ;Set name of parameter
        widget_control,tprm1label_w,set_value='Alpha:'
                                ;Show only parameter that is used
        widget_control,tprm1base_w,/map
        widget_control,tprm2base_w,map=0       
        tapd=7
    END
    'Temporal filter type.Cosine     ':BEGIN
                                ;Change filter type name
        widget_control,tfilter_w,set_value=titles(8)
                                ;Set value of parameter
        widget_control,tprm1value_w,/map,set_value=strtrim(string(tapd_defs(0,8)),2)
                                ;Set name of parameter
        widget_control,tprm1label_w,set_value='Plateau width:'        
                                ;Show only parameter that is used
        widget_control,tprm1base_w,/map
        widget_control,tprm2base_w,map=0
        tapd=8
    END
    'Temporal filter type.Cosine^2   ':BEGIN
                                ;Change filter type name
        widget_control,tfilter_w,set_value=titles(9)
                                ;Set value of parameter
        widget_control,tprm1value_w,/map,set_value=strtrim(string(tapd_defs(0,9)),2)
                                ;Set name of parameter
        widget_control,tprm1label_w,set_value='Plateau width:'        
                                ;Show only parameter that is used
        widget_control,tprm1base_w,/map
        widget_control,tprm2base_w,map=0
        tapd=9
    END
    'FID Temporal filter type.Rectangular':BEGIN
                                ;Change filter type name
        widget_control,tfilter1_w,set_value=titles(0)
                                ;Show only parameters that are used
        widget_control,tprm11base_w,map=0
        widget_control,tprm12base_w,map=0
        tapd=0
    END
    'FID Temporal filter type.Bartlett   ':BEGIN
                                ;Change filter type name
        widget_control,tfilter1_w,set_value=titles(1)
                                ;Show only parameters that are used
        widget_control,tprm11base_w,map=0
        widget_control,tprm12base_w,map=0
        tapd=1
    END
    'FID Temporal filter type.Hann       ':BEGIN
                                ;Change filter type name
        widget_control,tfilter1_w,set_value=titles(2)
                                ;Show only parameters that are used
        widget_control,tprm11base_w,map=0
        widget_control,tprm12base_w,map=0
        tapd=2
    END
    'FID Temporal filter type.Hamming    ':BEGIN
                                ;Change filter type name 
        widget_control,tfilter1_w,set_value=titles(3)
                                ;Show only parameters that are used
        widget_control,tprm11base_w,map=0
        widget_control,tprm12base_w,map=0
        tapd=3
    END
    'FID Temporal filter type.Gaussian   ':BEGIN
                                ;Change filter type name
        widget_control,tfilter1_w,set_value=titles(4)
                                ;Set value of parameter
        widget_control,tprm11value_w,/map,set_value=strtrim(string(tapd1_defs(0,4)),2)
                                ;Set name of parameter
        widget_control,tprm11label_w,set_value='Beta:'
                                ;Show only parameter that is used
        widget_control,tprm11base_w,/map
        widget_control,tprm12base_w,map=0
        tapd1=4
    END
    'FID Temporal filter type.Fermi      ':BEGIN
                                ;Change filter type name
        widget_control,tfilter1_w,set_value=titles(5)
                                ;Set values of parameters
        widget_control,tprm11value_w,/map,set_value=strtrim(string(tapd1_defs(0,5)),2)
        widget_control,tprm12value_w,/map,set_value=strtrim(string(tapd1_defs(1,5)),2)
                                ;Set names of parameters
        widget_control,tprm11label_w,set_value='Mu:'
        widget_control,tprm12label_w,set_value='Beta:'
                                ;Show both parameters
        widget_control,tprm11base_w,/map
        widget_control,tprm12base_w,/map
        tapd1=5
    END
    'FID Temporal filter type.Chebyshev  ':BEGIN
                                ;Change filter type name
        widget_control,tfilter1_w,set_value=titles(6)
                                ;Set value of parameter
        widget_control,tprm11value_w,/map,set_value=strtrim(string(tapd1_defs(0,6)),2)
                                ;Set name of parameter
        widget_control,tprm11label_w,set_value='Beta:'
                                ;Show only parameter that is used
        widget_control,tprm11base_w,/map
        widget_control,tprm12base_w,map=0
        tapd1=6
    END
    'FID Temporal filter type.Exponential':BEGIN
                                ;Change filter type name
        widget_control,tfilter1_w,set_value=titles(7)
                                ;Set value of parameter
        widget_control,tprm11value_w,/map,set_value=strtrim(string(tapd1_defs(0,7)),2)
                                ;Set name of parameter
        widget_control,tprm11label_w,set_value='Alpha:'
                                ;Show only parameter that is used
        widget_control,tprm11base_w,/map
        widget_control,tprm12base_w,map=0       
        tapd1=7
    END
    'FID Temporal filter type.Cosine     ':BEGIN
                                ;Change filter type name
        widget_control,tfilter1_w,set_value=titles(8)
                                ;Set value of parameter
        widget_control,tprm11value_w,/map,set_value=strtrim(string(tapd1_defs(0,8)),2)
                                ;Set name of parameter
        widget_control,tprm11label_w,set_value='Plateau width:'        
                                ;Show only parameter that is used
        widget_control,tprm11base_w,/map
        widget_control,tprm12base_w,map=0
        tapd1=8
    END
    'FID Temporal filter type.Cosine^2   ':BEGIN
                                ;Change filter type name
        widget_control,tfilter1_w,set_value=titles(9)
                                ;Set value of parameter
        widget_control,tprm11value_w,/map,set_value=strtrim(string(tapd1_defs(0,9)),2)
                                ;Set name of parameter
        widget_control,tprm11label_w,set_value='Plateau width:'        
                                ;Show only parameter that is used
        widget_control,tprm11base_w,/map
        widget_control,tprm12base_w,map=0
        tapd1=8
    END
    ENDCASE
END

;This program handles events generated when apd parameters are changed
pro set_apd,event
COMMON rsettings
COMMON defaults
CASE event.id OF
    sprm1base_w:BEGIN
        widget_control,sfilter_w,get_value=ftype
        CASE ftype OF
            'Gaussian   ':BEGIN
                                ;Read new value of parameter
                widget_control,sprm1value_w,get_value=tmp
                                ;Set value of parameter in common block
                sapd_defs(0,4)=tmp
                                ;Display parameter in standard format
                widget_control,sprm1value_w,set_value=strtrim(float(tmp),2)
            END
            'Fermi      ':BEGIN
                                ;Read new value of parameter
                widget_control,sprm1value_w,/map,get_value=tmp
                                ;Set value of parameter in common block
                sapd_defs(0,5)=tmp
                                ;Display parameter in standard format
                widget_control,sprm1value_w,set_value=strtrim(float(tmp),2)
            END
            'Chebyshev  ':BEGIN
                                ;Read new value of parameter
                widget_control,sprm1value_w,/map,get_value=tmp
                                ;Set value of parameter in common block
                sapd_defs(0,6)=tmp
                                ;Display parameter in standard format
                widget_control,sprm1value_w,set_value=strtrim(float(tmp),2)
            END
            'Exponential':BEGIN
                                ;Read new value of parameter
                widget_control,sprm1value_w,/map,get_value=tmp
                                ;Set value of parameter in common block
                sapd_defs(0,7)=tmp
                                ;Display parameter in standard format
                widget_control,sprm1value_w,set_value=strtrim(float(tmp),2)
            END
            'Cosine     ':BEGIN
                                ;Read new value of parameter
                widget_control,sprm1value_w,/map,get_value=tmp
                                ;Set value of parameter in common block
                sapd_defs(0,8)=tmp
                                ;Display parameter in standard format
                widget_control,sprm1value_w,set_value=strtrim(float(tmp),2)
            END
        ENDCASE
    END

    sprm2base_w:BEGIN
                                ;Read new value of parameter
        widget_control,sprm2value_w,/map,get_value=tmp
                                ;Set value of parameter in common block
        sapd_defs(1,5)=tmp
                                ;Display parameter in standard format
        widget_control,sprm2value_w,set_value=strtrim(float(tmp),2)
    END
    tprm1base_w:BEGIN
        widget_control,tfilter_w,get_value=ftype
        CASE ftype OF
            'Gaussian   ':BEGIN
                                ;Read new value of parameter 
                widget_control,tprm1value_w,get_value=tmp
                                ;Set value of parameter in common block
                tapd_defs(0,4)=tmp
                                ;Display parameter in standard format
                widget_control,tprm1value_w,set_value=strtrim(float(tmp),2)
            END
            'Fermi      ':BEGIN
                                ;Read new values of parameters
                widget_control,tprm1value_w,/map,get_value=tmp
                                ;Set value of parameter in common block
                tapd_defs(0,5)=tmp
                                ;Display parameter in standard format
                widget_control,tprm1value_w,set_value=strtrim(float(tmp),2)
            END
            'Chebyshev  ':BEGIN
                                ;Read new value of parameter
                widget_control,tprm1value_w,/map,get_value=tmp
                                ;Set value of parameter in common block
                tapd_defs(0,6)=tmp
                                ;Display parameter in standard format
                widget_control,tprm1value_w,set_value=strtrim(float(tmp),2)
            END
            'Exponential':BEGIN
                                ;Read new value of parameter
                widget_control,tprm1value_w,/map,get_value=tmp
                                ;Set value of parameter in common block
                tapd_defs(0,7)=tmp
                                ;Display parameter in standard format
                widget_control,tprm1value_w,set_value=strtrim(float(tmp),2)
            END
            'Cosine     ':BEGIN
                                ;Read new value of parameter
                widget_control,tprm1value_w,/map,get_value=tmp
                                ;Set value of parameter in common block
                tapd_defs(0,8)=tmp
                                ;Display parameter in standard format
                widget_control,tprm1value_w,set_value=strtrim(float(tmp),2)
            END
        ENDCASE
    END
    tprm2base_w:BEGIN
                                ;Read new value of parameter
        widget_control,tprm2value_w,/map,get_value=tmp
                                ;Set value of parameter in common block
        tapd_defs(1,5)=tmp
                                ;Display parameter in standard format
        widget_control,tprm2value_w,set_value=strtrim(float(tmp),2)
    END
    tprm11base_w:BEGIN
        widget_control,tfilter1_w,get_value=ftype
        CASE ftype OF
            'Gaussian   ':BEGIN
                                ;Read new value of parameter 
                widget_control,tprm11value_w,get_value=tmp
                                ;Set value of parameter in common block
                tapd1_defs(0,4)=tmp
                                ;Display parameter in standard format
                widget_control,tprm11value_w,set_value=strtrim(float(tmp),2)
            END
            'Fermi      ':BEGIN
                                ;Read new values of parameters
                widget_control,tprm11value_w,/map,get_value=tmp
                                ;Set value of parameter in common block
                tapd1_defs(0,5)=tmp
                                ;Display parameter in standard format
                widget_control,tprm11value_w,set_value=strtrim(float(tmp),2)
            END
            'Chebyshev  ':BEGIN
                                ;Read new value of parameter
                widget_control,tprm11value_w,/map,get_value=tmp
                                ;Set value of parameter in common block
                tapd1_defs(0,6)=tmp
                                ;Display parameter in standard format
                widget_control,tprm11value_w,set_value=strtrim(float(tmp),2)
            END
            'Exponential':BEGIN
                                ;Read new value of parameter
                widget_control,tprm11value_w,/map,get_value=tmp
                                ;Set value of parameter in common block
                tapd1_defs(0,7)=tmp
                                ;Display parameter in standard format
                widget_control,tprm11value_w,set_value=strtrim(float(tmp),2)
            END
            'Cosine     ':BEGIN
                                ;Read new value of parameter
                widget_control,tprm11value_w,/map,get_value=tmp
                                ;Set value of parameter in common block
                tapd1_defs(0,8)=tmp
                                ;Display parameter in standard format
                widget_control,tprm11value_w,set_value=strtrim(float(tmp),2)
            END
        ENDCASE
    END
    tprm12base_w:BEGIN
                                ;Read new value of parameter
        widget_control,tprm12value_w,/map,get_value=tmp
                                ;Set value of parameter in common block
        tapd1_defs(1,5)=tmp
                                ;Display parameter in standard format
        widget_control,tprm12value_w,set_value=strtrim(float(tmp),2)
    END
ENDCASE
END

;This subroutine creates the FID temporal echo selections widgets
pro fidtwin
COMMON rsettings
COMMON defaults
COMMON defroi
junk={CW_PDMENU_S,flags:0,name:''}

row6_w=widget_base(row4a_w,row=1)

rset_menu = [ $
                { CW_PDMENU_S,       1, 'FID Temporal filter type' }, $ ;        0
                { CW_PDMENU_S,       0, 'Rectangular' }, $ ;        1
                { CW_PDMENU_S,       0, 'Bartlett   ' }, $ ;        2
                { CW_PDMENU_S,       0, 'Hann       ' }, $ ;        3
                { CW_PDMENU_S,       0, 'Hamming    ' }, $ ;        4
                { CW_PDMENU_S,       0, 'Gaussian   ' }, $ ;        5
                { CW_PDMENU_S,       0, 'Fermi      ' }, $ ;        6
                { CW_PDMENU_S,       0, 'Chebyshev  ' }, $ ;        7
                { CW_PDMENU_S,       0, 'Exponential' }, $ ;        8
                { CW_PDMENU_S,       0, 'Cosine     ' }, $ ;        9
                { CW_PDMENU_S,       0, 'Cosine^2   ' }  $ ;       10
              ]
tpd1menu_w = cw_pdmenu(row6_w, rset_menu, /return_full_name, $
                      uvalue='pdmenu')

                                ;filter type name widget
tfilter1_w = widget_label(row6_w, value=titles(tapd1))
                                ;base widget for temporal filter param values
tprm10base_w=widget_base(row6_w,row=1)
                                ;temporal parameter 1 compound widget
tprm11base_w=cw_field(tprm10base_w,title=param_names(0,8),$
                     value=strtrim(string(0),2),/ret,/float)
tprm11label_w=widget_info(tprm11base_w,/child) ;temporal parameter 1 name widget
tprm11value_w=widget_info(tprm11label_w,/sib) ;spatial parameter 1 value widget

widget_control,tprm11label_w,set_value=param_names(0,tapd1) ;set label
                                ;set value
widget_control,tprm11value_w,set_value=strtrim(string(tapd1_defs(0,tapd1)),2)

IF tapd1 LE 3 THEN BEGIN         ;Don't map parameter widget unless it's needed
    widget_control,tprm11base_w,map=0
ENDIF

                                ;temporal parameter 2 compound widget
tprm12base_w=cw_field(tprm10base_w,title=param_names(0,8),$
                     value=strtrim(string(0),2),/ret,/float)

tprm12label_w=widget_info(tprm12base_w,/child) ;spatial parameter 2 name widget
tprm12value_w=widget_info(tprm12label_w,/sib) ;spatial parameter 2 value widget

widget_control,tprm12label_w,set_value=param_names(1,tapd1) ;set label
                                ;set value
widget_control,tprm12value_w,set_value=strtrim(string(tapd1_defs(1,tapd1)),2)
IF tapd1 NE 5 THEN BEGIN         ;Don't map parameter widget unless it's needed
    widget_control,tprm12base_w,map=0
ENDIF
widget_control,row6_w,/realize
END


;This subroutine creates the settings widget
pro recon_settings
COMMON rsettings
COMMON defaults
COMMON defroi
junk={CW_PDMENU_S,flags:0,name:''}

rsetbase_w=widget_base(column=1,title='Recon Settings') ; Base widget
                                ;Select file type widget

row1_w=widget_base(rsetbase_w,row=1)
setftype_w=cw_bgroup(row1_w,label_left='Raw File Type',$
                     ['55','5xb','5xa','5x','4x','LX','EPI'],$
                     set_value=version,/exclusive,row=1)
                                ;Spatial filter symmetry widget
IF version EQ 0 OR version EQ 5 THEN BEGIN
    etl_w=cw_field(row1_w,title='Echo train length',/integer,xsize=1,value=etl,$
                   /return)
ENDIF

setspfilt_w=cw_bgroup(rsetbase_w,label_left='Spatial filter symmetry:',$
                      ['Rectangular','Circular'],set_value=filtsym,/exclusive,row=1)
row4a_w=widget_base(rsetbase_w,col=1)
row4_w=widget_base(row4a_w,row=1)

rset_menu = [ $
                { CW_PDMENU_S,       1, 'Spatial filter type' }, $ ;        0
                { CW_PDMENU_S,       0, 'Rectangular' }, $ ;        1
                { CW_PDMENU_S,       0, 'Bartlett   ' }, $ ;        2
                { CW_PDMENU_S,       0, 'Hann       ' }, $ ;        3
                { CW_PDMENU_S,       0, 'Hamming    ' }, $ ;        4
                { CW_PDMENU_S,       0, 'Gaussian   ' }, $ ;        5
                { CW_PDMENU_S,       0, 'Fermi      ' }, $ ;        6
                { CW_PDMENU_S,       0, 'Chebyshev  ' }, $ ;        7
                { CW_PDMENU_S,       0, 'Exponential' }, $ ;        8
                { CW_PDMENU_S,       0, 'Cosine     ' }, $ ;        9
                { CW_PDMENU_S,       0, 'Cosine^2   ' }  $ ;       10
              ]
spdmenu_w = cw_pdmenu(row4_w, rset_menu, /return_full_name, $
                      uvalue='pdmenu')

                                ;filter type name widget
sfilter_w = widget_label(row4_w, value=titles(sapd))
                                ;base widget for spatial filter param values
sprmbase_w=widget_base(row4_w,row=1)
                                ;spatial parameter 1 compound widget
sprm1base_w=cw_field(sprmbase_w,title=param_names(0,8),$
                     value=strtrim(string(0),2),/ret,/float)
sprm1label_w=widget_info(sprm1base_w,/child) ;spatial parameter 1 name widget
sprm1value_w=widget_info(sprm1label_w,/sib) ;spatial parameter 1 value widget

widget_control,sprm1label_w,set_value=param_names(0,sapd) ;set label
                                ;set value
widget_control,sprm1value_w,set_value=strtrim(string(sapd_defs(0,sapd)),2)

IF sapd LE 3 THEN BEGIN         ;Don't map parameter widget unless it's needed
    widget_control,sprm1base_w,map=0
ENDIF

                                ;spatial parameter 2 compound widget
sprm2base_w=cw_field(sprmbase_w,title=param_names(0,8),$
                     value=strtrim(string(0),2),/ret,/float)
sprm2label_w=widget_info(sprm2base_w,/child) ;spatial parameter 2 name widget
sprm2value_w=widget_info(sprm2label_w,/sib) ;spatial parameter 2 value widget

widget_control,sprm2label_w,set_value=param_names(1,sapd) ;set label
                                ;set value
widget_control,sprm2value_w,set_value=strtrim(string(sapd_defs(1,sapd)),2)

IF sapd NE 5 THEN BEGIN         ;Don't map paramater widget unless it's needed
    widget_control,sprm2base_w,map=0
ENDIF
 
                               ;Temporal filter type pull down menu
row5_w=widget_base(rsetbase_w,row=1)

rset_menu = [ $
                { CW_PDMENU_S,       1, 'Temporal filter type' }, $ ;        0
                { CW_PDMENU_S,       0, 'Rectangular' }, $ ;        1
                { CW_PDMENU_S,       0, 'Bartlett   ' }, $ ;        2
                { CW_PDMENU_S,       0, 'Hann       ' }, $ ;        3
                { CW_PDMENU_S,       0, 'Hamming    ' }, $ ;        4
                { CW_PDMENU_S,       0, 'Gaussian   ' }, $ ;        5
                { CW_PDMENU_S,       0, 'Fermi      ' }, $ ;        6
                { CW_PDMENU_S,       0, 'Chebyshev  ' }, $ ;        7
                { CW_PDMENU_S,       0, 'Exponential' }, $ ;        8
                { CW_PDMENU_S,       0, 'Cosine     ' }, $ ;        9
                { CW_PDMENU_S,       0, 'Cosine^2   ' }  $ ;       10
              ]

tpdmenu_w = cw_pdmenu(row5_w, rset_menu, /return_full_name, $
                      uvalue='pdmenu')

                                ;filter type name widget
tfilter_w = widget_label(row5_w, value=titles(tapd))
                                ;base widget for temporal filter param values
tprmbase_w=widget_base(row5_w,row=1)
                                ;temporal parameter 1 compound widget
tprm1base_w=cw_field(tprmbase_w,title=param_names(0,8),$
                     value=strtrim(string(0),2),/ret,/float)
tprm1label_w=widget_info(tprm1base_w,/child) ;temporal parameter 1 name widget
tprm1value_w=widget_info(tprm1label_w,/sib) ;spatial parameter 1 value widget

widget_control,tprm1label_w,set_value=param_names(0,tapd) ;set label
                                ;set value
widget_control,tprm1value_w,set_value=strtrim(string(tapd_defs(0,tapd)),2)

IF tapd LE 3 THEN BEGIN         ;Don't map parameter widget unless it's needed
    widget_control,tprm1base_w,map=0
ENDIF

                                ;spatial parameter 2 compound widget
tprm2base_w=cw_field(tprmbase_w,title=param_names(0,8),$
                     value=strtrim(string(0),2),/ret,/float)

tprm2label_w=widget_info(tprm2base_w,/child) ;spatial parameter 2 name widget
tprm2value_w=widget_info(tprm2label_w,/sib) ;spatial parameter 2 value widget

widget_control,tprm2label_w,set_value=param_names(1,tapd) ;set label
                                ;set value
widget_control,tprm2value_w,set_value=strtrim(string(tapd_defs(1,tapd)),2)
IF tapd NE 5 THEN BEGIN         ;Don't map parameter widget unless it's needed
    widget_control,tprm2base_w,map=0
ENDIF

;Extra Temporal window choice for double acquisition

IF etl EQ 2 THEN fidtwin

rsetquit_w=widget_button(rsetbase_w,value='OK') ;Quit widget

widget_control,rsetbase_w,/realize ;Realize widgets

xmanager,'recon_settings',rsetbase_w,event='rsettings_event',/modal ;Register widgets

END
