;This procedure saves the current roi. The code was relocated from
;analyze_roi.pro
;Revision History
; 9Oct98 ASB First version.
; 3Mar99 ASB Added COMMON blocks rsettings and sumfile.
;            Changed initializtion call from asb_def_nmode to
;            asb_def_update.
;            Removed reference to s.psd_w.
;            Added code to handle results of td analysis.
;            TD summary file code not yet written.
;23Feb00 ASB Changed calls to qfile to make code more portable.
;22May00 ASB Added code to byteswap I/O on LINUX machines.
;            Fixed bug in format of .fds and .tds files.
;            Fixed typo.
pro save_roi
COMMON defaults
COMMON roistats
COMMON defroi
COMMON initcsi
COMMON rsettings
COMMON sumfile

asb_defroi_update

pos=strpos(roiname(0),'_')
IF pos NE -1 THEN roiname=strmid(roiname,0,pos)
root=strtrim(roiname,2)            
;If roi contains only a single voxel, add coordinate to roi name
IF count EQ 1 THEN BEGIN 
    xx=t32 mod 32
    yy=t32 / 32
    root=root+'_'+strtrim(xx,2)+','+strtrim(yy,2)
ENDIF

;Generate name of files
IF version EQ 6 THEN BEGIN ; Adalsteinsson
    IF cur_slice LE 9 THEN strn='0'+strtrim(cur_slice,2) ELSE $
      strn=strtrim(cur_slice,2)
ENDIF ELSE strn=strtrim(cur_slice,2)
root1='sl'+strn+'.'+root

;TD results or FD results?
IF tdflag THEN widget_control,s.tf_w,get_uvalue=tdflag1 $ ;td
ELSE tdflag1 = 0                    ;fd
    
IF tdflag1 EQ 0 THEN file=strtrim(root1,2)+'.fds' $
ELSE file=strtrim(root1,2)+'.tds'

widget_control,s.read_w,get_uvalue=old_dir
;Overwrite protection
spawn,getenv('SPECHOME')+'/qfile '+file,res
IF res(0) EQ 1 THEN BEGIN       ;file exists
    over=roiwarning(file)
    IF over EQ 'No' THEN RETURN
ENDIF
;Write in .fds or .tds file
openw,unit,old_dir+'/'+file(0),/get
;Print time domain or frequency domain label
CASE tdflag1 OF
    0: text='Frequency domain analysis results.'
    1: text='Time domain analysis results.'
ENDCASE
printf,unit,text

;Print spatial filter info
foo=['Rectangular','Circular']
text='Spatial Filter Symmetry: '+foo(filtsym)
printf,unit,text
text='Spatial Filter Type: '+strcompress(titles(sapd),/rem)
IF sapd GT 3 THEN BEGIN
    text=text+'     '+strcompress(param_names(0,sapd),/rem)
    text=text+' '+strcompress(strint(sapd_defs(sapd,0)),/rem)
    IF sapd EQ 5 THEN BEGIN
        text=text+',    '+strcompress(param_names(1,sapd),/rem)
        text=text+' '+strcompress(strint(sapd_defs(sapd,1)),/rem)
    ENDIF
ENDIF
printf,unit,text

IF tdflag1 EQ 0 THEN BEGIN
;Print temporal filter info
    text='Temporal Filter Type: '+strcompress(titles(tapd),/rem)
    IF tapd GT 3 THEN BEGIN
        text=text+'  '+strcompress(param_names(0,tapd))
        text=text+' '+strcompress(string(tapd_defs(0,tapd)))
        IF tapd EQ 5 THEN BEGIN
            text=text+', '+strcompress(param_names(1,tapd))
            text=text+' '+strcompress(string(tapd_defs(1,tapd)))
        ENDIF
    ENDIF
    printf,unit,text

;Print bin width
    printf,unit,'Bin width: ',strcompress(string(binwidth))
ENDIF ELSE BEGIN
    text='Apodization Filter Type: 0'
    printf,unit,text
ENDELSE

;Print number of pixels in roi
printf,unit,'Number of pixels in ROI: '+strcompress(string(count),/rem)

;Print statistics an pixel list
printf,unit
IF tdflag1 EQ 0 THEN BEGIN ;Time domain analysis results
    printf,unit,stattitles(0)
    printf,unit,stattitles(2)
    printf,unit,statstring2(0)
    printf,unit,statstring1
ENDIF ELSE BEGIN ;Frequency domain analysis results
    printf,unit,stattitles(1)
    printf,unit,stattitles(3)
    printf,unit,statstring2(1)
    printf,unit,statstring1
ENDELSE

free_lun,unit

;Write in .roi file
file=strtrim(root1,2)+'.roi'
;            widget_control,s.read_w,get_uvalue=old_dir
openw,unit,old_dir+'/'+file(0),/get

widget_control,s.thresh_w,get_value=thresh
widget_control,s.subs,get_uvalue=subs
nn=n_elements(subs)
IF bsflag THEN BEGIN
    byteorder,thresh
    byteorder,nn,/lswap
    byteorder,subs,/lswap
ENDIF
writeu,unit,thresh
writeu,unit,nn
writeu,unit,subs

free_lun,unit

IF tdflag1 EQ 0 THEN BEGIN ;summary file for FD
;Write in summary file
    roiname='             '
    strput,roiname,root(0)
;            widget_control,s.read_w,get_uvalue=old_dir
    IF append EQ 0 THEN BEGIN
        openw,unit,old_dir+'/'+sumfile,/get
        printf,unit,format='(7a12)','ROI name    ',$
          'CHO ','CRE ','NAA ','LAC ','INO '
        append=1
    ENDIF ELSE openu,unit,old_dir+'/'+sumfile,/get,/append
    
    printf,unit,format='(a12,5(2x,f10.2))',roiname,$
      stats.cho_av,stats.cre_av,stats.naa_av,stats.lac_av,stats.ino_av
    free_lun,unit
ENDIF ELSE BEGIN
;Insert code here for td summary file. 
ENDELSE

END

