;  991208  ASB  Added code to permit specplot to show real, imaginary,
;               magnitude or phase of raw, time domain or frequency domain data.
PRO specplot, x, y,show_positions=show_positions,image=image,imflag=imflag
; Arguments and keywords
;    x                   x-coordinate of spectrum
;    y                   y-coordinate of spectrum
;    show_positions      flag to show limits of integration fo each
;                                metabolite
;    image               image of plot (for save_pict)
;    imflag              1-make image, 0-don't make images
COMMON defroi
COMMON posdata
COMMON mets
COMMON defaults
COMMON roistats
COMMON mrsidata
COMMON initcsi
COMMON dsettings
COMMON ctable

;flag=0-> normal plot, flag=1-> dried spectra and fit
widget_control,s.tf_w,get_uvalue=flag

;Compute positions of integration limit lines

ppm_to_point = spec_freq * spc_len/ sweep_width
ir=round(binwidth*ppm_to_point/2.0)
cho_offset = (naa_shift - cho_shift) * ppm_to_point
cre_offset = (naa_shift - cre_shift) * ppm_to_point
lac_offset = (naa_shift - lac_shift) * ppm_to_point
ino_offset = (naa_shift - ino_shift) * ppm_to_point
h2o_offset = (naa_shift - h2o_shift) * ppm_to_point

IF s.smode THEN BEGIN ;ROI mode
    IF count GT 0 THEN BEGIN
                                ;Find nominal position of naa peak
        pt=fix(ppm_to_point*(4.69-2.023)+spc_len/2)
        xx=t32 mod pe_dim(0)      ;get x coordinate from t32
        yy=t32/pe_dim(0)          ;get y coordinate from t32
        spectrum = abs(fd_dat(*,xx(0),yy(0))) ;initialize sum        
        sft=pt-positions(t32(0))
        spectrum=shift(spectrum,sft)
        FOR kk = 1, count - 1 DO BEGIN ;loop over voxels in roi
            sft=pt-positions(t32(kk)) ;Shift spectrum
            foo=shift(abs(fd_dat(*,xx(kk),yy(kk))),sft) ;Next spectrum shifted
            spectrum=spectrum+foo ;add to sum
        ENDFOR
        spectrum=spectrum/count ;renormalize
        plot, paxis, spectrum, color=coffset2+6,$
          xstyle=1, xtit='PPM', xrange=[last_ppm_lf, last_ppm_rt], $
          title='ROI',max_value=max_spec_val
        IF keyword_set(imflag) THEN BEGIN
            image=tvrd()
            plt1=where(image NE 0)
            image(plt1)=coffset2+6
        ENDIF
    ENDIF 

;Display average metabolite values for defined ROI
    widget_control,s.crev_w,set_value=stats.cre_av+0.5
    widget_control,s.chov_w,set_value=stats.cho_av+0.5
    widget_control,s.naav_w,set_value=stats.naa_av+0.5
    widget_control,s.lacv_w,set_value=stats.lac_av+0.5
    widget_control,s.inov_w,set_value=stats.ino_av+0.5
;    widget_control,s.lacv_w,set_value=stats.h2o_av+0.5

;show sensitivity of ROI if class map is displayed
    show_sense

ENDIF ELSE BEGIN ;pixel mode
    last_pos=[x,y]

    IF n_params() GE 2 THEN BEGIN
        last_x_pos=x
        last_y_pos=y
    END

;  Check range of indices:

    IF last_x_pos GT sp_dim(0)-1 THEN last_x_pos = sp_dim(0)-1
    IF last_x_pos LT 0 THEN last_x_pos = 0
    IF last_y_pos GT sp_dim(1)-1 THEN last_y_pos = sp_dim(1)-1
    IF last_y_pos LT 0 THEN last_y_pos = 0


    IF flag THEN BEGIN ;Code for plotting results of TD fits__ not up to date
        spectrum = reform( abs(dry_dat(*,last_x_pos, last_y_pos)) )
        fit = reform( abs(fit_dat(*,last_x_pos, last_y_pos)) )
    ENDIF ELSE BEGIN
        CASE plotparms(0) OF
            0:BEGIN             ;long te
                CASE plotparms(1) OF
                    0: spectrum=csi_dat(0:fid_len-1,last_x_pos,last_y_pos)
                    1: spectrum=td_dat(0:fid_len-1,last_x_pos,last_y_pos)
                    2: spectrum=fd_dat(*,last_x_pos,last_y_pos)
                ENDCASE
            END
            1:BEGIN             ;short te
                CASE plotparms(1) OF
                    0: spectrum=csi_dat1(0:fid_len-1,last_x_pos,last_y_pos)
                    1: spectrum=td_dat1(0:fid_len-1,last_x_pos,last_y_pos)
                    2: spectrum=fd_dat1(*,last_x_pos,last_y_pos)
                ENDCASE
            END
        ENDCASE
    ENDELSE

    CASE plotparms(2) OF
        0: spectrum=reform(abs(spectrum))
        1: spectrum=reform(atan(imaginary(spectrum),float(spectrum)))
        2: spectrum=reform(float(spectrum))
        3: spectrum=reform(imaginary(spectrum))
    ENDCASE

    pt=new_value                ;NAA

    title=string( format='(I2,",",I2)',x,y)
    IF plotparms(1) NE 2 THEN BEGIN
        plot, spectrum, color=coffset2+6,xstyle=1, ys=2, xtit='Time',title=title
        IF keyword_set(imflag) THEN BEGIN
            image=tvrd()
            plt1=where(image NE 0)
            image(plt1)=coffset2+6
        ENDIF
        RETURN
    ENDIF
    plot, paxis, spectrum, color=coffset2+6,$
      xstyle=1, xtit='PPM', xrange=[last_ppm_lf, last_ppm_rt], $
      title=title,max_value=max_spec_val
    IF keyword_set(imflag) THEN BEGIN
        image=tvrd()
        plt1=where(image NE 0)
        image(plt1)=coffset2+6
    ENDIF

;Display metabolite value of selected pixel
IF flag THEN BEGIN ;time domain results
    widget_control,s.crev_w,set_value=cre_td(x,y)+0.5
    widget_control,s.chov_w,set_value=cho_td(x,y)+0.5
    widget_control,s.naav_w,set_value=naa_td(x,y)+0.5
    widget_control,s.lacv_w,set_value=0
;    widget_control,s.inov_w,set_value=0
    widget_control,s.h2o1v_w,set_value=0
    widget_control,s.h2o2v_w,set_value=0
    widget_control,s.h2o3v_w,set_value=0
    widget_control,s.h2o4v_w,set_value=0
    widget_control,s.h2o5v_w,set_value=0
    widget_control,s.h2o6v_w,set_value=0
ENDIF ELSE BEGIN ;frequency domain results
    widget_control,s.crev_w,set_value=cre(x,y)+0.5
    widget_control,s.chov_w,set_value=cho(x,y)+0.5
    widget_control,s.naav_w,set_value=naa(x,y)+0.5
    widget_control,s.lacv_w,set_value=lac(x,y)+0.5
    widget_control,s.inov_w,set_value=ino(x,y)+0.5
    widget_control,s.h2o1v_w,set_value=h2o(x,y,0)+0.5
    widget_control,s.h2o2v_w,set_value=h2o(x,y,1)+0.5
    widget_control,s.h2o3v_w,set_value=h2o(x,y,2)+0.5
    widget_control,s.h2o4v_w,set_value=h2o(x,y,3)+0.5
    widget_control,s.h2o5v_w,set_value=h2o(x,y,4)+0.5
    widget_control,s.h2o6v_w,set_value=h2o(x,y,5)+0.5
ENDELSE
;show sensitivity of ROI if class map is displayed
    show_sense,x,y
ENDELSE

IF n_elements(show_positions) EQ 0 THEN RETURN 
IF show_positions EQ 0 OR pt EQ -1 THEN RETURN 

IF flag EQ 1 THEN BEGIN ;plot fitted spectrum
    oplot,paxis,fit,color=170 ;This line is not 24 bit compatable
    IF keyword_set(imflag) THEN BEGIN
        tmp=tvrd()
        tmp(plt1)=0
        plt2=where(tmp NE 0)
        image(plt2)=170
    ENDIF
ENDIF ELSE BEGIN
;Plot NAA limits
    oplot,[paxis(pt-ir),paxis(pt-ir)],[!y.crange(0), !y.crange(1)],color=coffset2
    oplot,[paxis(pt+ir),paxis(pt+ir)],[!y.crange(0), !y.crange(1)],color=coffset2
    IF keyword_set(imflag) THEN BEGIN
        tmp=tvrd()
        tmp(plt1)=0
        plt2=where(tmp NE 0)
        IF plt2(0) NE -1 THEN BEGIN
            image(plt2)=coffset2
            plt1=[plt1,plt2]
        ENDIF
    ENDIF

;Plot LAC limits
    oplot,[paxis(pt+lac_offset-ir),paxis(pt+lac_offset-ir)],[!y.crange(0), !y.crange(1)],$
      color=coffset2+1
    oplot,[paxis(pt+lac_offset+ir),paxis(pt+lac_offset+ir)],[!y.crange(0),!y.crange(1)],$
     color=coffset2+1
    IF keyword_set(imflag) THEN BEGIN
        tmp=tvrd()
        tmp(plt1)=0
        plt2=where(tmp NE 0)
        IF plt2(0) NE -1 THEN BEGIN
            image(plt2)=coffset2+1
            plt1=[plt1,plt2]
        ENDIF
    ENDIF

;Plot h2o limits
    oplot,[paxis(pt+h2o_offset-ir),paxis(pt+h2o_offset-ir)],[!y.crange(0), !y.crange(1)],$
      color=coffset2+1
    oplot,[paxis(pt+h2o_offset+ir),paxis(pt+h2o_offset+ir)],[!y.crange(0),!y.crange(1)],$
     color=coffset2+1
    IF keyword_set(imflag) THEN BEGIN
        tmp=tvrd()
        tmp(plt1)=0
        plt2=where(tmp NE 0)
        IF plt2(0) NE -1 THEN BEGIN
            image(plt2)=coffset2+1
            plt1=[plt1,plt2]
        ENDIF
    ENDIF

;do not allow cre and cho regions to overlap
    cho2=pt+cho_offset+ir
    cre1=pt+cre_offset-ir
    IF cho2 GT cre1 THEN begin
        cho2=pt+(cho_offset+cre_offset)/2
        dev=convert_coord(paxis(cho2),40,/data,/to_device)
;   dev(0)=dev(0)+1
        cre1=cho2+1
    ENDIF
    pt=pt+cho_offset            ;cho
    oplot,[paxis(pt-ir),paxis(pt-ir)],[!y.crange(0),!y.crange(1)],color=coffset2+2
    oplot,[paxis(cho2),paxis(cho2)],[!y.crange(0),!y.crange(1)],color=coffset2+2
    IF keyword_set(imflag) THEN BEGIN
        tmp=tvrd()
        tmp(plt1)=0
        plt2=where(tmp NE 0)
        IF plt2(0) NE -1 THEN BEGIN
            image(plt2)=coffset2+2
            plt1=[plt1,plt2]
        ENDIF
    ENDIF
    pt=pt-cho_offset+cre_offset ;cre
    oplot,[paxis(cre1),paxis(cre1)],[!y.crange(0), !y.crange(1)],color=coffset2+3
    oplot,[paxis(pt+ir),paxis(pt+ir)],[!y.crange(0), !y.crange(1)],color=coffset2+3
    IF keyword_set(imflag) THEN BEGIN
        tmp=tvrd()
        tmp(plt1)=0
        plt2=where(tmp NE 0)
        IF plt2(0) NE -1 THEN BEGIN
            image(plt2)=coffset2+3
            plt1=[plt1,plt2]
        ENDIF
    ENDIF
ENDELSE

END
