;  891230  GSS  First Writing
;          GSS  Added previous limits
;  901010  GSS  Operate on default system variables.
;  901029  GSS  First SCCS mod. - added SCCS keywords,
;               added variable size array support,
;               calculate magnitude on-the-fly.
;  910225  GSS  Adapted for spectral data in '!csi_dat'.
;  910226  GSS  Fixed various bugs.  Index stuff, etc.
;  910304  GSS  Added keyword, '/ref' to put up an interactive cursor and
;               fill '!ref_pos(0)' with the data point.
;  910519  GSS  Changed keywords!  '/ref' -> '/ref_set', added
;               '/ref_plot' to just display the ref. cursor.
;  950630  asb  changed title line on plot
;  951013  asb  added plotting integration ranges for naa, cho, and cre
;  951613  asb  added keyword width
;  960318  asb  added common block posdata
;  970718  asb  Removed reference to !spectro.
;  980310  Qin  added avg. spectrum and y scales in oplot at edit win.
;  980518  QIN  Changed test for avg_spec.
;  980715  asb  Replaced lines to show integration limits for each
;               metabolite.
;  980716  asb  Fixed bug in display of ROI spectrum.
;               Added integration limits to ROI spectrum plots.
;  980720  asb  Added last_pos to store last position.
;  980721  asb  Removed keyword width.  Added call to get width from widget.
;  980722  asb  Moved assignment of last_pos.
;  980727  asb  Added calls to display coordinates, ROI name.
;               Added COMMON mets.
;  980807  asb  Added common blocks roistats, defaults.
;               Added logic to display pixel or roi values.
;  980828  asb  Added calls to show_sense.
;  980915  ASB  Replaced !csi_dat with fd_dat in COMMON freqdomain.
;  980915  ASB  Added COMMON initcsi and removed system variables.
;  980916  ASB  Changed name of COMMON freqdomain to mrsidata.
;  980918  ASB  Added COMMON dsettings.
;  981007  ASB  Added code to display results of time domain fits.
;  990324  ASB  Changed logic for testing whether to show integration limits.
;  990407  ASB  Added roundoff instead of truncate for integration limits.

PRO specplot, x, y, real=real,show_positions=show_positions
COMMON defroi
COMMON posdata
COMMON mets
COMMON defaults
COMMON roistats
COMMON mrsidata
COMMON initcsi
COMMON dsettings

;flag=0-> normal plot, flag=1-> dried spectra and fit
widget_control,s.tf_w,get_uvalue=flag

;Compute positions of integration limit lines

ppm_to_point = spec_freq * spc_len/ sweep_width
ir=binwidth*ppm_to_point/2.0
cho_offset = (2.023 - 3.25) * ppm_to_point
cre_offset = (2.023 - 3.05) * ppm_to_point
lac_offset = (2.023 - 1.33) * ppm_to_point
h2o_offset = (2.023 - 4.75) * ppm_to_point

IF s.smode THEN BEGIN ;ROI mode
    IF count GT 0 THEN BEGIN
                                ;Find nominal position of naa peak
        pt=fix(ppm_to_point*(4.69-2.023)+spc_len/2)
        xx=t32 mod pe_dim(0)      ;get x coordinate from t32
        yy=t32/pe_dim(0)          ;get y coordinate from t32
        spectrum = abs(fd_dat(*,xx(0),yy(0))) ;initialize sum        
        sft=pt-positions(t32(0))
        spectrum=shift(spectrum,sft)
        FOR kk = 1, count - 1 DO BEGIN ;loop over voxels in roi
            sft=pt-positions(t32(kk)) ;Shift spectrum
            tmp=shift(abs(fd_dat(*,xx(kk),yy(kk))),sft) ;Next spectrum shifted
            spectrum=spectrum+tmp ;add to sum
        ENDFOR
        spectrum=spectrum/count ;renormalize
        plot, paxis, spectrum, color=208,$
          xstyle=1, xtit='PPM', xrange=[last_ppm_lf, last_ppm_rt], $
          title='ROI',max_value=max_spec_val
    ENDIF 

;Display average metabolite values for defined ROI
    widget_control,s.crev_w,set_value=stats.cre_av+0.5
    widget_control,s.chov_w,set_value=stats.cho_av+0.5
    widget_control,s.naav_w,set_value=stats.naa_av+0.5
    CASE s.mets(3) OF
        3: widget_control,s.lacv_w,set_value=stats.lac_av+0.5
        4: widget_control,s.lacv_w,set_value=stats.ino_av+0.5
        5: widget_control,s.lacv_w,set_value=stats.h2o_av+0.5
    ENDCASE

;show sensitivity of ROI if class map is displayed
    show_sense

ENDIF ELSE BEGIN ;pixel mode
    last_pos=[x,y]

    IF n_params() GE 2 THEN BEGIN
        last_x_pos=x
        last_y_pos=y
    END

;  Check range of indices:

    IF last_x_pos GT sp_dim(0)-1 THEN last_x_pos = sp_dim(0)-1
    IF last_x_pos LT 0 THEN last_x_pos = 0
    IF last_y_pos GT sp_dim(1)-1 THEN last_y_pos = sp_dim(1)-1
    IF last_y_pos LT 0 THEN last_y_pos = 0

    IF flag THEN BEGIN
        spectrum = reform( abs(dry_dat(*,last_x_pos, last_y_pos)) )
        fit = reform( abs(fit_dat(*,last_x_pos, last_y_pos)) )
    ENDIF ELSE IF keyword_set(real) THEN BEGIN ; Use REAL data-points
        type='Real'
        spectrum = reform( fd_dat(*,last_x_pos, last_y_pos) )
    ENDIF ELSE BEGIN              ; Use MAGNITUDE data-points
        type='Magnitude'
        spectrum = reform( abs(fd_dat(*,last_x_pos, last_y_pos)) )
    ENDELSE

    pt=new_value                ;NAA

    title=string( format='(I2,",",I2)',x,y)
    plot, paxis, spectrum, color=208,$
      xstyle=1, xtit='PPM', xrange=[last_ppm_lf, last_ppm_rt], $
      title=title,max_value=max_spec_val

;Display metabolite value of selected pixel
IF flag THEN BEGIN ;time domain results
    widget_control,s.crev_w,set_value=cre_td(x,y)+0.5
    widget_control,s.chov_w,set_value=cho_td(x,y)+0.5
    widget_control,s.naav_w,set_value=naa_td(x,y)+0.5
    widget_control,s.lacv_w,set_value=0
ENDIF ELSE BEGIN ;frequency domain results
    widget_control,s.crev_w,set_value=cre(x,y)+0.5
    widget_control,s.chov_w,set_value=cho(x,y)+0.5
    widget_control,s.naav_w,set_value=naa(x,y)+0.5
    CASE s.mets(3) OF
        3: widget_control,s.lacv_w,set_value=lac(x,y)+0.5
        4: widget_control,s.lacv_w,set_value=ino(x,y)+0.5
        5: widget_control,s.lacv_w,set_value=h2o(x,y)+0.5
    ENDCASE
ENDELSE
;show sensitivity of ROI if class map is displayed
    show_sense,x,y
ENDELSE

IF n_elements(show_positions) EQ 0 THEN RETURN 
IF show_positions EQ 0 OR pt EQ -1 THEN RETURN 

IF flag EQ 1 THEN BEGIN
    oplot,paxis,fit,color=170
ENDIF ELSE BEGIN
;Plot NAA limits
    oplot,[paxis(round(pt-ir)),paxis(round(pt-ir))],[!y.crange(0), !y.crange(1)],color=202
    oplot,[paxis(round(pt+ir)),paxis(round(pt+ir))],[!y.crange(0), !y.crange(1)],color=202

;Plot LAC limits
    oplot,[paxis(round(pt+lac_offset-ir)),paxis(round(pt+lac_offset-ir))],[!y.crange(0), !y.crange(1)],$
      color=203
    oplot,[paxis(round(pt+lac_offset+ir)),paxis(round(pt+lac_offset+ir))],[!y.crange(0),!y.crange(1)],$
     color=203

;Plot h2o limits
    oplot,[paxis(round(pt+h2o_offset-ir)),paxis(round(pt+h2o_offset-ir))],[!y.crange(0), !y.crange(1)],$
      color=203
    oplot,[paxis(round(pt+h2o_offset+ir)),paxis(round(pt+h2o_offset+ir))],[!y.crange(0),!y.crange(1)],$
     color=203

;do not allow cre and cho regions to overlap
    cho2=round(pt+cho_offset+ir)
    cre1=round(pt+cre_offset-ir)
    IF cho2 GT cre1 THEN begin
        cho2=pt+(cho_offset+cre_offset)/2
        dev=convert_coord(paxis(cho2),40,/data,/to_device)
;   dev(0)=dev(0)+1
        cre1=cho2+1
    ENDIF
    pt=pt+cho_offset            ;cho
    oplot,[paxis(round(pt-ir)),paxis(round(pt-ir))],[!y.crange(0),!y.crange(1)],color=204
    oplot,[paxis(cho2),paxis(cho2)],[!y.crange(0),!y.crange(1)],color=204
    pt=pt-cho_offset+cre_offset ;cre
    oplot,[paxis(cre1),paxis(cre1)],[!y.crange(0), !y.crange(1)],color=205
    oplot,[paxis(round(pt+ir)),paxis(round(pt+ir))],[!y.crange(0), !y.crange(1)],color=205
ENDELSE

END

