;This subroutine uses Jan Willem's algorithm to find the water peak.
;Revision History
;981009 ASB First version.  Code from original by JWV.
;990407 ASB Added code to roundoff pixel number.
;           Chemical shifts now stored in COMMON mets.
pro tdpick

COMMON defroi
COMMON mrsidata
COMMON posdata
COMMON initcsi
COMMON mets

;******** Calculate mask for pixels on which to perform td analysis

afft=dblarr(pe_dim(0),pe_dim(1))

FOR i=0,31 DO FOR j=0,31 DO BEGIN
    afft(i,j)=max(abs(fd_dat(*,i,j))) ;height of water peaks
ENDFOR

immax=max(afft)
;Make mask of pixels to fit.  Include pixels with water peak heights
;within a factor of 50 of largest pixel.
w=filter_mask(afft,immax/50.)   

;Make B0 map from time domain data
ndata=fid_len-2
istart=1
iend=fid_len-2
f_map=dblarr(pe_dim(0),pe_dim(1))
id=findgen(ndata-1)-float(ndata-1)/2.

FOR i1=0,pe_dim(0)-1 DO BEGIN
  FOR i2=0,pe_dim(1)-1 DO BEGIN
    f_map(i1,i2)=0.
    IF (w(i1,i2) gt 0) THEN BEGIN
      test=reform(td_dat(istart:iend,i1,i2))
      maxtest=max(abs(test))/10.
      test=test/maxtest

      ap=atan(test) ;phase of time points
      b=ap(0:ndata-2)-ap(1:ndata-1)

;take derivative of phase to get frequency
      ix=where(b GT !dpi,found)
      IF (found GT 0) THEN b(ix)=b(ix)-2*!dpi
      ix=where(b LE -!dpi,found)
      IF (found GT 0) THEN b(ix)=b(ix)+2*!dpi
      pxx=ladfit(id,b)          ;Robust linear fit
      f_map(i1,i2)=pxx(0)       ;Frequency map
    ENDIF
  ENDFOR
ENDFOR

;Smooth the frequency map
f_map=filter_freq(f_map,w)

ppm_to_point = spec_freq * spc_len/ sweep_width
naa_offset = (h2o_shift-naa_shift) * ppm_to_point
positions=round(spc_len/2*(1-f_map/!pi)+naa_offset)
mask=where(w EQ 0)
positions(mask) = -1
widget_control,s.save_ids(1),sensitive=1 ;Save Positions
widget_control,s.save_ids(4),sensitive=1 ;Save Positions
widget_control,s.restore_ids(2),sensitive=1 ;Save Positions
widget_control,s.restore_ids(3),sensitive=1 ;Save Positions

RETURN
END
