;These programs test the spectra in an roi and report the results.
;Revision History
;09Aug96 ASB First SCCS version
;05Sep96 ASB Fixed bug involving rois containing one point.
;            Added code to read default file.
;18Jul97 ASB Removed reference to !spectro.
; 7Aug98 ASB Removed reference to s.num_w.
; 9Aug98 ASB Added common roistats.
;            Changed index to t32.
;14Aug98 ASB Fixed bugs.
;            Now can display spectrum by clicking on text in examine window.
;980915  ASB Replaced !csi_dat with fd_dat in COMMON freqdomain.
;980915  ASB Added COMMON initcsi and removed system variables.
;980916  ASB Changed name of COMMON freqdomain to mrsidata.
;990303  ASB Added COMMON initcsi to examine_event.

function test_roi
COMMON defroi
COMMON posdata
COMMON thresh,thresh_s
COMMON roistats
COMMON defaults
COMMON mrsidata
COMMON initcsi

set_thresh                      ;Set thresholds
thresh1=thresh_s.th1
thresh2=thresh_s.th2
thresh3=thresh_s.th3
;Intialize stuff
ppm_to_point = spec_freq * spc_len/ sweep_width
cho_offset = (2.023 - 3.25) * ppm_to_point
cre_offset = (2.023 - 3.05) * ppm_to_point
lac_offset = (2.023 - 1.33) * ppm_to_point ;not used
inos_offset = (2.023 - 3.53) * ppm_to_point ;not used

ir=fix(binwidth*ppm_to_point/2.0+0.5)

;Get current roi

count=count(0)
;convert to 2D coordinates
x=t32 mod 32
y=t32 / 32
bad=[-1]
bad1=[-1]
bad2=[-1]
bad3=[-1]

FOR k=0,count-1 DO BEGIN
    spectrum=abs(fd_dat(*,x(k),y(k)))

    pt=positions(x(k),y(k))
    IF pt EQ -1 THEN BEGIN
        bad=[bad,t32(k)]
    ENDIF ELSE BEGIN
        cho2=pt+cho_offset+ir
        cre1=pt+cre_offset-ir
        IF cho2 GT cre1 THEN begin
            cho2=pt+(cho_offset+cre_offset)/2
            cre1=cho2+1
        ENDIF
        chomax=max(spectrum(pt+cho_offset-ir:cho2))
        cremax=max(spectrum(cre1:pt+cre_offset+ir))
        chocremin=min(spectrum(pt+cho_offset:pt+cre_offset))
;Test for creatine-choline separation
        IF chocremin GT thresh1*min([chomax,cremax]) THEN BEGIN
            bad1=[bad1,t32(k)]
        ENDIF 
;Test for excessive noise outside to left of choline peak
        leftnoise=max(spectrum(pt+cho_offset-2*ir:pt+cho_offset-ir))
;        IF leftnoise GT thresh2*min([chomax,cremax]) THEN BEGIN
        IF leftnoise GT thresh2*chomax THEN BEGIN
            bad2=[bad2,t32(k)]
        ENDIF 
;Test for excessive noise around naa peak
        naamax=max(spectrum(pt-ir:pt+ir))
        naanoise1=max(spectrum(pt-2*ir:pt-ir))
        naanoise2=max(spectrum(pt+ir:pt+2*ir))
        naanoise=max([naanoise1,naanoise2])
        IF naanoise GT thresh3*naamax THEN BEGIN
            bad3=[bad3,t32(k)]
        ENDIF
    ENDELSE
ENDFOR 
;combine lists, strip leading -1, and remove duplicates

;combine bad and bad1
nbad1=n_elements(bad1)-1
IF nbad1 GT 0 THEN BEGIN
    bad1=bad1(1:nbad1)
    bad=[bad,bad1]
ENDIF

;combine bad and bad2
nbad2=n_elements(bad2)-1
IF nbad2 GT 0 THEN BEGIN
    bad2=bad2(1:nbad2)
    bad=[bad,bad2]
ENDIF

;combine bad and bad3
nbad3=n_elements(bad3)-1
IF nbad3 GT 0 THEN BEGIN
    bad3=bad3(1:nbad3)
    bad=[bad,bad3]
ENDIF

nbad=n_elements(bad)-1
IF nbad GT 0 THEN BEGIN
;strip leading -1
    bad=bad(1:nbad) 
;remove duplicates
    barray=bytarr(32,32)
    barray(bad)=1
    bad=where(barray EQ 1)
ENDIF

;make structure containing lists
bad={t:bad,s1:bad1,s2:bad2,s3:bad3}
RETURN,bad
END

pro examine_event,event
COMMON test_spectra,test_spectra_s
COMMON defroi
COMMON test,test_s
COMMON examine,examine_s
COMMON posdata
COMMON roistats
COMMON defaults
COMMON initcsi

CASE event.id OF
    examine_s.done_w: BEGIN
        num=n_elements(examine_s.buttons_w)-1
        n=s.npts
        dum32=intarr(32,32)
        widget_control,examine_s.buttons_w(1),get_uvalue=statarr
        FOR k = 0,test_s.nbad-1 DO BEGIN                        
            x=test_s.bad.t(k) mod 32 ;coordinate of bad point
            y=test_s.bad.t(k) / 32
            IF statarr(k) EQ 0 THEN BEGIN
                dum32(test_s.bad.t(k))=2 ;dum32 = 0 except for current point
            END
        ENDFOR
        dum256=rebin(dum32,256,256,/sample) ;dum256 = 0 except for current point
        widget_control,s.subs,get_uvalue=t256 ;ROI on high resolution images
        IF (size(t32))(0) NE 0 THEN BEGIN
            dum256(t256)=dum256(t256)+1
            dum32(t32)=dum32(t32)+1
        ENDIF
        t256=where(dum256 eq 1)
        t32=where(dum32 eq 1,count)
        widget_control,s.subs,set_uvalue=t256
        asb_defroi_update
        asb_defroi_draw,-1,/fill
        asb_defroi_draw, n-1    ;draw new point
        widget_control,examine_s.base_w,/destroy
        widget_control,test_s.base_w,/destroy
        asb_defroi_nmode
    END 

    examine_s.next_w: BEGIN     ;show next page
        widget_control,examine_s.base_w,get_uvalue=pages
        npages=pages(0)
        page=pages(1)+1
        lpp=pages(2)
        pages(1)=page
        widget_control,examine_s.base_w,set_uvalue=pages
        widget_control,examine_s.prev_w,sensitive=1
        title=string( format='( " Page ", I2, " of ", I2 )',page,npages)
        widget_control,examine_s.page_w,set_value=title
        IF page EQ npages THEN $
          widget_control,examine_s.next_w,sensitive=0
;update message
        IF page LT npages THEN nsp=lpp ELSE BEGIN
            nsp=test_s.nbad mod lpp
            IF nsp EQ 0 THEN nsp=lpp
        END 


        widget_control,examine_s.buttons_w(1),get_uvalue=statarr

        FOR kk=1,nsp DO BEGIN   ;make list of pixels
            k=kk+(page-1)*lpp
            x=test_s.bad.t(k-1) mod 32 ;coordinate of bad point
            y=test_s.bad.t(k-1) / 32
            flag=bytarr(3)
            IF (size(where(test_s.bad.s1 EQ test_s.bad.t(k-1))))(0) NE 0 THEN $
              flag(0)=1
            IF (size(where(test_s.bad.s2 EQ test_s.bad.t(k-1))))(0) NE 0 THEN $
              flag(1)=1
            IF (size(where(test_s.bad.s3 EQ test_s.bad.t(k-1))))(0) NE 0 THEN $
              flag(2)=1
            nf=fix(total(flag))
            CASE nf OF
                0: BEGIN
                    flunked=': Zapped spectrum'
                    widget_control,examine_s.buttons_w(kk),sensitive=0,$
                      set_value=0
                END
                
                1: BEGIN
                    flunked=': Flunked test '
                    nn=(where(flag EQ 1))(0)
                    nn=nn(0) + 1
                    flunked=flunked+strcompress(nn,/r)
                    widget_control,examine_s.buttons_w(kk),sensitive=1,$
                      set_value=statarr(k-1)
                END
                
                2: BEGIN
                    nn=(where(flag EQ 0))(0)
                    CASE nn OF
                        0: flunked=': Flunked tests 2 and 3'
                        1: flunked=': Flunked tests 1 and 3'
                        2: flunked=': Flunked tests 1 and 2'
                    ENDCASE
                    widget_control,examine_s.buttons_w(kk),sensitive=1,$
                      set_value=statarr(k-1)
                END
                
                3: BEGIN
                    flunked=': Flunked tests 1, 2, and 3'
                    widget_control,examine_s.buttons_w(kk),sensitive=1,$
                      set_value=statarr(k-1)
                END
                
            ENDCASE
            
            foo=string( format='( " spectrum (", I2, ",", I2, ")")',x,y)
            foo=foo+flunked
            title='                                                  ' 
            strput,title,foo
            widget_control,examine_s.text_w(kk),set_value=title
        ENDFOR                  ;kk
        FOR kk=nsp+1,lpp DO $
          widget_control,examine_s.text_w(kk),set_value='',sensitive=0
    END

    examine_s.prev_w: BEGIN
        widget_control,examine_s.base_w,get_uvalue=pages
        npages=pages(0)
        page=pages(1)-1
        lpp=pages(2)
        pages(1)=page
        widget_control,examine_s.base_w,set_uvalue=pages
        widget_control,examine_s.next_w,sensitive=1
        title=string( format='( " Page ", I2, " of ", I2 )',page,npages)
        widget_control,examine_s.page_w,set_value=title
        IF page EQ 1 THEN $
          widget_control,examine_s.prev_w,sensitive=0
;update message
        IF page LT npages THEN nsp=lpp ELSE BEGIN
            nsp=test_s.nbad mod lpp
            IF nsp EQ 0 THEN nsp=lpp
        END 

        widget_control,examine_s.buttons_w(1),get_uvalue=statarr

        FOR kk=1,nsp DO BEGIN
            k=kk+(page-1)*lpp
            x=test_s.bad.t(k-1) mod 32 ;coordinate of bad point
            y=test_s.bad.t(k-1) / 32
            flag=bytarr(3)
            IF (size(where(test_s.bad.s1 EQ test_s.bad.t(k-1))))(0) NE 0 THEN $
              flag(0)=1
            IF (size(where(test_s.bad.s2 EQ test_s.bad.t(k-1))))(0) NE 0 THEN $
              flag(1)=1
            IF (size(where(test_s.bad.s3 EQ test_s.bad.t(k-1))))(0) NE 0 THEN $
              flag(2)=1
            nf=fix(total(flag))
            CASE nf OF
                0: BEGIN
                    flunked=': Zapped spectrum'
                    widget_control,examine_s.buttons_w(kk),sensitive=0,$
                      set_value=0
                END
                
                1: BEGIN
                    flunked=': Flunked test '
                    nn=(where(flag EQ 1))(0)
                    nn=nn(0) + 1
                    flunked=flunked+strcompress(nn,/r)
                    widget_control,examine_s.buttons_w(kk),sensitive=1,$
                      set_value=statarr(k-1)
                END
                
                2: BEGIN
                    nn=(where(flag EQ 0))(0)
                    CASE nn OF
                        0: flunked=': Flunked tests 2 and 3'
                        1: flunked=': Flunked tests 1 and 3'
                        2: flunked=': Flunked tests 1 and 2'
                    ENDCASE
                    widget_control,examine_s.buttons_w(kk),sensitive=1,$
                      set_value=statarr(k-1)
                END
                
                3: BEGIN
                    flunked=': Flunked tests 1, 2, and 3'
                    widget_control,examine_s.buttons_w(kk),sensitive=1,$
                      set_value=statarr(k-1)
                END
                
            ENDCASE
            
            foo=string( format='( " spectrum (", I2, ",", I2, ")")',x,y)
            foo=foo+flunked
            title='                                                  ' 
            strput,title,foo
            widget_control,examine_s.text_w(kk),set_value=title
        ENDFOR                  ;kk
    END

    ELSE: BEGIN 
;Display spectrum to examine
        widget_control,examine_s.base_w,get_uvalue=pages
        npages=pages(0)
        page=pages(1)
        lpp=pages(2)

        k=where(examine_s.buttons_w EQ event.id)
        IF k(0) NE -1 THEN BEGIN  ;Include or exclude buttons
            widget_control,examine_s.buttons_w(k),get_value=foo        
            widget_control,examine_s.buttons_w(1),get_uvalue=statarr
            kk=k+(page-1)*lpp
            statarr(kk-1)=foo
            widget_control,examine_s.buttons_w(1),set_uvalue=statarr
        ENDIF ELSE BEGIN
            k=where(examine_s.text_w EQ event.id)
        ENDELSE
        kk=k+(page-1)*lpp
        x=test_s.bad.t(kk-1) mod 32 ;coordinates of spectrum
        y=test_s.bad.t(kk-1) / 32
        IF positions(x,y) LE 0 THEN ref_pos(0) = 0	$
        ELSE ref_pos(0) = positions(x,y)
        wset,s.swin
        update_specplot,x,y            
    END

ENDCASE

END

pro examine
;Create widget for examining spectra that failed test
COMMON test_spectra,test_spectra_s
COMMON defroi
COMMON test,test_s
COMMON examine,examine_s

line_w=[-1]
text_w=[-1]
buttons_w=[-1]
lpp=15
;set up pages
npages=fix(test_s.nbad+lpp-1)/lpp
statarr=replicate(byte(1),test_s.nbad)
page=1
base_w=widget_base(/column,title='Examine',uvalue=[npages,page,lpp])
IF npages GT 1 THEN nsp=lpp ELSE BEGIN
    nsp=test_s.nbad mod lpp
    IF nsp EQ 0 THEN nsp=lpp
END 

flag=bytarr(3)
FOR k=1,nsp DO BEGIN
    x=test_s.bad.t(k-1) mod 32  ;coordinate of bad point
    y=test_s.bad.t(k-1) / 32
    foo=widget_base(base_w,/row)
    line_w=[line_w,foo]
    foo=cw_bgroup(line_w(k),row=1,['Exclude','Include'],/return_name,$
                    /exclusive,set_value=1)
    buttons_w=[buttons_w,foo]
    flag(*)=0
    IF (size(where(test_s.bad.s1 EQ test_s.bad.t(k-1))))(0) NE 0 THEN $
      flag(0)=1
    IF (size(where(test_s.bad.s2 EQ test_s.bad.t(k-1))))(0) NE 0 THEN $
      flag(1)=1
    IF (size(where(test_s.bad.s3 EQ test_s.bad.t(k-1))))(0) NE 0 THEN $
      flag(2)=1
    nf=fix(total(flag))
    CASE nf OF
        0: BEGIN
            flunked=': Zapped spectrum'
            widget_control,foo,sensitive=0,set_value=0
            statarr(k-1)=0
        END
        
        1: BEGIN
            flunked=': Flunked test '
            nn=(where(flag EQ 1))(0)
            nn=nn(0) + 1
            flunked=flunked+strcompress(nn,/r)
            widget_control,foo,sensitive=1
        END
        
        2: BEGIN
            nn=(where(flag EQ 0))(0)
            CASE nn OF
                0: flunked=': Flunked tests 2 and 3'
                1: flunked=': Flunked tests 1 and 3'
                2: flunked=': Flunked tests 1 and 2'
            ENDCASE
            widget_control,foo,sensitive=1
        END
        
        3: BEGIN
            flunked=': Flunked tests 1, 2, and 3'
            widget_control,foo,sensitive=1
        END
        
    ENDCASE    
    foo=string( format='( " spectrum (", I2, ",", I2, ")")',x,y)
    foo=foo+flunked
    title='                                                  ' 
    strput,title,foo
    foo=widget_button(line_w(k),value=title)
    text_w=[text_w,foo]
ENDFOR                          ;k
FOR k = nsp+1,test_s.nbad DO BEGIN
    flag(*)=0
    IF (size(where(test_s.bad.s1 EQ test_s.bad.t(k-1))))(0) NE 0 THEN $
      flag(0)=1
    IF (size(where(test_s.bad.s2 EQ test_s.bad.t(k-1))))(0) NE 0 THEN $
      flag(1)=1
    IF (size(where(test_s.bad.s3 EQ test_s.bad.t(k-1))))(0) NE 0 THEN $
      flag(2)=1
    nf=fix(total(flag))
    IF nf EQ 0 THEN statarr(k-1)=0
ENDFOR


IF npages GT 1 THEN BEGIN        ;page choice widget
    pbase_w=widget_base(base_w,/row)
    next_w=widget_button(pbase_w,value='Next page')
    prev_w=widget_button(pbase_w,value='Previous page')
    title=string( format='( " Page ", I2, " of ", I2 )',page,npages)
    page_w=widget_text(pbase_w,value=title)
ENDIF ELSE BEGIN
    next_w=-1
    prev_w=-1
    page_w=-1
ENDELSE

done_w=widget_button(base_w,value='Done')
examine_s={base_w:base_w,$
           text_w:text_w,$ 
           buttons_w:buttons_w,$
           next_w:next_w,$
           prev_w:prev_w,$
           page_w:page_w,$
           done_w:done_w}

widget_control,base_w,/realize
IF npages GT 1 THEN widget_control,prev_w,sensitive=0
;put status array in uvalue of first button
widget_control,buttons_w(1),set_uvalue=statarr
xmanager,'examine',base_w,/modal

END


pro test_spectra_event,event
COMMON defroi
COMMON posdata
COMMON test,test_s
COMMON examine,examine_s
COMMON roistats

CASE event.id OF
    test_s.buttons_w: BEGIN
        CASE event.value OF 

            'Examine bad spectra':BEGIN
                IF test_s.nbad GT 0 THEN examine
            END

            'Exclude bad spectra':BEGIN
                IF test_s.nbad GT 0 THEN BEGIN
                    n=s.npts
;define region containing metabolite image pixel
                    dum32=intarr(32,32)
                    FOR k = 0,test_s.nbad-1 DO BEGIN                        
                        x=test_s.bad.t(k) mod 32 ;coordinate of bad point
                        y=test_s.bad.t(k) / 32
;dum32 = 0 except for current point
                        dum32(test_s.bad.t(k))=2 
                    ENDFOR
;dum256 = 0 except for current point
                    dum256=rebin(dum32,256,256,/sample)
;ROI on high resolution images
                    widget_control,s.subs,get_uvalue=t256 
;ROI on low resolution images
;                    widget_control,s.num_w,get_uvalue=t32 
                    IF (size(t32))(0) NE 0 THEN BEGIN
                        dum256(t256)=dum256(t256)+1
                        dum32(t32)=dum32(t32)+1
                    ENDIF
                    t256=where(dum256 eq 1)
                    t32=where(dum32 eq 1,count)
                    widget_control,s.subs,set_uvalue=t256
 ;                   widget_control,s.num_w,set_uvalue=t32
                    asb_defroi_update
                    asb_defroi_draw,-1,/fill
                    
                    asb_defroi_draw, n-1 ;draw new point
                    asb_defroi_nmode
                ENDIF
                widget_control,test_s.base_w,/destroy
            END
            
            'Quit':BEGIN
                widget_control,test_s.base_w,/destroy
            END
        END
    ENDCASE
ENDCASE
END

pro set_thresh_event,event
;Only the done button generates an event!
COMMON thresh,thresh_s
;Set values in structure equal to widget values and destroy widgets
widget_control,thresh_s.th1_w,get_value=foo
thresh_s.th1=foo
widget_control,thresh_s.th2_w,get_value=foo
thresh_s.th2=foo
widget_control,thresh_s.th3_w,get_value=foo
thresh_s.th3=foo
widget_control,thresh_s.base_w,/destroy
END


pro set_thresh
;This program allows the user to set the thresholds for the exclusion
;test
COMMON thresh,thresh_s
COMMON defaults
COMMON roistats

base_w=widget_base(/column,title='Set thresholds')
th1_w=cw_field(base_w,title='Cre-Cho separation threshold',/floating)
th2_w=cw_field(base_w,title='Cho noise threshold',/floating)
th3_w=cw_field(base_w,title='NAA noise threshold',/floating)
done_w=widget_button(base_w,value='done')

IF n_elements(thresh_s) EQ 0 THEN BEGIN
    widget_control,th1_w,set_value=crechosep
    widget_control,th2_w,set_value=chonoi
    widget_control,th3_w,set_value=naanoi
    thresh_s={base_w:base_w,$
;              done_w:done_w,$
              th1_w:th1_w,$
              th2_w:th2_w,$
              th3_w:th3_w,$
              th1:0.5,$
              th2:0.25,$
              th3:0.25}
ENDIF ELSE BEGIN
    thresh_s.base_w=base_w
;    thresh_s.done_w=done_w
    thresh_s.th1_w=th1_w
    thresh_s.th2_w=th2_w
    thresh_s.th3_w=th3_w
    widget_control,th1_w,set_value=thresh_s.th1
    widget_control,th2_w,set_value=thresh_s.th2
    widget_control,th3_w,set_value=thresh_s.th3
ENDELSE

widget_control,base_w,/realize
xmanager,'set_thresh',base_w,/modal

END

pro test_spectra
COMMON test_spectra,test_spectra_s
COMMON defroi
COMMON test,test_s
COMMON roistats

IF count EQ 0 THEN RETURN
bad=test_roi()
IF bad.t(0) EQ -1 THEN nbad=0 ELSE nbad=n_elements(bad.t) 
;count=n_elements(t32)
base_w=widget_base(/column,title='Test')
message=strarr(2)
message(0)=string('The ROI contains ',strtrim(count,2),' spectra.')
CASE nbad OF
    0:message(1)=string('They all passed the test')
    1:message(1)=string(strtrim(nbad,2),' spectrum flunked the test')
    ELSE:message(1)=string(strtrim(nbad,2),' spectra flunked the test.')
ENDCASE
text_w=widget_text(base_w,value=message,xsize=30,ysize=2)
buttons_w=cw_bgroup(base_w,row=4,['Examine bad spectra',$
                         'Exclude bad spectra','Quit'],/return_name)
test_s={base_w:base_w,$
        text_w:text_w,$
        buttons_w:buttons_w,$
        bad:bad,$
        nbad:nbad}
        
widget_control,base_w,/realize
xmanager,'test_spectra',base_w,/modal

END
