;This file contains three function needed to perform a time domain fit
;to a spectrum assumed to consist of NAA, CRE and CHO.

;curvefit performs the non-linear least squares fit using the
;Levenberg/Marquart algorithm. This version has been modified to
;supply graphic output to show the progress of the fitting procedure.

;fitexp calls curvefit to fit the three complex exponentials.

;funct evaluates the trial function and its derivatives.
;Revision History
; 981009 ASB Added graphical output to code by JWV.
; 990303 ASB Added code to compute error estimates.
;            Removed unused code from funct.

FUNCTION CURVEFIT, f_nr,X,Y,W,PAR,SIGMAA,chisqr
COMMON initcsi
;	ON_ERROR,2					;RETURN TO CALLER IF ERROR
        junk = CHECK_MATH(1,1)
;
        type = size(par)
        type = type(type(0)+1)
        if (type ne 4) and (type ne 5) then par = float(par)
;
	NTERMS = N_ELEMENTS(PAR)			;# OF PARAMS.
	NFREE = (N_ELEMENTS(Y)<N_ELEMENTS(X))-NTERMS 	;Degs of freedom
	IF NFREE LE 0 THEN STOP,'Curvefit - not enough data points.'
;
	FLAMBDA = 0.001					;Initial lambda
	DIAG = LINDGEN(NTERMS)*(NTERMS+1) 		;SUBSCRIPTS OF DIAGONAL ELEMENTS
;
;
	FOR ITER = 1,40 DO BEGIN			;Iteration loop
;
;		EVALUATE ALPHA AND BETA MATRICIES. # is matrix multiplication
;
		FUNCT, f_nr,X,PAR,YFIT,PDER			;COMPUTE FUNCTION AT PAR.

;Plot fit in time domain for debugging
                doplot=1
                IF doplot THEN BEGIN
                    tmp=dcomplexarr(512)
                    tmp1=tmp
                    tmp(0)=dcomplex(y(0:213),y(214:*))
                    tmp=shift(tmp,-107)
                    tmp=shift(fft(tmp),256)

                    tmp1(0)=dcomplex(yfit(0:213),yfit(214:*))
                    tmp1=shift(tmp1,-107)
                    tmp1=shift(fft(tmp1),256)

                    plot,paxis,abs(tmp),xrange=[4,0]
                    oplot,paxis,abs(tmp1),color=170
                ENDIF
		BETA = (Y-YFIT)*W # PDER
		ALPHA = TRANSPOSE(PDER) # (W # (FLTARR(NTERMS)+1)*PDER)
		IF (CHECK_MATH(1,1) NE 0) THEN GOTO,NOTDONE
;
		CHISQ1 = TOTAL(W*(Y-YFIT)^2)/NFREE 		;PRESENT CHI SQUARED.
;
;		INVERT MODIFIED CURVATURE MATRIX TO FIND NEW PARAMETERS.
;
		REPEAT BEGIN
;			C = SQRT(ALPHA(DIAG) # ALPHA(DIAG))      ; old way
                        c = sqrt(alpha(diag))                    ; minim. chance of under/overflow
                        c = c # c
			ARRAY = ALPHA/C
			array(diag) = array(diag)*(1.+flambda)
			ARRAY = INVERT(ARRAY)
        		IF (CHECK_MATH(1,1) NE 0) THEN GOTO,NOTDONE
			B = PAR+ ARRAY/C # TRANSPOSE(BETA) 	;NEW PARAMS
        		IF (CHECK_MATH(1,1) NE 0) THEN GOTO,NOTDONE
			FUNCT, f_nr,X,B,YFIT				;EVALUATE FUNCTION
			CHISQR = TOTAL(W*(Y-YFIT)^2)/NFREE 	;NEW CHISQR
			FLAMBDA = FLAMBDA*10.			;ASSUME FIT GOT WORSE
		ENDREP UNTIL CHISQR LE CHISQ1
;
		FLAMBDA = FLAMBDA/100.				;DECREASE FLAMBDA BY FACTOR OF 10
		PAR=B						;SAVE NEW PARAMETER ESTIMATE.
;		PRINT,'ITERATION =',ITER,' ,CHISQR =',CHISQR
;		PRINT,PAR

        	IF (CHECK_MATH(1,1) NE 0) THEN GOTO,NOTDONE
		IF ((CHISQ1-CHISQR)/CHISQ1) LE .001 THEN GOTO,DONE ;Finished?
	ENDFOR						;ITERATION LOOP
;Plot final iteration in time domain for debugging
       doplot=1
        IF doplot THEN BEGIN
            tmp=dcomplexarr(512)
            tmp1=tmp
            tmp(0)=dcomplex(y(0:213),y(214:*))
            tmp=shift(tmp,-107)
            tmp=shift(fft(tmp),256)

            tmp1(0)=dcomplex(yfit(0:213),yfit(214:*))
            tmp1=shift(tmp1,-107)
            tmp1=shift(fft(tmp1),256)

            plot,paxis,abs(tmp),xrange=[4,0]
            oplot,paxis,abs(tmp1),color=170
        ENDIF
;
NOTDONE:	print, 'Failed to converge' ;,chisq1,chisqr
        SIGMAA = fltarr(NTERMS)
        PAR(*)=0.
        RETURN,YFIT
;
DONE:   SIGMAA = SQRT(ARRAY(DIAG)/ALPHA(DIAG)) 		;RETURN SIGMA'S

        junk = CHECK_MATH(0,0)
	RETURN,YFIT					;RETURN RESULT
END
;***************************************************************************
;+
; CALLING SEQUENCE:
;	FUNCT, f_nr, X,FPAR,FUN,PDER
; INPUTS:
;       f_nr = function nr 1: single complex spectral line
;                          2: the NAA, Cho, Cr triplet.
;	T    = VALUES OF INDEPENDENT VARIABLE. (time here)
;	FPAR = PARAMETERS OF EQUATION DESCRIBED BELOW.
;       X    = PARAMETERS (depends on function nr)
; OUTPUTS:
;	FUN = VALUE OF FUNCTION AT EACH X(I).
;
; OPTIONAL OUTPUT PARAMETERS:
;	DFUN = (N_ELEMENTS(X), npar) ARRAY CONTAINING THE
;		PARTIAL DERIVATIVES.  DFUN(I,J) = DERIVATIVE
;		AT ITH POINT W/RESPECT TO JTH PARAMETER.
; PROCEDURE:
;  function nr = 2:
;  FUN(0:cut-1) = sum(i) (A1i sin((w0+wi)t) + A2i cos((w0+wi)t)) exp(-bt)
;  FUN(cut:*  ) = sum(i) (A1i cos((w0+wi)t) - A2i sin((w0+wi)t)) exp(-bt)
;
; MODIFICATION HISTORY:
;	WRITTEN, DMS, RSI, SEPT, 1982.
;       Jan Willem van der Veen, jan 1998
;-
PRO	FUNCT, f_nr,T,FPAR,FUN,DFUN

s = N_ELEMENTS(T)
NTERMS = N_ELEMENTS(FPAR)

cut = s/2

type = size(FPAR)
type = type(type(0)+1)
dpar = type eq 5

if dpar then begin
  FUN = dblarr(s)
  IF (N_PARAMS(0) GT 4) THEN DFUN = dblarr(s, nterms)
endif else begin
  FUN = fltarr(s)
  IF (N_PARAMS(0) GT 4) THEN DFUN = fltarr(s, nterms)
endelse

WNAA = (2.0100-4.75) * 2. * !dpi * 63.85 ;!spec_freq
WCR  = (3.0436-4.75) * 2. * !dpi * 63.85 ;!spec_freq. was 3.0469
WCHO = (3.2234-4.75) * 2. * !dpi * 63.85 ;!spec_freq. was 3.2314

if (f_nr EQ 1) then begin
	A1 = FPAR(0)				; amplitude cos 1
	A2 = FPAR(1)				; amplitude sin 1
	B  = FPAR(2)				; damping
	W  = FPAR(3)				; frequency
	T2 = T(0:cut-1)

	B    = ABS(B)
	if (B lt 1.e-9) then B=1.e-9
	BEXP = EXP(-B*T2)				; decay
	FS   = SIN(W*T2)				; sin part
	FC   = COS(W*T2)				; cos part

	FUN(0:cut-1) = ( A1*FS+A2*FC )*BEXP
	FUN(cut:s-1) = ( A1*FC-A2*FS )*BEXP

	IF N_PARAMS(0) LE 4 THEN RETURN 		; need derivatives

	DFUN(0:cut-1,0) = FS*BEXP		; d FUN / d A11
	DFUN(cut:s-1,0) = FC*BEXP

 	DFUN(0:cut-1,1) = FC*BEXP 		; d FUN / d A21
	DFUN(cut:s-1,1) =-FS*BEXP 

	DFUN(0:cut-1,2) = FUN(0:cut-1)*(-T2)
	DFUN(cut:s-1,2) = FUN(cut:s-1)*(-T2)

	DFUN(0:cut-1,3) = ( A1*FC-A2*FS)*T2*BEXP
	DFUN(cut:s-1,3) = (-A1*FS-A2*FC)*T2*BEXP

endif
if (f_nr EQ 2) then begin
	A11 = FPAR(0)					; amplitude cos 1
	A21 = FPAR(1)					; amplitude sin 1
	A12 = FPAR(2)					; amplitude cos 2
	A22 = FPAR(3)					; amplitude sin 2
	A13 = FPAR(4)					; amplitude cos 3
	A23 = FPAR(5)					; amplitude sin 3
	B  = FPAR(6)					; damping
	W  = FPAR(7)					; frequency
        T2 = T(0:cut-1)

	B = ABS(B)
	if (B lt 1.e-9) then B=1.e-9
	BEXP = EXP(-B*ABS(T2))				; decay (abs T2 for echo)
        BEXPT21 = EXP(-3.33*T2)                         ; T2 component NAA
        BEXPT22 = EXP(-5.26*T2)                         ; T2 component Cr
        BEXPT23 = EXP(-3.70*T2)                         ; T2 component Cho
       
	FS1   = SIN((W+WNAA)*T2)			; sin part NAA
	FC1   = COS((W+WNAA)*T2)			; cos part NAA
	FS2   = SIN((W+WCR )*T2)			; sin part Cr
	FC2   = COS((W+WCR )*T2)			; cos part Cr
	FS3   = SIN((W+WCHO)*T2)			; sin part Cho
	FC3   = COS((W+WCHO)*T2)			; cos part Cho

	FUN(0:cut-1) = ( ( A11*FS1+A21*FC1)*BEXPT21 + ( A12*FS2+A22*FC2)*BEXPT22 + ( A13*FS3+A23*FC3)*BEXPT23 )*BEXP
	FUN(cut:s-1) = ( ( A11*FC1-A21*FS1)*BEXPT21 + ( A12*FC2-A22*FS2)*BEXPT22 + ( A13*FC3-A23*FS3)*BEXPT23 )*BEXP

	IF N_PARAMS(0) LE 4 THEN RETURN 		; need derivatives

	DFUN(0:cut-1,0) =  FS1*BEXP*BEXPT21		; d FUN / d A11
	DFUN(cut:s-1,0) =  FC1*BEXP*BEXPT21		; d FUN / d A11
	DFUN(0:cut-1,1) =  FC1*BEXP*BEXPT21		; d FUN / d A21
	DFUN(cut:s-1,1) = -FS1*BEXP*BEXPT21		; d FUN / d A21

	DFUN(0:cut-1,2) =  FS2*BEXP*BEXPT22		; d FUN / d A12
	DFUN(cut:s-1,2) =  FC2*BEXP*BEXPT22		; d FUN / d A12
	DFUN(0:cut-1,3) =  FC2*BEXP*BEXPT22		; d FUN / d A22
	DFUN(cut:s-1,3) = -FS2*BEXP*BEXPT22		; d FUN / d A22

	DFUN(0:cut-1,4) =  FS3*BEXP*BEXPT23		; d FUN / d A13
	DFUN(cut:s-1,4) =  FC3*BEXP*BEXPT23		; d FUN / d A13
	DFUN(0:cut-1,5) =  FC3*BEXP*BEXPT23		; d FUN / d A23
	DFUN(cut:s-1,5) = -FS3*BEXP*BEXPT23		; d FUN / d A23

	DFUN(0:cut-1,6) = FUN(0:cut-1)*(-ABS(T2))	; d FUN / d B
	DFUN(cut:s-1,6) = FUN(cut:s-1)*(-ABS(T2))	; d FUN / d B

	DFUN(0:cut-1,7) = ( ( A11*FC1-A21*FS1)*BEXPT21 + ( A12*FC2-A22*FS2)*BEXPT22 + ( A13*FC3-A23*FS3)*BEXPT23 )*T2*BEXP
	DFUN(cut:s-1,7) = ( (-A11*FS1-A21*FC1)*BEXPT21 + (-A12*FS2-A22*FC2)*BEXPT22 + (-A13*FS3-A23*FC3)*BEXPT23 )*T2*BEXP
endif
if (f_nr EQ 3) then begin
	A11 = FPAR(0)					; amplitude cos 1
	A21 = FPAR(1)					; amplitude sin 1
	A12 = FPAR(2)					; amplitude cos 2
	A22 = FPAR(3)					; amplitude sin 2
	A13 = FPAR(4)					; amplitude cos 3
	A23 = FPAR(5)					; amplitude sin 3
	B  = FPAR(6)					; damping
	W1 = FPAR(7)					; frequency
        W2 = FPAR(8)
        W3 = FPAR(9)
        T2 = T(0:cut-1)

	B = ABS(B)
	if (B lt 1.e-9) then B=1.e-9
	BEXP = EXP(-B*ABS(T2))				; decay (abs T2 for echo)
        BEXPT21 = EXP(-3.33*T2)                         ; T2 component NAA
        BEXPT22 = EXP(-5.26*T2)                         ; T2 component Cr
        BEXPT23 = EXP(-3.70*T2)                         ; T2 component Cho
       
	FS1   = SIN((W1+WNAA)*T2)			; sin part NAA
	FC1   = COS((W1+WNAA)*T2)			; cos part NAA
	FS2   = SIN((W2+WCR )*T2)			; sin part Cr
	FC2   = COS((W2+WCR )*T2)			; cos part Cr
	FS3   = SIN((W3+WCHO)*T2)			; sin part Cho
	FC3   = COS((W3+WCHO)*T2)			; cos part Cho

	FUN(0:cut-1) = ( ( A11*FS1+A21*FC1)*BEXPT21 + ( A12*FS2+A22*FC2)*BEXPT22 + ( A13*FS3+A23*FC3)*BEXPT23 )*BEXP
	FUN(cut:s-1) = ( ( A11*FC1-A21*FS1)*BEXPT21 + ( A12*FC2-A22*FS2)*BEXPT22 + ( A13*FC3-A23*FS3)*BEXPT23 )*BEXP

	IF N_PARAMS(0) LE 4 THEN RETURN 		; need derivatives

	DFUN(0:cut-1,0) =  FS1*BEXP*BEXPT21		; d FUN / d A11
	DFUN(cut:s-1,0) =  FC1*BEXP*BEXPT21		; d FUN / d A11
	DFUN(0:cut-1,1) =  FC1*BEXP*BEXPT21		; d FUN / d A21
	DFUN(cut:s-1,1) = -FS1*BEXP*BEXPT21		; d FUN / d A21

	DFUN(0:cut-1,2) =  FS2*BEXP*BEXPT22		; d FUN / d A12
	DFUN(cut:s-1,2) =  FC2*BEXP*BEXPT22		; d FUN / d A12
	DFUN(0:cut-1,3) =  FC2*BEXP*BEXPT22		; d FUN / d A22
	DFUN(cut:s-1,3) = -FS2*BEXP*BEXPT22		; d FUN / d A22

	DFUN(0:cut-1,4) =  FS3*BEXP*BEXPT23		; d FUN / d A13
	DFUN(cut:s-1,4) =  FC3*BEXP*BEXPT23		; d FUN / d A13
	DFUN(0:cut-1,5) =  FC3*BEXP*BEXPT23		; d FUN / d A23
	DFUN(cut:s-1,5) = -FS3*BEXP*BEXPT23		; d FUN / d A23

	DFUN(0:cut-1,6) = FUN(0:cut-1)*(-ABS(T2))	; d FUN / d B
	DFUN(cut:s-1,6) = FUN(cut:s-1)*(-ABS(T2))	; d FUN / d B

	DFUN(0:cut-1,7) = ( ( A11*FC1-A21*FS1)*BEXPT21 )*T2*BEXP
	DFUN(cut:s-1,7) = ( (-A11*FS1-A21*FC1)*BEXPT21 )*T2*BEXP

	DFUN(0:cut-1,8) = ( ( A12*FC2-A22*FS2)*BEXPT22 )*T2*BEXP
	DFUN(cut:s-1,8) = ( (-A12*FS2-A22*FC2)*BEXPT22 )*T2*BEXP

	DFUN(0:cut-1,9) = ( ( A13*FC3-A23*FS3)*BEXPT23 )*T2*BEXP
	DFUN(cut:s-1,9) = ( (-A13*FS3-A23*FC3)*BEXPT23 )*T2*BEXP
endif
if (f_nr EQ 4) then begin
	A1  = FPAR(0)					; amplitude cos 1
	A2  = FPAR(1)					; amplitude sin 1
	B   = FPAR(2)					; damping
	B2  = FPAR(3)					; T2 damping
	W   = FPAR(4)					; frequency

	T2  = T(0:cut-1)

	B2 = ABS(B2)
	B  = ABS(B)
	if (B  lt 1.e-9) then B =1.e-9
	if (B2 lt 1.e-9) then B2=1.e-9
	BEXP   = EXP(-B*ABS(T2))			; decay (abs T2 for echo)
        BEXPT2 = EXP(-B2*T2) 				; T2 component
      
	FS   = SIN(W*T2)				; sin part
	FC   = COS(W*T2)				; cos part

	FUN(0:cut-1) = (A1*FS+A2*FC)*BEXPT2*BEXP
	FUN(cut:s-1) = (A1*FC-A2*FS)*BEXPT2*BEXP

	IF N_PARAMS(0) LE 4 THEN RETURN 		; need derivatives

	DFUN(0:cut-1,0) =  FS*BEXP*BEXPT2		; d FUN / d A1
	DFUN(cut:s-1,0) =  FC*BEXP*BEXPT2		; d FUN / d A1

	DFUN(0:cut-1,1) =  FC*BEXP*BEXPT2		; d FUN / d A2
	DFUN(cut:s-1,1) = -FS*BEXP*BEXPT2		; d FUN / d A2

	DFUN(0:cut-1,2) = FUN(0:cut-1)*(-ABS(T2))	; d FUN / d B
	DFUN(cut:s-1,2) = FUN(cut:s-1)*(-ABS(T2))	; d FUN / d B

	DFUN(0:cut-1,3) = FUN(0:cut-1)*(-T2)	        ; d FUN / d B2
	DFUN(cut:s-1,3) = FUN(cut:s-1)*(-T2)	        ; d FUN / d B2

	DFUN(0:cut-1,4) = ( A1*FC-A2*FS)*BEXPT2*T2*BEXP
	DFUN(cut:s-1,4) = (-A1*FS-A2*FC)*BEXPT2*T2*BEXP
endif
if (f_nr EQ 5) then begin
	A1  = FPAR(0)					; amplitude cos 1
	A2  = FPAR(1)					; amplitude sin 1
	B   = FPAR(2)					; damping sinc
	B2  = FPAR(3)					; T2 damping
	W   = FPAR(4)					; frequency

	T2  = T(0:cut-1)

	B = ABS(B)
	if (B lt 1.e-6) then B=1.e-6
	ix = where (T2 ne 0.)
	ix2= where (T2 eq 0.)
	BSINC  = sin(T2*B)				; decay (abs T2 for echo)
        BSINC(ix)=BSINC(ix)/(B*T2(ix))
        BSINC(ix2)=double(1.)

	B2 = ABS(B2)
	if (B2 lt 1.e-9) then B2=1.e-9
        BEXP = EXP(-B2*T2)
      
	FS   = SIN(W*T2)				; sin part
	FC   = COS(W*T2)				; cos part

	FUN(0:cut-1) = (A1*FS+A2*FC)*BSINC*BEXP
	FUN(cut:s-1) = (A1*FC-A2*FS)*BSINC*BEXP

	IF N_PARAMS(0) LE 4 THEN RETURN 		; need derivatives

	DFUN(0:cut-1,0) = FS*BEXP*BSINC
	DFUN(cut:s-1,0) = FC*BEXP*BSINC

	DFUN(0:cut-1,1) = FC*BEXP*BSINC
	DFUN(cut:s-1,1) =-FS*BEXP*BSINC

	DFUN(0:cut-1,2) = (A1*FS+A2*FC)*BEXP*((cos(B*T2) - BSINC)/B)
	DFUN(cut:s-1,2) = (A1*FC-A2*FS)*BEXP*((cos(B*T2) - BSINC)/B)

	DFUN(0:cut-1,3) = FUN(0:cut-1)*(-T2)
	DFUN(cut:s-1,3) = FUN(cut:s-1)*(-T2)

	DFUN(0:cut-1,4) = ( A1*FC-A2*FS)*BEXP*T2*BSINC
	DFUN(cut:s-1,4) = (-A1*FS-A2*FC)*BEXP*T2*BSINC

endif
if (f_nr EQ 6) then begin
	A11 = FPAR(0)					; amplitude cos 1
	A21 = FPAR(1)					; amplitude sin 1
	A12 = FPAR(2)					; amplitude cos 2
	A22 = FPAR(3)					; amplitude sin 2
	A13 = FPAR(4)					; amplitude cos 3
	A23 = FPAR(5)					; amplitude sin 3
	B  = FPAR(6)					; damping
	W1 = FPAR(7)					; frequency
	W2 = FPAR(8)
	W3 = FPAR(9)
	T2 = T(0:cut-1)
        T2T2 = T2*T2

	B = ABS(B)
	if (B lt 1.e-9) then B=1.e-9
	BEXP = EXP(-B*T2T2)				; decay (T2*T2 for echo)
	BEXPT21 = EXP(-3.33*T2)                         ; T2 component NAA
	BEXPT22 = EXP(-5.26*T2)                         ; T2 component Cr
	BEXPT23 = EXP(-3.70*T2)                         ; T2 component Cho
       
	FS1   = SIN((W1+WNAA)*T2)			; sin part NAA
	FC1   = COS((W1+WNAA)*T2)			; cos part NAA
	FS2   = SIN((W2+WCR )*T2)			; sin part Cr
	FC2   = COS((W2+WCR )*T2)			; cos part Cr
	FS3   = SIN((W3+WCHO)*T2)			; sin part Cho
	FC3   = COS((W3+WCHO)*T2)			; cos part Cho

	FUN(0:cut-1) = ( ( A11*FS1+A21*FC1)*BEXPT21 + ( A12*FS2+A22*FC2)*BEXPT22 + ( A13*FS3+A23*FC3)*BEXPT23 )*BEXP
	FUN(cut:s-1) = ( ( A11*FC1-A21*FS1)*BEXPT21 + ( A12*FC2-A22*FS2)*BEXPT22 + ( A13*FC3-A23*FS3)*BEXPT23 )*BEXP

	IF N_PARAMS(0) LE 4 THEN RETURN 		; need derivatives

	DFUN(0:cut-1,0) =  FS1*BEXP*BEXPT21		; d FUN / d A11
	DFUN(cut:s-1,0) =  FC1*BEXP*BEXPT21		; d FUN / d A11
	DFUN(0:cut-1,1) =  FC1*BEXP*BEXPT21		; d FUN / d A21
	DFUN(cut:s-1,1) = -FS1*BEXP*BEXPT21		; d FUN / d A21

	DFUN(0:cut-1,2) =  FS2*BEXP*BEXPT22		; d FUN / d A12
	DFUN(cut:s-1,2) =  FC2*BEXP*BEXPT22		; d FUN / d A12
	DFUN(0:cut-1,3) =  FC2*BEXP*BEXPT22		; d FUN / d A22
	DFUN(cut:s-1,3) = -FS2*BEXP*BEXPT22		; d FUN / d A22

	DFUN(0:cut-1,4) =  FS3*BEXP*BEXPT23		; d FUN / d A13
	DFUN(cut:s-1,4) =  FC3*BEXP*BEXPT23		; d FUN / d A13
	DFUN(0:cut-1,5) =  FC3*BEXP*BEXPT23		; d FUN / d A23
	DFUN(cut:s-1,5) = -FS3*BEXP*BEXPT23		; d FUN / d A23

	DFUN(0:cut-1,6) = FUN(0:cut-1)*(-T2T2)		; d FUN / d B
	DFUN(cut:s-1,6) = FUN(cut:s-1)*(-T2T2)		; d FUN / d B

	DFUN(0:cut-1,7) = ( ( A11*FC1-A21*FS1)*BEXPT21 )*T2*BEXP
	DFUN(cut:s-1,7) = ( (-A11*FS1-A21*FC1)*BEXPT21 )*T2*BEXP

	DFUN(0:cut-1,8) = ( ( A12*FC2-A22*FS2)*BEXPT22 )*T2*BEXP
	DFUN(cut:s-1,8) = ( (-A12*FS2-A22*FC2)*BEXPT22 )*T2*BEXP

	DFUN(0:cut-1,9) = ( ( A13*FC3-A23*FS3)*BEXPT23 )*T2*BEXP
	DFUN(cut:s-1,9) = ( (-A13*FS3-A23*FC3)*BEXPT23 )*T2*BEXP
endif
if (f_nr EQ 7) then begin
	A1 = FPAR(0)				; amplitude cos 1
	A2 = FPAR(1)				; amplitude sin 1
	B  = FPAR(2)				; damping
	W  = FPAR(3)				; frequency
	T2 = T(0:cut-1)
	T2T2 = T2*T2

	B    = ABS(B)
	if (B lt 1.e-9) then B=1.e-9
	BEXP = EXP(-B*T2T2)				; decay
	FS   = SIN(W*T2)				; sin part
	FC   = COS(W*T2)				; cos part

	FUN(0:cut-1) = ( A1*FS+A2*FC )*BEXP
	FUN(cut:s-1) = ( A1*FC-A2*FS )*BEXP

	IF N_PARAMS(0) LE 4 THEN RETURN 		; need derivatives

	DFUN(0:cut-1,0) = FS*BEXP			; d FUN / d A11
	DFUN(cut:s-1,0) = FC*BEXP

 	DFUN(0:cut-1,1) = FC*BEXP 			; d FUN / d A21
	DFUN(cut:s-1,1) =-FS*BEXP 

	DFUN(0:cut-1,2) = FUN(0:cut-1)*(-T2T2)
	DFUN(cut:s-1,2) = FUN(cut:s-1)*(-T2T2)

	DFUN(0:cut-1,3) = ( A1*FC-A2*FS)*T2*BEXP
	DFUN(cut:s-1,3) = (-A1*FS-A2*FC)*T2*BEXP

endif
if (f_nr EQ 8) then begin
	A1 = FPAR(0)				; amplitude cos 1
	A2 = FPAR(1)				; amplitude sin 1
	B  = FPAR(2)				; damping
	W  = FPAR(3)				; frequency
	T2 = T(0:cut-1)

	BEXP = EXP(-B*B*T2)				; decay
	FS   = SIN(W*T2)				; sin part
	FC   = COS(W*T2)				; cos part

	FUN(0:cut-1) = ( A1*FS+A2*FC )*BEXP
	FUN(cut:s-1) = ( A1*FC-A2*FS )*BEXP

	IF N_PARAMS(0) LE 4 THEN RETURN 		; need derivatives

	DFUN(0:cut-1,0) = FS*BEXP		; d FUN / d A11
	DFUN(cut:s-1,0) = FC*BEXP

 	DFUN(0:cut-1,1) = FC*BEXP 		; d FUN / d A21
	DFUN(cut:s-1,1) =-FS*BEXP 

	DFUN(0:cut-1,2) = FUN(0:cut-1)*(-T2)*2.*B
	DFUN(cut:s-1,2) = FUN(cut:s-1)*(-T2)*2.*B

	DFUN(0:cut-1,3) = ( A1*FC-A2*FS)*T2*BEXP
	DFUN(cut:s-1,3) = (-A1*FS-A2*FC)*T2*BEXP

endif

if (f_nr EQ 9) then begin
	A1 = FPAR(0)				; amplitude cos 1
	A2 = FPAR(1)				; amplitude sin 1
	B  = FPAR(2)				; damping
	W  = FPAR(3)				; frequency
	T2 = T(0:cut-1)
	T2T2 = T2*T2

	BEXP = EXP(-B*B*T2T2)				; decay
	FS   = SIN(W*T2)				; sin part
	FC   = COS(W*T2)				; cos part

	FUN(0:cut-1) = ( A1*FS+A2*FC )*BEXP
	FUN(cut:s-1) = ( A1*FC-A2*FS )*BEXP

	IF N_PARAMS(0) LE 4 THEN RETURN 		; need derivatives

	DFUN(0:cut-1,0) = FS*BEXP			; d FUN / d A11
	DFUN(cut:s-1,0) = FC*BEXP

 	DFUN(0:cut-1,1) = FC*BEXP 			; d FUN / d A21
	DFUN(cut:s-1,1) =-FS*BEXP 

	DFUN(0:cut-1,2) = FUN(0:cut-1)*(-T2T2)*2.*B
	DFUN(cut:s-1,2) = FUN(cut:s-1)*(-T2T2)*2.*B

	DFUN(0:cut-1,3) = ( A1*FC-A2*FS)*T2*BEXP
	DFUN(cut:s-1,3) = (-A1*FS-A2*FC)*T2*BEXP

endif
if (f_nr EQ 10) then begin      ;fit 10 parameters
	A11 = FPAR(0)					; amplitude cos 1
	A21 = FPAR(1)					; amplitude sin 1
	A12 = FPAR(2)					; amplitude cos 2
	A22 = FPAR(3)					; amplitude sin 2
	A13 = FPAR(4)					; amplitude cos 3
	A23 = FPAR(5)					; amplitude sin 3
	B  = FPAR(6)					; damping
	W1 = FPAR(7)					; frequency
	W2 = FPAR(8)
	W3 = FPAR(9)
	T2 = T(0:cut-1)
        T2T2 = T2*T2

	BEXP = EXP(-B*B*T2T2)				; decay (T2*T2 for echo)
	BEXPT21 = EXP(-2.27*T2)                         ; T2 component NAA was 3.33
	BEXPT22 = EXP(-4.83*T2)                         ; T2 component Cr  was 5.26
	BEXPT23 = EXP(-2.75*T2)                         ; T2 component Cho was 3.70
       
	FS1   = SIN((W1+WNAA)*T2)			; sin part NAA
	FC1   = COS((W1+WNAA)*T2)			; cos part NAA
	FS2   = SIN((W2+WCR )*T2)			; sin part Cr
	FC2   = COS((W2+WCR )*T2)			; cos part Cr
	FS3   = SIN((W3+WCHO)*T2)			; sin part Cho
	FC3   = COS((W3+WCHO)*T2)			; cos part Cho

	FUN(0:cut-1) = ( ( A11*FS1+A21*FC1)*BEXPT21 + ( A12*FS2+A22*FC2)*BEXPT22 + ( A13*FS3+A23*FC3)*BEXPT23 )*BEXP
	FUN(cut:s-1) = ( ( A11*FC1-A21*FS1)*BEXPT21 + ( A12*FC2-A22*FS2)*BEXPT22 + ( A13*FC3-A23*FS3)*BEXPT23 )*BEXP

	IF N_PARAMS(0) LE 4 THEN RETURN 		; need derivatives

	DFUN(0:cut-1,0) =  FS1*BEXP*BEXPT21		; d FUN / d A11
	DFUN(cut:s-1,0) =  FC1*BEXP*BEXPT21		; d FUN / d A11
	DFUN(0:cut-1,1) =  FC1*BEXP*BEXPT21		; d FUN / d A21
	DFUN(cut:s-1,1) = -FS1*BEXP*BEXPT21		; d FUN / d A21

	DFUN(0:cut-1,2) =  FS2*BEXP*BEXPT22		; d FUN / d A12
	DFUN(cut:s-1,2) =  FC2*BEXP*BEXPT22		; d FUN / d A12
	DFUN(0:cut-1,3) =  FC2*BEXP*BEXPT22		; d FUN / d A22
	DFUN(cut:s-1,3) = -FS2*BEXP*BEXPT22		; d FUN / d A22

	DFUN(0:cut-1,4) =  FS3*BEXP*BEXPT23		; d FUN / d A13
	DFUN(cut:s-1,4) =  FC3*BEXP*BEXPT23		; d FUN / d A13
	DFUN(0:cut-1,5) =  FC3*BEXP*BEXPT23		; d FUN / d A23
	DFUN(cut:s-1,5) = -FS3*BEXP*BEXPT23		; d FUN / d A23

	DFUN(0:cut-1,6) = FUN(0:cut-1)*(-T2T2)*2.*B	; d FUN / d B
	DFUN(cut:s-1,6) = FUN(cut:s-1)*(-T2T2)*2.*B	; d FUN / d B

	DFUN(0:cut-1,7) = ( ( A11*FC1-A21*FS1)*BEXPT21 )*T2*BEXP
	DFUN(cut:s-1,7) = ( (-A11*FS1-A21*FC1)*BEXPT21 )*T2*BEXP

	DFUN(0:cut-1,8) = ( ( A12*FC2-A22*FS2)*BEXPT22 )*T2*BEXP
	DFUN(cut:s-1,8) = ( (-A12*FS2-A22*FC2)*BEXPT22 )*T2*BEXP

	DFUN(0:cut-1,9) = ( ( A13*FC3-A23*FS3)*BEXPT23 )*T2*BEXP
	DFUN(cut:s-1,9) = ( (-A13*FS3-A23*FC3)*BEXPT23 )*T2*BEXP
endif
if (f_nr EQ 11) then begin      ;Fit 8 parameters
	A11 = FPAR(0)					; amplitude cos 1
	A21 = FPAR(1)					; amplitude sin 1
	A12 = FPAR(2)					; amplitude cos 2
	A22 = FPAR(3)					; amplitude sin 2
	A13 = FPAR(4)					; amplitude cos 3
	A23 = FPAR(5)					; amplitude sin 3
	B  = FPAR(6)					; damping
	W  = FPAR(7)					; frequency
        T2 = T(0:cut-1)
        T2T2 = T2*T2

	BEXP = EXP(-B*B*T2T2)				; decay (T2*T2 for echo)
	BEXPT21 = EXP(-2.27*T2)                         ; T2 component NAA was 3.33
	BEXPT22 = EXP(-4.83*T2)                         ; T2 component Cr  was 5.26
	BEXPT23 = EXP(-2.75*T2)                         ; T2 component Cho was 3.70
       
	FS1   = SIN((W+WNAA)*T2)			; sin part NAA
	FC1   = COS((W+WNAA)*T2)			; cos part NAA
	FS2   = SIN((W+WCR )*T2)			; sin part Cr
	FC2   = COS((W+WCR )*T2)			; cos part Cr
	FS3   = SIN((W+WCHO)*T2)			; sin part Cho
	FC3   = COS((W+WCHO)*T2)			; cos part Cho

	FUN(0:cut-1) = ( ( A11*FS1+A21*FC1)*BEXPT21 + ( A12*FS2+A22*FC2)*BEXPT22 + ( A13*FS3+A23*FC3)*BEXPT23 )*BEXP
	FUN(cut:s-1) = ( ( A11*FC1-A21*FS1)*BEXPT21 + ( A12*FC2-A22*FS2)*BEXPT22 + ( A13*FC3-A23*FS3)*BEXPT23 )*BEXP

	IF N_PARAMS(0) LE 4 THEN RETURN 		; need derivatives

	DFUN(0:cut-1,0) =  FS1*BEXP*BEXPT21		; d FUN / d A11
	DFUN(cut:s-1,0) =  FC1*BEXP*BEXPT21		; d FUN / d A11
	DFUN(0:cut-1,1) =  FC1*BEXP*BEXPT21		; d FUN / d A21
	DFUN(cut:s-1,1) = -FS1*BEXP*BEXPT21		; d FUN / d A21

	DFUN(0:cut-1,2) =  FS2*BEXP*BEXPT22		; d FUN / d A12
	DFUN(cut:s-1,2) =  FC2*BEXP*BEXPT22		; d FUN / d A12
	DFUN(0:cut-1,3) =  FC2*BEXP*BEXPT22		; d FUN / d A22
	DFUN(cut:s-1,3) = -FS2*BEXP*BEXPT22		; d FUN / d A22

	DFUN(0:cut-1,4) =  FS3*BEXP*BEXPT23		; d FUN / d A13
	DFUN(cut:s-1,4) =  FC3*BEXP*BEXPT23		; d FUN / d A13
	DFUN(0:cut-1,5) =  FC3*BEXP*BEXPT23		; d FUN / d A23
	DFUN(cut:s-1,5) = -FS3*BEXP*BEXPT23		; d FUN / d A23

	DFUN(0:cut-1,6) = FUN(0:cut-1)*(-T2T2)*2.*B	; d FUN / d B
	DFUN(cut:s-1,6) = FUN(cut:s-1)*(-T2T2)*2.*B	; d FUN / d B

	DFUN(0:cut-1,7) = ( ( A11*FC1-A21*FS1)*BEXPT21 + ( A12*FC2-A22*FS2)*BEXPT22 + ( A13*FC3-A23*FS3)*BEXPT23 )*T2*BEXP
	DFUN(cut:s-1,7) = ( (-A11*FS1-A21*FC1)*BEXPT21 + (-A12*FS2-A22*FC2)*BEXPT22 + (-A13*FS3-A23*FC3)*BEXPT23 )*T2*BEXP
endif

RETURN
END
;**************************************************************************************

function fitexp, f_nr, t_in, s_in, s0, sigmas, sigmaa, chisqr
t=REFORM(t_in)					; remove dim 1 from array's
s=REFORM(s_in)					; 

npoints = n_elements(t)				; set-up and check dimensions

if npoints ne n_elements(s) then begin		; test match b and s numbers
	print, ' '
	print, ' trifit  - different number of elements in t and data vectors, '
	print, '           execution terminated, returning zero values '
	print, ' '
	return, [0.,0.]
end

a = s0						; store params in a
type = size(a)
type = type(type(0)+1)

funct, f_nr, t, a, yfit

								; call curvefit to refine parameters
;if( n_params(0) gt 4) then w=1/sigmas^2 			; use user-supplied standard deviations

starting = a							;save starting values
nn=n_elements(sigmas)
IF nn EQ 1 THEN w= replicate(1/sigmas^2,npoints) ELSE $
IF nn EQ npoints THEN w=1/sigmas^2 ELSE $
if type eq 5 then w = replicate(double(1.),npoints) else w = replicate(1.,npoints)	;create weight vector

yfit = curvefit(f_nr,t, s, w, a, sigmaa,chisqr)			;fit


return, a
end


