;This subroutine finds the peak locations by identifying the largest
;value in the magnitude spectrum as the position of the water peak.

;Revision History
;980915 ASB Removed code from analyze_roi.pro.
;980915 ASB Replaced !csi_dat with fd_dat in COMMON freqdomain.
;980916 ASB Changed name of COMMON freqdomain to mrsidata.
;981008 ASB Added code to sensitise widgets.
;990407 ASB Added code to roundoff pixel number.
;           Chemical shifts now stored in COMMON mets.
;991222 ASB Added test for null mask.

pro waterpick

COMMON defroi
COMMON mrsidata
COMMON posdata
COMMON initcsi
COMMON mets

afft=dblarr(pe_dim(0),pe_dim(1))

FOR i=0,31 DO FOR j=0,31 DO BEGIN
    afft(i,j)=max(abs(fd_dat(*,i,j))) ;height of water peaks
ENDFOR

immax=max(afft)
;Make mask of pixels to fit.  Include pixels with water peak heights
;within a factor of 50 of largest pixel.
w=filter_mask(afft,immax/50.)   

pt_per_hz = spc_len/sweep_width
positions=intarr(sp_dim(0),sp_dim(1))
naa_offset = (h2o_shift-naa_shift)*spec_freq*pt_per_hz 

FOR kk=0,sp_dim(0)-1 DO BEGIN
    FOR jj=0,sp_dim(1)-1 DO BEGIN
        foo=max(abs(fd_dat(*,kk,jj)),loc)
        positions(kk,jj)=round(loc+naa_offset)
    ENDFOR
ENDFOR

;Apply mask
mask=where(w EQ 0)
IF mask(0) NE -1 THEN positions(mask) = -1

widget_control,s.save_ids(1),sensitive=1 ;Save Positions
widget_control,s.save_ids(4),sensitive=1 ;Save Positions
widget_control,s.restore_ids(2),sensitive=1 ;Save Positions
widget_control,s.restore_ids(3),sensitive=1 ;Save Positions

RETURN
END
