/***********************************************************************
  NEW EPI Sequence
  X. Hu, and N. Tsekos, U. of Minnesota
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

#include "standard.h"
#include "group.h"
#include <string.h>

#define RISETIME 0.0005
#define MINRISETIME 0.00008
#define usecfact 1.0e+6

extern double rint();

pulsesequence()
{
  /* * INTERNAL VARIABLE DECLARATIONS ******************** */
  int     arraydim, i, cntr, j, groticks;

  double  etl, internaltr;
  double  tau1, tau2, readsign;
  double  agro, agss, agcrush, grate, gssrint, groint, grorint, gpeint;
  double  nucgamma, atf, frewind[MAXSLICE];
  double  dseg, minte, mintr, eff_echo, grof2, gpef;
  double  dwell[8192], summdwell;
  int     phasestep[512], phasestep1[512];
  char    acq3d[MAXSTR], contrast[MAXSTR];
  double  skip, alfa, dtpwr1, gsst;
  double  gssrt, grort, gpet, input_tpe, blipgpe, grot;
  double  totalgrot;
  double  risetime, riseslope, dwell0, dwellBase;
  double  acqnpRamp, acqnpPlateau;
  double  startacqdelay;
  double  gssign, grosign;
  double  gssrshapef;
  double  gradelement, dwell_adj, usergmax;
  double  ramptime, plateautime;
  double  gperephase, npes, errorcode;
  char    readtype[MAXSTR];
  char    phasetype[MAXSTR];
  char    navecho[MAXSTR];
  char    phaseref[MAXSTR];
  char    hlfourier[MAXSTR];
  int     orient_error;
  int     etlint;
  char    read, phase, slice;
  int     gronpRamp, gronpPlateau, blipnp, gronpLobe, gronpTotal;
  int     nseg, npesegm, negphases, readmult;
  Gpattern readshape[8192], phaseshape[8192], phaseshape2[8192],
          readshape1[8192];
  char    gropat1[MAXSTR], gpepat1[MAXSTR], gpepat2[MAXSTR], gpepat3[MAXSTR],
          gropat2[MAXSTR];
  char    fullinfo[MAXSTR];
  char    singlegrad[MAXSTR];
  double  truenp, conv, gpemul, readfactor, gsspre, gpe_adj, psspre, slctopre;
  double  time1, time2, deltate;

  initparms_sis();
  nucgamma = 1e6 * sfrq / B0;

  arraydim = getval("arraydim");
  atf = getval("atf");
  truenp = getval("readres");
  dwell_adj = getval("dwell_adj");
  truenp = 2. * truenp;
  dseg = getval("dseg");
  grof2 = getval("grof2");
  gpef = getval("gpef");
  cntr = getval("cntr");
  gsspre = getval("gsspre");
  psspre = getval("psspre");
  gpe = getval("gpe");
  skip = getval("ref_pos") + 1;
  alfa = getval("alfa");
  tpwr1 = getval("tpwr1");
  dtpwr1 = getval("dtpwr1");
  input_tpe = getval("tpe1");
  risetime = getval("risetime");
  groticks = getval("groticks");
  gradelement = getval("gradelement");
  usergmax = getval("usergmax");
  negphases = getval("negphases");
  nseg = getval("nseg");
  npesegm = rint(nv / nseg);
  npes = getval("phaseres");

  agro = fabs(gro);
  agss = fabs(gss);
  if (gss > 0)
    gssign = 1.0;
  else if (gss == 0.0)
    gssign = 0.;
  else
    gssign = -1.0;
  if (gro > 0)
    grosign = 1.0;
  else if (gro == 0.0)
    grosign = 0.;
  else
    grosign = -1.0;
  agcrush = fabs(gcrush);
  getstr("contrast", contrast);
  getstr("readtype", readtype);
  getstr("phasetype", phasetype);
  getstr("navecho", navecho);
  getstr("phaseref", phaseref);
  getstr("hlfourier", hlfourier);
  getstr("fullinfo", fullinfo);
  getstr("singlegrad", singlegrad);
  if (singlegrad[0] == 'n')
    dwell_adj = 1.0;
  getstr("acq3d", acq3d);

  if (risetime == 0.0 || risetime < MINRISETIME)
    risetime = trise;
  conv = gradstepsz / gmax;
  riseslope = usergmax / risetime;
  grate = 1.0 / riseslope;
  errorcode = 0.0;

  /* * get orientation */
  orient_error = getorientation(&read, &phase, &slice, "orient");

  if (orient_error < 0) {
    printf("Orientation Error \n");
    abort(1);
  }
  /* * set freq for inversion pulse */
  slctopre = resto + 4258. * psspre * gsspre;
  etl = (double) nv / nseg;
  etlint = (int) (etl + 0.5);

  /*
   * if alternating, first echo will be TE oterhwise linear
   * phase encoding steps, middle of echo trains
   */

  if (!strcmp(phasetype, "lin"))
    eff_echo = (double) etl / 2.0;
  else
    eff_echo = 1.0;

  /* Phase Encoding Gradient  Parameter Calculation ***/

  if (gpef > 0.0) {
    gpeint = B0 / (lpe * sfrq * 1.e6);
    gpe2 = B0 / (lpe2 * sfrq * 1e6) / gssrt * 2.0;
    gpet = sqrt(grate * gpeint * nseg);
    if (gpet / grate > gmax) {
      if (!strcmp(phasetype, "lin"))
        gpet = gpeint * nseg / gmax / 2.;
      else
        gpet = gpeint * nseg / gmax;
    }
    /* * increase gpet for safeguard */
    gpet = gpet * 1.2;

    /* * single blip gradient strength */
    blipgpe = gpeint / gpet;

  } else {
    gpe2 = 0.0;
    blipgpe = 0.0;
    gpet = 0.0;
  }

  /* * SLICE SELECTION GRADIENT */
  gsst = grate * agss;
  gssrshapef = 0.5;
  gssrint = agss * (p1 / 2.0 + rof2 + gsst / 2.0);
  gssr = 2.0 * agss;
  gssrt = 2.0 * gssrint / gssr;
  if (acq3d[0] == 'y') {
    gssrt = 2 * gssrt;
    gssr = 0.5 * gssr;
  }

  /* * establishing readout gradient shapes */
  if (agro == 0.0) {
    acqnpRamp = 0;
    acqnpPlateau = np;
    plateautime = 0.0;
    startacqdelay = 0.0;
    grort = 0.0;
    gror = 0.0;
    grot = 0.0;
    ramptime = 0.0;
    gronpRamp = 0;
    gronpPlateau = 0;
    gronpTotal = 0;
    readmult = 0.0;
  } else {
    if (navecho[0] == 'y' && !strcmp(phasetype, "lin")) {
      readsign = -1.;
    } else {
      readsign = 1.;
    }

    /* * READ-OUT GRADIENT */
    acqnpPlateau = 0.0;
    acqnpPlateau = 0.0;
    plateautime = 0.0;
    ramptime = 0.0;
    dwell0 = 1.0 / (nucgamma * agro * lro);
    groint = truenp / 2. / lro / nucgamma;

    if (singlegrad[0] == 'y') {
      readmult = (int) rint(etl);
      if (navecho[0] == 'y')
        readmult = readmult - 1;
    } else
      readmult = 1;

    /* * determine read gradient shapes */
    switch (gropat[2]) {
     case 'a':                    /* trapzoidal */
      gronpRamp = rint(grate * agro / gradelement);
      gronpPlateau = rint(truenp / 2.0 * dwell0 / gradelement);

      /* * ramp sampling */
      if (readtype[1] == 'a') {       /* ramp sampling */
        gronpPlateau -= gronpRamp;
        if (gronpPlateau < 0) {
          printf("EPI SEQUENCE ERROR: Riserate too slow for trapezoidal read
gradient with readtype=na\n");
          printf("                    Reduce Gro or change gropat to sine or
trian\n");
          errorcode = 1.0;
        }
      }
      plateautime = gronpPlateau * gradelement;
      ramptime = grate * agro;
      grorint = agro * (plateautime / 2.0 + grate * agro / 2.0);
      gronpLobe = 2 * gronpRamp + 1;    /* XH new */
      grot = 2. * grate * agro + plateautime;

      /* set phase-encoding duration calculate the scale factor for gpe */
      blipnp = ceil((gronpRamp * 2. + gronpPlateau * 1.) * gpet / grot);
      gpe_adj = gpet / grot * (gronpRamp * 2. + gronpPlateau * 1.) /
(double) (blipnp);
      gronpTotal = readmult * gronpLobe;
      /* * concatenate waveforms */
      if (ix == 1) {
        for (j = 0; j < readmult; j++) {
          if (j % 2 == 0) {
            if (grof2 < 1.)
              readfactor = 1;
            else
              readfactor = 1. / grof2;
          } else {
            if (grof2 < 1.)
              readfactor = -grof2;
            else
              readfactor = -1.;
          }
          if (navecho[0] == 'y')
            readfactor *= -1;
          readshape[j * gronpLobe + gronpRamp].amp = readfactor * 32767.0;
          readshape[j * gronpLobe + gronpRamp].time = gronpPlateau;
          for (i = 0; i < gronpRamp; i++)
          {
            readshape[j * gronpLobe + i].amp = readfactor * 32767.0 *
(double) (i + 1) / (gronpRamp);
            readshape[j * gronpLobe + i + gronpRamp + 1].amp = readfactor *
32767.0 * (double) (gronpRamp - i - 1) / (double) gronpRamp;
            readshape[j * gronpLobe + i + gronpRamp + 1].time = 1.0;
            readshape[j * gronpLobe + i].time = 1.0;
          }
          if (singlegrad[0] == 'y') {
            phaseshape[j * (1 + 2 * blipnp)].amp = 0.0;
            phaseshape[j * (1 + 2 * blipnp)].time = gronpPlateau + 2. *
gronpRamp - 2. * blipnp;
            phaseshape2[j * (1 + 2 * blipnp)].amp = 0.0;
            phaseshape2[j * (1 + 2 * blipnp)].time = gronpPlateau + 2. *
gronpRamp - 2. * blipnp;
            for (i = 0; i < blipnp; i++)
            {
              phaseshape[j * (1 + 2 * blipnp) + i + 1].amp = 32767.0 * (i +
1.) / (double) blipnp;
              phaseshape[j * (1 + 2 * blipnp) + i + 1].time = 1.0;
              phaseshape[j * (1 + 2 * blipnp) + i + 1 + blipnp].amp = 32767.
* (blipnp - i - 1.0) / (double) blipnp;
              phaseshape[j * (1 + 2 * blipnp) + i + 1 + blipnp].time = 1.0;
              phaseshape2[j * (1 + 2 * blipnp) + i + 1].amp = 32767.0 * (i +
1.) / (double) blipnp;
              phaseshape2[j * (1 + 2 * blipnp) + i + 1].time = 1.0;
              phaseshape2[j * (1 + 2 * blipnp) + i + 1 + blipnp].amp =
32767. * (blipnp - i - 1.0) / (double) blipnp;
              phaseshape2[j * (1 + 2 * blipnp) + i + 1 + blipnp].time = 1.0;
              if (j == 0)
              {
                phaseshape2[j * (1 + 2 * blipnp) + i + 1].amp = 0.0;
                phaseshape2[j * (1 + 2 * blipnp) + i + 1 + blipnp].amp =
0.0;
              }
            }
          }
        }
      }
      if (singlegrad[0] == 'y') {
        sprintf(gropat1, "%s%s%s", "read_", gropat, "_single");
        sprintf(gpepat1, "%s%s", "phase_", "single");
        sprintf(gpepat3, "%s", "phase_junk");

        if (ix == 1) {
          /* * shift the concatenated shape so that the blips sit
           * between readout gradient lobes */
          if (phaseshape[0].amp == 0.0)
            phaseshape[0].time += blipnp;
          else {
            printf("Error in GPE shape: the first point should be zero!\n");
            abort();
          }
          /* * zero the last blip which is not needed */
          for (i = (1 + blipnp * 2) * (readmult - 1) + 1; i < (1 + blipnp *
2) * readmult; i++)
            phaseshape[i].amp = 0.0;

          init_gradpattern(gropat1, readshape, gronpTotal);
          init_gradpattern(gpepat1, phaseshape, (blipnp * 2 + 1) * readmult
- blipnp);
          init_gradpattern(gpepat3, phaseshape2, (blipnp * 2 + 1) * readmult
- blipnp);
        }
      } else {
        sprintf(gropat1, "%s%s", "read_", gropat);
        if (ix == 1)
          init_gradpattern(gropat1, readshape, gronpTotal);
      }

      if (ix == 1) {
        readshape1[gronpRamp].amp = 32767.0;
        readshape1[gronpRamp].time = gronpPlateau;
        for (i = 0; i < gronpRamp; i++) {
          readshape1[i].amp = 32767.0 * (double) (i + 1) / (gronpRamp);
          readshape1[i + gronpRamp + 1].amp = 32767.0 * (double) (gronpRamp
- i - 1) / (double) gronpRamp;
          readshape1[i + gronpRamp + 1].time = 1.0;
          readshape1[i].time = 1.0;
        }

        phaseshape[0].amp = 0.0;
        phaseshape[0].time = gronpPlateau + 2. * gronpRamp - 2 * blipnp;
        for (i = 0; i < blipnp; i++) {
          phaseshape[i + 1].amp = 32767.0 * (i + 1.) / (double) blipnp;
          phaseshape[i + 1].time = 1.0;
          phaseshape[i + 1 + blipnp].amp = 32767. * (blipnp - i - 1.0) /
(double) blipnp;
          phaseshape[i + 1 + blipnp].time = 1.0;
        }
      }
      sprintf(gpepat2, "%s", "gpe_single");
      if (ix == 1)
        init_gradpattern(gpepat2, phaseshape, 2 * blipnp + 1);

      sprintf(gropat2, "%s%s", "read_", "current");
      if (ix == 1)
        init_gradpattern(gropat2, readshape1, gronpLobe);
      gror = agro / 2.;
      grort = (grorint / gror) + (grate * agro);
      break;

     case 'i':                    /* triangular */
      if (readtype[1] == 'f') {
        printf("EPI SEQUENCE ERROR: No flat portion for readout, change your
readtype or gropat\n");
        errorcode = 1.0;
      }
      grot = groint / agro * 2;
      ramptime = grot / 2.0;
      gronpRamp = rint(grot / 2. / gradelement);
      gronpPlateau = 0;
      grorint = groint / 2.;
      gronpLobe = 2 * gronpRamp;  /* XH new */

      /* * set phase-encoding duration calculate the scale factor
       * for using gro strength to set gpe */
      blipnp = ceil((gronpRamp * 2.) * gpet / grot);
      gpe_adj = (double) gronpLobe / grot * gpet / (double) blipnp;

      gronpTotal = readmult * gronpLobe;
      if (ix == 1) {
        for (j = 0; j <= readmult; j++) {
          if (j % 2 == 0) {
            if (grof2 < 1.)
              readfactor = 1;
            else
              readfactor = 1. / grof2;
          } else {
            if (grof2 < 1.)
              readfactor = -grof2;
            else
              readfactor = -1.;
          }
          if (navecho[0] == 'y')
            readfactor *= -1;
          for (i = 0; i < gronpRamp; i++) {
            readshape[j * gronpLobe + i].amp = readfactor * 32767.0 *
(double) (i + 1) / (gronpRamp);
            readshape[j * gronpLobe + i + gronpRamp].amp = readfactor *
32767.0 * (double) (gronpRamp - i - 1) / (double) gronpRamp;
            readshape[j * gronpLobe + i + gronpRamp].time = 1.0;
            readshape[j * gronpLobe + i].time = 1.0;
          }
          if (singlegrad[0] == 'y') {
            phaseshape[j * (1 + 2 * blipnp)].amp = 0.0;
            phaseshape[j * (1 + 2 * blipnp)].time = 2. * gronpRamp - 2. *
blipnp;
            phaseshape2[j * (1 + 2 * blipnp)].amp = 0.0;
            phaseshape2[j * (1 + 2 * blipnp)].time = 2. * gronpRamp - 2. *
blipnp;
            for (i = 0; i < blipnp; i++) {
              phaseshape[j * (1 + 2 * blipnp) + i + 1].amp = 32767.0 * (i +
1.) / (double) blipnp;
              phaseshape[j * (1 + 2 * blipnp) + i + 1].time = 1.0;
              phaseshape[j * (1 + 2 * blipnp) + i + 1 + blipnp].amp = 32767.
* (blipnp - i - 1.0) / (double) blipnp;
              phaseshape[j * (1 + 2 * blipnp) + i + 1 + blipnp].time = 1.0;
              phaseshape2[j * (1 + 2 * blipnp) + i + 1].amp = 32767.0 * (i +
1.) / (double) blipnp;
              phaseshape2[j * (1 + 2 * blipnp) + i + 1].time = 1.0;
              phaseshape2[j * (1 + 2 * blipnp) + i + 1 + blipnp].amp =
32767. * (blipnp - i - 1.0) / (double) blipnp;
              phaseshape2[j * (1 + 2 * blipnp) + i + 1 + blipnp].time = 1.0;
              if (j == 0) {
                phaseshape2[j * (1 + 2 * blipnp) + i + 1].amp = 0.0;
                phaseshape2[j * (1 + 2 * blipnp) + i + 1 + blipnp].amp =
0.0;
              }
            }
          }
        }
      }
      if (singlegrad[0] == 'y') {
        sprintf(gropat1, "%s%s%s", "read_", gropat, "_single");
        sprintf(gpepat1, "%s%s", "phase_", "single");
        sprintf(gpepat3, "%s", "phase_junk");

        /*
         * shift the concatenated shape so that the blips sit
         * between readout gradient lobes
         */
        if (ix == 1) {
          if (phaseshape[0].amp == 0.0)
            phaseshape[0].time += blipnp;
          else {
            printf("Error in GPE shape: the first point should be zero!\n");
            abort();
          }
          /* * zero the last blip which is not needed */
          for (i = (1 + blipnp * 2) * (readmult - 1) + 1; i < (1 + blipnp *
2) * readmult; i++)
            phaseshape[i].amp = 0.0;

          init_gradpattern(gropat1, readshape, gronpTotal);
          init_gradpattern(gpepat1, phaseshape, (blipnp * 2 + 1) * readmult
- blipnp);
          init_gradpattern(gpepat3, phaseshape2, (blipnp * 2 + 1) * readmult
- blipnp);
        }
      } else {
        sprintf(gropat1, "%s%s", "read_", gropat);
        if (ix == 1)
          init_gradpattern(gropat1, readshape, gronpTotal);
      }

      /* * the following line to be removed after john "do the job" */

      if (ix == 1) {
        phaseshape[0].amp = 0.0;
        phaseshape[0].time = 2. * gronpRamp - 2 * blipnp;
        for (i = 0; i < blipnp; i++)
        {
          phaseshape[i + 1].amp = 32767.0 * (i + 1.) / (double) blipnp;
          phaseshape[i + 1].time = 1.0;
          phaseshape[i + 1 + blipnp].amp = 32767. * (blipnp - i - 1.0) /
(double) blipnp;
          phaseshape[i + 1 + blipnp].time = 1.0;
        }
      }
      sprintf(gpepat2, "%s", "gpe_single");
      if (ix == 1) {
        init_gradpattern(gpepat2, phaseshape, 2 * blipnp + 1);

        for (i = 0; i < gronpRamp; i++)
        {
          readshape1[i].amp = 32767.0 * (double) (i + 1) / (gronpRamp);
          readshape1[i + gronpRamp].amp = 32767.0 * (double) (gronpRamp - i
- 1) / (double) gronpRamp;
          readshape1[i + gronpRamp].time = 1.0;
          readshape1[i].time = 1.0;
        }
      }
      sprintf(gropat2, "%s%s", "read_", "current");
      gror = agro / 2.0;
      grort = grot;
      if (ix == 1) {
        init_gradpattern(gropat2, readshape1, gronpLobe);
        printf("gror=%f, grort=%f, gropat2=%s\n", gror, grort, gropat2);
      }
      break;

     case 'n':                    /* sinusoidal */
      if (readtype[1] == 'f') {
        printf("No flat portion for readout, change your readtype or
gropat\n");
        exit();
      }
      plateautime = 0.0;
      grot = groint / agro * PI / 2;
      ramptime = grot / 2.0;
      grorint = groint / 2.;
      gronpRamp = rint(grot / gradelement);
      gronpPlateau = 0;
      gronpLobe = gronpRamp;

      /* * set phase-encoding duration calculate the scale factor
       * for using gro strength to set gpe */
      blipnp = ceil((gronpRamp) * gpet / grot);
      gpe_adj = (double) gronpRamp *gpet / grot / (double) blipnp;

      gronpTotal = readmult * gronpLobe;
      if (ix == 1) {
        for (j = 0; j <= readmult; j++) {
          if (j % 2 == 0) {
            if (grof2 < 1.)
              readfactor = 1;
            else
              readfactor = 1. / grof2;
          } else {
            if (grof2 < 1.)
              readfactor = -grof2;
            else
              readfactor = -1.;
          }
          if (navecho[0] == 'y')
            readfactor *= -1;
          for (i = 0; i < gronpRamp; i++) {
            readshape[j * gronpLobe + i].amp = readfactor * 32767.0 * sin(PI
* (double) (i + 1) / (gronpRamp));
            readshape[j * gronpLobe + i].time = 1.0;
          }
          if (singlegrad[0] == 'y') {
            phaseshape[j * (1 + 2 * blipnp)].amp = 0.0;
            phaseshape[j * (1 + 2 * blipnp)].time = gronpRamp - 2. * blipnp;
            phaseshape2[j * (1 + 2 * blipnp)].amp = 0.0;
            phaseshape2[j * (1 + 2 * blipnp)].time = gronpRamp - 2. *
blipnp;
            for (i = 0; i < blipnp; i++) {
              phaseshape[j * (1 + 2 * blipnp) + i + 1].amp = 32767.0 * (i +
1.) / (double) blipnp;
              phaseshape[j * (1 + 2 * blipnp) + i + 1].time = 1.0;
              phaseshape[j * (1 + 2 * blipnp) + i + 1 + blipnp].amp = 32767.
* (blipnp - i - 1.0) / (double) blipnp;
              phaseshape[j * (1 + 2 * blipnp) + i + 1 + blipnp].time = 1.0;
              phaseshape2[j * (1 + 2 * blipnp) + i + 1].amp = 32767.0 * (i +
1.) / (double) blipnp;
              phaseshape2[j * (1 + 2 * blipnp) + i + 1].time = 1.0;
              phaseshape2[j * (1 + 2 * blipnp) + i + 1 + blipnp].amp =
32767. * (blipnp - i - 1.0) / (double) blipnp;
              phaseshape2[j * (1 + 2 * blipnp) + i + 1 + blipnp].time = 1.0;
              if (j == 0) {
                phaseshape2[j * (1 + 2 * blipnp) + i + 1].amp = 0.0;
                phaseshape2[j * (1 + 2 * blipnp) + i + 1 + blipnp].amp =
0.0;
              }
            }
          }
        }
      }
      if (singlegrad[0] == 'y') {
        sprintf(gropat1, "%s%s%s", "read_", gropat, "_single");
        sprintf(gpepat1, "%s%s", "phase_", "single");
        sprintf(gpepat3, "%s", "phase_junk");

        /* * shift the concatenated shape so that the blips sit
         * between readout gradient lobes */
        if (ix == 1) {
          if (phaseshape[0].amp == 0.0)
            phaseshape[0].time += blipnp;
          else {
            printf("Error in GPE shape: the first point should be zero!\n");
            abort();
          }
          /* * zero the last blip which is not needed */
          for (i = (1 + blipnp * 2) * (readmult - 1) + 1; i < (1 + blipnp *
2) * readmult; i++)
            phaseshape[i].amp = 0.0;

          init_gradpattern(gropat1, readshape, gronpTotal);
          init_gradpattern(gpepat1, phaseshape, (blipnp * 2 + 1) * readmult
- blipnp);
          init_gradpattern(gpepat3, phaseshape2, (blipnp * 2 + 1) * readmult
- blipnp);
        }
      } else {
        sprintf(gropat1, "%s%s", "read_", gropat);
        if (ix == 1)
          init_gradpattern(gropat1, readshape, gronpTotal);
      }

      if (ix == 1) {
        phaseshape[0].amp = 0.0;
        phaseshape[0].time = gronpRamp - 2 * blipnp;
        for (i = 0; i < blipnp; i++) {
          phaseshape[i + 1].amp = 32767.0 * (i + 1.) / (double) blipnp;
          phaseshape[i + 1].time = 1.0;
          phaseshape[i + 1 + blipnp].amp = 32767. * (blipnp - i - 1.0) /
(double) blipnp;
          phaseshape[i + 1 + blipnp].time = 1.0;
        }
      }
      sprintf(gpepat2, "%s", "gpe_single");
      if (ix == 1) {
        init_gradpattern(gpepat2, phaseshape, 2 * blipnp + 1);

        for (i = 0; i < gronpRamp; i++) {
          readshape1[i].amp = 32767.0 * sin(PI * (double) (i + 1) /
(gronpRamp));
          readshape1[i].time = 1.0;
        }
      }
      sprintf(gropat2, "%s%s", "read_", "current");
      gror = agro / 2.0;
      grort = grot;
      if (ix == 1) {
        init_gradpattern(gropat2, readshape1, gronpLobe);
        printf("gror=%f, grort=%f, gropat2=%s\n", gror, grort, gropat2);
      }
      break;

     default:
      printf("Unrecognizable Read-Out Shape (gropat): %s\n", gropat);
      exit(1);
    }

    /*
     * this is needed only for John's program for now, should be
     * gone in the future if (readtype[0] == 'n' && readtype[1]
     * == 'f') interpolation[0] = 'n'; else interpolation[0] =
     * 'y';
     */
    if (readtype[1] == 'f')
      startacqdelay = grate * agro;
    else
      startacqdelay = 0.0;

    /* * nonlinear acquisition table */
    dwell0 = 1.0 / (nucgamma * agro * lro);

    if (readtype[0] == 'n' && readtype[1] != 'f') {
      switch (gropat[2]) {
       case 'a':                  /* trapzoidal */
        dwellBase = sqrt(4. * groint * grate / truenp);
        acqnpRamp = rint(grate * agro / (2.0 * dwell0));
        acqnpPlateau = (np / 2.0) - 2.0 * acqnpRamp;
        plateautime = acqnpPlateau * dwell0;
        for (i = 0; i < acqnpRamp; i++) {
          dwell[i] = dwellBase * (sqrt(i + 1.0) - sqrt((double) i));
        }
        for (i = 0; i < acqnpRamp; i++) {
          dwell[(int) rint(acqnpRamp + acqnpPlateau + i)] = (sqrt(acqnpRamp
- i) - sqrt(acqnpRamp - i - 1)) * dwellBase;
        }
        for (i = acqnpRamp; i < acqnpRamp + acqnpPlateau; i++) {
          dwell[i] = dwell0;
        }
        break;

       case 'i':                  /* triangular */
        plateautime = 0.0;
        dwellBase = sqrt(4. * groint / truenp / (2. * agro / grot));
        acqnpRamp = np / 4.0;
        acqnpPlateau = 0.0;
        for (i = 0; i < acqnpRamp; i++) {
          dwell[i] = dwellBase * (sqrt(i + 1.0) - sqrt((double) i));
          dwell[(int) rint(2 * acqnpRamp + acqnpPlateau - 1 - i)] =
dwell[i];
        }

        break;
       case 'n':                  /* sinusoidal */
        plateautime = 0.0;
        grorint = agro * grot * 2.0 / PI;
        dwell[0] = (at / PI) * acos(1 - PI * dwell0);
        acqnpRamp = np / 2.0;
        acqnpPlateau = 0.0;

        dwell[0] = grot / PI * acos(1.0 - 4. / truenp);
        for (i = 1; i < acqnpRamp; i++) {
          dwell[i] = grot * acos(cos(dwell[i - 1] / grot * PI) - 4. /
truenp) / PI;
        }
        for (i = acqnpRamp - 1; i > 0; i--) {
          dwell[i] -= dwell[i - 1];
        }
        dwell[(int) acqnpRamp - 1] = dwell[0];
        break;

       default:
        printf("Unrecognizable Read-Out Shape (gropat): %s\n", gropat);
        exit();

      }
    } else {
      acqnpRamp = 0;
      acqnpPlateau = np;
    }
  }

  /* * check options consistency */
  if ((hlfourier[0] == 'y') && (!strcmp(phasetype, "alt"))) {
    printf("Half Fourier cannot be used to alternate phase-encoding order
\n");
    exit();
  }
  /* * Phase encoding Table Generation */
  /* * imaging data phase-encoding table */
  gpemul = 1.0 * nseg;
  if (nseg > 1.0) {
    if (!strcmp(phasetype, "alt")) {
      gpemul = nseg / 2.;
      for (i = 0; i < etlint; i++) {
        for (j = 0; j < nseg; j++) {
          phasestep[i + etlint * j] = (j < nseg / 2) ? -nseg / 2 : nseg / 2;
        }
      }
      if (navecho[0] == 'y') {
        for (j = 0; j < nseg; j++)
          phasestep[etlint * j] = j - nseg / 2.0;
      } else {
        for (j = 0; j < nseg; j++)
          phasestep1[j] = j - nseg / 2.0;
        settable(t4, (int) nseg, phasestep1);
      }
    } else {
      for (i = 0; i < etlint; i++) {
        for (j = 0; j < nseg; j++) {
          phasestep[i + etlint * j] = nseg;
        }
      }

      if (navecho[0] == 'y') {
        for (j = 0; j < nseg; j++)
          phasestep[etlint * j] = j - nseg / 2.0;
      } else {
        for (j = 0; j < nseg; j++)
          phasestep1[j] = j - nseg / 2.0;
        settable(t4, (int) nseg, phasestep1);
      }
    }
  } else {
    for (i = 0; i < nv; i++) {
      phasestep[i] = 1;
    }
  }
  settable(t3, (int) nv, phasestep);
  /* * reference phase-encoding table */
  for (i = 0; i < nv; i++) {
    phasestep[i] = 0;
  }
  settable(t2, (int) nv, phasestep);
  if (nseg > 1.0) {
    for (i = 0; i < nseg; i++) {
      phasestep1[i] = 0;
    }
    settable(t5, (int) nseg, phasestep1);
  }
  /* * phase-encoded reference scan if desired */
  if (nseg > 1.0) {
    if (!strcmp(phasetype, "alt")) {
      for (i = 0; i < etlint; i++) {
        for (j = 0; j < nseg; j++) {
          phasestep[i + etlint * j] = (j < nseg / 2) ? -nseg / 2 : nseg / 2;
        }
      }
      for (j = 0; j < nseg; j++)
        phasestep[etlint * j] = 0;
      if (navecho[0] == 'y') {
        for (j = 0; j < nseg; j++)
          phasestep[etlint * j + 1] = rint(j - nseg / 2);
      } else {
        for (j = 0; j < nseg; j++)
          phasestep1[j] = rint(j - nseg / 2);
        settable(t6, (int) nseg, phasestep1);
      }
    } else {
      for (i = 0; i < etlint; i++) {
        for (j = 0; j < nseg; j++) {
          phasestep[i + etlint * j] = rint(nseg / 2);
        }
      }

      for (j = 0; j < nseg; j++)
        phasestep[etlint * j] = 0.0;
      if (navecho[0] == 'y') {
        for (j = 0; j < nseg; j++)
          phasestep[etlint * j + 1] = j;
      } else {
        for (j = 0; j < nseg; j++)
          phasestep1[1] = j;
        settable(t6, (int) nseg, phasestep1);
      }
    }
  } else {
    for (i = 0; i < nv; i++) {
      phasestep[i] = 1;
    }
    phasestep[0] = 0;
  }
  settable(t7, (int) nv, phasestep);
  /* * more timing check */
  if (gssrt > grort) {
    time1 = gssrt;
    time2 = gssrt - grort;
  } else {
    time1 = grort;
    time2 = 0.0;
  }

  /* * TAU1 - MINIMUM TIME BETWEEN ALPHA AND FIRST ECHO */
  tau1 = 0.5 * p1 + rof2 + agss * grate + time1 + 0.5 * grot;
  tau2 = grot;
  minte = tau1 + (eff_echo - 1) * tau2;
  if (te < minte) {
    if (ix == 1)
      printf("TE set to %f5.1 msec\n", minte * 1000.);
    deltate = 0.0;
  } else
    deltate = te - minte;
  if (te < minte) {
    deltate = 0.0;
    te = minte;
  } else {
    deltate = te - minte;
  }
  mintr = grate * agss + rof1 + p1 / 2.0 + deltate +
          (etl - eff_echo) * tau2 + at / 2.0 + grate * agro +
          tcrush + grate * agcrush;
  if (tr < mintr) {
    if (ix == 1)
      printf("TR set to %8.1f msec\n", 1000 * mintr);
    tr = mintr;
  }
  internaltr = tr - mintr;
  totalgrot = ((double) readmult) * grot;

  /* * CALCULATE PHASE REFOCUSSING TO CORRECT FOR FREQUENCY JUMPS */
  for (i = 0; i < ns; i++) {
    frewind[i] = (atf + at + OFFSET_DELAY) * nucgamma * (gro * pro - gss *
pss[i]);
    frewind[i] = -remainder(frewind[i], 1.0);
    frewind[i] = resto + nucgamma * gss * pss[i]
            + frewind[i] / (gpet + OFFSET_DELAY);
  }

  /* * print info for the sequence */
  if ((ix == 1) && (fullinfo[0] == 'y')) {
    if (agro > 0.0) {
      printf("EPI SEQUENCE INFORMATION:\n");
      printf("Risetime = %.1f (usec) Riseslope = %.1f (G/cm/ms)\n", usecfact
* risetime, usergmax / (1000. * risetime));
      printf("Number of PE steps per seg = %.0f, Segments = %d, Type
ofAcq:%s\n", etl, nseg, phasetype);
      printf(" Minimum TE = %.2f (msec), TR = %.2f (msec), Segment = %.2f
(msec)\n", minte * (1e+3), mintr * (1e+3), etl * mintr * (1e+3));
      printf(" Applied TE = %.2f (msec), TR = %.2f (msec), Segment = %.2f
(msec)\n", (minte + deltate) * (1e+3), (mintr + internaltr) * (1e+3), etl *
(mintr + internaltr) * (1e+3));
      printf("Slice Selection Gradient: \n");
      printf(" gss = %.3f (G/cm), Ramps = %.1f (usec), Total = %.1f (usec)
\n", agss, usecfact * gsst, usecfact * (2 * gsst + rof1 + p1 + rof2));
      printf(" gssr = %.3f (G/cm), Total Duration = %.1f (usec) \n", gssr,
usecfact * gssrt);

      if (singlegrad[0] == 'y')
        printf("Single Read-out Gradient Shape: %s Grad Repetitions: %d\n",
gropat1, readmult);
      else
        printf("Read-out Gradient Shape: %s (in shapelib)\n", gropat1);
      printf(" grot = %.1f [%.1f] (usec), gror = %.3f (G/cm), grort = %.1f
(usec)\n", usecfact * totalgrot, usecfact * grot, gror, usecfact * grort);
      printf(" Duration: Ramp = %.1f (usec), Plateau =%.1f\n", usecfact *
ramptime, usecfact * plateautime);
      printf(" Gradient Points: Ramp = %d, Plateau = %d, Total = %d \n",
gronpRamp, gronpPlateau, gronpTotal);
      printf(" Acquisition Points: Ramp = %.1f, Plateau = %.1f \n",
acqnpRamp, acqnpPlateau);
      printf("Phase-encode Gradient:\n");
      printf(" Gpe: Nominal = %.3f (G/cm), Adjusted = %.3f (G/cm), Duration
= %.1f(us)\n", blipgpe, gpef * blipgpe, usecfact * gpet);
      if (acq3d[0] == 'y') {
        printf(" gpe2 = %.3f (G/cm) Total grad= %.1f (G/cm)  \n", gpe2, gssf
* gssr + (nv2 / 2) * gpe2);
      }
      printf("Additional information of EPI:\n");
      printf("****** Gradients in DAC Units: \n");
      printf(" Gss = %.1f, Gssr = %.1f \n", gss * conv, gssr * conv);
      printf(" Gro = %.1f, Gror = %.1f \n", gro * conv, gror * conv);
      printf(" Gpe = %.1f, Gpe2 = %.1f \n", blipgpe * conv, gpe2 * conv);
      printf("****** Dwell Times and Time Summs: \n");
      if (readtype[0] == 'n') {
        summdwell = 0.0;
        for (i = 0; i < np / 2.0; i++) {
          summdwell = summdwell + dwell[i];
        }
      }
      if (readtype[0] == 'l') {
        printf("   Uniform dwell time = %.1f (usec)\n", usecfact / sw);
      }
    } else {
      printf("EPI SEQUENCE INFORMATION:\n");
      printf("Sequence Operates without read-out gradients: Shimming
Mode\n");
    }
    if (errorcode > 0.0) {
      printf("EPI SEQUENCE ABORTED: Check Error Messages\n");
      exit();
    }
  }
  if ((ix == 2) && (fullinfo[1] == 'p')) {
    if (gpef > 0.0) {
      printf("****** Phase Encoding Step Tables: \n");
      for (i = 0; i < nv; i++) {
        printf("   Phase encoding step: %d strength: %d\n", i,
phasestep[i]);
      }
    }
  }
  /* * PULSE SEQUENCE ************************************ */
  /* * Relaxation delay ********************************** */
  status(A);
  rcvron();

  /* * RTM Phase cycling stuff coordinated with the cntr var ** */
  assign(zero, v1);
  if (agro > 0.0) {
    initval(nv == 0 ? 1 : nv / etl, v8);
    assign(zero, v10);
    assign(zero, v12);
    initval(etl, v11);
    assign(zero, v4);

    /* * loop for ss & ns inversion & segmentation */
    delay(d1);
    loop(v8, v9);                 /* the segment loop */
    hlv(v8, v2);
    sub(v9, v2, v2);
    if (acq3d[0] == 'y') {
      peloop(seqcon[3], nv2, v13, v14);
      sp3off();
      delay(dseg);
      sp3on();
    } else {
      msloop(seqcon[1], ns, v13, v14);
      poffset_list(pss, gss, ns, v14);
      sp3off();
      delay(dseg);
      sp3on();
    }
    /* * Gating ************************************** */
    xgate(ticks);
    if (ix != (int) skip) {
      gperephase = 1.0;
    } else {
      gperephase = 0.;
    }
    if (phaseref[0] == 'y')
      gperephase = 1.0;
    /* * 90 degree pulse *********************************** */
    observepower(tpwr1);
    shapedgradient("rampup", gsst, gss * conv, slice, 1, WAIT);
    shapedpulse(p1pat, p1, v1, rof1, rof2);
    shapedgradient("rampdown", gsst, gss * conv, slice, 1, WAIT);
    if (acq3d[0] == 'y') {
      shapedvgradient("tri20", gssrt, -(gssf * gssr + (nv2 / 2) * gpe2) *
conv, gpe2 * conv, v4, slice, one, NOWAIT, 1);
    } else
      shapedgradient("tri20", gssrt, -gssf * gssr * conv, slice, 1, NOWAIT);
    delay(time2 + deltate);

    obsoffset(resto);

    /* * getting phase-encoding multiplier */
    if (ix == (int) skip) {
      if (phaseref[0] == 'y')
        getelem(t7, v10, v7);
      else
        getelem(t2, v10, v7);
    } else
      getelem(t3, v10, v7);

    if (navecho[0] == 'y') {
      /* * getting navigator echo */

      /* * read pre-encoding */
      shapedgradient(gropat2, grort, -gror * conv * grof, read, 1, WAIT);
      /* * readout gradient for navigator echo */
      shapedgradient(gropat2, grot, gro * conv, read, 1, NOWAIT);
      /* * with phase encoding at the end */
      if (ix == (int) skip && phaseref[0] != 'y')
        shapedvgradient(gpepat2, grot, 0, 0.0, v7, phase, one, NOWAIT, 1);
      else
        shapedvgradient(gpepat2, grot, 0, -blipgpe * conv * gpe_adj, v7,
phase, one, NOWAIT, 1);
      poffset(pro, gro);
        delay(alfa + startacqdelay);
      if (readtype[0] == 'n') {
        sp1on();
        for (i = 0; i < np / 2.0; i++) {
          acquire(2.0, dwell[i]);
        }
        sp1off();
      } else {
        sp1on();
        acquire(np, 1.0 / sw);
        sp1off();
        if (readtype[1] == 'f')
          delay(grate * agro);
      }
      zero_all_gradients();

      incr(v10);

      /* * phase pre-encoding for linear phase encoding traversal */
      if (!strcmp(phasetype, "lin")) {
        if (readtype[1] == 'f') {
          if (hlfourier[0] == 'y') {
            shapedgradient("tri20", grort, (int) (gperephase * gpeint *
negphases * 2 / grort * conv), phase, 1, WAIT);
          } else {
            shapedgradient("tri20", grort, (int) (gperephase * gpeint * npes
/ grort * conv), phase, 1, WAIT);
          }
        } else {
          if (hlfourier[0] == 'y') {
            shapedgradient(gropat2, grort, (int) (2 * gperephase * gror /
truenp * (negphases * 2) * conv), phase, 1, WAIT);
          } else {
            shapedgradient(gropat2, grort, (int) (2. * gror * gperephase /
truenp * npes * conv), phase, 1, WAIT);
          }
        }
      }
    } else {
      /* * pre-encoding in gpe */
      if (!strcmp(phasetype, "lin")) {
        if (readtype[1] == 'f') {
          if (hlfourier[0] == 'y') {
            shapedgradient("tri20", grort, (int) (gperephase * gpeint *
negphases * 2 / grort * conv), phase, 1, NOWAIT);
          } else {
            shapedgradient("tri20", grort, (int) (gperephase * gpeint * npes
/ grort * conv), phase, 1, NOWAIT);
          }
        } else {
          if (hlfourier[0] == 'y') {
            shapedgradient(gropat2, grort, (int) (2 * gperephase * gror /
truenp * (negphases * 2) * conv), phase, 1, NOWAIT);
          } else {
            shapedgradient(gropat2, grort, (int) (2. * gror * gperephase /
truenp * npes * conv), phase, 1, NOWAIT);
          }
        }
      }
      /* * read pre-encoding */
      shapedgradient(gropat2, grort, -gror * conv * grof, read, 1, WAIT);

      /*
       * preencoding in phase direction if segmented. This is
       * currently done with a full readout lobe for
       * consistency.  It can be shortened if necessary.
       */

      if (nseg > 1.0) {
        if (ix == (int) skip) {
          if (phaseref[0] == 'y')
            getelem(t6, v9, v7);
          else
            getelem(t5, v9, v7);
        } else
          getelem(t4, v9, v7);

        shapedvgradient(gpepat2, grot, 0, -blipgpe * conv * gpe_adj, v2,
phase, one, WAIT, 1);
      }
    }
    zero_all_gradients();

    if (singlegrad[0] == 'y') {
      if (ix == (int) skip) {
        if (phaseref[0] == 'y')
          getelem(t3, v10, v7);
        else
          getelem(t2, v10, v7);
      } else
        getelem(t3, v10, v7);
      shapedgradient(gropat1, totalgrot, gro * conv, read, 1, NOWAIT);
      if (ix == (int) skip) {
        if (phaseref[0] == 'y')
          shapedgradient(gpepat3, totalgrot, -blipgpe * conv * gpe_adj *
gpemul, phase, 1, NOWAIT);
        else
          shapedvgradient(gpepat1, totalgrot, 0, 0.0, v7, phase, one,
NOWAIT, 1);
      } else
        shapedvgradient(gpepat1, totalgrot, 0, -blipgpe * conv * gpe_adj,
v7, phase, one, NOWAIT, 1);
      delay(alfa);
      if (navecho[0] == 'y') {
        initval(etl - 1.0, v3);
      } else {
        initval(etl, v3);
      }
    } else {
      if (navecho[0] == 'y') {
        peloop(seqcon[2], etl - 1, v5, v6);
      } else {
        peloop(seqcon[2], etl, v5, v6);
      }
      sp1on();

      if (ix == (int) skip) {
        if (phaseref[0] == 'y')
          getelem(t7, v10, v7);
        else
          getelem(t2, v10, v7);
      } else
        getelem(t3, v10, v7);
      mod2(v6, v12);

      ifzero(v12);
      shapedgradient(gropat1, totalgrot, gro * conv, read, 1, NOWAIT);
      shapedvgradient(gpepat2, totalgrot, 0, -blipgpe * conv * gpe_adj, v7,
phase, one, NOWAIT, 1);
      delay(alfa);
      /* poffset(pro, readsign * gro); */
      elsenz(v12);
      shapedgradient(gropat1, totalgrot, -gro * conv * grof2, read, 1,
NOWAIT);
      shapedvgradient(gpepat2, totalgrot, 0, -blipgpe * conv * gpe_adj, v7,
phase, one, NOWAIT, 1);
      delay(alfa);
      /* poffset(pro, -readsign * gro); */
      endif(v12);
      delay(startacqdelay);
      initval(1.0, v3);
    }
    if (readtype[0] == 'n') {
      sp1on();
      starthardloop(v3);
      for (i = 0; i < np / 2.0; i++) {
        acquire(2.0, dwell[i] * dwell_adj);
      }
      endhardloop();
      sp1off();
    } else {
      starthardloop(v3);
      acquire(np, 1.0 / sw * dwell_adj);
      endhardloop();
      if (readtype[1] == 'f')
        delay(grate * agro);
    }
    if (singlegrad[0] == 'n') {
      incr(v10);
      endpeloop(seqcon[2], v6);
      sp1off();
      sub(v10, v11, v10);
    } else {
      if (navecho[0] == 'y') {
        for (i = 0; i < etl - 1; i++)
          incr(v10);
      } else {
        for (i = 0; i < etl; i++)
          incr(v10);
      }
      sub(v10, v11, v10);
    }
    /* * crusher gradients */
    rgradient(slice, gcrush * conv);
    rgradient(read, gcrush * conv);
    rgradient(phase, gcrush * conv);
    delay(tcrush);
    zero_all_gradients();
    delay(grate * agcrush);

    delay(internaltr);
    if (acq3d[0] == 'y') {
      incr(v4);
      endpeloop(seqcon[3], v14);
    } else {
      endmsloop(seqcon[1], v14);
    }
    add(v10, v11, v10);
    endloop(v9);
    rcvroff();
  } else {
    if (ix == 1)
      printf("Shimming Mode\n");
    offset(resto, TODEV);
    observepower(tpwr1);
    if (agss > 0.0) {
      shapedgradient("rampup", gsst, gss * conv, slice, 1, WAIT);
      shapedpulse(p1pat, p1, v1, rof1, rof2);
      shapedgradient("rampdown", gsst, gss * conv, slice, 1, WAIT);
      shapedgradient("tri20", gssrt, -gssf * gssr * conv, slice, 1, WAIT);
    } else
    {
      shapedpulse(p1pat, p1, v1, rof1, rof2);
    }
    acquire(np, 1.0 / sw);
  }
}
