/***********************************************************************
    MP_BEPI_NAV.C

    Blipped interleaved echo-planar imaging sequence.
    Complete with nav echoes.

    VERSION 3.01	04/01/93	ARR
    Last modified	20/03/97	JSG
			04/07/97	JSG
			22/09/97	JSG,RSM,BG

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
     _                       __                                 __     _
    |                       |                                     |     |
RF  | -----[alpha]----------|---------------- |** ACQ **|---------|     |
    |                       |                                     |     |
    |          < - - - - - -|- -esp - - - - - - - >               |     |
    |        ____           |                                     |     |
GSS | ______/////\     _____|_____________________________________|     |
    | <---->      \\\\/ gssr|                                     |     |
    |   predelay  <--> tref |                                     | etl | ns
    |                       |                                     |     |
    |                       |                                     |     |
GPE |  _____________________|______________/\_____________________|     |
    |                       |              \/                     |     |
    |                       |                                     |     |
    |         gror          |                  _________          |     |
GRO |  ___________     _____|_________________//////////\_________|     |
    |             \\\\/     |                                     |     |
    |_                      |__                                 __|    _|

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

#include "standard.h"
#include "group.h"
extern double rint();

pulsesequence()
{
    extern codeint v15;
    /* INTERNAL VARIABLE DECLARATIONS *********************/
    int     cntr,table=0;
    double  esp,etl,predelay,seqtime,tref;
    double  risetref,tau1,tau2;
    double  agro,agss,agcrush,grate,gssrint,grorint,gpeint;
    double  risetref2,tref2,etl_delay;
    double  dseg,minte,eff_echo,grof2,tbpe,sgbpe;
    double  tbrefmin,delay90_180,delay180_echo;
    char    contrast[MAXSTR],cntron[MAXSTR],rtm_flag[MAXSTR],intlv[MAXSTR];
    char    spinecho[MAXSTR],scope[MAXSTR];

    char str[ MAXSTR ];
    char numStr1[ MAXSTR ];
    char numStr2[ MAXSTR ];
    int idx, num1, num2, endflag;
    char *ptr;

    initparms_sis();

    esp = getval("esp");
    dseg = getval("dseg");
    grof2 = getval("grof2");
    cntr = getval("cntr");
    grate = trise/gmax;
    agro = fabs(gro);
    agss = fabs(gss);
    agcrush = fabs(gcrush);
    getstr("contrast",contrast);
    getstr("cntron",cntron);
    getstr("rtm_flag",rtm_flag);
    getstr("intlv",intlv);
    getstr("scope",scope);
    getstr("spinecho",spinecho);

    /* Decide on number of pe steps per segment or image ****/
    /* by parsing the table string **************************/
    /* table name must be of form epi(#pesteps)([alt]/[lin])(#shots)k***/

    if (strcmp(petable,"n")  &&  strcmp(petable,"N")  &&  strcmp(petable,"")) {
      loadtable(petable);
      table = 1;
    }

    ptr = petable;
    *numStr1 = '\0';
    *numStr2 = '\0';
    *str = '\0';
 
    endflag = 0;
    num2 = 1;
    num1 = 1;
 
     while( ! isdigit( *ptr ) )
             ptr++;

     idx = 0;       
     while( (*ptr != '\0') && isdigit( *ptr ) )
     {
             numStr1[idx] = *ptr;
             numStr1[idx + 1] = '\0';
             idx++;
             ptr++;
     }

     idx = 0;
     while((*ptr != '\0') && (! isdigit( *ptr )))
     {
             str[idx] = *ptr;
             str[idx + 1] = '\0';
             idx++;
             ptr++;
     }
 
     idx = 0;
     while((*ptr != '\0') && isdigit( *ptr ))
     {
             numStr2[idx] = *ptr;
             numStr2[idx + 1] = '\0';
             idx++;
             ptr++;
             endflag = 1;
     }
          
    num1 = atoi( numStr1 );
    num2 = atoi( numStr2 );
    if (num2 == 0) num2 = 1;

    if (endflag)
      etl = (double)num1/num2;
    else
      etl = (double)num1;

    if (ix == 1) {
       printf("# of PE steps per seg: %f \n", etl);
       printf("# of segments (shots): %d \n", num2);
       printf("Phase encode type: %s \n", str);
    }

/* if alternating, first echo will be TE
   oterhwise linear phase encoding steps, middle of echo trains */

    if (str[0] == 'a') eff_echo = 1.0;
    else eff_echo = floor((double)etl/2.0);

    if (ix == 1)
      fprintf(stderr,"etl: %7.4f    eff_echo: %7.4f\n",etl,eff_echo);

    if (nv - num1 != 0.0) {
          printf("Invalid petable name - incorrect number of pe steps.");
    }

    if (grof == 0.0  ||  !var_active("grof",CURRENT) )
      grof = 1.0;
    if (grof2 == 0.0  ||  !var_active("grof2",CURRENT) )
      grof2 = 1.0;
    if (gssf == 0.0  ||  !var_active("gssf",CURRENT) )
      gssf = 1.0;

  /****
  * Calculate minimum time for phase-encoding, slice refocussing
  * and readout prefocussing block.

  ****/
  /* GRADIENT*TIME INTEGRALS FOR SLICE SELECTION, PHASE AND READOUT */
    grorint = grof*agro*(at/2.0 + grate*agro/2.0);
    gssrint = gssf*agss*(p1/2.0 + rof2 + agss*grate/2.0);
    gpeint = B0 / (lpe*sfrq*1e6);

  /* MINIMUM SLICE REFOCUSSING BLOCK TIME.  TRAP IF GMAX IS EXCEEDED */
    if (trise*gmax >= gssrint) {
	tref = sqrt(grate*gssrint);
	risetref = tref;
    }
    else {
	tref = grate*gmax + gssrint/gmax;
	risetref = trise;
    }

  /* MINIMUM REFOCUSSING BLOCK TIME FOR READOUT REWINDER */
    if (trise*gmax >= grorint) {
	tref2 = sqrt(grate*grorint);
	risetref2 = tref2;
    }
    else {
	tref2 = grate*gmax + grorint/gmax;
	risetref2 = trise;
    }

    tbrefmin = grate*agro;
    if (nv > 0) {
      tbpe = tbrefmin;
      sgbpe = gpeint/tbpe;
      sgbpe = gmax/gradstepsz*rint(sgbpe*gradstepsz/gmax);
      tbrefmin = gpeint/sgbpe;
    }
    else 
     {
      sgbpe = 0.0;
     }


  /* CALCULATE GSSR AND GROR WITH NEW TREF's */
    if (pilot[0] == 'y'  ||  pilot[0] == 'Y') {
      gror = -grorint/tref2;
      gssr = -gssrint/tref;
      if (ix == 1)
        fprintf(stderr,"gror: %7.4f    gssr: %7.4f\n",gror,gssr);
    }
    risetref = grate*fabs(gssr);

  /* CALCULATE TAU1 - MINIMUM TIME BETWEEN ALPHA AND FIRST ECHO (NAV)
     & CALCULATE TAU2 - MINIMUM TIME BETWEEN ECHOES */

    tau1 = p1/2.0 + rof2 + grate*(agss + fabs(gror) + agro) + tref
		+ risetref + tref2 + tbrefmin + at/2.0;
    tau2 = at + 2.0*grate*agro; 

  /* CALCULATE MINIMUM TE AND DELAY TO HAVE FLEXIBLE TE */
    minte = tau1 + eff_echo * tau2;
    etl_delay = te - minte;

    if (te < minte) {
	printf("MP_BEPI: te too small.  Minimum te = %f\n",minte);
	abort(1);
    }

    if (ix == 1) {
       fprintf(stderr,"tau1: %9.8f\n",tau1);
       fprintf(stderr,"tau2: %9.8f\n",tau2);
    }

  /* CALCULATE RELAXATION DELAY FOR BOTH INT and NON-INT CASES */
    if (intlv[0] == 'y') {
      seqtime = ( grate*agss + rof1 + p1/2.0 + te + 
		  (etl-eff_echo-1) * tau2 + at/2.0 + grate*agro +
		  tcrush + 2.0*grate*agcrush ) * ns;
      if (tr - seqtime < 0.0) {
          printf("MP_BEPI: Requested tr too short.  Min tr = %f\n",seqtime);
          abort(1);
      }
    }
    else {
      seqtime = grate*agss + rof1 + p1/2.0 + te + 
		  (etl-eff_echo-1) * tau2 + at/2.0 + grate*agro +
		  tcrush + 2.0*grate*agcrush;
      if (tr - seqtime < 0.0) {
          printf("MP_BEPI: Requested tr too short.  Min tr = %f\n",seqtime);
          abort(1);
      }
    }

    /* Timing for spin echo delays *********************/
    delay90_180 = 0;
    delay180_echo = 0;
    if (spinecho[0] == 'y') {
      delay90_180 = te/2.0 - (p1/2.0+rof2+2.0*grate*agss+tref+risetref+
                                grate*agcrush+tcrush+grate*(agcrush-agss)+
				rof1+p2/2.0);
      delay180_echo = te/2.0 - (p2/2.0+rof2+grate*(agcrush-agss)+tcrush+
				grate*agcrush+grate*agss+tref2+tbrefmin+
                                grate*agro+at/2.0+tau2);
    }

    if (intlv[0] == 'y')
      predelay = (tr - seqtime)/ns;
    else
      predelay = (tr - seqtime);

    /*** Set all phase encodes to zero when cntr = 0 ***/
    if (cntr == 0)
    {
      sgbpe = 0.0;
    }

  /* PULSE SEQUENCE *************************************/
    /* Phase cycle: Part I ********************************/
    mod2(ct,v3);                    /*0101 0101*/
    dbl(v3,v3);                     /*0202 0202*/
    hlv(ct,v4);                     /*0011 2233*/
    mod2(v4,v4);                    /*0011 0011*/
    add(v3,v4,v1);                  /*0213 0213*/
    assign(v1,oph);

    /* Phase cycle: Part II *******************************/
    mod2(ct,v4);
    ifzero(v4); 
      add(one,v1,v2);
    elsenz(v4);
      add(three,v1,v2);
    endif(v4);

    status(A);

    /* RTM Phase cycling stuff coordinated with the cntr var ***/
    if ((cntr > 0) && (rtm_flag[0] == 'y'))
      assign(zero,v1);

    initval(nv == 0 ? 1 : nv/etl,v8);
    initval(0.0,v15);
    initval(etl,v11);

    /******************************************************/
    /**** PULSE SEQUENCE FOR INTERLEAVED SLICE EPI ********/
    /******************************************************/

    if (intlv[0] == 'y')
    {
      if (cntr == 1 && cntron[0] == 's')
      {
       sp1on();
       delay(0.00001);
       sp1off();
       delay(dseg-0.00001);
      }
      else
       delay(dseg);

      assign(v15,v10);
      loop(v8,v9);
        msloop(seqcon[1],ns,v13,v14);
          poffset_list(pss,gss,ns,v14);
          delay(predelay);

          /* Scope trigger *************************/
          if (scope[0] == 'y') {
             sp2on();
             delay(5e-6);
             sp2off();
          }

          /* Gating *****/
          xgate(ticks);

          /* 90 degree pulse ************************************/ 
          observepower(tpwr1);
          obl_gradient(0.0,0.0,gss);
          delay(grate*agss);
          shapedpulse(p1pat,p1,v1,rof1,rof2);
          zero_all_gradients();
	  delay(grate*agss);

          /* Read & Slice refocus and phase encode pulse ********/
          obl_gradient(0.0,0.0,gssr);
	  delay(tref);
          zero_all_gradients();
	  delay(risetref);

          /* Spin echo section - with Crushers *********************/
          if (spinecho[0] == 'y'){
            delay(delay90_180);
            observepower(tpwr2);
            obl_gradient(0.0,0.0,gcrush);
            delay(grate*agcrush+tcrush);
            obl_gradient(0.0,0.0,gss);
            delay(grate*(agcrush-agss));
            shapedpulse(p2pat,p2,v1,rof1,rof2);
            obl_gradient(0.0,0.0,gcrush);
            delay(grate*(agcrush-agss)+tcrush);
            zero_all_gradients();
	    delay(grate*agcrush);
            delay(delay180_echo);
          }
          else
	    delay(etl_delay);

          obl_gradient(gror,0.0,0.0);
	  delay(tref2);
	  zero_all_gradients();
	  delay(grate*fabs(gror));

          peloop(seqcon[2],etl,v5,v6);
            if (nv == 0)
	      assign(v15,v7);
            else if (table)
              getelem(t2,v10,v7);
            else
	      assign(v10,v7);
            mod2(v6,v12);

            /* Acquire echo ***************************************/
            ifzero(v12);
              phase_encode_gradient(0.0,0.0,0.0,-sgbpe,v7,nv/etl,psi,phi,theta);
	      delay(tbrefmin);
              obl_gradient(gro,0.0,0.0);
	      poffset(pro,gro);
            elsenz(v12);
              phase_encode_gradient(0.0,0.0,0.0,-sgbpe,v7,nv/etl,psi,phi,theta);
	      delay(tbrefmin);
              obl_gradient(-gro,0.0,0.0);
	      poffset(pro,-gro);
            endif(v12);
            delay(grate*agro);
            acquire(np,1.0/sw);
            incr(v10);
          endpeloop(seqcon[2],v6);
          sub(v10,v11,v10);
	  zero_all_gradients();
	  delay(grate*agro);
	  obl_gradient(gcrush,gcrush,gcrush);
	  delay(tcrush+grate*agcrush);
	  zero_all_gradients();
 	  delay(grate*agcrush);
        endmsloop(seqcon[1],v14);
      add(v10,v11,v10);
      endloop(v9);
    } /*** END INTERLEAVE PS ***/


    /*****************************************************/
    /**** PULSE SEQUENCE FOR NON-INTERLEAVED CASE ********/
    /*****************************************************/

    else
    {

    if (cntr == 1 && cntron[0] == 's')
    {
     sp1on();
     delay(0.00001);
     sp1off();
     delay(dseg-0.00001);
    }
    else
     delay(dseg);

    if (cntr == 1 && cntron[0] == 'f')
       sp1on();

    msloop(seqcon[1],ns,v13,v14);
      assign(v15,v10);

      /* Inversion pulse **************/
       if (contrast[0] == 'i' && rtm_flag[0] == 'n')
       { 
         obsoffset(resto);
         obspower(tpwri);
         shapedpulse(pipat,pi,zero,rof1,rof2);
         obl_gradient(1.5,-1.5,1.5);
         delay(0.010);
         zero_all_gradients();
         delay(ti-0.010);
       }

       loop(v8,v9);
        poffset_list(pss,gss,ns,v14);
        delay(predelay);

        /* Scope trigger pulse *********************/
        if (scope[0] == 'y') {
          sp2on();
          delay(1e-6);
          sp2off();
        }

        /* Gating ***************************************/
        xgate(ticks);
       
        /* 90 degree pulse ************************************/ 
        observepower(tpwr1);
        obl_gradient(0.0,0.0,gss);
        delay(grate*agss);
        shapedpulse(p1pat,p1,v1,rof1,rof2);
        zero_all_gradients();
 	delay(grate*agss);
  
        /* Read & Slice refocus and phase encode pulse ********/
        obl_gradient(0.0,0.0,gssr);
 	delay(tref);
        zero_all_gradients();
 	delay(risetref);
 	delay(etl_delay);
  
        obl_gradient(gror,0.0,0.0);
 	delay(tref2);
        zero_all_gradients();
	delay(grate*fabs(gror));
  
        peloop(seqcon[2],etl,v5,v6);
          if (nv == 0)
	    assign(v15,v7);
          else if (table)
            getelem(t2,v10,v7);
          else
            assign(v10,v7);
          mod2(v6,v12);

          /* Acquire echo ***************************************/
          ifzero(v12);
            phase_encode_gradient(0.0,0.0,0.0,-sgbpe,v7,nv/etl,psi,phi,theta);
	    delay(tbrefmin);
            obl_gradient(gro,0.0,0.0);
            poffset(pro,gro);
          elsenz(v12);
            phase_encode_gradient(0.0,0.0,0.0,-sgbpe,v7,nv/etl,psi,phi,theta);
            delay(tbrefmin);
            obl_gradient(-gro,0.0,0.0);
            poffset(pro,-gro);
          endif(v12);
          delay(grate*agro);
          acquire(np,1.0/sw);
          incr(v10);
        endpeloop(seqcon[2],v6);

        zero_all_gradients();
        delay(grate*agro);
        obl_gradient(gcrush,gcrush,gcrush);
        delay(tcrush+grate*agcrush);
        zero_all_gradients();
        delay(grate*agcrush);
       endloop(v9);
      endmsloop(seqcon[1],v14);

      if (cntr == 3 && cntron[0] == 'f')
        sp1off();
    } /* END NON-INTERLEAVE PS */
}
