#ifndef LINT
static char SCCSid[] = "@(#)flash.c 10.5 05 Mar 1996 Copyright (c) 1994 Varian Assoc.,Inc. All Rights Reserved";
#endif
/***********************************************************************
    MP_FLASH3D.C

    Magnetization Prepared Gradient-echo fast imaging sequence.                     

    VERSION 2.0        6/94/94  
    Revisions:
      2.1   2/96    Fixed rfspoiling for nv=0
      2.2   3/97    JSG - Completed changes to 3D

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
        __                                                  __ 
       |                                                      |
  RF   |----- [ p1 ] ----------------------- |** ACQ **| -----|
       |                                                      |
       |         < - - - - - - - te - - - - - - - >           |
       |       ____                                           |
  GSS  |______/////\     _____________________________________|
       |<---->      \\\\/ gssr                                |
       |  predelay  <--> tref                                 | nv
       |             ___                                  ___ |
  GPE  |____________/___\________________________________/___\|
       |            \___/                                \___/|
       |                 < - tA - >                           |
       |                            ____________________      |
  GRO  |____________     __________/////////////////////\_____|
       |       gror \\\\/          < tspoil >                 |
       |                                                      |
       |< - - - - - - - - - - - tr - - - - - - - - - - - - - >|
       |__                                                  __|

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

#include <standard.h>
#define OFFSETDELAY 26e-6
#define RFSPOILDELAY 50e-6

pulsesequence()
{
    /* Internal variable declarations *********************/
    double  predelay,seqtime,tref,risetref,dseg;
    double  gpe2int, tpe2min, sgpe2;
    double  trefmin,trormin,tpemin,temin,tA;
    double  agro,agss,sgpe,grate,gssrint,grorint,gpeint;      
    double  reffreq,nextphase,spoilfreqs[1024];
    int     i,len,counter,midflag,endflag;
    double  val1,val2,nsteps;
    char    refocus[MAXSTR],contrast[MAXSTR],min_phase[MAXSTR];;

    initparms_sis();
    loop_check();

    getstr("refocus",refocus);
    getstr("contrast",contrast);
    getstr("min_phase",min_phase);
    dseg=getval("dseg");
    grate = trise/gmax;
    agro = fabs(gro);
    agss = fabs(gss);

    /* Decide on number of pe steps per segment or image ****/
    /* by parsing the table string **************************/
    /* table name must be of form #pestepsxxx#segmentsyyy ***/
    loadtable(petable);
    len = strlen(petable);
    counter = 0;
    midflag = 0;
    endflag = 0;
    val1 = 0.0;
    val2 = 0.0;
    while (counter < len) {
      if (isdigit(petable[counter]) && !midflag) {
        val1 =  atof(&(petable[counter]));
          /* skip past remaining part of first number set */
          while (isdigit(petable[counter ++]))
          ;
      }  
      if (!isdigit(petable[counter]))
        midflag = 1;
      if (isdigit(petable[counter]) && midflag) {
        val2 = 10.0 *val2 + atof(&(petable[counter]));
          /* skip past remaining part of second number set */
          while (isdigit(petable[counter ++]))
          ;
        endflag = 1;
      }  
    counter++;
    }
    if (endflag)
      nsteps = val1/val2;
    else
      nsteps = val1;
    printf("number of pe steps per image or image segment is %f\n",nsteps);
 


    /* GRADIENT*TIME integrals for slice selection and readout */
    grorint = agro*(at/2.0 + tspoil + grate*agro/2.0);
    gssrint = agss*(p1/2.0 + rof2 + agss*grate/2.0);

    /* DO for minimum phase pulse ****/
    if (min_phase[0] == 'y')
      gssrint = agss*(p1*0.135 + rof2 + agss*grate/2.0);

    gpeint = B0/(lpe*sfrq*1e6);
    gpe2int = B0/(lpe2*sfrq*1e6);

    /*******************************************************
    * Calculate minimum time for phase-encoding, slice refocussing
    * and readout prefocussing block.  Trapezoidal if GMAX is exceeded.
    * TSSRMIN does not include the gradient fall time.  RISETREF is
    * the gradient fall time for the PE block.
    *******************************************************/
    if (trise*gmax >= grorint)
        trormin = sqrt(grate*grorint);
    else
	trormin = grorint/gmax;

    if (trise*gmax >= gpeint*nv/2.0)
        tpemin = sqrt(grate*gpeint*nv/2.0);
    else
	tpemin = gpeint*nv/2.0/gmax;

    if (trise*gmax >= (gssrint + gpe2int*nv2/2.0) )
        tpe2min = sqrt(grate*(gssrint + gpe2int*nv2/2.0) );
    else
        tpe2min = (gssrint + gpe2int*nv2/2.0)/gmax;

    /* Minimum refocussing block time is the longest of calculated minima */
    trefmin = trormin;
    trefmin = (trefmin > tpemin) ? trefmin : tpemin;
    trefmin = (trefmin > tpe2min) ? trefmin : tpe2min;

    /*******************************************************
    * If TREFMIN implies trapezoidal PE block, mimimum RISETREF
    * is RISETIME, otherwise minimum RISETREF is TREFMIN.
    *******************************************************/
    risetref = (trefmin > trise) ? trise : trefmin;

    /* Check minimum TE ***********************************/
    temin = p1/2.0 + rof2 + trefmin + risetref + tspoil
          + at/2.0 + grate*(agss + agro);
    if (te < temin) {
	printf("%s: te too short.  Minimum te = %f\n",seqfil,temin);
	abort(1);
    }

    /*******************************************************
    * TREF is the refocussing block length, not including gradient fall
    * time.  TREF is scaled up to a maximum value consistent with TE
    *******************************************************/
    tref = trefmin + 0.5*(te - temin);
    tref = (tref > 2.0*trefmin) ? 2.0*trefmin : tref;

    /* Calculate GPE, adjusting for integral dac value, then adjust TREF */
    sgpe = gpeint/tref;
    sgpe = gmax/gradstepsz*floor(sgpe*gradstepsz/gmax + 0.5);
    sgpe2 = gpe2int/tref;
    sgpe2 = gmax/gradstepsz*floor(sgpe2*gradstepsz/gmax + 0.5);

    /*******************************************************
    * A single solution for tref is not guaranteed to exist for the two
    * values of sgpe and sgpe2.  A good compromise is the geometric mean.
    * If this tref results in a distortion in either PE direction of more
    * than 1%, issue an advisory message.
    *******************************************************/
    tref = sqrt((gpeint/sgpe)*(gpe2int/sgpe2));
    if (nth2D == 1  &&  (fabs(1.0 - sgpe*tref/gpeint) > 0.01  ||
      fabs(1.0 - sgpe2*tref/gpe2int) > 0.01)) {
        printf("%s:  ADVISORY: lpe distortion =%6.2f%%      lpe2 =%6.2f%%\n",
            seqfil,100.0-sgpe*tref/gpeint*100,100.0-sgpe2*tref/gpe2int*100.0);
    }

    /* If nv is 0 or 1 we want a read profile so turn off phase-encode ***/
    if (nv < 2)
      sgpe = 0.0;

    /* Calculate GSSR and GROR with new TREF **************/
    if (pilot[0] == 'y'  ||  pilot[0] == 'Y') {
	gror = -grorint/tref*gro/fabs(gro);
	gssr = -gssrint/tref*gss/fabs(gss);
        fprintf(stderr,"%s:  gror = %-8.4f     gssr = %-8.4f\n",
	    seqfil,gror,gssr);
    }

    /* Determine PE block gradient fall time with new TREF */
    risetref = fabs(gror);
    risetref = risetref > fabs(sgpe*nv/2.0) ? risetref : fabs(sgpe*nv/2.0);
    risetref = risetref > fabs(gssr) + fabs(sgpe2*nv2/2.0) ?
               risetref : fabs(gssr) + fabs(sgpe2*nv2/2.0);
    risetref *= grate;

    /* tA is the delay between refocussing and readout ****/
    tA = te - (p1/2.0 + rof2 + tref + risetref + tspoil + at/2.0
       + grate*(agss + agro));

    /* Relaxation delay ***********************************/
    seqtime = te + p1/2.0 + rof1 + at/2.0 + grate*(agss + agro)
	    + tref + risetref + dseg;
    if (rfspoil[0] == 'y')
	seqtime += RFSPOILDELAY;
    if (tr < seqtime) {
        printf("%s: Requested tr too short.  Min tr = %f\n",seqfil,seqtime);
        abort(1);
    }
    predelay = tr - seqtime - hold;

    /*******************************************************
    * Calculation of frequencies to get phase shifts for optional
    * RF spoiling.  Phase shifts are obtained by hopping to variable
    * frequency for fixed time.
    * Ref:  Zur, Y., Magn. Res. Med., 21, 251, (1991).
    *******************************************************/
    if (rfspoil[0] == 'y' && nv > 0) {
	reffreq = resto + sfrq*1e6/B0*gro*pro;
        for (i=1; i<= (int)(nv+0.5); i++) {
	    nextphase = i*rfphase;
	    nextphase = nextphase - 360.0*floor(nextphase/360.0);
	    spoilfreqs[i] = reffreq + nextphase/RFSPOILDELAY/360.0;
        }
        create_offset_list(spoilfreqs,nv,TODEV,0);
    }


    /* PULSE SEQUENCE *************************************/

    /* Phase cycle ****************************************/
    mod2(ct,v3);          /* v3: 0  1  0  1  0  1  0  1 */
    dbl(v3,v3);           /* v3: 0  2  0  2  0  2  0  2 */
    hlv(ct,v4);           /* v4: 0  0  1  1  2  2  3  3 */
    mod2(v4,v4);          /* v4: 0  0  1  1  0  0  1  1 */
    add(v3,v4,v1);        /* v1: 0  2  1  3  0  2  1  3 */
    assign(v1,oph);      /* oph: 0  2  1  3  0  2  1  3 */

    status(A);

    /* Initialialize segmented acquisition counter **********/
    initval(nsteps,v7);
    /* Initialialize steady state counter *******************/
    initval(ssc,v9);

    /* Wait for trigger and delay 'hold' seconds after it ***/
    xgate(ticks);
    delay(hold);

    /* Compressed Phase-encode loop *************************/
    peloop(seqcon[3],nv2,v11,v12);
    peloop(seqcon[2],nv,v5,v6);

    /* Load gradient table element *************************/
    getelem(t1,v6,v14);

    /* Calculate segment number v8 **************************/
    /* v8 is zero for the first line in each segment ********/
    modn(v6,v7,v8);

    /* Inter-segment delay **********************************/
    /* Execute only on first line of each segment ***********/
    ifzero(v8);
    delay(dseg);
    endif(v8);

    /* Scope trigger **************************************/
    sp1on();
    delay(1e-6);
    sp1off();

    /* Pre-Contrast pulses **********************************/
    /* Execute only on 1st line of each segment *************/
    ifzero(v8);
    /* Inversion pulse **************************************/
      if (contrast[0] == 'i') {
        obsoffset(resto);
        obspower(tpwri);
        shapedpulse(pipat,pi,zero,rof1,rof2);
        obl_gradient(1.0,-1.0,1.0);
        delay(0.010);
        zero_all_gradients();
        delay(ti-0.010);
      }
    endif(v8);
 
    /* Optional dummy states ******************************/       
    /* Executed only at the beginning of each segment *****/
    ifzero(v8);
      loop(v9,v10);
      /* RF pulse *******************************************/
      obspower(tpwr1);
      poffset(pss[0],gss);
      delay(predelay);
      obl_gradient(0.0,0.0,gss);
      delay(grate*agss);
      shapedpulse(p1pat,p1,v1,rof1,rof2);
      zero_all_gradients();
      delay(grate*agss);
 
      /* Read & Slice refocus and phase encode pulse ********/
      pe_gradient(gror,-sgpe/2.0,gssr,-sgpe,v14);
      delay(tref);
      zero_all_gradients();
      delay(risetref);
 
      /* Pre-acquire delay **********************************/
      poffset(pro,gro);
      delay(tA);
 
      /* Acquire echo ***************************************/
      obl_gradient(gro,0.0,0.0);
      delay(grate*agro + tspoil + 0.5/fb + at);
      zero_all_gradients();
      delay(grate*agro);

      /* Optional gradient refocussing **********************/
      if (refocus[0] == 'y') {
        pe_gradient(0.0,sgpe/2.0,gss,sgpe,v14);
        delay(tref);
        zero_all_gradients();
        delay(risetref);
      }

      /*******************************************************
      * Optional RF spoiling.  REMINDER: Do not place any events
      * after this block or before poffset at the start of peloop.
      *******************************************************/
      if (rfspoil[0] == 'y' && nv > 0) {
        voffset(0,v6);
        delay(RFSPOILDELAY - OFFSETDELAY);
      }
      endloop(v10);
    endif(v8);
      
      /* RF pulse *******************************************/ 
      obspower(tpwr1);
      poffset(pss[0],gss);
      delay(predelay);
      obl_gradient(0.0,0.0,gss);
      delay(grate*agss);
      shapedpulse(p1pat,p1,v1,rof1,rof2);
      zero_all_gradients();
      delay(grate*agss);

      /* Read & Slice refocus and phase encode pulse ********/
      pe2_gradient(gror,-sgpe/2.0,sgpe2*nv2/2.0-sgpe2/2.0
			+gssr,-sgpe,-sgpe2,v14,v12);
      delay(tref);
      zero_all_gradients();
      delay(risetref);

      /* Pre-acquire delay **********************************/
      poffset(pro,gro);
      delay(tA);

      /* Acquire echo ***************************************/
      obl_gradient(gro,0.0,0.0);
      delay(grate*agro + tspoil + 0.5/fb);
      acquire(np,1.0/sw);
      zero_all_gradients();
      delay(grate*agro);
     
      /* Optional gradient refocussing **********************/ 
      if (refocus[0] == 'y') {
        pe2_gradient(0.0,sgpe/2.0,-sgpe2*nv2/2.0+sgpe2/2.0
			+gcrush,sgpe,sgpe2,v14,v12);
        delay(tref);
        zero_all_gradients();
        delay(risetref);
      }

      /*******************************************************
      * Optional RF spoiling.  REMINDER: Do not place any events
      * after this block or before poffset at the start of peloop.
      *******************************************************/
      if (rfspoil[0] == 'y' && nv > 0) {
        voffset(0,v6);
        delay(RFSPOILDELAY - OFFSETDELAY);
      }

    endpeloop(seqcon[2],v6);
    endpeloop(seqcon[3],v12);
}
