function eddy_test_results, fname, te, fov, delta_acq_delay, ret_val=ret_val

; te should be in ms, e.g., 2.5 for te=0.0025 (secs)
; fov should be in cm, e.g., 32.0 for fov=32.0cm
; delta_acq_delay is the increment value for pre_acq_del in ms, e.g., 0.2
;     for pre_acq_del increment of 0.0002 (sec)
; output is in gauss/cm

if (n_elements(ret_val) eq 0) then ret_val=0

usage="out=eddy_test_results(fname, te, fov, delta_acq_delay, ret_val=ret_val)"

if (n_elements(te) eq 0) then begin
  print, "Error: incorrect input parameters"
  print, "usage : "+usage
endif

if (n_elements(fov) eq 0) then begin
  print, "Error: incorrect input parameters"
  print, "usage : "+usage
endif

if (n_elements(delta_acq_delay) eq 0) then begin
  print, "Error: incorrect input parameters"
  print, "usage : "+usage
endif

noise_thresh=0.1

in_arr=read_varian_bin_data(fname)

dims=size(in_arr)
echo_size=dims(1)/2
num_echoes=dims(2)

eddy_current=fltarr(num_echoes)
spectra=complexarr(echo_size, num_echoes)

for i=0, num_echoes-1 do begin
  tmp_arr=reform(in_arr(*,i), 2, echo_size)
  cmplx_tmp_arr=reform(complex(tmp_arr(0,*), tmp_arr(1,*)))
  ft_cmplx_tmp_arr=shift(fft(shift(cmplx_tmp_arr, echo_size/2),1),echo_size/2)
  spectra(*,i)=ft_cmplx_tmp_arr
endfor

phase_unwrap, spectra, phase0, phase1, phase_echoes

delta_x=fov/echo_size
gamma=2.0*!pi*4257.5  ; gamma in radians/gauss

eddy_curr_grad=phase1/te*1000.0/delta_x/gamma

if (ret_val eq 0) then begin
  return, spectra
endif

if (ret_val eq 1) then begin
  plot, eddy_curr_grad
  return, eddy_curr_grad
endif

if (ret_val eq 2) then begin
  plot, phase1
  return, phase1
endif

if (ret_val eq 3) then begin
  return, phase_echoes
endif

if (ret_val eq 4) then begin
  x=indgen(num_echoes)*float(delta_acq_delay)
  plot, x, eddy_curr_grad, xtitle="time, ms", ytitle="eddy gradient, g/cm"
  set_plot, 'PS'
  plot, x, eddy_curr_grad, xtitle="time, ms", ytitle="eddy gradient, g/cm"
  device, /close
  set_plot, 'X'
  print, "sending plot to printer"
  spawn, 'lp idl.ps'
endif

return, eddy_curr_grad ; default

end
