pro make_eddy_ramps

; procedure to make a waveform that slowly rises up and a separate
; waveform that quickly ramps down. Used as part of eddy current
; compensation

slow_ramp_time=1.0	; duration of ramp up (seconds)
gmax=3.7		; maximum gradient strength
trise=200.0e-6		; minimum rise time to full gradient strength

max_dac=32767

if (slow_ramp_time/2.e-6 gt 10000) then begin
  n_slow_pts=10000
endif else begin
  n_slow_pts=fix(slow_ramp_time/2.e-6+0.5)
endelse
if (trise/2.e-6 gt 100) then begin
  n_fast_pts=10000
endif else begin
  n_fast_pts=fix(trise/2.e-6+0.5)
endelse

fast_ramp_time=trise

ramp_up=fix(findgen(n_slow_pts)/n_slow_pts*max_dac)
ramp_down=rotate(fix(findgen(n_fast_pts)/n_fast_pts*max_dac),2)

get_lun, unit
openw, unit, "~/vnmrsys/shapelib/eddy_up.GRD"
for i=0, n_slow_pts-1 do begin
  printf, unit, strtrim(string(ramp_up(i)),2)+'  1'
endfor
printf, unit, ''
close, unit
free_lun, unit

print, "set ramp_up_del to ", strtrim(string(slow_ramp_time),2), " secs"

get_lun, unit
openw, unit, "~/vnmrsys/shapelib/eddy_dn.GRD"
for i=0, n_fast_pts-1 do begin
  printf, unit, strtrim(string(ramp_down(i)),2)+'  1'
endfor
printf, unit, ''
close, unit
free_lun, unit

print, "set ramp_down_del to ", strtrim(string(trise),2), " secs"

end
