function phased_fft, ref_arr, in_arr, npoints, nviews, nslices, nints, ntpts, $
   verbose=verbose, baseline=baseline

if (n_elements(verbose) eq 0) then verbose=0
if (n_elements(baseline) eq 0) then baseline=0

;=============================================================================
; CALCULATE THE EPI NYQUIST GHOST PHASE CORRECTION
;=============================================================================
; odd_k is the odd k space line nearest spin echo
; even_k is the even k space line nearest spin echo
odd_k=nviews/2-1
even_k=nviews/2
; odd_k=0
; even_k=1

print, "using odd reference line : ", odd_k
print, "using even reference line : ", even_k
ref_phases=ref_phase_calc(ref_arr, odd_k, even_k)

if (verbose) then begin
  tvscl, rebin(abs(ref_arr(*,*,0)), 128,128), 0
  for sl_no=0, nslices-1 do begin
    tvscl, rebin(abs(ref_arr(*,*,sl_no)), 128,128), sl_no
  endfor
  wait, 1
endif

ref_phase_arr=fltarr(nviews, nslices, 2)
for sl_no=0, nslices-1 do begin
  for i=0, nviews/(2*nints)-1 do begin
    for j=0, nints-1 do begin
      k_line=i*2*nints+j
      ref_phase_arr(k_line, sl_no, *)=ref_phases(sl_no, *)
    endfor
  endfor
endfor
 
if (verbose) then begin
  for sl_no=0, nslices-1 do begin
    ref_arr(*,*,sl_no)=apply_phase(ref_arr(*,*,sl_no), $
           reform(ref_phase_arr(*,sl_no,*), nviews, 2))
    tvscl, rebin(abs(ref_arr(*,*,sl_no)), 128, 128), sl_no
  endfor
  wait, 1
endif
 
if (baseline eq 1) then begin
  for time_no=0, ntpts-1 do begin
    for sl_no=0, nslices-1 do begin
      if (nslices gt 1) then begin
        base_line=total(in_arr(*,nviews-1,sl_no, time_no))/(npoints/2)
        in_arr(*,*,sl_no, time_no)=in_arr(*,*,sl_no, time_no)-base_line
      endif else begin
        base_line=total(in_arr(*,nviews-1,time_no))/(npoints/2)
        in_arr(*,*,time_no)=in_arr(*,*,time_no)-base_line
      endelse
    endfor
  endfor
endif

for time_no=0, ntpts-1 do begin
  for sl_no=0, nslices-1 do begin
    if (nslices gt 1) then begin
      in_arr(*,*,sl_no, time_no)=apply_phase(in_arr(*,*,sl_no, time_no), $
                         reform(ref_phase_arr(*,sl_no,*), nviews, 2))
      if (verbose) then tvscl, rebin(abs(shift(fft(shift(in_arr(*,*,sl_no, $
         time_no), npoints/4, nviews/2), 1), npoints/4, nviews/2)), 128, 128), $
         sl_no
    endif else begin
      in_arr(*,*,time_no)=apply_phase(in_arr(*,*,time_no), $
                         reform(ref_phase_arr(*,0,*), nviews, 2))
      if (verbose) then tvscl, rebin(abs(shift(fft(shift(in_arr(*,*,time_no), $
         npoints/4, nviews/2), 1), npoints/4, nviews/2)), 128, 128), sl_no
    endelse
  endfor
endfor

return, in_arr

end
