function varian_bin_to_epi, in_arr, npoints, nviews, nslices, nints, $
    ntpts, verbose=verbose, baseline=baseline

if (n_elements(npoints) eq 0) then npoints=0
if (n_elements(nviews) eq 0) then nviews=0
if (n_elements(nslices) eq 0) then nslices=0
if (n_elements(nints) eq 0) then nints=0
if (n_elements(ntpts) eq 0) then ntpts=0
if (n_elements(verbose) eq 0) then verbose=0
if (n_elements(baseline) eq 0) then baseline=0

usage="out_arr=varian_bin_to_epi(in_arr, npoints, nviews, nslices, nints, ntps[, verbose=0|1, baseline=0|1])"

dims=size(in_arr)
if (dims(dims(0)+2) ne long(npoints)*long(nviews)*long(nslices)* $
           long(ntpts)) then begin
  print, "Error: incorrect dimensions specified for input array"
  print, "usage: "+usage
  return, -1
endif
if (ntpts gt 1) then begin
  if (dims(dims(0)) ne long(ntpts)) then begin
    print, "Error: incorrect dimensions specified for input array"
    print, "usage: "+usage
    return, -1
  endif
endif

if (verbose eq 1) then begin
  print, "npoints = "+strtrim(string(npoints),2)
  print, "nviews = "+strtrim(string(nviews),2)
  print, "nslices = "+strtrim(string(nslices),2)
  print, "nints = "+strtrim(string(nints),2)
  print, "ntpts = "+strtrim(string(ntpts),2)
endif

; assume that data is in the form in_arr(2*npoints/2*nechoes*nslices, nviews)
 
if (dims(dims(0)+1) eq 3) then begin ; long integer words
  out_arr=lonarr(2,npoints/2,nviews,nslices,ntpts)
endif
if (dims(dims(0)+1) eq 2) then begin ; short integer words
  out_arr=intarr(2,npoints/2,nviews,nslices,ntpts)
endif
if (dims(dims(0)+1) eq 4) then begin ; floating point words
  out_arr=fltarr(2,npoints/2,nviews,nslices,ntpts)
endif

for tpnt_no=0, ntpts-1 do begin
  tmp_trace=in_arr(*,tpnt_no)
  ; reform the in_arr to separate the real and imaginary variables
  tmp_trace=reform(tmp_trace, 2,long(npoints/2),long(nviews)*long(nslices))
  for int_no=0, nints-1 do begin
    int_indices=indgen(nviews/nints)*nints+int_no
    int_offset=long(int_no)*long(nviews/nints)*long(nslices)
    out_arr(*,*,int_indices,*,tpnt_no)=tmp_trace(*,*,int_offset: $
             int_offset+(nviews/nints)*nslices-1)

    ; reverse the appropriate lines of k space
    for rev_no=0, nviews/nints/2-1 do begin
      for slice_no=0, nslices-1 do begin
        out_arr(0,*,int_indices(2*rev_no),slice_no,tpnt_no)= $
         rotate(out_arr(0,*,int_indices(2*rev_no),slice_no,tpnt_no), 2)
        out_arr(1,*,int_indices(2*rev_no),slice_no,tpnt_no)= $
         rotate(out_arr(1,*,int_indices(2*rev_no),slice_no,tpnt_no), 2)
      endfor
    endfor
  endfor
endfor

out_arr=reform(complex(out_arr(0,*,*,*,*), out_arr(1,*,*,*,*)))

return, out_arr

end
