function varian_bin_to_fieldmap, in_arr, npoints, nviews, nslices, nechoes, $
    verbose=verbose, baseline=baseline

if (n_elements(npoints) eq 0) then npoints=0
if (n_elements(nviews) eq 0) then nviews=0
if (n_elements(nslices) eq 0) then nslices=0
if (n_elements(nechoes) eq 0) then nechoes=0
if (n_elements(verbose) eq 0) then verbose=0
if (n_elements(baseline) eq 0) then baseline=0

usage="out_arr=binary_to_fieldmap(in_arr, npoints, nviews, nslices, nechoes[, verbose=0|1, baseline=0|1])"

dims=size(in_arr)
if (dims(dims(0)+2) ne long(npoints)*long(nviews)*long(nslices)* $
           long(nechoes)) then begin
  print, "Error: incorrect dimensions specified for input array"
  print, "usage: "+usage
  return, -1
endif
if (dims(dims(0)) ne nviews) then begin
  print, "Error: incorrect dimensions specified for input array"
  print, "usage: "+usage
  return, -1
endif

if (verbose eq 1) then begin
  print, "npoints = "+strtrim(string(npoints),2)
  print, "nviews = "+strtrim(string(nviews),2)
  print, "nslices = "+strtrim(string(nslices),2)
  print, "nechoes = "+strtrim(string(nechoes),2)
endif

; assume that data is in the form in_arr(2*npoints/2*nechoes*nslices, nviews)

if (dims(dims(0)+1) eq 3) then begin ; long integer words
  out_arr=lonarr(2,npoints/2,nviews,nslices,nechoes)
endif
if (dims(dims(0)+1) eq 2) then begin ; short integer words
  out_arr=intarr(2,npoints/2,nviews,nslices,nechoes)
endif
if (dims(dims(0)+1) eq 4) then begin ; floating point words
  out_arr=fltarr(2,npoints/2,nviews,nslices,nechoes)
endif

for view_no=0, nviews-1 do begin
  tmp_trace=in_arr(*,view_no)
  ; reform the in_arr to separate the real and imaginary variables
  tmp_trace=reform(tmp_trace, 2,long(npoints/2)*long(nechoes)*long(nslices))
  for slice_no=0, nslices-1 do begin
    sl_offset=long(slice_no)*long(npoints/2)*long(nechoes)
    for echo_no=0, nechoes-1 do begin
      out_arr(*,*,view_no,slice_no,echo_no)=tmp_trace(*,sl_offset+ $
           long(echo_no)*long(npoints/2):sl_offset+long(echo_no+1)* $
           long(npoints/2)-1)
    endfor
  endfor
endfor

out_arr=reform(complex(out_arr(0,*,*,*,*),out_arr(1,*,*,*,*)))

if (baseline eq 1) then begin
  if (verbose gt 0) then print, "baseline correcting"
  for echo_no=0, nechoes-1 do begin
    for slice_no=0, nslices-1 do begin
      ; calculate baseline offset from last k space line collected
      base_line=out_arr(npoints/2-1,*,slice_no,echo_no)
      base_line=total(base_line)/nviews
      out_arr(*,*,slice_no,echo_no)=out_arr(*,*,slice_no,echo_no)-base_line
    endfor
  endfor
endif

return, out_arr

end
