#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <avwio.h>

#include "data.h"

#define MASK_SCALE_FACTOR 0.3

int main(int argc,char* argv[])
{
  struct datafilehead fh;
  struct datablockhead bh;
  short ppl,lpi,ipv,vols,dt,i,j;
  short mppl,mlpi,mipv,mvols,mdt,masked;
  int slice;
  float *buffer,*mask;
  AVW *ip,*mp;
  FILE *fp;

  if(argc<3){
    printf("usage: overwrite_phasefile phasefile AVWfile [slice -mask mask]\n");
    return(1);
  }

  if((ip=AvwOpen(argv[2],"r"))==NULL){
    printf("lv error: Failed to open AVW file: %s.\n",argv[2]);
    return(1);
  }
  AvwGetDim(ip,&ppl,&lpi,&ipv,&vols);
  AvwGetDataType(ip,&dt);

  if(argc>3){
    if(!sscanf(argv[3],"%d",&slice)){
      printf("lv error: Can't interpret slice\n");
      return(1);
    }
  } else {
    slice = 0;
  }

  if(slice>=ipv){
    printf("lv error: Slice index exceeds datasize\n");
    return(1);
  }
  if(slice<0){
    printf("lv error: Slice index exceeds datasize\n");
    return(1);
  }
  if(dt!=DT_FLOAT){
    printf("lv error: Can't deal with AVW files other than type float.\n");
    return(1);
  }

  masked=0;
  if(argc>4){
    if(!strcmp(argv[4],"-mask")){
      masked=1;
      if((mp=AvwOpen(argv[5],"r"))==NULL){
	printf("lv error: Failed to open AVW file: %s.\n",argv[5]);
	return(1);
      }
      AvwGetDim(mp,&mppl,&mlpi,&mipv,&mvols);
      AvwGetDataType(ip,&mdt);
      if(mdt!=DT_FLOAT){
	printf("lv error: Can't deal with AVW files other than type float.\n");
	return(1);
      }
      if((mppl!=ppl)||(mlpi!=lpi)||(mipv!=ipv)){
	printf("lv error: Mask not compatible with image.\n");
	return(1);
      }
      if((mask=(float *)malloc(ppl*lpi*sizeof(float)))==NULL){
	printf("lv error: Malloc failed.\n");
	return(1);
      }
      if(AvwReadSliceSeries(mp,mask,slice,1)!=1){
	printf("lv error: Read error from file: %s.\n",argv[5]);
	return(1);
      }
      for(i=0;i<ppl*lpi;i++){
	if(mask[i]>0.0)mask[i]=1;
	else mask[i]=MASK_SCALE_FACTOR;
      }
    }
  }

  if((fp=fopen(argv[1],"r+"))==NULL){
    printf("lv error: Failed to open file: %s.\n",argv[1]);
    return(1);
  } 
  if(fread(&fh,sizeof(fh),1,fp)!=1){
    printf("lv error: Read error from file: %s.\n",argv[1]);
    return(1);
  }
  
  if(fh.ebytes!=sizeof(float)){
    printf("lv error: Phasefile ebytes not 4.\n");
    return(1);
  }
  if(fh.nblocks*fh.ntraces!=lpi){
    printf("lv error: Phasefile datasize not compatible with AVW\n");
    return(1);
  }
  if(fh.np!=ppl){
    printf("lv error: Phasefile datasize not compatible with AVW\n");
    return(1);
  }

  if((buffer=(float *)malloc(ppl*lpi*sizeof(float)))==NULL){
    printf("lv error: Malloc failed.\n");
    return(1);
  }
  if(AvwReadSliceSeries(ip,buffer,slice,1)!=1){
    printf("lv error: Read error from file: %s.\n",argv[2]);
    return(1);
  }

  if(masked){
    for(i=0;i<ppl*lpi;i++)buffer[i]*=mask[i];
  }

  fseek(fp,sizeof(fh),SEEK_SET);

  if(fh.nblocks==1){
    fseek(fp,sizeof(bh),SEEK_CUR);
    if(fwrite(buffer,sizeof(float),fh.np*fh.ntraces,fp)!=fh.np*fh.ntraces){
      printf("lv error: Write error to file: %s.\n",argv[1]);
      return(1);
    }
  } else {
    for(i=0;i<fh.nblocks;i++){
      fseek(fp,sizeof(bh),SEEK_CUR);
      for(j=0;j<fh.ntraces;j++){
	if(fwrite(&buffer[(i*fh.ntraces+j)*fh.np],sizeof(float),fh.np,fp)!=fh.np){
	  printf("lv error: Write error to file: %s.\n",argv[1]);
	  return(1);
	}
      }
    }
  }
  
  AvwClose(ip);
  fclose(fp);
  return(0);
}
