
;=============================================================================
pro preview_dropout, fname
;=============================================================================

common correction_info, pha0, pha1, scal_fact

; first need to look at the procpar file to gain info on n_slices etc

get_lun, unit
line=string(0)

openr, unit, fname+'/procpar'
; search for np
while (not eof(unit)) do begin
  readf, unit, line
  if (strpos(line, 'np ') eq 0) then begin
    readf, unit, dummy1, np
    dim0=fix(np/2)
    print, "dim0 = ", dim0
  endif
endwhile
close, unit

openr, unit, fname+'/procpar'
; search for nv
while (not eof(unit)) do begin
  readf, unit, line
  if (strpos(line, 'nv ') eq 0) then begin
    readf, unit, dummy1, nv
    dim1=fix(nv)
    print, "dim1 = ", dim1
  endif
endwhile
close, unit

openr, unit, fname+'/procpar'
; search for num slices
while (not eof(unit)) do begin
  readf, unit, line
  if (strpos(line, 'pss ') eq 0) then begin
    readf, unit, ns
    n_slices=fix(ns)
    print, "num slices = ", n_slices
  endif
endwhile
close, unit

openr, unit, fname+'/procpar'
; search for num time points
while (not eof(unit)) do begin
  readf, unit, line
  if (strpos(line, 'arraydim ') eq 0) then begin
    readf, unit, null, arraydim
    n_t_pts=fix(arraydim)
    print, "num time points = ", n_t_pts
  endif
endwhile
close, unit

; now read the data

file_head=bytarr(32)
block_head=bytarr(28)

in_arr=intarr(2, dim0, dim1, n_slices)

openr, unit, fname+'/fid'
readu, unit, file_head

readu, unit, block_head
readu, unit, in_arr
close, unit
out_arr=reform(complex(in_arr(0,*,*,*),in_arr(1,*,*,*)))

max_val=-1
for j=0, n_slices-1 do begin
  tmp_im=out_arr(*,*,j)
  for k=0, dim1/2-1 do tmp_im(*,2*k)=rotate(tmp_im(*,2*k), 2)
  tmp_im=abs(shift(fft(shift(tmp_im, dim0/2, dim1/2), 1), dim0/2, dim1/2))
  tmp_im(dim0/2, dim1/2)=(total(tmp_im(dim0/2-1:dim0/2+1,dim1/2-1:dim1/2+1))-tmp_im(dim0/2,dim1/2))/8
  if (max(tmp_im) gt max_val) then max_val=max(tmp_im)
endfor

cos_pha1=cos(float((indgen(dim0)-dim0/2)*pha1)/180.0*!pi)
sin_pha1=sin(float((indgen(dim0)-dim0/2)*pha1)/180.0*!pi)
cos_pha0=cos(float(pha0)/180.0*!pi)
sin_pha0=sin(float(pha0)/180.0*!pi)

for j=0, n_slices-1 do begin
  tmp_im=out_arr(*,*,j)
  for k=0, dim1/2-1 do tmp_im(*,2*k)=rotate(tmp_im(*,2*k), 2)
  for k=0, dim1/2-1 do begin
    line=shift(fft(shift(tmp_im(*,2*k), dim0/2), 1), dim0/2)
    re_line=float(line)*cos_pha1+imaginary(line)*sin_pha1
    im_line=-float(line)*sin_pha1+imaginary(line)*cos_pha1
    line=complex(re_line, im_line)
    tmp_im(*,2*k)=shift(fft(shift(line, dim0/2), -1), dim0/2)
  endfor
  for k=0, dim1/2-1 do begin
    line=tmp_im(*,2*k)
    re_line=float(line)*cos_pha0+imaginary(line)*sin_pha0
    im_line=-float(line)*sin_pha0+imaginary(line)*cos_pha0
    tmp_im(*,2*k)=complex(re_line, im_line)
  endfor
  tmp_im=abs(shift(fft(shift(tmp_im, dim0/2, dim1/2), 1), dim0/2, dim1/2))
  tmp_im(dim0/2, dim1/2)=(total(tmp_im(dim0/2-1:dim0/2+1,dim1/2-1:dim1/2+1))-tmp_im(dim0/2,dim1/2))/8
  tv, rebin(bytscl(tmp_im, min=0, max=max_val/scal_fact), 128, 128), j
endfor

free_lun, unit

end

;=============================================================================
pro check_dropout, fname
;=============================================================================

common correction_info, pha0, pha1, scal_fact

; first need to look at the procpar file to gain info on n_slices etc

get_lun, unit
line=string(0)

openr, unit, fname+'/procpar'
; search for np
while (not eof(unit)) do begin
  readf, unit, line
  if (strpos(line, 'np ') eq 0) then begin
    readf, unit, dummy1, np
    dim0=fix(np/2)
    print, "dim0 = ", dim0
  endif
endwhile
close, unit

openr, unit, fname+'/procpar'
; search for nv
while (not eof(unit)) do begin
  readf, unit, line
  if (strpos(line, 'nv ') eq 0) then begin
    readf, unit, dummy1, nv
    dim1=fix(nv)
    print, "dim1 = ", dim1
  endif
endwhile
close, unit

openr, unit, fname+'/procpar'
; search for num slices
while (not eof(unit)) do begin
  readf, unit, line
  if (strpos(line, 'pss ') eq 0) then begin
    readf, unit, ns
    n_slices=fix(ns)
    print, "num slices = ", n_slices
  endif
endwhile
close, unit

openr, unit, fname+'/procpar'
; search for num time points
while (not eof(unit)) do begin
  readf, unit, line
  if (strpos(line, 'arraydim ') eq 0) then begin
    readf, unit, null, arraydim
    n_t_pts=fix(arraydim)
    print, "num time points = ", n_t_pts
  endif
endwhile
close, unit

; now read the data

file_head=bytarr(32)
block_head=bytarr(28)

in_arr=intarr(2, dim0, dim1, n_slices)

openr, unit, fname+'/fid'
readu, unit, file_head

readu, unit, block_head
readu, unit, in_arr
close, unit
out_arr=reform(complex(in_arr(0,*,*,*),in_arr(1,*,*,*)))

max_val=-1
for j=0, n_slices-1 do begin
  tmp_im=out_arr(*,*,j)
  for k=0, dim1/2-1 do tmp_im(*,2*k)=rotate(tmp_im(*,2*k), 2)
  tmp_im=abs(shift(fft(shift(tmp_im, dim0/2, dim1/2), 1), dim0/2, dim1/2))
  tmp_im(dim0/2, dim1/2)=(total(tmp_im(dim0/2-1:dim0/2+1,dim1/2-1:dim1/2+1))-tmp_im(dim0/2,dim1/2))/8
  if (max(tmp_im) gt max_val) then max_val=max(tmp_im)
endfor

print, ""
print, "Using first order phase value: ", pha1
print, "Using zeroth order phase value: ", pha0
print, "Using scale factor: ", scal_fact

openr, unit, fname+'/fid'
readu, unit, file_head

cos_pha1=cos(float((indgen(dim0)-dim0/2)*pha1)/180.0*!pi)
sin_pha1=sin(float((indgen(dim0)-dim0/2)*pha1)/180.0*!pi)
cos_pha0=cos(float(pha0)/180.0*!pi)
sin_pha0=sin(float(pha0)/180.0*!pi)

tmp_ss=fltarr(n_slices)
ss=fltarr(n_slices)
tmp_im_arr=fltarr(dim0, dim1, n_slices)
running_av=fltarr(n_slices,2048)
noise_thresh=5.0   ; noise threshold (%)

i=0
jump=3
!err=0
while (not eof(unit)) do begin
  readu, unit, block_head
  readu, unit, in_arr
  i=i+1
  out_arr=reform(complex(in_arr(0,*,*,*),in_arr(1,*,*,*)))
  print, "time point ", strtrim(string(i), 2)
  xyouts, 200, 20, "!17time point "+strtrim(string(i), 2), size=3, /device
  for j=0, n_slices-1 do begin
    tmp_im=out_arr(*,*,j)
    for k=0, dim1/2-1 do tmp_im(*,2*k)=rotate(tmp_im(*,2*k), 2)
    for k=0, dim1/2-1 do begin
      line=shift(fft(shift(tmp_im(*,2*k), dim0/2), 1), dim0/2)
      re_line=float(line)*cos_pha1+imaginary(line)*sin_pha1
      im_line=-float(line)*sin_pha1+imaginary(line)*cos_pha1
      line=complex(re_line, im_line)
      tmp_im(*,2*k)=shift(fft(shift(line, dim0/2), -1), dim0/2)
    endfor
    for k=0, dim1/2-1 do begin
      line=tmp_im(*,2*k)
      re_line=float(line)*cos_pha0+imaginary(line)*sin_pha0
      im_line=-float(line)*sin_pha0+imaginary(line)*cos_pha0
      tmp_im(*,2*k)=complex(re_line, im_line)
    endfor
    tmp_im=abs(shift(fft(shift(tmp_im, dim0/2, dim1/2), 1), dim0/2, dim1/2))
    tmp_im(dim0/2, dim1/2)=(total(tmp_im(dim0/2-1:dim0/2+1,dim1/2-1:dim1/2+1))-tmp_im(dim0/2,dim1/2))/8
    tv, rebin(bytscl(tmp_im, min=0, max=max_val/scal_fact), 128, 128), j
    mask=where(tmp_im gt noise_thresh/100.0*max_val)
    if (total(mask) gt -1) then running_av(j,i)=total(tmp_im)
    if (i eq jump) then tmp_im_arr(*,*,j)=tmp_im
    if (i gt jump) then begin
      tmp_ss(j)=total((tmp_im-tmp_im_arr(*,*,j))^2)/(dim0*dim1)
    endif
  endfor
  if (i gt jump) then ss=[[ss],[tmp_ss]]
  if (!err ne 4) then cursor,x,y,/NOWAIT,/dev
  if (!err eq 4) then begin
    !err=0
    cursor,x,y,/DOWN,/dev
    if (!err eq 2) then begin
      adjct
      !err=4
    endif
  endif
  if (!err eq 2) then begin
    goto, break_out
  endif
  xyouts, 200, 20, "!17time point "+strtrim(string(i), 2), size=3, /device, color=0
endwhile

running_av=running_av(*,1:i)

break_out:

n_tpts=i

close, unit
free_lun, unit

; post processing on the median info
if (n_tpts gt jump) then begin

  med_ss=tmp_ss
  for i=0, n_slices-1 do begin
    med_ss(i)=median(ss(i,*))
  endfor
  
  dropout_mask=intarr(n_slices, n_tpts)
  thresh_val=20
  for i=jump, n_tpts-1 do begin
    for j=0, n_slices-1 do begin
      if (ss(j,i-jump)/med_ss(j) gt thresh_val) then begin
        dropout_mask(j, i)=1
      endif else begin
        dropout_mask(j, i)=0
      endelse
    endfor
  endfor

  for i=0, n_tpts-1 do begin
   print, i+1, dropout_mask(*,i)
  endfor

  print, ""
  print, "Total bad slices = ", total(dropout_mask)

  print, ''
  for i=0, n_slices-1 do begin
    total_av=total(running_av(i,jump:n_tpts-1))/(n_tpts-jump)
    running_av(i,*)=running_av(i,*)/total_av*100.0
    plot, running_av(i,jump:n_tpts-1), $
         yrange=[95,105]
         title='slice '+strtrim(string(i+1),2)
    wait, 1
    stdev=total((running_av(i,jump:n_tpts-1)-100.0)^2)/(n_tpts-jump-1)
    print, 'Slice '+strtrim(string(i+1),2)+ $
        ' stdev =',sqrt(stdev), ', max = ', $
        max(running_av(i,jump:n_tpts-1))-100.0, $
        ', min = ', min(running_av(i,jump:n_tpts-1))-100.0
; stop 
  debug_mode=0
  if (debug_mode eq 1) then begin
    if (i eq 9) then begin
      set_plot, 'PS'
      plot, running_av(i,jump:n_tpts-1), $
         yrange=[95,105]
         title='slice '+strtrim(string(i+1),2)
      device, /close
      set_plot, 'X'
    endif
  endif
  endfor

  running_av=running_av(*,jump:n_tpts-1)
  save, filename="~/graphs.idl", running_av
endif

end

;=============================================================================
function attempt_read, fname
;=============================================================================

on_ioerror, cant_read_files
on_error, 2

get_lun, unit
openr, unit, fname
close, unit
free_lun, unit

return, 1

cant_read_files:
return, -1

end


;=============================================================================
PRO load_win_event, event
;=============================================================================
; This is the event handler for a text widget with a label.

; Get the widget ID of any widget touched and put it into 'eventval'.
; Pressing the RETURN key inside a text widget generates an event:

common file_info, directory, filename
common load_bases, load_base
common misc_flags, file_checked_flag
common correction_info, pha0, pha1, scal_fact

WIDGET_CONTROL, event.id, GET_UVALUE = eventval

; Perform different actions based upon 'eventval':

temp_var=string(0)

CASE eventval OF
	"DIRECTORY": BEGIN
		  ;A RETURN has been pressed in the Directory field
                  WIDGET_CONTROL, event.id, GET_VALUE=temp_var
                  directory=temp_var(0)
                  file_checked_flag=0
		  END

	"FILENAME": BEGIN
		  ;A RETURN has been pressed in the Filename field
                  WIDGET_CONTROL, event.id, GET_VALUE=temp_var
                  filename=temp_var(0)
                  file_checked_flag=0
		  END

	"FIRST_PHA": BEGIN
		  ;A RETURN has been pressed in the First Order Phase field
                  WIDGET_CONTROL, event.id, GET_VALUE=temp_var
                  pha1=float(temp_var(0))
                  load_ok_flag=0
                  ; check to see if file is there
                  fname=directory+'/'+filename
                  if (attempt_read(fname+'/fid') eq 1) then begin
                    load_ok_flag=1
                  endif else begin
                    load_ok_flag=0
                    print, 'Error : File '+fname+'/fid not found'
                  endelse
                  if (load_ok_flag eq 1) then begin
                    print, "previewing data..."
                    file_checked_flag=1
                    tmp_scal_fact=scal_fact
                    scal_fact=scal_fact*2.0
                    preview_dropout, fname
                    scal_fact=tmp_scal_fact
                  endif
		  END

	"ZERO_PHA": BEGIN
		  ;A RETURN has been pressed in the Zero Order Phase field
                  WIDGET_CONTROL, event.id, GET_VALUE=temp_var
                  pha0=float(temp_var(0))
                  load_ok_flag=0
                  ; check to see if file is there
                  fname=directory+'/'+filename
                  if (attempt_read(fname+'/fid') eq 1) then begin
                    load_ok_flag=1
                  endif else begin
                    load_ok_flag=0
                    print, 'Error : File '+fname+'/fid not found'
                  endelse
                  if (load_ok_flag eq 1) then begin
                    print, "previewing data..."
                    file_checked_flag=1
                    tmp_scal_fact=scal_fact
                    scal_fact=scal_fact*2.0
                    preview_dropout, fname
                    scal_fact=tmp_scal_fact
                  endif
		  END

	"SCAL": BEGIN ;A RETURN has been pressed in the Scal Fact field
                  WIDGET_CONTROL, event.id, GET_VALUE=temp_var
                  scal_fact=float(temp_var(0))
                  load_ok_flag=0
                  ; check to see if file is there
                  fname=directory+'/'+filename
                  if (attempt_read(fname+'/fid') eq 1) then begin
                    load_ok_flag=1
                  endif else begin
                    load_ok_flag=0
                    print, 'Error : File '+fname+'/fid not found'
                  endelse
                  if (load_ok_flag eq 1) then begin
                    print, "previewing data..."
                    file_checked_flag=1
                    preview_dropout, fname
                  endif
		  END

	"PREVIEW": BEGIN
                  WIDGET_CONTROL, load_base, SENSITIVE=0
                  load_ok_flag=0
                  ; check to see if file is there
                  fname=directory+'/'+filename
                  if (attempt_read(fname+'/fid') eq 1) then begin
                    load_ok_flag=1
                  endif else begin
                    load_ok_flag=0
                    print, 'Error : File '+fname+'/fid not found'
                  endelse
                  if (load_ok_flag eq 1) then begin
                    print, "previewing data..."
                    file_checked_flag=1
                    preview_dropout, fname
                  endif
                  WIDGET_CONTROL, load_base, SENSITIVE=1
                  END 

	"EXECUTE": BEGIN
                  WIDGET_CONTROL, load_base, SENSITIVE=0
                  load_ok_flag=0
                  ; check to see if file is there
                  fname=directory+'/'+filename
                  if (attempt_read(fname+'/fid') eq 1) then begin
                    load_ok_flag=1
                  endif else begin
                    load_ok_flag=0
                    print, 'Error : File '+fname+'/fid not found'
                  endelse
                  if (load_ok_flag eq 1) then begin
                    print, "previewing data..."
                    file_checked_flag=1
                    erase
                    check_dropout, fname
                  endif
                  WIDGET_CONTROL, load_base, SENSITIVE=1
                  END 

	"CANCEL": BEGIN
                  WIDGET_CONTROL, load_base, /DESTROY
                  END 

ENDCASE

END

;=============================================================================
PRO load_win, GROUP = GROUP
;=============================================================================

common file_info, directory, filename
common misc_flags, file_checked_flag
common load_bases, load_base
common correction_info, pha0, pha1, scal_fact

window, title='CHECK DROPOUT'

file_checked_flag=1
pha0=0.0
pha1=0.0
scal_fact=1.0

load_base = WIDGET_BASE(TITLE = 'LOAD MENU', /COLUMN, XOFFSET=0, YOFFSET=0)

; Make a ROW base widget so the label and text field can be
; side by side:

directory='/tmp'
directory='/scratch/stuart1/001438_1999_10_29'
filename='series_2.1.fid'

row1 = WIDGET_BASE(load_base, /ROW, /FRAME)
namelabel1 = WIDGET_LABEL(row1, VALUE = 'Directory:')
name1 = WIDGET_TEXT(row1, /EDITABLE, XSIZE=40, YSIZE = 1, $
		   UVALUE = 'DIRECTORY', VALUE = directory)

row2 = WIDGET_BASE(load_base, /ROW, /FRAME)
namelabel2 = WIDGET_LABEL(row2, VALUE = 'Filename:')
name2 = WIDGET_TEXT(row2, /EDITABLE, XSIZE = 40, YSIZE = 1, $
		   UVALUE = 'FILENAME', VALUE = filename)

row3 = WIDGET_BASE(load_base, /ROW, /FRAME)
namelabel3 = WIDGET_LABEL(row3, VALUE = 'First Order Phase Correction:')
name3 = WIDGET_TEXT(row3, /EDITABLE, XSIZE = 20, YSIZE = 1, $
	   UVALUE = 'FIRST_PHA', VALUE = strtrim(string(pha1),2))

row4 = WIDGET_BASE(load_base, /ROW, /FRAME)
namelabel4 = WIDGET_LABEL(row4, VALUE = 'Zeroth Order Phase Correction:')
name4 = WIDGET_TEXT(row4, /EDITABLE, XSIZE = 20, YSIZE = 1, $
	   UVALUE = 'ZERO_PHA', VALUE = strtrim(string(pha0),2))

row5 = WIDGET_BASE(load_base, /ROW, /FRAME)
namelabel5 = WIDGET_LABEL(row5, VALUE = 'Scaling Factor:')
name5 = WIDGET_TEXT(row5, /EDITABLE, YSIZE = 1, $
	   UVALUE = 'SCAL', VALUE = strtrim(string(scal_fact),2))

prev_button=WIDGET_BUTTON(load_base, UVALUE="PREVIEW", VALUE="preview")

exec_button=WIDGET_BUTTON(load_base, UVALUE="EXECUTE", VALUE="execute")

canc_button=WIDGET_BUTTON(load_base, UVALUE="CANCEL", VALUE="cancel")

; Realize the widgets:
WIDGET_CONTROL, load_base, /REALIZE

; Hand off to the XMANAGER:
XMANAGER, 'load_win', load_base

END

;=============================================================================
pro dropout
;=============================================================================

load_win

end
