/***********************************************************************
    OX_EPI.C

    Blipped interleaved gradient-echo echo-planar imaging sequence.
    Peter Jezzard, NIH July-Dec 1997, Oxford Jan 1998 -

    adapted from mp_bepi.c from the University of Western Ontario, London
    added shaped waveforms to all ramped gradients, Aug 1997

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
     _                               __         __           _
    |  tpwr1       |  tpwr2  |      |             |           |
 rf | --oOo------------oOo----------|-|ACQ|-|ACQ|-|-----------|
    |              |         |      |             |           |
    |                               |             |<--------->|
    |   ____       |  _____  |      |             | postdelay |
GSS | _/////\     ___//////\________|_____________|___________|
    |        \\\\/ |  gssr   |      |             |           |
    |        <--> tref              |             | etl/2     | ns
    |              |     tref2 <--->|             |           |
    |                               |             |           |
GPE | _____________|_________|_ gbd |/\____/\_____|___________|
    |                          \\\\/|             |           |
    |              |         |      |             |           |
    |                               |  ____       |           |
GRO | _____________|_________|_gror |_/////\      |___________|
    |                          \\\\/|       \\\\\/|           |
    |_             |         |      |__         __|          _|
                       ^
                       |
                if spinecho='y'

  |  __    __    __    __     postdelay
  |_|  |  |  |  |  |  |  |__________________
  |    |__|  |__|  |__|
  | ^                                      |
    |______________________________________|
     multi slice loop (ns)
  ^
  |_________________________________________|
            interleave loop (nv/etl)


	Bug fix: March 31st 1999: added correct position_offset_list 
                statements to inversion, excitation and refocussing
		pulse slice selection. Previously was not selecting properly

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

#include "standard.h"
#include "group.h"
extern double rint();
#include "sequence_utilities.h"

/*==========================================================================*
  define the pulse sequence variables outside the pulse sequence so that
  they can be seen as global variables by the safety monitoring code
*==========================================================================*/
    int     image=0, num_ints;
    double  length,etl,postdelay,seqtime,tref,echo_del,tep;
    double  trefmin,tau1,tau1_bef,tau1_aft,tau2,ted1,ted2;
    double  agro,agss,gssi,agcrush,grate,gssrint,grorint,gpeint,gbdint,maxgint;
    double  pw_gro1, pw_gss, pw_gss_d, pw_gss2, pw_gssi, pw_gssr, pw_end_gcrush;
    double  gss2, extra_te_error;
    double  trefmin2,risetref2,tref2,etl_delay,nucgamma,gro1,grise;
    double  del_seq,minte,eff_echo,tbpe,sgbpe,gbd,sgbd,gbdf,epigroa;
    char    phase_cycle_flag[MAXSTR],shape_name[MAXSTR],spinecho[MAXSTR];
    double  episwing, epigrof;
    int     prescan_flag, num_segs;
    int     curlist,invlist,exlist,selist;
    char    err_string[MAXSTR],gcoil[MAXSTR];
    /* fat sat stuff */
    double  pw_gfatk, tpwrfat, pfat, total_fat_sat_time;
    char fat_sat[MAXSTR], pfatpat[MAXSTR], half_kspace[MAXSTR];


#include "ox_epi_safety_calc.h"

/* Spatial Sat declarations */
double gsssat,pw_gsssat;
double presat_off=0.0,post_presat_del,sat_thk,sat_psi,sat_phi,sat_theta;
double presat=0.0,pw_gcrush;
double t_presat_b,t_presat_a;

calculate_spatial_saturation_gradients(double *t_presat_b,double *t_presat_a)
{
  char shape_name[MAXSTR];

  /* PJPJ */
  sprintf(shape_name,"gsat"); 
  /* 
  sprintf(shape_name,"gsssat");
  */
  pw_gsssat = create_trap_gradient(fabs(gsssat),shape_name,
         psat+rof1+rof2,grate,ix);

  *t_presat_b = fabs(gsssat)*grate + rof1 + psat/2;
  *t_presat_a = fabs(gsssat)*grate + rof2 + psat/2 + post_presat_del;
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
    /* PJPJ */
    sprintf(shape_name,"gcr");
    /* 
    sprintf(shape_name,"gcrusher");
    */
    pw_gcrush = create_ramp_gradient(fabs(gcrush),shape_name,1,grate,ix);
    pw_gcrush = create_ramp_gradient(fabs(gcrush),shape_name,-1,grate,ix);
    if (pw_gcrush > tcrush) {
      sprintf(err_string,"%s: tcrush too small. Min tcrush = %f\n",
              seqfil,pw_gcrush);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
    *t_presat_a=*t_presat_a + 2.0*pw_gcrush + tcrush;
  }
}

spatial_saturation_pulse()
{
  /* uses psat : satpwr : satpat : gsssat */
  /* also uses sat_psi, sat_phi and sat_theta rather than psi, phi, theta */
  /* Enables definition of independent sat planes relative to slice plane */
  position_offset(presat_off,gsssat,resto,OBSch);
  obspower(satpwr);

  oblique_shapedgradient("","","gsat",pw_gsssat,0.0,0.0,gsssat,sat_psi,sat_phi
,sat_theta,1,NOWAIT);

  delay(fabs(gsssat)*grate+tep);
  sp4on();
  shaped_pulse(satpat,psat,v1,rof1,rof2);
  sp4off();
  if(tep<fabs(gsssat)*grate)delay(fabs(gsssat)*grate-tep);

  /* Crushers */
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
/* PJPJ */
    obl_shapedgradient("gcr_up","gcr_up","gcr_up",
                       pw_gcrush,gcrush,gcrush,gcrush,1,WAIT);
    delay(tcrush);
    obl_shapedgradient("gcr_dn","gcr_dn","gcr_dn",
                       pw_gcrush,gcrush,gcrush,gcrush,1,WAIT);
/*
    obl_shapedgradient("gcrusher_up","gcrusher_up","gcrusher_up",
                       pw_gcrush,gcrush,gcrush,gcrush,1,WAIT);
    delay(tcrush);
    obl_shapedgradient("gcrusher_dn","gcrusher_dn","gcrusher_dn",
                       pw_gcrush,gcrush,gcrush,gcrush,1,WAIT);
*/
  }
  delay(post_presat_del);
}

pulsesequence()
{

    /* Note that the sequence_utility definitions don't like being in the
       global field, so leave them in the pulse sequence code */
    double  create_ramp_gradient(), create_trap_gradient();
    double  create_epi_read_gradients(), create_epi_phase_gradients();
    initparms_sis();
nv = 64.0;
np = 128.0;
at = 640e-6;

    /*************************************************************************/
    /* get the global parameters */
    /*************************************************************************/
    P_getreal(GLOBAL,"grise",&grise,1); /* grad risetime to full strength */
    if ((grise < 0.0) || (grise > 0.002)) {
      sprintf(err_string,"%s: impropper grise time defined in global file\n",
                       seqfil);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
    P_getreal(GLOBAL,"tep",&tep,1); /* small tweaker time (us) to allow for
                            the gradient propagation time, causes echoes to
                            appear symmetrically about echo center line */
    tep=tep/1e6;
    epigroa = 0.0; /* default */
    P_getreal(GLOBAL,"epigroa",&epigroa,1); /* read dephase tweaker (percent) */
    epigrof = 0.0; /* default */
    P_getreal(GLOBAL,"epigrof",&epigrof,1); /* grad risetime to full strength */
    episwing = 0.0; /* default */
    P_getreal(GLOBAL,"episwing",&episwing,1); /* allows extra time to be
                                  inserted into the length of each flat top */
    episwing=episwing /1.0e6;  /* convert from us */
    /*************************************************************************/

    del_seq = getval("del_seq"); /* short delay at start of each overall AQ */
    gssi = getval("gssi"); /* inversion recovery gradient strength */
    gss2 = getval("gss2"); /* 180 degree pulse gradient strength */
    prescan_flag = getval("prescan_flag"); /* prescan flag (1=yes, 0=no) */
    gbdf = getval("gbdf"); /* phase encode gradient tweaker (default=1.0) */
    num_ints = getval("num_ints");  /* number of interleaves */
    image = (int)(getval("image")); /* trigger and counter bit pattern */
    curlist=-1; /* general position offset list variable for slice selects */
    invlist=-1; /* position offset list variable for inversion select */
    exlist=-1;  /* position offset list variable for excitation select */
    selist=-1;  /* position offset list variable for spin echo select */
    /* bit 1 = wait for ticks triggers before beginning scan */
    /* bit 2 = puts out high on sp3 for duration of volume */
    /* bit 3 = epi blips off (set bit on), otherwise blips on */
    /* bit 4 = make inversion non slice selective (on) or slice 
               selective (off) (only operates when ti > 0.0) */

    getstr("spinecho",spinecho);  /* spinecho flag 'y'=se. 'n'=ge. */

    /* fat sat stuff */
    getstr("fat_sat",fat_sat);    /* fat saturation flag 'y'=on. 'n'=off. */
    getstr("half_kspace",half_kspace); /* half k-spaceflag 'y'=on. 'n'=off */
    getstr("pfatpat",pfatpat);    /* modulation pattern for fat sat pulse */
    getstr("gcoil",gcoil);        /* gradient coil in use */
    tpwrfat = getval("tpwrfat");  /* fat sat power in dB */
    pfat = getval("pfat");        /* duration of fat sat pulse in us */

    echo_del = getval("echo_del"); /* small tweaker time (us) to compensate
                                for situations where the phase encode blip 
                                needs a longer duration than grate*agro */
    echo_del=echo_del/1e6;
    grate = grise/gmax; /* switching rate of gradients [sec/(gauss/cm)] */
    gcrush=gmax/3.0;
    agcrush = fabs(gcrush); /* absolute crusher gradient strength */
    getstr("phase_cycle_flag",phase_cycle_flag); /* 'y' or 'n' */
    nucgamma = 1e6*sfrq/B0;  /* nucleus gamma in Hz/gauss */
                             /* derived from sfrq in MHz and B0 in gauss */

           /* Spatial Sat Variables */
    presat=getval("presat");
    if (presat) {
      gsssat = getval("gsssat");
      presat_off=getval("presat_off");
      sat_psi=getval("sat_psi");
      sat_phi=getval("sat_phi");
      sat_theta=getval("sat_theta");
      sat_thk=getval("sat_thk");
      post_presat_del=getval("post_presat_del");
    }
    if (presat) calculate_spatial_saturation_gradients(&t_presat_b,&t_presat_a);

    if (num_ints == 0) num_ints = 1;  /* num of interleaves = 1 as default */

    if ((half_kspace[0] == 'y') || (half_kspace[0] == 'Y')) {
      if (num_ints > 1) {
        sprintf(err_string,
              "%s: Half K space acquisition requires 1 interleaf\n",seqfil);
        printf(err_string);
        log_error(err_string);
	abort(1);
      }
      eff_echo = nv/2.0;  /* echo position of TE */
      if (nv == 36.0) {
        eff_echo = 4.0;  /* echo position of TE */
      } 
      if (nv == 72.0) {
        eff_echo = 8.0;  /* echo position of TE */
      } 
      if (nv == 144.0) {
        eff_echo = 16.0; /* echo position of TE */
      }
      etl = nv;
    } else {
      etl = (double)((int)(nv/num_ints));  /* echo train length/interleaf */
      eff_echo = etl/2.0;  /* echo position of TE */
    }

   /* if alternating, first echo will be TE
   otherwise linear phase encoding steps, middle of echo trains */

    /*====================================================================*/
    /* read gradient calculations                                         */
    /*====================================================================*/

    gro1 = np/2.0/at/lro/nucgamma;  /* note read gradient calculated here */

    agro = fabs(gro1); /* absolute read out gradient strength */

    sprintf(shape_name,"gro1");
    pw_gro1=create_ramp_gradient(fabs(gro1),shape_name,1,grate,ix);
    pw_gro1=create_ramp_gradient(fabs(gro1),shape_name,-1,grate,ix);

    /* PJPJ */
    sprintf(shape_name,"epird");
    /*
    sprintf(shape_name,"epi_read");
    */
    pw_gro1=create_epi_read_gradients(fabs(gro1),shape_name,grate,1.0/sw,
        (int)np/2,(int)etl,epigroa,echo_del,ix);
    /* PJPJ */
    sprintf(shape_name,"epiph");
    /*
    sprintf(shape_name,"epi_phase");
    */
    pw_gro1=create_epi_phase_gradients(shape_name,pw_gro1,1.0/sw,
        (int)np/2,(int)etl,echo_del,ix);

    grorint = agro*(at/2.0 + pw_gro1/2.0);  /* necessary gror intergral */


    /*====================================================================*/
    /* phase encode gradient calculations                                 */
    /*====================================================================*/

    /* fat sat stuff */
    sprintf(shape_name,"gfatk"); /* 3ms fat sat killer gradient pulse */
    pw_gfatk=create_trap_gradient(fabs(gmax)/2.0,shape_name,3.0e-3,grate,ix);
    total_fat_sat_time=pfat + pw_gfatk;

    gpeint = B0 * (double)num_ints / (lpe*sfrq*1e6); /* int num_ints blips */
    gbdint = gpeint * eff_echo; /* phase encode dephaser g-t integral */

    /*====================================================================*/
    /* slice gradient calculations                                        */
    /*====================================================================*/

    agss = fabs(gss); /* absolute slice select gradient strength */
    if (agss == 0.0) {
        sprintf(err_string,"%s: Zero slice select gradient\n",seqfil);
        printf(err_string);
        log_error(err_string);
	abort(1);
    }

    sprintf(shape_name,"gss");
    pw_gss=create_trap_gradient(agss,shape_name,rof1+p1+rof2,grate,ix);

    sprintf(shape_name,"gss2");
    pw_gss2=create_trap_gradient(fabs(gss2),shape_name, 
               rof1+p2+rof2+2.0*tspoil,grate,ix);

    sprintf(shape_name,"gssigd");
    pw_gssi=create_ramp_gradient(fabs(gssi),shape_name,1,grate,ix);
    pw_gssi=create_ramp_gradient(fabs(gssi),shape_name,-1,grate,ix);

    gssrint = agss*(p1/2.0 + rof2 + agss*grate/2.0); /* nec. gssr integral */

    if ( !var_active("gssf",CURRENT) )
      gssf = 0.0;

    /* if the gssrint can be done as a triangular waveform then set it up */
    if (grise*gmax >= gssrint*(1.0+gssf/100.0)) {
	trefmin = sqrt(grate*gssrint*(1.0+gssf/100.0)); 
                       /* since g_required=trefmin/grate */
    }
    /* otherwise set up waveform as a trapezoidal shape and hit the rail */
    else {
	trefmin = grate*gmax + gssrint*(1.0+gssf/100.0)/gmax;
    }

  /* TREF IS THE REFOCUSSING BLOCK LENGTH, NOT INCLUDING GRADIENT FALL TIME */
    tref = trefmin;

    gssr = -gssrint*(1.0+gssf/100.0)/tref;

    sprintf(shape_name,"gssrgd");
    pw_gssr=create_ramp_gradient(fabs(gssr),shape_name,1,grate,ix);
    pw_gssr=create_ramp_gradient(fabs(gssr),shape_name,-1,grate,ix);

    if (tref < pw_gssr) tref=pw_gssr;

    /*====================================================================*/
    /* coincident gradient calculations                                   */
    /*====================================================================*/

  /* MINIMUM REFOCUSSING BLOCK TIME FOR READOUT AND PHASE REWINDER */
    /* figure out the bigger of gbdint and grorint  giving the maxgint */
    maxgint = grorint*(1.0+epigrof/100.0) > gbdint*(1.0+gbdf/100.0) ?
       grorint*(1.0+epigrof/100.0) : gbdint*(1.0+gbdf/100.0);

    if (grise*gmax >= maxgint) {
	trefmin2 = sqrt(grate*maxgint);
	risetref2 = trefmin2;
    }
    else {
	trefmin2 = grate*gmax + maxgint/gmax;
	risetref2 = grise;
    }

  /* CREATE TREF2 FOR READOUT AND PHASE ENCODE REWINDER */
    tref2 = trefmin2;

    /* make sure that risetref2 is not less than MAX_GRAD_RES*GRAD_UPDATE_TIME
       and if it is then generate a file that is close to within the nearest
       integral of GRAD_UPDATE_TIME */
    /* PJPJ */
    sprintf(shape_name,"gdp");
    /*
    sprintf(shape_name,"gdephase");
    */
    risetref2=create_ramp_gradient(maxgint/tref2,shape_name,1,grate,ix);
    risetref2=create_ramp_gradient(maxgint/tref2,shape_name,-1,grate,ix);

    if (tref2 < risetref2) tref2=risetref2;
    sgbd = 0.0; /* default */

    tbpe = pw_gro1+echo_del;
    sgbpe = gpeint/tbpe;
    sgbpe = gmax/gradstepsz*floor(sgbpe/gmax*gradstepsz+0.5);
    gbd = -gbdint*(1.0+gbdf/100.0)/tref2;
    gbd = gmax/gradstepsz*floor(gbd/gmax*gradstepsz+0.5);
    sgbd = (sgbpe*tbpe)/((double) num_ints)/tref2;
    sgbd = gmax/gradstepsz*floor(sgbd/gmax*gradstepsz+0.5);

    if (grate*fabs(sgbpe) > pw_gro1) {
        sprintf(err_string,"%s: Blips error. Increase sw, lpe or echo_del\n",
                         seqfil);
        printf(err_string);
        log_error(err_string);
	abort(1);
    }

    pw_end_gcrush=0.0;
    if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
      /* PJPJ */
      sprintf(shape_name,"e_gcr");
      /* 
      sprintf(shape_name,"end_gcrusher");
      */
      pw_end_gcrush=create_ramp_gradient(fabs(gcrush),shape_name,1,grate,ix);
      pw_end_gcrush=create_ramp_gradient(fabs(gcrush),shape_name,-1,grate,ix);
      if (pw_end_gcrush > tcrush) {
        sprintf(err_string,"%s: tcrush too small. Min tcrush = %f\n",
                         seqfil,pw_end_gcrush);
        printf(err_string);
        log_error(err_string);
        abort(1);
      }
    }

    /*====================================================================*/
    /*====================================================================*/

    /* account for case of tep being longer than the gradient ramp time */
    pw_gss_d = agss*grate - tep;
    extra_te_error = 0.0;
    if (pw_gss_d < 0.0) {
      pw_gss_d = 0.0;
      extra_te_error = tep-agss*grate;
    }

    if (agss*grate+tspoil < tep) {
        sprintf(err_string,"%s: tspoil too short. Min tspoil = %f\n",
                         seqfil, tep-agss*grate);
        printf(err_string);
        log_error(err_string);
	abort(1);
    }

  /* CALCULATE TAU1 - MINIMUM TIME BETWEEN ALPHA AND FIRST ECHO (NAV)
     & CALCULATE TAU2 - MINIMUM TIME BETWEEN ECHOES */

    ted1=0.0; /* default spin echo filler before 180 */
    ted2=0.0; /* default spin echo filler after 180 */
    tau1_bef = p1/2.0 + rof2 + agss*grate + tref + pw_gssr + extra_te_error;
    tau1_aft = tref2 + risetref2 + pw_gro1 + episwing + at/2.0;
    tau2 = pw_gro1 + episwing + at + pw_gro1 + echo_del; 
    if (spinecho[0] == 'y')
    {
      if (tau1_bef < (tau1_aft+eff_echo*tau2))
        ted1=(tau1_aft+eff_echo*tau2)-tau1_bef;
      if (tau1_bef > (tau1_aft+eff_echo*tau2))
        ted2=tau1_bef-(tau1_aft+eff_echo*tau2);
    }
    tau1=tau1_bef+ted1+tau1_aft+ted2;
    if (spinecho[0] == 'y')
      tau1=tau1 + agss*grate + tspoil + rof1 + p2 + rof2 + tspoil + agss*grate;

  /* CALCULATE GSSR, GBD AND GROR WITH NEW TREF */
    gror = -grorint*(1.0+epigrof/100.0)/tref2;
    if (ix == 1) { /* print info on first array pass (when ix=1) */
      fprintf(stderr,"gror: %7.4f    gssr: %7.4f\n",gror,gssr);
      fprintf(stderr,"gro1: %8.6f inter_echo: %8.6f tgro1: %8.6f\n",
                      gro1,tau2,pw_gro1);
    }

  /* CALCULATE MINIMUM TE AND DELAY TO HAVE FLEXIBLE TE */
    minte = tau1 + eff_echo * tau2;
    etl_delay = te - minte;

    if (te < minte) {
        sprintf(err_string,"%s: TE too short. Min TE = %f\n",seqfil,minte);
        printf(err_string);
        log_error(err_string);
	abort(1);
    }

  /* CALCULATE RELAXATION DELAY FOR SEQUENCE */
    seqtime = ( agss*grate + rof1 + p1/2.0 + te + tep +
		  (etl-eff_echo-1.0) * tau2 + at/2.0 + pw_gro1 +
		  tcrush + pw_end_gcrush );
    if (ti > 0.0) {
      seqtime = seqtime + ti + 2.0*p3 + 2.0*pw_gssi + rof1 + rof2;
    }
    if (fat_sat[0] == 'y') {
      seqtime = seqtime + total_fat_sat_time;
    }
    if (presat) seqtime = seqtime + t_presat_b + t_presat_a;
    seqtime = seqtime * ns;
    if (tr - seqtime < 0.0) {
          sprintf(err_string,"%s: TR too short. Min TR = %f\n",seqfil,seqtime);
          printf(err_string);
          log_error(err_string);
          abort(1);
    }
    if ((ti - (tcrush+pw_end_gcrush) < 0.0) && (ti > 0.0)) {
          sprintf(err_string,"%s: TI too short. Min TI = %f\n",
               seqfil,tcrush+pw_end_gcrush);
          printf(err_string);
          log_error(err_string);
          abort(1);
    }
    postdelay = (tr - seqtime)/ns;

    if (prescan_flag == 1) {
      num_segs=1;
    } else {
      num_segs=(int)(nv/etl);
    }

    if ((image & 4) == 4)
    {
      sgbpe = 0.0;
      gbd=0.0;
      sgbd=0.0;
    }

    if (ox_epi_safety_calc() == -1) {
      sprintf(err_string,"%s: Failed duty cycle safety check\n",seqfil);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }

  /* PULSE SEQUENCE *************************************/
    /* Phase cycle: Part I ********************************/
    mod2(ct,v3);                    /* v3 = 0101 0101 */
    dbl(v3,v3);                     /* v3 = 0202 0202 */
    hlv(ct,v4);                     /* v4 = 0011 2233 */
    mod2(v4,v4);                    /* v4 = 0011 0011 */
    add(v3,v4,v1);                  /* v1 = 0213 0213 */
    assign(v1,oph);                 /* oph = 0213 0213 */
    add(one,v1,v2);

    assign(one,v7);  /* phase encode increment size */
    status(A);
    sp4off();
    rcvroff();
    rcvron();

    initval(num_segs,v8);  /* set v8 to the num of echoes/shot */
    assign(zero,v12);

    if ((image & 1) == 1) xgate(ticks);  /* wait for start scan button */
    if ((image & 2) == 2) sp3on(); else sp3off();

      sp2off();

      delay(del_seq);

      loop(v8,v9);  /* loop v8 (num_ints) times using counter v9 */
        msloop(seqcon[1],ns,v13,v14); /* sets v13 to ns and incs v14 up to it */

        /* 180 degree inversion pulse */
        if (ti > 0.0) {
          /* selective inversion */
          if ((image & 8) == 0) {
            curlist=curlist+1;
            invlist=curlist;
            position_offset_list(pss,gssi,ns,resto,OBSch,invlist,v14); 
                                   /* set the frequency offset to
                                   the v14th element in the pss array */
            obspower(tpwr3);
            obl_shapedgradient("","","gssigd_up",pw_gssi,0.0,0.0,gssi,1,WAIT);
            sp4on();
            shapedpulse(p3pat,p3,v1,rof1,rof2);
            sp4off();
            obl_shapedgradient("","","gssigd_dn",pw_gssi,0.0,0.0,gssi,1,WAIT);
            delay(p3);
            if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
/* PJPJ */
              obl_shapedgradient("e_gcr_up","e_gcr_up",
                 "e_gcr_up",pw_end_gcrush,gcrush,gcrush,gcrush,1,WAIT);
	      delay(tcrush-pw_end_gcrush);
              obl_shapedgradient("e_gcr_dn","e_gcr_dn",
                 "e_gcr_dn",pw_end_gcrush,gcrush,gcrush,gcrush,1,WAIT);
/*
              obl_shapedgradient("end_gcrusher_up","end_gcrusher_up",
                 "end_gcrusher_up",pw_end_gcrush,gcrush,gcrush,gcrush,1,WAIT);
	      delay(tcrush-pw_end_gcrush);
              obl_shapedgradient("end_gcrusher_dn","end_gcrusher_dn",
                 "end_gcrusher_dn",pw_end_gcrush,gcrush,gcrush,gcrush,1,WAIT);
*/
              delay(ti-tcrush-pw_end_gcrush);
            } else {
              delay(ti);
            }
          } 
          /* non selective inversion */
          if ((image & 8) == 8) {
            poffset(0.0,0.0);
            obspower(tpwr3);
            sp4on();
            shapedpulse(p3pat,p3,v1,rof1,rof2);
            sp4off();
            obl_shapedgradient("","","gssigd_up",pw_gssi,0.0,0.0,gssi,1,WAIT);
            delay(p3);
            obl_shapedgradient("","","gssigd_dn",pw_gssi,0.0,0.0,gssi,1,WAIT);
            if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
/* PJPJ */
              obl_shapedgradient("e_gcr_up","e_gcr_up",
                 "e_gcr_up",pw_end_gcrush,gcrush,gcrush,gcrush,1,WAIT);
	      delay(tcrush-pw_end_gcrush);
              obl_shapedgradient("e_gcr_dn","e_gcr_dn",
                 "e_gcr_dn",pw_end_gcrush,gcrush,gcrush,gcrush,1,WAIT);
/*
              obl_shapedgradient("end_gcrusher_up","end_gcrusher_up",
                 "end_gcrusher_up",pw_end_gcrush,gcrush,gcrush,gcrush,1,WAIT);
	      delay(tcrush-pw_end_gcrush);
              obl_shapedgradient("end_gcrusher_dn","end_gcrusher_dn",
                 "end_gcrusher_dn",pw_end_gcrush,gcrush,gcrush,gcrush,1,WAIT);
*/
              delay(ti-tcrush-pw_end_gcrush);
            } else {
              delay(ti);
            }
          }
        }

        if (presat) spatial_saturation_pulse();

        /* ================================================================*/
        /* Fat Saturation if the fat_sat flag is set */
        /* ================================================================*/

        if (fat_sat[0] == 'y') {
          obspower(tpwrfat);
          obsoffset(resto-3.45*sfrq); /* move the centre freq to fat peak */
          sp4on();
          shapedpulse(pfatpat,pfat,v1,rof1,rof2);
          sp4off();
          obl_shapedgradient("","gfatk","",pw_gfatk,0.0,gmax/2,0.0,1,WAIT);
          obsoffset(resto);  /* reset centre frequency back to water */
        }

        /* ================================================================*/

xgate(ticks);
        curlist=curlist+1;
        exlist=curlist;
        position_offset_list(pss,gss,ns,resto,OBSch,exlist,v14); 
                                   /* set the frequency offset to
                                   the v14th element in the pss array */

        /* 90 degree pulse ************************************/ 
        sp1on();  /* put out trigger on sp1 every spin excitation */
        ifzero(v9);
          ifzero(v14);
            sp2on();  /* put out trigger on sp2 at start of every volume */
          endif(v14);
        endif(v9);
        obspower(tpwr1);
        obl_shapedgradient("","","gss",pw_gss,0.0,0.0,gss,1,NOWAIT);
        delay(agss*grate+tep);
        sp4on();
        shapedpulse(p1pat,p1,v1,rof1,rof2);
        sp4off();
        delay(pw_gss_d);
        
        sp1off();
        sp2off();

        /* Slice refocus pulse ********/
        obl_shapedgradient("","","gssrgd_up",pw_gssr,0.0,0.0,gssr,1,WAIT);
	delay(tref-pw_gssr);
        obl_shapedgradient("","","gssrgd_dn",pw_gssr,0.0,0.0,gssr,1,WAIT);

	delay(etl_delay/2.0);

        if (spinecho[0] == 'y')
        {

          curlist=curlist+1;
          selist=curlist;
          position_offset_list(pss,gss2,ns,resto,OBSch,selist,v14); 
                                   /* set the frequency offset to
                                   the v14th element in the pss array */
          obspower(tpwr2);
          delay(ted1);
          /* 180 degree RF pulse ********************************/
          obl_shapedgradient("","","gss2",pw_gss2,0.0,0.0,gss2,1,NOWAIT);
          delay(fabs(gss2)*grate+tspoil+tep);
          sp4on();
          shapedpulse(p2pat,p2,v2,rof1,rof2);
          sp4off();
          delay(fabs(gss2)*grate+tspoil-tep);
          delay(ted2);
        }

	delay(etl_delay/2.0);

        /* Read dephaser and phase encode dephaser pulse ********/
/* PJPJ */
        phase_encode_shapedgradient("gdp_up",risetref2,gror,gbd,0.0,sgbd,
            v9,num_ints,psi,phi,theta,v7,WAIT,0);
	delay(tref2-risetref2);
        phase_encode_shapedgradient("gdp_dn",risetref2,gror,gbd,0.0,sgbd,
            v9,num_ints,psi,phi,theta,v7,WAIT,1);

/*
        phase_encode_shapedgradient("gdephase_up",risetref2,gror,gbd,0.0,sgbd,
            v9,num_ints,psi,phi,theta,v7,WAIT,0);
	delay(tref2-risetref2);
        phase_encode_shapedgradient("gdephase_dn",risetref2,gror,gbd,0.0,sgbd,
            v9,num_ints,psi,phi,theta,v7,WAIT,1);
*/

        obsoffset(resto);

/* PJPJ */
        obl_shapedgradient("epird","epiph","",(2.0*pw_gro1+echo_del+
                at)*etl,gro1,sgbpe,0.0,1,NOWAIT);
/*
        obl_shapedgradient("epi_read","epi_phase","",(2.0*pw_gro1+echo_del+
                at)*etl,gro1,sgbpe,0.0,1,NOWAIT);
*/
        delay(tep);  /* gradient propagation delay */
length = (2.0*pw_gro1+echo_del+at)*etl;
acquire(np*nv,2.0*length/np/nv);

        if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
/* PJPJ */
          obl_shapedgradient("e_gcr_up","e_gcr_up",
             "e_gcr_up",pw_end_gcrush,gcrush,gcrush,gcrush,1,WAIT);
	  delay(tcrush-pw_end_gcrush);
          obl_shapedgradient("e_gcr_dn","e_gcr_dn",
             "e_gcr_dn",pw_end_gcrush,gcrush,gcrush,gcrush,1,WAIT);
/*
          obl_shapedgradient("end_gcrusher_up","end_gcrusher_up",
             "end_gcrusher_up",pw_end_gcrush,gcrush,gcrush,gcrush,1,WAIT);
	  delay(tcrush-pw_end_gcrush);
          obl_shapedgradient("end_gcrusher_dn","end_gcrusher_dn",
             "end_gcrusher_dn",pw_end_gcrush,gcrush,gcrush,gcrush,1,WAIT);
*/
        }

	zero_all_gradients(); /* double check that all gradients are zero */

        delay(postdelay);
        endmsloop(seqcon[1],v14); /* increment the msloop counter v14 */
      endloop(v9);  /* increment the interleaves loop counter v9 */
}
