FUNCTION avw_read, fname, verbose

; written by Marcus Weiger, Oxford, March 2001

COMMON Avw, avw_key, avw_dim, avw_his

; read header
avw_head
GET_LUN, unit
OPENR, unit, fname+'.hdr'
READU, unit, avw_key, avw_dim, avw_his
CLOSE, unit
FREE_LUN, unit

swap_bytes = avw_key.sizeof_hdr(0) NE 348

IF swap_bytes THEN BEGIN
   avw_key.sizeof_hdr       = swap_endian(avw_key.sizeof_hdr)
   avw_key.extents          = swap_endian(avw_key.extents)
   avw_key.session_error    = swap_endian(avw_key.session_error)
   avw_dim.dim              = swap_endian(avw_dim.dim)
   avw_dim.unused8_14       = swap_endian(avw_dim.unused8_14)
   avw_dim.datatype         = swap_endian(avw_dim.datatype)
   avw_dim.bitpix           = swap_endian(avw_dim.bitpix)
   avw_dim.dim_un0          = swap_endian(avw_dim.dim_un0)
   avw_dim.pixdim           = swap_endian(avw_dim.pixdim)
   avw_dim.funused8_13      = swap_endian(avw_dim.funused8_13)
   avw_dim.compressed       = swap_endian(avw_dim.compressed)
   avw_dim.verified         = swap_endian(avw_dim.verified)
   avw_dim.glmax            = swap_endian(avw_dim.glmax)
   avw_dim.glmin            = swap_endian(avw_dim.glmin)
   avw_his.views            = swap_endian(avw_his.views)
   avw_his.vols_added       = swap_endian(avw_his.vols_added)
   avw_his.start_field      = swap_endian(avw_his.start_field)
   avw_his.field_skip       = swap_endian(avw_his.field_skip)
   avw_his.omax             = swap_endian(avw_his.omax)
   avw_his.omin             = swap_endian(avw_his.omin)
   avw_his.smax             = swap_endian(avw_his.smax)
   avw_his.smin             = swap_endian(avw_his.smin)
END

IF verbose THEN BEGIN 
   PRINT, 'sizeof_hdr ', avw_key.sizeof_hdr
   PRINT, 'data_type ', STRING(avw_key.data_type)
   PRINT, 'db_name ', STRING(avw_key.db_name)    
   PRINT, 'extents ', avw_key.extents        
   PRINT, 'session_error ', avw_key.session_error 
   PRINT, 'regular ', STRING(avw_key.regular)         
   PRINT, 'hkey_un0 ', STRING(avw_key.hkey_un0)       
   PRINT, 'dim ', avw_dim.dim            
   PRINT, 'unused8_14 ', avw_dim.unused8_14      
   PRINT, 'datatype ', avw_dim.datatype     
   PRINT, 'bitpix ', avw_dim.bitpix       
   PRINT, 'dim_un0 ', avw_dim.dim_un0        
   PRINT, 'pixdim ', avw_dim.pixdim         
   PRINT, 'funused8_13 ', avw_dim.funused8_13    
   PRINT, 'compressed ', avw_dim.compressed     
   PRINT, 'verified ', avw_dim.verified        
   PRINT, 'glmax ', avw_dim.glmax           
   PRINT, 'glmin ', avw_dim.glmin            
   PRINT, 'descrip ', STRING(avw_his.descrip)        
   PRINT, 'aux_file ', STRING(avw_his.aux_file)         
   PRINT, 'orient ', STRING(avw_his.orient)         
   PRINT, 'originator ', STRING(avw_his.originator)     
   PRINT, 'generated ', STRING(avw_his.generated)       
   PRINT, 'scannum ', STRING(avw_his.scannum)         
   PRINT, 'patient_id ', STRING(avw_his.patient_id)      
   PRINT, 'exp_date ', STRING(avw_his.exp_date)        
   PRINT, 'exp_time ', STRING(avw_his.exp_time)        
   PRINT, 'hist_un0 ', STRING(avw_his.hist_un0)       
   PRINT, 'views ', avw_his.views           
   PRINT, 'vols_added ', avw_his.vols_added     
   PRINT, 'start_field ', avw_his.start_field     
   PRINT, 'field_skip ', avw_his.field_skip      
   PRINT, 'omax ', avw_his.omax          
   PRINT, 'omin ', avw_his.omin            
   PRINT, 'smax ', avw_his.smax            
   PRINT, 'smin ', avw_his.smin            
END 

; read file
n_dims = avw_dim.dim(0)
n_cols = avw_dim.dim(1)
n_rows = avw_dim.dim(2)
n_slices = avw_dim.dim(3)
n_vols = avw_dim.dim(4)

CASE avw_dim.datatype(0) OF 
   2: BEGIN
      tmp_im=REFORM(BYTARR(n_cols, n_rows, n_slices))
      im=REFORM(BYTARR(n_cols, n_rows, n_slices, n_vols))
   END
   4: BEGIN
      tmp_im=REFORM(INTARR(n_cols, n_rows, n_slices))
      im=REFORM(INTARR(n_cols, n_rows, n_slices, n_vols))
   END
   8: BEGIN
      tmp_im=REFORM(LONARR(n_cols, n_rows, n_slices))
      im=REFORM(LONARR(n_cols, n_rows, n_slices, n_vols))
   END
   16: BEGIN
      tmp_im=REFORM(FLTARR(n_cols, n_rows, n_slices))
      im=REFORM(FLTARR(n_cols, n_rows, n_slices, n_vols))
   END
   32: BEGIN
      tmp_im=REFORM(COMPLEXARR(n_cols, n_rows, n_slices))
      im=REFORM(COMPLEXARR(n_cols, n_rows, n_slices, n_vols))
   END
   64: BEGIN
      tmp_im=REFORM(DBLARR(n_cols, n_rows, n_slices))
      im=REFORM(DBLARR(n_cols, n_rows, n_slices, n_vols))
   END
ENDCASE

GET_LUN, unit
OPENR, unit, fname+'.img'
FOR i=0, n_vols-1 DO BEGIN
   READU, unit, tmp_im
   IF swap_bytes THEN tmp_im = swap_endian(tmp_im)
   IF (n_slices EQ 1) THEN im(*,*,i)=tmp_im ELSE im(*,*,*,i)=tmp_im
ENDFOR
CLOSE, unit
FREE_LUN, unit

RETURN, im
END

