function calc_field

;*******************************************************************
; This portion of the program calculates the base field map from the
; information read in by read_[system]_data.pro
;*******************************************************************
; now uses mark j's unwrap code (JW, Mar 01)

common share1, roimap, dim0, dim1, dim2
common shareshim, slo, fov, fovz, te
common field_map_data, phsmap, allimg, fieldmap, strucmap
common mask_info, weight_type,weightf,wfactor,vox0,vox1,vox2,grad,$
                  tmatrix,noweightf,freq_limit

; name filename params
phs_fname='~/vnmrsys/shims/autoshim_phs'
struc_fname='~/vnmrsys/shims/autoshim_sphs'
mask_fname='~/vnmrsys/shims/autoshim_mphs'
uphs_fname='~/vnmrsys/shims/autoshim_uphs'

;make sure old files no longer exist
if file_exists(phs_fname+'.img') then spawn, '/bin/rm -rf '+phs_fname+'.*'
if file_exists(struc_fname+'.img') then spawn, '/bin/rm -rf '+struc_fname+'.*'
if file_exists(mask_fname+'.img') then spawn, '/bin/rm -rf '+mask_fname+'.*'
if file_exists(uphs_fname+'.img') then spawn, '/bin/rm -rf '+uphs_fname+'.*'

; save phase maps, mask and struc (including headers) in ~vnmrsys/shims/ 
; need to specify voxel dimensions for correct header writing
voxdim_3d=fltarr(8)
voxdim_3d(0)=3.0
voxdim_3d(1:2)=fov/float(dim0)
voxdim_3d(3)=fovz/float(dim2)
avw_write, float(roimap), mask_fname, voxdim_3d
voxdim_4d=fltarr(8)
voxdim_4d(0)=4.0
voxdim_4d(1:2)=fov/float(dim0)
voxdim_4d(3)=fovz/float(dim2)
voxdim_4d(4)=2.0
avw_write, phsmap, phs_fname, fltarr(8)+1.0
avw_write, abs(allimg), struc_fname, fltarr(8)+1.0

; now run unwrap code on the phase maps
spawn, '/usr/local/fsl/bin/prelude -p '+phs_fname $
      +' -m '+mask_fname+' -u '+uphs_fname+' -a '+struc_fname

; read in unwrapped phase maps
uphsmap=avw_read(uphs_fname,0)

; derive fieldmap
fieldmap=(uphsmap(*,*,*,1)-uphsmap(*,*,*,0))/(te)

; make sure fieldmap has correct offset
; find correct 2pi offset that gives min mean field
mask_arr=where(roimap(*,*,*) ne 0)
avfield=total(fieldmap(mask_arr))/float(n_elements(mask_arr))
numshifts=round(avfield*te)
fieldmap=fieldmap - ((roimap ne 0)*numshifts/te)

; mask fieldmap with max fieldmap deviation
if ((freq_limit gt 0.0) and (freq_limit lt 1000.0)) then begin
  print, "   field map masked with maximum field deviation = "+$
         strtrim(string(freq_limit),2)+" Hz"
  roimap=roimap*(abs(fieldmap) le freq_limit*2.0*!pi)
  ; alter all weight masks
  weightf=weightf*roimap
  noweightf=noweightf*roimap
endif

fieldmap=fieldmap*roimap

; alter fieldmap so background equals mean field value
back_val=total(fieldmap(mask_arr))/n_elements(mask_arr)
disp_fieldmap=fieldmap + ((avfield-numshifts/te)*(roimap eq 0))

print, "   field map..."
err=display_ims(bytscl(disp_fieldmap))
wait, 1

; delete all written files
spawn, '/bin/rm -rf '+phs_fname+'.*'
spawn, '/bin/rm -rf '+struc_fname+'.*'
spawn, '/bin/rm -rf '+mask_fname+'.*'
spawn, '/bin/rm -rf '+uphs_fname+'.*'

return, 0 ; everything fine

end

