function calc_shims, system_descrip

; now option of weight_type to give preferential weighting to various
; areas. In this code, weightf is an array holding the weighting fn

common file_info, directory, filename, file_type, smooth_val, info_filename
common share1, roimap, dim0, dim1, dim2
common shareeigen,ipmatrix, phsrow, result, bndv,pndpoint, curlimit,orgshim,nch
common shareshim, slo, fov, fovz, te
common shim_info, shimstrength, shimfunc, channels, magfctr, x, y, z
common field_map_data, phsmap, allimg, fieldmap, strucmap
common sys_names_info, sys_names, curr_sys
common grads, gx, gy, gz
common mask_info, weight_type,weightf,wfactor,vox0,vox1,vox2,grad,$
                  tmatrix,noweightf,freq_limit

on_ioerror, read_write_problem

nch=-1
shim_defs, system_descrip
if (total(nch) eq -1) then begin
  print, "Error reading in system calibration coeffs from shim_defs.pro"
  return, 1
endif
orgshim=fltarr(nch) ; will hold the original (i.e., previous) shim DAC/amp vals

;**************************************************************************
;Read in old shim set values from directory/shimset.old :
;**************************************************************************
curr_fname="shimset.old"
if file_exists(directory+'/'+curr_fname) then begin
  get_lun, unit
  openr, unit, directory+"/"+curr_fname
  tempstr=string(0)
  tempvl=float(0)
  for i=0, nch-1 do begin
    readf, unit, tempvl, tempstr
    orgshim(i)=tempvl
  endfor
  close, unit
  free_lun, unit
endif else goto, read_write_problem

; set inpweight and sls weight for use with field shim
; these are now redundant
;inpweight=1.0
;slsweight=1.0

for i=0, nch-1 do begin
  shimfunc(*,*,*,i)=shimfunc(*,*,*,i)*roimap(*,*,*)*shimstrength(i)
endfor

;**************************************************************************
;Calculate the inner-product matrix:
;**************************************************************************
; instead of using weightf, set weightf to weightf*weightf
weightf_tmp=weightf
weightf=weightf_tmp*weightf_tmp

; calculate the grads of field and shimfunc- need for grads measure & gradshim
field_grad=fltarr(dim0,dim1,dim2,3)
calc_grad, fieldmap  ; calc grad of fieldmap

; create array or points not to consider (as field_grad value too high)
; assumed too high due to bad unwrapping. Max grad val = 2500 rads-1 (400Hz)
high_grad=fltarr(dim0,dim1,dim2,3)
high_grad(*,*,*,0)=float((gx*roimap) le 2500.0)
high_grad(*,*,*,1)=float((gy*roimap) le 2500.0)
high_grad(*,*,*,2)=float((gz*roimap) le 2500.0)

; fill field_grad array
field_grad(*,*,*,0)=gx*roimap*(vox0*dim0/fov)*high_grad(*,*,*,0)
field_grad(*,*,*,1)=gy*roimap*(vox1*dim1/fov)*high_grad(*,*,*,1)
field_grad(*,*,*,2)=gz*roimap*(vox2*dim2/fovz)*high_grad(*,*,*,2)

shimf_grad=fltarr(dim0,dim1,dim2,nch,3)
for i=0,nch-1 do begin
  calc_grad, shimfunc(*,*,*,i)  ; calc grad of shimfunc
  shimf_grad(*,*,*,i,0)=gx*roimap*(vox0*dim0/fov)*high_grad(*,*,*,0)
  shimf_grad(*,*,*,i,1)=gy*roimap*(vox1*dim1/fov)*high_grad(*,*,*,1)
  shimf_grad(*,*,*,i,2)=gz*roimap*(vox2*dim2/fovz)*high_grad(*,*,*,2)
endfor

; now three options: field shim or grad shim or both
if ((grad eq 0) or (grad eq 2)) then begin ;FIELD SHIM
  if (grad eq 0) then begin
    opt_method="Field"
  endif
  ipmatrix=fltarr(nch, nch)
  phsrow=fltarr(nch)

  mask_arr=where(roimap(*,*,*) ne 0)
  norshimf=fltarr(dim0,dim1,dim2,nch)
  for i=0, nch-1 do begin
    shimftmp=fltarr(dim0,dim1,dim2)
    shimftmp=shimfunc(*,*,*,i)
    avshimf=total(shimftmp(mask_arr))/n_elements(mask_arr)
    norshimf(*,*,*,i)=(shimfunc(*,*,*,i)-avshimf)*roimap
  endfor
 
  norphsmap=fltarr(dim0,dim1,dim2)
  avfield=total(fieldmap(mask_arr))/n_elements(mask_arr)
  norphsmap=(fieldmap-avfield)*roimap
  
  for i=0, nch-1 do begin
    if (curlimit(0,i) ne 0) then begin
      for j=i, nch-1 do begin
        if (curlimit(0,j) ne 0) then begin
          ipmatrix(i,j)=total(norshimf(*,*,*,i)*norshimf(*,*,*,j)*$
                        weightf(*,*,*)*roimap(*,*,*))
          ipmatrix(j,i)=ipmatrix(i,j)
        endif
      endfor
      phsrow(i)=total(norphsmap(*,*,*,0)*norshimf(*,*,*,i)*$
                weightf(*,*,*)*roimap(*,*,*))
    endif else begin
      ipmatrix(i,i)=1
    endelse
  endfor

  ; store array temporarily if finding grad shim as well
  if (grad eq 2) then begin
    ipmatrix_tmp=ipmatrix
    phsrow_tmp=phsrow
  endif

endif 

if ((grad eq 1) or (grad eq 2)) then begin ; GRAD SHIM
  if (grad eq 1) then begin
    opt_method="Field Gradients"
  endif
  ipmatrix=fltarr(nch, nch)
  phsrow=fltarr(nch)
  for i=0, nch-1 do begin
    if (curlimit(0,i) ne 0) then begin
      for j=i, nch-1 do begin
        if (curlimit(0,j) ne 0) then begin
          for dir=0,2 do begin
            ipmatrix(i,j)=ipmatrix(i,j)+total(shimf_grad(*,*,*,i,dir)*$
                          shimf_grad(*,*,*,j,dir)*$
                          weightf(*,*,*)*roimap(*,*,*))
          endfor
          ipmatrix(j,i)=ipmatrix(i,j)
        endif 
      endfor
      for dir=0,2 do begin
        phsrow(i)=phsrow(i)+total(shimf_grad(*,*,*,i,dir)*$
                  field_grad(*,*,*,dir)*$
                  weightf(*,*,*)*roimap(*,*,*))
      endfor
    endif else begin
      ipmatrix(i,i)=1
    endelse
  endfor
endif

if (grad eq 2) then begin ; FIELD AND GRAD SHIM
  opt_method="Field and the Field Gradients"
  ; find mean of phsrow and phsrow_tmp 
  ; ipmatrix and ipmatrix_tmp assumed equivalent
  stats_field=moment(abs(phsrow_tmp))
  stats_grad=moment(abs(phsrow))
  ; make phsrow mean equivalent and add - use weight factor of 5 for field
  if (stats_field(0) ne 0.0) then begin
    phsrow=(0.2*phsrow) + (phsrow_tmp*stats_grad(0)/stats_field(0))
    ipmatrix=(0.2*ipmatrix) + (ipmatrix_tmp*stats_grad(0)/stats_field(0))
  endif else begin ; if stats_field(0)=0 then gradshim phsrow and ipmatrix used 
    print, "   ************************************************"
    print, "   ERROR - cannot combine field and field gradients"
    print, "   ************************************************"
    print, "   Using Field Gradients Only....
  endelse
endif

; output info to screen
print, "   optimising shim on the "+opt_method
print, "   relative shim voxel dimensions are:"
print, "                             x-dir "+strtrim(string(vox0*10.0),2)
print, "                             y-dir "+strtrim(string(vox1*10.0),2)
print, "                             z-dir "+strtrim(string(vox2*10.0),2)

; set limits on shim currents based on present shim values
curlimit(1,*)=(curlimit(0,*)+orgshim(*))/magfctr(*)
curlimit(0,*)=(curlimit(0,*)-orgshim(*))/magfctr(*)

;**************************************************************************
;Solve optimization value problem:
;**************************************************************************
searchpoint

;**************************************************************************
;Display corrected fieldmap and output results into the file "shimset"
;**************************************************************************
; calculate corrected fieldmap and field grads
corr_fieldmap=fieldmap
for i=0, nch-1 do begin
  corr_fieldmap=corr_fieldmap+result(i)*shimfunc(*,*,*,i)
endfor
corr_fieldmap=corr_fieldmap*roimap
 
corr_field_grad=fltarr(dim0,dim1,dim2,3)
calc_grad, corr_fieldmap  ; calc grad of corr_fieldmap
corr_field_grad(*,*,*,0)=gx*roimap*(vox0*dim0/fov)*high_grad(*,*,*,0)
corr_field_grad(*,*,*,1)=gy*roimap*(vox1*dim1/fov)*high_grad(*,*,*,1)
corr_field_grad(*,*,*,2)=gz*roimap*(vox2*dim2/fovz)*high_grad(*,*,*,2)

; display corrected fieldmap
; alter fieldmap so background equals mean field value
mask_arr=where(roimap(*,*,*) ne 0)
back_val=total(corr_fieldmap(mask_arr))/n_elements(mask_arr)
disp_fieldmap=corr_fieldmap + (back_val*(roimap eq 0))

b0mapimg=bytscl(disp_fieldmap)
print, "   corrected field map..."
err=display_ims(b0mapimg(*,*,*,0))
wait, 1

curr_fname="shimset"
get_lun, unit
openw, unit, directory+"/"+curr_fname
for i=0, nch-1 do begin
  if (sys_names(curr_sys, 3) eq 'int') then begin
    printf, unit, fix(result(i)*magfctr(i)+orgshim(i)), "  ", channels(i)
  endif else begin
    printf, unit, result(i)*magfctr(i)+orgshim(i), "  ", channels(i)
  endelse
endfor
close, unit
free_lun, unit

;***********************************************************************
;calc field measures 
;***********************************************************************
; calc field S.D. before and after, weighted with noweightf
no_wmask_arr=where(noweightf(*,*,*) ne 0.0)
if (total(no_wmask_arr) gt -1) then begin
  orig_av_field=total(fieldmap(no_wmask_arr))/n_elements(no_wmask_arr)
  orig_var_field=(fieldmap(no_wmask_arr)-orig_av_field)*$
          (fieldmap(no_wmask_arr)-orig_av_field)
  orig_var_field=total(orig_var_field*noweightf(no_wmask_arr))$
                        /(total(noweightf(no_wmask_arr)))
  orig_field=sqrt(orig_var_field)/(2.0*!pi)
endif
if (total(no_wmask_arr) gt -1) then begin
  final_av_field=total(corr_fieldmap(no_wmask_arr))/n_elements(no_wmask_arr)
  final_var_field=(corr_fieldmap(no_wmask_arr)-final_av_field)*$
          (corr_fieldmap(no_wmask_arr)-final_av_field)
  final_var_field=total(final_var_field*noweightf(no_wmask_arr))$
                         /(total(noweightf(no_wmask_arr)))
  final_field=sqrt(final_var_field)/(2.0*!pi)
endif

; calc grads measure before and after
if (total(noweightf) ne 0) then begin
  orig_grads=0.0
  for i=0,2 do begin
    orig_grads=orig_grads+total(noweightf(*,*,*)$
                             *field_grad(*,*,*,i)*field_grad(*,*,*,i))$
                             /total(noweightf(*,*,*))
  endfor
  final_grads=0.0
  for i=0,2 do begin
    final_grads=final_grads+total(noweightf(*,*,*)$
                         *corr_field_grad(*,*,*,i)$
                         *corr_field_grad(*,*,*,i))/total(noweightf(*,*,*))
  endfor
endif

; for weighted brain shimming:
if (weight_type eq 2) then begin
  wmask_arr=where(weightf(*,*,*) ne 0.0)
  ; calc weighted field S.D. before and after
  if (total(wmask_arr) gt -1) then begin
    orig_w_av_field=total(fieldmap(wmask_arr))/n_elements(wmask_arr)
    orig_w_var_field=(fieldmap(wmask_arr)-orig_w_av_field)*$
            (fieldmap(wmask_arr)-orig_w_av_field)
    orig_w_var_field=total(orig_w_var_field*weightf(wmask_arr))$
                     /(total(weightf(wmask_arr)))
    orig_w_field=sqrt(orig_w_var_field)/(2.0*!pi)
  endif
  if (total(wmask_arr) gt -1) then begin
    final_w_av_field=total(corr_fieldmap(wmask_arr))/n_elements(wmask_arr)
    final_w_var_field=(corr_fieldmap(wmask_arr)-final_w_av_field)*$
            (corr_fieldmap(wmask_arr)-final_w_av_field)
    final_w_var_field=total(final_w_var_field*weightf(wmask_arr))$
                           /(total(weightf(wmask_arr)))
    final_w_field=sqrt(final_w_var_field)/(2.0*!pi)
  endif
  
  ; calc grads measure before and after
  if (total(weightf) ne 0) then begin
    orig_w_grads=0.0
    for i=0,2 do begin
      orig_w_grads=orig_w_grads+total(weightf(*,*,*)$
                                 *field_grad(*,*,*,i)*field_grad(*,*,*,i))$
                                 /total(weightf(*,*,*))
    endfor
    final_w_grads=0.0
    for i=0,2 do begin
      final_w_grads=final_w_grads+total(weightf(*,*,*)$
                           *corr_field_grad(*,*,*,i)$
                           *corr_field_grad(*,*,*,i))/total(weightf(*,*,*))
    endfor
  endif
endif

;**************************************************************************
;print field measures to file 
;**************************************************************************
get_lun, unit
openw, unit, info_filename
printf, unit, "Automated Shimming Results"
printf, unit, "=========================="
printf, unit, ""
printf, unit, "Shimsets:"
printf, unit, "    Before    After    Shim"
for i=0, nch-1 do begin
  if (sys_names(curr_sys, 3) eq 'int') then begin
    printf, unit, "  ",fix(orgshim(i))," ",fix(result(i)*magfctr(i)+orgshim(i)),$
                  "    ",channels(i)
  endif else begin
    printf, unit, "  ",orgshim(i)," ",result(i)*magfctr(i)+orgshim(i),$
                  "    ",channels(i)
  endelse
endfor

printf, unit,""
printf, unit, "Using Fieldmap:  "+directory+"/"+filename
printf, unit,""
if (weight_type eq 0) then begin
  printf, unit,"Shimming on:  Phantom"
  printf, unit,""
  printf, unit, "Mask Theshold:  "+strtrim(string(wfactor(0)),2)+"%"
endif else begin
  if (weight_type eq 1) then begin
    printf, unit,"Shimming on:  Brain"
  endif else begin
    printf, unit,"Shimming on:  Brain (Weighted)"
  endelse
  printf, unit,""
  printf, unit, "Transformation Matrix:"
  printf, unit, tmatrix
endelse

if ((freq_limit gt 0.0) and (freq_limit lt 1000.0)) then begin
  printf, unit, ""
  printf, unit, "Fieldmap Masked with Maximum Frequency Deviation:  "$
                +strtrim(string(freq_limit),2)+" Hz"
endif

;print optimisation type
printf, unit,""
printf, unit, "Optimising Shim on:  "+opt_method
printf, unit, "Relative Shim Voxel Dimensions:"
printf, unit, "    x-dir "+strtrim(string(vox0*10.0),2)
printf, unit, "    y-dir "+strtrim(string(vox1*10.0),2)
printf, unit, "    z-dir "+strtrim(string(vox2*10.0),2)

;print global measures
printf, unit, ""
printf, unit, "Global Measures:"
printf, unit, "                     field sd meas       grad meas"
printf, unit, "    before (expt)    ",orig_field, "         100.000"
printf, unit, "    after (theor)    ",final_field,"   ", 100.0*final_grads/orig_grads

;print weighted measures for brain
if (weight_type eq 2) then begin
  printf, unit, ""
  printf, unit, "Brain Weightings:"
  printf, unit, "         posterior "+strtrim(string(wfactor(0)),2)
  printf, unit, "    superior front "+strtrim(string(wfactor(1)),2)
  printf, unit, "    inferior front "+strtrim(string(wfactor(2)),2)
  printf, unit, "     inferior temp "+strtrim(string(wfactor(3)),2)
  printf, unit, "        cerebellum "+strtrim(string(wfactor(4)),2)
  if (wfactor(5) gt 0.0) then begin
    printf, unit, "             extra "+strtrim(string(wfactor(5)),2)
  endif
  printf, unit, ""
  printf, unit, "Weighted Measures:"
  printf, unit, "                     field sd meas       grad meas"
  printf, unit, "    before (expt)    ",orig_w_field, "         100.000"
  printf, unit, "    after (theor)    ",final_w_field,$
                "   ", 100.0*final_w_grads/orig_w_grads
endif

close, unit
free_lun, unit

return, 0

read_write_problem:
  print, "Error: Read/write problem "
  return, 1

end
