pro calc_field

;*******************************************************************
; This portion of the program calculates the base field map from the
; information read in by read_[system]_data.pro
;*******************************************************************
; now uses mark j's unwrap code:PRELUDE (JW, Mar 01)

common share1, dim0, dim1, dim2, diml, dimh, stindex, numfiles, $
           allmagimg,allmagmax,allmagmin
common shareshim, slo, slsep, fov, fovz, te, plane
common field_map_data, phsmap, allimg, fieldmap, strucmap
common shareroi, roimap, topb, botb, lefb, rightb

; name filename params
phs_fname='~/calib_phs'
struc_fname='~/calib_sphs'
mask_fname='~/calib_mphs'
uphs_fname='~/calib_uphs'

; save phase maps, mask and struc (including headers) in mask_files
; need to specify voxel dimensions for correct header writing
voxdim_3d=fltarr(8)
voxdim_3d(0)=3.0
voxdim_3d(1:2)=fov/float(dim0)
voxdim_3d(3)=fovz/float(dim2)
avw_write, float(roimap), mask_fname, voxdim_3d
voxdim_4d=fltarr(8)
voxdim_4d(0)=4.0
voxdim_4d(1:2)=fov/float(dim0)
voxdim_4d(3)=fovz/float(dim2)
voxdim_4d(4)=2.0
avw_write, phsmap, phs_fname, fltarr(8)+1.0
avw_write, abs(allimg), struc_fname, fltarr(8)+1.0

; now run unwrap code on the phase maps
spawn, '/usr/local/fsl/bin/prelude -p '+phs_fname $
      +' -m '+mask_fname+' -u '+uphs_fname+' -a '+struc_fname

; read in unwrapped phase maps
uphsmap=avw_read(uphs_fname,0)

; derive fieldmap
fieldmap=(uphsmap(*,*,*,1)-uphsmap(*,*,*,0))/(te)

; make sure fieldmap has correct offset
; find correct 2pi offset that gives min mean field
mask_arr=where(roimap(*,*,*) ne 0)
avfield=total(fieldmap(mask_arr))/float(n_elements(mask_arr))
numshifts=round(avfield*te)
fieldmap=fieldmap - ((roimap ne 0)*numshifts/te)

fieldmap=fieldmap*roimap

; alter fieldmap so background equals mean field value
back_val=total(fieldmap(mask_arr))/n_elements(mask_arr)
disp_fieldmap=fieldmap + ((avfield-numshifts/te)*(roimap eq 0))

print, "   field map..."
err=display_ims(bytscl(disp_fieldmap))

; remove temporary files
spawn, '/bin/rm -rf '+phs_fname+'.*'
spawn, '/bin/rm -rf '+struc_fname+'.*'
spawn, '/bin/rm -rf '+mask_fname+'.*'
spawn, '/bin/rm -rf '+uphs_fname+'.*'

end

