pro field_regress

common fit_coeff_bases, fit_coeff_base, menu1_coeffs, menu2_coeffs, $
     dac_delta, norm_term
common shareroi, roimap, topb, botb, lefb, rightb
common share1, dim0, dim1, dim2, diml, dimh, stindex, numfiles, $
           allmagimg,allmagmax,allmagmin
common cal_sys_bases, cal_sys_base, diff_fieldmap, calc_arr, nudge_im_slide
common shareshim, slo, slsep, fov, fovz, te, plane

;*************************************************************************
; define the x, y and z matrices giving the spatial coordinates
;*************************************************************************

x=float(indgen(dim0, 1, 1)-dim0/2)*fov/dim0
x=rebin(x, dim0, dim1, dim2)
 
y=float(indgen(1,dim1,1)-dim1/2)*fov/dim1
y=rebin(y, dim0, dim1, dim2)
 
z=(float(indgen(1,1,dim2))-dim2/2.0+1.0)*fovz/dim2+slo
z=rebin(z, dim0, dim1, dim2)
;*************************************************************************

roi_mask=where(roimap eq 1)
num_dependents=n_elements(roi_mask)

num_regressors=0

if (total(menu1_coeffs) ne -1) then num_regressors=num_regressors+ $
       total(menu1_coeffs)

if (total(menu2_coeffs) ne -1) then num_regressors=num_regressors+ $
       total(menu2_coeffs)

depend_matrix=fltarr(num_regressors, num_dependents)
regress_index=0 ; index to keep track of how many regressors have been defined

;*************************************************************************
; set up the dependent variable matrix for menu 1
;*************************************************************************
menu1_index=where(menu1_coeffs eq 1)
if total(menu1_index ne -1) then begin
  for i=0, n_elements(menu1_index)-1 do begin
    index_val=menu1_index(i)
    case index_val of

    0: BEGIN  ; B0 term
         depend_matrix(regress_index,*)=fltarr(num_dependents)+1.0
         regress_index=regress_index+1
       END

    1: BEGIN  ; x term
         tmp_m=x
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

    2: BEGIN  ; y term
         tmp_m=y
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

    3: BEGIN  ; z term
         tmp_m=z
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

    4: BEGIN  ; z2 term
         tmp_m=z*z
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

    5: BEGIN  ; (x2+y2) term
         tmp_m=x*x+y*y
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

    6: BEGIN  ; xy term
         tmp_m=x*y
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

    7: BEGIN  ; zx term
         tmp_m=z*x
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

    8: BEGIN  ; zy term
         tmp_m=z*y
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

    9: BEGIN  ; (x2-y2) term
         tmp_m=x*x-y*y
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

   10: BEGIN  ; z3 term
         tmp_m=z*z*z
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

 else: BEGIN
         print, "unknown term selected from menu 1 in field_regress.pro"
       END 
    endcase
  endfor
endif
;*************************************************************************

;*************************************************************************
; Do the same for menu 2
;*************************************************************************
menu2_index=where(menu2_coeffs eq 1)
if total(menu2_index ne -1) then begin
  for i=0, n_elements(menu2_index)-1 do begin
    index_val=menu2_index(i)
    case index_val of

    0: BEGIN  ; xz2 term
         tmp_m=x*z*z
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

    1: BEGIN  ; yz2 term
         tmp_m=y*z*z
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

    2: BEGIN  ; xy2 term
         tmp_m=x*y*y
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

    3: BEGIN  ; yx2 term
         tmp_m=y*x*x
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

    4: BEGIN  ; x(x2+y2) term
         tmp_m=x*(x*x+y*y)
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

    5: BEGIN  ; y(x2+y2) term
         tmp_m=y*(x*x+y*y)
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

    6: BEGIN  ; z(x2+y2) term
         tmp_m=z*(x*x+y*y)
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

    7: BEGIN  ; z(x2-y2) term
         tmp_m=z*(x*x-y*y)
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

    8: BEGIN  ; x3 term
         tmp_m=x*x*x
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

    9: BEGIN  ; y3 term
         tmp_m=y*y*y
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

   10: BEGIN  ; xyz term
         tmp_m=x*y*z
         depend_matrix(regress_index,*)=reform(tmp_m(roi_mask), num_dependents)
         regress_index=regress_index+1
       END

 else: BEGIN
         print, "unknown term selected from menu 2 in field_regress.pro"
       END 
    endcase
  endfor
endif
;*************************************************************************

weight_arr=fltarr(num_dependents)+1.0
print, "doing multiple regression..."
;regress_coeffs=regress(depend_matrix, diff_fieldmap(roi_mask))
regress_coeffs=regress2(depend_matrix, diff_fieldmap(roi_mask), weight_arr, $
   relative_weight=1)

regress_coeffs=reform(regress_coeffs, num_regressors)

calc_arr=fltarr(dim0, dim1, dim2)

regress_index=0 ; reset to zero

;*************************************************************************
; now dispaly the results for the menu 1 selections
;*************************************************************************
print, ""
menu1_index=where(menu1_coeffs eq 1)
if total(menu1_index ne -1) then begin
  for i=0, n_elements(menu1_index)-1 do begin
    index_val=menu1_index(i)
    case index_val of

    0: BEGIN  ; B0 term
         tmp_m=fltarr(dim0, dim1, dim2)+regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+ $
           " B0"
         regress_index=regress_index+1
       END

    1: BEGIN  ; x term
         tmp_m=x*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+" x"
         regress_index=regress_index+1
       END

    2: BEGIN  ; y term
         tmp_m=y*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+" y"
         regress_index=regress_index+1
       END

    3: BEGIN  ; z term
         tmp_m=z*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+" z"
         regress_index=regress_index+1
       END

    4: BEGIN  ; z2 term
         tmp_m=z*z*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+ $
           " z2"
         regress_index=regress_index+1
       END

    5: BEGIN  ; (x2+y2) term
         tmp_m=(x*x+y*y)*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+ $
           " (x2+y2)"
         regress_index=regress_index+1
       END

    6: BEGIN  ; xy term
         tmp_m=x*y*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+ $
           " xy"
         regress_index=regress_index+1
       END

    7: BEGIN  ; zx term
         tmp_m=z*x*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+ $
             " zx"
         regress_index=regress_index+1
       END

    8: BEGIN  ; zy term
         tmp_m=z*y*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+ $
             " zy"
         regress_index=regress_index+1
       END

    9: BEGIN  ; (x2-y2) term
         tmp_m=(x*x-y*y)*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+ $
             " (x2-y2)"
         regress_index=regress_index+1
       END

   10: BEGIN  ; z3 term
         tmp_m=z*z*z*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+ $
             " z3"
         regress_index=regress_index+1
       END

 else: BEGIN
         print, "unknown term selected from menu 1 in field_regress.pro"
       END 
    endcase
  endfor
endif
;*************************************************************************

;*************************************************************************
; and secondly dispaly the results for the menu 2 selections
;*************************************************************************
menu2_index=where(menu2_coeffs eq 1)
if total(menu2_index ne -1) then begin
  for i=0, n_elements(menu2_index)-1 do begin
    index_val=menu2_index(i)
    case index_val of

    0: BEGIN  ; xz2 term
         tmp_m=x*z*z*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+ $
           " xz2"
         regress_index=regress_index+1
       END

    1: BEGIN  ; yz2 term
         tmp_m=y*z*z*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+ $
            " yz2"
         regress_index=regress_index+1
       END

    2: BEGIN  ; xy2 term
         tmp_m=x*y*y*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+ $
            " xy2"
         regress_index=regress_index+1
       END

    3: BEGIN  ; yx2 term
         tmp_m=y*x*x*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+ $
            " yx2"
         regress_index=regress_index+1
       END

    4: BEGIN  ; x(x2+y2) term
         tmp_m=x*(x*x+y*y)*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+ $
            " x(x2+y2)"
         regress_index=regress_index+1
       END

    5: BEGIN  ; y(x2+y2) term
         tmp_m=y*(x*x+y*y)*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+ $
            " y(x2+y2)"
         regress_index=regress_index+1
       END

    6: BEGIN  ; z(x2+y2) term
         tmp_m=z*(x*x+y*y)*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+ $
             " z(x2+y2)"
         regress_index=regress_index+1
       END

    7: BEGIN  ; z(x2-y2) term
         tmp_m=z*(x*x-y*y)*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+ $
             " z(x2-y2)"
         regress_index=regress_index+1
       END

    8: BEGIN  ; x3 term
         tmp_m=x*x*x*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+ $
             " x3"
         regress_index=regress_index+1
       END

    9: BEGIN  ; y3 term
         tmp_m=y*y*y*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+ $
             " y3"
         regress_index=regress_index+1
       END

   10: BEGIN  ; xyz term
         tmp_m=x*y*z*regress_coeffs(regress_index)
         calc_arr=calc_arr+tmp_m
         print, strtrim(string(regress_coeffs(regress_index)/dac_delta), 2)+ $
             " xyz"
         regress_index=regress_index+1
       END

 else: BEGIN
         print, "unknown term selected from menu 2 in field_regress.pro"
       END 
    endcase
  endfor
endif
;*************************************************************************

if ((norm_term ge 1) and (norm_term le n_elements(regress_coeffs))) $
         then begin
  norm_factor=regress_coeffs(norm_term-1)
  print, ""
  print, "normalized terms :"
  print, regress_coeffs/norm_factor
  print, ""
  print, "shimstrength = "+strtrim(string(norm_factor/dac_delta), 2)
endif

print, ""
residual_map=(diff_fieldmap-calc_arr)*roimap
tmp_m=residual_map
tmp_m(roi_mask)=tmp_m(roi_mask)-total(tmp_m(roi_mask))/n_elements(roi_mask)
b0mapimg=bytscl(tmp_m)
print, "residual map..."
dims=size(b0mapimg)
erase
err=display_ims(b0mapimg(*,*,*,0))

print, "residuals summary..."
; calculate the variance
av_field=0.0
std_field=0.0
if (total(roi_mask) gt -1) then begin
  av_field=total(residual_map(roi_mask))/n_elements(roi_mask)
  var_field=(residual_map(roi_mask)-av_field)*(residual_map(roi_mask)-av_field)
  var_field=total(var_field)/(n_elements(roi_mask)-1)
  std_field=sqrt(var_field)
endif
 
print, "average residual field (Hz) = ", av_field/(2.0*!pi)
print, "residual standard deviation (Hz) = ", std_field/(2.0*!pi)

end
