PRO fit_coeff_win_event, event
; This is the event handler for a "pop-up" widget.

; The COMMON block is used because both main_win and 
; main_win_event must know about the group leader.
COMMON groupleader, base
common cal_sys_bases, cal_sys_base, diff_fieldmap, calc_arr, nudge_im_slide
common fit_coeff_bases, fit_coeff_base, menu1_coeffs, menu2_coeffs, $
     dac_delta, norm_term
common shareroi, roimap, topb, botb, lefb, rightb

; Use WIDGET_CONTROL to get the user value of any widget touched and put
; that value into 'eventval':

WIDGET_CONTROL, event.id, GET_UVALUE = eventval

; Perform actions based on the user value of the button which was pressed.

CASE eventval OF

   'CANCEL': BEGIN
                WIDGET_CONTROL, fit_coeff_base, /DESTROY

                ; Re-sensitize the main base
                WIDGET_CONTROL, cal_sys_base, SENSITIVE=1
             END

   'EXECUTE': BEGIN
                WIDGET_CONTROL, fit_coeff_base, SENSITIVE=0
                if ((total(menu1_coeffs) le 0) and $
                       (total(menu2_coeffs) le 0)) then begin
                  print, "No terms selected..."
                endif else begin
                  print, "performing fit..."
                  field_regress
                endelse
                WIDGET_CONTROL, fit_coeff_base, SENSITIVE=1
             END

    'MENU1': BEGIN
                WIDGET_CONTROL, event.id, GET_VALUE=menu1_coeffs
             END 

    'MENU2': BEGIN
                WIDGET_CONTROL, event.id, GET_VALUE=menu2_coeffs
             END 

   'X_PLOT': BEGIN
                if (n_elements(calc_arr) gt 0) then begin
                  tmp_arr=diff_fieldmap-calc_arr
                  dims=size(tmp_arr)
                  if (dims(0) eq 2) then begin
                    ; protect against the occasion of single slice data
                    dims=[3,dims(1),dims(2),1,dims(3),dims(4)]
                  endif
                  tmp_arr2=fltarr(dims(2), dims(3), dims(1))
                  tmp_roimap=fltarr(dims(2), dims(3), dims(1))
                  for i=0, dims(1)-1 do begin
                    tmp_arr2(*,*,i)=tmp_arr(i,*,*)
                    tmp_roimap(*,*,i)=roimap(i,*,*)
                  endfor
                  roi_mask=where(tmp_roimap eq 1)
                  tmp_arr2(roi_mask)=tmp_arr2(roi_mask)- $
                      total(tmp_arr2(roi_mask))/n_elements(roi_mask)
                  plot, tmp_arr2(roi_mask)
                endif else begin 
                  print, "Error: No fit has been performed..." 
                endelse
             END

   'Y_PLOT': BEGIN
                if (n_elements(calc_arr) gt 0) then begin
                  tmp_arr=diff_fieldmap-calc_arr
                  dims=size(tmp_arr)
                  if (dims(0) eq 2) then begin
                    ; protect against the occasion of single slice data
                    dims=[3,dims(1),dims(2),1,dims(3),dims(4)]
                  endif
                  tmp_arr2=fltarr(dims(1), dims(3), dims(2))
                  tmp_roimap=fltarr(dims(1), dims(3), dims(2))
                  for i=0, dims(2)-1 do begin
                    tmp_arr2(*,*,i)=tmp_arr(*,i,*)
                    tmp_roimap(*,*,i)=roimap(*,i,*)
                  endfor
                  roi_mask=where(tmp_roimap eq 1)
                  tmp_arr2(roi_mask)=tmp_arr2(roi_mask)- $
                      total(tmp_arr2(roi_mask))/n_elements(roi_mask)
                  plot, tmp_arr2(roi_mask)
                endif else begin
                  print, "Error: No fit has been performed..."
                endelse
             END

   'Z_PLOT': BEGIN
                if (n_elements(calc_arr) gt 0) then begin
                  tmp_arr=diff_fieldmap-calc_arr
                  roi_mask=where(roimap eq 1)
                  tmp_arr(roi_mask)=tmp_arr(roi_mask)- $
                     total(tmp_arr(roi_mask))/n_elements(roi_mask)
                  plot, tmp_arr(roi_mask)
                endif else begin 
                  print, "Error: No fit has been performed..." 
                endelse
             END

    'DELTA_DAC': BEGIN
                WIDGET_CONTROL, event.id, GET_VALUE=temp_var
                dac_delta=float(temp_var(0))
             END 

    'NORM_TERM': BEGIN
                WIDGET_CONTROL, event.id, GET_VALUE=temp_var
                norm_term=fix(temp_var(0))
             END 

    'ELSE': BEGIN
            END

ENDCASE
END


PRO fit_coeff_win, GROUP=GROUP
; This is the procedure that creates a widget application

; The COMMON block is used because both main_win and 
; main_win_event must know the group leader.
COMMON groupleader, base
common cal_sys_bases, cal_sys_base, diff_fieldmap, calc_arr, nudge_im_slide
common fit_coeff_bases, fit_coeff_base, menu1_coeffs, menu2_coeffs, $
     dac_delta, norm_term

; A top-level base widget with the title "Pop-Up Widget Example" will
; hold the exclusive buttons:

menu1_coeffs=-1 ; default to start
menu2_coeffs=-1 ; default to start

fit_coeff_base = WIDGET_BASE( TITLE = 'FIT COEFFS', XOFFSET=1, YOFFSET=1, $
                /COLUMN)

coeff_arr1=['B0','x','y','z','z2','(x2+y2)','xy','zx','zy','(x2-y2)','z3']

coeff_arr2=['xz2','yz2','xy2','yx2','x(x2+y2)','y(x2+y2)','z(x2+y2)', $
   'z(x2-y2)', 'x3','y3', 'xyz']

inner_base=WIDGET_BASE(fit_coeff_base, /ROW)

menu1=CW_BGROUP(inner_base, UVALUE='MENU1', /COLUMN, /NONEXCLUSIVE, coeff_arr1)
menu2=CW_BGROUP(inner_base, UVALUE='MENU2', /COLUMN, /NONEXCLUSIVE, coeff_arr2)

if (n_elements(dac_delta) eq 0) then dac_delta=1.0 ; default
row1 = WIDGET_BASE(fit_coeff_base, /ROW, /FRAME)
namelabel1 = WIDGET_LABEL(row1, VALUE = 'Input Delta:')
name1 = WIDGET_TEXT(row1, /EDITABLE, XSIZE = 10, YSIZE = 1, $
                   UVALUE = 'DELTA_DAC', VALUE = strtrim(string(dac_delta),2))

if (n_elements(norm_term) eq 0) then norm_term=1 ; default
row2 = WIDGET_BASE(fit_coeff_base, /ROW, /FRAME)
namelabel2 = WIDGET_LABEL(row2, VALUE = 'Normalize to term:')
name2 = WIDGET_TEXT(row2, /EDITABLE, XSIZE = 6, YSIZE = 1, $
                   UVALUE = 'NORM_TERM', VALUE = strtrim(string(norm_term),2))

exe_button = WIDGET_BUTTON(fit_coeff_base, UVALUE='EXECUTE', value='execute')

xprof_button = WIDGET_BUTTON(fit_coeff_base, UVALUE='X_PLOT', $
    value='plot x variation')

yprof_button = WIDGET_BUTTON(fit_coeff_base, UVALUE='Y_PLOT', $
    value='plot y variation')

zprof_button = WIDGET_BUTTON(fit_coeff_base, UVALUE='Z_PLOT', $
    value='plot z variation')

canc_button = WIDGET_BUTTON(fit_coeff_base, UVALUE='CANCEL', value='cancel')

; Realize the widgets:
WIDGET_CONTROL, fit_coeff_base, /REALIZE

; Hand off control of the widget to the XMANAGER:
XMANAGER, "fit_coeff_win", fit_coeff_base, GROUP_LEADER=GROUP

END
