;***********************************************
; Written by James Wilson, Jan 00, so as to connect 
; ends of unconnected pi bounces to other ends
; or mask edge.
;***********************************************
pro join_boundary
common share1, dim0, dim1, dim2, diml, dimh, stindex, numfiles, $
           allmagimg,allmagmax,allmagmin
common shareunwrap, alignrow, aligncolumn, connectionmap, uncoloredmap,$
      colortab, colorlink, colorindex, colorindmap, totcolor, linkcord,$
      pointbak, slcn,map, max_colors, bounce_thresh, roiconnmap, connmap,$
      no_color_bounce
common shareroi, roimap, topb, botb, lefb, rightb
common share_edge, orig_edges, orig_connmap, min_edges, max_dist, edge_pass
common roi_def_info2, bounce_thresh_val, bounce_change_val, bounce_min_val,$
   min_edges_val, max_dist_val

on_error, 2  ; return to higher routine

; define a few variables
bounce_change=bounce_change_val
bounce_min=bounce_min_val
min_edges=fix(min_edges_val)
max_dist=fix(max_dist_val)

;save original connmap for this slice as orig_connmap
orig_connmap=connmap

;************************************************
;Complete any disconnections along phase bounces:
;************************************************
; Assumes that boundaries along phase bounces are almost complete
; Find ends of unconnected boundaries
conn_0=intarr(dim0,dim1)
conn_1=intarr(dim0,dim1)
conn_2=intarr(dim0,dim1)
conn_3=intarr(dim0,dim1)
conn_0(*,*)=orig_connmap(0,*,*)
conn_1(*,*)=orig_connmap(1,*,*)
conn_2(*,*)=orig_connmap(2,*,*)
conn_3(*,*)=orig_connmap(3,*,*)
;0: 0d means that last interface is in dir 0 with no interface "beneath" it
end_bound0d=(((conn_0 eq 0) and (conn_3 eq 1) and $
              (shift(conn_0,0,1) eq 1) and $
              (shift(conn_3,1,0) eq 1)) $
         and ((conn_1 eq 0) or $
              (shift(conn_0,0,-1) eq 0) or $
              (shift(conn_1,1,0) eq 0)))
;1: 0u means that last interface is in dir 0 with no interface "above" it
end_bound0u=(((conn_0 eq 0) and (conn_1 eq 1) and $
              (shift(conn_0,0,-1) eq 1) and $
              (shift(conn_1,1,0) eq 1)) $
         and ((conn_3 eq 0) or $
              (shift(conn_0,0,1) eq 0) or $
              (shift(conn_3,1,0) eq 0)))
;2: 1l means that last interface is in dir 1 with no interface "left" of it
end_bound1l=(((conn_1 eq 0) and (conn_0 eq 1) and $
              (shift(conn_1,1,0) eq 1) and $
              (shift(conn_0,0,-1) eq 1)) $
         and ((conn_2 eq 0) or $
              (shift(conn_1,-1,0) eq 0) or $
              (shift(conn_2,0,-1) eq 0))) 
;3: 1r means that last interface is in dir 1 with no interface "right" of it
end_bound1r=(((conn_1 eq 0) and (conn_2 eq 1) and $
              (shift(conn_1,-1,0) eq 1) and $
              (shift(conn_2,0,-1) eq 1)) $ 
         and ((conn_0 eq 0) or $
              (shift(conn_1,1,0) eq 0) or $
              (shift(conn_0,0,-1) eq 0))) 

;if both end_bound0d and end_bound1r then not an end point
end_bound=((end_bound0d eq 1) or (end_bound0u eq 1) or $
           (end_bound1l eq 1) or (end_bound1r eq 1)) - $
          ((end_bound0d eq 1) and (end_bound1r eq 1))

; now only one of the above end_bound's is non-zero for any end
; Obtain coords for these end points
no_ends=total(end_bound)
end_cords=0
if (no_ends gt 0) then begin
  ; end_cords holds x-cord, y-cord and end interface
  end_cords=intarr(3,no_ends)
  end_cords(0,*)=fix(where(end_bound eq 1) - dim0*(where(end_bound eq 1)/dim0))
  end_cords(1,*)=fix(where(end_bound eq 1)/dim0)
  end_cords(2,*)=0 + (end_bound0u(end_cords(0,*),end_cords(1,*)) eq 1) $
                 + 2*(end_bound1l(end_cords(0,*),end_cords(1,*)) eq 1) $
                 + 3*(end_bound1r(end_cords(0,*),end_cords(1,*)) eq 1) 

  ; loop over each boundary end
  for i=0,no_ends-1 do begin
    orig_edges=intarr(3,10) -1
    edge_pass=0
    edge_link, end_cords(0,i), end_cords(1,i), end_cords(2,i), 1, 0
    ; if viable end-point then zoom in:
    if (edge_pass eq 1) then begin
      ; find opp dir for end_cords(2,i)
      if ((end_cords(2,i) eq 0) or (end_cords(2,i) eq 2)) then begin
        rev_edge_cord=end_cords(2,i)+1
      endif else begin
        rev_edge_cord=end_cords(2,i)-1
      endelse
      ; increase bounce_val so first value is with val=thresh
      bounce_val=bounce_thresh+bounce_change
      edge_pass=0
      while ((edge_pass eq 0) and (bounce_val ge bounce_min)) do begin
        bounce_val=bounce_val-bounce_change
        connect, bounce_val, 1
        ; with new connmap see if phase boundary now complete
        edge_link, end_cords(0,i), end_cords(1,i), rev_edge_cord, 0, 0 
       endwhile
    endif
  endfor
endif

end
