PRO load_base_win_event, event
; This is the event handler for a text widget with a label.

; Get the widget ID of any widget touched and put it into 'eventval'.
; Pressing the RETURN key inside a text widget generates an event:

common file_info, directory, filename, filename2, map_dim0, map_dim1, $
       map_slices, map2_dim0, map2_dim1, map2_slices, file_type, smooth_val
common misc_flags, data_loaded_flag, field_unwrapped_flag
common groupleader, base
common buttons, roi_button, cal_button
common load_bases, load_base, file_format_button
common sys_names_info, sys_names, curr_sys
common field_map_data, phsmap, allimg, fieldmap, strucmap

WIDGET_CONTROL, event.id, GET_UVALUE = eventval

; Perform different actions based upon 'eventval':

temp_var=string(0)

CASE eventval OF
	"DIRECTORY": BEGIN
		  ;A RETURN has been pressed in the Directory field
                  WIDGET_CONTROL, event.id, GET_VALUE=temp_var
                  directory=temp_var(0)
		  END

	"FILENAME": BEGIN
		  ;A RETURN has been pressed in the Filename field
                  WIDGET_CONTROL, event.id, GET_VALUE=temp_var
                  filename=temp_var(0)
		  END

	"SPEC_TYPE": BEGIN
                  curr_sys=curr_sys+1
                  if (curr_sys eq n_elements(sys_names(*,0))) then curr_sys=0
                  file_string=sys_names(curr_sys,0)
                  file_type=fix(sys_names(curr_sys,1))
                  WIDGET_CONTROL, file_format_button, SET_VALUE=file_string
		  END

        "SMOOTH":BEGIN
                  WIDGET_CONTROL, event.id, GET_VALUE=str
                  smooth_val=fix(str(0))
                  END

       "EXECUTE":BEGIN
                  WIDGET_CONTROL, load_base, SENSITIVE=0
                  ; execute loading in seperate code so that can also be
                  ; called from shimcalc_file.pro
                  execute_load                  
                  WIDGET_CONTROL, base, SENSITIVE=1
                  if (data_loaded_flag eq 1) then begin
                    WIDGET_CONTROL, roi_button, SENSITIVE=1
                    WIDGET_CONTROL, cal_button, SENSITIVE=1
                    fieldmap=0.0
                  endif else begin
                    WIDGET_CONTROL, roi_button, SENSITIVE=0
                    WIDGET_CONTROL, cal_button, SENSITIVE=0
                  endelse
                  WIDGET_CONTROL, load_base, /DESTROY
                  END 

	"CANCEL": BEGIN
                  WIDGET_CONTROL, base, SENSITIVE=1
                  WIDGET_CONTROL, load_base, /DESTROY
                  END 

ENDCASE

END



PRO load_base_win, GROUP = GROUP

common file_info, directory, filename, filename2, map_dim0, map_dim1, $
       map_slices, map2_dim0, map2_dim1, map2_slices, file_type, smooth_val
common misc_flags, data_loaded_flag, field_unwrapped_flag
common groupleader, base
common load_bases, load_base, file_format_button
common run_status, first_run
common sys_names_info, sys_names, curr_sys

if (first_run eq 1) then begin
  spawn, './find_file.scr ~/shimcalc_defaults.pro', defaults_flag
  if (fix(defaults_flag(0)) eq 1) then begin
    old_path=!PATH
    !PATH = '~:' + !PATH
    shimcalc_defaults                ; read in the users default values
    !PATH = old_path
  endif else begin
    sys_defaults                   ; read in the system default values
  endelse
endif

first_run=0 ; so that defaults file is only read in once

load_base = WIDGET_BASE(TITLE = 'LOAD BASE DATA', $
		   /COLUMN, $ ;Organize subsequent widgets in columns.
                   XOFFSET=1, YOFFSET=1)

; Make a ROW base widget so the label and text field can be
; side by side:

WIDGET_CONTROL, base, SENSITIVE=0

row1 = WIDGET_BASE(load_base, /ROW, /FRAME)
namelabel1 = WIDGET_LABEL(row1, VALUE = 'Directory:')
name1 = WIDGET_TEXT(row1, /EDITABLE, XSIZE = 30, YSIZE = 1, $
		   UVALUE = 'DIRECTORY', VALUE = directory)

row2 = WIDGET_BASE(load_base, /ROW, /FRAME)
namelabel2 = WIDGET_LABEL(row2, VALUE = ' Filename:')
name2 = WIDGET_TEXT(row2, /EDITABLE, XSIZE = 30, YSIZE = 1, $
		   UVALUE = 'FILENAME', VALUE = filename)

sys_dims=size(sys_names)
if (curr_sys ge sys_dims(1)) then curr_sys=0  ; in case wrong value read in

file_string=sys_names(curr_sys, 0)
file_type=fix(sys_names(curr_sys, 1))
file_format_button=WIDGET_BUTTON(load_base, UVALUE="SPEC_TYPE", $
    VALUE=file_string)

min_smooth=1
max_smooth=10
smooth_slide = WIDGET_SLIDER(load_base, minimum=min_smooth, xsize=300, $
    maximum=max_smooth, title='smooth value', /FRAME, UVALUE='SMOOTH', $
    value=smooth_val)

exec_button=WIDGET_BUTTON(load_base, UVALUE="EXECUTE", VALUE="execute")

canc_button=WIDGET_BUTTON(load_base, UVALUE="CANCEL", VALUE="cancel")

; Realize the widgets:
WIDGET_CONTROL, load_base, /REALIZE

; Hand off to the XMANAGER:
XMANAGER, 'load_base_win', load_base, GROUP_LEADER = base

END
