pro make_test_data

; this program craetes fake test Omega data for 3D shimming

dim0=fix(64)
dim1=fix(64)
dim2=fix(16)
dim3=2.0
slo=0.0
slsep=0.0
fov=100.0
fovz=100.0
te=7500.0
tintec=5800.0
plane=1.0
flipflag=0.0
oblangle=0.0

; set up the fake shim field:

x=float(indgen(dim0, 1, 1)-dim0/2)*fov/dim0
x=rebin(x, dim0, dim1, dim2)
 
y=float(indgen(1,dim1,1)-dim1/2)*fov/dim1
y=rebin(y, dim0, dim1, dim2)
 
z=(float(indgen(1,1,dim2))-dim2/2.0+0.5)*fovz/dim2+slo
z=rebin(z, dim0, dim1, dim2)

shim=fltarr(dim0, dim1, dim2)

; fake z term
tmp_shim=500.0*z*(-2.9862*0.0058)
shim=shim+tmp_shim

; fake z2 term
tmp_shim=-500.0*(-21.0*z + z*z - 0.61*(x*x+y*y))*(0.11034*0.0058)
shim=shim+tmp_shim

; fake x term
tmp_shim=500.0*y*(-2.8345*0.0058)
shim=shim+tmp_shim

; fake y term
tmp_shim=-500.0*x*(2.8397*0.0058)
shim=shim+tmp_shim

; fake zx term
tmp_shim=500.0*(3.72*x + 3.35*y + 13.38*z - 0.29*x*y + z*y)*(0.09460*0.0058)
shim=shim+tmp_shim

; fake zy term
tmp_shim=-500.0*(7.98*x - 3.17*y + 1.43*z + z*x - 0.511*z*y)*(0.11419*0.0058)
shim=shim+tmp_shim

; fake xy term
tmp_shim=500.0*(-2.32*x + 4.79*y - 1.11*z + x*y -0.525*(x*x-y*y))*(0.08672*0.0058)
shim=shim+tmp_shim

; fake x2-y2 term
tmp_shim=500.0*(1.61*x + 3.07*y + 1.3*z + 0.0495*z*z + 1.55*x*y $
                      + (x*x-y*y))*(0.04924*0.0058)
shim=shim+tmp_shim

; fake z2x term
tmp_shim=-500.0*(-38.0*x - 267.0*y + 36.5*z + y*z*z)*((-0.0114/10.0)*0.0058)
shim=shim+tmp_shim

; fake z2y term
tmp_shim=500.0*(-175.0*x + 34.0*y + x*z*z)*((-0.0121/10.0)*0.0058)
shim=shim+tmp_shim

; fake zxy term
tmp_shim=-500.0*(101.0*x + 6.8*y - 2.6*(x*x-y*y) - 0.47*z*(x*x-y*y) $
                         + x*y*z)*((0.0103/10.0)*0.0058)
shim=shim+tmp_shim

; fake z(x2-y2) term
tmp_shim=500.0*(162.6*x + 8.5*x*y + 7.83*(x*x-y*y) + z*(x*x-y*y) $
                       + 1.42*x*y*z)*((-5.78e-3/10.0)*0.0058)
shim=shim+tmp_shim

; make fake data
im1=fltarr(dim0, dim1, dim2)+1.0
for i=0, dim2-1 do begin
  small_radius=sin(!pi*float(i)/float(dim2-1))*0.42*float(dim1)
  for j=0, dim0-1 do begin
    for k=0, dim1-1 do begin
      if (((float(j-dim0/2))^2+(float(k-dim1/2))^2) le small_radius^2) then $
        im1(j,k,i)=100.0
    endfor
  endfor
endfor

im1=complex(im1, fltarr(dim0, dim1, dim2))

shim_phase=shim*tintec/1000000.0
print, "maximum phase = ", max(shim_phase)
print, "minimum phase = ", min(shim_phase)
re_im2=float(im1)*cos(shim_phase)+imaginary(im1)*sin(shim_phase)
im_im2=-float(im1)*sin(shim_phase)+imaginary(im1)*cos(shim_phase)
im2=complex(re_im2, im_im2)

allimg=complexarr(dim0, dim1, dim2, dim3)
allimg(*,*,*,0)=im1
allimg(*,*,*,1)=im2

;***********************************************************************
; Put in the phase switching of the Omega images.
;***********************************************************************
  phtemp1=(-1)^indgen(dim2)
  phtemp2=(-1)^indgen(dim1)
  phtemp3=(-1)^indgen(dim0)

  phasecor=intarr(dim0,dim1,dim2)
  phasecor(*,*,0)=phtemp3#phtemp2

  for i=1, dim2-1 do begin
  phasecor(*,*, i)=-phasecor(*,*, i-1)
  endfor

  for i=0, dim3-1 do begin
    allimg(*,*,*,i)=allimg(*,*,*,i)*phasecor
  endfor

;***********************************************************************

;******************************
;write out complex image files
;******************************

out_file="test_data.raw"

get_lun, unit
openw, unit, out_file
printf, unit, dim0, dim1, dim2, dim3, slo, slsep, fov, fovz, $
   te, tintec,  plane, flipflag, oblangle

; write out the data
writeu, unit, allimg
close, unit
free_lun, unit 

end
