function prepare_signa_data, directory, fname

on_error, 2
on_ioerror, problem_with_files

out_dim=64  ; output dimension of data

; /****************************************************************/"
; /* Program to create field map files to be read in to shimcalc  */"
; /* You should have a set of P files from Signa, each file       */"
; /* corresponding to a component of a set of raw shim scans with */"
; /* the dual echo raw data collected using the 2 echo GRE seq    */"
; /* This program will take those files as input and will output  */"
; /* field map files which can be read by shimcalc                */"
; /*                Peter Jezzard, NIH, Feb 1997                  */"
; /****************************************************************/"

out_file="tmp.fieldmap"

err_code=0 ; default is correct exit...

spawn, "find_file.scr "+directory+"/"+fname, file_exists
if (fix(file_exists(0)) eq 0) then begin
  print, "file "+directory+"/"+fname+" does not exist"
  print, "exiting..."
  err_code=-1
  goto, end_prog
endif

print, "reading file information..."

;===========================================================================
; open file and look at the image acquisition information
;===========================================================================

head1=bytarr(38916)  ; to skip to the rdb_hdr_image field
head2=bytarr(26) ; to skip the first unwanted entries in rdb_hdr_image
head3=bytarr(76) ; to skip more unwanted entries in rdb_hdr_image
head4=bytarr(74) ; to skip more unwanted entries in rdb_hdr_image
head5=bytarr(32) ; to skip more unwanted entries in rdb_hdr_image
head6=bytarr(134) ; to skip the other unwanted entries in rdb_hdr_image
temp_int=long(0) ; dummy long int
temp_short=fix(0) ; dummy short
temp_float=float(0) ; dummy float

get_lun, unit
openr, unit, directory+'/'+fname
  readu, unit, head1, head2

  readu, unit, temp_float
  slthick=temp_float

  readu, unit, temp_short ; skip
  readu, unit, temp_short ; skip

  readu, unit, temp_float
  opfov=temp_float

  readu, unit, head3 ; skip

  readu, unit, temp_short
  opplane=temp_short

  readu, unit, temp_float
  slspace=temp_float

  readu, unit, head4 ; skip

  readu, unit, temp_int
  tr=temp_int

  readu, unit, temp_int
  ti=temp_int

  readu, unit, temp_int
  te=temp_int

  readu, unit, temp_int
  te2=temp_int

  readu, unit, temp_short
  num_echoes=temp_short

  readu, unit, temp_short ; skip
  readu, unit, temp_float ; skip

  readu, unit, temp_float
  nex=temp_float

  readu, unit, head5 ; skip

  readu, unit, temp_short
  mr_flip=temp_short

  readu, unit, temp_int ; skip
  readu, unit, temp_short ; skip

  readu, unit, temp_short
  opswappf=temp_short  ; whether phase and frequency were swapped

  readu, unit, head6 ; skip

  readu, unit, temp_short
  slquant=temp_short
close, unit
free_lun, unit

;===========================================================================
; open file again and look at the pass and image size information...
;===========================================================================

head=bytarr(64)
head2=bytarr(22)
get_lun, unit
openr, unit, directory+'/'+fname
  readu, unit, head

  readu, unit, temp_short
  rhnpasses=temp_short   ; number of passes
  readu, unit, temp_short
  readu, unit, temp_short
  readu, unit, temp_short
  rhnecho=temp_short     ; number of echoes per slice
  readu, unit, temp_short

  readu, unit, temp_short
  dim1=temp_short

  readu, unit, temp_short ; skip this
  rhbline=temp_short
  readu, unit, temp_short ; skip this

  readu, unit, temp_short
  dim0=temp_short

  readu, unit, head2 ; skip this
  
  readu, unit, temp_short
  rhdayres=temp_short
close, unit
free_lun, unit

;===========================================================================
; open file again and look at the slice location information...
;===========================================================================

head=bytarr(10240)
slice_found_in_pass=intarr(slquant)
slice_pos_in_pass=intarr(slquant)
slice_pos=fltarr(slquant)
temp_slice_info=intarr(2)
pos_info=fltarr(9)

get_lun, unit
openr, unit, directory+'/'+fname
  readu, unit, head

  for i=0, slquant-1 do begin
    readu, unit, temp_slice_info
    readu, unit, pos_info
    slice_found_in_pass(i)=temp_slice_info(0)
    slice_pos_in_pass(i)=temp_slice_info(1)
    slice_pos(i)=pos_info(2)
  endfor

close, unit
free_lun, unit

tot_num_passes=max(slice_found_in_pass+1)
tot_num_slices_per_pass=max(slice_pos_in_pass)

print, ""
print, "slices = "+strtrim(string(slquant), 2)
print, "tr     = "+strtrim(string(tr/1000.0), 2)+" ms"
print, "te     = "+strtrim(string(te/1000.0), 2)+" ms"
print, "te2    = "+strtrim(string(te2/1000.0), 2)+" ms"
print, "echoes = "+strtrim(string(num_echoes), 2)
print, "nex    = "+strtrim(string(nex), 2)
print, "flip   = "+strtrim(string(mr_flip), 2)+" degrees"
print, "fov    = "+strtrim(string(opfov), 2)+" mm"
print, "slthk  = "+strtrim(string(slthick), 2)+" mm"
print, "slspac = "+strtrim(string(slspace), 2)+" mm"
print, "xres   = "+strtrim(string(dim0), 2)
print, "yres   = "+strtrim(string(dim1), 2)
print, "passes = "+strtrim(string(tot_num_passes), 2)
print, "plane  = "+strtrim(string(opplane), 2)
print, "first slice = "+strtrim(string(slice_pos(0)), 2)+" mm"
print, "last slice  = "+strtrim(string(slice_pos(slquant-1)), 2)+" mm"

print, ""
print, "note the echo time difference for this scan : "+ $
               strtrim(string(te2-te), 2)+" us"

if ((slquant lt 1) or (slquant > 512)) then begin
  print, ""
  print, "Error: slices is out of range (0 - 512)"
  print, "Exiting..."
  err_code=-1
  goto, end_prog
endif

if ((dim0 lt 32) or (dim0 > 1024)) then begin
  print, ""
  print, "Error: xres is out of range (32 - 1024)"
  print, "Exiting..."
  err_code=-1
  goto, end_prog
endif

if ((dim1 lt 32) or (dim1 > 1024)) then begin
  print, ""
  print, "Error: yres is out of range (32 - 1024)"
  print, "Exiting..."
  err_code=-1
  goto, end_prog
endif

if (slice_pos(0) eq slice_pos(slquant-1)) then begin
  print, ""
  print, "Error: shim data must be collected in the axial plane"
  print, "Exiting..."
  err_code=-1
  goto, end_prog    
endif

read_baseline=0 ; default is that no baseline scan is collected
if (rhdayres > dim1) then read_baseline = 1

;===========================================================================
; now read the data from the raw files
;===========================================================================

base_scan=intarr(2, dim0)
im=intarr(2, dim0, dim1)
big_im=complexarr(out_dim, out_dim, slquant, num_echoes)
head=bytarr(39940)

start_file_val=strmid(fname, 1, 5)
start_file_val=long(start_file_val)

fext="" ; default
if (start_file_val+rhnpasses-1 lt 10000) then fext='0'
if (start_file_val+rhnpasses-1 lt 1000) then fext='00'
if (start_file_val+rhnpasses-1 lt 100) then fext='000'
if (start_file_val+rhnpasses-1 lt 10) then fext='0000'

last_file_name="P"+fext+strtrim(string(start_file_val+rhnpasses-1), 2)
spawn, "find_file.scr "+directory+"/"+last_file_name, file_exists
if (fix(file_exists(0)) eq 0) then begin
  print, "Error: file "+directory+"/"+last_file_name+" does not exist"
  print, "exiting..."
  err_code=-1
  goto, end_prog
endif

print, ""
for pass_no=0, tot_num_passes-1 do begin

  fext="" ; default
  if (start_file_val+pass_no lt 10000) then fext='0'
  if (start_file_val+pass_no lt 1000) then fext='00'
  if (start_file_val+pass_no lt 100) then fext='000'
  if (start_file_val+pass_no lt 10) then fext='0000'

  get_lun, unit

  print, "reading from file "+directory+"/"+"P"+fext+ $
               strtrim(string(start_file_val+pass_no), 2)
  openr, unit, directory+"/"+"P"+fext+strtrim(string(start_file_val+pass_no), 2)

  readu, unit, head

  for im_no=0, tot_num_slices_per_pass-1 do begin

    poss_passes=where(slice_found_in_pass eq pass_no)
    poss_pass_positions=where(slice_pos_in_pass eq (im_no+1))
    curr_slice=-1
    for xx=0, n_elements(poss_passes)-1 do begin
      test_equal=where(poss_pass_positions eq poss_passes(xx))
      if (test_equal(0) ne -1) then curr_slice=poss_passes(xx)
    endfor

    if (curr_slice ne -1) then begin
      for echo_no=0, num_echoes-1 do begin

        if (read_baseline eq 1) then readu, unit, base_scan
        readu, unit, im ; read first raw image

        re_base=base_scan(0,*)
        im_base=base_scan(1,*)
        comp_base=complex(re_base, im_base)
        comp_base=reform(comp_base, dim0) ; complex baseline for image

        re_im=im(0,*,*)
        im_im=im(1,*,*)
        comp_im=complex(re_im, im_im)
        comp_im=reform(comp_im, dim0, dim1)

        if ((out_dim ge dim0) and (out_dim ge dim1)) then $
          big_im(out_dim/2-dim0/2:out_dim/2+dim0/2-1, $
            out_dim/2-dim1/2:out_dim/2+dim1/2-1,curr_slice,echo_no)=comp_im

        if ((out_dim lt dim0) and (out_dim ge dim1)) then $
          big_im(*,out_dim/2-dim1/2:out_dim/2+dim1/2-1,curr_slice,echo_no)= $
           comp_im(dim0/2-out_dim/2:dim0/2+out_dim/2-1,*)

        if ((out_dim ge dim0) and (out_dim lt dim1)) then $
          big_im(out_dim/2-dim0/2:out_dim/2+dim0/2-1,*, $
           curr_slice,echo_no)=comp_im(*,dim1/2-out_dim/2:dim1/2+out_dim/2-1)

        if ((out_dim lt dim0) and (out_dim lt dim1)) then $
          big_im(*,*,curr_slice,echo_no)= $
           comp_im(dim0/2-out_dim/2:dim0/2+out_dim/2-1, $
           dim1/2-out_dim/2:dim1/2+out_dim/2-1)

      endfor
    endif

  endfor

  close, unit
  free_lun, unit

endfor

;===========================================================================
; need to flip phase and frequency directions if opswappf is on
;===========================================================================

if (opswappf eq 1) then begin
  print, "swapping phase and frequency directions..."
  for i=0, slquant-1 do begin
    for j=0, num_echoes-1 do begin
      big_im(*,*,i,j)=rotate(big_im(*,*,i,j), 4)
    endfor
  endfor
endif

;===========================================================================
; need to make sure that z direction coverage is consistently from neg to pos
;===========================================================================

if (slice_pos(slquant-1) lt slice_pos(0)) then begin
  print, "reordering slices to run from negative z to positive z..."
  for i=0, slquant/2-1 do begin
    for j=0, num_echoes-1 do begin
      temp_im=big_im(*,*,i,j)
      big_im(*,*,i,j)=big_im(*,*,slquant-1-i,j)
      big_im(*,*,slquant-1-i,j)=temp_im
    endfor
  endfor
endif

;===========================================================================
; Fourier transform the data
;===========================================================================

print, "Fourier transforming the data..."
for i=0, slquant-1 do begin
  for j=0, num_echoes-1 do begin
    big_im(*,*,i,j)=shift(fft(shift(big_im(*,*,i,j), out_dim/2,out_dim/2), 1), $
       out_dim/2, 0)
  endfor
endfor

;===========================================================================
; write out the data
;===========================================================================

; set up the values to appear in the header for shimcalc program
dim0=fix(out_dim)
dim1=fix(out_dim)
dim2=fix(slquant)
dim3=float(num_echoes)
slo=float(max(slice_pos)-min(slice_pos))/2.0+float(min(slice_pos))
slsep=float(slthick+slspace)
fov=float(opfov)
fovz=float(max(slice_pos)-min(slice_pos))*float(dim2)/float(dim2-1)
te=float(te)
tintec=float(te2-te)
plane=float(1) ; always axial
flipflag=float(opswappf)
oblangle=float(0) ; never oblique

print, "writing out file "+directory+"/"+out_file
get_lun, unit
openw, unit, directory+"/"+out_file
printf, unit, dim0, dim1, dim2, dim3, slo, slsep, fov, fovz, $
   te, tintec,  plane, flipflag, oblangle
writeu, unit, big_im
close, unit
free_lun, unit

goto, end_prog  ; for normal exit

problem_with_files:
  print, "Error. Problem reading file..."
  print, "exiting..."
  err_code=-1

end_prog:

return, err_code

end
