function read_analyse_data, directory, in_file, smooth_val

; this function reads in Analyse data for 3D shimming
; the files that are required are:
;   in_file+'_struc.img'
;   in_file+'_1.img' - this is optional
;   in_file+'_2.img'
;   in_file+'.header' - where asym time should be set to TE

common roi_def_info, thresh_val, erosion_val, start_im, stop_im, draw_im,$
   num_slices, drawn_roi, dilation_val, edit_im, nudge_im
common share1, dim0, dim1, dim2, diml, dimh, stindex, numfiles, $
           allmagimg,allmagmax,allmagmin
common shareroi, roimap, topb, botb, lefb, rightb
common shareshim, slo, slsep, fov, fovz, te, plane
common field_map_data, phsmap, allimg, fieldmap, strucmap

on_error, 2
on_ioerror, bad_file

dim0=fix(256)
dim1=fix(256)
dim2=fix(1)

;******************************
;Read in image files
;******************************

print, "trying to read "+in_file+".header"
get_lun, unit
openr, unit, directory+'/'+in_file+".header"
readf, unit, dim0, dim1, dim2, dim3, slo, slsep, fov, fovz, $
   te, tintec, plane, flipflag, oblangle
close, unit
free_lun, unit

; initialize some of the arrays and variables based on the dimensions read in
numfiles=dim3
roimap=intarr(dim0, dim1, dim2)+1  ; set all to 1 to start with

; check all files are there
no_phase1=0
spawn, 'find_file.scr '+directory+'/'+in_file+'_struc.img', flag_struc
spawn, 'find_file.scr '+directory+'/'+in_file+'_1.img', flag_1
spawn, 'find_file.scr '+directory+'/'+in_file+'_2.img', flag_2
if (fix(flag_struc(0)) ne 1) then begin
  print, 'Error: File '+directory+'/'+in_file+'_struc.img not found'
  goto, bad_file
endif
if (fix(flag_2(0)) ne 1) then begin
  print, 'Error: File '+directory+'/'+in_file+'_2.img not found'
  goto, bad_file
endif
if (fix(flag_1(0)) ne 1) then begin
  no_phase1=1
endif

; read in the data
; first the structural data
allimg=fltarr(dim0,dim1,dim2,dim3)
strucmap=fltarr(dim0,dim1,dim2)
print, "trying to read "+in_file+"_struc.img"
get_lun, unit
openr, unit, directory+'/'+in_file+"_struc.img"
readu, unit, strucmap
close, unit
free_lun, unit
allimg(*,*,*,0)=strucmap(*,*,*)

; now the second phase map
phsmap=fltarr(dim0, dim1, dim2, dim3)
phasemap2=fltarr(dim0,dim1,dim2)
print, "trying to read "+in_file+"_2.img"
get_lun, unit
openr, unit, directory+'/'+in_file+"_2.img"
readu, unit, phasemap2
close, unit
free_lun, unit
phsmap(*,*,*,1)=phasemap2(*,*,*)

; now the first phase map - if it exists
if (no_phase1 ne 1) then begin 
  phasemap1=fltarr(dim0,dim1,dim2)
  print, "trying to read "+in_file+"_1.img"
  get_lun, unit
  openr, unit, directory+'/'+in_file+"_1.img"
  readu, unit, phasemap1
  close, unit
  free_lun, unit
  phsmap(*,*,*,0)=phasemap1(*,*,*)
endif else begin
  phsmap(*,*,*,0)=0.0
endelse

if (total(allimg) eq -1) then goto, bad_file
if (total(phsmap) eq -1) then goto, bad_file

if (dim3 eq 1) then begin
  te=te/1000000.0
endif else begin
  te=tintec/1000000.0
endelse
print, "inter echo delay = "+strtrim(string(te), 2)+" secs"

print, ""
if (smooth_val gt 1) then begin  ; smooth the structural data only
  print, "smoothing data"
  for i=0, dim2-1 do begin
    structemp=allimg(*,*,i,0)
    structemp=smooth(structemp, smooth_val)
    allimg(*,*,i,0)=structemp
  endfor
endif

for j=0, dim3-1 do begin
  print, "phase data from echo number "+strtrim(string(j+1), 2)+"..."
  err=display_ims(bytscl(phsmap(*,*,*,j)))
  wait, 1
endfor

print, "magnitude data..."
strucmap=fltarr(dim0, dim1,dim2)
strucmap(*,*,*)=abs(allimg(*,*,*,0))
tmp_im=bytscl(strucmap)
err=display_ims(tmp_im)

; initialize a few variables for later
stop_im=fix(dim2)
num_slices=fix(dim2)
if (n_elements(draw_im) lt 1) then begin
  draw_im=num_slices/2
  if (num_slices eq 1) then draw_im=1
endif else begin
  if (draw_im gt num_slices) then draw_im=num_slices/2
endelse
edit_im=draw_im
nudge_im=draw_im

return, 0  ; 0 indicates that read_analyse_data made it through function

bad_file:

return, -1

end
