function read_omega_data, in_file, smooth_val

; this function reads in Omega data for 3D shimming

common roi_def_info, thresh_val, erosion_val, start_im, stop_im, draw_im,$
   num_slices, drawn_roi, dilation_val, edit_im, nudge_im
common share1, dim0, dim1, dim2, diml, dimh, stindex, numfiles, $
           allmagimg,allmagmax,allmagmin
common shareroi, roimap, topb, botb, lefb, rightb
common shareshim, slo, slsep, fov, fovz, te, plane
common field_map_data, phsmap, allimg, fieldmap

on_error, 2
on_ioerror, bad_file

dim0=fix(256)
dim1=fix(256)
dim2=fix(1)

iii=complex(0,1)

;******************************
;Read in complex image files
;******************************

get_lun, unit
openr, unit, in_file
readf, unit, dim0, dim1, dim2, dim3, slo, slsep, fov, fovz, $
   te, tintec,  plane, flipflag, oblangle

; initialize some of the arrays and variables based on the dimensions read in
allimg=complexarr(dim0, dim1, dim2, dim3)
numfiles=dim3
roimap=intarr(dim0, dim1, dim2)+1  ; set all to 1 to start with

; read in the data
readu, unit, allimg
close, unit
free_lun, unit 

;*************************************************************************
; when num_slices = 32 the Omega export program seems to screw up, so
; reorder the data to compensate...
if (dim2 gt 16) then begin
  tmp_arr=allimg(*,*,dim2/2:dim2-1,0)
  allimg(*,*,dim2/2:dim2-1,0)=allimg(*,*,0:dim2/2-1,1)
  allimg(*,*,0:dim2/2-1,1)=tmp_arr
endif
;*************************************************************************

if (dim3 eq 1) then begin
  te=te/1000000.0
endif else begin
  te=tintec/1000000.0
endelse
print, "inter echo delay = "+strtrim(string(te), 2)+" secs"

;***********************************************************************
;***********************************************************************
;Correct phase switching of the Omega images.
;***********************************************************************
  phtemp1=(-1)^indgen(dim2)
  phtemp2=(-1)^indgen(dim1)
  phtemp3=(-1)^indgen(dim0)

  phasecor=intarr(dim0,dim1,dim2)
  phasecor(*,*,0)=phtemp3#phtemp2

  for i=1, dim2-1 do begin
  phasecor(*,*, i)=-phasecor(*,*, i-1)
  endfor

  for i=0, dim3-1 do begin
    allimg(*,*,*,i)=allimg(*,*,*,i)*phasecor
  endfor

;***********************************************************************
;***********************************************************************

print, ""
if (smooth_val gt 1) then begin  ; smooth the data
  print, "smoothing data"
  for i=0, dim2-1 do begin
    for j=0, dim3-1 do begin
      temp=allimg(*,*,i,j)
      temp=complex(smooth(float(temp), smooth_val), $
        smooth(imaginary(temp), smooth_val))
      allimg(*,*,i,j)=temp
    endfor
  endfor
endif

phsmap=fltarr(dim0, dim1, dim2, dim3)

for i=0, dim3-1 do begin
  phsmap(*,*,*,i)=acos(float(allimg(*,*,*,i))/abs(allimg(*,*,*,i)))
  phsmap(*,*,*,i)=phsmap(*,*,*,i)*2.0*((float(-allimg(*,*,*,i)*iii) gt 0.0)-0.5)
endfor

for j=0, dim3-1 do begin
  print, "phase data from echo number "+strtrim(string(j+1), 2)+"..."
  err=display_ims(bytscl(phsmap(*,*,*,j)))
  wait, 2
endfor

print, "magnitude data..."
tmp_im=bytscl(abs(allimg(*,*,*,0)))
err=display_ims(tmp_im)

; initialize a few variables for later
if (n_elements(stop_im) lt 1) then stop_im=fix(dim2)
num_slices=fix(dim2)
if (n_elements(draw_im) lt 1) then begin
  draw_im=num_slices/2
  if (num_slices eq 1) then draw_im=1
endif else begin
  if (draw_im gt num_slices) then draw_im=num_slices/2
endelse
edit_im=draw_im
nudge_im=draw_im

return, 0  ; 0 indicates that read_omega_data made it through function

bad_file:

return, -1

end
