function read_varian_data, directory, in_file, smooth_val

; this function reads in Varian data for 3D shimming
; assumes that data has been preprocessed to get it into a fieldmap format.
; At present read_varian_bin_data.pro and binary_to_fieldmap.pro are used to
; accomplish this.

common roi_def_info, thresh_val, erosion_val, start_im, stop_im, draw_im,$
   num_slices, drawn_roi, dilation_val, edit_im, nudge_im
common share1, dim0, dim1, dim2, diml, dimh, stindex, numfiles, $
           allmagimg,allmagmax,allmagmin
common shareroi, roimap, topb, botb, lefb, rightb
common shareshim, slo, slsep, fov, fovz, te, plane
common field_map_data, phsmap, allimg, fieldmap, strucmap

on_error, 2
on_ioerror, bad_file

;******************************************************
; first put the varian data in a form that can be read
;******************************************************

iii=complex(0,1)

dim0=fix(256)
dim1=fix(256)
dim2=fix(1)

;******************************
;Read in complex image files
;******************************

print, "trying to read "+in_file+".header"
get_lun, unit
openr, unit, directory+'/'+in_file+".header"
readf, unit, dim0, dim1, dim2, dim3, slo, slsep, fov, fovz, $
   te, tintec, plane, flipflag, oblangle
close, unit
free_lun, unit

; initialize some of the arrays and variables based on the dimensions read in
numfiles=dim3
roimap=intarr(dim0, dim1, dim2)+1  ; set all to 1 to start with

; read in the data

; this line is suitable for asymmetric spin echo sequences
allimg=fieldmap_ft(varian_bin_to_fieldmap2(read_varian_bin_data(directory+ $
   "/"+in_file), dim0*2,dim1,dim2, dim3, baseline=1, interleave=0))

if (total(allimg) eq -1) then goto, bad_file

if (dim3 eq 1) then begin
  te=te/1000000.0
endif else begin
  te=tintec/1000000.0
endelse
print, "inter echo delay = "+strtrim(string(te), 2)+" secs"

;save structural map
strucmap=fltarr(dim0, dim1, dim2)
strucmap(*,*,*)=abs(allimg(*,*,*,0))

print, ""
if (smooth_val gt 1) then begin  ; smooth the data
  print, "smoothing data"
  for i=0, dim2-1 do begin
    for j=0, dim3-1 do begin
      temp=allimg(*,*,i,j)
      temp=complex(smooth(float(temp), smooth_val), $
        smooth(imaginary(temp), smooth_val))
      allimg(*,*,i,j)=temp
    endfor
  endfor
endif

phsmap=fltarr(dim0, dim1, dim2, dim3)

for i=0, dim3-1 do begin
  phsmap(*,*,*,i)=acos(float(allimg(*,*,*,i))/abs(allimg(*,*,*,i)))
  phsmap(*,*,*,i)=phsmap(*,*,*,i)*2.0*((float(-allimg(*,*,*,i)*iii) gt 0.0)-0.5)
endfor

print, "magnitude data..."
tmp_im=bytscl(abs(allimg(*,*,*,0)))
err=display_ims(tmp_im)

; initialize a few variables for later
stop_im=fix(dim2)
num_slices=fix(dim2)
if (n_elements(draw_im) lt 1) then begin
  draw_im=num_slices/2
  if (num_slices eq 1) then draw_im=1
endif else begin
  if (draw_im gt num_slices) then draw_im=num_slices/2
endelse
edit_im=draw_im
nudge_im=draw_im

return, 0  ; 0 indicates that read_varian_data made it through function

bad_file:

return, -1

end
