function varian_bin_to_fieldmap2, in_arr, npoints, nviews, nslices, nechoes, $
    verbose=verbose, baseline=baseline, interleaved=interleaved

; this version of varian_bin_to_fieldmap should be used when the te (in the
; case of a gradient echo sequence) or the asym_time (in the case of a
; spin echo sequence) has been arrayed to create the multi echo data set...

if (n_elements(npoints) eq 0) then npoints=0
if (n_elements(nviews) eq 0) then nviews=0
if (n_elements(nslices) eq 0) then nslices=0
if (n_elements(nechoes) eq 0) then nechoes=0
if (n_elements(verbose) eq 0) then verbose=0
if (n_elements(baseline) eq 0) then baseline=0
if (n_elements(interleaved) eq 0) then interleaved=0

usage="out_arr=binary_to_fieldmap2(in_arr, npoints, nviews, nslices, nechoes[, verbose=0|1, baseline=0|1])"

dims=size(in_arr)
if (dims(dims(0)+2) ne long(npoints)*long(nviews)*long(nslices)* $
           long(nechoes)) then begin
  print, "Error: incorrect dimensions specified for input array"
  print, "usage: "+usage
  return, -1
endif
if (dims(dims(0)) ne long(nviews)*long(nechoes)) then begin
  print, "Error: incorrect dimensions specified for input array"
  print, "usage: "+usage
  return, -1
endif

if (verbose eq 1) then begin
  print, "npoints = "+strtrim(string(npoints),2)
  print, "nviews = "+strtrim(string(nviews),2)
  print, "nslices = "+strtrim(string(nslices),2)
  print, "nechoes = "+strtrim(string(nechoes),2)
endif

; assume that data is in the form in_arr(2*npoints/2*nslices, nviews*nechoes)

if (dims(dims(0)+1) eq 3) then begin ; long integer words
  out_arr=lonarr(2,npoints/2,nviews,nslices,nechoes)
endif
if (dims(dims(0)+1) eq 2) then begin ; short integer words
  out_arr=intarr(2,npoints/2,nviews,nslices,nechoes)
endif
if (dims(dims(0)+1) eq 4) then begin ; floating point words
  out_arr=fltarr(2,npoints/2,nviews,nslices,nechoes)
endif

for view_no=0, nviews-1 do begin
  for echo_no=0, nechoes-1 do begin
    tmp_trace=in_arr(*,view_no*nechoes+echo_no)
    ; reform the in_arr to separate the real and imaginary variables
    tmp_trace=reform(tmp_trace, 2,long(npoints/2)*long(nslices))
    for slice_no=0, nslices-1 do begin
      sl_offset=long(slice_no)*long(npoints/2)
      out_arr(*,*,view_no,slice_no,echo_no)=tmp_trace(*, $
           sl_offset:sl_offset+long(npoints/2)-1)
    endfor
  endfor
endfor

out_arr=reform(complex(out_arr(0,*,*,*,*),out_arr(1,*,*,*,*)))
out_arr=reform(out_arr,npoints/2,nviews,nslices,nechoes)

if (baseline eq 1) then begin
  if (verbose gt 0) then print, "baseline correcting"
  for echo_no=0, nechoes-1 do begin
    for slice_no=0, nslices-1 do begin
      ; calculate baseline offset from last k space line collected
      base_line=out_arr(npoints/2-1,*,slice_no,echo_no)
      base_line=total(base_line)/nviews
      out_arr(*,*,slice_no,echo_no)=out_arr(*,*,slice_no,echo_no)-base_line
    endfor
  endfor
endif

; sort out the interleaved slices if interleave=1
cnt=0
if (interleaved eq 1) then begin
  new_out_arr=out_arr
  for slice_no=0, nslices-1 do begin
    new_slice=slice_no*2
    if (new_slice ge nslices) then begin
      new_slice=(slice_no-cnt)*2+1
    endif else begin
      cnt=cnt+1
    endelse
    new_out_arr(*,*,new_slice,*)=out_arr(*,*,slice_no,*)
  endfor
  out_arr=new_out_arr
endif

return, out_arr

end
