function display_ims, in_arr

; function to display a set of images
; in_arr should be a bytarr that is already grey-scale adjusted for display
; e.g., err=display_ims(bytscl(im_arr))

on_error, 2

if (n_elements(in_arr) eq 0) then begin
  print, "Error: no images to display"
  return, -1
endif

dims=size(in_arr)

if (dims(0) eq 2) then begin
  tv, in_arr
  return, 0
endif

if (dims(0) ne 3) then begin
  print, "Error: display array must be 2 or 3D bytarr"
  return, -1
endif

screen_xsize=!d.x_vsize
screen_ysize=!d.y_vsize

x_im_size=dims(1)
y_im_size=dims(2)

if (x_im_size lt 128) then x_im_size=128
if (y_im_size lt 128) then y_im_size=128

num_ims_x=fix(float(screen_xsize)/float(x_im_size))
num_ims_y=fix(float(screen_ysize)/float(x_im_size))

num_screens=fix(float(dims(3))/(num_ims_x*num_ims_y))
if (float(dims(3))/(num_ims_x*num_ims_y) gt float(num_screens)+1) then begin
  num_screens=num_screens+1
endif
if (num_screens eq 0) then num_screens=1

for screen_no=0, num_screens-1 do begin
  erase
  !err=0
  for y_no=0, num_ims_y-1 do begin
    y_coord=screen_ysize-(y_no+1)*y_im_size
    for x_no=0, num_ims_x-1 do begin
      x_coord=x_no*x_im_size
      im_indx=screen_no*num_ims_x*num_ims_y+y_no*num_ims_x+x_no
      if (im_indx lt dims(3)) then begin
        tv, rebin(in_arr(*,*,im_indx),x_im_size, y_im_size, sample=1), $
             y_no*num_ims_x+x_no
        xyouts, x_coord, y_coord, strtrim(string(im_indx+1),2), /device
      endif
    endfor
  endfor
  if (screen_no lt (num_screens-1)) then begin
    wait,1
  endif
endfor

return, 0

end
