function maskbrain

; option to mask brain or use magnitude mask

; uses mark j's flirt registration algorithm to register and 
; apply a brain mask from a reference brain.
; the map to be masked must be 32*32*16

; 05/06/00: weight_type set up so that.... 0 = phantom
;                                          1 = brain
;                                          2 = weighted brain

; 13/06/00: numbered roi's in wfactor are..... 0 = posterior part
;                                              1 = supfront lobe
;                                              2 = inffront lobe
;                                              3 = inftemp lobe
;                                              4 = cerebellum

common roi_def_info, thresh_val, erosion_val,dilation_val
common field_map_data, phsmap, allimg, fieldmap, strucmap
common share1, roimap, dim0, dim1, dim2
common mask_info, weight_type,weightf,wfactor,vox0,vox1,vox2,grad,$
                  tmatrix,noweightf,freq_limit
common shareshim, slo, fov, fovz, te

on_ioerror, bad_reg

; set arrays to defaults
brainmap=fltarr(dim0,dim1,dim2)
tmatrix=fltarr(4,4)
weightf=fltarr(dim0,dim1,dim2)
noweightf=fltarr(dim0,dim1,dim2)

; MASK PHANTOM
if (weight_type eq 0) then begin 
  print, "masking phantom"
  print, "   mask theshold is "+strtrim(string(wfactor(0)),2)+"%"

  ; set mask threshold to wfactor
  thresh_val = wfactor(0)
  erosion_val=2
  dilation_val=2
  construct_voi
  weightf=float(roimap)

  ;display masked strucmap
  print, "   masked phantom..."
  err=display_ims(bytscl(strucmap*roimap))
  wait, 1


  noweightf=float(roimap)

; MASK BRAIN
endif else begin 
  print, "masking brain"
  ;set brainmap maskthreshold
  brainmap_thresh=0.45

  ;setup filenames: 
  ; reference files are within mask_files/
  ; all written files are within ~/vnmrsys/shims/
  resamp_file="mask_files/noresamplerc"
  ref_struc ="mask_files/ref_struc"
  ref_mask  ="mask_files/ref_mask"
  ref_weight="mask_files/ref_weight"
  ref_weightf="~/vnmrsys/shims/autoshim_rweightf"
  ref_xtra_roimask="~/vnmrsys/shims/autoshim_xtra_roimask"
  shim_struc="~/vnmrsys/shims/autoshim_sstruc"
  shim_mask ="~/vnmrsys/shims/autoshim_smask"
  shim_weightf="~/vnmrsys/shims/autoshim_sweightf"
  forward   ="~/vnmrsys/shims/autoshim_forward.mat"
  backward  ="~/vnmrsys/shims/autoshim_backward.mat"

  ;make sure old files no longer exist 
  if file_exists(shim_mask+'.img') then spawn, '/bin/rm -rf '+shim_mask+'.*'
  if file_exists(shim_struc+'.img') then spawn, '/bin/rm -rf '+shim_struc+'.*'
  if file_exists(forward) then spawn, '/bin/rm -rf '+forward
  if file_exists(backward) then spawn, '/bin/rm -rf '+backward

  ; save structural map - need to include correct voxel dims
  voxdim=fltarr(8)
  voxdim(0)=3.0
  voxdim(1:2)=fov/float(dim0)
  voxdim(3)=fovz/float(dim2)
  avw_write, strucmap, shim_struc, voxdim

  print, "   registering structural map to reference head"
  print, "   transformation matrix..."
  ;register shimcalc_struc to ref_struc
  spawn, '/usr/local/fsl/bin/flirt -noresample -ref '+ref_struc+$
         ' -in '+shim_struc+' -omat '+forward+' -schedule '+resamp_file+$
         ' -verbose 0'

  ;create inverse transform matrix
  spawn, '/usr/local/fsl/bin/convert_xfm -ref '+ref_struc+$
         ' -in '+shim_struc+' -inverse -omat '+backward+' '+forward+$
         ' -verbose 0'

  ;xform ref mask mask to shimcalc space
  spawn, '/usr/local/fsl/bin/flirt -ref '+shim_struc+' -in '+ref_mask+$
         ' -init '+backward+' -applynonisoxfm -o '+shim_mask+' -verbose 0'

  ;check that mask now exists
  if NOT file_exists(shim_mask+'.img') then goto, bad_reg
  
  ;read in mask
  brainmap=avw_read(shim_mask,0)
  roimap=(roimap eq 1) and (brainmap gt brainmap_thresh)
  
  ;further threshold each slice at 10% of max for that slice
  ;and if less than 9 pixels in mask slice then mask out whole slice
  for i=0, dim2-1 do begin
    minforslice=0.15*max(strucmap(*,*,i))
    roimap(*,*,i)=roimap(*,*,i)*(strucmap(*,*,i) ge minforslice)
    if (total(roimap(*,*,i)) lt 9) then begin
      roimap(*,*,i)=0
    endif
  endfor

  ;display masked strucmap
  print, "   masked brain..."
  err=display_ims(bytscl(strucmap*brainmap*roimap))
  wait, 1

  ;copy over, read in and store transformation matrix
  get_lun, unit
  openr, unit, forward
  readf, unit, tmatrix
  close, unit
  free_lun, unit

  ; SPATIAL WEIGHTING (if required)
  if (weight_type eq 1) then begin
    ;weightf equal to brain mask
    weightf=float(roimap)
  endif else begin
    print, "   creating weighting map"
    print, "   weightings are:       posterior "+strtrim(string(wfactor(0)),2)
    print, "                    superior front "+strtrim(string(wfactor(1)),2)
    print, "                    inferior front "+strtrim(string(wfactor(2)),2)
    print, "                     inferior temp "+strtrim(string(wfactor(3)),2)
    print, "                        cerebellum "+strtrim(string(wfactor(4)),2)

    weightf_ref=fltarr(dim0,dim1,dim2)

    ; if extra roimask used then load up and integrate
    if (wfactor(5) gt 0.0) then begin
      print, "                             extra "+strtrim(string(wfactor(5)),2)

      ; read in ref_xtra_roimask
      xtra_roimask=fltarr(dim0,dim1,dim2)
      xtra_roimask=avw_read(ref_xtra_roimask,0)

      ; start to construct weightf in ref space
      weightf_ref=weightf_ref+(wfactor(5)*xtra_roimask)
    endif

    ; read in ref_weight image
    ; in image, values are: 1=pos,2=sup,3=inffront,4=inftemp,5=cereb
    weight_ref=fltarr(dim0,dim1,dim2)
    weight_ref=avw_read(ref_weight,0)

    ; create weightf_ref array in ref space
    for i=0,4 do begin
      weightf_ref=weightf_ref+(wfactor(i)*float(weight_ref eq (i+1)))
    endfor

    ; save weightf_ref
    avw_write, weightf_ref, ref_weightf, voxdim

    ;xform ref_weightf mask to shimcalc space
    spawn, '/usr/local/fsl/bin/flirt -ref '+shim_struc+' -in '+$
           ref_weightf+ ' -init '+backward+' -applynonisoxfm -o '+$
           shim_weightf+' -verbose 0'
  
    ; read in shimcalc space weightf
    weightf=avw_read(shim_weightf,0)

    ; delete weightf files
    spawn, '/bin/rm -rf '+ref_weightf+'.*'
    spawn, '/bin/rm -rf '+shim_weightf+'.*'
  endelse

  ; delete all other written files
  spawn, '/bin/rm -rf '+shim_struc+'.*'
  spawn, '/bin/rm -rf '+shim_mask+'.*'
  spawn, '/bin/rm -rf '+forward
  spawn, '/bin/rm -rf '+backward

endelse

; spatially smooth weightf array in vivo space with kernel of 3
; hence preferentially weighting more definite brain areas, away from edge
weightf=smooth(weightf,3)*roimap
; create noweightf and do the same smoothing
noweightf=smooth(roimap,3)*roimap

; if weighted brain shim, display weighting mask
if (weight_type eq 2) then begin
  print, "   weighting mask..."
  err=display_ims(bytscl(weightf))
  wait, 1
endif

return, 0 ; indicates that registration worked

bad_reg:

return, 1

end


