function read_varian_data, directory, in_file, smooth_val

; this function reads in Varian data for 3D shimming
; assumes that data has been preprocessed to get it into a fieldmap format.
; At present read_varian_bin_data.pro and varian_bin_to_fieldmap2.pro 
; are used to accomplish this.

common share1, roimap, dim0, dim1, dim2
common shareshim, slo, fov, fovz, te
common field_map_data, phsmap, allimg, fieldmap, strucmap

on_error, 2
on_ioerror, bad_file

;******************************************************
; first put the varian data in a form that can be read
;******************************************************
iii=complex(0,1)

dim0=fix(256)
dim1=fix(256)
dim2=fix(1)

;******************************
;Read in complex image files
;******************************
get_lun, unit
openr, unit, directory+'/'+in_file+".header"
readf, unit, dim0, dim1, dim2, dim3, slo, slsep, fov, fovz, $
   te, tintec, plane, flipflag, oblangle
close, unit
free_lun, unit
  
; read in the data
; this line is suitable for double echo GRE sequences
; allimg=fieldmap_ft(varian_bin_to_fieldmap(read_varian_bin_data(directory+ $
;     "/"+in_file), dim0*2,dim1,dim2, dim3, baseline=1, interleave=0))
  
; this line is suitable for asymmetric spin echo sequences
data_arr=varian_bin_to_fieldmap2(read_varian_bin_data(directory+ $
   "/"+in_file), dim0*2,dim1,dim2, dim3, baseline=1, interleave=0)

; ft data array
allimg=fieldmap_ft(data_arr)
  
;******************
;Process image data
;******************
; initialize some arrays and variables based on the dimensions read in
roimap=intarr(dim0, dim1, dim2)+1  ; set all to 1 to start with
  
if (total(allimg) eq -1) then goto, bad_file
  
if (dim3 eq 1) then begin
  te=te/1000000.0
endif else begin
  te=tintec/1000000.0
endelse
print, "   inter echo delay = "+strtrim(string(te), 2)+" secs"

;save structural map
strucmap=fltarr(dim0, dim1, dim2)
strucmap(*,*,*)=abs(allimg(*,*,*,0))

if (smooth_val gt 1) then begin  ; smooth the data
  for i=0, dim2-1 do begin
    for j=0, dim3-1 do begin
      temp=allimg(*,*,i,j)
      temp=complex(smooth(float(temp), smooth_val), $
        smooth(imaginary(temp), smooth_val))
      allimg(*,*,i,j)=temp
    endfor
  endfor
endif

phsmap=fltarr(dim0, dim1, dim2, dim3)

for i=0, dim3-1 do begin
  phsmap(*,*,*,i)=acos(float(allimg(*,*,*,i))/abs(allimg(*,*,*,i)))
  phsmap(*,*,*,i)=phsmap(*,*,*,i)*2.0*((float(-allimg(*,*,*,i)*iii) gt 0.0)-0.5)
endfor

; added bit to deal with any NaN in phsmaps
phsmap_arr=where(finite(phsmap) eq 0, count)
for num=0,count-1 do begin
  phsmap(phsmap_arr(num))=0.0
endfor

print, "   magnitude data..."
; deal with possible dc offset
for i=0,dim2-1 do begin
  mean_centre=0.25*(strucmap((dim0/2.0)+1.0,(dim1/2.0),i)+$
                    strucmap((dim0/2.0)-1.0,(dim1/2.0),i)+$
                    strucmap((dim0/2.0),(dim1/2.0)+1.0,i)+$
                    strucmap((dim0/2.0),(dim1/2.0)-1.0,i))
  strucmap(dim0/2.0,dim1/2.0,i)=mean_centre
endfor

err=display_ims(bytscl(strucmap))
wait,1

return, 0  ; 0 indicates that read_varian_data made it through function

bad_file:

return, -1

end
