pro shim_defs, system_name

common shareeigen,ipmatrix, phsrow, result, bndv,pndpoint, curlimit,orgshim,nch
common shim_info, shimstrength, shimfunc, channels, magfctr, x, y, z

; The shimstrength values are the numbers that come back from the fit to the
; individual shim terms (once the ratio between the coefficients has been 
; accounted for). [See also the magfctr description below].
;
; The magfctr values are used if the shimfunc terms differ too much and cause
; singularalities in the solution matrix. They can be made artificially more
; equivalent by introducing a multiplication factor (magfctr), in which case
; the final shim value results are corrected back again before final print out.
; Typically the 3rd order terms (which are very weak) will have a magfctr
; value of 10. NOTE that if a magfctr value is used then the corresponding
; shimstrength value in this table should be multiplied by magfctr.
;
; The currlimit values are simply the largest magnitude amp value (or DAC
; value) that can be entered into each shim term (e.g. 4.0 for plus/minus
; 4 amps maximum current, or 4096 for plus/minus 4096 DAC units)
; 
; units of shimstrngth are rad/s/mm/DAC for the linear terms,
; rad/s/mm^2/DAC for the quadratic terms, etc.

case system_name of

  'inova47T': begin
    nch=11  ; number of channels
    define_shim_arrays

    channels(0)='z1'
    shimfunc(*,*,*,0)= z
    shimstrength(0)=-2.7156
    magfctr(0)=1.0
    curlimit(0,0)=2047.0

    channels(1)='z2'
    shimfunc(*,*,*,1)= -10.996*z + z*z - 0.725*(x*x+y*y)
    shimstrength(1)=0.0662
    magfctr(1)=1.0
    curlimit(0,1)=2047.0

    channels(2)='x1'
    shimfunc(*,*,*,2)= y
    shimstrength(2)=2.865
    magfctr(2)=1.0
    curlimit(0,2)=2047.0

    channels(3)='y1'
    shimfunc(*,*,*,3)= x
    shimstrength(3)=-2.806
    magfctr(3)=1.0
    curlimit(0,3)=2047.0

    channels(4)='xz'
    shimfunc(*,*,*,4)= 4.76*x + 8.546*y + 1.989*z + 0.4689*z*x + z*y
    shimstrength(4)=0.08387
    magfctr(4)=1.0
    curlimit(0,4)=2047.0

    channels(5)='yz'
    shimfunc(*,*,*,5)= 9.973*x - 4.4*y + 1.498*z + z*x - 0.495*z*y
    shimstrength(5)=0.0822
    magfctr(5)=1.0
    curlimit(0,5)=2047.0

    channels(6)='xy'
    shimfunc(*,*,*,6)= -2.52*x + 3.57*y - 0.89*z + x*y -0.531*(x*x-y*y)
    shimstrength(6)=0.06486
    magfctr(6)=1.0
    curlimit(0,6)=2047.0

    channels(7)='x2y2'
    shimfunc(*,*,*,7)= 1.474*x + 3.56*y + 2.282*z + 1.533*x*y + (x*x-y*y)
    shimstrength(7)=0.0374
    magfctr(7)=1.0
    curlimit(0,7)=2047.0

    channels(8)='xz2'
    shimfunc(*,*,*,8)= -61.3*x - 270.5*y + 24.9*z + y*z*z
    shimstrength(8)=-0.0095  ; i.e., value found from fit was -0.00095
    magfctr(8)=10.0
    curlimit(0,8)=2047.0

    channels(9)='yz2'
    shimfunc(*,*,*,9)= -208.7*x + 74.0*y + 76.0*z + x*z*z
    shimstrength(9)=-0.00811  ; i.e., value found from fit was -0.000811
    magfctr(9)=10.0
    curlimit(0,9)=2047.0

    channels(10)='zxy'
    shimfunc(*,*,*,10)= 112.0*y + 6.37*x*y - 0.363*z*(x*x-y*y) + x*y*z
    shimstrength(10)=0.00979  ; i.e., value found from fit was 0.000979
    magfctr(10)=10.0
    curlimit(0,10)=2047.0

  end

  'anmr': begin
    nch=15  ; number of channels
    define_shim_arrays

    channels(0)='Z'
    shimfunc(*,*,*,0)= z
    shimstrength(0)=1.0615e-3
    magfctr(0)=1.0
    curlimit(0,0)=8000.0

    channels(1)='Z2'
    shimfunc(*,*,*,1)= z*z - 0.5194*(x*x+y*y)
    shimstrength(1)=3.607e-5  ; i.e., value found from fit was 3.607e-6
    magfctr(1)=10.0
    curlimit(0,1)=8000.0

    channels(2)='Z3'
    shimfunc(*,*,*,2)= z*z*z - 44.7*z*z - 3.8*z*(x*x+y*y)
    shimstrength(2)=1.475e-5  ; i.e., value found from fit was 1.475e-8
    magfctr(2)=1000.0
    curlimit(0,2)=8000.0

    channels(3)='X'
    shimfunc(*,*,*,3)= x
    shimstrength(3)=1.075e-3
    magfctr(3)=1.0
    curlimit(0,3)=8000.0

    channels(4)='Y'
    shimfunc(*,*,*,4)= y
    shimstrength(4)=1.07e-3
    magfctr(4)=1.0
    curlimit(0,4)=8000.0

    channels(5)='ZX'
    shimfunc(*,*,*,5)= z*x
    shimstrength(5)=-7.15e-6
    magfctr(5)=1.0
    curlimit(0,5)=8000.0

    channels(6)='ZY'
    shimfunc(*,*,*,6)= z*y - 3.0*y
    shimstrength(6)=-7.0e-6
    magfctr(6)=1.0
    curlimit(0,6)=8000.0

    channels(7)='X2Y2'
    shimfunc(*,*,*,7)= x*y
    shimstrength(7)=7.35e-6
    magfctr(7)=1.0
    curlimit(0,7)=8000.0

    channels(8)='XY'
    shimfunc(*,*,*,8)= (x*x-y*y)
    shimstrength(8)=3.725e-5
    magfctr(8)=1.0
    curlimit(0,8)=8000.0

    channels(9)='Z2X'
    shimfunc(*,*,*,9)= x*z*z - 0.27*x*y*y - 0.265*x*x*x - 12.86*z*x
    shimstrength(9)=4.8e-6  ; i.e., value found from fit was 4.8e-8
    magfctr(9)=100.0
    curlimit(0,9)=8000.0

    channels(10)='Z2Y'
    shimfunc(*,*,*,10)= y*z*z - 0.29*y*x*x - 0.3*y*y*y - 13.0*z*y
    shimstrength(10)=4.6e-6  ; i.e., value found from fit was 4.6e-8
    magfctr(10)=100.0
    curlimit(0,10)=8000.0

    channels(11)='ZX2Y2'  ; AMP CURRENTLY BROKEN
    shimfunc(*,*,*,11)= z*(x*x-y*y) - 302.0*x - 255.0*y
    shimstrength(11)=5.1e-7  ; i.e., value found from fit was 5.1e-8
    magfctr(11)=10.0
    curlimit(0,11)=0.0

    channels(12)='ZXY'
    shimfunc(*,*,*,12)= z*x*y + 123.0*x - 127.0*y
    shimstrength(12)=1.02e-6  ; i.e., value found from fit was 1.02e-8
    magfctr(12)=10.0
    curlimit(0,12)=5000.0

    channels(13)='X3'  ; AMP CURRENTLY BROKEN
    shimfunc(*,*,*,13)= x*x*x - 3325.0*x
    shimstrength(13)=2.5375e-6  ; i.e., value found from fit was 2.5375e-9
    magfctr(13)=1000.0
    curlimit(0,13)=0.0  ; since it is a very weak term and often maxes out

    channels(14)='Y3'
    shimfunc(*,*,*,14)= y*y*y - 3.1*y*x*x
    shimstrength(14)=1.76e-6  ; i.e., value found from fit was 1.76e-8
    magfctr(14)=100.0
    curlimit(0,14)=5000.0

  end

  'oxford3T_main': begin
    nch=8  ; number of channels
    define_shim_arrays

    channels(0)='z1'
    shimfunc(*,*,*,0)= z
    shimstrength(0)=-9.82285e-3
    magfctr(0)=1.0
    curlimit(0,0)=8000.0

    channels(1)='z2'
    shimfunc(*,*,*,1)= z*z - 0.5609*(x*x+y*y)
    shimstrength(1)=-1.10675e-4  ; i.e., value found from fit was -1.10675e-5
    magfctr(1)=10.0
    curlimit(0,1)=20000.0

    channels(2)='x1'
    shimfunc(*,*,*,2)= y
    shimstrength(2)=-1.03019e-2
    magfctr(2)=1.0
    curlimit(0,2)=8000.0

    channels(3)='y1'
    shimfunc(*,*,*,3)= x
    shimstrength(3)=-1.00822e-2
    magfctr(3)=1.0
    curlimit(0,3)=8000.0

    channels(4)='xz'
    shimfunc(*,*,*,4)= z*y
    shimstrength(4)=2.008645e-4  ; i.e., value found from fit was 2.008645e-6
    magfctr(4)=100.0
    curlimit(0,4)=20000.0

    channels(5)='yz'
    shimfunc(*,*,*,5)= z*x
    shimstrength(5)=2.03143e-4  ; i.e., value found from fit was 2.03143e-6
    magfctr(5)=100.0
    curlimit(0,5)=20000.0

    channels(6)='xy'
    shimfunc(*,*,*,6)= x*y
    shimstrength(6)=-4.164215e-4  ; i.e., value found from fit was -4.164215e-6
    magfctr(6)=100.0
    curlimit(0,6)=20000.0

    channels(7)='x2y2'
    shimfunc(*,*,*,7)= (x*x-y*y)
    shimstrength(7)=-2.0778e-4  ; i.e., value found from fit was -2.0778e-6
    magfctr(7)=100.0
    curlimit(0,7)=20000.0

  end

  'oxford3T_head': begin
    nch=8  ; number of channels
    define_shim_arrays

; basis functions from 31st August 2001
    channels(0)='z1'
    shimfunc(*,*,*,0)= z 
    shimstrength(0)=-1.44725e-2
    magfctr(0)=1.0
    curlimit(0,0)=8000.0

    channels(1)='z2'
    shimfunc(*,*,*,1)= z*z - (9.247 * z) - (0.4284 * (x*x+y*y)) 
    shimstrength(1)=-1.33544e-4  ; i.e., value found from fit was -1.33544e-5
    magfctr(1)=10.0
    curlimit(0,1)=20000.0

    channels(2)='x1'
    shimfunc(*,*,*,2)= y
    shimstrength(2)=-1.47115e-2
    magfctr(2)=1.0
    curlimit(0,2)=8000.0

    channels(3)='y1'
    shimfunc(*,*,*,3)= x
    shimstrength(3)=-1.50110e-2
    magfctr(3)=1.0
    curlimit(0,3)=8000.0

    channels(4)='xz'
    shimfunc(*,*,*,4)= z*y
    shimstrength(4)=2.36062e-4  ; i.e., value found from fit was 2.36062e-6
    magfctr(4)=100.0
    curlimit(0,4)=20000.0

    channels(5)='yz'
    shimfunc(*,*,*,5)= z*x
    shimstrength(5)=2.14907e-4  ; i.e., value found from fit was 2.14907e-6
    magfctr(5)=100.0
    curlimit(0,5)=20000.0

    channels(6)='xy'
    shimfunc(*,*,*,6)= x*y
    shimstrength(6)=-4.37289e-4  ; i.e., value found from fit was -4.37289e-6
    magfctr(6)=100.0
    curlimit(0,6)=20000.0

    channels(7)='x2y2'
    shimfunc(*,*,*,7)= x*x-y*y
    shimstrength(7)=-2.07423e-4  ; i.e., value found from fit was -2.07423e-6
    magfctr(7)=100.0
    curlimit(0,7)=20000.0

; old basis functions
;    channels(0)='z1'
;    shimfunc(*,*,*,0)= z 
;    shimstrength(0)=-1.46018e-2
;    magfctr(0)=1.0
;    curlimit(0,0)=8000.0
;
;    channels(1)='z2'
;    shimfunc(*,*,*,1)= z*z - (11.1886 * z) - (0.42750 * (x*x+y*y)) 
;    shimstrength(1)=-1.35532e-4  ; i.e., value found from fit was -1.35532e-5
;    magfctr(1)=10.0
;    curlimit(0,1)=20000.0
;
;    channels(2)='x1'
;    shimfunc(*,*,*,2)= y
;    shimstrength(2)=-1.47776e-2
;    magfctr(2)=1.0
;    curlimit(0,2)=8000.0
;
;    channels(3)='y1'
;    shimfunc(*,*,*,3)= x
;    shimstrength(3)=-1.49673e-2
;    magfctr(3)=1.0
;    curlimit(0,3)=8000.0
;
;    channels(4)='xz'
;    shimfunc(*,*,*,4)= z*y
;    shimstrength(4)=2.26982e-4  ; i.e., value found from fit was 2.26982e-6
;    magfctr(4)=100.0
;    curlimit(0,4)=20000.0
;
;    channels(5)='yz'
;    shimfunc(*,*,*,5)= z*x
;    shimstrength(5)=2.08028e-4  ; i.e., value found from fit was 2.08028e-6
;    magfctr(5)=100.0
;    curlimit(0,5)=20000.0
;
;    channels(6)='xy'
;    shimfunc(*,*,*,6)= x*y
;    shimstrength(6)=-4.32418e-4  ; i.e., value found from fit was -4.32418e-6
;    magfctr(6)=100.0
;    curlimit(0,6)=20000.0
;
;    channels(7)='x2y2'
;    shimfunc(*,*,*,7)= x*x-y*y
;    shimstrength(7)=-1.95309e-4  ; i.e., value found from fit was -1.95309e-6
;    magfctr(7)=100.0
;    curlimit(0,7)=20000.0

  end

  else: begin
    print, "No corresponding calibration table for this system"
  end

endcase

end
