pro shimcalc_file, directory_tmp, filename_tmp, weight_tmp, wfactor0_tmp,$
                   wfactor1_tmp, wfactor2_tmp, wfactor3_tmp, wfactor4_tmp,$
                   wfactor5_tmp, vox0_tmp, vox1_tmp, vox2_tmp, limit_tmp,$
                   grad_tmp, info_filename_tmp,epi_flag_tmp
                   

; this code should only be called directly for Oxford 3T (head) defaults
common file_info, directory, filename, file_type, smooth_val, info_filename
common sys_names_info, sys_names, curr_sys
common field_map_data, phsmap, allimg, fieldmap, strucmap
common mask_info, weight_type,weightf,wfactor,vox0,vox1,vox2,grad,$
                  tmatrix,noweightf,freq_limit
common share1, roimap, dim0, dim1, dim2

window, title='SHIMCALC (James Wilson / Peter Jezzard, 2001, v.4)',$
              XSIZE=512, YSIZE=512

; read in command line parameters
directory=directory_tmp
filename=filename_tmp
weight_type=weight_tmp

wfactor=fltarr(6)
wfactor(0)=wfactor0_tmp
wfactor(1)=wfactor1_tmp
wfactor(2)=wfactor2_tmp
wfactor(3)=wfactor3_tmp
wfactor(4)=wfactor4_tmp
wfactor(5)=wfactor5_tmp

vox0=vox0_tmp/10.0
vox1=vox1_tmp/10.0
vox2=vox2_tmp/10.0

grad=grad_tmp
freq_limit=limit_tmp
info_filename=info_filename_tmp
epi_flag=epi_flag_tmp          ; not used here
 
; smooth real and imaginary image values wih the following kernel
smooth_val=2

; set file_type for that for Oxford 3T (head)
curr_sys=3
file_type=1

read_sys_names

print, "STARTING SHIMCALC..."
print, ""

;***************************
; execute load of base field
;***************************
print, "loading base field map data"
load_err=execute_load()
if (load_err eq 1) then begin ; error in inputting files
  print, "*****ERROR: Can't read input file*****"
endif else begin

  ;****************************************
  ; mask the brain and create weighted mask
  ;****************************************
  print, ""
  reg_err=maskbrain()
  if (reg_err eq 1) then begin  ; error in registration
    print, "*****ERROR: Can't mask brain*****"
  endif else begin

    ;*********************
    ; calculate base field
    ;*********************
    print, ""
    print, "calculating field map"
    calc_err=calc_field()
    if (calc_err eq 1) then begin  ; error in calculating field
      print, "*****ERROR: Can't calculate field map*****"
    endif else begin
  
      ;*****************************
      ; calculate optimum shim terms
      ;*****************************
      print, ""
      print, "calculating optimum shim settings"
      shim_err=calc_shims(sys_names(curr_sys, 2))
      if (shim_err eq 1) then begin  ; error in calculating shims 
        print, "*****ERROR: Can't calculate shim terms*****"
      endif else begin
        ; update autoshim_complete_flag value to 9
        get_lun, unit
        openw, unit, "~/.xvnmr_pages/autoshim_complete_flag"
        printf, unit, strtrim(string(9),2)
        close, unit    
        free_lun, unit
       
        ;*****************************
        ; end!
        ;*****************************
      endelse
    endelse
  endelse
endelse

print, ""
print, "exiting IDL..."
  
end
