#ifndef LINT
static char SCCSid[] = "@(#)flash.c 11.1 05/07/96 Copyright (c) 1994-1996 Varian Assoc.,Inc. All Rights Reserved";
#endif
/***********************************************************************
    KSPACE.C

    K space mapping sequence.                     

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

#include "standard.h"
#include "math.h"
#include "group.h"
#include "sequence_utilities.h"
#include "sine_grad.h"
#define OFFSETDELAY 26e-6
#define RFSPOILDELAY 50e-6

/*==========================================================================*
  define the pulse sequence variables outside the pulse sequence so that
  they can be seen as global variables by the safety monitoring code
*==========================================================================*/
    double  predelay,seqtime,tref,risetref,grise,tep,rftep,at_des;
    double  trefmin,tssrmin,trormin,tpemin,temin,tA;
    double  agro,agss,sgpe,grate,gssrint,grorint,gpeint;      
    double  reffreq,nextphase,spoilfreqs[1024],pw_gss_d,extra_te_error;
    double  pw_gro,pw_gss,wave_time,true_wave_time,greadfc,gslicefc;
    int     i,image=0,dummy_ssc;
    char    shape_name[MAXSTR],flow_comp[MAXSTR],err_string[MAXSTR];
    char    gcoil[MAXSTR];

pulsesequence()
{

    double pw_sine, cycle_duration, total_duration;
    int num_cycles;

    initparms_sis();
    loop_check();

    /*************************************************************************/
    /* get the global parameters */
    /*************************************************************************/
    P_getreal(GLOBAL,"grise",&grise,1); /* grad risetime to full strength */
    if ((grise < 0.0) || (grise > 0.002)) {
      sprintf(err_string,"%s: impropper grise time defined in global file\n",
                       seqfil);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
    /*************************************************************************/

    cycle_duration=getval("cycle_duration");
    total_duration=getval("total_duration");
    getstr("gcoil",gcoil);          /* gradient coil in use */
    image = (int)(getval("image"));
    /* 1 = read direction gradient */
    /* 2 = phase encode direction gradient */
    /* 3 = slice direction gradient */

    grate = grise/gmax;

    sprintf(shape_name,"sine");
    pw_sine=create_sine_gradient(fabs(gro),shape_name,cycle_duration,grate,
      (double)1.0,ix);

    num_cycles=(int)(total_duration/cycle_duration+0.5);

    /* PULSE SEQUENCE *************************************/

    initval((int)num_cycles,v5);

    loop(v5,v6);
      if (image == 1) {
        obl_shapedgradient("sine","","",pw_sine,gro,0.0,0.0,1,NOWAIT);
      }
      if (image == 2) {
        obl_shapedgradient("","sine","",pw_sine,0.0,gro,0.0,1,NOWAIT);
      }
      if (image == 3) {
        obl_shapedgradient("","","sine",pw_sine,0.0,0.0,gro,1,NOWAIT);
      }
      delay(pw_sine);
    endloop(v6);

    acquire(np,1.0/sw);

    zero_all_gradients(); /* double check that all gradients are zero */
}
