/* Patricia, January 2000 
   added to by PJ, March 2000 
   adapted by karla, spring 2001
*/

#define MAX_CW_RF 36.0
#define MAX_CW_LEN 5.0
double pasl,tpwrasl,gasl,tag_off,mean_pss;
double slew_tweak,tag_tweak;
double gflow,tflow,pw_flow;
double concat1,concat2;
int tag_direction;
double switchdelay;
char gasl_on[MAXSTR];

calculate_flowsuppress(double *t_flowsup)
{
  char shape_name[MAXSTR];

  gflow=getval("gflow");
  tflow=getval("tflow");
  sprintf(shape_name,"gflow");

  pw_flow = create_trap_gradient(fabs(gflow), shape_name, tflow, grate, ix);
  *t_flowsup = 2.0*pw_flow;
}



flowsuppress_gradients(double polarity)
{
  obl_shapedgradient("","","gflow",pw_flow,0.0,0.0,polarity*gflow,1,WAIT);
}



calculate_casl_gradients(double *t_asl)
{
  char shape_name[MAXSTR];
  char err_string[MAXSTR];

  /* duration of CW pulse */
  pasl = getval("pasl");

  /* strength of CW pulse */
  tpwrasl = getval("tpwrasl");

  /* strength of CW gradient */
  gasl = getval("gasl");

  /* tweaks slew rate of gasl */
  slew_tweak = getval("slew_tweak");

  /* tweaks offset of tag_offset in control pulse */
  tag_tweak = getval("tag_tweak");

  /* delay required before/after switching RF mode */
  switchdelay = getval("switchdelay");

  /* for multislice, offset of center slice */
  mean_pss = getval("mean_pss");

  /* tag offset */
  tag_off = getval("tag_off");

  /* set to 'y' to have gasl on, 'n' to turn off gasl */
  getstr("gasl_on",gasl_on);

  /* tag_direction controls the gradient/rf sign */
  /*      tag_direction = 1   : gasl +ive, tag_off tag
          tag_direction = 2   : gasl +ive, tag_off control
          tag_direction = 3   : gasl -ive, tag_off tag
          tag_direction = 4   : gasl -ive, tag_off control  */
  tag_direction = (int)(getval("tag_direction"));


  /* add a safety section to watch out for excessive RF */
  if (tpwrasl > MAX_CW_RF) {
    sprintf(err_string,"%s: value of tpwrasl is over safe level\n", seqfil);
    printf(err_string);
    log_error(err_string);
    abort(1);
  }
  
  if (pasl > MAX_CW_LEN) {
    sprintf(err_string,"%s: value of pasl is over safe level\n", seqfil);
    printf(err_string);
    log_error(err_string);
    abort(1);
  }
  
  sprintf(shape_name,"gasl");
  pw_gasl=create_ramp_gradient(fabs(gasl),shape_name,1,grate*slew_tweak,ix);
  pw_gasl=create_ramp_gradient(fabs(gasl),shape_name,-1,grate*slew_tweak,ix);

  *t_asl=pw_gasl+rof1+pasl+rof2+pw_gasl;
  *t_asl = *t_asl+2*switchdelay;

}


casl_pulse_cw()   /* single long pulse version */
{
  double actual_gasl, actual_tag_off;

  if (tag_direction == 1) { /* tag */
    actual_gasl=gasl;
    actual_tag_off=mean_pss+tag_off;
  }
  else if (tag_direction == 2) { /* control */
    actual_gasl=gasl;
    actual_tag_off=mean_pss-(tag_off+tag_tweak);
  }
  else if (tag_direction == 3) { /* tag */
    actual_gasl=-gasl;
    actual_tag_off=-mean_pss-tag_off;
  }
  else {  /* assume must be 4 */ /* control */
    actual_gasl=-gasl;
    actual_tag_off=-mean_pss+(tag_off+tag_tweak);
  }
  
  printf("   changing to CW mode\n");
  setampmode12('c','i'); /* set RF amp in CW mode */
  delay(switchdelay);     /* delay associated with switching RF mode */

  printf("   calling poffset\n");
  /*  poffset(actual_tag_off,actual_gasl); */
  position_offset(actual_tag_off,actual_gasl,resto,OBSch);
  obspower(tpwrasl);

  /* set actual_gasl to zero if gasl_on=='n' */
  if(gasl_on[0] == 'n') actual_gasl=0.0;
  printf("   calling gradients width %f\n",pw_gasl);
  obl_shapedgradient("","","gasl_up",pw_gasl,0.0,0.0,actual_gasl,1,WAIT);

  printf("   calling pulse\n");
  delay(tep);
  sp4on();
  pulse(pasl,v1);
  sp4off();

  printf("   calling gradients\n");
  obl_shapedgradient("","","gasl_dn",pw_gasl,0.0,0.0,actual_gasl,1,WAIT);

  printf("   changing to pulsed mode\n");
  setampmode12('p','i');  /* set RF amp back in pulsed mode */

  /* note: delay associated with switching RF mode (switchdelay)
           is included in t_delay so that crushers can be played 
           out in this time if desired */

}

casl_pulse_concat()    /* concatenated shorter RF pulse version */
{
  int i,n;
  double tau;
  double actual_gasl, actual_tag_off;
  
  printf("called casl_pulse_concat\n");

  concat1=getval("concat1");
  concat2=getval("concat2");

  if (tag_direction == 1) {
    actual_gasl=gasl;
    actual_tag_off=mean_pss+tag_off;
  }
  else if (tag_direction == 2) {
    actual_gasl=gasl;
    actual_tag_off=mean_pss-(tag_off+tag_tweak);
  }
  else if (tag_direction == 3) {
    actual_gasl=-gasl;
    actual_tag_off=-mean_pss-tag_off;
  }
  else {  /* assume must be 4 */
    actual_gasl=-gasl;
    actual_tag_off=-mean_pss+(tag_off+tag_tweak);
  }
  
  n=(int)(pasl/(concat1+concat2));  /* number of RF pulses to be played out */
  tau=fmod(pasl,(concat1+concat2)); /* residual delay after RF pulse train */
  
  /*  poffset(actual_tag_off,actual_gasl); */
  position_offset(actual_tag_off,actual_gasl,resto,OBSch); 
  obspower(tpwrasl);

  printf("poffset called with %f\n",actual_tag_off);

  if(gasl_on[0] == 'n') actual_gasl=0.0;
  obl_shapedgradient("","","gasl_up",pw_gasl,0.0,0.0,actual_gasl,1,WAIT);

  delay(tep);
  for(i=0;i<n;i++){
    sp4on();
    pulse(concat1,v1);
    sp4off();
    delay(concat2);
  }
  pulse(tau,v1);

  obl_shapedgradient("","","gasl_dn",pw_gasl,0.0,0.0,actual_gasl,1,WAIT);
}







