#ifndef LINT
static char SCCSid[] = "@(#)press.c 13.1 10/10/97 Copyright (c) 1991-1995 Varian Assoc.,Inc. All Rights Reserved";
#endif
/* 
 * Varian Assoc.,Inc. All Rights Reserved.
 * This software contains proprietary and confidential
 * information of Varian Assoc., Inc. and its contributors.
 * Use, disclosure and reproduction is prohibited without
 * prior consent.
 */
/***********************************************************
    Mar1_csi2d.C

    Volume localized spectroscopy using the double spin
    echo PRESS technique, with optional water suppression.
************************************************************/

#include <standard.h>
#include "group.h"
#include "sequence_utilities.h"

    /* INTERNAL VARIABLE DECLARATIONS *********************/
    double tau1,tau2,tau3;          /*Timing balance delays*/
    double tref,tref1,satpwr1;  /*gradient refocus delays*/
    double gvox1r;   /*refocus gradients*/
    double tep,tep_diff1,pet,pet1,m_tref,m_trefmin;
    double xint1;                    /*gradient integrals*/	
    double grate;		    /*gradient slew rate*/
    double te2;	                    /*te for second spin-echo*/
    double pwr_tweak;
    double seqtime,predelay;
    double pw_spoil,pw_gvox1,pw_gvox2,pw_gvox3,pw_crush1,pw_crush2;
    double crush_ramp_time1,crush_ramp_time2,tpemin,tpe2min;
    double pw_gvox1r,pw_gpe,refreq;
    double agvox1,agvox2,agvox3,agvox1r,agvsp1,agvsp2,agsp,agcr,agcr2;
    double tacq,agvsp3,agvsp4,sgpe,sgpe2,gpeint,gpe2int,agpe,agpe2;
    char shape_name[MAXSTR];

    pulsesequence()
{
    /* PARAMETER INITIALIZATION ***************************/
    initparms_sis();
    P_getreal(GLOBAL,"pwr_tweak",&pwr_tweak,1);
    P_getreal(GLOBAL,"tep",&tep,1);
    tep=tep/1e6;
	refreq=getval("refreq");
	/*resfreq=resto-2.2*sfrq;*/
    te2 = getval("te2");

    grate = trise/gmax;
    tacq=getval("tacq");
    agvox1=fabs(gvox1);
    agvox2=fabs(gvox2);
    agvox3=fabs(gvox3);
    agpe=fabs(gpe);
    agpe2=fabs(gpe2);
    agcr=fabs(gcrush);
    agsp=fabs(gspoil);
    agcr2=1.5*agcr;

    agvsp1 = agcr > agvox2  ?  agcr : agvox2;
    agvsp2 = agcr2 > agvox3  ?  agcr2 : agvox3;

    agvsp3=agcr;
	agvsp4=agcr2;


/* Extra delay calculations due to tep*/

    tep_diff1=0.0;
    pet=agvox1*grate-tep;
    pet1=pet;
   
    if(pet < 0)
    {
    tep_diff1=-pet;
    pet1=pet+tep;
}
    
    if ((satpwr+pwr_tweak+5.0) > 63.0) {
      printf("satpwr too high. Maximum satpwr = %f\n", 63.0-(pwr_tweak+5.0));
      abort(1);
      
    }

    /* GRADIENT CALCULATIONS*/

    sprintf(shape_name,"gvox1");
pw_gvox1=create_trap_gradient(agvox1,shape_name,rof1+p1+rof1,grate,ix);
sprintf(shape_name,"gvoxsp1");
pw_gvox2=create_spin_echo_gradient(shape_name,agvox2,rof1+p2+rof1,agcr,tcrush,tcrush,grate,ix);


    sprintf(shape_name,"gvoxsp2");

    pw_gvox3=create_spin_echo_gradient(shape_name,agvox3,rof1+p2+rof1,agcr2,tcrush,tcrush,grate,ix);
sprintf(shape_name,"gvoxsp3");
    pw_crush1=create_spin_echo_gradient(shape_name,0.0,rof1+p2+rof1,agcr,tcrush,tcrush,grate,ix);
sprintf(shape_name,"gvoxsp4");
   pw_crush2=create_spin_echo_gradient(shape_name,0.0,rof1+p2+rof1,agcr2,tcrush,tcrush,grate,ix);

     crush_ramp_time1=(pw_gvox2-2.0*tcrush-2.0*rof1-p2)/4.0;
     crush_ramp_time2=(pw_gvox3-2.0*tcrush-2.0*rof1-p2)/4.0;


    sprintf(shape_name,"gspl");
    pw_spoil=create_trap_gradient(agsp,shape_name,tspoil,grate,ix);
	 sprintf(shape_name,"gpe");
pw_gpe=create_ramp_gradient(fabs(gmax),shape_name,1,grate,ix);
	pw_gpe=create_ramp_gradient(fabs(gmax),shape_name,-1,grate,ix);



    /* GRADIENT INTEGRAL AND TIMING CALCULATIONS **********/

	gpeint=B0/(lpe*sfrq*1e6);
	gpe2int=B0/(lpe2*sfrq*1e6);
/* the gratdient fall time for the PE block*/
	if (trise*gmax >= gpeint*nv/2.0)
	tpemin = sqrt(grate*gpeint*nv/2.0);
	else
	 tpemin=gpeint*nv/2.0/gmax;
    if (trise*gmax >= gpe2int*nv2/2.0)
         tpe2min = sqrt(grate*gpe2int*nv2/2.0);
    else
         tpe2min=gpe2int*nv2/2.0/gmax;
	m_tref=p1/2.0;
	m_trefmin=tpemin;
	if(m_trefmin < tpe2min)
	 m_trefmin=tpe2min;
	if (m_tref < m_trefmin)
	m_tref=m_trefmin;
	sgpe=gpe*tpe/m_tref;
sgpe= gmax/gradstepsz*floor(sgpe*gradstepsz/gmax + 0.5);
	m_tref=gpe*tpe/sgpe;
	sgpe2=gpe2*tpe/m_tref;
sgpe2= gmax/gradstepsz*floor(sgpe2*gradstepsz/gmax + 0.5);
	m_tref=gpe*tpe/sgpe2;
/*******************************************************
    * A single solution for tref is not guaranteed to exist for the two
    * values of sgpe and sgpe2.  A good compromise is the geometric mean.
    * If this tref results in a distortion in either PE direction of more
    * than 1%, issue an advisory message.
    *******************************************************/
m_tref = sqrt((gpeint/sgpe)*(gpe2int/sgpe2));
    if (nth2D == 1  &&  (fabs(1.0 - sgpe*m_tref/gpeint) > 0.01  ||
      fabs(1.0 - sgpe2*m_tref/gpe2int) > 0.01)) {
        printf("%s:  ADVISORY: lpe distortion =%6.2f%%      lpe2 =%6.2f%%\n",
            seqfil,100.0-sgpe*m_tref/gpeint*100,100.0-sgpe2*m_tref/gpe2int*100.0);
    }
    xint1=p1/2.0+rof1+grate*agvox1/2.0;
    if(trise*gmax >= xint1)
    tref1=2.0*sqrt(grate*xint1);
    else
    tref1=grate*gmax+xint1/gmax;
    tref = p1/2.0;
    if(tref < tref1)
    tref=tref1;

    gvox1r = -gvox1*(p1/2.0 + rof1 + grate*fabs(gvox1)/2.0)/tref;
    agvox1r=fabs(gvox1r);

    sprintf(shape_name,"gssr4");
    pw_gvox1r=create_ramp_gradient(agvox1r,shape_name,1,grate,ix);
    pw_gvox1r=create_ramp_gradient(agvox1r,shape_name,-1,grate,ix);

    /****
    * tau1 is determined by adding all of the events in the first half
    * echo period, and subtracting from te/2.  Similarly for tau2 and
    * tau3.*/


tau1=p1/2.0+rof1+pet1+tep_diff1+tref+pw_gvox1r+2.0*crush_ramp_time1+tcrush;
tau1=tau1+p2/2.0+rof1+tep;
tau1=te/2.0-tau1;

if(tau1 <0.0)
{
        printf("te too short for period 1.  Minimum te = %f\n",
          (te/2.0 - tau1));
        abort(1);
    }

 tau2=p2/2.0+rof1+2.0*crush_ramp_time1+tcrush+p2/2.0+2.0*crush_ramp_time2+tcrush;
tau2=tau2+rof1+tep;
tau2=te/2.0+te2/2.0-tau2;
    if (tau2 < 0.0) {
        printf("te or te2 too short.  Min total = %f\n",((te+te2)/2.0-tau2));
        abort(1);
    }

tau3=p2/2.0+2.0*crush_ramp_time2+tcrush+rof1+tacq+trise+trise+m_tref;
    tau3 = te2/2.0 - tau3;
    if (tau3 < 0.0) {
        printf("te2 too short for period 3.  Min te2 = %f\n",(te2/2.0-tau3));
        abort(1);
    }

    seqtime = te + te2 + at + p1/2.0 + rof1 + agvox1*grate+tep;
    if (ws[0] == 'y') {
        seqtime = seqtime + 3.0*(psat + tspoil + trise + 2.0*rof1);
    }
    predelay = tr - seqtime;
    if (predelay < 0.0) {
        printf("tr too short.  Minimum tr = %f\n",seqtime);
        abort(1);
    }
    /* PULSE SEQUENCE *************************************/

    sp4off();

    /* Phase cycle ****************************************
    ct  =  1  2  3  4    5  6  7  8    10 11 12 13 ......
    v2  =  0  1  2  3    0  1  2  3    0  1  2  3
    v1  =  0  0  1  1    2  2  3  3    4  4  5  5
    v1  =  0  0  0  0    1  1  1  1    2  2  2  2
    v1  =  0  0  0  0    1  1  1  1    2  2  2  2
    oph =  0  2  4  6    0  2  4  6    0  2  4  6
    v9  =  0  0  0  0    2  2  2  2    4  4  4  4
    oph =  0  2  4  6    2  4  6  8    4  6  8  10

    v10 =  0  0  1  1    2  2  3  3    4  4  5  5
    v10 =  0  0  0  0    1  1  1  1    2  2  2  2  
    v10 =  0  0  0  0    0  0  0  0    1  1  1  1
    v10 =  0  0  0  0    0  0  0  0    0  0  0  0
    v3  =  1  1  1  1    1  1  1  1    1  1  1  1
    *******************************************************/

    mod4(ct,v2);
    hlv(ct,v1);
    hlv(v1,v1);
    mod4(v1,v1);
    dbl(v2,oph);
    dbl(v1,v9);
    add(v9,oph,oph);
    /* CYCLOPS */
    hlv(ct,v10);
    hlv(v10,v10);
    hlv(v10,v10);
    hlv(v10,v10);
    mod4(v10,v10);
    add(one,v10,v3);
    add(v10,v1,v1);
    add(v10,v2,v2);
    add(v10,oph,oph);
 /* Phase cycle: Part I, Cyclops quadrature phase ******/
    /*mod2(ct,v3);*/                    /* v3: 0101 0101 */
   /* dbl(v3,v3);*/                     /* v3: 0202 0202 */
    hlv(ct,v4);                     /* v4: 0011 2233 */
    mod2(v4,v4);                    /* v4: 0011 0011 */
    /*add(v3,v4,v1);*/                  /* v1: 0213 0213 */
   /* assign(v1,oph);*/                 /* p1 and rcvr phase */

/*dummy scans*/

	/*initval(4,v13);*/


	  /*peloop(seqcon[3],nv2,v7,v8);
    peloop(seqcon[2],nv,v5,v6);*/

	/*add(v6,v8,v12);

	ifzero(v12);

	loop(v13,v11);*/

 /* 90 degree RF pulse *********************************/
		/*delay(predelay);

    obspower(tpwr1);
    poffset(pos1,gvox1);
    obl_shapedgradient("gvox1","","",pw_gvox1,agvox1,0.0,0.0,1,NOWAIT);
    delay(agvox1*grate+tep);
    sp4on();
    shapedpulse(p1pat,p1,v3,rof1,rof1);
    sp4off();
    if(agvox1*grate > tep)
    {
    delay(agvox1*grate -tep);
}*/


    /* First half spin-echo period ************************/
    /*obl_shapedgradient("gssr4_up","","",pw_gvox1r,gvox1r,0.0,0.0,1,WAIT);
    delay(tref-pw_gvox1r);
    obl_shapedgradient("gssr4_dn","","",pw_gvox1r,gvox1r,0.0,0.0,1,WAIT);
        zero_all_gradients();
    delay(tau1);*/

    /* First 180 degree pulse with crushers*/
    /*obspower(tpwr2);
    poffset(pos2,gvox2);
    obl_shapedgradient("gvoxsp3","gvoxsp1","gvoxsp3",pw_gvox2,agvsp3,agvsp1,agvsp3,1,NOWAIT);
    delay(2.0*crush_ramp_time1+tcrush+tep);
    sp4on();
    shapedpulse(p2pat,p2,v2,rof1,rof1);
    sp4off();
    if(2.0*crush_ramp_time1+tcrush > tep)
{
    delay(2.0*crush_ramp_time1+tcrush-tep);
}

        zero_all_gradients();

    delay(tau2);*/


    /* Second 180 degree pulse ****************************/
    /*obspower(tpwr2);
    poffset(pos3,gvox3);
    obl_shapedgradient("gvoxsp4","gvoxsp4","gvoxsp2",pw_gvox3,agvsp4,agvsp4,agvsp2,1,NOWAIT);
    delay(2.0*crush_ramp_time2+tcrush+tep);
    sp4on();
    shapedpulse(p2pat,p2,v1,rof1,rof1);
    sp4off();
        if(2.0*crush_ramp_time2+tcrush > tep)
{
    delay(2.0*crush_ramp_time2+tcrush-tep);
}
        zero_all_gradients();

	 delay(tau3+at);

endloop(v11);

endif(v12);*/




  peloop(seqcon[3],nv2,v7,v8);
    peloop(seqcon[2],nv,v5,v6);

        /* Phase cycle: Part II, p2 alternates each phase step */
        /* to move non-phase-encoded residual fid from 180 *****/
        /* out of image center to phase-encoding edges. ********/
        mod2(v6,v4);
        /*ifzero(v4);
          add(one,v1,v2); */          /* v2 = v1 + 90: 1324 */
        /*elsenz(v4);*/                 /*    or              */
         /* add(three,v1,v2);*/         /* v2 = v1 - 90: 3546 */
       /*endif(v4);*/

        /* Alternate v2 phase each transient to cancel ********/
        /* residual FID from 180 ******************************/
        /*add(v3,v2,v2);*/              /* v2: 1526 = 1122 or 3748 = 3300*/


    /* Relaxation delay ***********************************/
    status(A);
    delay(predelay);
    xgate(ticks);

    /* Optional water suppression *************************/
    if (ws[0] == 'y') {
        obspower(satpwr+pwr_tweak);
        obsoffset(wsfrq);

        sp4on();
	shapedpulse(satpat,psat,zero,rof1,rof1);
        sp4off();

	
                /*delay(0.001);
                obsoffset(wsfrq-740.0);
                sp4on();
                shapedpulse(satpat,psat,zero,rof1,rof1);
                sp4off();*/


     obl_shapedgradient("gspl","","gspl",pw_spoil,agsp,0.0,agsp,1,WAIT);
	zero_all_gradients();
	delay(0.150);

	obspower(satpwr+pwr_tweak);
	obsoffset(wsfrq);
        sp4on();
	shapedpulse(satpat,psat,zero,rof1,rof1);
        sp4off();

	/*delay(0.001);
                obsoffset(wsfrq-740.0);
                sp4on();
                shapedpulse(satpat,psat,zero,rof1,rof1);
                sp4off();*/

      obl_shapedgradient("gspl","gspl","",pw_spoil,agsp,agsp,0.0,1,WAIT);
	zero_all_gradients();
	delay(0.080);

	obspower(satpwr+pwr_tweak+5.0);
	obsoffset(wsfrq);
        sp4on();
	shapedpulse(satpat,psat,zero,rof1,rof1);
        sp4off();



	/*delay(0.001);
                obsoffset(wsfrq-740.0);
                sp4on();
                shapedpulse(satpat,psat,zero,rof1,rof1);
                sp4off();*/

       obl_shapedgradient("","gspl","gspl",pw_spoil,0.0,agsp,agsp,1,WAIT);
	zero_all_gradients();

	 delay(0.160);
	obspower(satpwr+pwr_tweak);
	obsoffset(wsfrq);
        sp4on();
	shapedpulse(satpat,psat,zero,rof1,rof1);
        sp4off();

/*	delay(0.001);
                obsoffset(wsfrq-740.0);
                sp4on();
                shapedpulse(satpat,psat,zero,rof1,rof1);
                sp4off();*/


	obl_shapedgradient("gspl","","gspl",pw_spoil,agsp,0.0,agsp,1,WAIT);
	zero_all_gradients();
	delay(0.080);


	obspower(satpwr+pwr_tweak+5.0);
        obsoffset(wsfrq);
        sp4on();
        shapedpulse(satpat,psat,zero,rof1,rof1);
        sp4off();


        obl_shapedgradient("gspl","gspl","",pw_spoil,agsp,agsp,0.0,1,WAIT);
        zero_all_gradients();
        delay(0.100);

	obspower(satpwr+pwr_tweak);
        obsoffset(wsfrq);
        sp4on();
        shapedpulse(satpat,psat,zero,rof1,rof1);
        sp4off();


        obl_shapedgradient("","gspl","gspl",pw_spoil,0.0,agsp,agsp,1,WAIT);
        zero_all_gradients();
        delay(0.030);

	obspower(satpwr+pwr_tweak+5.0);
        obsoffset(wsfrq);
        sp4on();
        shapedpulse(satpat,psat,zero,rof1,rof1);
        sp4off();

        obl_shapedgradient("gspl","","gspl",pw_spoil,agsp,0.0,agsp,1,WAIT);
        zero_all_gradients();
        delay(0.026);



	}


if (ws[0] == 'n') {
	

        delay(psat+rof1+rof1);
      obl_shapedgradient("gspl","gspl","",pw_spoil,agsp,agsp,0.0,1,WAIT);
        zero_all_gradients();
	delay(0.15);

        delay(psat+rof1+rof1);
      obl_shapedgradient("gspl","","gspl",pw_spoil,agsp,0.0,agsp,1,WAIT);
        zero_all_gradients();
	delay(0.08);

        delay(psat+rof1+rof1);
      obl_shapedgradient("","gspl","gspl",pw_spoil,0.0,agsp,agsp,1,WAIT);
        zero_all_gradients();
	delay(0.16);

	delay(psat+rof1+rof1);
	obl_shapedgradient("gspl","gspl","",pw_spoil,agsp,agsp,0.0,1,WAIT);
        zero_all_gradients();
	delay(0.08);

	delay(psat+rof1+rof1);
        obl_shapedgradient("gspl","","gspl",pw_spoil,agsp,0.0,agsp,1,WAIT);
        zero_all_gradients();
        delay(0.10);

	delay(psat+rof1+rof1);
        obl_shapedgradient("","gspl","gspl",pw_spoil,0.0,agsp,agsp,1,WAIT);
        zero_all_gradients();
	delay(0.03);

	delay(psat+rof1+rof1);
        obl_shapedgradient("gspl","gspl","",pw_spoil,agsp,agsp,0.0,1,WAIT);
        zero_all_gradients();
	delay(0.026);

	

	
    }


    /* 90 degree RF pulse *********************************/
    obspower(tpwr1);
	position_offset(pos1,gvox1,refreq,OBSch);
    /*poffset(pos1,gvox1);*/
    obl_shapedgradient("gvox1","","",pw_gvox1,agvox1,0.0,0.0,1,NOWAIT);
    delay(agvox1*grate+tep);
    sp4on();
    shapedpulse(p1pat,p1,v3,rof1,rof1);
    sp4off();
    if(agvox1*grate > tep)
    {
    delay(agvox1*grate -tep);
}


    /* First half spin-echo period ************************/
    obl_shapedgradient("gssr4_up","","",pw_gvox1r,gvox1r,0.0,0.0,1,WAIT);
    delay(tref-pw_gvox1r);
    obl_shapedgradient("gssr4_dn","","",pw_gvox1r,gvox1r,0.0,0.0,1,WAIT);
	zero_all_gradients();
    delay(tau1);

    /* First 180 degree pulse with crushers*/
    obspower(tpwr2);
	position_offset(pos2,gvox2,refreq,OBSch);
    /*poffset(pos2,gvox2);*/
    obl_shapedgradient("gvoxsp3","gvoxsp1","gvoxsp3",pw_gvox2,agvsp3,agvsp1,agvsp3,1,NOWAIT);
    delay(2.0*crush_ramp_time1+tcrush+tep);
    sp4on();
    shapedpulse(p2pat,p2,v2,rof1,rof1);
    sp4off();
    if(2.0*crush_ramp_time1+tcrush > tep)
{
    delay(2.0*crush_ramp_time1+tcrush-tep);
}

	zero_all_gradients();

    delay(tau2);


    /* Second 180 degree pulse ****************************/
    obspower(tpwr2);
	position_offset(pos3,gvox3,refreq,OBSch);
    /*poffset(pos3,gvox3);*/
    obl_shapedgradient("gvoxsp4","gvoxsp4","gvoxsp2",pw_gvox3,agvsp4,agvsp4,agvsp2,1,NOWAIT);
    delay(2.0*crush_ramp_time2+tcrush+tep);
    sp4on();
    shapedpulse(p2pat,p2,v1,rof1,rof1);
    sp4off();
        if(2.0*crush_ramp_time2+tcrush > tep)
{
    delay(2.0*crush_ramp_time2+tcrush-tep);
}
	zero_all_gradients();
/*startfifo();*/

 /*pe3_shapedgradient("gpe_up",pw_gpe,0.0,-sgpe2*nv2/2.0,-sgpe*nv/2.0,0.0,sgpe2,sgpe,zero,v6,v8);
        delay(m_tref-pw_gpe);
pe3_shapedgradient("gpe_dn",pw_gpe,0.0,-sgpe2*nv2/2.0,-sgpe*nv/2.0,0.0,sgpe2,sgpe,zero,v6,v8);*/
pe3_gradient(-sgpe2*nv2/2.0,-sgpe*nv/2.0,0.0,sgpe2,sgpe,0.0,v8,v6,zero);

delay(m_tref);

        zero_all_gradients();

    delay(tau3);
 

    obsoffset(tof);
    acquire(np,1.0/sw);
    endpeloop(seqcon[2],v6);
  endpeloop(seqcon[3],v8);

}

/*********************************************************************
                      MODIFICATION HISTORY

20may97(ss) New sequence, derived from steam.c.
22May97(ss) Cyclops phase cycling added.

*********************************************************************/
