
/* diff_eddy - get abs of difference and diff of abs   */
/* in multi fid data from eddy sequence.               */
/* calculates difference between first and second half */
/* of fid series (eg. 4-1,5-2,6-3) in file.            */
/* output is stored in same file, first half is        */
/* abs of diff., second part is diff of abs            */
/* 
/* Peter van Gelderen, In vivo NMR Center, 17 july 97  */
/* 
/* Based on :             */
/* diff2d.c -  T. Jellard Nov '91 */


/* to compile:
      cc diff_eddy.c -lm -o diff_eddy 

   Calling sequence:
      diff_eddy file */

#include <stdio.h>
#include <malloc.h>
#include <math.h>

float c_abs(a_rl, a_im)
float a_rl, a_im;
{
  return sqrt(a_rl*a_rl+ a_im*a_im);
}

main (argc, argv )
int argc;
char *argv[];

{
struct fileHeader{
   long nblocks, ntraces, np, ebytes, tbytes, bbytes;
   short vers_id, status;
   long nbheaders; };
struct blockHeader{
   short scale, status, index, mode;
   long ctcount;
   float lpval, rpval, lvl,tlt; };
struct hypercmplxbHead{
   short s_spare1, status, s_spare2, s_spare_3;
   long l_spare1;
   float lpval1, rpval1, f_spare1, f_spare2; };

struct fileHeader  fheader;
struct blockHeader bheader;
struct hypercmplxbHead xheader;
int k,i;
long  *lde, l1, l2, headsize, pos, no_elements, ne_b, offset;
short *ide, i1, i2;
char *filename;
FILE *file;
float x;

/* check for correct usage */

if ( argc != 2 ) {
   fprintf(stderr, "Usage: diff_eddy file \n" );
   exit(1); }
    
filename = argv[1];

/* open the file */

file = fopen( filename,"r+");
if (file == NULL) { 
  fprintf( stderr, "error: unable to open file %s\n", filename);
  exit(1);}

/* read the file headers */

if (fread( &fheader, sizeof(fheader), 1, file) != 1) {
  fprintf( stderr, "error: fread failed for header\n");
  exit(1);}
ne_b = fheader.np *fheader.ntraces;
no_elements = fheader.nblocks*ne_b;

if (fheader.ebytes == 2) {
  ide= (short *) malloc(no_elements* sizeof(int));
  if (ide == NULL) {
    fprintf( stderr, "error: unable to allocated memory for ide\n");
    exit(1);}}
else {
  lde= (long *) malloc(no_elements* sizeof(long));
  if (lde == NULL) {
    fprintf( stderr, "error: unable to allocated memory for lde\n");
    exit(1);} }

  
/* print the file header info */
/* 
printf( "FILE HEADER\n" );
printf( "number of blocks in file = %d\n", fheader.nblocks );
printf( "number of traces per block = %d\n", fheader.ntraces );
printf( "number of elements per trace = %d\n", fheader.np );
printf( "number of bytes per element = %d\n", fheader.ebytes );
printf( "number of bytes per trace = %d\n", fheader.tbytes );
printf( "number of bytes per block = %d\n", fheader.bbytes );
printf( "version_id = %d\n", fheader.vers_id );
printf( "status = %d\n", fheader.status );
printf( "number of block headers = %d\n", fheader.nbheaders );
fprintf( stderr, "\nrunning");  
*/

pos= sizeof(fheader);
if (fheader.nbheaders != 1) {headsize=  sizeof(bheader)+  sizeof(xheader);}  
else {headsize= sizeof(bheader);}


/* Read the data, skipping the header(s) */

for ( k = 0; k < fheader.nblocks; k++ ) {
  if (fseek(file, pos+headsize, SEEK_SET) != 0) {
    fprintf( stderr, "error: fseek failed\n");
    exit(1);} 
  pos += headsize;
  
  if (fheader.ebytes == 2) { 
    if (fread(ide+k*ne_b,sizeof(short),ne_b,file) != ne_b) {
      fprintf( stderr, "error: fread failed \n");
      exit(1);} }
  else {
    if (fread(lde+k*ne_b,sizeof(long),ne_b,file) != ne_b) {
      fprintf( stderr, "error: fread failed \n");
      exit(1);} }
  pos+= ne_b*fheader.ebytes; 
}

/* Do the math */
offset= no_elements/2;
if (fheader.ebytes == 2) { 
  for (k=0;k<offset;k+=2){
    i1= (short) c_abs((float) (ide[k]- ide[k+offset]), (float) (ide[k+1]- ide[k+offset+1]));
    i2= (short) (c_abs((float) ide[k], (float) ide[k+1])- c_abs((float) ide[k+offset], (float) ide[k+offset+1]) );
    ide[k]= i1;
    ide[k+offset]= i2;}}
else { 
  for (k=0;k<offset;k+=2){
    l1= (long) c_abs((float) (lde[k]- lde[k+offset]), (float) (lde[k+1]- lde[k+offset+1]));
    l2= (long) (c_abs((float) lde[k], (float) lde[k+1])- c_abs((float) lde[k+offset], (float) lde[k+offset+1]) );
    lde[k]= l1;
    lde[k+offset]= l2;}}
    
pos=  sizeof(fheader);
/* Put data back */
for ( k = 0; k < fheader.nblocks; k++ ) {
  if (fseek(file, pos+headsize, SEEK_SET) != 0) {
    fprintf( stderr, "error: fseek failed\n");
    exit(1);} 
  pos += headsize;
  
  if (fheader.ebytes == 2) { 
    if (fwrite(ide+ k*ne_b,sizeof(short),ne_b,file) != ne_b) {
      fprintf( stderr, "error: fwrite failed \n");
      exit(1);} }
  else {
    if (fwrite(lde+ k*ne_b,sizeof(long),ne_b,file) != ne_b) {
      fprintf( stderr, "error: fwrite failed \n");
      exit(1);} }
  pos+= ne_b*fheader.ebytes; 
}    
         
/* Clean up */

fprintf( stderr, "done\n" );
fclose(file);
if (fheader.ebytes == 2) {
  free(ide);}
else {
  free(lde);}

}
