#ifndef LINT
static char SCCSid[] = "@(#)flash.c 11.1 05/07/96 Copyright (c) 1994-1996 Varian Assoc.,Inc. All Rights Reserved";
#endif
/***********************************************************************
    FLASH.C

    Gradient-echo fast imaging sequence.                     

    VERSION 2.0        6/94/94  
    Revisions:
      2.1   2/96    Fixed rfspoiling for nv=0

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
        __                                                  __ 
       |                                                      |
  RF   |----- [ p1 ] ----------------------- |** ACQ **| -----|
       |                                                      |
       |         < - - - - - - - te - - - - - - - >           |
       |       ____                                           |
  GSS  |______/////\     _____________________________________|
       |<---->      \\\\/ gssr                                |
       |  predelay  <--> tref                                 | nv
       |             ___                                  ___ |
  GPE  |____________/___\________________________________/___\|
       |            \___/                                \___/|
       |                 < - tA - >                           |
       |                            ____________________      |
  GRO  |____________     __________/////////////////////\_____|
       |       gror \\\\/          < tspoil >                 |
       |                                                      |
       |< - - - - - - - - - - - tr - - - - - - - - - - - - - >|
       |__                                                  __|

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

#include <standard.h>
#include "sequence_utilities.h"
#define OFFSETDELAY 26e-6
#define RFSPOILDELAY 50e-6


pulsesequence()
{
    /* Internal variable declarations *********************/
    double  predelay,seqtime,tref,risetref;
    double  trefmin,tssrmin,trormin,tpemin,temin,tA;
    double  agro,agss,sgpe,grate,gssrint,grorint,gpeint;      
    double  reffreq,nextphase,spoilfreqs[1024];
    int     i,image;
    char    err_string[MAXSTR];

    initparms_sis();
    loop_check();

    image = (int)(getval("image")); /* trigger and counter bit pattern */
    /* bit 1 = wait for ticks triggers before beginning scan */
    /* bit 2 = puts out high on sp3 for duration of volume */
    /* bit 3 = reserved for epi blips on/off */

    grate = trise/gmax;
    agro = fabs(gro);
    agss = fabs(gss);

    /* GRADIENT*TIME integrals for slice selection and readout */
    grorint = agro*(at/2.0 + tspoil + grate*agro/2.0);
    gssrint = agss*(p1/2.0 + rof1 + agss*grate/2.0);
    gpeint = B0/(lpe*sfrq*1e6);

    /*******************************************************
    * Calculate minimum time for phase-encoding, slice refocussing
    * and readout prefocussing block.  Trapezoidal if GMAX is exceeded.
    * TSSRMIN does not include the gradient fall time.  RISETREF is
    * the gradient fall time for the PE block.
    *******************************************************/
    if (trise*gmax >= gssrint)
	tssrmin = sqrt(grate*gssrint);
    else
	tssrmin = gssrint/gmax;

    if (trise*gmax >= grorint)
        trormin = sqrt(grate*grorint);
    else
	trormin = grorint/gmax;

    if (trise*gmax >= gpeint*nv/2.0)
        tpemin = sqrt(grate*gpeint*nv/2.0);
    else
	tpemin = gpeint*nv/2.0/gmax;

    /* Minimum refocussing block time is the longest of calculated minima */
    trefmin = (tssrmin > trormin) ? tssrmin : trormin;
    trefmin = (trefmin > tpemin) ? trefmin : tpemin;

    /*******************************************************
    * If TREFMIN implies trapezoidal PE block, mimimum RISETREF
    * is RISETIME, otherwise minimum RISETREF is TREFMIN.
    *******************************************************/
    risetref = (trefmin > trise) ? trise : trefmin;

    /* Check minimum TE ***********************************/
    temin = p1/2.0 + rof1 + trefmin + risetref + tspoil
          + at/2.0 + grate*(agss + agro);
    if (te < temin) {
        sprintf(err_string,"%s: TE too short. Min TE = %f\n",seqfil,temin);
	printf(err_string);
        log_error(err_string);
	abort(1);
    }

    /*******************************************************
    * TREF is the refocussing block length, not including gradient fall
    * time.  TREF is scaled up to a maximum value consistent with TE
    *******************************************************/
    tref = trefmin + 0.5*(te - temin);
    tref = (tref > 2.0*trefmin) ? 2.0*trefmin : tref;

    /* Calculate GPE, adjusting for integral dac value, then adjust TREF */
    sgpe = gpeint/tref;
    sgpe = gmax/gradstepsz*floor(sgpe*gradstepsz/gmax + 0.5);
    tref = gpeint/sgpe;

    /* Calculate GSSR and GROR with new TREF **************/
    gror = -grorint/tref*gro/fabs(gro);
    gssr = -gssrint/tref*gss/fabs(gss);

    /* Determine PE block gradient fall time with new TREF */
    risetref = fabs(gssr) > fabs(gror) ? fabs(gssr) : fabs(gror);
    risetref = risetref > fabs(sgpe*nv/2.0) ? risetref : fabs(sgpe*nv/2.0);
    risetref *= grate;

    /* tA is the delay between refocussing and readout ****/
    tA = te - (p1/2.0 + rof1 + tref + risetref + tspoil + at/2.0
       + grate*(agss + agro));

    /* Relaxation delay ***********************************/
    seqtime = te + p1/2.0 + rof1 + at/2.0 + grate*(agss + agro)
	    + tref + risetref;
    if (rfspoil[0] == 'y'  ||  rfspoil[0] == 'Y')
	seqtime += RFSPOILDELAY;
    if (tr < seqtime) {
        sprintf(err_string,"%s: TR too short. Min TR = %f\n",seqfil,seqtime);
	printf(err_string);
        log_error(err_string);
        abort(1);
    }
    predelay = tr - seqtime;

    /*******************************************************
    * Calculation of frequencies to get phase shifts for optional
    * RF spoiling.  Phase shifts are obtained by hopping to variable
    * frequency for fixed time.
    * Ref:  Zur, Y., Magn. Res. Med., 21, 251, (1991).
    *******************************************************/
    if ((rfspoil[0] == 'y' || rfspoil[0] == 'Y') && nv > 0) {
	reffreq = resto + sfrq*1e6/B0*gro*pro;
        for (i=1; i<= (int)(nv+0.5); i++) {
	    nextphase = (double)i/2.0*((double)i-1.0)*rfphase;
	    nextphase = nextphase - 360.0*floor(nextphase/360.0);
	    spoilfreqs[i] = reffreq + nextphase/RFSPOILDELAY/360.0;
        }
        create_offset_list(spoilfreqs,nv,TODEV,0);
    }


    /* PULSE SEQUENCE *************************************/

    /* Phase cycle ****************************************/
    mod2(ct,v3);          /* v3: 0  1  0  1  0  1  0  1 */
    dbl(v3,v3);           /* v3: 0  2  0  2  0  2  0  2 */
    hlv(ct,v4);           /* v4: 0  0  1  1  2  2  3  3 */
    mod2(v4,v4);          /* v4: 0  0  1  1  0  0  1  1 */
    add(v3,v4,v1);        /* v1: 0  2  1  3  0  2  1  3 */
    assign(v1,oph);      /* oph: 0  2  1  3  0  2  1  3 */

    status(A);
    sp4off();
    if ((image & 1) == 1) xgate(ticks);
    if ((image & 2) == 2) sp3on(); else sp3off();

    sp2on();
    delay(0.00005);  /* 50 us trigger to indicate start of volume acq */
    sp2off();

    obspower(tpwr1);
    peloop(seqcon[2],nv,v5,v6);

      /* Relaxation delay ***********************************/       
      poffset(pss[0],gss);
      delay(predelay);
       
      /* RF pulse *******************************************/ 
      sp1on();  /* put out trigger on sp1 every spin excitation */
      obl_gradient(0.0,0.0,gss);
      delay(grate*agss);
      sp4on();
      shapedpulse(p1pat,p1,v1,rof1,rof1);
      sp4off();
      zero_all_gradients();
      delay(grate*agss);
      sp1off();

      /* Read & Slice refocus and phase encode pulse ********/
      pe_gradient(gror,sgpe*nv/2.0,gssr,-sgpe,v6);
      delay(tref);
      zero_all_gradients();
      delay(risetref);

      /* Pre-acquire delay **********************************/
      delay(tA);

      /* Acquire echo ***************************************/
      poffset(pro,gro);
      obl_gradient(gro,0.0,0.0);
      delay(grate*agro + tspoil + 0.5/fb);
      acquire(np,1.0/sw);
      zero_all_gradients();
      delay(grate*agro);
      pe_gradient(0.0,-sgpe*nv/2.0,gss,sgpe,v6);
      delay(tref);
      zero_all_gradients();
      delay(risetref);

      /*******************************************************
      * Optional RF spoiling.  REMINDER: Do not place any events
      * after this block or before poffset at the start of peloop.
      *******************************************************/
      if ((rfspoil[0] == 'y' || rfspoil[0] == 'Y') && nv > 0) {
        voffset(0,v6);
        delay(RFSPOILDELAY - OFFSETDELAY);
      }

    endpeloop(seqcon[2],v6);
}
