#ifndef LINT
static char SCCSid[] = "@(#)fsems.c 13.1 10/10/97 Copyright (c) 1991-1996 Varian Assoc.,Inc. All Rights Reserved";
#endif
/***********************************************************************
    FSE.C

    Fast spin-echo imaging sequence with inversion recovery pulse.

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
     _                       __                                 __     _
    |                       |                                     |     |
RF  | ------ [90] ----------|-- [180] ------- |** ACQ **|---------|     |
    |                       |                                     |     |
    |          < - - - - - -|- -esp - - - - - - - >               |     |
    |        ____           |   _____                             |     |
GSS | ______/////\     _____|__//////\____________________________|     |
    | <---->      \\\\/ gssr|                                     |     |
    |   predelay  <--> tref |                                     | etl | ns
    |                       |                                     |     |
    |                       |            ___               ___    |     |
GPE |  _____________________|___________/   \_____________/   \___|     |
    |                       |           \___/             \___/   |     |
    |                       |                                     |     |
    |         gror ___      |                  _________          |     |
GRO |  ___________////\_____|_________________//////////\_________|     |
    |                       |                                     |     |
    |_                      |__                                 __|    _|

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

/*
  Tables
  t1  : FSE table
  t41 : tpwr1 + rfmod
  t42 : tpwr2 + rfmod
  t43 : tpwrfat + rfmod
  t44 : tpwri + rfmod
  t45 : tpwr3 + rfmod
  t59 : linear modulation (sat)
  t60 : linear modulation
*/

#include <standard.h>
#include <group.h>
#include "sequence_utilities.h"

codeint v15,v16;

/* INTERNAL VARIABLE DECLARATIONS *********************/
int     i,table=0;
double  esp,etl,predelay,seqtime,tref;
double  risetref,risetpe,act_tpe,trefmin,espmin,tpemin,tau1,tau2,tau3;
double  agro,agss,sgpe,grate,gssrint,grorint,gpeint,maxgint,grise;
double  gss2,agss2,agcrush,agssc,crush_ramp_time;
double  nucgamma,atf,frewind1[MAXSLICE],frewind2[MAXSLICE],frewind3;
char    shape_name[MAXSTR];
char    prep[MAXSTR],scope[MAXSTR],pspat[MAXSTR],cgate[MAXSTR];
char    fat_sat[MAXSTR];
double  tiact,te1,te2,te3,phase_res;
double  tsat,pss_sat1,pss_sat2,gss_sat,agss_sat,gss_prep;
double  nsat,dsat,tfscrush;
double  tpwrs,ps,pfat,tpwrfat;
double  tep,rftep,pw_gss,pw_gss2,pw_gro,pw_tref,pw_risetpe,pw_gspoil;
double  prescan_flag,pw_gssi;
char    gcoil[MAXSTR],err_string[MAXSTR],pfatpat[MAXSTR];
char    rfmod[MAXSTR];

#include "fse_safety_calc.h"

/* Spatial Sat declarations */
/* declare values additionally for satpat, psat various list params */
double gsssat,pw_gsssat;
double presat_off=0.0,post_presat_del,sat_thk,sat_psi,sat_phi,sat_theta;
double presat=0.0,pw_gcrush;
double t_presat_b,t_presat_a;

calculate_spatial_saturation_gradients(double *t_presat_b,double *t_presat_a)
{
  char shape_name[MAXSTR];

  sprintf(shape_name,"gsat");
  pw_gsssat = create_trap_gradient(fabs(gsssat),shape_name,
         psat+rof1+rof2,grate,ix);

  *t_presat_b = fabs(gsssat)*grate + rof1 + psat/2;
  *t_presat_a = fabs(gsssat)*grate + rof2 + psat/2 + post_presat_del;
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
    sprintf(shape_name,"gcr");
    pw_gcrush = create_ramp_gradient(fabs(gcrush),shape_name,1,grate,ix);
    pw_gcrush = create_ramp_gradient(fabs(gcrush),shape_name,-1,grate,ix);
    if (pw_gcrush > tcrush) {
      sprintf(err_string,"%s: tcrush too small. Min tcrush = %f\n",
              seqfil,pw_gcrush);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
    *t_presat_a=*t_presat_a + 2.0*pw_gcrush + tcrush;
  }
}

spatial_saturation_pulse()
{
  /* uses psat : satpwr : satpat : gsssat */
  /* also uses sat_psi, sat_phi and sat_theta rather than psi, phi, theta */
  /* Enables definition of independent sat planes relative to slice plane */
  position_offset(presat_off,gsssat,resto,OBSch);
  obspower(satpwr);

  oblique_shapedgradient("","","gsat",pw_gsssat,0.0,0.0,gsssat,sat_psi,sat_phi,sat_theta,1,NOWAIT);

  delay(fabs(gsssat)*grate+rftep);
  sp4on();
  shaped_pulse(satpat,psat,v1,rof1,rof2);
  sp4off();
  if(rftep<fabs(gsssat)*grate)delay(fabs(gsssat)*grate-rftep);

  /* Crushers */
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
    obl_shapedgradient("gcr_up","gcr_up","gcr_up",
                       pw_gcrush,gcrush,gcrush,gcrush,1,WAIT);
    delay(tcrush);
    obl_shapedgradient("gcr_dn","gcr_dn","gcr_dn",
                       pw_gcrush,gcrush,gcrush,gcrush,1,WAIT);
  }
  delay(post_presat_del);
}

calculate_rf_modulation()
{
  double pss1,pss2,pwr1,pwr2,target,newtarg,dbi;
  char tpwr_file[MAXSTR],rfcoil[MAXSTR],full_fname[MAXSTR],*home_dir;
  int pwr_mod[MAXSTR],db_incr[MAXSTR],i,ret;
  FILE *fp,*table;
  char err_string[MAXSTR];
  double satpss[2];
  int db_incr_sat[2],pwr_mod_sat[2];

  getstr("rfcoil",rfcoil);

  home_dir=(char *)getenv("HOME");
  sprintf(tpwr_file,"%s/vnmrsys/shapelib/%s.SCALE",home_dir,rfcoil);

  fp=fopen(tpwr_file,"r");
  if(fp==NULL){
    sprintf(tpwr_file,"/vnmr/shapelib/%s.SCALE",rfcoil);
    fp=fopen(tpwr_file,"r");
    if(fp==NULL){
      sprintf(err_string,"%s: Unable to open file: %s\n"
	      ,seqfil,tpwr_file);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
  }
  
  for(i=0;i<ns;i++){
    rewind(fp);
    ret=fscanf(fp,"%lf\t%lf\n",&pss1,&pwr1);
    rewind(fp);
    while(1){
      ret=fscanf(fp,"%lf\t%lf\n",&pss2,&pwr2);
      if(ret==EOF){
	sprintf(err_string,"%s: pss out of range of tpwr_file\n",seqfil);
	printf(err_string);
	log_error(err_string);
	abort(1);
      }
      if(pss2>pss[i]){
	target=pwr1;
	pwr2-=pwr1;
	pwr2*=((pss[i]-pss1)/(pss2-pss1));
	target+=pwr2;
	break;
      }
      pss1=pss2;
      pwr1=pwr2;
    }
    /* calculate db increase */
    dbi=20*log10(target);
    db_incr[i]=ceil(dbi);
    newtarg=pow(10,(double)db_incr[i]/20);
    newtarg=target/newtarg;
    pwr_mod[i]=(int)(newtarg*32767);
  }

  /* Sat bands */
  satpss[0]=pss_sat1;satpss[1]=pss_sat2;
  for(i=0;i<2;i++){
    rewind(fp);
    ret=fscanf(fp,"%lf\t%lf\n",&pss1,&pwr1);
    rewind(fp);
    while(1){
      ret=fscanf(fp,"%lf\t%lf\n",&pss2,&pwr2);
      if(ret==EOF){
	sprintf(err_string,"%s: pss out of range of tpwr_file\n",seqfil);
	printf(err_string);
	log_error(err_string);
	abort(1);
      }
      if(pss2>satpss[i]){
	target=pwr1;
	pwr2-=pwr1;
	pwr2*=((satpss[i]-pss1)/(pss2-pss1));
	target+=pwr2;
	break;
      }
      pss1=pss2;
      pwr1=pwr2;
    }
    dbi=20*log10(target);
    db_incr_sat[i]=ceil(dbi);
    newtarg=pow(10,(double)db_incr[i]/20);
    newtarg=target/newtarg;
    pwr_mod_sat[i]=(int)(newtarg*32767);
  }

  fclose(fp);

  sprintf(full_fname,"%s/vnmrsys/tablib/tpwr_table",home_dir);
  table=fopen(full_fname,"w");
  if(table==NULL){
    sprintf(err_string,"%s: Unable to open ~/vnmrsys/tablib/tpwr_table\n",
	    seqfil);
    printf(err_string);
    log_error(err_string);
    abort(1);
  }
  fprintf(table,"t41 = ");
  for(i=0;i<ns;i++){
    fprintf(table,"%d ",db_incr[i]+(int)tpwr1);
  }
  fprintf(table,"\nt42 = ");
  for(i=0;i<ns;i++){
    fprintf(table,"%d ",db_incr[i]+(int)tpwr2);
  }
  fprintf(table,"\nt43 = ");
  for(i=0;i<ns;i++){
    fprintf(table,"%d ",db_incr[i]+(int)tpwrfat);
  }
  fprintf(table,"\nt44 = ");
  for(i=0;i<ns;i++){
    fprintf(table,"%d ",db_incr[i]+(int)tpwri);
  }
  fprintf(table,"\nt45 = ");
  for(i=0;i<2;i++){
    fprintf(table,"%d ",db_incr_sat[i]+(int)tpwrs);
  }
  /*
  fprintf(table,"\nt59 = ");
  for(i=0;i<2;i++){
    fprintf(table,"%d ",pwr_mod_sat[i]);
    }*/
  fprintf(table,"\nt60 = ");
  for(i=0;i<ns;i++){
    fprintf(table,"%d ",pwr_mod[i]);
  }
  fprintf(table,"\n");
  fclose(table);
  
  loadtable("tpwr_table");
}

pulsesequence()
{
  
  initparms_sis();
  nucgamma = 1e6*sfrq/B0;
  if (strcmp(petable,"n")  &&  strcmp(petable,"N")  &&  strcmp(petable,"")) {
    loadtable(petable);
    table = 1;
  }
  getstr("gcoil",gcoil);
  etl = getval("etl");
  phase_res = getval("phase_res");  /* phase encode resolution */
  esp = getval("delta_te");
  atf = getval("atf");
  gss2 = getval("gss2");
  pss_sat1 = getval("pss_sat1");
  pss_sat2 = getval("pss_sat2");
  gss_sat = getval("gss_sat");
  gss_prep = getval("gss_prep");
  tpwrs = getval("tpwrs");
  ps = getval("ps");
  nsat = getval("nsat");
  dsat = getval("dsat");
  tfscrush = getval("tfscrush");
  agro = fabs(gro);
  agss = fabs(gss);
  agss2 = fabs(gss2);
  agcrush = fabs(gcrush);
  getstr("prep",prep);
  getstr("scope",scope);
  getstr("pspat",pspat);
  getstr("cgate",cgate);
  agss_sat = fabs(gss_sat);
  getstr("rfmod",rfmod);

  /* fat sat stuff */
  getstr("fat_sat",fat_sat);    /* fat saturation flag 'y'=on. 'n'=off. */
  getstr("pfatpat",pfatpat);    /* modulation pattern for fat sat pulse */
  tpwrfat = getval("tpwrfat");  /* fat sat power in dB */
  pfat = getval("pfat");        /* duration of fat sat pulse in us */



  P_getreal(GLOBAL,"tep",&tep,1); /* small tweaker time (us) to allow for
				     the gradient propagation time,
				     causes echoes to appear symmetrically
				     about echo center line */
  tep=tep/1e6;
  P_getreal(GLOBAL,"rftep",&rftep,1); /* small tweaker time (us) to allow for
					 the gradient propagation time,
					 w.r.t. RF pulses */
  rftep=rftep/1e6;
  P_getreal(GLOBAL,"grise",&grise,1); /* grad risetime to full strength */
  if ((grise < 0.0) || (grise > 0.002)) {
    sprintf(err_string,"%s: impropper grise time defined in global file\n",
                     seqfil);
    printf(err_string);
    log_error(err_string);
    abort(1);
  }
  grate = grise/gmax;
  
  /* Spatial Sat Variables */
    presat=getval("presat");
    if (presat) {
      gsssat = getval("gsssat");
      presat_off=getval("presat_off");
      sat_psi=getval("sat_psi");
      sat_phi=getval("sat_phi");
      sat_theta=getval("sat_theta");
      sat_thk=getval("sat_thk");
      post_presat_del=getval("post_presat_del");
    }
    if (presat) calculate_spatial_saturation_gradients(&t_presat_b,&t_presat_a);

  if (grof == 0.0  ||  !var_active("grof",1) )
    grof = 1.0;
  if (gssf == 0.0  ||  !var_active("gssf",1) )
    gssf = 1.0;
  
  sprintf(shape_name,"gss");
  pw_gss=create_trap_gradient(agss,shape_name,rof1+p1+rof2,grate,ix);

  sprintf(shape_name,"gss2");
  pw_gss2=create_spin_echo_gradient(shape_name,gss2,rof1+p2+rof2,gcrush,
				    tcrush,tcrush,grate,ix);
  crush_ramp_time=(pw_gss2-(rof1+p2+rof2+2.0*tcrush))/4.0;
  /* need to send the bigger of gcrush and gss2 to the obl_shapedgradient
       playout, since waveform is scaled to 32767 max */
    agssc = fabs(gcrush) > fabs(gss2) ? fabs(gcrush) : fabs(gss2);
  sprintf(shape_name,"gro");
  pw_gro=create_trap_gradient(agro,shape_name,at+2.0*tspoil,grate,ix); 
  sprintf(shape_name,"spoil");
  pw_gspoil=create_trap_gradient(fabs(gspoil),shape_name,2.0*tspoil,grate,ix); 

  /****
   * Calculate minimum time for phase-encoding, slice refocussing
   * and readout prefocussing block.
   ****/
  /* GRADIENT*TIME INTEGRALS FOR SLICE SELECTION AND READOUT */
  grorint = agro*(at/2.0 + tspoil + agro*grate/2.0);
  gssrint = agss*(p1/2.0 + rof2 + agss*grate/2.0);
  
  /* MINIMUM REFOCUSSING BLOCK TIME.  TRAPEZOIDAL IF GMAX IS EXCEEDED */
  /* NOTE, TREFMIN DOES NOT INCLUDE FALL TIME */
  maxgint = grorint > gssrint ? grorint : gssrint;
  if (grise*gmax >= maxgint) {
    trefmin = sqrt(grate*maxgint);  /* triangular pulse */
    risetref = trefmin;
  }
  else {
    trefmin = grate*gmax;           /* trapezoidal pulse */
    risetref = grise;
  }
  
  /* calculate minimum possible values for tpe and risetpe */
  gpeint=B0/(lpe*sfrq*1.0e6);  /* gradient integral for one k space line */
  maxgint = gpeint*phase_res/2.0;
  if (grise*gmax >= maxgint) {
    tpemin = sqrt(grate*maxgint);  /* triangular pulse */
    risetpe = tpemin;
  }
  else {
    tpemin = grate*gmax;           /* trapezoidal pulse */
    risetpe = grise;
  }
  
  /* THE TAUs ARE THE MINIMUM SUMS OF ALL EVENTS IN EACH HALF-ECHO PERIOD */
  
  tau1 = p1/2.0 + rof2 + grate*agss + trefmin + risetref +
    crush_ramp_time + tcrush + crush_ramp_time + rof1 + p2/2;
  if(grate*agss < rftep) tau1 += (rftep-grate*agss);

  tau2 = p2/2.0 + rof2 + crush_ramp_time + tcrush +
    crush_ramp_time + tpemin + risetpe + grate*agro + tspoil + at/2.0;
  if((crush_ramp_time+tcrush+crush_ramp_time)<rftep)
    tau2 += (rftep-(crush_ramp_time+tcrush+crush_ramp_time));

  tau3 = at/2.0 + tspoil + grate*agro + tpemin + risetpe + 
    crush_ramp_time + tcrush + crush_ramp_time + rof1 + p2/2.0;
  if(grate*agro<tep) tau3 += (tep-grate*agro);

  espmin = tau1 > tau2 ? tau1 : tau2;
  espmin = tau3 < espmin ? 2.0*espmin : 2.0*tau3;
  if (esp < espmin) {
    sprintf(err_string,"%s: delta_te too small. Minimum delta_te = %f ms\n",
                     seqfil, espmin*1000.0);
    printf(err_string);
    log_error(err_string);
    abort(1);
  }
  
  /* TREF IS THE REFOCUSSING BLOCK LENGTH, NOT INCLUDING GRADIENT FALL TIME */
  /* TREF IS SCALED UP TO A MAXIMUM VALUE CONSISTENT WITH tau1 */
  tref = trefmin + 0.8*(esp/2.0 - tau1);
  tref = (tref > at/2.0  &&  trefmin <= at/2.0) ? at/2.0 : tref;
  
  /* CALCULATE GPE, ADJUSTING FOR INTEGRAL DAC VALUE, THEN ADJUST TPE */
  act_tpe = tpemin + 0.8*(esp/2.0 - tau2);
  act_tpe = act_tpe > 8.0*tpemin ? 8.0*tpemin : act_tpe;
  sgpe = gpeint/act_tpe;
  /* This assumes sgpe is always positive */
  sgpe = gmax/gradstepsz*floor(sgpe*gradstepsz/gmax + 0.5);
  act_tpe = fabs(gpeint/sgpe);
  risetpe = grate*fabs(sgpe*phase_res/2.0);
  
  /* CALCULATE GSSR AND GROR WITH NEW TREF */
  gror =  grof*grorint/tref*gro/fabs(gro);
  gssr = -gssf*gssrint/tref*gss/fabs(gss);
  
  risetref = fabs(gssr) > fabs(gror) ? grate*fabs(gssr) : grate*fabs(gror);

  sprintf(shape_name,"gtref");
  if (fabs(gssr) > fabs(gror)) {
    pw_tref=create_trap_gradient(fabs(gssr),shape_name,tref-risetref,grate,ix);
  } else {
    pw_tref=create_trap_gradient(fabs(gror),shape_name,tref-risetref,grate,ix);
  }

  if(prep[0]=='i'){
    sprintf(shape_name,"gssi");
    pw_gssi=create_trap_gradient(agss2,shape_name,pi+rof1+rof2,grate,ix);
  }

  sprintf(shape_name,"gdp");
  pw_risetpe=create_ramp_gradient(fabs(risetpe/grate),shape_name,1,grate,ix);
  pw_risetpe=create_ramp_gradient(fabs(risetpe/grate),shape_name,-1,grate,ix);

  /* First half-TE delay *******************************/
  te1 = esp/2.0 - (pw_gss/2.0 + pw_tref + pw_gss2/2.0);
  if(grate*agss < rftep) te1 -= (rftep-grate*agss);

    /* Second half-TE delay *******************************/
  te2 = esp/2.0 - (pw_gss2/2.0 + act_tpe + pw_risetpe + pw_gro/2.0);
  if((crush_ramp_time+tcrush+crush_ramp_time) < rftep)
    te2 -= (rftep-(crush_ramp_time+tcrush+crush_ramp_time));

    /* Post acq second half-TE delay ********/
  te3 = esp/2.0 - (pw_gro/2.0 + act_tpe + pw_risetpe + pw_gss2/2.0);
  if(grate*agro < tep) te3 -= (tep-grate*agro);

    /* inversion delay */
  tiact = ti - pi/2.0 - rof1 - agss2*grate;
  if (tiact < 0.0) tiact=0.0;
  
    /* Saturation band time (not checked) **/
  tsat = 4.0*(rof1+grate*agss_sat) + 2.0*(ps + 0.01);

  /* Relaxation delay */
  seqtime = grate*agss + rof1 + p1/2.0 + etl*esp + at/2.0 + tspoil + 
     act_tpe + risetpe + te3 + pw_gspoil;

  if ((fat_sat[0] == 'y') || (fat_sat[0] == 'Y'))
    seqtime += nsat*(pfat + tfscrush + dsat);
  
  if (prep[0] == 's')
    seqtime += tsat;

  if presat==1) seqtime = seqtime + t_presat_b + t_presat_a;

  if (prep[0] == 'i') {
    if (tr - ((pi/2 + grate*agss2 + rof1 + ti + seqtime)*ns) < 0.0) {
      printf("FSE: Requested tr too short.  Min tr = %f\n",
	     (ti + pi/2 + grate*agss2 + rof1 + seqtime)*ns);
      abort(1);
    }
    predelay = (tr - ((ti + pi/2 + grate*agss2 + rof1 + seqtime)*ns))/ns;
  } else {
    if ( (tr - ns*seqtime) < 0.0) {
      sprintf(err_string,"%s: Requested tr too short. Min tr = %f ms\n",
	      seqfil, ns*seqtime*1000.0);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
    predelay = (tr - ns*seqtime)/ns;
  }

  /* CALCULATE PHASE REFOCUSSING TO CORRECT FOR FREQUENCY JUMPS */
  /* atf is a tweaker used to correct for frequency hop delays */
  for (i=0; i<ns; i++) {
    frewind1[i] = -(p1/2.0+rof2)*nucgamma*gss*pss[i];
    frewind1[i] = remainder(frewind1[i],1.0);
    frewind1[i] = resto+frewind1[i]/(pw_tref+OFFSET_DELAY);
    frewind2[i] = -(rof1+p2+rof2)*nucgamma*gss2*pss[i];
    frewind2[i] = remainder(frewind2[i],1.0);
    frewind2[i] = resto+frewind2[i]/(grate*agro+tep+tspoil+OFFSET_DELAY);
  }
  frewind3 = -(ACQUIRE_START_DELAY+at+ACQUIRE_STOP_DELAY+OFFSET_DELAY)
              *nucgamma*gro*pro;
  frewind3 = remainder(frewind3,1.0);
  frewind3 = resto+frewind3/(act_tpe+pw_risetpe+OFFSET_DELAY);
  
  /* Initialialize steady state counter *******************/
  initval(nsat,v15);

  prescan_flag=getval("prescan_flag");
  if(prescan_flag > 0.0){
    sgpe = 0.0;
  }

  /* Gradient safety calculation */
  if (fse_safety_calc() == -1) {
    sprintf(err_string,"%s: Failed duty cycle safety check\n",seqfil);
    printf(err_string);
    log_error(err_string);
    abort(1);
  }

  if (ns > 1) {
    create_offset_list(frewind1,ns,OBSch,0);
    create_offset_list(frewind2,ns,OBSch,1);
    position_offset_list(pss,gss,ns,resto,OBSch,2,v14);
    position_offset_list(pss,gss2,ns,resto,OBSch,3,v14);
  
  }
  
  if(rfmod[0]=='y'){
    calculate_rf_modulation();
  }

  /* PULSE SEQUENCE *************************************/

  /* Phase cycle: Part I ********************************/
  mod2(ct,v3);                    /*0101 0101*/
  dbl(v3,v3);                     /*0202 0202*/
  hlv(ct,v4);                     /*0011 2233*/
  mod2(v4,v4);                    /*0011 0011*/
  add(v3,v4,v1);                  /*0213 0213*/
  assign(v1,oph);

  /* Phase cycle: Part II *******************************/
  mod2(ct,v4);			/* 0101 0101 */
  ifzero(v4); 
    add(one,v1,v2);		/* 1*2* 1*2* */
  elsenz(v4);
    add(three,v1,v2);		/* *1*2 *1*2 */
  endif(v4);

  /* so final v1 and v2 are: */
  /* v1 : 0213 0213 */
  /* v2 : 1122 1122 */

  /* Relaxation delay ***********************************/       
  status(A);
  initval(prescan_flag > 0.0 ? 1 : phase_res/etl,v8);
  assign(zero,v10);
  initval(etl,v11);

  /* Turn on for Cardiac Gating */
  if (cgate[0] == 'y') sp3on();
      
  loop(v8,v9);  /* segments loop */
  xgate(ticks);
  msloop(seqcon[1],ns,v13,v14);  /* multi-slice loop */
  delay(predelay);
       
  /* Scope Trigger ****/
  if (scope[0] == 'y') {
    sp2on();
    delay(5e-6);
    sp2off();
  }

  /* Fat Saturation Pulse *******************************/
  if ((fat_sat[0] == 'y') || (fat_sat[0] == 'Y')) {
    
    if(rfmod[0]=='y'){
      getelem(t43,v14,v12);
      power(v12,OBSch);
      getelem(t60,v14,v12);
    } else {
      obspower(tpwrfat);
    }

    obsoffset(resto-3.5*sfrq);/* move the frequency to fat peak */
    loop(v15,v16);
    sp4on();
    if(rfmod[0]=='y'){
      shapedvpulse(pfatpat,pfat,v12,v1,rof1,rof2);
    } else {
      shapedpulse(pfatpat,pfat,v1,rof1,rof2);
    }
    sp4off();
    obl_shapedgradient("hsine","hsine","hsine",tfscrush,1.5,-1.5,1.5,
		       1,NOWAIT);
    delay(tfscrush);
    zero_all_gradients();
    delay(dsat);
    endloop(v16);
  }

  /* Inversion Pulse ***********************************/
  if (prep[0] == 'i') {
    if(rfmod[0]=='y'){
      getelem(t44,v14,v12);
      power(v12,OBSch);
      getelem(t60,v14,v12);
    } else obspower(tpwri);

    obl_shapedgradient("","","gssi",pw_gssi,0.0,0.0,gss2,1,NOWAIT);
    delay(grate*agss2 + rftep);
    position_offset_list(pss,gss2,ns,resto,OBSch,3,v14);
    sp4on();
    if(rfmod[0]=='y'){   
      shapedvpulse(pipat,pi,v12,v2,rof1,rof2);
    } else {
      shapedpulse(pipat,pi,v2,rof1,rof2);
    }

    sp4off();
    if(grate*agss2 > rftep) delay(grate*agss2 - rftep);
    obsoffset(resto);
    delay(tiact);
  }

  /* Oxford style saturation bands */
  if (presat==1) spatial_saturation_pulse();

  /* Saturation bands ***********************************/
  if (prep[0] == 's') {
    printf("%f\n",pss_sat1);
    printf("%f\n",pss_sat2);
    if(rfmod[0]=='y'){
      getelem(t45,zero,v12);
      power(v12,OBSch);
      getelem(t59,zero,v12);
    } else obspower(tpwrs);

    poffset(pss_sat1,gss_sat);
    obl_gradient(0.0,0.0,gss_sat);
    delay(grate*agss_sat);
    sp4on();
    if(rfmod[0]=='y'){
      shapedvpulse(pspat,ps,v12,v1,rof1,rof1);
    } else {
      shapedpulse(pspat,ps,v1,rof1,rof1);
    }
    sp4off();
    zero_all_gradients();
    delay(grate*agss_sat);
    obl_shapedgradient("hsine","hsine","hsine",0.01,
		       1.0,-1.0,1.0,1,NOWAIT);
    delay(0.01);

    if(rfmod[0]=='y'){
      getelem(t5,one,v12);
      power(v12,OBSch);
      getelem(t59,one,v12);
    } else obspower(tpwrs);

    poffset(pss_sat2,gss_sat);
    obl_gradient(0.0,0.0,gss_sat);
    delay(grate*agss_sat);
    sp4on();
    if(rfmod[0]=='y'){
      shapedvpulse(pspat,ps,v12,v1,rof1,rof1);
    } else {
      shapedpulse(pspat,ps,v1,rof1,rof1);
    }
    sp4off();
    zero_all_gradients();
    delay(grate*agss_sat);
    obl_shapedgradient("hsine","hsine","hsine",0.01,
		       1.0,-1.0,1.0,1,NOWAIT);
    delay(0.01);
  }

  /* 90 degree pulse ************************************/ 
  if(rfmod[0]=='y'){
    getelem(t41,v14,v12);
    power(v12,OBSch);
    getelem(t60,v14,v12);
  }
  else observepower(tpwr1);

  obl_shapedgradient("","","gss",pw_gss,0.0,0.0,gss,1,NOWAIT);
  delay(grate*agss+rftep);
  sp4on();
  position_offset_list(pss,gss,ns,resto,OBSch,2,v14);
					/* move frequency for slice select */
  if(rfmod[0]=='y') {
    shapedvpulse(p1pat,p1,v12,v1,rof1,rof2);
  } else {
    shaped_pulse(p1pat,p1,v1,rof1,rof2);
  }
  obsoffset(resto);			/* set frequency back to resto */
  sp4off();
  if(grate*agss>rftep)delay(grate*agss-rftep);

        /* Read & Slice refocus and phase encode pulse ********/
  obl_shapedgradient("gtref","","gtref",pw_tref,gror,0.0,gssr,1,NOWAIT);
  if (ns > 1) {
    voffset(0,v14);
  } else {
    offset(frewind1[0],OBSch);
  }
  delay(pw_tref);
  obsoffset(resto);
  
        /* First half-TE delay ********************************/
  delay(te1);

  peloop(seqcon[2],etl,v5,v6);
  if (prescan_flag == 1.0) {
    assign(zero,v7);
  } else if (table) {
    getelem(t1,v10,v7);
  } else {
    assign(v10,v7);
  }
  incr(v10);

  /* 180 degree pulse ***********************************/
  if(rfmod[0]=='y'){
    getelem(t42,v14,v12);
    power(v12,OBSch);
    getelem(t60,v14,v12);
  }
  else observepower(tpwr2);

  obl_shapedgradient("","","gss2",pw_gss2,0.0,0.0,agssc,1,NOWAIT);
  delay(crush_ramp_time + tcrush + crush_ramp_time + rftep);
  sp4on();
  position_offset_list(pss,gss2,ns,resto,OBSch,3,v14);
					/* move frequency for slice select */
  if(rfmod[0]=='y'){
    shapedvpulse(p2pat,p2,v12,v2,rof1,rof2);
  } else {
    shapedpulse(p2pat,p2,v2,rof1,rof2);
  }
  obsoffset(resto);
  sp4off();
  if((crush_ramp_time+tcrush+crush_ramp_time) > rftep)
    delay(crush_ramp_time+tcrush+crush_ramp_time - rftep);

  /* Phase-encode gradient ******************************/
  if (table) {
    pe2_shapedgradient("gdp_up",pw_risetpe,0.0,0.0,0.0,-sgpe,0.0,v7,zero);
    delay(act_tpe-pw_risetpe);
    pe2_shapedgradient("gdp_dn",pw_risetpe,0.0,0.0,0.0,-sgpe,0.0,v7,zero);
  } else {
    pe2_shapedgradient("gdp_up",pw_risetpe,0.0,0.5*sgpe*phase_res,0.0,
           -sgpe,0.0,v7,zero);
    delay(act_tpe-pw_risetpe);
    pe2_shapedgradient("gdp_dn",pw_risetpe,0.0,0.5*sgpe*phase_res,0.0,
           -sgpe,0.0,v7,zero);
  }

  /* Second half-TE period ******************************/
  delay(te2);

  /* Acquire echo ***************************************/
  obl_shapedgradient("gro","","",pw_gro,gro,0.0,0.0,1,NOWAIT);
/*
  if (ns > 1) {
    voffset(1,v14);
  }
  else
    offset(frewind2[0],OBSch);
*/
  delay(grate*agro+tep+tspoil);
  /* at this point hop away from frewind2 and move to pro frequency */

  poffset(pro,gro);
  acquire(np,1.0/sw);
  obsoffset(resto);

  if(tspoil+grate*agro>tep) delay(tspoil+grate*agro-tep);

  /* Rewinding phase-encode gradient ********************/
  offset(frewind3,OBSch);
  if (table) {
    pe2_shapedgradient("gdp_up",pw_risetpe,0.0,0.0,0.0,sgpe,0.0,v7,zero);
    delay(act_tpe-pw_risetpe);
    pe2_shapedgradient("gdp_dn",pw_risetpe,0.0,0.0,0.0,sgpe,0.0,v7,zero);
  } else {
    pe2_shapedgradient("gdp_up",pw_risetpe,0.0,-0.5*sgpe*phase_res,0.0,
           sgpe,0.0,v7,zero);
    offset(frewind3,OBSch);
    delay(act_tpe-pw_risetpe);
    obsoffset(resto);
    pe2_shapedgradient("gdp_dn",pw_risetpe,0.0,-0.5*sgpe*phase_res,0.0,
           sgpe,0.0,v7,zero);
  }
  obsoffset(resto);
  
  /* Second half-TE delay *******************************/
  delay(te3);

  endpeloop(seqcon[2],v6);
  sub(v10,v11,v10);
  endmsloop(seqcon[1],v14);
  add(two,v1,v1);
  assign(v1,oph);
  add(v10,v11,v10);

  obl_shapedgradient("","","spoil",pw_gspoil,0.0,0.0,gspoil,1,WAIT);

  zero_all_gradients(); /* double check that everything is zero */

  endloop(v9);

      /* Turn cardiac gating off */
  if (cgate[0] == 'y')
    sp3off();
}

/*****************************************************************************
Modification History

20sep96(ss): Sign of phase-encode gradient changed to get similar image 
	     orientation in F1 axis. Note: flash, fsems use compressed data
	     which are sorted by flashc and rarec.
29May97(ss): Inversion recovery pulse added.
14Jul97(ss): Timing error corrected in trefmin. floor() replaces rint().
28Feb01(SC): Added some shaped gradient commands along with tep/rftep.

*****************************************************************************/
