#ifndef LINT
static char SCCSid[] = "@(#)fsems.c 13.1 10/10/97 Copyright (c) 1991-1996 Varian Assoc.,Inc. All Rights Reserved";
#endif
/* 
 * Varian Assoc.,Inc. All Rights Reserved.
 * This software contains proprietary and confidential
 * information of Varian Assoc., Inc. and its contributors.
 * Use, disclosure and reproduction is prohibited without
 * prior consent.
 */
/***********************************************************************
    FSEMS.C

    Fast spin-echo imaging sequence with inversion recovery pulse.

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
     _                       __                                 __     _
    |                       |                                     |     |
RF  | ------ [90] ----------|-- [180] ------- |** ACQ **|---------|     |
    |                       |                                     |     |
    |          < - - - - - -|- -esp - - - - - - - >               |     |
    |        ____           |   _____                             |     |
GSS | ______/////\     _____|__//////\____________________________|     |
    | <---->      \\\\/ gssr|                                     |     |
    |   predelay  <--> tref |                                     | etl | ns
    |                       |                                     |     |
    |                       |            ___               ___    |     |
GPE |  _____________________|___________/   \_____________/   \___|     |
    |                       |           \___/             \___/   |     |
    |                       |                                     |     |
    |         gror ___      |                  _________          |     |
GRO |  ___________////\_____|_________________//////////\_________|     |
    |                       |                                     |     |
    |_                      |__                                 __|    _|

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

#include <standard.h>
#include <group.h>
#include "sequence_utilities.h"

codeint v15,v16;

/* INTERNAL VARIABLE DECLARATIONS *********************/
int     i,table=0;
double  esp,etl,predelay,seqtime,tref;
double  risetref,risetpe,trefmin,espmin,tpemin,tau1,tau2,tau3;
double  agro,agss,sgpe,grate,gssrint,grorint,maxgint;
double  gss2,agss2,agcrush;
double  nucgamma,atf,frewind[MAXSLICE],frewind2[MAXSLICE];
char    shape_name[MAXSTR];
char    prep[MAXSTR],scope[MAXSTR],pspat[MAXSTR],cgate[MAXSTR];
double  tiact,te1,te2,te3;
double  tsat,pss_sat1,pss_sat2,gss_sat,agss_sat;
double  nsat,dsat,tfscrush;
double  tpwrs,ps;
double  tep,rftep,pw_gss,pw_gss2,pw_gro;
int     prescan_flag;
char    gcoil[MAXSTR],err_string[MAXSTR];

#include "fse_safety_calc.h"

pulsesequence()
{
  
  initparms_sis();
  nucgamma = 1e6*sfrq/B0;
  if (strcmp(petable,"n")  &&  strcmp(petable,"N")  &&  strcmp(petable,"")) {
    loadtable(petable);
    table = 1;
  }
  getstr("gcoil",gcoil);
  etl = getval("etl");
  esp = getval("esp");
  atf = getval("atf");
  gss2 = getval("gss2");
  pss_sat1 = getval("pss_sat1");
  pss_sat2 = getval("pss_sat2");
  gss_sat = getval("gss_sat");
  tpwrs = getval("tpwrs");
  ps = getval("ps");
  nsat = getval("nsat");
  dsat = getval("dsat");
  tfscrush = getval("tfscrush");
  grate = trise/gmax;
  agro = fabs(gro);
  agss = fabs(gss);
  agcrush = fabs(gcrush);
  getstr("prep",prep);
  getstr("scope",scope);
  getstr("pspat",pspat);
  getstr("cgate",cgate);
  agss2 = fabs(gss2);
  agss_sat = fabs(gss_sat);

  P_getreal(GLOBAL,"tep",&tep,1); /* small tweaker time (us) to allow for
				     the gradient propagation time,
				     causes echoes to appear symmetrically
				     about echo center line */
  tep=tep/1e6;
  P_getreal(GLOBAL,"rftep",&rftep,1); /* small tweaker time (us) to allow for
					 the gradient propagation time,
					 w.r.t. RF pulses */
  rftep=rftep/1e6;
  
  
  if (grof == 0.0  ||  !var_active("grof",1) )
    grof = 1.0;
  if (gssf == 0.0  ||  !var_active("gssf",1) )
    gssf = 1.0;
  
  sprintf(shape_name,"gss");
  pw_gss=create_trap_gradient(agss,shape_name,rof1+p1+rof2,grate,ix);

  sprintf(shape_name,"gss2");
  pw_gss2=create_spin_echo_gradient(shape_name,gss2,rof1+p2+rof2,gspoil,
				    tcrush,tcrush,grate,ix);
  sprintf(shape_name,"gro");
  pw_gro=create_trap_gradient(agro,shape_name,at+2.0*tspoil,grate,ix); 

  /****
   * Calculate minimum time for phase-encoding, slice refocussing
   * and readout prefocussing block.
   ****/
  /* GRADIENT*TIME INTEGRALS FOR SLICE SELECTION AND READOUT */
  grorint = agro*(at/2.0 + tspoil + grate*agro/2.0);
  gssrint = agss*(p1/2.0 + rof1 + agss*grate/2.0);
  
  /* MINIMUM REFOCUSSING BLOCK TIME.  TRAPEZOIDAL IF GMAX IS EXCEEDED */
  /* NOTE, TREFMIN DOES NOT INCLUDE FALL TIME */
  maxgint = grorint > gssrint ? grorint : gssrint;
  if (trise*gmax >= maxgint) {
    trefmin = sqrt(grate*maxgint);  /* triangular pulse */
    risetref = trefmin;
  }
  else {
    trefmin = grate*gmax;           /* trapezoidal pulse */
    risetref = trise;
  }
  
  /* Make an assumption about risetpe */
  risetpe = gpe*nv*grate;
  
  /* TAU1, TAU2 AND TAU3 ARE THE SUMS OF ALL EVENTS IN EACH HALF-ECHO PERIOD */
  
  tau1 = (p1 + p2)/2.0 + 2.0*rof1 + trefmin + 
    grate*(agss+agcrush+fabs(gcrush-gss2)) + tcrush + risetref;
  if(grate*gss<rftep) tau1 += (rftep-grate*gss);

  tau2 = (p2 + at)/2.0 + rof1 + tpe +
    grate*(fabs(gcrush-gss2)+agro) + risetpe + tcrush;
  if((grate*fabs(gcrush-gss2)+tcrush+grate*agcrush)<rftep)
    tau2 += (rftep-(grate*fabs(gcrush-gss2)+tcrush+grate*agcrush));

  tau3 = (at + p2)/2.0 + tpe + risetpe
    + grate*(agcrush+fabs(gcrush-gss2)) + tcrush + rof1;
  if(grate*agro<tep) tau3 += (tep-grate*agro);


  /*
  if (ix == 1)
    fprintf(stderr,"tau1 = %7.6f  tau2 = %7.6f  tau3 = %7.6f\n",tau1,tau2,tau3);
  */

  espmin = tau1 > tau2 ? tau1 : tau2;
  espmin = tau3 < espmin ? 2.0*espmin : 2.0*tau3;
  if (esp < espmin) {
    printf("FSEMS: esp too small.  Minimum esp = %f\n",espmin);
    abort(1);
  }
  
  /* TREF IS THE REFOCUSSING BLOCK LENGTH, NOT INCLUDING GRADIENT FALL TIME */
  /* TREF IS SCALED UP TO A MAXIMUM VALUE CONSISTENT WITH tau1 */
  tref = trefmin + 0.8*(esp/2.0 - tau1);
  tref = (tref > at/2.0  &&  trefmin <= at/2.0) ? at/2.0 : tref;
  
  /* CALCULATE GPE, ADJUSTING FOR INTEGRAL DAC VALUE, THEN ADJUST TPE */
  tpemin = tpe;
  tpe = tpemin + 0.4*(esp/2.0 - tau2);
  tpe = tpe > 4.0*tpemin ? 4.0*tpemin : tpe;
  sgpe = gpe*tpemin/tpe;
  /* This assumes sgpe is always positive */
  sgpe = gmax/gradstepsz*floor(sgpe*gradstepsz/gmax + 0.5);
  if (fabs(gpe) > 0.0)
    tpe = fabs(gpe*tpemin/sgpe);
  risetpe = grate*fabs(sgpe*nv/2.0);
  risetpe = (nv == 0) ? tpe : risetpe;
  
  /* CALCULATE PHASE REFOCUSSING TO CORRECT FOR FREQUENCY JUMPS */
  /* atf is a tweaker used to correct for frequency hop delays */
  for (i=0; i<ns; i++) {
    frewind[i] = (atf + at + OFFSET_DELAY)*nucgamma*(gro*pro - gss2*pss[i]);
    frewind[i] = -remainder(frewind[i],1.0);
    frewind[i] = resto + nucgamma*gss2*pss[i] 
      + frewind[i]/(atf + risetpe + OFFSET_DELAY);
    frewind2[i] = (atf + at + OFFSET_DELAY)*nucgamma*(gro*pro - gss2*pss[i]);
    frewind2[i] = -remainder(frewind2[i],1.0);
    frewind2[i] = resto + nucgamma*gss2*pss[i] 
      + frewind2[i]/(atf + risetpe + OFFSET_DELAY);
  }
  
  /* CALCULATE GSSR AND GROR WITH NEW TREF */
  gror =  grof*grorint/tref*gro/fabs(gro);
  gssr = -gssf*gssrint/tref*gss/fabs(gss);
  
  risetref = fabs(gssr) > fabs(gror) ? grate*fabs(gssr) : grate*fabs(gror);
  
  /* First half-TE delay *******************************/
  te1 = esp/2.0 - (p1+p2)/2.0 - 2.0*rof1 - tref - risetref - tcrush
    - grate*(agss+agcrush+fabs(gcrush-gss2));
  if(grate*gss<rftep) te1 -= (rftep-grate*gss);

    /* Second half-TE delay *******************************/
  te2 = esp/2.0 - (p2+at)/2.0 - rof1 - tpe - risetpe - tcrush
    - grate*(fabs(gcrush-gss2)+agcrush+agro)-tspoil;    
  if((grate*fabs(gcrush-gss2)+tcrush+grate*agcrush)<rftep)
    te2 -= (rftep-(grate*fabs(gcrush-gss2)+tcrush+grate*agcrush));

    /* Post acq second half-TE delay ********/
  te3 = esp/2.0 - (at+p2)/2.0 - rof1 - tpe - risetpe - tcrush - tspoil
    - grate*(fabs(gcrush-gss2)+agcrush+agro);
  if(grate*agro<tep) te3 -= (tep-grate*agro);

    /* inversion delay */
  tiact = ti - 0.5*(pi + p1) - grate*(agss+agss2) - (2*rof1);
  if (tiact < 0.0) tiact=0.0;
  
    /* Saturation band time **/
  tsat = 4.0*(rof1+grate*agss_sat) + 2.0*(ps + 0.01);

  /* Relaxation delay */
  seqtime = grate*agss + rof1 + p1/2.0 + etl*esp + at/2.0 + tpe + risetpe + te3;

  if (prep[0] == 'f')
    seqtime += nsat*(p3 + tfscrush + dsat);

  /*
    if (prep[0] == 'i') {
    if (tr - (grate*agss2 + rof1 + ti + ns*seqtime)) < 0.0) {
    printf("FSEMS: Requested tr too short.  Min tr = %f\n",ti+ns*seqtime);
    abort(1);
    }
    predelay = (tr - (tiact + ns*seqtime))/ns;
  */
    
  if ( (tr - ns*seqtime) < 0.0) {
    printf("FSEMS: Requested tr too short.  Min tr = %f\n",ns*seqtime);
    abort(1);
  }
  predelay = (tr - ns*seqtime)/ns;

  if (prep[0] == 's')
    seqtime = seqtime + tsat;

  /* Initialialize steady state counter *******************/
  initval(nsat,v15);

  prescan_flag=getval("prescan_flag");
  if(prescan_flag){
    sgpe = 0.0;
  }

  /* Gradient safety calculation */
  if (fse_safety_calc() == -1) {
    sprintf(err_string,"%s: Failed duty cycle safety check\n",seqfil);
    printf(err_string);
    log_error(err_string);
    abort(1);
  }

  /* PULSE SEQUENCE *************************************/

  /* Phase cycle: Part I ********************************/
  mod2(ct,v3);                    /*0101 0101*/
  dbl(v3,v3);                     /*0202 0202*/
  hlv(ct,v4);                     /*0011 2233*/
  mod2(v4,v4);                    /*0011 0011*/
  add(v3,v4,v1);                  /*0213 0213*/
  assign(v1,oph);


  /* Phase cycle: Part II *******************************/
  mod2(ct,v4);			/* 0101 0101 */
  ifzero(v4); 
    add(one,v1,v2);		/* 1 2  1 2  */
  elsenz(v4);
    add(three,v1,v2);		/*  1 2  1 2 */
  endif(v4);

  /* so final v1 and v2 are: */
  /* v1 : 0213 0213 */
  /* v2 : 1122 1122 */

  /* Relaxation delay ***********************************/       
  status(A);
  initval(nv == 0 ? 1 : nv/etl,v8);
  assign(zero,v10);
  initval(etl,v11);

  /* Turn on for Cardiac Gating */
  if (cgate[0] == 'y') sp3on();
      
  loop(v8,v9);
  xgate(ticks);
  msloop(seqcon[1],ns,v13,v14);
  delay(predelay);
       
  /* Scope Trigger ****/
  if (scope[0] == 'y') {
    sp2on();
    delay(5e-6);
    sp2off();
  }

  /* Fat Saturation Pulse *******************************/
  if (prep[0] == 'f') {
    obspower(tpwr3);
    obsoffset(resto-3.5*sfrq);/* move the frequency to fat peak */
    loop(v15,v16);
    sp4on();
    shapedpulse(p3pat,p3,v1,rof1,rof2);
    sp4off();
    obl_shapedgradient("hsine","hsine","hsine",tfscrush,1.5,-1.5,1.5,
		       1,NOWAIT);
    delay(tfscrush);
    zero_all_gradients();
    delay(dsat);
    endloop(v16);
  }

  /* Inversion Pulse ***********************************/
  if (prep[0] == 'i') {
    obsoffset(resto);
    obspower(tpwr2);
    obl_gradient(0.0,0.0,gss2);
    delay(grate*agss2);
    sp4on();
    shapedpulse(p2pat,p2,v1,rof1,rof1);
    sp4off();
    zero_all_gradients();
    delay(tiact);
  }

  /* Saturation bands ***********************************/
  if (prep[0] == 's') {
    printf("%f\n",pss_sat1);
    printf("%f\n",pss_sat2);
    obspower(tpwrs);
    poffset(pss_sat1,gss_sat);
    obl_gradient(0.0,0.0,gss_sat);
    delay(grate*agss_sat);
    sp4on();
    shapedpulse(pspat,ps,v1,rof1,rof1);
    sp4off();
    zero_all_gradients();
    delay(grate*agss_sat);
    obl_shapedgradient("hsine","hsine","hsine",0.01,
		       1.0,-1.0,1.0,1,NOWAIT);
    delay(0.01);

    poffset(pss_sat2,gss_sat);
    obl_gradient(0.0,0.0,gss_sat);
    delay(grate*agss_sat);
    sp4on();
    shapedpulse(pspat,ps,v1,rof1,rof1);
    sp4off();
    zero_all_gradients();
    delay(grate*agss_sat);
    obl_shapedgradient("hsine","hsine","hsine",0.01,
		       1.0,-1.0,1.0,1,NOWAIT);
    delay(0.01);
  }

  /* 90 degree pulse ************************************/ 
  observepower(tpwr1);
  position_offset_list(pss,gss,ns,resto,OBSch,0,v14);

  obl_shapedgradient("","","gss",pw_gss,0.0,0.0,gss,1,NOWAIT);
  delay(grate*agss+rftep);
  sp4on();
  shapedpulse(p1pat,p1,v1,rof1,rof2);
  sp4off();
  if(grate*agss>rftep)delay(grate*agss-rftep);

        /* Read & Slice refocus and phase encode pulse ********/
  obl_gradient(gror,0.0,gssr);
  delay(tref);
  zero_all_gradients();
  delay(risetref);
  
        /* First half-TE delay ********************************/
  delay(te1);

  peloop(seqcon[2],etl,v5,v6);
  if (nv == 0)
    assign(zero,v7);
  else if (table)
    getelem(t1,v10,v7);
  else
    assign(v10,v7);
  incr(v10);

  /* 180 degree pulse ***********************************/
  position_offset_list(pss,gss2,ns,resto,OBSch,1,v14);
  observepower(tpwr2);

  obl_shapedgradient("","","gss2",pw_gss2,0.0,0.0,gss2,1,NOWAIT);
  delay(grate*agcrush + tcrush + grate*fabs(gcrush-gss2) + rftep);
  sp4on();
  shapedpulse(p2pat,p2,v2,rof1,rof1);
  sp4off();
  if((grate*fabs(gcrush-gss2)+tcrush + grate*agcrush)>rftep)
    delay(grate*fabs(gcrush-gss2) +  tcrush + grate*agcrush - rftep);

  /* Phase-encode gradient ******************************/
  if (table)
    pe_gradient(0.0,0.0,0.0,-sgpe,v7);
  else
    pe_gradient(0.0,0.5*sgpe*nv,0.0,-sgpe,v7);
  delay(tpe);
  zero_all_gradients();
  delay(risetpe);

  /* Second half-TE period ******************************/
  delay(te2);

  /* Acquire echo ***************************************/
  obl_shapedgradient("gro","","",pw_gro,gro,0.0,0.0,1,NOWAIT);
  delay(grate*agro+tep+tspoil);

  poffset(pro,gro);
  acquire(np,1.0/sw);
  position_offset_list(pss,gss2,ns,resto,OBSch,1,v14);

  if(tspoil+grate*agro>tep)delay(tspoil+grate*agro-tep);

  /* Rewinding phase-encode gradient ********************/
  if (table)
    pe_gradient(0.0,0.0,0.0,sgpe,v7);
  else
    pe_gradient(0.0,-0.5*sgpe*nv,0.0,+sgpe,v7);
  delay(tpe);
  zero_all_gradients();

  if (ns > 1) {
    create_offset_list(frewind2,ns,OBSch,2);
    voffset(2,v14);
  }
  else
    offset(frewind2[0],OBSch);
  
  delay(risetpe);
  position_offset_list(pss,gss2,ns,resto,OBSch,1,v14);
 

  /* Second half-TE delay *******************************/
  delay(te3);

  endpeloop(seqcon[2],v6);
  sub(v10,v11,v10);
  endmsloop(seqcon[1],v14);
  add(two,v1,v1);
  assign(v1,oph);
  add(v10,v11,v10);
  endloop(v9);

      /* Turn cardiac gating off */
  if (cgate[0] == 'y')
    sp3off();
}

/*****************************************************************************
Modification History

20sep96(ss): Sign of phase-encode gradient changed to get similar image 
	     orientation in F1 axis. Note: flash, fsems use compressed data
	     which are sorted by flashc and rarec.
29May97(ss): Inversion recovery pulse added.
14Jul97(ss): Timing error corrected in trefmin. floor() replaces rint().
28Feb01(SC): Added some shaped gradient commands along with tep/rftep.

*****************************************************************************/
