#ifndef LINT
static char SCCSid[] = "@(#)fieldmap.c 1.0 07/28/97 Copyright (c) 1997 P. Jezzard, NIH. All Rights Reserved";
#endif
/***********************************************************************
    FIELDMAP.C

    Double echo gradient-echo fast imaging sequence.                     

    VERSION 1.0        7/28/97  
    Revisions:
	based on FLASH.C sequence, but with rfspoiling fixed
	and 2nd echo added.

    seqcon should be set to 'ccsnn'

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
      __  __                                                      __   __
     |   |                                                          |    |
  RF |   |----- [ p1 ] ------------- | ACQ1 | ------| ACQn | -------|    |
     |   |                                    _            _        |    |
     |   |         < - - - - te - - - - >    |              | tspoil|    |
     |   |       ____                        |              | _____ |    |
  GSS|   |______/////\     __________________|______________|/     \|    |
     |   |<---->      \\\\/ gssr             |              |       |    |
     |   |predelay    <--> tref              |<------------>|       | ns | nv
     |   |             ___                   |   delta_te   | _____ |    |
  GPE|   |____________/___\__________________|______________|/     \|    |
     |   |            \___/                  |              |       |    |
     |   |                 < - tA - >  AQ1   |        AQn   |       |    |
     |   |                            ______ |       ______ | _____ |    |
  GRO|   |____________     __________///////\|      ///////\|/     \|    |
     |   |       gror \\\\/                  |\\\\\/        |       |    |
     |   |                                   | \\\/         |       |    |
     |   |< - - - - - - - - - - - - - - - - -|- - - - - - - |- - - >|    |
     |__ |__                     tr/ns       |_            _|     __|  __|
                                                            ne-1

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

#include <standard.h>
#include "sequence_utilities.h"
#define OFFSETDELAY 26e-6
#define RFSPOILDELAY 50e-6

pulsesequence()
{
    /* Internal variable declarations *********************/
    double  predelay,seqtime,tref,tref2,risetref,risetref2,del_seq;
    double  trefmin,tref2min,tssrmin,trormin,tpemin,temin,tA;
    double  agro,agss,agror2,sgpe,agspoil,grate,gssrint,grorint,gror2;
    double  gror2int,gpeint,gspoilint,extra_echo_time,delta_te,te2;
    double  reffreq,nextphase,spoilfreqs[1024],pss[512],pssfreqs[512];
    int     i,num_pss_els;
    char grad_spoil[MAXSTR],err_string[MAXSTR];

    initparms_sis();
    loop_check();

    del_seq = getval("del_seq"); /* short delay at start of each overall AQ */
    delta_te = getval("delta_te");  /* echo spacing if multi echo */
    te2 = getval("te2");  /* 2nd echo time if two echoes */
    getstr("grad_spoil",grad_spoil); /* gradient spoiler flag 'y' or 'n' */
    num_pss_els=getarray("pss",pss);  /* grab the pss array */

    if (num_pss_els != (int)ns) {
        sprintf(err_string,"%s: ns and pss array size incompatible\n",seqfil);
        printf(err_string);
        log_error(err_string);
        abort(1);
    }

    if ((ne == 2) && (te2 > te)) {
      delta_te=te2-te;
    }

    if (fabs(gspoil) > 0.75*gmax)
    {
      gspoil=gspoil/fabs(gspoil)*0.75*gmax;
      sprintf(err_string,"%s: resetting gspoil to %f\n",seqfil,gspoil);
      printf(err_string);
      log_error(err_string);
    }

    grate = trise/gmax;
    agro = fabs(gro);
    agss = fabs(gss);
    if (agss == 0.0) {
        sprintf(err_string,"%s: Zero slice select gradient\n",seqfil);
        printf(err_string);
        log_error(err_string);
        abort(1);
    }
    agspoil=fabs(gspoil);

    /* GRADIENT*TIME integrals for slice selection and readout */
    grorint = agro*(at/2.0 + grate*agro/2.0);
    gror2int = agro*(at + grate*agro);
    gssrint = agss*(p1/2.0 + rof2 + grate*agss/2.0);
    gpeint = B0/(lpe*sfrq*1e6);
    gspoilint = agspoil*(tspoil + grate*agspoil/2.0);

    /*******************************************************
    * Calculate minimum time for phase-encoding, slice refocussing
    * and readout prefocussing block.  Trapezoidal if GMAX is exceeded.
    * TSSRMIN does not include the gradient fall time.  RISETREF is
    * the gradient fall time for the PE block.
    *******************************************************/
    if (trise*gmax >= gssrint)
	tssrmin = sqrt(grate*gssrint);
    else
	tssrmin = gssrint/gmax;

    if (trise*gmax >= grorint)
        trormin = sqrt(grate*grorint);
    else
	trormin = grorint/gmax;

    if (trise*gmax >= gpeint*nv/2.0)
        tpemin = sqrt(grate*gpeint*nv/2.0);
    else
	tpemin = gpeint*nv/2.0/gmax;

    /* Minimum refocussing block time is the longest of calculated minima */
    trefmin = (tssrmin > trormin) ? tssrmin : trormin;
    trefmin = (trefmin > tpemin) ? trefmin : tpemin;

    /*******************************************************
    * If TREFMIN implies trapezoidal PE block, mimimum RISETREF
    * is RISETIME, otherwise minimum RISETREF is TREFMIN.
    *******************************************************/
    risetref = (trefmin > trise) ? trise : trefmin;

    /* Check minimum TE ***********************************/
    temin = p1/2.0 + rof2 + grate*agss + trefmin + risetref +
            grate*agro + at/2.0;
    if (te < temin) {
        sprintf(err_string,"%s: TE too short. Min TE = %f\n",seqfil,temin);
        printf(err_string);
        log_error(err_string);
	abort(1);
    }

    /*******************************************************
    * TREF is the refocussing block length, not including gradient fall
    * time.  TREF is scaled up to a maximum value consistent with TE
    *******************************************************/
    /* this part adds some time on to trefmin to put a little less stress on
       the gradients, but doesn't let tref get any longer than 2.0*trefmin */
    tref = trefmin + 0.5*(te - temin);
    tref = (tref > 2.0*trefmin) ? 2.0*trefmin : tref;

    /* Calculate GPE, adjusting for integral dac value, then adjust TREF */
    sgpe = gpeint/tref;
    sgpe = gmax/gradstepsz*floor(sgpe*gradstepsz/gmax + 0.5);
    tref = gpeint/sgpe;

    /* Calculate GSSR and GROR with new TREF **************/
    gror = -grorint/tref*gro/fabs(gro);
    gssr = -gssrint/tref*gss/fabs(gss);

    /* Determine PE block gradient fall time with new TREF */
    risetref = fabs(gssr) > fabs(gror) ? fabs(gssr) : fabs(gror);
    risetref = risetref > fabs(sgpe*nv/2.0) ? risetref : fabs(sgpe*nv/2.0);
    risetref *= grate;  /* to turn it back in to a time */

    /* determine the values needed for the echo train refocuser */
    if (trise*gmax >= gror2int)
	tref2min = sqrt(grate*gror2int);
    else
	tref2min = gror2int/gmax;

    /*******************************************************
    * If TREF2MIN implies trapezoidal PE block, mimimum RISETREF2
    * is RISETIME, otherwise minimum RISETREF2 is TREF2MIN.
    *******************************************************/
    risetref2 = (tref2min > trise) ? trise : tref2min;

    extra_echo_time=0.0; /* default */
    if (ne > 1)
    {
      /* extra_echo_time is the minimum possible for time being */
      extra_echo_time = tref2min + risetref2 + grate*agro + at + grate*agro;
      if (delta_te < extra_echo_time)
      {
        printf("%s: delta_te too short.  Minimum delta_te = %f\n",
                      seqfil,extra_echo_time);
        abort(1);
      }
    }
    
    /* this part adds some time on to tref2min to put a little less stress on
       the gradients, but doesn't let tref2 get any longer than 2.0*tref2min */
    tref2 = tref2min + 0.5*(delta_te-extra_echo_time);
    tref2 = (tref2 > 2.0*tref2min) ? 2.0*tref2min : tref2;

    /* now calculate the required value of gror2 */
    gror2 = -gror2int/tref2;
    agror2=fabs(gror2);

    extra_echo_time = delta_te - (tref2 + grate*agror2 + grate*agro + 
                      at + grate*agro);
    if (extra_echo_time < 0.0)
    {
      sprintf(err_string,"%s: delta_te too short. Min delta_te = %f\n",
                    seqfil,delta_te-extra_echo_time);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }

    /* tA is the delay between refocussing and readout ****/
    tA = te - (p1/2.0 + rof2 + grate*agss + tref + risetref +
       + grate*agro + at/2.0);

    /* Relaxation delay ***********************************/
    seqtime = grate*agss + p1/2.0 + rof1 + te + at/2.0 + grate*agro
	    + (double)(ne-1)*(delta_te) + 0.0001;  
                           /* extra 100us for good measure */

    if (rfspoil[0] == 'y'  ||  rfspoil[0] == 'Y')
	seqtime += RFSPOILDELAY;

    if (grad_spoil[0] == 'y'  ||  grad_spoil[0] == 'Y')
	seqtime += (tspoil + grate*agspoil);

    if (tr < seqtime*ns) {
        sprintf(err_string,"%s: TR too short. Min TR = %f\n",seqfil,seqtime*ns);
        printf(err_string);
        log_error(err_string);
        abort(1);
    }
    predelay = (tr - seqtime*ns)/ns;

    /* calculate the pss frequency list from the pss array */
    /* this has to be done because of conflicts with the spoilfreqs
       list which must also be generated */
    reffreq = resto;
    for (i=0; i<= (int)(ns + 0.5); i++) {
      pssfreqs[i] = reffreq + sfrq*1e6/B0*gss*pss[i];
    }
    create_offset_list(pssfreqs,ns,TODEV,0);

    /*******************************************************
    * Calculation of frequencies to get phase shifts for optional
    * RF spoiling.  Phase shifts are obtained by hopping to variable
    * frequency for fixed time.
    * Ref:  Zur, Y., Magn. Res. Med., 21, 251, (1991).
    *******************************************************/
    if ((rfspoil[0] == 'y' || rfspoil[0] == 'Y') && nv > 0) {
      reffreq = resto + sfrq*1e6/B0*gro*pro;
      spoilfreqs[0]=0.0;  /* initialize 1st element in list as 0.0 */
      for (i=1; i<= (int)(nv+0.5); i++) {
        nextphase = (double)i/2.0*((double)i-1.0)*rfphase;
        nextphase = nextphase - 360.0*floor(nextphase/360.0);
        spoilfreqs[i] = reffreq + nextphase/RFSPOILDELAY/360.0;
      }
      /* use list 1 since list 0 is used to hold pss_freqs */
      create_offset_list(spoilfreqs,nv,TODEV,1);
    }


    /* PULSE SEQUENCE *************************************/

    /* Phase cycle ****************************************/
    mod2(ct,v3);          /* v3: 0  1  0  1  0  1  0  1 */
    dbl(v3,v3);           /* v3: 0  2  0  2  0  2  0  2 */
    hlv(ct,v4);           /* v4: 0  0  1  1  2  2  3  3 */
    mod2(v4,v4);          /* v4: 0  0  1  1  0  0  1  1 */
    add(v3,v4,v1);        /* v1: 0  2  1  3  0  2  1  3 */
    assign(v1,oph);      /* oph: 0  2  1  3  0  2  1  3 */

    initval(ne-1,v8);    /* initialize counter v8 to the value of ne-1 */

    /* ifzero(v6); */
    if (ix == 1) delay(del_seq);  /* delay at start of sequence */
    /* endif(v6); */

    status(A);
    sp4off();
    obspower(tpwr1);

    peloop(seqcon[2],nv,v5,v6);

      msloop(seqcon[1],ns,v13,v14); /* sets v13 to ns and incs v14 up to it */

        /* Relaxation delay ***********************************/       
        voffset(0,v14); /* voffset selects the freq offset from pssfreqs list */
        delay(predelay);
       
        /* RF pulse *******************************************/ 
        obl_gradient(0.0,0.0,gss);
        delay(grate*agss);
        sp4on();
        shapedpulse(p1pat,p1,v1,rof1,rof1);
        sp4off();
        zero_all_gradients();
        delay(grate*agss);

        /* Read & Slice refocus and phase encode pulse ********/
        pe_gradient(gror,sgpe*nv/2.0,gssr,-sgpe,v6);
        delay(tref);
        zero_all_gradients();
        delay(risetref);

        /* Acquire echo ***************************************/
        /* place poffset before the TE filler so that PTS phase shifts
           are coherent in case the sequence is used for field mapping */
        poffset(pro,gro);

        /* Pre-acquire delay **********************************/
        delay(tA);

        obl_gradient(gro,0.0,0.0);
        delay(grate*agro);
        acquire(np,1.0/sw);
        zero_all_gradients();
        delay(grate*agro);

        if (ne > 1)
        {
          loop(v8,v9);  /* loop v8 (ne-1) times using counter v9 */
            obl_gradient(gror2,0.0,0.0);
            delay(tref2);
            zero_all_gradients();
            delay(grate*agror2);

            delay(extra_echo_time);  /* filler to bring up to delta_te */

            obl_gradient(gro,0.0,0.0);
            delay(grate*agro);
            acquire(np,1.0/sw);
            zero_all_gradients();
            delay(grate*agro);
          endloop(v9);
        }

        if (grad_spoil[0] == 'y'  ||  grad_spoil[0] == 'Y')
        {
          obl_gradient(gspoil,gspoil,gspoil);
          delay(tspoil);
          zero_all_gradients();
          delay(grate*agspoil);
        }

        /*******************************************************
        * Optional RF spoiling.  REMINDER: Do not place any events
        * after this block or before voffset at the start of msloop.
        *******************************************************/
        if ((rfspoil[0] == 'y' || rfspoil[0] == 'Y') && nv > 0) {
          voffset(1,v6);  /* voffset selects the freq offset from spoifreqs */
          delay(RFSPOILDELAY - OFFSETDELAY);
        }

      endmsloop(seqcon[1],v14); /* increment the msloop counter v14 */ 

    endpeloop(seqcon[2],v6);
}
