/***********************************************************************
    GRAD_TEST.C

    gradient test sequence.
    Peter Jezzard, NIH July 1997

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

#include "standard.h"
#include "group.h"
#include "sequence_utilities.h"
extern double rint();

pulsesequence()
{
    /* INTERNAL VARIABLE DECLARATIONS *********************/
    int     image=0;
    double  nucgamma, grad_del, pre_acq_del, seqtime, postdelay;
    double  gread, agread, gphase, agphase, gslice, agslice, grise;
    double  pw_ramp, max_grad, grate;
    double  create_ramp_gradient();
    char shape_name[MAXSTR], err_string[MAXSTR];

    initparms_sis();

    /*************************************************************************/
    /* get the global parameters */
    /*************************************************************************/
    P_getreal(GLOBAL,"grise",&grise,1); /* grad risetime to full strength */
    if ((grise < 0.0) || (grise > 0.002)) {
      sprintf(err_string,"%s: impropper or no grise defined in global file\n",
                       seqfil);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
    /*************************************************************************/

    grad_del = getval("grad_del"); /* duration of gradient pulse */
    pre_acq_del = getval("pre_acq_del"); /* pre acquisition delay */
    gread = getval("gread"); /* gradient strength in read direction */
    gphase = getval("gphase"); /* gradient strength in phase direction */
    gslice = getval("gslice"); /* gradient strength in slice direction */
    image = (int)(getval("image")); /* useful counter */

    grate = grise/gmax; /* switching rate of gradients [sec/(gauss/cm)] */

    nucgamma = 1e6*sfrq/B0;  /* nucleus gamma in Hz/gauss */
                             /* derived from sfrq in MHz and B0 in gauss */

    agread = fabs(gread); /* absolute read gradient strength */
    agphase = fabs(gphase); /* absolute phase encode gradient strength */
    agslice = fabs(gslice); /* absolute slice select gradient strength */

    max_grad = fabs(gread) > fabs(gphase) ? fabs(gread) : fabs(gphase);
    max_grad = max_grad > fabs(gslice) ? max_grad : fabs(gslice);

    sprintf(shape_name,"gramp");
    pw_ramp=create_ramp_gradient(max_grad,shape_name,1,grate,ix);
    pw_ramp=create_ramp_gradient(max_grad,shape_name,-1,grate,ix);

  /* CALCULATE RELAXATION DELAY FOR SEQUENCE */
    seqtime = (pw + rof1 + rof2 + 4*pw_ramp + 2*grad_del + at + pre_acq_del);
    if (tr - seqtime < 0.0) {
          sprintf(err_string,"%s: TR too short. Min TR = %f\n",seqfil,seqtime);
          printf(err_string);
          log_error(err_string);
          abort(1);
    }
    postdelay = (tr - seqtime);

    status(A);
    sp4off();
    rcvroff();
    rcvron();

  peloop(seqcon[2],nv,v5,v6);

    /* 90 degree pulse ************************************/ 
    sp1on();
    sp4on();
    pulse(pw,oph);  /* implicitly uses tpwr for power level */
    sp4off();
    sp1off();

    if ((fabs(gread) > 0.0) || (fabs(gphase) > 0.0) || (fabs(gslice) > 0.0)) {
      /* first dephase the signal using a shaped gradient pulse */
      obl_shapedgradient("gramp_up","gramp_up","gramp_up",pw_ramp,
          gread,gphase,gslice,1,WAIT);
      delay(grad_del);
      obl_shapedgradient("gramp_dn","gramp_dn","gramp_dn",pw_ramp,
          gread,gphase,gslice,1,WAIT);

      /* now put gradients in opposite direction to refocus the signal */
      obl_shapedgradient("gramp_up","gramp_up","gramp_up",pw_ramp,
          -gread,-gphase,-gslice,1,WAIT);
      delay(grad_del);
      obl_shapedgradient("gramp_dn","gramp_dn","gramp_dn",pw_ramp,
          -gread,-gphase,-gslice,1,WAIT);

      zero_all_gradients();  /* double check that all gradients are off */
    }

    delay(pre_acq_del);
    acquire(np, 1.0/sw);
    delay(postdelay);

  endpeloop(seqcon[2],v6);

}
