#ifndef LINT
static char SCCSid[] = "@(#)flash.c 11.1 05/07/96 Copyright (c) 1994-1996 Varian Assoc.,Inc. All Rights Reserved";
#endif
/***********************************************************************
    KSPACE.C

    K space mapping sequence.                     

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
        __                                                  __ 
       |                                                      |
  RF   |----- [ p1 ] ----------------------- |** ACQ **| -----|
       |                                                      |
       |         < - - - - - - - te - - - - - - - >           |
       |       ____                                           |
  GSS  |______/////\     _____________________________________|
       |<---->      \\\\/ gssr                                |
       |  predelay  <--> tref                                 | nv
       |                            ____________________      |
  GRO  |___________________________/////////////////////\_____|
       |                                shaped pulse          |
       |                                                      |
       |< - - - - - - - - - - - tr - - - - - - - - - - - - - >|
       |__                                                  __|

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

#include "standard.h"
#include "math.h"
#include "group.h"
#include "sequence_utilities.h"

/*==========================================================================*
  define the pulse sequence variables outside the pulse sequence so that
  they can be seen as global variables by the safety monitoring code
*==========================================================================*/
    double  predelay,seqtime,tref,risetref,grise,tep,rftep,at_des;
    double  trefmin,tssrmin,trormin,tpemin,temin,tA;
    double  agro,agss,sgpe,grate,gssrint,grorint,gpeint;      
    double  reffreq,nextphase,spoilfreqs[1024],pw_gss_d,extra_te_error;
    double  pw_gro,pw_gss,wave_time,true_wave_time,greadfc,gslicefc;
    int     i,image=0,dummy_ssc;
    char    shape_name[MAXSTR],flow_comp[MAXSTR],err_string[MAXSTR];
    char    gcoil[MAXSTR];

#include "spgr_safety_calc.h"

pulsesequence()
{

    double rgrad, pgrad;

    initparms_sis();
    loop_check();

    /*************************************************************************/
    /* get the global parameters */
    /*************************************************************************/
    P_getreal(GLOBAL,"grise",&grise,1); /* grad risetime to full strength */
    if ((grise < 0.0) || (grise > 0.002)) {
      sprintf(err_string,"%s: impropper grise time defined in global file\n",
                       seqfil);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
    P_getreal(GLOBAL,"tep",&tep,1); /* small tweaker time (us) to allow for
                            the gradient propagation time, causes echoes to
                            appear symmetrically about echo center line */
    tep=tep/1e6;
    P_getreal(GLOBAL,"rftep",&rftep,1); /* small tweaker time (us) to allow for
                            the gradient propagation time, w.r.t. RF pulses */
    rftep=rftep/1e6;
    /*************************************************************************/

    getstr("gcoil",gcoil);          /* gradient coil in use */
    image = (int)(getval("image"));
    /* 1 = read direction control pulse */
    /* 2 = phase direction control pulse */
    /* 3 = rgrad.GRD, duration d1, strength rgrad along read */
    /* 4 = pgrad.GRD, duration d1, strength pgrad along phase */
    rgrad = getval("rgrad");
    pgrad = getval("pgrad");

    grate = grise/gmax;

    agss = fabs(gss);
    if (agss == 0.0) {
        sprintf(err_string,"%s: Zero slice select gradient\n",seqfil);
        printf(err_string);
        log_error(err_string);
        abort(1);
    }

    sprintf(shape_name,"gss");
    pw_gss=create_trap_gradient(fabs(gss),shape_name,rof1+p1+rof2,grate,ix);

    tA=3.0e-3; /* default value */

    /* account for case of rftep being longer than the gradient ramp time */
    pw_gss_d = agss*grate - rftep;
    if (pw_gss_d < 0) pw_gss_d=0.0;

    /* PULSE SEQUENCE *************************************/

    /* Phase cycle ****************************************/
    mod2(ct,v3);          /* v3: 0  1  0  1  0  1  0  1 */
    dbl(v3,v3);           /* v3: 0  2  0  2  0  2  0  2 */
    hlv(ct,v4);           /* v4: 0  0  1  1  2  2  3  3 */
    mod2(v4,v4);          /* v4: 0  0  1  1  0  0  1  1 */
    add(v3,v4,v1);        /* v1: 0  2  1  3  0  2  1  3 */
    assign(v1,oph);      /* oph: 0  2  1  3  0  2  1  3 */

    assign(one,v7);  /* phase encode increment size */
    status(A);
    sp4off();
    obspower(tpwr1);

    sp3on();
    delay(0.00005);  /* 50 us trigger to indicate start of acq */
    sp3off();

    delay(tr);

    /* Relaxation delay ***********************************/       
    poffset(pss[0],gss);

    /* RF pulse *******************************************/ 
    sp1on(); /* put out trigger on sp1 every spin excitation */
    if ((image == 1) || (image == 3)) {
      obl_shapedgradient("gss","","",pw_gss,gss,0.0,0.0,1,NOWAIT);
    } else {
      obl_shapedgradient("","gss","",pw_gss,0.0,gss,0.0,1,NOWAIT);
    }

    delay(agss*grate+rftep);
    sp4on();
    shapedpulse(p1pat,p1,v1,rof1,rof1);
    sp4off();
    delay(pw_gss_d);
    sp1off();

    if ((image == 1) || (image == 3)) {
      obl_shapedgradient("gss","","",pw_gss,-gss/2,0.0,0.0,1,WAIT);
    } else {
      obl_shapedgradient("","gss","",pw_gss,0.0,-gss/2,0.0,1,WAIT);
    }

    obsoffset(resto);
    delay(tA);

    if (image == 3) {
      obl_shapedgradient("rgrad","","",d1,rgrad,0.0,0.0,1,NOWAIT);
    }
    if (image == 4) {
      obl_shapedgradient("","pgrad","",d1,0.0,pgrad,0.0,1,NOWAIT);
    }
    acquire(np,1.0/sw);

    zero_all_gradients(); /* double check that all gradients are zero */

}
