; ==========================================================================
; Spiral Reconstruction: Tools For B0-correction
; ==========================================================================
; written in IDL by Markus Weiger, FMRIB Centre, University OF Oxford, 2001


; -------------- reading data --------------


FUNCTION Read_b0scan, name, res, ns
   ; se-fieldmap xvnmr/vnmr
   
   logstring, '----- read_B0scan -----', 4              
   
   data = rd_varian_bin(name, ns)
   s = SIZE(data)
   n = s(1)/ns
   nte = 2
   logstring, 'n, ns, nte = '+trimstr(n)+', '+trimstr(ns)+', '+trimstr(nte), 4
   pss = get_varian_par(name, 'pss')
   sindex = SORT(pss)
   IF res EQ 0 THEN res = n
   image = COMPLEXARR(res,res,ns,nte)
   temp = COMPLEXARR(n,n)
   data = REFORM(data, n, ns, nte, n)
   FOR ite = 0, nte-1 DO FOR is = 0, ns-1 DO BEGIN
      FOR in = 0, n-1 DO temp(*,in) = data(*, sindex(is), ite, in)
      temp1 = zerofill(temp, res)
      ; swap phase-frequency = no, does it matter ?
      image(*,*,is,ite) = TRANSPOSE(REVERSE(swap(fft(swap(temp1(*,*)),1))))
   END
   RETURN, image
END


; ---------- map processing -------------


FUNCTION Lysis, A, steps
; removes iteratively 'lonely' pixels
   
   s = SIZE(A)
   m = s(1)+2
   n = s(2)+2
   dum = INTARR(m, n)
   dum(1:m-2, 1:n-2) = A GT 0
   neigh = INTARR(m-2, n-2)
   FOR k = 0, steps-1+2*(steps EQ -1) DO BEGIN
      neigh(*, *) = 0
      FOR i = -1, 1 DO FOR j = -1, 1 DO IF (ABS(j)+ABS(i) NE 0) THEN BEGIN
         neigh = neigh + dum(1+i:m-2+i, 1+j:n-2+j)
      END
      temp = dum
      dum(1:m-2, 1:n-2) = dum(1:m-2, 1:n-2) AND (neigh GE 4)
      IF steps EQ -1 THEN k = (MAX(temp-dum) EQ 0)-1
   END
   RETURN, A*dum(1:m-2, 1:n-2)
END

FUNCTION Create_mask, image, threshold
   image = image/MAX(ABS(image))
   mask = ABS(image)^2 GT (threshold^2)
   mask = lysis(mask, -1)
   RETURN, mask
END

FUNCTION Process_map, map, image, weight, width, extrapol
   
   mult = weight*ABS(image/MAX(ABS(image)))^2
   
   ; define region for extrapolation
   s = SIZE(map)
   nx_ref = s(1)
   ny_ref = s(2)
   to_grow = INTARR(nx_ref, ny_ref)
   to_grow(*, *) = 0
   neigh = INTARR(nx_ref, ny_ref)
   dum_weight = INTARR(nx_ref+2, ny_ref+2)
   dum_weight(1:nx_ref, 1:ny_ref) = weight
   IF (extrapol GT 0) THEN FOR steps = 0, extrapol-1 DO BEGIN
      neigh(*, *) = 0
      FOR x = -1, 1 DO FOR y = -1, 1 DO neigh(*, *) = neigh(*, *) + dum_weight(1+x:nx_ref+x, 1+y:ny_ref+y)
      wo = ((neigh GE 2) AND (dum_weight(1:nx_ref, 1:ny_ref) EQ 0))
      to_grow = to_grow+wo
      dum_weight(1:nx_ref, 1:ny_ref) = dum_weight(1:nx_ref, 1:ny_ref)+wo
   END

   ; smooth and extrapolate
   
   IF width GT 0 THEN BEGIN
      
      to_do = INTARR(nx_ref, ny_ref)
      need = INTARR(nx_ref, ny_ref)
      fuss = width-(width MOD 1)
      sum_f = FLTARR(nx_ref+2, ny_ref+2)
      sum_g = FLTARR(nx_ref, ny_ref)

      ; inner region
      
      temp = INTARR(nx_ref+2*fuss, ny_ref+2*fuss)
      FOR i = 0, 2*fuss DO FOR j = 0, 2*fuss DO temp(i:i+nx_ref-1, j:j+ny_ref-1) = temp(i:i+nx_ref-1, j:j+ny_ref-1) + weight 
      to_do = temp(fuss:fuss+nx_ref-1, fuss:fuss+ny_ref-1) GE (2*fuss+1)^2
      temp(*, *) = 0
      FOR i = 0, 2*fuss DO FOR j = 0, 2*fuss DO temp(i:i+nx_ref-1, j:j+ny_ref-1) = temp(i:i+nx_ref-1, j:j+ny_ref-1) + to_do  
      need = temp(fuss:fuss+nx_ref-1, fuss:fuss+ny_ref-1) GT 0 
      
      i_arr = (FINDGEN(2*fuss+1, 2*fuss+1) MOD (2*fuss+1))-fuss           
      j_arr = TRANSPOSE(i_arr)
      gauss_mask = EXP(-(i_arr^2+j_arr^2)/(0.5*width)^2)
      x_raw = fuss-(FINDGEN(2*fuss+1, 2*fuss+1) MOD (2*fuss+1))
      y_raw = TRANSPOSE(x_raw)
      FOR i = 0, nx_ref-1 DO BEGIN
         FOR j = 0, ny_ref-1 DO IF need(i, j) THEN BEGIN
            x0 = MAX([0, fuss-i])
            x1 = MIN([fuss-i+nx_ref-1, 2*fuss])
            y0 = MAX([0, fuss-j])
            y1 = MIN([fuss-j+ny_ref-1, 2*fuss])
            g = gauss_mask(x0:x1, y0:y1)*mult(i, j)   
            x = x_raw(x0:x1, y0:y1)				
            y = y_raw(x0:x1, y0:y1)
            i0 = i-fuss+x0
            i1 = i-fuss+x1
            j0 = j-fuss+y0
            j1 = j-fuss+y1
            sum_g(i0:i1, j0:j1) = sum_g(i0:i1, j0:j1) + g   
            sum_f(i0+1:i1+1, j0+1:j1+1) = sum_f(i0+1:i1+1, j0+1:j1+1)+map(i, j)*g
         END
      END
      FOR i = 0, nx_ref-1 DO FOR j = 0, ny_ref-1 DO $
        IF to_do(i, j) THEN map(i, j) = sum_f(i+1, j+1)/sum_g(i, j)
      kern = to_do

      ; border region
      
      sum_f(*, *) = 0
      sum_g(*, *) = 0
      need(*, *) = 0
      fuss2 = 0
      to_do = (weight OR to_grow)-kern
      temp_to_do = to_do

      REPEAT BEGIN
         fuss2 = fuss2+1
         temp = INTARR(nx_ref+2*fuss2+1, ny_ref+2*fuss2+1)
         kernel = INTARR(2*fuss2+1, 2*fuss2+1)
         kernel(*, *) = 1
         temp(fuss2+1:nx_ref+fuss2, fuss2+1:ny_ref+fuss2) = weight
         temp = convol(temp, kernel)
         ok = temp_to_do AND (temp(fuss2+1:nx_ref+fuss2, fuss2+1:ny_ref+fuss2) GE ((fuss+1)^2))
         temp_to_do = temp_to_do - ok
         temp(*, *) = 0
         temp(fuss2+1:nx_ref+fuss2, fuss2+1:ny_ref+fuss2) = ok
         temp = convol(temp, kernel)
         ok = weight AND (temp(fuss2+1:nx_ref+fuss2, fuss2+1:ny_ref+fuss2) GT 0)
         need = need*(1-ok)
         need = need+fuss2*ok
      END UNTIL MAX(temp_to_do) EQ 0       
      
      i_arr = (FINDGEN(2*fuss2+1, 2*fuss2+1) MOD (2*fuss2+1))-fuss2           
      j_arr = TRANSPOSE(i_arr)
      gauss_mask = EXP(-(i_arr^2+j_arr^2)/(1.0*width)^2)
      x_raw = fuss2-(FINDGEN(2*fuss2+1, 2*fuss2+1) MOD (2*fuss2+1))
      y_raw = TRANSPOSE(x_raw)
      FOR i = 0, nx_ref-1 DO BEGIN
         FOR j = 0, ny_ref-1 DO IF (need(i, j) GT 0) THEN BEGIN
            x0 = MAX([fuss2-need(i, j), fuss2-i])
            x1 = MIN([fuss2-i+nx_ref-1, fuss2+need(i, j)])
            y0 = MAX([fuss2-need(i, j), fuss2-j])
            y1 = MIN([fuss2-j+ny_ref-1, fuss2+need(i, j)])
            g = gauss_mask(x0:x1, y0:y1)*mult(i, j)   
            x = x_raw(x0:x1, y0:y1)				
            y = y_raw(x0:x1, y0:y1)
            i0 = i-fuss2+x0
            i1 = i-fuss2+x1
            j0 = j-fuss2+y0
            j1 = j-fuss2+y1
            sum_g(i0:i1, j0:j1) = sum_g(i0:i1, j0:j1) + g   
            sum_f(i0+1:i1+1, j0+1:j1+1) = sum_f(i0+1:i1+1, j0+1:j1+1)+map(i, j)*g
         END
      END
      FOR i = 0, nx_ref-1 DO FOR j = 0, ny_ref-1 DO $
        IF to_do(i, j) THEN map(i, j) = sum_f(i+1, j+1)/sum_g(i, j)
   END
   ;weight = weight OR to_do
   ;wmean = TOTAL(map)/TOTAL(weight)
   ;map = map+wmean*(NOT weight)
   
   RETURN, map
END

FUNCTION Phasemap, fname, n, datatype
; read data with different echo times
; get phase image and frequency map
; remove background noise, smooth, and extrapolate
; n = returned matrix size

   type         = 0             ; 0 = difference first, 1 = unwrap first, probably identical
   threshold    = 0.1           ; fraction of magn. amplitude for cutting noise
   res          = 64            ; resolution while generating map 
   width        = 3             ; width of gaussian kernel for smoothing [pixel]
   extrapol     = 4             ; # steps of extrapolation [pixel]
   
   
   logstring, '----- phasemap -----', 2              
   
   IF datatype EQ 0 THEN BEGIN
      ; phase map from se-sequence
      images = Read_b0scan(fname, res, ns)
   END

   IF datatype EQ 1 THEN BEGIN
      ; spiral phase map
      images = read_avw(fname,0)
      images = reduce_fourier(images,res)
      s = SIZE(images)
      IF s(0) GT 2 THEN ns = s(3) ELSE ns = 1
   END

   phmaps = FLTARR(res,res,ns,2)
   phmap = FLTARR(res,res,ns)
   masks = FLTARR(res,res,ns)
   imag = FLTARR(res,res,ns)

   FOR is = 0, ns-1 DO BEGIN

      ; get phase images and unwrap
      i1 = REFORM(images(*,*,is,0))
      i2 = REFORM(images(*,*,is,1))
      iav = (i1+i2)/2.0
      
      ph1 = phase(i1)
      ph2 = phase(i2)
      dph = unwrap(ph2-ph1)
     
      ; --------- processing -------------

      ; get rid of background noise
      mask = create_mask(iav,threshold)
      dph = mask*dph
      IF TOTAL(mask) NE 0 THEN dph = dph + (dph EQ 0)*(TOTAL(dph*mask,/nan)/TOTAL(mask))
      phmaps(*,*,is,0) = ph1
      phmaps(*,*,is,1) = ph2
      phmap(*,*,is) = dph
      masks(*,*,is) = mask      
      imag(*,*,is) = iav      
   END

   temppath = '~/'
   phmapname = temppath+'phmap'
   masksname = temppath+'masks'
   unwrapname = temppath+'unwrap'
   IF type EQ 0 THEN write_avw, phmap, phmapname ELSE write_avw, phmaps, phmapname
   write_avw, masks, masksname
   unwrap_call = 'prelude -p '+phmapname+' -a '+masksname+' -m '+masksname+' -u '+unwrapname
   logstring, unwrap_call, 4
   SPAWN, unwrap_call
   phmaps = read_avw(unwrapname, 0)
   delete_avw, phmapname
   delete_avw, masksname
   delete_avw, unwrapname

   FOR is = 0, ns-1 DO BEGIN
      IF type EQ 0 THEN dph = REFORM(phmaps(*,*,is)) ELSE BEGIN
         ph1 = REFORM(phmaps(*,*,is,0))
         ph2 = REFORM(phmaps(*,*,is,1))
         dph = ph2-ph1
      END
      mask = REFORM(masks(*,*,is))
      iav = REFORM(imag(*,*,is))
      dph = Process_map(dph, iav, mask, width, extrapol)
      phmap(*,*,is) = dph
   END
   ipos = FINDGEN(n)/FLOAT(n)*FLOAT(res)
   IF ns GT 1 THEN BEGIN
      spos = FINDGEN(ns)
      result = interpolate(phmap,ipos,ipos,spos,/grid)
   END ELSE result = interpolate(phmap,ipos,ipos,/grid)
   RETURN, result
END



; ---------- Chebyshev Interpolation for CPR ----------


FUNCTION Chebyshev, n, a, b
; chebyshev approximation for sin and cos
; see "Numerical Recipes"

   ; calculate chebyshev coefficients
   csin = DBLARR(n)
   ccos = DBLARR(n)
   csin(*) = 0
   ccos(*) = 0

   bma = 0.5*(b-a)
   bpa = 0.5*(b+a)
   
   FOR j=0, n-1 DO BEGIN
      FOR k=0, n-1 DO BEGIN
         arg = !pi*(k+0.5)/DOUBLE(n)
         y = COS(arg)
         csin(j) = csin(j)+SIN(y*bma+bpa)*COS(j*arg)
         ccos(j) = ccos(j)+COS(y*bma+bpa)*COS(j*arg)
      END
   END
   csin = 2.0/FLOAT(n)*csin
   ccos = 2.0/FLOAT(n)*ccos

   ; transfer chebyshev to polynomial coefficients
   dsin = DBLARR(n)
   dcos = DBLARR(n)
   ddsin = DBLARR(n)
   ddcos = DBLARR(n)
   dsin(*) = 0
   dcos(*) = 0
   ddsin(*) = 0
   ddcos(*) = 0
   
   dsin(0) = csin(n-1)
   dcos(0) = ccos(n-1)
   FOR j=n-2, 1, -1 DO BEGIN
      FOR k=n-j, 1, -1 DO BEGIN
         sv = dsin(k)
         dsin(k) = 2*dsin(k-1)-ddsin(k)
         ddsin(k) = sv
         sv = dcos(k)
         dcos(k) = 2*dcos(k-1)-ddcos(k)
         ddcos(k) = sv
      END
      sv = dsin(0)
      dsin(0) = -ddsin(0) + csin(j)
      ddsin(0) = sv
      sv = dcos(0)
      dcos(0) = -ddcos(0) + ccos(j)
      ddcos(0) = sv
   END
   FOR j=n-1, 1, -1 DO BEGIN
      dsin(j) = dsin(j-1) - ddsin(j)
      dcos(j) = dcos(j-1) - ddcos(j)
   END
   dsin(0) = -ddsin(0) + 0.5*csin(0)
   dcos(0) = -ddcos(0) + 0.5*ccos(0)

   ; coefficient shift
   cnst = 2.0/(b-a)
   fac = cnst
   FOR j=1, n-1 DO BEGIN
      dsin(j) = dsin(j)*fac
      dcos(j) = dcos(j)*fac
      fac = fac*cnst
   END
   cnst = 0.5*(a+b)
   FOR j=0, n-2 DO BEGIN
      FOR k=n-2, j, -1 DO BEGIN
         dsin(k) = dsin(k)-cnst*dsin(k+1)
         dcos(k) = dcos(k)-cnst*dcos(k+1)
      END
   END	
   RETURN, dcomplex(dcos, -dsin)
END
