/***********************************************************************
    MP_BEPI_NAV.C

    Blipped interleaved echo-planar imaging sequence.
    Complete with nav echoes.

    VERSION 3.01	04/01/93	ARR
    Last modified	20/03/97	JSG
			04/07/97	JSG
			22/09/97	JSG,RSM,BG
                        01/04/98        PJ (Oxford) corrected tref and tref2
                                        calculation.
                        01/04/98        PJ added shaped gradient waveforms and
                                        grise variable.

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
     _                  __                                 __          _
    |                  |                                     |          |
RF  |-[alpha]----------|---------------- |** ACQ **|---------|----      |
    |                  |                                     |          |
    |     < - - - - - -|- -esp - - - - - - - >               |          |
    |   ____           |                                     |          |
GSS |__/////\     _____|_____________________________________|____      |
    |        \\\\/ gssr|                                     |          |
    |        <--> tref |                                     |     etl  | ns
    |                  |                                     |          |
    |                  |                                     |          |
GPE |__________________|______________/\_____________________|____      |
    |                  |              \/                     |          |
    |                  |                                     |<----->   |
    |    gror          |                  _________          | post_del |
GRO |________     _____|_________________//////////\_________|____      |
    |        \\\\/     |                                     |          |
    |                  |__                                 __|         _|

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

#include "standard.h"
#include "group.h"
#include "sequence_utilities.h"
extern double rint();

pulsesequence()
{
    extern codeint v15,v16;
    /* INTERNAL VARIABLE DECLARATIONS *********************/
    int     table=0,image=0,num_segs;
    double  esp,etl,post_del,seqtime,tref;
    double  risetref,tau1,tau2,grise,tep,epigroa,episwing,epigrof;
    double  agro,agss,agcrush,grate,gssrint,grorint,gpeint;
    double  risetref2,tref2,etl_delay,pw_gss_d,extra_te_error;
    double  dseg,minte,eff_echo,tbpe,sgbpe;
    double  tbrefmin,delay90_180,delay180_echo;
    double  pw_gro,pw_gror,pw_gss,pw_gss2,pw_gssr,pw_gcrush;
    double  create_ramp_gradient(), create_first_epiblip_gradient();
    double  half_gmax,half_grise,scope_trig_time;
    double  create_epiblip_gradient();
    int     prescan_flag;
    char    contrast[MAXSTR],rtm_flag[MAXSTR],intlv[MAXSTR];
    char    spinecho[MAXSTR],scope[MAXSTR],shape_name[MAXSTR];
    char    err_string[MAXSTR];

    char str[ MAXSTR ];
    char numStr1[ MAXSTR ];
    char numStr2[ MAXSTR ];
    int idx, num1, num2, endflag;
    char *ptr;

    initparms_sis();

    /*************************************************************************/
    /* get the global parameters */
    /*************************************************************************/
    P_getreal(GLOBAL,"grise",&grise,1); /* grad risetime to full strength */
    if ((grise < 0.0) || (grise > 0.002)) {
      sprintf(err_string,"%s: impropper grise time defined in global file\n",
                       seqfil);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
    P_getreal(GLOBAL,"tep",&tep,1); /* small tweaker time (us) to allow for
                            the gradient propagation time, causes echoes to
                            appear symmetrically about echo center line */
    tep=tep/1e6;
    epigroa = 0.0; /* default */
    P_getreal(GLOBAL,"epigroa",&epigroa,1); /* read dephase tweaker (percent) */
    epigrof = 0.0; /* default */
    P_getreal(GLOBAL,"epigrof",&epigrof,1); /* grad risetime to full strength */
    episwing = 0.0; /* default */
    P_getreal(GLOBAL,"episwing",&episwing,1); /* allows extra time to be
                                  inserted into the length of each flat top */
    episwing=episwing /1.0e6;  /* convert from us */
    /*************************************************************************/

    esp = getval("esp");
    gss2 = getval("gss2");
    dseg = getval("dseg");
    grate = grise/gmax;
    prescan_flag=getval("prescan_flag");
    agro = fabs(gro);
    agss = fabs(gss);
    if (agss == 0.0) {
        sprintf(err_string,"%s: Zero slice select gradient\n",seqfil);
        printf(err_string);
        log_error(err_string);
        abort(1);
    }
    agcrush = fabs(gcrush);
    getstr("contrast",contrast);
    getstr("rtm_flag",rtm_flag);
    getstr("intlv",intlv);
    getstr("scope",scope);
    getstr("spinecho",spinecho);
    gcrush=gmax/2.0;
    scope_trig_time=50.0e-6; /* scope trigger time on spare line 2 */
    image = (int)(getval("image")); /* trigger and counter bit pattern */
    /* bit 1 = wait for ticks triggers before beginning scan */
    /* bit 2 = puts out high on sp3 for duration of volume */
    /* bit 3 = epi blips off (set bit on), otherwise blips on */

    half_gmax=gmax/2.0;
    half_grise=grise/2.0;

    /* Decide on number of pe steps per segment or image ****/
    /* by parsing the table string **************************/
    /* table name must be of form epi(#pesteps)([alt]/[lin])(#shots)k***/

    if (strcmp(petable,"n")  &&  strcmp(petable,"N")  &&  strcmp(petable,"")) {
      loadtable(petable);
      table = 1;
    }

    ptr = petable;
    *numStr1 = '\0';
    *numStr2 = '\0';
    *str = '\0';
 
    endflag = 0;
    num2 = 1;
    num1 = 1;
 
     while( ! isdigit( *ptr ) )
             ptr++;

     idx = 0;       
     while( (*ptr != '\0') && isdigit( *ptr ) )
     {
             numStr1[idx] = *ptr;
             numStr1[idx + 1] = '\0';
             idx++;
             ptr++;
     }

     idx = 0;
     while((*ptr != '\0') && (! isdigit( *ptr )))
     {
             str[idx] = *ptr;
             str[idx + 1] = '\0';
             idx++;
             ptr++;
     }
 
     idx = 0;
     while((*ptr != '\0') && isdigit( *ptr ))
     {
             numStr2[idx] = *ptr;
             numStr2[idx + 1] = '\0';
             idx++;
             ptr++;
             endflag = 1;
     }
          
    num1 = atoi( numStr1 );
    num2 = atoi( numStr2 );
    if (num2 == 0) num2 = 1;

    if (endflag)
      etl = (double)num1/num2;
    else
      etl = (double)num1;

    if (ix == 1) {
       printf("# of PE steps per seg: %f \n", etl);
       printf("# of segments (shots): %d \n", num2);
       printf("Phase encode type: %s \n", str);
    }

   /* if alternating, first echo will be TE
   otherwise linear phase encoding steps, middle of echo trains */

    if (str[0] == 'a') eff_echo = 1.0;
    else eff_echo = floor((double)etl/2.0);

    if (nv - num1 != 0.0) {
          sprintf(err_string,"%s: Invalid petable name or number of pe steps\n",
                         seqfil);
          printf(err_string);
          log_error(err_string);
          abort;
    }

    if (gssf == 0.0  ||  !var_active("gssf",CURRENT) )
      gssf = 0.0;

    /* create blip_read.GRD and blip_phase.GRD */

    sprintf(shape_name,"gro");
    pw_gro=create_ramp_gradient(agro,shape_name,-1,grate,ix);

    sprintf(shape_name,"fstbp");
    pw_gro=create_first_epiblip_gradient(agro,shape_name,grate,ix);

    sprintf(shape_name,"blip");
    pw_gro=create_epiblip_gradient(agro,shape_name,grate,1.0/sw,
          (int)(np/2.0),ix);
    if (pw_gro == -1.0) {
      sprintf(err_string,"%s: Error in create_epiblip_gradient routine\n",
                      seqfil);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }

    sprintf(shape_name,"gss");
    pw_gss=create_trap_gradient(agss,shape_name,rof1+p1+rof2,grate,ix);

    sprintf(shape_name,"gss2");
    pw_gss2=create_trap_gradient(fabs(gss2),shape_name,  
                   rof1+p2+rof2*2.0*tcrush,grate,ix);

  /****
  * Calculate minimum time for phase-encoding, slice refocussing
  * and readout prefocussing block.

  ****/
  /* GRADIENT*TIME INTEGRALS FOR SLICE SELECTION, PHASE AND READOUT */
    grorint = (1.0+epigrof/100.0)*agro*(at/2.0 + grate*agro/2.0);
    gssrint = (1.0+gssf/100.0)*agss*(p1/2.0 + rof2 + agss*grate/2.0);
    gpeint = B0 / (lpe*sfrq*1e6);

  /* MINIMUM SLICE REFOCUSSING BLOCK TIME.  TRAP IF 0.5*GMAX IS EXCEEDED */
    if (grise*half_gmax >= gssrint) {
	tref = sqrt(grate*gssrint);
	risetref = tref;
    }
    else {
	tref = gssrint/half_gmax;
	risetref = half_grise;
    }

  /* MINIMUM REFOCUSSING BLOCK TIME FOR READOUT REWINDER */
    if (grise*gmax >= grorint) {
	tref2 = sqrt(grate*grorint);
	risetref2 = tref2;
    }
    else {
	tref2 = grorint/gmax;
	risetref2 = grise;
    }

    tbrefmin = pw_gro;
    tbpe = tbrefmin;
    sgbpe = gpeint/tbpe;
    sgbpe = gmax/gradstepsz*rint(sgbpe*gradstepsz/gmax);


  /* CALCULATE GSSR AND GROR WITH NEW TREF's */
    gror = -grorint/tref2;
    gssr = -gssrint/tref;

    sprintf(shape_name,"gror");
    pw_gror=create_ramp_gradient(fabs(gror),shape_name,1,grate,ix);
    pw_gror=create_ramp_gradient(fabs(gror),shape_name,-1,grate,ix);
    if (tref2 < pw_gror) tref2=pw_gror;  /* in case odd us difference */

    sprintf(shape_name,"gssr");
    pw_gssr=create_ramp_gradient(fabs(gssr),shape_name,1,grate,ix);
    pw_gssr=create_ramp_gradient(fabs(gssr),shape_name,-1,grate,ix);
    risetref = pw_gssr;
    if (tref < risetref) tref=risetref;  /* in case odd us difference */

    pw_gcrush=0.0;
    if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
      sprintf(shape_name,"gcr");
      pw_gcrush=create_ramp_gradient(fabs(gcrush),shape_name,1,grate,ix);
      pw_gcrush=create_ramp_gradient(fabs(gcrush),shape_name,-1,grate,ix);
      if (pw_gcrush > tcrush) {
        sprintf(err_string,"%s: tcrush too small. Min tcrush = %f\n",
                           seqfil,pw_gcrush);
        printf(err_string);
        log_error(err_string);
        abort(1);
      }
    }

  /* CALCULATE TAU1 - MINIMUM TIME BETWEEN ALPHA AND FIRST ECHO (NAV)
     & CALCULATE TAU2 - MINIMUM TIME BETWEEN ECHOES */

    /* account for case of tep being longer than the gradient ramp time */
    pw_gss_d = agss*grate - tep;
    extra_te_error = 0.0;
    if (pw_gss_d < 0.0) {
      pw_gss_d = 0.0;
      extra_te_error = tep-agss*grate;
    }

    if (agss*grate+tcrush < tep) {
        sprintf(err_string,"%s: tcrush too short. Min tcrush = %f\n",
                         seqfil, tep-agss*grate);
        printf(err_string);
        log_error(err_string);
        abort(1);
    }

    tau1 = p1/2.0 + rof2 + grate*agss + extra_te_error + tref + risetref 
       + grate*fabs(gror) + tref2 + risetref2 + tbrefmin + grate*agro + at/2.0;
    tau2 = at + 2.0*grate*agro; 

  /* CALCULATE MINIMUM TE AND DELAY TO HAVE FLEXIBLE TE */
    minte = tau1 + eff_echo * tau2;
    etl_delay = te - minte;

    if (te < minte) {
        sprintf(err_string,"%s: TE too short. Min TE = %f\n",seqfil,minte);
        printf(err_string);
        log_error(err_string);
	abort(1);
    }

  /* CALCULATE RELAXATION DELAY FOR BOTH INT and NON-INT CASES */
    if (intlv[0] == 'y') {
      seqtime = ( grate*agss + rof1 + p1/2.0 + te + 
		  (etl-eff_echo-1) * tau2 + at/2.0 + grate*agro +
		  tcrush + 2.0*grate*agcrush ) * ns + scope_trig_time;
    }
    else {
      seqtime = grate*agss + rof1 + p1/2.0 + te + 
		  (etl-eff_echo-1) * tau2 + at/2.0 + grate*agro +
		  tcrush + 2.0*grate*agcrush + scope_trig_time/ns;
    }

    if (tr - seqtime < 0.0) {
        sprintf(err_string,"%s: TR too short. Min TR = %f\n",seqfil,seqtime);
        printf(err_string);
        log_error(err_string);
        abort(1);
    }

    /* Timing for spin echo delays *********************/
    delay90_180 = 0.0;
    delay180_echo = 0.0;
    if (spinecho[0] == 'y') {
      delay90_180 = te/2.0 - (p1/2.0+rof2+2.0*grate*agss+tref+risetref+
                                grate*agss + tcrush + p2/2);
      delay180_echo = te/2.0 - (p2/2+tcrush+grate*agss+tref2+grate*agro
                                +2.0*tbrefmin+at/2.0+tau2);
      if ((delay90_180 < 0.0) || (delay180_echo < 0.0)) {
          if (delay90_180 < delay180_echo) {
            sprintf(err_string,"%s: TE too short. Min TE = %f\n",seqfil,
                te-delay90_180*2.0);
            printf(err_string);
            log_error(err_string);
            abort(1);
          } else {
            sprintf(err_string,"%s: TE too short. Min TE = %f\n",seqfil,
                te-delay180_echo*2.0);
            printf(err_string);
            log_error(err_string);
            abort(1);
          }
      }
    }

    if (prescan_flag == 1) {
      num_segs=1;
    } else {
      num_segs=(int)(nv/etl);
    }

    if (intlv[0] == 'y')
      post_del = (tr - seqtime)/ns;
    else
      post_del = (tr - seqtime);

    /*** Set all phase encodes to zero when image bit 3 set ***/
    if ((image & 4) == 4)
    {
      sgbpe = 0.0;
    }

  /* PULSE SEQUENCE *************************************/
    /* Phase cycle: Part I ********************************/
    mod2(ct,v3);                    /*0101 0101*/
    dbl(v3,v3);                     /*0202 0202*/
    hlv(ct,v4);                     /*0011 2233*/
    mod2(v4,v4);                    /*0011 0011*/
    add(v3,v4,v1);                  /*0213 0213*/
    assign(v1,oph);

    /* Phase cycle: Part II *******************************/
    mod2(ct,v4);
    ifzero(v4); 
      add(one,v1,v2);
    elsenz(v4);
      add(three,v1,v2);
    endif(v4);

    status(A);
    sp4off();

    /* RTM Phase cycling stuff coordinated with the blips off var ***/
    if (((image & 4) != 4) && (rtm_flag[0] == 'y'))
      assign(zero,v1);

    initval(num_segs,v8);
    initval(0,v15);
    initval(etl,v11);

    /******************************************************/
    /**** PULSE SEQUENCE FOR INTERLEAVED SLICE EPI ********/
    /******************************************************/

    if (intlv[0] == 'y')
    {
      assign(v15,v10);

      if ((image & 1) == 1) xgate(ticks);  /* wait for start scan button */
      if ((image & 2) == 2) sp3on(); else sp3off();

      sp2on();
      delay(scope_trig_time);  /* trigger to indicate start of volume acq */
      sp2off();

      loop(v8,v9);

        msloop(seqcon[1],ns,v13,v14);
          poffset_list(pss,gss,ns,v14);

          /* 90 degree pulse ************************************/ 
          observepower(tpwr1);
          sp1on();  /* put out trigger on sp1 every spin excitation */
          obl_shapedgradient("","","gss",pw_gss,0.0,0.0,gss,1,NOWAIT);
          delay(agss*grate+tep);
          sp4on();
          shapedpulse(p1pat,p1,v1,rof1,rof2);
          sp4off();
          delay(pw_gss_d);
          sp1off();

          /* Slice refocus pulse ********/
          obl_shapedgradient("","","gssr_up",pw_gssr,0.0,0.0,gssr,1,WAIT);
	  delay(tref-pw_gssr);
          obl_shapedgradient("","","gssr_dn",pw_gssr,0.0,0.0,gssr,1,WAIT);

          /* Spin echo section - with Crushers *********************/
          if (spinecho[0] == 'y'){
            delay(delay90_180);
            observepower(tpwr2);
            obl_shapedgradient("","","gss2",pw_gss2,0.0,0.0,gss2,1,NOWAIT);
            delay(agss*grate+tcrush+tep);
            sp4on();
            shapedpulse(p2pat,p2,v1,rof1,rof2);
            sp4off();
            delay(agss*grate+tcrush-tep);
            delay(delay180_echo);
          }
          else
	    delay(etl_delay);

          obl_shapedgradient("gror_up","","",pw_gror,gror,0.0,0.0,1,WAIT);
	  delay(tref2-pw_gror);
          obl_shapedgradient("gror_dn","","",pw_gror,gror,0.0,0.0,1,WAIT);

          if (prescan_flag == 1) {
	    assign(v15,v7);
          } else {
           if (table)
             getelem(t2,v10,v7);
           else
	     assign(v10,v7);
          }

          pe_oblique_shapedgradient("fstbp1_rd","fstbp_ph","",
                  2.0*pw_gro,gro*(1.0+epigroa/100.0),0.0,0.0,
                  -sgbpe,v7,num_segs,psi,phi,theta,NOWAIT,5);
          delay(2.0*pw_gro);

          incr(v10);
          assign(v11,v16);

          peloop(seqcon[2],etl,v5,v6);
            sub(v16,one,v16);
            ifzero(v16);
              assign(v15,v7);
            elsenz(v16);
              if (prescan_flag == 1) {
  	        assign(v15,v7);
              } else {
                if (table)
                  getelem(t2,v10,v7);
                else
                  assign(v10,v7);
              }
            endif(v16);
            mod2(v6,v12);

            /* Acquire echo ***************************************/
            
            ifzero(v12);
              pe_oblique_shapedgradient("blip_rd","blip_ph","",
                  2.0*pw_gro+np/(2.0*sw),gro*(1.0+epigroa/100.0),0.0,0.0,
                  -sgbpe,v7,num_segs,psi,phi,theta,NOWAIT,7);
              poffset(pro,gro);
              delay(tep);
            elsenz(v12);
              pe_oblique_shapedgradient("blip_rd","blip_ph","",
                  2.0*pw_gro+np/(2.0*sw),-gro,0.0,0.0,-sgbpe,v7,num_segs,
                  psi,phi,theta,NOWAIT,6);
              poffset(pro,-gro);
              delay(tep);
            endif(v12);

            acquire(np,1.0/sw);
            if (2.0*pw_gro > tep) {
              delay(2.0*pw_gro-tep);
            } else {
              delay(0.0);
            }

            incr(v10);

          endpeloop(seqcon[2],v6);
          sub(v10,v11,v10);
          sub(v10,one,v10);  /* need to subtract off the extra one too */

          /* ramp down at end of echo train */
          if (fmod(etl,2.0) == 1.0) {
            obl_shapedgradient("gro_dn","","",pw_gro,-gro,0.0,0.0,1,WAIT);
          } else {
            obl_shapedgradient("gro_dn","","",pw_gro,gro,0.0,0.0,1,WAIT);
          }

          if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
            obl_shapedgradient("gcr_up","gcr_up","gcr_up",pw_gcrush,
               gcrush,gcrush,gcrush,1,WAIT);
            delay(tcrush-pw_gcrush);
            obl_shapedgradient("gcr_dn","gcr_dn","gcr_dn",
               pw_gcrush,gcrush,gcrush,gcrush,1,WAIT);
          }

        delay(post_del);
        endmsloop(seqcon[1],v14);

        zero_all_gradients();  /* double check that all gradients are off */

        add(v10,v11,v10);
      endloop(v9);
    } /*** END INTERLEAVE PS ***/


    /*****************************************************/
    /**** PULSE SEQUENCE FOR NON-INTERLEAVED CASE ********/
    /*****************************************************/

    else
    {

    if ((image & 1) == 1) xgate(ticks);  /* wait for start scan button */
    if ((image & 2) == 2) sp3on(); else sp3off();

    sp2on();
    delay(scope_trig_time);  /* trigger to indicate start of volume acq */
    sp2off();

    msloop(seqcon[1],ns,v13,v14);
      assign(v15,v10);

      /* Inversion pulse **************/
       if (ti > 0.0) {
         poffset_list(pss,gss,ns,v14);
         obspower(tpwri);
         obl_shapedgradient("","","gss",pw_gss,0.0,0.0,gss,1,NOWAIT);
         delay(agss*grate+tep);
         sp4on();
         shapedpulse(p2pat,p2,zero,rof1,rof2);
         sp4off();
         delay(pw_gss_d);
         obl_shapedgradient("gcr_up","gcr_up","gcr_up",pw_gcrush,
             gcrush,-gcrush,gcrush,1,WAIT);
         delay(0.01+pw_gcrush);
         obl_shapedgradient("gcr_dn","gcr_dn","gcr_dn",
             pw_gcrush,gcrush,-gcrush,gcrush,1,WAIT);
         delay(ti-0.010);
       }

       loop(v8,v9);
         poffset_list(pss,gss,ns,v14);

         /* 90 degree pulse ************************************/ 
         observepower(tpwr1);
         sp1on();  /* put out trigger on sp1 every spin excitation */
         obl_shapedgradient("","","gss",pw_gss,0.0,0.0,gss,1,NOWAIT);
         delay(agss*grate+tep);
         sp4on();
         shapedpulse(p1pat,p1,v1,rof1,rof2);
         sp4off();
         delay(pw_gss_d);
         sp1off();
  
         /* slice refocus pulse ********/
         obl_shapedgradient("","","gssr_up",pw_gssr,0.0,0.0,gssr,1,WAIT);
   	 delay(tref-pw_gssr);
         obl_shapedgradient("","","gssr_dn",pw_gssr,0.0,0.0,gssr,1,WAIT);
  	 delay(etl_delay);
  
         obl_shapedgradient("gror_up","","",pw_gror,gror,0.0,0.0,1,WAIT);
 	 delay(tref2-pw_gror);
         obl_shapedgradient("gror_dn","","",pw_gror,gror,0.0,0.0,1,WAIT);

         if (prescan_flag == 1) {
	   assign(v15,v7);
         } else {
           if (table)
             getelem(t2,v10,v7);
           else
             assign(v10,v7);
         }

         pe_oblique_shapedgradient("fstbp1_rd","fstbp_ph","",
              2.0*pw_gro,gro*(1.0+epigroa/100.0),0.0,0.0,
              -sgbpe,v7,num_segs,psi,phi,theta,NOWAIT,5);
         delay(2.0*pw_gro);

         incr(v10);
         assign(v11,v16);

         peloop(seqcon[2],etl,v5,v6);
           sub(v16,one,v16);
           ifzero(v16);
             assign(v15,v7);
           elsenz(v16);
             if (prescan_flag == 1) {
	       assign(v15,v7);
             } else {
               if (table)
                 getelem(t2,v10,v7);
               else
                 assign(v10,v7);
             }
           endif(v16);
           mod2(v6,v12);

           /* Acquire echo ***************************************/
           ifzero(v12);
             pe_oblique_shapedgradient("blip_rd","blip_ph","",
                 2.0*pw_gro+np/(2.0*sw),gro*(1.0+epigroa/100.0),0.0,0.0,
                 -sgbpe,v7,num_segs,psi,phi,theta,NOWAIT,7);
             poffset(pro,gro);
             delay(tep);
           elsenz(v12);
             pe_oblique_shapedgradient("blip_rd","blip_ph","",
                 2.0*pw_gro+np/(2.0*sw),-gro,0.0,0.0,-sgbpe,v7,num_segs,
                 psi,phi,theta,NOWAIT,6);
             poffset(pro,-gro);
             delay(tep);
           endif(v12);
 
           acquire(np,1.0/sw);
           if (2.0*pw_gro > tep) {
             delay(2.0*pw_gro-tep);
           } else {
             delay(0.0);
           }
           incr(v10);

         endpeloop(seqcon[2],v6);

         sub(v10,one,v10);  /* need to subtract off the extra one too */

         /* ramp down at end of echo train */
         if (fmod(etl,2.0) == 1.0) {
           obl_shapedgradient("gro_dn","","",pw_gro,-gro,0.0,0.0,1,WAIT);
         } else {
           obl_shapedgradient("gro_dn","","",pw_gro,gro,0.0,0.0,1,WAIT);
         }

         if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
           obl_shapedgradient("gcr_up","gcr_up","gcr_up",pw_gcrush,
               gcrush,gcrush,gcrush,1,WAIT);
           delay(tcrush-pw_gcrush);
           obl_shapedgradient("gcr_dn","gcr_dn","gcr_dn",
               pw_gcrush,gcrush,gcrush,gcrush,1,WAIT);
         }

         delay(post_del);
       endloop(v9);
      endmsloop(seqcon[1],v14);

      zero_all_gradients();  /* double check that all gradients are off */

    } /* END NON-INTERLEAVE PS */
}
