/****************************************************************************

   This program is copyrighted worldwide by the John P. Robarts Research
   Institute.  Any distribution, copying or redistribution is expressly
   forbidden.

   Use this program to reconstruct an arrayed Varian 3D dataset in the
   3rd dimension only. The 3D images can then be viewed with ft2d() or dmi
   from Vnmr.  The data must have already been flashc and tabc processed.
   This program should be compatible with the multielement arrayed studies
   such as those used for functional imaging but this has yet to be checked.

 gcc -O3 -o fft3d fft3d.c fourn.c shiftc.c -lm

 20/1/97	Ravi S. Menon Written.
 22/1/97	Ravi S. Menon Added slice position calculation. Need to verify.
 24/1/97	Ravi S. Menon Slice order corrected and fermi added in 2nd PE 
                              dimension. 
 28/1/97	Ravi S. Menon Fermi filters in all 3 dimensions added.
 12/2/97	Ravi S. Menon Ellipsoidal Fermi incorporated in 2-D. Linear 
		Fermi in 2nd-PE direction.
 14/1/99	Peter Jezzard (PJ) alterned sign of phase ramp to centre the
		slice block when performing the 3rd FFT
 25/8/99        Peter Jezzard Added capability for zero filling to next 
                nearest power of 2 in the slice direction 
******************************************************************************/

/* include library files for the program */
/* these allow compilation in RRI environment */
#include <sys/file.h>
#include <sys/fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <float.h>

#define PI 3.1415926
#define F_FRAC 0.9
#define BACK_SPACE 8
#define MAX_NUM_STR 80
#define NUM_DC 4
#define MAX_NP 512
#define MAX_NV 512
#define MAX_NV2 256

char *itoa();
void bckSpcPrt();

/*
 * structure definitions are given explicitly below
 */

struct datafilehead
{
   long    nblocks;                 /* number of blocks in file     */
   long    ntraces;                 /* number of traces per block   */
   long    np;                      /* number of elements per trace */
   long    ebytes;                  /* number of bytes per element  */
   long    tbytes;                  /* number of bytes per trace    */
   long    bbytes;                  /* number of bytes per block    */
   short   transf;                  /* transposed storage flag      */
   short   status;                  /* status of whole file         */
   long    spare1;                  /* reserved for future use      */
}       main_header1;

struct datablockhead
{
   short   scale;                  /* scaling factor               */
   short   status;                 /* status of data in block      */
   short   index;                  /* block index                  */
   short   spare3;                 /* reserved for future use      */
   long    ctcount;                /* completed transients in fids */
   float   lpval;                  /* left phase in phasefile      */
   float   rpval;                  /* right phase in phasefile     */
   float   lvl;                    /* level drift correction       */
   float   tlt;                    /* tilt drift correction        */
}      *block_header;

main(argc, argv)
   int     argc;
   char   *argv[];
{
   long    np, maxvalue, scale_factor;
   short  *sbuff;
   char   *inbuff;

   int 	   blkOff, bIdx, bIdx2, trcOff, tIdx, offset, dc_flag;
   int     i, j, k, l, m, nn[1], ndummy, ni, narray, nv, nv2, new_nv2, old_nv2,
          *lbuff, eb, nfids_in, nfids_out, in_file1, out_file, numBlkPts, nt;
   int 	   ns, arraydim, arrayelem, buff_offset;
   int     orig_nblocks, new_nblocks;

   float   lpe2, lpe, rdc, idc, pss, thk, s_incr, s_start, phaseoffset;
   float  *fdummy, fmax = 0.0, *fbuff, s_pos[512], *ss, *cs;

   double  filter, temp, cutoff, scale, r1, r2, r3, r;
   double  fermi[MAX_NV][MAX_NP],slice[MAX_NV2];

   FILE   *fp, *out;

   char    infile_name[100], proc_string[999], string[100], string1[100];
   char   *malloc();

   if (argc < 2)
   {
      printf("Usage: %s origfid \n", argv[0] );
      exit(1);
   }
   /****
   * Get input and output directory names from command line arguments.
   ****/
   else
   {
      strcpy(infile_name, argv[1]);
   }

/* For debugging
   printf( "sizeof(struct datablockhead) = %d\n", sizeof(struct datablockhead));
*/

   /* Determine the accessibility of the input directory. */
   if (access(infile_name, F_OK | R_OK))
   {
      perror("Access");
      printf("Can not access file: %s\n", infile_name);
      exit(2);
   }

   /* Open input procpar file. */
   strcpy(string, infile_name);
   strcat(string, "/procpar");
   if ((fp = fopen(string, "r")) == NULL)
      printf("unable to open %s\n", string);

   /* Move original procpar to procpar.orig for backup. */
   sprintf(string, "mv %s/procpar %s/procpar.orig", infile_name, infile_name);
   system(string);
   
   /* Read the procpar and get the relevant values. */
   while (fgets(proc_string, 999, fp) != NULL)
   {
      if (!strncmp(proc_string, "nt ", 3))
      {
         fgets(proc_string, 999, fp);
         sscanf(proc_string, "%d %d", &i, &nt);
         fgets(proc_string, 999, fp);
         dc_flag=(int)fmod((double)nt,2.0);
      }
      if (!strncmp(proc_string, "nv2 ", 4))
      {
         fgets(proc_string, 999, fp);
         sscanf(proc_string, "%d %d", &i, &nv2);
         fgets(proc_string, 999, fp);
      }
      if (!strncmp(proc_string, "nv ", 3))
      {
         fgets(proc_string, 999, fp);
         sscanf(proc_string, "%d %d", &i, &nv);
         fgets(proc_string, 999, fp);
      }
      if (!strncmp(proc_string, "ni ", 3))
      {
         fgets(proc_string, 999, fp);
         sscanf(proc_string, "%d %d", &i, &ni);
         fgets(proc_string, 999, fp);
      }
      if (!strncmp(proc_string, "lpe2 ", 5))
      {
         fgets(proc_string, 999, fp);
         sscanf(proc_string, "%d %f", &i, &lpe2);
         fgets(proc_string, 999, fp);
      }
      if (!strncmp(proc_string, "lpe ", 4))
      {
         fgets(proc_string, 999, fp);
         sscanf(proc_string, "%d %f", &i, &lpe);
         fgets(proc_string, 999, fp);
      }
      if (!strncmp(proc_string, "pss ", 4))
      {
         fgets(proc_string, 999, fp);
         sscanf(proc_string, "%d %f", &i, &pss);
         fgets(proc_string, 999, fp);
      }
      if (!strncmp(proc_string, "thk ", 4))
      {
         fgets(proc_string, 999, fp);
         sscanf(proc_string, "%d %f", &i, &thk);
         fgets(proc_string, 999, fp);
      }
   }
   fclose(fp);

   printf( "nv2 = %d\n", nv2);
   old_nv2=nv2;
   new_nv2=nv2;  /* default */
   if ((nv2 != 16) && (nv2 != 32) && (nv2 != 64) && (nv2 != 128) && 
      (nv2 != 256)) {
     /* need to zero fill */
     if (nv2 < 256) new_nv2=256;
     if (nv2 < 128) new_nv2=128;
     if (nv2 < 64) new_nv2=64;
     if (nv2 < 32) new_nv2=32;
     if (nv2 < 16) new_nv2=16;
     printf( "...zero filling up to nv2 = %d\n", new_nv2);
   }
   printf( "nt = %d\n", nt);
   printf( "nv = %d\n", nv);
   printf( "lpe = %f\n", lpe);
   printf( "lpe2 = %f\n", lpe2);
   printf( "pss = %f\n", pss);
   printf( "thk (cm) = %f\n", thk/10.);

   /* Set variables that depend on 2nd dimension phase-encode */
   nn[0] = new_nv2;
   ns = new_nv2;
   thk=thk/10.;
   if (ni==0 && nv !=0) ni=nv;

   /* Calculate slice offset array for 2D slices */
   s_incr = lpe2/ns;
   s_start = lpe2*0.5;
   for(i=0; i<ns; i++)
      s_pos[i] = pss-s_start+i*s_incr+s_incr/2.0;

   strcpy(string, infile_name);
   strcat(string, "/fid");
   in_file1 = open(string, O_RDONLY);
   if (in_file1 < 0)
   {
      perror("Open");
      printf("Can't open input fid file.\n");
      exit(4);
   }

   /****
   * Read main_header at top of input file.
   ****/
   if (read(in_file1, &main_header1, 32) != 32)
   {
      perror("read");
      exit(5);
   }
   /* memory allocation */
   numBlkPts = main_header1.ntraces * main_header1.np * main_header1.nblocks;
   numBlkPts = (int)((float)numBlkPts * ((float)new_nv2)/((float)nv2));
   inbuff = malloc( 4 * numBlkPts );

/* For debugging

    printf( "inbuff == %d != %d\n", 
           main_header1.nblocks * (main_header1.bbytes - 28),
           main_header1.np * 4 * main_header1.nblocks);
    printf("numBlkPts = %d\n",numBlkPts);

    printf( "main_header1.ebytes = %d\n",  main_header1.ebytes );
    printf( "sizeof( int ) = %d\n", sizeof( int ));
    printf( "sizeof( short ) = %d\n", sizeof( short ));
    printf( "sizeof( float ) = %d\n", sizeof( float ));
*/

   orig_nblocks = main_header1.nblocks;
   new_nblocks = (int)((float)main_header1.nblocks 
            * ((float)new_nv2)/((float)nv2));

   block_header = (struct datablockhead *)malloc(new_nblocks 
                * sizeof(struct datablockhead));

   sbuff = (short *) inbuff;
   lbuff = (int *) inbuff;
   fbuff = (float *) inbuff;
   fdummy = (float *) malloc(2 * new_nv2 * sizeof(float));
   ss = (float *) malloc(new_nv2 * sizeof(float));
   cs = (float *) malloc(new_nv2 * sizeof(float));
   np = main_header1.np;

   if (new_nv2 != nv2) {
     for (i = 0; i < numBlkPts*4; i++) {
      inbuff[i]=0;
     }
   }

   /* read in all the fid data */
   printf( "Reading data from fid file\n" );
   buff_offset = (new_nv2-nv2)/2;
   for (i = 0; i < main_header1.nblocks/nv2; i++)
   {
      for (j = 0; j < nv2; j++)
      {
        if (read(in_file1, &block_header[i*nv2+j], 28) != 28)
        {
           perror("read");
           exit(5);
        }
        if ((k = read(in_file1, inbuff + (i*new_nv2 + j + buff_offset) * 
              (main_header1.bbytes - 28), main_header1.bbytes - 28)) 
              != (main_header1.bbytes - 28))
        {
/* For debugging
           printf("For block %d, %d / %d bytes read...\n", 
              i, k, main_header1.bbytes - 28);
           printf("errno = %d\n", errno);
*/
           perror("read");
           exit(5);
        }
      }
   }
   close(in_file1);

   /* data conversion to float */
   printf( "Data conversion from int to float\n");
   for (i = numBlkPts - 1; i >= 0; i--)
   {
      if (main_header1.ebytes == 4)
         fbuff[i] = (float) lbuff[i];
      else
         fbuff[i] = (float) sbuff[i];
   }

   /* DC offset removal */
   printf( "Reading DC offset values\n" );
   rdc=block_header[0].lvl;  /* Read DC offset from fid file (PJ) */
   idc=block_header[0].tlt;  /* Read DC offset from fid file (PJ) */


     /*   ===================================================================
     Comment out the old way of doing the DC correction. Better to take from
     fid file itself (PJ)

   rdc = idc = 0.0;
   for (bIdx = 0; bIdx < new_nblocks/new_nv2; bIdx++)
   {
      blkOff = bIdx * main_header1.ntraces * main_header1.np * new_nv2
                    + main_header1.ntraces * main_header1.np * buff_offset;
      for (bIdx2 = 0; bIdx2 < nv2; bIdx2++) 
      {
        for( tIdx = 0; tIdx<main_header1.ntraces; tIdx++)
        {
           trcOff = tIdx * main_header1.np;
           offset = blkOff + trcOff;

           for (j = 0; j < NUM_DC; j++)
           {
              rdc += fbuff[offset + j * 2] 
                 + fbuff[offset + (main_header1.np / 2 - j -1) * 2];
   
              idc += fbuff[offset + j * 2 + 1] 
                 + fbuff[offset + (main_header1.np / 2 - j -1) * 2 + 1];
           }
        }	
        blkOff = blkOff + main_header1.ntraces * main_header1.np;
      }

   }

   rdc /= (2.0 * NUM_DC * main_header1.nblocks * main_header1.ntraces);
   idc /= (2.0 * NUM_DC * main_header1.nblocks * main_header1.ntraces);
    ===================================================================  */

	printf( "rdc = %f\n", rdc );
	printf( "idc = %f\n", idc );
        printf("DC removal flag = %d\n", dc_flag);

   for (bIdx = 0; bIdx < new_nblocks/new_nv2; bIdx++)
   {
      blkOff = bIdx * main_header1.ntraces * main_header1.np * new_nv2
                    + main_header1.ntraces * main_header1.np * buff_offset;
      for (bIdx2 = 0; bIdx2 < nv2; bIdx2++) 
      {
        for( tIdx = 0; tIdx<main_header1.ntraces; tIdx++)
        {
           trcOff = tIdx * main_header1.np;
           offset = blkOff + trcOff;
           if (dc_flag) {
             for (j = 0; j < main_header1.np / 2; j++)
             {
               fbuff[offset + 2 * j] -= rdc;
               fbuff[offset + 2 * j + 1] -= idc;
             }
           }
        }
        blkOff = blkOff + main_header1.ntraces * main_header1.np;
     }
   }

   /* now that data read in, set nv2=new_nv2 */
   nv2=new_nv2;

   /* now that DC correction complete, update main_header1.nblocks */
   main_header1.nblocks = new_nblocks;

   /* Fermi filter coefficients in 3D k-space  */
   cutoff = (double)(0.5*F_FRAC);
   for (i = 1; i < (1+nv2/2); i++) {
		r1 = (double)i/(double)(nv2/2.);
		filter =  1.0/(1.0+exp((r1-cutoff)*3.0));
		slice[(nv2/2)-i] = filter;
        	slice[((nv2/2)-1)+i] = filter;
   }
   cutoff = (double)F_FRAC;
   for (j = 1; j < (1+nv/2); j++) {
  	for (k = 1; k < (1+np/4); k++) {
		r2 = (double)j/(double)(nv/2.);
		r3 = (double)k/(double)(np/4.);
		r = r2*r2 + r3*r3;
		filter =  1.0/(1.0+exp((r-cutoff)*5.0));
		fermi[(nv/2)-j][((np/4)-1)+k] = filter;
		fermi[((nv/2)-1)+j][(np/4)-k] = filter;
		fermi[(nv/2)-j][(np/4)-k] = filter;
		fermi[((nv/2)-1)+j][((np/4)-1)+k] = filter;
	}
    }

   /* Fourier Transform in the third dimension */
   /* coefficients for phase-rewinding for off-center slab */
   /* sign of phase ramp changed for Oxford implementation */
   /* was -2.000000 * PI / lpe2 * pss originally  (PJ, Oxford) */
   phaseoffset = 2.000000 * PI / lpe2 * pss;
   for (m = 0; m < nn[0]; m++)
   {
   ss[m] = (float) sin((double) phaseoffset * (double) (m - nn[0] / 2));
   cs[m] = (float) cos((double) phaseoffset * (double) (m - nn[0] / 2));
   }

   narray = (main_header1.ntraces * main_header1.nblocks) / nv / nv2;
   printf( "number of arrayed elements (other than slice) = %d\n", narray );

   fprintf( stderr, "Starting 3rd dimension FT\n");
   ndummy = main_header1.np * nv2 * narray;
   for (l = 0; l < nv; l++)  /* loop through nv phase-encoding lines */
   {
      for (j = 0; j < narray; j++)/* number of array elements */
      {
         /* scanning through image matrix */
         for (i = 0; i < main_header1.np / 2; i++) 
         {
            /* extracting data in 3rd dimension */
            for (k = 0; k < nv2; k++)
            {
               fdummy[2 * k] = fermi[l][i] * slice[k] *
                  fbuff[k * main_header1.np 
                     + j * main_header1.np * nv2 
                     + l * ndummy 
                     + i * 2];

               fdummy[2 * k + 1] = fermi[l][i] * slice[k] *
                  fbuff[k * main_header1.np 
                     + j * main_header1.np * nv2 
                     + l * ndummy 
                     + i * 2 
                     + 1];
            }

            /* off center slab phase-rewinding */
            for (m = 0; m < nn[0]; m++)
            {
               rdc = fdummy[2 * m] * cs[m] - fdummy[2 * m + 1] * ss[m];

               fdummy[2 * m + 1] = fdummy[2 * m] * ss[m] 
                  + fdummy[2 * m + 1] * cs[m];

               fdummy[2 * m] = rdc;
            }

            /* Fourier transform */
            shiftc(fdummy, &nn[0], 1);
            fourn(fdummy - 1, &nn[0] - 1, 1, 1);
            shiftc(fdummy, &nn[0], 1);

            /* replace the original data with the FT'ed data */

            for (k = 0; k < nv2; k++)        /* 3rd dimension */
            {
               fbuff[k * main_header1.np 
                  + j * main_header1.np * nv2 
                  + l * ndummy 
                  + i * 2] = fdummy[2 * k];

               fbuff[k * main_header1.np 
                  + j * main_header1.np * nv2 
                  + l * ndummy 
                  + i * 2 + 1] = fdummy[2 * k + 1];
            }
         }
      }
   }
/* For debugging
   bckSpcPrt( NULL );
*/
   fprintf( stderr, "Done\n");

   printf( "Data scaling and conversion back to ints\n" );
   /* data scaling and conversion */
   for (i = 0; i < numBlkPts; i++)
   {
      fmax = (fmax > (float) fabs((double) fbuff[i])) ? 
         fmax : (float) fabs((double) fbuff[i]);
   }

   if (main_header1.ebytes == 4)
   {
      maxvalue = 2147483647;
      scale = (double) maxvalue / fmax;
      for (i = 0; i < numBlkPts; i++)
      {
         lbuff[i] = (int)((double) (fbuff[i] * scale));
      }
   } 
   else
   {
      maxvalue = 32767;
      scale = (double) maxvalue / fmax;
      for (i = 0; i < numBlkPts; i++)
      {
         sbuff[i] = (short) (int)((double) (fbuff[i] * scale));
      }
   }

   /* Open input procpar file. */
   strcpy(string, infile_name);
   strcat(string, "/procpar.orig");
   if ((fp = fopen(string, "r")) == NULL)
      printf("unable to open %s\n", string);

   /* Open output procpar file. */
   strcpy(string, infile_name);
   strcat(string, "/procpar");
   if ((out = fopen(string, "w")) == NULL)
      printf("unable to open %s\n", string);

   /* Read the procpar and set the relevant values. */
   while (fgets(proc_string, 999, fp) != NULL)
   {
      if (!strncmp(proc_string, "celem", 5))
      {
         fputs(proc_string, out);
         fgets(proc_string, 999, fp);
         sscanf(proc_string, "%d %d", &i, &j);
         if (j == 0) fprintf(out, "1 %d \n", ns);
         else fprintf(out, "1 %d \n", j * ns);
         fgets(proc_string, 999, fp);
      }   
      if (!strncmp(proc_string, "arrayelem", 9))
      {
         fputs(proc_string, out);
         fgets(proc_string, 999, fp);
         sscanf(proc_string, "%d %d", &i, &arrayelem);
         fprintf(out, "1 %d \n", arrayelem);
         fgets(proc_string, 999, fp);
      }   
      if (!strncmp(proc_string, "arraydim", 8))
      {
         fputs(proc_string, out);
         fgets(proc_string, 999, fp);
         sscanf(proc_string, "%d %d", &i, &j);
         fprintf(out, "1 %d \n", (j/old_nv2)*ns*nv);
         fgets(proc_string, 999, fp);
      }   
      if (!strncmp(proc_string, "seqcon ", 7))
      {
         fputs(proc_string, out);
         fgets(proc_string, 999, fp);
         sscanf(proc_string, "%d %s", &i, string);
	 fprintf(out, "1 \"ncsnn\"\n");
         fgets(proc_string, 999, fp);
      }
      if (!strncmp(proc_string, "array", 5))
      {
         fputs(proc_string, out);
         fgets(proc_string, 999, fp);
         sscanf(proc_string, "%d %s", &i, string);
         k = strlen(string);
         for (i = 0; i < k - 1; i++)
            string1[i] = string[i];
         string1[k - 1] = '\0';
	 if (narray > 1)
         	strcat(string1, ",pss\"");
	 else
         	strcat(string1, "pss\"");
         fprintf(out, "1 %s \n", string1);
         fgets(proc_string, 999, fp);
      }
      if (!strncmp(proc_string, "nf ", 3))
      {
	 fputs(proc_string, out);
         fgets(proc_string, 999, fp);
	 fprintf(out, "1 1\n");
         fgets(proc_string, 999, fp);
      }
      if (!strncmp(proc_string, "nD ", 3))
      {
	 fputs(proc_string, out);
         fgets(proc_string, 999, fp);
	 fprintf(out, "1 2\n");
         fgets(proc_string, 999, fp);
      }
      if (!strncmp(proc_string, "nv ", 3))
      {
	 fputs(proc_string, out);
         fgets(proc_string, 999, fp);
	 fprintf(out, "1 %d\n", nv);
         fgets(proc_string, 999, fp);
      }
      if (!strncmp(proc_string, "ns ", 3))
      {
	 fputs(proc_string, out);
         fgets(proc_string, 999, fp);
	 fprintf(out, "1 1\n");
         fgets(proc_string, 999, fp);
      }
      if (!strncmp(proc_string, "nv2 ", 4))
      {
	 fputs(proc_string, out);
         fgets(proc_string, 999, fp);
	 fprintf(out, "1 0\n");
         fgets(proc_string, 999, fp);
      }
      if (!strncmp(proc_string, "ni ", 3))
      {
	 fputs(proc_string, out);
         fgets(proc_string, 999, fp);
	 fprintf(out, "1 %d\n",nv);
         fgets(proc_string, 999, fp);
      }
      if (!strncmp(proc_string, "t_ns ", 5))
      {
	 fputs(proc_string, out);
         fgets(proc_string, 999, fp);
	 fprintf(out, "1 %d\n",nv2);
         fgets(proc_string, 999, fp);
      }
      if (!strncmp(proc_string, "thk ", 4))
      {
	 fputs(proc_string, out);
         fgets(proc_string, 999, fp);
	 fprintf(out, "1 %.4f\n",10.0*lpe2/((float)nv2));
         fgets(proc_string, 999, fp);
      }
      if (!strncmp(proc_string, "t_thk ", 6))
      {
	 fputs(proc_string, out);
         fgets(proc_string, 999, fp);
	 fprintf(out, "1 %.4f\n",10.0*lpe2/((float)nv2));
         fgets(proc_string, 999, fp);
      }
      if (!strncmp(proc_string, "pss ", 4))
      {
	 fputs(proc_string, out);
         fgets(proc_string, 999, fp);
	 fprintf(out, "%d ",nv2);
	 for (i =0; i < nv2; i++)
	 	fprintf(out, "%5.3f ",s_pos[i]);
	 fprintf(out, "\n");
         fgets(proc_string, 999, fp);
      }
      fputs(proc_string, out);
   }
   fclose(fp);
   fclose(out);

   /* Move original fid to fid.orig for backup */
   sprintf(string, "mv %s/fid %s/fid.orig", infile_name, infile_name);
   system(string);

   /* Output data files */
   printf( "Writing output data file\n" );
   strcpy(string, infile_name);
   strcat(string, "/fid");
   in_file1 = open(string, O_WRONLY | O_CREAT | O_TRUNC, 0644);
   write(in_file1, &main_header1, 32);
   ndummy = main_header1.bbytes - 28;
   for (i = 0; i < main_header1.nblocks; i++)
   {
      if (i < orig_nblocks) {
        block_header[i].lvl=0.0;  /* zero in output file since DC done (PJ) */
        block_header[i].tlt=0.0;  /* zero in output file since DC done (PJ) */
        write(in_file1, block_header + i, 28);
      } else {
        block_header[orig_nblocks - 1].index = i+1;
        block_header[orig_nblocks - 1].lvl=0.0;  /* zero since DC done (PJ) */
        block_header[orig_nblocks - 1].tlt=0.0;  /* zero since DC done (PJ) */
        write(in_file1, block_header + orig_nblocks - 1, 28);
      }
      write(in_file1, inbuff + i * ndummy, ndummy);
   }
   close(in_file1);
   printf( "All done!\n\n" );
}


void bckSpcPrt( str )
   char *str;
{
   static int bckSpcCnt = 0;
   int idx;

   for( idx=0; idx<bckSpcCnt; idx++)
      putc( BACK_SPACE, stderr );

   fflush( stderr );

   if (str == NULL )
   {
      bckSpcCnt = 0;
      return;
   }
   else
      bckSpcCnt = strlen(str);
 
   fputs( str, stderr );
   fflush( stderr );
}     

char *itoa( iNum )
   int iNum;
{
   static char iStr[ MAX_NUM_STR ];

   /* Returns a static string */
   /* Don't try to free */
   /* String only valid until next call */

   sprintf( iStr, "%d", iNum );

   return( iStr );
}

