#ifndef LINT
static char SCCSid[] = "@(#)flash.c 10.5 05 Mar 1996 Copyright (c) 1994 Varian Assoc.,Inc. All Rights Reserved";
#endif
/***********************************************************************
    MP_FLASH3D.C

    Magnetization Prepared Gradient-echo fast imaging sequence.                     

    VERSION 2.0        6/94/94  
    Revisions:
      2.1   2/96    Fixed rfspoiling for nv=0
      2.2   3/97    JSG - Completed changes to 3D
      3.0   6/98    Integrated into Oxford xvnmr software, P. Jezzard, Oxford
      3.1   6/98    added shaped waveform statements, P. Jezzard, Oxford

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
       __  __                                                  __   __
      |   |                                                      |    |
  RF  |   |----- [ p1 ] ----------------------- |** ACQ **| -----|    |
      |   |                                                      |    |
      |   |         < - - - - - - - te - - - - - - - >           |    |
      |   |       ____                                       ___ |    |
  GSS |   |______/////\     ________________________________/___\|    |
      |   |<---->      \\\\/ gssr                           \___/|    |
      |   |             ===                                      |    |
      |   |  predelay  <--> tref                                 | nv | nv2
      |   |             ___                                  ___ |    |
  GPE |   |____________/___\________________________________/___\|    |
      |   |            \___/                                \___/|    |
      |   |                 < - tA - >                           |    |
      |   |                            ____________________      |    |
  GRO |   |____________     __________/////////////////////\_____|    |
      |   |       gror \\\\/                       < tspoil >    |    |
      |   |                                                      |    |
      |   |< - - - - - - - - - - - tr - - - - - - - - - - - - - >|    |
      |__ |__                                                  __|  __|

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

#include <standard.h>
#include <group.h>
#include "sequence_utilities.h"
#include "logical_to_physical.h"
#define OFFSETDELAY 26e-6
#define RFSPOILDELAY 50e-6

/*==========================================================================*
  define the pulse sequence variables outside the pulse sequence so that
  they can be seen as global variables by the safety monitoring code
*==========================================================================*/

    double  predelay,seqtime,tref,risetref,dseg;
    double  gpe2int, sgpe2;
    double  trefmin,txmin,tymin,tzmin,temin,tA;
    double  agro,agss,sgpe,grate,gssrint,grorint,gpeint;      
    double  reffreq,nextphase,spoilfreqs[1024];
    int     i,len,counter,midflag,endflag,dummy_ssc;
    double  val1,val2,nsteps,pw_gss_d,extra_te_error;
    double  grise,tep,rftep,pw_gro,pw_gss,pw_end_gcrush,temp_val,agcrush;
    char    shape_name[MAXSTR],err_string[MAXSTR],dummy_scans[MAXSTR];
    char    refocus[MAXSTR],min_phase[MAXSTR],gcoil[MAXSTR];
    double xout,yout,zout;

#include "mp_flash3d_safety_calc.h"

/* Spatial Sat declarations */
double gsssat,pw_gsssat;
double presat_off=0.0,post_presat_del,sat_thk,sat_psi,sat_phi,sat_theta;
double presat=0.0,pw_gcrush;
double t_presat_b,t_presat_a;

calculate_spatial_saturation_gradients(double *t_presat_b,double *t_presat_a)
{
  char shape_name[MAXSTR];

  sprintf(shape_name,"gsat");
  pw_gsssat = create_trap_gradient(fabs(gsssat),shape_name,
         psat+rof1+rof2,grate,ix);

  *t_presat_b = fabs(gsssat)*grate + rof1 + psat/2;
  *t_presat_a = fabs(gsssat)*grate + rof2 + psat/2 + post_presat_del;
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
    sprintf(shape_name,"gcr");
    pw_gcrush = create_ramp_gradient(fabs(gcrush),shape_name,1,grate,ix);
    pw_gcrush = create_ramp_gradient(fabs(gcrush),shape_name,-1,grate,ix);
    if (pw_gcrush > tcrush) {
      sprintf(err_string,"%s: tcrush too small. Min tcrush = %f\n",
              seqfil,pw_gcrush);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
    *t_presat_a=*t_presat_a + 2.0*pw_gcrush + tcrush;
  }
}

spatial_saturation_pulse()
{
  /* uses psat : satpwr : satpat : gsssat */
  /* also uses sat_psi, sat_phi and sat_theta rather than psi, phi, theta */
  /* Enables definition of independent sat planes relative to slice plane */
  position_offset(presat_off,gsssat,resto,OBSch);
  obspower(satpwr);

  oblique_shapedgradient("","","gsat",pw_gsssat,0.0,0.0,gsssat,sat_psi,sat_phi,sat_theta,1,NOWAIT);

  delay(fabs(gsssat)*grate+rftep);
  sp4on();
  shaped_pulse(satpat,psat,v1,rof1,rof2);
  sp4off();
  if(rftep<fabs(gsssat)*grate)delay(fabs(gsssat)*grate-rftep);

  /* Crushers */
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
    obl_shapedgradient("gcr_up","gcr_up","gcr_up",
                       pw_gcrush,gcrush,gcrush,gcrush,1,WAIT);
    delay(tcrush);
    obl_shapedgradient("gcr_dn","gcr_dn","gcr_dn",
                       pw_gcrush,gcrush,gcrush,gcrush,1,WAIT);
  }
  delay(post_presat_del);
}

pulsesequence()
{
    double  create_ramp_gradient(), create_trap_gradient();

    initparms_sis();
    loop_check();

    /*************************************************************************/
    /* get the global parameters */
    /*************************************************************************/
    P_getreal(GLOBAL,"grise",&grise,1); /* grad risetime to full strength */
    if ((grise < 0.0) || (grise > 0.002)) {
      sprintf(err_string,"%s: impropper grise time defined in global file\n",
                       seqfil);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
    P_getreal(GLOBAL,"tep",&tep,1); /* small tweaker time (us) to allow for
                            the gradient propagation time, causes echoes to
                            appear symmetrically about echo center line */
    tep=tep/1e6;
    P_getreal(GLOBAL,"rftep",&rftep,1); /* small tweaker time (us) to allow for
                            the gradient propagation time, w.r.t. RF pulses */
    rftep=rftep/1e6;
    /*************************************************************************/

    getstr("gcoil",gcoil);          /* gradient coil in use */
    getstr("refocus",refocus);
    getstr("min_phase",min_phase);
    getstr("dummy_scans",dummy_scans);
    dseg=getval("dseg");
    grate = grise/gmax;
    agro = fabs(gro);
    agss = fabs(gss);
    if (agss == 0.0) {
        sprintf(err_string,"%s: Zero slice select gradient\n",seqfil);
        printf(err_string);
        log_error(err_string);
        abort(1);
    }
    gcrush = fabs(gmax/2);
    agcrush = fabs(gcrush);

    /* Spatial Sat Variables */
    presat=getval("presat");
    if (presat) {
      gsssat = getval("gsssat");
      presat_off=getval("presat_off");
      sat_psi=getval("sat_psi");
      sat_phi=getval("sat_phi");
      sat_theta=getval("sat_theta");
      sat_thk=getval("sat_thk");
      post_presat_del=getval("post_presat_del");
    }
    if (presat) calculate_spatial_saturation_gradients(&t_presat_b,&t_presat_a);

    /* Decide on number of pe steps per segment or image ****/
    /* by parsing the table string **************************/
    /* table name must be of form #pestepsxxx#segmentsyyy ***/
    loadtable(petable);
    len = strlen(petable);
    counter = 0;
    midflag = 0;
    endflag = 0;
    val1 = 0.0;
    val2 = 0.0;
    while (counter < len) {
      if (isdigit(petable[counter]) && !midflag) {
        val1 =  atof(&(petable[counter]));
          /* skip past remaining part of first number set */
          while (isdigit(petable[counter ++]))
          ;
      }  
      if (!isdigit(petable[counter]))
        midflag = 1;
      if (isdigit(petable[counter]) && midflag) {
        val2 = 10.0 *val2 + atof(&(petable[counter]));
          /* skip past remaining part of second number set */
          while (isdigit(petable[counter ++]))
          ;
        endflag = 1;
      }  
    counter++;
    }
    if (endflag)
      nsteps = val1/val2;
    else
      nsteps = val1;

    printf("number of pe steps per image or image segment is %f\n",nsteps);

    if(dummy_scans[0]=='n') dummy_ssc=0;
    else {
      /* figure out a sensible number of dummy scans */
      dummy_ssc=(int)((10.0*exp(-tr/0.1)+6.0*exp(-tr/0.6))+0.5);
      /* if the sequence is interleaved then double number of steady states */
      if ((endflag) && (dseg != 0.0)) dummy_ssc=dummy_ssc*2;
    }
    printf("number of dummy scans is %d\n",dummy_ssc);

    /* GRADIENT*TIME integrals for slice selection and readout */
    grorint = agro*(at/2.0 + grate*agro/2.0);
    gssrint = agss*(p1/2.0 + rof2 + agss*grate/2.0);

    /* Correct if minimum phase r.f. excitation pulse being used ****/
    if (min_phase[0] == 'y') {
      gssrint = agss*(p1*0.135 + rof2 + agss*grate/2.0);
    }

    gpeint = B0/(lpe*sfrq*1.0e6);
    gpe2int = B0/(lpe2*sfrq*1.0e6);

    /*******************************************************
    * Calculate minimum time for phase-encoding, slice refocussing
    * and readout prefocussing block.  Trapezoidal if GMAX is exceeded.
    * TSSRMIN does not include the gradient fall time.  RISETREF is
    * the gradient fall time for the PE block.
    *******************************************************/

    logical_to_physical(phi,psi,theta,
			grorint,gpeint*nv/2.0,gssrint + gpe2int*nv2/2.0,
			&xout,&yout,&zout);
    
    if (trise*gmax >= xout)
      txmin = sqrt(grate*xout);
    else
      txmin = xout/gmax;
    
    if (trise*gmax >= yout)
      tymin = sqrt(grate*yout);
    else
      tymin = yout/gmax;
    
    if (trise*gmax >= zout)
      tzmin = sqrt(grate*zout);
    else
      tzmin = zout/gmax;
    
    trefmin = (txmin > tymin) ? txmin : tymin;
    trefmin = (trefmin > tzmin) ? trefmin : tzmin;

    printf("trefmin = %f\n",trefmin);

    /*******************************************************
    * If TREFMIN implies trapezoidal PE block, mimimum RISETREF
    * is RISETIME, otherwise minimum RISETREF is TREFMIN.
    *******************************************************/
    risetref = (trefmin > trise) ? trise : trefmin;

    /* account for case of rftep being longer than the gradient ramp time */
    pw_gss_d = agss*grate - rftep;
    extra_te_error = 0.0;
    if (pw_gss_d < 0.0) {
      pw_gss_d = 0.0;
      extra_te_error = rftep-agss*grate;
    }

    if (tspoil < rftep) {
        sprintf(err_string,"%s: tspoil too short. Min tspoil = %f\n",
                         seqfil, rftep);
        printf(err_string);
        log_error(err_string);
        abort(1);
    }

    /* Check minimum TE ***********************************/
    temin = p1/2.0 + rof2 + trefmin + risetref + extra_te_error
          + at/2.0 + grate*(agss + agro);
    if (te < temin) {
        sprintf(err_string,"%s: TE too short. Min TE = %f\n",seqfil,temin);
        printf(err_string);
        log_error(err_string);
	abort(1);
    }

    /*******************************************************
    * TREF is the refocussing block length, not including gradient fall
    * time.  TREF is scaled up to a maximum value consistent with TE
    *******************************************************/
    tref = trefmin + 0.5*(te - temin);
    tref = (tref > 2.0*trefmin) ? 2.0*trefmin : tref;

    /* Calculate GPE, adjusting for integral dac value, then adjust TREF */
    sgpe = gpeint/tref;
    sgpe = gmax/gradstepsz*floor(sgpe*gradstepsz/gmax + 0.5);
    sgpe2 = gpe2int/tref;
    sgpe2 = gmax/gradstepsz*floor(sgpe2*gradstepsz/gmax + 0.5);

    /*******************************************************
    * A single solution for tref is not guaranteed to exist for the two
    * values of sgpe and sgpe2.  A good compromise is the geometric mean.
    * If this tref results in a distortion in either PE direction of more
    * than 1%, issue an advisory message.
    *******************************************************/
    tref = sqrt((gpeint/sgpe)*(gpe2int/sgpe2));
    if (nth2D == 1  &&  (fabs(1.0 - sgpe*tref/gpeint) > 0.01  ||
      fabs(1.0 - sgpe2*tref/gpe2int) > 0.01)) {
        sprintf(err_string,"%s: NOTE: lpe distortion =%6.2f%% lpe2 =%6.2f%%\n",
            seqfil,100.0-sgpe*tref/gpeint*100,100.0-sgpe2*tref/gpe2int*100.0);
        printf(err_string);
        log_error(err_string);
    }

    /* If nv is 0 we want a read profile so turn off all phase-encode ***/
    if (nv == 0) {
      sgpe = 0.0;
      sgpe2 = 0.0;
    }

    /* Calculate GSSR and GROR with new TREF **************/
    gror = -grorint/tref*gro/fabs(gro);
    gssr = -gssrint/tref*gss/fabs(gss);

    /* Determine PE block gradient fall time with new TREF */
    risetref = fabs(gror);
    risetref = risetref > fabs(sgpe*nv/2.0) ? risetref : fabs(sgpe*nv/2.0);
    risetref = risetref > fabs(gssr) + fabs(sgpe2*nv2/2.0) ?
               risetref : fabs(gssr) + fabs(sgpe2*nv2/2.0);
    risetref *= grate;

    /* create the shaped gradient waveforms */

    sprintf(shape_name,"gro");
    pw_gro=create_ramp_gradient(agro,shape_name,1,grate,ix);
    pw_gro=create_ramp_gradient(agro,shape_name,-1,grate,ix);

    sprintf(shape_name,"gss");
    pw_gss=create_trap_gradient(agss,shape_name,rof1+p1+rof2,grate,ix);

    sprintf(shape_name,"e_gcr");
    pw_end_gcrush=create_trap_gradient(agcrush,shape_name,tcrush,grate,ix);

    sprintf(shape_name,"gdp");
    temp_val=create_ramp_gradient(risetref/grise*gmax,shape_name,1,grate,ix);
    risetref=create_ramp_gradient(risetref/grise*gmax,shape_name,-1,grate,ix);

    /* tA is the delay between refocussing and readout ****/
    tA = te - (p1/2.0 + rof2 + tref + risetref + at/2.0
       + grate*(agss + agro));

    /* Relaxation delay ***********************************/
    seqtime = te + p1/2.0 + rof1 + at/2.0 + grate*(agss + agro) + tspoil;
    if ((refocus[0] == 'y') || (refocus[0] == 'Y')) {
      seqtime = seqtime + tref + risetref;
    } else {
      seqtime = seqtime + pw_end_gcrush;
    }
    if (rfspoil[0] == 'y')
	seqtime += RFSPOILDELAY;
    /* Spatial Sat Time */
    if (presat==1) seqtime = seqtime + t_presat_b + t_presat_a;
    if (tr < seqtime) {
        sprintf(err_string,"%s: TR too short. Min TR = %f\n",seqfil,seqtime);
        printf(err_string);
        log_error(err_string);
        abort(1);
    }
    predelay = tr - seqtime;

    /*******************************************************
    * Calculation of frequencies to get phase shifts for optional
    * RF spoiling.  Phase shifts are obtained by hopping to variable
    * frequency for fixed time.
    * Ref:  Zur, Y., Magn. Res. Med., 21, 251, (1991).
    *******************************************************/
    if (rfspoil[0] == 'y' && nv > 0) {
	reffreq = resto + sfrq*1e6/B0*gro*pro;
        for (i=1; i<= (int)(nv+0.5); i++) {
	    nextphase = (double)i/2.0*((double)i-1.0)*rfphase;
	    nextphase = nextphase - 360.0*floor(nextphase/360.0);
	    spoilfreqs[i] = reffreq + nextphase/RFSPOILDELAY/360.0;
        }
        create_offset_list(spoilfreqs,nv,TODEV,0);
    }

    /* PULSE SEQUENCE *************************************/

    /* Phase cycle ****************************************/
    mod2(ct,v3);          /* v3: 0  1  0  1  0  1  0  1 */
    dbl(v3,v3);           /* v3: 0  2  0  2  0  2  0  2 */
    hlv(ct,v4);           /* v4: 0  0  1  1  2  2  3  3 */
    mod2(v4,v4);          /* v4: 0  0  1  1  0  0  1  1 */
    add(v3,v4,v1);        /* v1: 0  2  1  3  0  2  1  3 */
    assign(v1,oph);      /* oph: 0  2  1  3  0  2  1  3 */

    if (mp_flash3d_safety_calc() == -1) {
      sprintf(err_string,"%s: Failed duty cycle safety check\n",seqfil);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }

    status(A);
    sp4off();
    rcvron();

    /* Initialialize segmented acquisition counter **********/
    initval(nsteps,v7);
    /* Initialialize steady state counter *******************/
    initval(dummy_ssc,v9);

    /* Wait for trigger and delay 'hold' seconds after it ***/
    xgate(ticks);
    delay(hold);

    /* Compressed Phase-encode loop *************************/
    peloop(seqcon[3],nv2,v11,v12);
    peloop(seqcon[2],nv,v5,v6);

    /* Load gradient table element *************************/
    getelem(t1,v6,v14);

    /* Calculate segment number v8 **************************/
    /* v8 is zero for the first line in each segment ********/
    modn(v6,v7,v8);

    /* Inter-segment delay **********************************/
    /* Execute only on first line of each segment ***********/
    ifzero(v8);
      delay(dseg);
    endif(v8);

    /* Pre-Contrast pulses **********************************/
    /* Execute only on 1st line of each segment *************/
    ifzero(v8);
    /* Inversion pulse **************************************/
      if (ti > 0.0) {
        obsoffset(resto);
        obspower(tpwri);
        sp4on();
        shapedpulse(pipat,pi,zero,rof1,rof2);
        sp4off();
        /* trapezoidal crusher */
        obl_shapedgradient("e_gcr","e_gcr","e_gcr",
                pw_end_gcrush,gcrush,-gcrush,gcrush,1,WAIT);
        delay(ti-pw_end_gcrush);
      }

      if (presat==2) {
        delay(predelay);
        spatial_saturation_pulse();	
        obspower(tpwr1);
        poffset(pss[0],gss);
      }

    endif(v8);
 
    

    /* Optional dummy states ******************************/       
    /* Executed only at the beginning of each segment *****/
    ifzero(v8);
      loop(v9,v10);
      /* RF pulse *******************************************/

      if (presat==1) {
        delay(predelay);
        spatial_saturation_pulse();
        obspower(tpwr1);
        poffset(pss[0],gss);
      } else {
        poffset(pss[0],gss);
        obspower(tpwr1);
        delay(predelay);
      }

      obl_shapedgradient("","","gss",pw_gss,0.0,0.0,gss,1,NOWAIT);
      delay(agss*grate+rftep);
      sp4on();
      shapedpulse(p1pat,p1,v1,rof1,rof2);
      sp4off();
      delay(pw_gss_d);
 
      /* Read dephase & Slice refocus and phase encode pulse ********/
      /* don't need phase_encode3_shapedgradient for the dummy scans */
      phase_encode_shapedgradient("gdp_up",risetref,gror,-sgpe/2.0,
            gssr,-sgpe,v14,nv/2,psi,phi,theta,one,WAIT,0);
      delay(tref-risetref);
      phase_encode_shapedgradient("gdp_dn",risetref,gror,-sgpe/2.0,
            gssr,-sgpe,v14,nv/2,psi,phi,theta,one,WAIT,1);

      /* Pre-acquire delay **********************************/
      poffset(pro,gro);
      delay(tA);
 
      /* Acquire dummy echo *********************************/
      obl_shapedgradient("gro_up","","",pw_gro,gro,0.0,0.0,1,WAIT);
      delay(0.5/fb + at + tspoil);
      obl_shapedgradient("gro_dn","","",pw_gro,gro,0.0,0.0,1,WAIT);

      /* Optional phase encode gradient refocussing *******************/
      if ((refocus[0] == 'y') || (refocus[0] == 'Y')) {
        /* don't need phase_encode3_shapedgradient for the dummy scans */
        phase_encode_shapedgradient("gdp_up",risetref,0.0,sgpe/2.0,
            0.0,sgpe,v14,nv/2,psi,phi,theta,one,WAIT,0);
        delay(tref-risetref);
        phase_encode_shapedgradient("gdp_dn",risetref,0.0,sgpe/2.0,
            0.0,sgpe,v14,nv/2,psi,phi,theta,one,WAIT,1);
      } else {
        obl_shapedgradient("e_gcr","e_gcr","e_gcr",
                pw_end_gcrush,gcrush,-gcrush,gcrush,1,WAIT);
      }

      /* double check that all gradients are off */
      zero_all_gradients();

      /*******************************************************
      * Optional RF spoiling.  REMINDER: Do not place any events
      * after this block or before poffset at the start of peloop.
      *******************************************************/
      if (rfspoil[0] == 'y' && nv > 0) {
        voffset(0,v6);
        delay(RFSPOILDELAY - OFFSETDELAY);
      }
      endloop(v10);
    endif(v8);
      
      /* Now collect the real data */
      /* RF pulse *******************************************/ 

      if (presat==1) {
        delay(predelay);
        spatial_saturation_pulse();
        obspower(tpwr1);
        poffset(pss[0],gss);
      } else {
        poffset(pss[0],gss);
        obspower(tpwr1);
        delay(predelay);
      }

      sp1on();  /* put out trigger on sp1 every spin excitation */
      obl_shapedgradient("","","gss",pw_gss,0.0,0.0,gss,1,NOWAIT);
      delay(agss*grate+rftep);
      sp4on();
      shapedpulse(p1pat,p1,v1,rof1,rof2);
      sp4off();
      delay(pw_gss_d);
      sp1off();

      /* Read dephase & Slice refocus and phase encode pulse ********/
      pe2_shapedgradient("gdp_up",risetref, gror,
          -sgpe/2.0,-sgpe2*nv2/2.0+sgpe2/2.0+gssr,-sgpe,sgpe2,v14,v12);
      delay(tref-risetref);
      pe2_shapedgradient("gdp_dn",risetref, gror,
          -sgpe/2.0,-sgpe2*nv2/2.0+sgpe2/2.0+gssr,-sgpe,sgpe2,v14,v12);

      /* Pre-acquire delay **********************************/
      poffset(pro,gro);
      delay(tA);

      /* Acquire echo ***************************************/
      obl_shapedgradient("gro_up","","",pw_gro,gro,0.0,0.0,1,WAIT);
      delay(0.5/fb+tep);
      acquire(np,1.0/sw);
      delay(tspoil-tep);
      obl_shapedgradient("gro_dn","","",pw_gro,gro,0.0,0.0,1,WAIT);

      /* Optional gradient refocussing **********************/ 
      if ((refocus[0] == 'y') || (refocus[0] == 'Y')) {

      pe2_shapedgradient("gdp_up",risetref, 0.0,
          sgpe/2.0,sgpe2*nv2/2.0-sgpe2/2.0,sgpe,-sgpe2,v14,v12);
      delay(tref-risetref);
      pe2_shapedgradient("gdp_dn",risetref, 0.0,
          sgpe/2.0,sgpe2*nv2/2.0-sgpe2/2.0,sgpe,-sgpe2,v14,v12);

      } else {
        obl_shapedgradient("e_gcr","e_gcr","e_gcr",
                pw_end_gcrush,gcrush,-gcrush,gcrush,1,WAIT);
      }

      /* double check that all gradients are off */
      zero_all_gradients();

      /*******************************************************
      * Optional RF spoiling.  REMINDER: Do not place any events
      * after this block or before poffset at the start of peloop.
      *******************************************************/
      if (rfspoil[0] == 'y' && nv > 0) {
        voffset(0,v6);
        delay(RFSPOILDELAY - OFFSETDELAY);
      }

    endpeloop(seqcon[2],v6);
    endpeloop(seqcon[3],v12);
  rcvroff();
}
