#ifndef LINT
static char SCCSid[] = "@(#)flash.c 11.1 05/07/96 Copyright (c) 1994-1996 Varian Assoc.,Inc. All Rights Reserved";
#endif
/***********************************************************************

    ox_ecc_cal.c

    eddy current sequence.                     

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

#include "standard.h"
#include "math.h"
#include "group.h"
#include "sequence_utilities.h"
#define OFFSETDELAY 26e-6
#define RFSPOILDELAY 50e-6

/*==========================================================================*
  define the pulse sequence variables outside the pulse sequence so that
  they can be seen as global variables by the safety monitoring code
*==========================================================================*/
    double  predelay,seqtime,tref,risetref,grise,tep,rftep,at_des;
    double  trefmin,tssrmin,trormin,tpemin,temin,tA;
    double  agro,agss,sgpe,grate,gssrint,grorint,gpeint;      
    double  reffreq,nextphase,spoilfreqs[1024],pw_gss_d,extra_te_error;
    double  pw_trap,pw_gss,wave_time,true_wave_time,greadfc,gslicefc;
    double  grad_up_time, tmp_grate, pw_ramp1, pw_ramp2;
    double  flat_top,pw_gssr,gssr,ramp1,ramp2;
    double  pw_blip;
    int     i,image=0,dummy_ssc;
    char    shape_name[MAXSTR],flow_comp[MAXSTR],err_string[MAXSTR];
    char    gcoil[MAXSTR];

pulsesequence()
{

    double  create_ge_slice_flow_comp(),gread;
    double  create_trap_gradient(), create_ramp_gradient();
    double  create_ge_read_flow_comp(), create_ge_phase_flow_comp();

    initparms_sis();
    loop_check();

    /*************************************************************************/
    /* get the global parameters */
    /*************************************************************************/
    P_getreal(GLOBAL,"grise",&grise,1); /* grad risetime to full strength */
    if ((grise < 0.0) || (grise > 0.002)) {
      sprintf(err_string,"%s: impropper grise time defined in global file\n",
                       seqfil);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
    P_getreal(GLOBAL,"tep",&tep,1); /* small tweaker time (us) to allow for
                            the gradient propagation time, causes echoes to
                            appear symmetrically about echo center line */
    tep=tep/1e6;
    P_getreal(GLOBAL,"rftep",&rftep,1); /* small tweaker time (us) to allow for
                            the gradient propagation time, w.r.t. RF pulses */
    rftep=rftep/1e6;
    /*************************************************************************/

    getstr("gcoil",gcoil);          /* gradient coil in use */
    image = (int)(getval("image"));
    gread = getval("gread");
    /* values for image: */
    /* 1 = read direction control pulse */
    /* 2 = phase direction control pulse */
    /* 3 = slice direction control pulse */
    /* 4 = trapezoid (flat top duration d1) along read before sequence */
    /* 5 = trapezoid (flat top duration d1) along phase before sequence */
    /* 6 = trapezoid (flat top duration d1) along slice before sequence */

    grate = grise/gmax;

    agss = fabs(gss);
    if (agss == 0.0) {
        sprintf(err_string,"%s: Zero slice select gradient\n",seqfil);
        printf(err_string);
        log_error(err_string);
        abort(1);
    }

    sprintf(shape_name,"gss");
    pw_gss=create_trap_gradient(fabs(gss),shape_name,rof1+p1+rof2,grate,ix);
    ramp1 = (pw_gss-rof1-p1-rof2)/2;
    flat_top = (pw_gss/2) - (3*ramp1/2);
    if(flat_top<0){
      gssr = sqrt((pw_gss*ramp1)-1);
      flat_top = 0;
    } else {
      gssr = gss;
    }

    sprintf(shape_name,"gssr");
    pw_gssr=create_trap_gradient(fabs(gssr),shape_name,flat_top,grate,ix);
    ramp2 = (pw_gssr-flat_top)/2;

    gssr = (gss*(pw_gss-ramp1)/2)/(pw_gssr-ramp2);

    pw_gss_d=agss*grate-rftep;
    if (pw_gss_d < 0.0) pw_gss_d=0.0;

    sprintf(shape_name,"ramp");
    grad_up_time=getval("grad_up_time");
    tmp_grate=grad_up_time/fabs(gread);
    pw_ramp1=create_ramp_gradient(fabs(gread),shape_name,1,tmp_grate,ix);
    pw_ramp2=create_ramp_gradient(fabs(gread),shape_name,-1,grate,ix);

    printf("grad_delay=%fms\n", (d2+pw_gss+pw_gssr+d3)*1000.0);

    /* PULSE SEQUENCE *************************************/

    /* Phase cycle ****************************************/
    mod2(ct,v3);          /* v3: 0  1  0  1  0  1  0  1 */
    dbl(v3,v3);           /* v3: 0  2  0  2  0  2  0  2 */
    hlv(ct,v4);           /* v4: 0  0  1  1  2  2  3  3 */
    mod2(v4,v4);          /* v4: 0  0  1  1  0  0  1  1 */
    add(v3,v4,v1);        /* v1: 0  2  1  3  0  2  1  3 */
    assign(v1,oph);      /* oph: 0  2  1  3  0  2  1  3 */

    status(A);
    sp4off();

    sp2on();
    delay(0.00005);  /* 50 us trigger to indicate start of acq */
    sp2off();

    delay(tr);

    if (image == 4) {
      obl_shapedgradient("ramp_up","","",pw_ramp1,gread,0.0,0.0,1,WAIT);
      obl_shapedgradient("ramp_dn","","",pw_ramp2,gread,0.0,0.0,1,WAIT);
    }
    if (image == 5) {
      obl_shapedgradient("","ramp_up","",pw_ramp1,0.0,gread,0.0,1,WAIT);
      obl_shapedgradient("","ramp_dn","",pw_ramp2,0.0,gread,0.0,1,WAIT);
    }
    if (image == 6) {
      obl_shapedgradient("","","ramp_up",pw_ramp1,0.0,0.0,gread,1,WAIT);
      obl_shapedgradient("","","ramp_dn",pw_ramp2,0.0,0.0,gread,1,WAIT);
    }
    if ((image == 1) || (image == 2) || (image == 3)) {
      delay(pw_ramp1+pw_ramp2);
    }

    obspower(tpwr1);
    /* Relaxation delay ***********************************/       
    poffset(pss[0],gss);

    delay(d2);

    /* RF pulse *******************************************/ 
    sp1on(); /* put out trigger on sp1 every spin excitation */
    if ((image == 1) || (image == 4) || (image == 7)) {
      obl_shapedgradient("gss","","",pw_gss,gss,0.0,0.0,1,NOWAIT);
    } 
    if ((image == 2) || (image == 5) || (image == 8)) {
      obl_shapedgradient("","gss","",pw_gss,0.0,gss,0.0,1,NOWAIT);
    } 
    if ((image == 3) || (image == 6) || (image == 9)) {
      obl_shapedgradient("","","gss",pw_gss,0.0,0.0,gss,1,NOWAIT);
    }
    delay(agss*grate+rftep);
    sp4on();
    shapedpulse(p1pat,p1,v1,rof1,rof1);
    sp4off();
    delay(pw_gss_d);
    sp1off();
    if ((image == 1) || (image == 4) || (image == 7)) {
      obl_shapedgradient("gssr","","",pw_gssr,-gssr,0.0,0.0,1,WAIT);
    }
    if ((image == 2) || (image == 5) || (image == 8)) {
      obl_shapedgradient("","gssr","",pw_gssr,0.0,-gssr,0.0,1,WAIT);
    }
    if ((image == 3) || (image == 6) || (image == 9)) {
      obl_shapedgradient("","","gssr",pw_gssr,0.0,0.0,-gssr,1,WAIT);
    }

    delay(d3);

    poffset(0.0,gread);

    acquire(np,1.0/sw);

    zero_all_gradients(); /* double check that all gradients are zero */

}

