#ifndef LINT
static char SCCSid[] = "@(#)sems.c 11.1 05/07/96 Copyright (c) 1994-1996 Varian Assoc.,Inc. All Rights Reserved";
#endif
/* " (c) 1992 Spectroscopy Imaging Systems" */
/***********************************************************
    OX_SEMS.C

    Standard multi-slice spin-echo oblique imaging sequence.

    Based on: SEMS.C
    VERSION 3.0        6/20/94  
    Revisions:
    3.1   2/96    Allow negative gradients

    adapted by P. Jezzard, University of Oxford

    added inversion pulse (PJ Nov 1997)
    added shaped gradient pulses (PJ Jan 1998)
    added capability to do an asymmetric spin echo (PJ Jan 1998)
    moved chopper phase cycling outside the multi-slice loop (PJ Jan 1998)
    general overhaul, added proper 180 slice select (PJ Aug 1999)

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
     __                                                            __ 
    |        p1                       p2                             |
RF  |------ [90] ------------------- [180] ------------- |** ACQ **| |
    |                                                                |
    |         < - - - - - - - - - - - te - - - - - - - - - - >       |
    |   gss ____           gss2/\___________///\                     |
GSS |______/////\_____________//////////////////\____________________|
    |<---->                   < - >         < - > tspoil_se2         |
    |  predelay  <--> tref                                           | ns
    |             ___                                                | 
GPE |____________/___\_______________________________________________|
    |            \___/                                               |
    |                                                                |
    |        gror ___                           gro ________________ |
GRO | ___________////\_____________________________/////////////////\|
    |                                       tspoil < - >             |
    |__                                                            __|

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

#include <standard.h>
#include "group.h"
#include "sequence_utilities.h"

/*==========================================================================*
  define the pulse sequence variables outside the pulse sequence so that
  they can be seen as global variables by the safety monitoring code
*==========================================================================*/

    extern  codeint v15,v16,v17,v19;
    double  predelay,seqtime,tref,asym_time;
    double  trefmin,tssrmin,trormin,temin,tau1,tau2;
    double  agro,agss,agssi,sgpe,grate,gssrint,grorint,gcrush,agcrush,agssc;
    double  agss2,tspoil_se1,tspoil_se2,crush_ramp_time,gssr,gspoil,agspoil;
    double  pw_gread,pw_gss,pw_gssi,pw_gssr,pw_gssc,pw_gpe,pw_end_gcrush;
    double  gssc,agssr,grise,rftep,tep,pw_gss_d,extra_te_error,gssi,gss2;
    char    shape_name[MAXSTR], flow_comp[MAXSTR], err_string[MAXSTR];
    char    gcoil[MAXSTR];
    int     exlist,invlist,selist,curlist;
    double  subset_ns[512], temp_ns, slices_per_sec, last_tr_delay[512];
    int     num_tr_blocks, n_tr_slices, i, seq_debug, predelay_list;
    char    full_fname[MAXSTR],*home_dir;
    FILE    *table;

#include "ox_sems_safety_calc.h"

/* Spatial Sat declarations */
double gsssat,pw_gsssat;
double presat_off=0.0,post_presat_del,sat_thk,sat_psi,sat_phi,sat_theta;
double presat=0.0,pw_gcrush;
double t_presat_b,t_presat_a;

calculate_spatial_saturation_gradients(double *t_presat_b,double *t_presat_a)
{
  char shape_name[MAXSTR];

  sprintf(shape_name,"gsat");
  pw_gsssat = create_trap_gradient(fabs(gsssat),shape_name,
         psat+rof1+rof2,grate,ix);

  *t_presat_b = fabs(gsssat)*grate + rof1 + psat/2;
  *t_presat_a = fabs(gsssat)*grate + rof2 + psat/2 + post_presat_del;
  pw_gcrush=0.0;
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
    sprintf(shape_name,"gcr");
    pw_gcrush = create_ramp_gradient(fabs(gcrush),shape_name,1,grate,ix);
    pw_gcrush = create_ramp_gradient(fabs(gcrush),shape_name,-1,grate,ix);
    if (pw_gcrush > tcrush) {
      sprintf(err_string,"%s: tcrush too small. Min tcrush = %f\n",
              seqfil,pw_gcrush);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
    *t_presat_a=*t_presat_a + 2.0*pw_gcrush + tcrush;
  }
}

spatial_saturation_pulse()
{
  /* uses psat : satpwr : satpat : gsssat */
  /* also uses sat_psi, sat_phi and sat_theta rather than psi, phi, theta */
  /* Enables definition of independent sat planes relative to slice plane */
  position_offset(presat_off,gsssat,resto,OBSch);
  obspower(satpwr);

  oblique_shapedgradient("","","gsat",pw_gsssat,0.0,0.0,gsssat,sat_psi,sat_phi
,sat_theta,1,NOWAIT);

  delay(fabs(gsssat)*grate+rftep);
  sp4on();
  shaped_pulse(satpat,psat,v1,rof1,rof2);
  sp4off();
  if(rftep<fabs(gsssat)*grate)delay(fabs(gsssat)*grate-rftep);

  /* Crushers */
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
    obl_shapedgradient("gcr_up","gcr_up","gcr_up",
                       pw_gcrush,gcrush,gcrush,gcrush,1,WAIT);
    delay(tcrush);
    obl_shapedgradient("gcr_dn","gcr_dn","gcr_dn",
                       pw_gcrush,gcrush,gcrush,gcrush,1,WAIT);
  }
  delay(post_presat_del);
}

pulsesequence()
{

    /* PARAMETER INITIALIZATION ***************************/
    initparms_sis();

    /*************************************************************************/
    /* get the global parameters */
    /*************************************************************************/
    P_getreal(GLOBAL,"grise",&grise,1); /* grad risetime to full strength */
    if ((grise < 0.0) || (grise > 0.002)) {
      sprintf(err_string,"%s: impropper grise time defined in global file\n",
                       seqfil);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
    P_getreal(GLOBAL,"tep",&tep,1); /* small tweaker time (us) to allow for
                            the gradient propagation time, causes echoes to
                            appear symmetrically about echo center line */
    tep=tep/1e6;
    P_getreal(GLOBAL,"rftep",&rftep,1); /* small tweaker time (us) to allow for
                            the gradient propagation time, w.r.t. RF pulses */
    rftep=rftep/1e6;
    /*************************************************************************/

    getstr("gcoil",gcoil);          /* gradient coil in use */
    getstr("flow_comp",flow_comp);  /* flow comp flag 'y'=fc. 'n'=no fc. */
    gcrush = getval("gcrush");      /* phase direction crusher */
    gspoil = getval("gspoil");      /* slice select spoiler */
    asym_time = getval("asym_time"); /* Asymmetric spin echo time */
    slices_per_sec = getval("slices_per_sec"); /* max slices per second */
    gssi = getval("gssi");          /* Inversion gradient strength */
    gss2 = getval("gss2");          /* 180 slice select gradient strength */

    agcrush = fabs(gcrush);

    grate = grise/gmax;

        /* Spatial Sat Variables */
    presat=getval("presat");
    if (presat) {
      gsssat = getval("gsssat");
      presat_off=getval("presat_off");
      sat_psi=getval("sat_psi");
      sat_phi=getval("sat_phi");
      sat_theta=getval("sat_theta");
      sat_thk=getval("sat_thk");
      post_presat_del=getval("post_presat_del");
    }
    if (presat) calculate_spatial_saturation_gradients(&t_presat_b,&t_presat_a);

    agspoil = fabs(gspoil);
    agss = fabs(gss);
    agss2 = fabs(gss2);
    agssi = fabs(gssi);
    agro = fabs(gro);

    exlist=invlist=selist=predelay_list=-1;
    curlist=0;

    if (agss == 0.0) {
        sprintf(err_string,"%s: Zero gss slice select gradient\n",seqfil);
        printf(err_string);
        log_error(err_string);
        abort(1);
    }

    if (agss2 == 0.0) {
        sprintf(err_string,"%s: Zero gss2 slice select gradient\n",seqfil);
        printf(err_string);
        log_error(err_string);
        abort(1);
    }

    sprintf(shape_name,"gss");
    pw_gss=create_trap_gradient(agss,shape_name,rof1+p1+rof2,grate,ix);

    sprintf(shape_name,"gssi");
    pw_gssi=create_trap_gradient(agssi,shape_name,rof1+p3+rof2+tspoil,grate,ix);

    sprintf(shape_name,"gread");
    pw_gread=create_trap_gradient(agro,shape_name,at+tspoil,grate,ix); 
        /* read trap */

    pw_end_gcrush=0.0;
    if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
      sprintf(shape_name,"e_gcr");
      pw_end_gcrush=create_trap_gradient(agcrush,shape_name,tcrush,grate,ix);
    }

    /* need to send the bigger of gspoil and gss2 to the obl_shapedgradient
       playout, since waveform is scaled to 32767 max */
    agssc = fabs(gspoil) > fabs(gss2) ? fabs(gspoil) : fabs(gss2);

    /* calculate phase encode and read rephaser gradient ramps */
    /* allow for up to a max gradient strength, and thus grise duration */
    sprintf(shape_name,"gpe");
    pw_gpe=create_ramp_gradient(fabs(gmax),shape_name,1,grate,ix);  /* up */
    pw_gpe=create_ramp_gradient(fabs(gmax),shape_name,-1,grate,ix); /* down */

    /****
    * Calculate minimum time for phase-encoding, slice refocussing
    * and readout prefocussing block.
    ****/
    /* GRADIENT*TIME INTEGRALS FOR SLICE SELECTION AND READOUT */
    grorint = agro*(at/2.0 + tspoil + grate*agro/2.0);
    gssrint = agss*(p1/2.0 + rof2 + grate*agss/2.0);
		   
    /* MINIMUM GRADIENT TIMES.  TRAPEZOIDAL IF GMAX IS EXCEEDED */
    if (grise*gmax >= grorint)
        trormin = 2.0*sqrt(grate*grorint);
    else
        trormin = grate*gmax + grorint/gmax;

    /* also calc tsrrmin, which is still used in flow_comp version */
    if (grise*gmax >= gssrint)
        tssrmin = 2.0*sqrt(grate*gssrint);
    else
        tssrmin = grate*gmax + gssrint/gmax;

    /* MINIMUM REFOCUSSING BLOCK TIME IS THE LONGEST OF CALCULATED MINIMA */
    /* note that tpe is calculated in imprep */
    trefmin = trormin > tpe ? trormin : tpe;
    trefmin = trefmin > tssrmin ? trefmin : tssrmin;
    trefmin = trefmin > 2.0*grise ? trefmin : 2.0*grise;

    /* account for case of rftep being longer than the gradient ramp time */
    pw_gss_d = agss*grate - rftep;
    extra_te_error = 0.0;
    if (pw_gss_d < 0.0) {
      pw_gss_d = 0.0;
      extra_te_error = rftep-agss*grate;
    }

    /* CALCULATE 180 slice gradient and 90 refocus */
    if (gspoil <= 0.0) {
        sprintf(err_string,"%s: gspoil must be a positive number\n",seqfil);
        printf(err_string);
        log_error(err_string);
        abort(1);
    }
    if ((flow_comp[0] == 'y') || (flow_comp[0] == 'Y')) {
      tspoil_se1=tspoil;
      tspoil_se2=tspoil;
    }
    else {
      tspoil_se1=0.0;
      tspoil_se2=gssrint/gspoil;
    }
    sprintf(shape_name,"gssc");
    pw_gssc=create_spin_echo_gradient(shape_name,gss2,rof1+p2+rof2,gspoil,
          tspoil_se1,tspoil_se2,grate,ix);
    crush_ramp_time=(pw_gssc-(rof1+p2+rof2+tspoil_se1+tspoil_se2))/4.0;

    if (crush_ramp_time*2.0+tspoil_se2 < rftep) {
        sprintf(err_string,"%s: tspoil or gspoil too small\n", seqfil);
        printf(err_string);
        log_error(err_string);
        abort(1);
    }
	       
    /* TAU1 AND TAU2 ARE THE SUMS OF ALL EVENTS IN EACH HALF-ECHO PERIOD */
    tau1 = p1/2.0 + rof2 + grate*agss + extra_te_error + trefmin + pw_gpe 
         + crush_ramp_time*2.0 +tspoil_se1 + p2/2.0 + rof1;
    if ((flow_comp[0] == 'y') || (flow_comp[0] == 'Y')) {
      tau1 = tau1 + 2.0*grate*agss + p1 + rof2*2.0;
    }
    tau2 = p2/2.0+rof2+crush_ramp_time*2.0+tspoil_se2+grate*agro+tspoil+at/2.0;
    if ((flow_comp[0] == 'y') || (flow_comp[0] == 'Y')) {
      tau2 = tau2 + 3.0*at/2.0 + 7.0*agro*grate/2.0 + 3.0*tspoil;
    }
    temin = tau1 < tau2 ? 2.0*tau2 : 2.0*tau1;
    if (te < temin) {
        sprintf(err_string,"%s: TE too short. Min TE = %f\n",seqfil,temin);
        printf(err_string);
        log_error(err_string);
        abort(1);
    }

    /* TREF IS THE REFOCUSSING BLOCK LENGTH, NOT INCLUDING GRADIENT FALL TIME */
    /* TREF IS SCALED UP TO A MAXIMUM VALUE CONSISTENT WITH tau1 */
    tref = trefmin + 0.8*(te/2.0 - tau1);
    tref = (tref > 2.0*trefmin) ? 2.0*trefmin : tref;
		 
    /* CALCULATE GPE, ADJUSTING FOR INTEGRAL DAC VALUE, THEN RE-ADJUST TREF */
    sgpe = gpe*tpe/tref;
    sgpe = gmax/gradstepsz*floor(sgpe*gradstepsz/gmax + 0.5);
    tref = gpe*tpe/sgpe;
    gssr = gssrint/tref;  /* still used in flow_comp */
    tau1 = tau1 - trefmin + tref;
				
    /* CALCULATE GROR WITH NEW TREF */
    gror = grorint/tref*gro/fabs(gro);

    if (ti > 0.0) {
      if (ti < (2*grate*agssi+p3+tspoil+pw_end_gcrush)) {
        sprintf(err_string,"%s: TI too short. Min TI = %f\n",seqfil,
                          2*grate*agssi+p3+tspoil+pw_end_gcrush);
        printf(err_string);
        log_error(err_string);
        abort(1);
      }
    }

    /* RELAXATION DELAY */
    seqtime = p1/2.0 + rof1 + te + at/2.0 + agro*grate + asym_time 
         + pw_end_gcrush;
    if (ti > 0.0) {
      seqtime = seqtime + agssi*grate+p3/2+ti;
    }
    if (presat) seqtime = seqtime + t_presat_b + t_presat_a;

    subset_ns[0]=ns;   /* default */
    if (seqcon[1] == 'c') {  /* set defaults */
        last_tr_delay[0] = (tr - subset_ns[i]*seqtime)/subset_ns[i];
    }
    else {
        last_tr_delay[0] = tr - seqtime;
    }
    num_tr_blocks=1.0; /* default */
    
    if (slices_per_sec > 0.0) {
      n_tr_slices = (int)(floor((tr)/seqtime)+0.01);
                                                /* .01 for round off error */
      if ((float)n_tr_slices/tr > slices_per_sec) {
        n_tr_slices=floor(slices_per_sec*tr+0.01);
      }
      num_tr_blocks = (int)(ceil(ns/n_tr_slices)+0.01);
      temp_ns=ns;
      for (i = 0; i < num_tr_blocks; i++) {
        if ((temp_ns - (double)n_tr_slices) > 0.0) {
          subset_ns[i]=(double)n_tr_slices;
          temp_ns=temp_ns-(double)n_tr_slices;
        } else {
          subset_ns[i]=temp_ns;
        }
        if (seqcon[1] == 'c') {
            last_tr_delay[i] = (tr - subset_ns[i]*seqtime)/subset_ns[i];
        }
        else {
            last_tr_delay[i] = tr - seqtime;
        }
      }
    }

    if (seqcon[1] == 'c') {
        seqtime=subset_ns[0]*seqtime;
    }
    if (last_tr_delay[0] < 0.0) {
        sprintf(err_string,"%s: TR too short. Min TR = %f\n",seqfil,seqtime);
        printf(err_string);
        log_error(err_string);
        abort(1);
    }

    /* Now create the lists in memory */
    if (predelay_list<0) predelay_list=curlist++;
    create_delay_list(last_tr_delay,num_tr_blocks,predelay_list);
    home_dir=(char *)getenv("HOME");
    sprintf(full_fname,"%s/vnmrsys/tablib/ns_table",home_dir);
    table=fopen(full_fname,"w");
    if(table==NULL){
      sprintf(err_string,"%s: Unable to open ~/vnmrsys/tablib/ns_table\n",
              seqfil);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
    fprintf(table,"t32 = ");
    for(i=0;i<num_tr_blocks;i++){
      fprintf(table,"%d ",(int)subset_ns[i]);
    }
    fprintf(table,"\n");
    fclose(table);
    loadtable("ns_table");

    if (ox_sems_safety_calc() == -1) {
      sprintf(err_string,"%s: Failed duty cycle safety check\n",seqfil);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }

    printf("num_tr_blocks = %d\n",num_tr_blocks);

seq_debug=0;
if (seq_debug == 1) {
  printf("\n\n debug\n");
  printf("n_tr_slices = %d\n",n_tr_slices);
  printf("seqtime=%f\n",seqtime/subset_ns[0]*1000.0);
  for (i=0; i<num_tr_blocks; i++) {
    printf("subset_ns[%d]=%f\n",i,subset_ns[i]);
    printf("last_tr_delay[%d]=%f\n",i,last_tr_delay[i]*1000.0);
  }
}
    
    /* PULSE SEQUENCE *************************************/

    /* Phase cycle: Part I, Cyclops quadrature phase ******/
    mod2(ct,v3);                    /* v3: 0101 0101 */
    dbl(v3,v3);                     /* v3: 0202 0202 */
    hlv(ct,v4);                     /* v4: 0011 2233 */
    mod2(v4,v4);                    /* v4: 0011 0011 */
    add(v3,v4,v1);                  /* v1: 0213 0213 */
    assign(v1,oph);                 /* p1 and rcvr phase */
    assign(one,v7);                 /* phase encode increment size */

    initval(num_tr_blocks,v15);  /* set v15 to the num of ti blocks */
    initval(0.0,v10);   /* v10 holds a zero for assignment */

    peloop(seqcon[2],nv,v5,v6);

     /* Phase cycle: Part II, p2 alternates each phase step */
     /* to move non-phase-encoded residual fid from 180 *****/
     /* out of image center to phase-encoding edges. ********/
     mod2(v6,v4);
     ifzero(v4); 
       add(one,v1,v2);           /* v2 = v1 + 90: 1324 */
     elsenz(v4);                 /*    or              */
       add(three,v1,v2);         /* v2 = v1 - 90: 3546 */
     endif(v4);

     /* Alternate v2 phase each transient to cancel ********/       
     /* residual FID from 180 ******************************/       
     add(v3,v2,v2);              /* v2: 1526 = 1122 or 3748 = 3300*/

     assign(v10,v17);   /* "current slice" counter in ms loop back to 0 */

     /* loop v15 times (num_tr_blocks) using counter v16 */
     loop(v15,v16);

     getelem(t32,v16,v19);

      /* replaces msloop, subset_ns slices using counter v14 */
      loop(v19,v14);

        /* Relaxation delay ***********************************/       
        status(A);
        sp4off();
        vdelay_list(predelay_list,v16);
       
        if (ti > 0.0) {
          if (invlist<0) invlist=curlist++;
          position_offset_list(pss,gssi,ns,resto,OBSch,invlist,v17);
          /* inversion recovery */
          /* 180 degree RF pulse ********************************/
          obspower(tpwr3);
          obl_shapedgradient("","","gssi",pw_gssi,0.0,0.0,gssi,1,NOWAIT);
          delay(agssi*grate+rftep);
          sp4on();
          shapedpulse(p3pat,p3,v1,rof1,rof1);
          sp4off();
          delay(agssi*grate+tspoil-rftep);
          if (pw_end_gcrush > 0.0) {
            obl_shapedgradient("","e_gcr","",pw_end_gcrush,0.0,gcrush,
                0.0,1,WAIT);
          }
          delay(ti-grate*agssi-p3/2-tspoil-pw_end_gcrush);
        }

        if (presat) spatial_saturation_pulse();

        /* 90 degree RF pulse *********************************/ 
        if (exlist<0) exlist=curlist++;
        position_offset_list(pss,gss,ns,resto,OBSch,exlist,v17);
        obspower(tpwr1);
        xgate(ticks);
        obl_shapedgradient("","","gss",pw_gss,0.0,0.0,gss,1,NOWAIT);
        delay(agss*grate+rftep);
        sp4on();
        shapedpulse(p1pat,p1,v1,rof1,rof1);
        sp4off();
        delay(pw_gss_d);

        if ((flow_comp[0] == 'y') || (flow_comp[0] == 'Y')) {
          obl_gradient(0.0,0.0,-gss);
          delay(grate*agss+p1+2.0*rof2);
          zero_all_gradients();
          delay(grate*agss);
          /* Read & Slice refocus and phase encode gradients ****/
          phase_encode_shapedgradient("gpe_up",pw_gpe,0.0,-0.5*sgpe*nv,gssr,
            sgpe,v6,nv,psi,phi,theta,v7,WAIT,0);
          delay(tref-pw_gpe);
          phase_encode_shapedgradient("gpe_dn",pw_gpe,0.0,-0.5*sgpe*nv,gssr,
            sgpe,v6,nv,psi,phi,theta,v7,WAIT,1);
        } else {
          /* Read and phase encode gradients */
          phase_encode_shapedgradient("gpe_up",pw_gpe,gror,-0.5*sgpe*nv,0.0,
            sgpe,v6,nv,psi,phi,theta,v7,WAIT,0);
          delay(tref-pw_gpe);
          phase_encode_shapedgradient("gpe_dn",pw_gpe,gror,-0.5*sgpe*nv,0.0,
            sgpe,v6,nv,psi,phi,theta,v7,WAIT,1);
        }

        delay(te/2.0 - tau1);

        /* 180 degree RF pulse ********************************/
        if (selist<0) selist=curlist++;
        position_offset_list(pss,gss2,ns,resto,OBSch,selist,v17);
        obspower(tpwr2);
        obl_shapedgradient("","","gssc",pw_gssc,0.0,0.0,agssc,1,NOWAIT);
        delay(2.0*crush_ramp_time+tspoil_se1+rftep);
        sp4on();
        shapedpulse(p2pat,p2,v2,rof1,rof2);
        sp4off();
        delay(2.0*crush_ramp_time+tspoil_se2-rftep);

        delay(te/2.0 - tau2);

        /* add flow comp in read direction */
        if ((flow_comp[0] == 'y') || (flow_comp[0] == 'Y')) {
          obl_gradient(gro,0.0,0.0);
          delay(tspoil+at/2+grate*agro/2);
          zero_all_gradients();
          delay(grate*agro);
          obl_gradient(-gro,0.0,0.0);
          delay(grate*agro+2.0*tspoil+at);
          zero_all_gradients();
          delay(grate*agro);
        }

        /* Acquire echo ***************************************/
        poffset(pro,gro);
        delay(asym_time);  /* put asym_time after the poffset statement so
                              that PTS phase accumulation is same for pure
                              spin echo and asymmetric spin echo sequences.
                              This allows phase unwrapping for field maps */
        obl_shapedgradient("gread","","",pw_gread,gro,0.0,0.0,1,NOWAIT);
        delay(agro*grate+tspoil+tep);
        acquire(np,1.0/sw);
        delay(agro*grate);

        if (pw_end_gcrush > 0.0) {
          obl_shapedgradient("","e_gcr","",pw_end_gcrush,0.0,gcrush,
                0.0,1,WAIT);
        }

        zero_all_gradients();  /* double check that the gradients are off */

      incr(v17);  /* incr the v17 counter since a slice has been collected */

      /* increment the msloop counter v14 */
      endloop(v14);

     /* increment the tr_block loop counter v16 */
     endloop(v16);

    endpeloop(seqcon[2],v6);
}
