#ifndef LINT
static char SCCSid[] = "@(#)pressi.c 13.1 10/10/97 Copyright (c) 1991-1995 Varian Assoc.,Inc. All Rights Reserved";
#endif
/* 
 * Varian Assoc.,Inc. All Rights Reserved.
 * This software contains proprietary and confidential
 * information of Varian Assoc., Inc. and its contributors.
 * Use, disclosure and reproduction is prohibited without
 * prior consent.
 */
/***********************************************************
    PRESSI.C

    Volume localized imaging using the double spin
    echo PRESS technique
************************************************************/

#include <standard.h>

pulsesequence()
{
    /* INTERNAL VARIABLE DECLARATIONS *********************/
    double tau1,tau2,tau3;          /*Timing balance delays*/
    double tref;	            /*gradient refocus delays*/
    double gvox1r;                  /*refocus gradients*/
    double grate;		    /*gradient slew rate*/
    double te2;	                    /*te for second spin-echo*/
    double seqtime,predelay;

    /* INTERNAL VARIABLE DECLARATION FOR IMAGING **********/
    double  trefi,risetref;
    double  trefmin,tssrmin,trormin,tpemin,temin;
    double  agro,agss,sgpe,gssrint,grorint,gpeint;
    double  tspoili, tau3b;

    /* PARAMETER INITIALIZATION ***************************/
    initparms_sis();
    te2 = getval("te2");

    /* Check some flags **********************************/
    if  (ws[0] == 'y') {
        printf(" ws flag must be set to 'n'\n");
        abort(1);
    }

    /* CALCULATE IMAGING PARAMETERS ***********************/
    grate = trise/gmax;
    agro = fabs(gro);
    agss = fabs(gss);

    /* GRADIENT*TIME integrals for slice selection and readout */
    tspoili = 0;   /* read gradient spoiler */
    grorint = agro*(at/2.0 + tspoili + grate*agro/2.0);
    gssrint = agss*(p1/2.0 + rof1 + agss*grate/2.0);
    gpeint = B0/(lpe*sfrq*1e6);

    /*******************************************************
    * Calculate minimum time for phase-encoding, slice refocussing
    * and readout prefocussing block.  Trapezoidal if GMAX is exceeded.
    * TSSRMIN does not include the gradient fall time.  RISETREF is
    * the gradient fall time for the PE block.
    *******************************************************/
    if (trise*gmax >= gssrint)
        tssrmin = sqrt(grate*gssrint);
    else
        tssrmin = gssrint/gmax;
   if (trise*gmax >= grorint)
        trormin = sqrt(grate*grorint);
    else
        trormin = grorint/gmax;

    if (trise*gmax >= gpeint*nv/2.0)
        tpemin = sqrt(grate*gpeint*nv/2.0);
    else
        tpemin = gpeint*nv/2.0/gmax;
 /* Minimum refocussing block time is the longest of calculated minima */
    trefmin = (tssrmin > trormin) ? tssrmin : trormin;
    trefmin = (trefmin > tpemin) ? trefmin : tpemin;

    /*******************************************************
    * If TREFMIN implies trapezoidal PE block, mimimum RISETREF
    * is RISETIME, otherwise minimum RISETREF is TREFMIN.
    *******************************************************/
    risetref = (trefmin > trise) ? trise : trefmin;

    /* Check minimum TE ***********************************/
    temin = p1/2.0 + rof1 + trefmin + risetref + tspoili
          + at/2.0 + grate*(agss + agro);
    if ((te/2.0) < temin) {
        printf("te too short.  Minimum te = %f\n",(temin*2));
        abort(1);
    }

    /*******************************************************
    * TREFI is the refocussing block length, not including gradient fall
    * time.  TREFI is scaled up to a maximum value consistent with TE
    *******************************************************/
    trefi = trefmin + 0.5*(te - temin);
    trefi = (trefi > 2.0*trefmin) ? 2.0*trefmin : trefi;

    /* Calculate GPE, adjusting for integral dac value, then adjust TREF */
    sgpe = gpeint/trefi;
    sgpe = gmax/gradstepsz*floor(sgpe*gradstepsz/gmax + 0.5);
    trefi = gpeint/sgpe;

    /* Calculate GSSR and GROR with new TREF **************/
    if (pilot[0] == 'y'  ||  pilot[0] == 'Y') {
        gror = -grorint/trefi*gro/fabs(gro);
        gssr = -gssrint/trefi*gss/fabs(gss);
        fprintf(stderr,"%s:  gror = %-8.4f     gssr = %-8.4f\n",
            seqfil,gror,gssr);
    }

    /* Determine PE block gradient fall time with new TREF */
    risetref = fabs(gssr) > fabs(gror) ? fabs(gssr) : fabs(gror);
    risetref = risetref > fabs(sgpe*nv/2.0) ? risetref : fabs(sgpe*nv/2.0);
    risetref *= grate;
    /* END OF IMAGING GRADIENT AND TIME CALCULATIONS ********/

    /* GRADIENT INTEGRAL AND TIMING CALCULATIONS **********/
    tref = p1/2.0;
    gvox1r = -gvox1*(p1/2.0 + rof1 + grate*fabs(gvox1)/2.0)/tref;

    /****
    * tau1 is determined by adding all of the events in the first half
    * echo period, and subtracting from te/2.  Similarly for tau2 and
    * tau3.
    ****/
    tau1 = (te - p1 - p2)/2.0 - tref - tcrush - 2.0*rof1
         - 3.0*trise - grate*fabs(gvox2);
    if (tau1 < 0.0) {
        printf("te too short for period 1.  Minimum te = %f\n",
          (te/2.0 - tau1));
        abort(1);
    }

    tau2 = (te + te2 - 2.0*p2)/2.0 - 2.0*tcrush - 3.0*trise
         - grate*fabs(gvox3) - 2.0*rof1;
    if (tau2 < 0.0) {
        printf("te or te2 too short.  Min total = %f\n",((te+te2)/2.0-tau2));
        abort(1);
    }

    tau3 = te2/2.0 - p2/2.0 - tcrush - rof1 - 2.0*trise;
    /* Calculate buffer time, allowing for imaging gradients */
    tau3 = tau3-trefi-risetref-(grate*agro)-tspoili-at/2.0;
    if (tau3 < 0.0) {
        printf("te2 too short for period 3.  Min te2 = %f\n",(te2/2.0-tau3));
        abort(1);
    }

    seqtime = te + te2 + at + p1/2.0 + rof1 + trise;
    if (ws[0] == 'y') {
        seqtime = seqtime + 3.0*(psat + tspoil + trise + 2.0*rof1);
    }
    seqtime = seqtime + trise;

    predelay = tr - seqtime;
    if (predelay < 0.0) {
        printf("tr too short.  Minimum tr = %f\n",seqtime);
        abort(1);
    }

    /* PULSE SEQUENCE *************************************/

    /* Phase cycle ****************************************
    ct  =  1  2  3  4    5  6  7  8    10 11 12 13 ......
    v2  =  0  1  2  3    0  1  2  3    0  1  2  3
    v1  =  0  0  1  1    2  2  3  3    4  4  5  5
    v1  =  0  0  0  0    1  1  1  1    2  2  2  2
    v1  =  0  0  0  0    1  1  1  1    2  2  2  2
    oph =  0  2  4  6    0  2  4  6    0  2  4  6
    v9  =  0  0  0  0    2  2  2  2    4  4  4  4
    oph =  0  2  4  6    2  4  6  8    4  6  8  10

    v10 =  0  0  1  1    2  2  3  3    4  4  5  5
    v10 =  0  0  0  0    1  1  1  1    2  2  2  2  
    v10 =  0  0  0  0    0  0  0  0    1  1  1  1
    v10 =  0  0  0  0    0  0  0  0    0  0  0  0
    v3  =  1  1  1  1    1  1  1  1    1  1  1  1
    *******************************************************/

    mod4(ct,v2);
    hlv(ct,v1);
    hlv(v1,v1);
    mod4(v1,v1);
    dbl(v2,oph);
    dbl(v1,v9);
    add(v9,oph,oph);
    /* CYCLOPS */
    hlv(ct,v10);
    hlv(v10,v10);
    hlv(v10,v10);
    hlv(v10,v10);
    mod4(v10,v10);
    add(one,v10,v3);
    add(v10,v1,v1);
    add(v10,v2,v2);
    add(v10,oph,oph);


  status(A);
  sp4off();
  /* PHASE ENCODE LOOP *************************************/
  peloop(seqcon[2],nv,v5,v6);

    /* Relaxation delay ***********************************/
    delay(predelay);
    xgate(ticks);

    /* Optional water suppression *************************/
    if (ws[0] == 'y') {
        obspower(satpwr);
        obsoffset(wsfrq);

        sp4on();
	shapedpulse(satpat,psat,zero,rof1,rof1);
        sp4off();
	obl_gradient(gspoil,0.0,gspoil);
	delay(tspoil);
	zero_all_gradients();
        delay(trise);

        sp4on();
	shapedpulse(satpat,psat,zero,rof1,rof1);
        sp4off();
	obl_gradient(gspoil,gspoil,0.0);
	delay(tspoil);
	zero_all_gradients();
        delay(trise);

        sp4on();
	shapedpulse(satpat,psat,zero,rof1,rof1);
        sp4off();
	obl_gradient(0.0,gspoil,gspoil);
	delay(tspoil);
	zero_all_gradients();
	delay(trise);
    }

    /* 90 degree RF pulse *********************************/
    obspower(tpwr1);
    poffset(pos1,gvox1);
    oblique_gradient(gvox1,0.0,0.0,vpsi,vphi,vtheta);
    delay(trise);
    sp4on();
    shapedpulse(p1pat,p1,v3,rof1,rof1);
    sp4off();
    zero_all_gradients();
    delay(trise);

    /* First half spin-echo period ************************/
    oblique_gradient(gvox1r,0.0,0.0,vpsi,vphi,vtheta);
    delay(tref);
    zero_all_gradients();
    delay(tau1 + trise);

    /* First half TE Gradient crusher *********************/
    obl_gradient(0.0,gcrush,0.0);
    delay(tcrush);
    zero_all_gradients();
    delay(trise);

    /* First 180 degree pulse *****************************/
    obspower(tpwr2);
    poffset(pos2,gvox2);
    oblique_gradient(0.0,gvox2,0.0,vpsi,vphi,vtheta);
    delay(grate*fabs(gvox2));
    sp4on();
    shapedpulse(p2pat,p2,v2,rof1,rof1);
    sp4off();
    zero_all_gradients();
    delay(trise);

    /* Second half TE Gradient crusher ********************/
    obl_gradient(0.0,gcrush,0.0);
    delay(tcrush);
    zero_all_gradients();
    delay(tau2 + trise);

    /* First half TE2 Gradient crusher ********************/
    obl_gradient(0.0,0.0,gcrush);
    delay(tcrush);
    zero_all_gradients();
    delay(trise);

    /* Second 180 degree pulse ****************************/
    obspower(tpwr2);
    poffset(pos3,gvox3);
    oblique_gradient(0.0,0.0,gvox3,vpsi,vphi,vtheta);
    delay(grate*fabs(gvox3));
    sp4on();
    shapedpulse(p2pat,p2,v1,rof1,rof1);
    sp4off();
    zero_all_gradients();
    delay(trise);

    /* Second half TE2 Gradient crusher *******************/
    obl_gradient(0.0,0.0,gcrush);
    delay(tcrush);
    zero_all_gradients();
    delay(trise);

    /* IMAGING PART OF THE SEQUENCE ***********************/
    /* Read and phase encode pulse ************************/
    pe_gradient(gror,sgpe*nv/2.0,0.0,-sgpe,v6);
    delay(trefi);
    zero_all_gradients();
    delay(risetref);

    /* Pre-acquire delay **********************************/
    delay(tau3);

    obl_gradient(gro,0.0,0.0);
    tau3b = grate*agro + tspoili;
    delay(tau3b);
    /* Acquire echo ***************************************/
    obsoffset(resto);
    acquire(np,1.0/sw);
    zero_all_gradients();
    delay(trise);

  endpeloop(seqcon[2],v6);    /*  phase encoding loop */
}

/*********************************************************************
                      MODIFICATION HISTORY

20May97(ss) New sequence, derived from steam.c.
22May97(ss) Cyclops phase cycling added.
26May97(ss) press.c modified for imaging 

*********************************************************************/
