#include <stdio.h>
#include <math.h>
#include <string.h>

/* 
 	calculates k-space density given a spiral k-space trajectory.
 
 rev 0	Copyright 1993, Craig Meyer, Stanford University.

 */

void calcden(int ns, float *kx, float *ky, float *den)
{
    int i, j, k, n, x1, y1;
    float maxden, gxf, gyf;
    double phi1, phi2;

    den[0] = 0;
    maxden = 0;
    for (j = 1; j < ns; j++)   {
	gxf = kx[j]-kx[j-1];
	gyf = ky[j]-ky[j-1];
	/* set density to zero sometimes to avoid atan2 errors  */
	if ((gyf == 0)||(ky[j-1] == 0))
	    den[j] = 0;
	else
	{
	    phi1 = atan2(gyf,gxf);
	    phi2 = atan2(ky[j-1],kx[j-1]); 
	    den[j] = hypot(gxf,gyf) * fabs(sin(phi1-phi2)); 
	    if(den[j] > maxden) maxden = den[j];
	}
    }
    for (j = 0; j < ns; j++)
	den[j] /= maxden;
    
    return;
}
