
double pw_gsp,pw_gspr; /* gsp=spiral gspr=spiral rewinder */
double spmatrix, sp_tep;
int acquire_points, spiral_points,total_points;

calculate_spiral_gradients(double *t_sp_b, double *t_sp_a)
{
  double nucgamma,gdwell_time,grocrit;
  char shape_name[MAXSTR];

  spmatrix=getval("spmatrix");
  sp_tep=getval("sp_tep");   /* spiral tep value */
  sp_tep=sp_tep/1.0e6;

  nucgamma = 1e6*sfrq/B0;
  grocrit = sw/lro/nucgamma;
  gro = gmax;/* should make this smaller of logical x or y */
  /* reduce gro if not critically sampled */
  if (gro > grocrit) gro = grocrit;
  
  gdwell_time=0.5/sw; /* gradient raster matches daq raster for now */

  if (ix == 1) {  /* only calculate spiral for the first array element */
    sprintf(shape_name,"sp");
    total_points=genspiral(shape_name,fabs(gro),grate,lro,(int)spmatrix,1,
                          gdwell_time,1.,ix,&spiral_points);
  }
printf("set np=%d\n",(int)((float)(spiral_points)/2.0+0.5)*2);
			  
  acquire_points=(int)((float)(spiral_points)/2.0+0.5)*2;
  at=(double)(acquire_points)/sw;
  np=(double)acquire_points;
  pw_gsp = total_points*gdwell_time;
  pw_gspr = (total_points-spiral_points)*gdwell_time;

  /* Calculate timing */
  *t_sp_a = total_points*gdwell_time;
  *t_sp_b = 0.;
}

spiral_gradients_and_acquire()
{
  obsoffset(resto);
  
  /* spiral gradients with acquisition */
  obl_shapedgradient("sprd","spph","",pw_gsp,gro,gro,0.0,1,NOWAIT);

  delay(sp_tep);
  
  /* Acquire echo */
  acquire(np,1.0/sw);
  delay(pw_gspr);

}

spiral_gradients_no_acquire()
{  
  /* spiral gradients without acquisition */
  obl_shapedgradient("sprd","spph","",pw_gsp,gro,gro,0.0,1,WAIT);

}

/*   multi- shot spiral design
*    uses Duyn's approximate slewrate limited design
*    augmented with archimedian gmax limit
*    inputs
*        fstem = output filename stem
*        gamp = design grad max, G/cm
*        grate = rise time, s/G/cm
*        fov = FOV, cm
*        spmat = matrix size
*        spint = number of interleaves
*	 dts = output sample spacing, s
*	 Tmax = longest acquisition allowed, s
*	 write_flag = switch for writing file
*    outputs
*        Gx, Gy -> written to files
*        sres = # of points in spiral -> returned in argument
*        gres = # of waveform points, including rewinders
*               -> returned as value of genspiral routine
*    time is in sec
*
*		rev 0 12/26/98	original
*		rev 1 4/15/99	little better calc of ts
*    Original code is from Gary Glover, Stanford University
*    Mods from here down are UCSD
*		99.09.08	ECW	No rampdown, report net area
*		99.10.20	ECW	Rewind here for better precision
*		00.01.26	ECW	Modified for use on Varian scanner
*/

/*==========================================================================*/
int genspiral(fstem,gamp,grate,fov,spmat,spint,dts,Tmax,write_flag,sres)
/*==========================================================================*/
char fstem[];
double gamp,grate,fov,dts,Tmax;
int spmat,spint,*sres;
unsigned long write_flag;
{  
  float gamma = 2*M_PI*4.257e3;

  float dt;
  float S0;
  int i, j, n, ari, gn, nsin, nrmp, sign, net, error, *gxy, axis, gresx, cnt;
  int gres,xflat,yflat,spres,*Gx,*Gy;
  float Ts, Tlim, sts, ttt, thetaa, dthdt, thetas, x, y;
  float a1, a2, beta, Gmax, c, s;
  float *gx, *gy;
  float gabs, sp_gmax, gslew;
  short gxi, gyi;
  float q;
  float islew, igamp, atmp, ampsin;
  double mult, del;
  char full_fname[MAXSTR], fname_ext[MAXSTR], sub_dir_name[MAXSTR];
  char *home_dir, prog_name[MAXSTR];
  FILE *fp=NULL;
#define LIMIT(x, xmin, xmax)   ( (x<xmax)? ((x>xmin)? x:xmin):xmax )
#define IRINT(f)  ((((f)-(int)(f)) < 0.5) ? ((int) (f)) : (1+(int)(f)))

  sprintf(prog_name, "genspiral");
  if (gamp == 0.0) {
    printf("Error: zero gradient value passed to %s\n", prog_name);
    exit(0);
  }
  
  gx = (float *)calloc((int)(Tmax/dts),sizeof(float));
  gy = (float *)calloc((int)(Tmax/dts),sizeof(float));
  Gx = (int *)calloc((int)(Tmax/dts),sizeof(int));
  Gy = (int *)calloc((int)(Tmax/dts),sizeof(int));

  gslew = 1./(100.*grate); /* s/G/cm to mT/m/ms */

  q = 5;		/* nice number  */
  S0 = gslew*100;       /* G/cm/s */
  dt = dts*.5;

  printf("entering genspiral spmat=%d, fov=%6.2f cm, spint=%d\n", spmat, fov, spint);

/*  slew-rate limited approximation  */

  sts=0.0; /* initialize */
  thetas=0.0; /* initialize */
  Ts = .666667/spint*sqrt(pow((double)(M_PI*spmat), (double)3.0)/(gamma*fov*S0));
  if(Ts > Tmax)  {
    printf("slew limited readout too long\n");
    return 0;
  }
  a2 = spmat*M_PI/(spint*pow(Ts, .666667));
  a1 = 1.5*S0/a2;
  beta = S0*gamma*fov/spint;
  Gmax = a1*pow(Ts, .333333);
  sp_gmax = 0;
  i = 0;
  for (ttt = 0; ttt<=Ts; ttt+=dt)  {
    x = pow(ttt, 1.333333);
    thetaa = .5*beta*ttt*ttt/(q + .5*beta/a2*x);
    y = q+.5*beta/a2*x;
    dthdt = beta*ttt*(q+.166667*beta/a2*x)/(y*y);
    c = cos(thetaa);
    s = sin(thetaa);
    gx[i] = spint/(fov*gamma)*dthdt*(c - thetaa*s);
    gy[i] = spint/(fov*gamma)*dthdt*(s + thetaa*c);
    gabs = hypot(gx[i], gy[i]);
    if(gabs>=gamp)  {
      if(sp_gmax==0)  sp_gmax = hypot(gamp/thetaa, gamp);
      if(gabs>sp_gmax) break;
    }
    sts = ttt;
    thetas = thetaa;
    i++;
  }

/*  sp_gmax limited approximation  */

  if(Gmax > gamp)  {
    Tlim = ((M_PI*spmat/spint)*(M_PI*spmat/spint) - thetas*thetas)/(2*gamma*gamp*fov/spint) + sts;
    if(Tlim > Tmax)  {
      printf("sp_gmax limited readout too long\n");
      return 0;
    }
    for (ttt=sts+dt; ttt<=Tlim; ttt+=dt)  {
      thetaa = sqrt(thetas*thetas + 2*gamma*gamp*fov*(ttt-sts)/spint);
      c = cos(thetaa);
      s = sin(thetaa);
      gx[i] = gamp*(c/thetaa - s);
      gy[i++] = gamp*(s/thetaa + c);
    }
  }

/*  decimate by 2 to get back to 4us sampling */

  n = 0;
  for (j=0; j<i; j+=2)  {
    x = LIMIT(gx[j], -gamp, gamp);
    gxi = x*MAX_DAC_VAL/gamp;
    Gx[n] = 2*(gxi/2);
    y = LIMIT(gy[j], -gamp, gamp);
    gyi = y*MAX_DAC_VAL/gamp;
    Gy[n++] = 2*(gyi/2);
  }
  spres = n; /* was n-1?? ECW */

  gresx=0; /* initialize */

/* Rewind */
  islew = 0.707 * S0 * dts * MAX_DAC_VAL / gamp;
  igamp = 0.707 * MAX_DAC_VAL;
  for (axis=0;axis<2;axis++) {
    ari = 0;
    if (axis==0) gxy = Gx;
    else gxy = Gy;
    for (i=0;i<spres;i++) ari += gxy[i];
    gn = gxy[spres-1];
    if ((ari+abs(gn)*gn/(islew*2))*gn>0) { /* ramp down + half sin */
      nrmp = abs(gn)/islew + 1;
      for (i=0;i<nrmp;i++) {          /* put in ramp */
	gxy[i+spres] = 2*IRINT((0.5*gn*(nrmp-1-i))/nrmp);
	ari += gxy[i+spres];
      }
      if (ari>0) sign = -1;
      else sign = 1;
      ari = abs(ari);
      nsin = M_PI * sqrt((0.5*ari)/islew) + 0.5; /* Calculate width of sin */
      ampsin = islew * nsin / M_PI;
      if (ampsin > igamp) {                       /* Stretch if necessary */
	nsin *= ampsin / igamp;
	ampsin = igamp;
      }
      for (i=0;i<nsin;i++) gy[i] = ampsin * sin((i+1)*M_PI/nsin);
      for (i=0,net=0;i<nsin;i++) net += IRINT(0.5*gy[i]);
      error = 2*net - ari;
      if (error>0) del = -0.001;
      else del = 0.001;
      for (net=1,mult=1.,cnt=0;net&&(cnt<50);cnt++) { /* Tweak sin */
	mult += del;
	for (i=0,net=0;i<nsin;i++) net += IRINT(0.5*mult*gy[i]);
	net = 2*net - ari;
	if (error*net < 0) del *= -0.5;
	error = net;
      }
      for (i=0;i<nsin;i++) gxy[spres+nrmp+i] = 2*sign*IRINT(0.5*mult*gy[i]);
      if (abs(error)>20) {
	printf("Spiral rewinder failed to converge\n");
	return 0;
      }
      else if (error) gxy[spres+nrmp+nsin-1] -= sign*error;
      gres = spres + nrmp + nsin;
    }
    else {    /* partial cosine */
      if (ari>0) sign = -1;
      else sign = 1;
      ari = abs(ari);
      if (abs(gn)<igamp) {
	thetaa = M_PI-asin(abs(gn)/igamp);
	atmp = (1.-cos(thetaa))*igamp*igamp/islew;
	if (atmp<ari) {    /* Stretch cos peaked at .707 */
          nsin = thetaa*igamp*ari/(islew*atmp) + 0.5;
	  for (i=0;i<nsin;i++) gy[i] = igamp * sin(thetaa-i*thetaa/(nsin-1.));
	}
	else {                  /* Cos peaked at < .707 */
          atmp = ari/(gn*gn/islew);
	  if (atmp<=0.5) {
	    nsin = ((float)abs(gn))/islew + 0.5;
	    for (i=0;i<nsin;i++) gy[i] = gn*(nsin-1-i)/nsin;
	  }
	  else {
	    thetaa = -atan(atmp*sqrt((2.*atmp-1.)/(atmp*atmp))/(atmp-1.));
	    if (thetaa<0.) thetaa += M_PI;
	    ampsin = abs(gn)/sin(thetaa);
	    nsin = ampsin*thetaa/islew;
	    if (ampsin>igamp) {
              nsin *= ampsin / igamp;
              ampsin = igamp;
	    }
	    for (i=0;i<nsin;i++) gy[i] = ampsin*sin(thetaa*(1.-i/(nsin-1.)));
	  }
	}
      }
      else {                    /* Pure cos peaked at gn */
	ampsin = abs(gn);
	nsin = M_PI * abs(ari) / (2*ampsin) + 1;
	for (i=0;i<nsin;i++) gy[i] = ampsin*cos(M_PI*i/(2.*(nsin-1)));
      }
      for (i=0;i<nsin;i++) gx[i] = sin(M_PI*i/(nsin-1.));
      for (i=0,net=0;i<nsin;i++) net += IRINT(0.5*gy[i]);
      error = 2*net - ari;
      if (error>0) del = -0.001;
      else del = 0.001;
      for (net=1,mult=0.,cnt=0;net&&(cnt<50);cnt++) { /* Tweak sin */
        mult += del;
	for (i=0,net=0;i<nsin;i++) net += IRINT(0.5*gy[i]*(1.+mult*gx[i]));
	net = 2*net - ari;
	if (error*net < 0) del *= -0.5;
	error = net;
      }
      for (i=0;i<nsin;i++) gxy[spres+i] = 2*sign*IRINT(0.5*gy[i]*(1.+del*gx[i]));
      if (abs(error)>20) {
	printf("Spiral rewinder failed to converge\n");
	return 0;
      }
      else if (error) gxy[spres+nsin-1] -= sign*error;
      gres = spres + nsin;

    }
    if (axis==0) gresx = gres;
  }
  if (gres>gresx) {
    xflat = gres-gresx;
    yflat = 0;
    for (i=0;i<xflat;i++) Gx[gresx+i] = 0;
  }
  else {
    yflat = gresx-gres;
    xflat = 0;
    for (i=0;i<yflat;i++) Gy[gres+i] = 0;
    gres = gresx;
  }

/* Write to file Varian style */
  if (write_flag == 1) {
    home_dir = (char *) getenv("HOME");
    strcpy(full_fname, home_dir);
    if (full_fname == "") {
      printf("Error determining HOME environment variable\n");
      exit(0);
    }
    sprintf(sub_dir_name,"/vnmrsys/shapelib/");
    strcat(full_fname, sub_dir_name);
    strcat(full_fname, fstem);

    sprintf(fname_ext,"rd.GRD");
    strcat(full_fname, fname_ext);

    if ((fp=fopen(full_fname, "w")) == NULL) {
      printf("Error opening output data file %s\n", full_fname);
      exit(0);
    }
    /* correct for a bug in the waveform memory calculation which doesn't
       allow for more than 255 clock cycles of a given DAC level */
    xflat /= 255;
    for (i=0; i<gres-(255*xflat); i++) fprintf(fp, "%d   1\n",Gx[i]);
    for (i=0; i<xflat; i++) fprintf(fp, "0   255\n");
    fprintf(fp, "\n");
    fclose(fp);
    
    strcpy(full_fname, home_dir);
    strcat(full_fname, sub_dir_name);
    strcat(full_fname, fstem);

    sprintf(fname_ext,"ph.GRD");
    strcat(full_fname, fname_ext);

    if ((fp=fopen(full_fname, "w")) == NULL) {
      printf("Error opening output data file %s\n", full_fname);
      exit(0);
    }
    /* correct for a bug in the waveform memory calculation which doesn't
       allow for more than 255 clock cycles of a given DAC level */
    yflat /= 255;
    for (i=0; i<gres-(255*yflat); i++) fprintf(fp, "%d   1\n",Gy[i]);
    for (i=0; i<yflat; i++) fprintf(fp, "0   255\n");
    fprintf(fp, "\n");
    fclose(fp);
  }

  *sres = spres;
  return gres;

}
