/***********************************************************************
    SPARK_TEST.C

    gradient sparking test sequence.
    Peter Jezzard, NIH July 1997

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

#include "standard.h"
#include "group.h"
#include "sequence_utilities.h"
extern double rint();

pulsesequence()
{
    /* INTERNAL VARIABLE DECLARATIONS *********************/
    int     image=0;
    double  nucgamma, grad_del, pre_acq_del, seqtime, postdelay;
    double  gread, agread, gphase, agphase, gslice, agslice, grise;
    double  pw_trap, max_grad, grate;
    double  create_trap_gradient();
    char shape_name[MAXSTR], err_string[MAXSTR];

    initparms_sis();

    /*************************************************************************/
    /* get the global parameters */
    /*************************************************************************/
    P_getreal(GLOBAL,"grise",&grise,1); /* grad risetime to full strength */
    if ((grise < 0.0) || (grise > 0.01)) {
      sprintf(err_string,"%s: impropper grise time defined in global file\n",
                       seqfil);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
    /*************************************************************************/

    grad_del = getval("grad_del"); /* duration of gradient pulse */
    pre_acq_del = getval("pre_acq_del"); /* pre acquisition delay */
    gread = getval("gread"); /* gradient strength in read direction */
    gphase = getval("gphase"); /* gradient strength in phase direction */
    gslice = getval("gslice"); /* gradient strength in slice direction */
    image = (int)(getval("image")); /* useful counter */

    grate = grise/gmax; /* switching rate of gradients [sec/(gauss/cm)] */

    nucgamma = 1e6*sfrq/B0;  /* nucleus gamma in Hz/gauss */
                             /* derived from sfrq in MHz and B0 in gauss */

    agread = fabs(gread); /* absolute read gradient strength */
    agphase = fabs(gphase); /* absolute phase encode gradient strength */
    agslice = fabs(gslice); /* absolute slice select gradient strength */

    max_grad = fabs(gread) > fabs(gphase) ? fabs(gread) : fabs(gphase);
    max_grad = max_grad > fabs(gslice) ? max_grad : fabs(gslice);

    sprintf(shape_name,"gtrap");
    pw_trap=create_trap_gradient(max_grad,shape_name,grad_del,grate,ix);

  /* CALCULATE RELAXATION DELAY FOR SEQUENCE */
    seqtime = (pw + rof1 + rof2 + pw_trap + pre_acq_del);
    if (at > seqtime - pw - rof1 - rof2) seqtime=at+pw+rof1+rof2;
    if (tr - seqtime < 0.0) {
          sprintf(err_string,"%s: TR too short. Min TR = %f\n",seqfil,seqtime);
          printf(err_string);
          log_error(err_string);
          abort(1);
    }
    postdelay = (tr - seqtime);

    status(A);
    rcvroff();
    rcvron();

  delay(d1);

  peloop(seqcon[2],nv,v5,v6);

    /* 90 degree pulse ************************************/ 
    sp1on();
    delay(0.0001);
    sp1off();

    if ((fabs(gread) > 0.0) || (fabs(gphase) > 0.0) || (fabs(gslice) > 0.0)) {
      /* first dephase the signal using a shaped gradient pulse */
      obl_shapedgradient("gtrap","gtrap","gtrap",pw_trap,
          gread,gphase,gslice,1,NOWAIT);
    }

    delay(pre_acq_del);
    acquire(np, 1.0/sw);

    delay(postdelay);
    zero_all_gradients();  /* double check that all gradients are off */

  endpeloop(seqcon[2],v6);

}
