#ifndef LINT
static char SCCSid[] = "@(#)flash.c 11.1 05/07/96 Copyright (c) 1994-1996 Varian Assoc.,Inc. All Rights Reserved";
#endif
/***********************************************************************
    SPGR.C

    Gradient-echo fast imaging sequence.                     

    VERSION 2.0        6/94/94  
    Revisions:
      2.1   2/96    Fixed rfspoiling for nv=0

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
        __                                                  __ 
       |                                                      |
  RF   |----- [ p1 ] ----------------------- |** ACQ **| -----|
       |                                                      |
       |         < - - - - - - - te - - - - - - - >           |
       |       ____                                           |
  GSS  |______/////\     _____________________________________|
       |<---->      \\\\/ gssr                                |
       |  predelay  <--> tref                                 | nv
       |             ___                                  ___ |
  GPE  |____________/___\________________________________/___\|
       |            \___/                                \___/|
       |                 < - tA - >                           |
       |                            ____________________      |
  GRO  |____________     __________/////////////////////\_____|
       |       gror \\\\/          < tspoil >                 |
       |                                                      |
       |< - - - - - - - - - - - tr - - - - - - - - - - - - - >|
       |__                                                  __|

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

#include "standard.h"
#include "math.h"
#include "group.h"
#include "sequence_utilities.h"
#define OFFSETDELAY 26e-6
#define RFSPOILDELAY 50e-6

/*==========================================================================*
  define the pulse sequence variables outside the pulse sequence so that
  they can be seen as global variables by the safety monitoring code
*==========================================================================*/
    double  predelay,seqtime,tref,risetref,grise,tep,rftep;
    double  trefmin,tssrmin,trormin,tpemin,temin,tA;
    double  agro,agss,sgpe,grate,gssrint,grorint,gpeint;      
    double  reffreq,nextphase,spoilfreqs[1024],pw_gss_d,extra_te_error;
    double  pw_gro,pw_gss,wave_time,true_wave_time,greadfc,gslicefc;
    int     i,image=0,dummy_ssc;
    char    shape_name[MAXSTR],flow_comp[MAXSTR],err_string[MAXSTR];
    char    gcoil[MAXSTR];

#include "spgr_safety_calc.h"

pulsesequence()
{

    double  create_ge_slice_flow_comp();
    double  create_trap_gradient(), create_ramp_gradient();
    double  create_ge_read_flow_comp(), create_ge_phase_flow_comp();
    int num_slices;

    initparms_sis();
    loop_check();

    /*************************************************************************/
    /* get the global parameters */
    /*************************************************************************/
    P_getreal(GLOBAL,"grise",&grise,1); /* grad risetime to full strength */
    if ((grise < 0.0) || (grise > 0.002)) {
      sprintf(err_string,"%s: impropper grise time defined in global file\n",
                       seqfil);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
    P_getreal(GLOBAL,"tep",&tep,1); /* small tweaker time (us) to allow for
                            the gradient propagation time, causes echoes to
                            appear symmetrically about echo center line */
    tep=tep/1e6;
    P_getreal(GLOBAL,"rftep",&rftep,1); /* small tweaker time (us) to allow for
                            the gradient propagation time, w.r.t. RF pulses */
    rftep=rftep/1e6;
    /*************************************************************************/

    getstr("gcoil",gcoil);          /* gradient coil in use */
    getstr("flow_comp",flow_comp);  /* flow comp flag 'y'=fc. 'n'=no fc. */
    num_slices = (int)(getval("ssc")); /* number of slices */
    image = (int)(getval("image")); /* trigger and counter bit pattern */
    /* bit 1 = wait for ticks triggers before beginning scan */
    /* bit 2 = puts out high on sp3 for duration of volume */
    /* bit 3 = reserved for epi blips on/off */

    gcrush=gmax*0.75; /* hard wire a crusher pulse in to the sequence */
    
    grate = grise/gmax;
    agro = fabs(gro);
    agss = fabs(gss);
    if (agss == 0.0) {
        sprintf(err_string,"%s: Zero slice select gradient\n",seqfil);
        printf(err_string);
        log_error(err_string);
        abort(1);
    }

    sprintf(shape_name,"gss");
    pw_gss=create_trap_gradient(fabs(gss),shape_name,rof1+p1+rof2,grate,ix);

    pw_gro=0.0; /* initialization value */
    if ((flow_comp[0] != 'y') && (flow_comp[0] != 'Y')) {
      sprintf(shape_name,"gro");
      pw_gro=create_ramp_gradient(fabs(gro),shape_name,1,grate,ix);
      pw_gro=create_ramp_gradient(fabs(gro),shape_name,-1,grate,ix);
    }

    /* figure out a sensible number of dummy scans */
    dummy_ssc=0;  /* default */
    if (nv > 0.0) {
      dummy_ssc=(int)((10.0*exp(-tr/0.1)+6.0*exp(-tr/0.6))+0.5)*2;
      /* leave as zero if a single slice time series */
      if ((num_slices == 1) && (ix > 1)) dummy_ssc=0;
    }

    wave_time=0.0; /* initialization value */
    greadfc=0.0; /* initialization value */
    gslicefc=0.0; /* initialization value */
    if ((flow_comp[0] == 'y') || (flow_comp[0] == 'Y')) {
      pw_gro=grate*fabs(gro);
      wave_time=te-p1/2-rof2-agss*grate+at/2+pw_gro;
      sprintf(shape_name,"read");
      greadfc=create_ge_read_flow_comp(fabs(gro),shape_name,tspoil+at/2,at/2,
           pw_gro,0.8*gmax,grate,wave_time);
      if (greadfc < 0.0) {
        sprintf(err_string,"%s: TE too short. Min TE = %f ms\n",seqfil,
                    (te-greadfc)*1e3);
        printf(err_string);
        log_error(err_string);
	abort(1);
      }
      sprintf(shape_name,"slice");
      gslicefc=create_ge_slice_flow_comp(fabs(gss),shape_name,p1/2,agss*grate,
           0.8*gmax,grate,wave_time);
    }

    /* GRADIENT*TIME integrals for slice selection and readout */
    grorint = agro*(at/2.0 + tspoil + pw_gro/2.0);
    gssrint = agss*(p1/2.0 + rof1 + agss*grate/2.0);
    gpeint = B0/(lpe*sfrq*1e6);

    /*******************************************************
    * Calculate minimum time for phase-encoding, slice refocussing
    * and readout prefocussing block.  Trapezoidal if GMAX is exceeded.
    * TSSRMIN does not include the gradient fall time.  RISETREF is
    * the gradient fall time for the PE block.
    *******************************************************/
    if (grise*gmax >= gssrint)
	tssrmin = sqrt(grate*gssrint);
    else
	tssrmin = gssrint/gmax;

    if (grise*gmax >= grorint)
        trormin = sqrt(grate*grorint);
    else
	trormin = grorint/gmax;

    if (grise*gmax >= gpeint*nv/2.0)
        tpemin = sqrt(grate*gpeint*nv/2.0);
    else
	tpemin = gpeint*nv/2.0/gmax;

    /* Minimum refocussing block time is the longest of calculated minima */
    trefmin = (tssrmin > trormin) ? tssrmin : trormin;
    trefmin = (trefmin > tpemin) ? trefmin : tpemin;

    /*******************************************************
    * If TREFMIN implies trapezoidal PE block, mimimum RISETREF
    * is RISETIME, otherwise minimum RISETREF is TREFMIN.
    *******************************************************/
    risetref = (trefmin > grise) ? grise : trefmin;

    /* Check minimum TE ***********************************/
    temin=0.0; /* default value */
    if ((flow_comp[0] != 'y') && (flow_comp[0] != 'Y')) {
      temin = p1/2.0 + rof1 + trefmin + risetref + tspoil
          + at/2.0 + agss*grate + pw_gro;
      if (te < temin) {
          sprintf(err_string,"%s: TE too short. Min TE = %f\n",seqfil,temin);
          printf(err_string);
          log_error(err_string);
	  abort(1);
      }
    }

    /*******************************************************
    * TREF is the refocussing block length, not including gradient fall
    * time.  TREF is scaled up to a maximum value consistent with TE
    *******************************************************/
    tref = trefmin + 0.5*(te - temin);
    tref = (tref > 2.0*trefmin) ? 2.0*trefmin : tref;

    /* Calculate GPE, adjusting for integral dac value, then adjust TREF */
    sgpe = gpeint/tref;
    sgpe = gmax/gradstepsz*floor(sgpe*gradstepsz/gmax + 0.5);
    tref = gpeint/sgpe;

    /* Calculate GSSR and GROR with new TREF **************/
    gror = -grorint/tref*gro/fabs(gro);
    gssr = -gssrint/tref*gss/fabs(gss);

    /* Determine PE block gradient fall time with new TREF */
    risetref = fabs(gssr) > fabs(gror) ? fabs(gssr) : fabs(gror);
    risetref = risetref > fabs(sgpe*nv/2.0) ? risetref : fabs(sgpe*nv/2.0);
    risetref *= grate;

    sprintf(shape_name,"gdp");
    risetref=create_ramp_gradient(risetref/grise*gmax,shape_name,1,grate,ix);
    risetref=create_ramp_gradient(risetref/grise*gmax,shape_name,-1,grate,ix);

    tA=0.0; /* default value */
    if ((flow_comp[0] != 'y') && (flow_comp[0] != 'Y')) {
      /* tA is the delay between refocussing and readout ****/
      tA = te - (p1/2.0 + rof1 + tref + risetref + tspoil + at/2.0
         + agss*grate + pw_gro);

    }

    true_wave_time=0.0; /* initialization value */
    if ((flow_comp[0] == 'y') || (flow_comp[0] == 'Y')) {
      sprintf(shape_name,"phase");
      true_wave_time=create_ge_phase_flow_comp(risetref/grise*gmax,shape_name,
          tref-risetref,risetref,wave_time);
    }

    /* account for case of rftep being longer than the gradient ramp time */
    pw_gss_d = agss*grate - rftep;
    extra_te_error = 0.0;
    if (pw_gss_d < 0.0) {
      pw_gss_d = 0.0;
      extra_te_error = rftep-agss*grate;
    }

    /* Relaxation delay ***********************************/
    seqtime = te + p1/2.0 + rof1 + rftep +extra_te_error + at/2.0 + agss*grate  
            + pw_gro + tref + risetref;
    if (rfspoil[0] == 'y'  ||  rfspoil[0] == 'Y')
	seqtime += RFSPOILDELAY;
    if (tr < seqtime) {
        sprintf(err_string,"%s: TR too short. Min TR = %f\n",seqfil,seqtime);
        printf(err_string);
        log_error(err_string);
        abort(1);
    }
    predelay = tr - seqtime;

    /*******************************************************
    * Calculation of frequencies to get phase shifts for optional
    * RF spoiling.  Phase shifts are obtained by hopping to variable
    * frequency for fixed time.
    * Ref:  Zur, Y., Magn. Res. Med., 21, 251, (1991).
    *******************************************************/
    if ((rfspoil[0] == 'y' || rfspoil[0] == 'Y') && nv > 0) {
	reffreq = resto + sfrq*1e6/B0*gro*pro;
        for (i=1; i<= (int)(nv+0.5); i++) {
	    nextphase = (double)i/2.0*((double)i-1.0)*rfphase;
	    nextphase = nextphase - 360.0*floor(nextphase/360.0);
	    spoilfreqs[i] = reffreq + nextphase/RFSPOILDELAY/360.0;
        }
        create_offset_list(spoilfreqs,nv,TODEV,0);
    }

    if (spgr_safety_calc() == -1) {
      sprintf(err_string,"%s: Failed duty cycle safety check\n",seqfil);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }

    /* PULSE SEQUENCE *************************************/

    /* Phase cycle ****************************************/
    mod2(ct,v3);          /* v3: 0  1  0  1  0  1  0  1 */
    dbl(v3,v3);           /* v3: 0  2  0  2  0  2  0  2 */
    hlv(ct,v4);           /* v4: 0  0  1  1  2  2  3  3 */
    mod2(v4,v4);          /* v4: 0  0  1  1  0  0  1  1 */
    add(v3,v4,v1);        /* v1: 0  2  1  3  0  2  1  3 */
    assign(v1,oph);      /* oph: 0  2  1  3  0  2  1  3 */

    assign(one,v7);  /* phase encode increment size */
    status(A);
    sp4off();
    obspower(tpwr1);

    sp2on();
    delay(0.00005);  /* 50 us trigger to indicate start of volume acq */
    sp2off();

    delay(d1);

    if ((image & 1) == 1) xgate(ticks);
    if ((image & 2) == 2) sp3on(); else sp3off();

    initval(one,v9);  /* steady state flag */
    initval(dummy_ssc,v10);  /* steady state counter */

    peloop(seqcon[2],nv,v5,v6);

      /* Relaxation delay ***********************************/       
      poffset(pss[0],gss);

      ifzero(v10);
        assign(zero,v9);
      endif(v10);
      sub(v10,one,v10);  /* steady state stuff (part 1) */

      delay(predelay);
       
      /* RF pulse *******************************************/ 
      sp1on(); /* put out trigger on sp1 every spin excitation */
      obl_shapedgradient("","","gss",pw_gss,0.0,0.0,gss,1,NOWAIT);
      delay(agss*grate+rftep);
      sp4on();  /* external RF blanking */
      shapedpulse(p1pat,p1,v1,rof1,rof1);
      sp4off();
      delay(pw_gss_d);
      sp1off();

      /* Read & Slice refocus and phase encode pulse ********/
      if ((flow_comp[0] != 'y') && (flow_comp[0] != 'Y')) { /* no flow comp */
        phase_encode_shapedgradient("gdp_up",risetref,gror,sgpe*nv/2.0,
            gssr,-sgpe,v6,nv,psi,phi,theta,v7,WAIT,0);
        delay(tref-risetref);
        phase_encode_shapedgradient("gdp_dn",risetref,gror,sgpe*nv/2.0,
            gssr,-sgpe,v6,nv,psi,phi,theta,v7,WAIT,1);
        delay(tA);
        poffset(pro,gro);
        obl_shapedgradient("gro_up","","",pw_gro,gro,0.0,0.0,1,WAIT);
        delay(tspoil + 0.5/fb);
      }
      if ((flow_comp[0] == 'y') || (flow_comp[0] == 'Y')) { /* flow comp */
        /* play out the long waveforms */
        pe_oblique_shapedgradient("read","phase","slice",true_wave_time,
          greadfc,sgpe*nv/2.0,gslicefc,-sgpe,v6,nv,psi,phi,theta,NOWAIT,2);
        delay(true_wave_time-at-pw_gro);
        poffset(pro,gro);
      }

      /* Pre-acquire delay **********************************/

      /* Acquire echo ***************************************/
      delay(tep);
      ifzero(v9);   /* steady state stuff (part 2) */
        acquire(np,1.0/sw);
      elsenz(v9);
        delay(at);
      endif(v9);
      if ((flow_comp[0] != 'y') && (flow_comp[0] != 'Y')) { /* no flow comp */
        obl_shapedgradient("gro_dn","","",pw_gro,gro,0.0,0.0,1,WAIT);
      }
      phase_encode_shapedgradient("gdp_up",risetref,0.0,-sgpe*nv/2.0,
            gcrush,sgpe,v6,nv,psi,phi,theta,v7,WAIT,0);
      delay(tref-risetref);
      phase_encode_shapedgradient("gdp_dn",risetref,0.0,-sgpe*nv/2.0,
            gcrush,sgpe,v6,nv,psi,phi,theta,v7,WAIT,1);

      zero_all_gradients(); /* double check that all gradients are zero */

      ifzero(v9);   /* steady state stuff (part 3) */
      elsenz(v9);
        sub(v6,one,v6);
      endif(v9);

      /*******************************************************
      * Optional RF spoiling.  REMINDER: Do not place any events
      * after this block or before poffset at the start of peloop.
      *******************************************************/
      if ((rfspoil[0] == 'y' || rfspoil[0] == 'Y') && nv > 0) {
        voffset(0,v6);
        delay(RFSPOILDELAY - OFFSETDELAY);
      }

    endpeloop(seqcon[2],v6);
}
