;=============================================================================
function phase_unwrap, phase_vals
;=============================================================================
; routine to unwrap phase of a 1D phase profil or 2D set of phase profiles

dims=size(phase_vals)
dim0=dims(1)
dim1=dims(2)
if (dims(0) eq 1) then dim1=1 ; 1D data set
 
bounce_thresh=3.1415  ; threshold above which a 2*pi bounce is assumed
pi=3.1415926 

out_phase=phase_vals
for i=0, dim1-1 do begin

  phase_arr=phase_vals(*,i)

  ; calculate d(phase)/d(point)
  diff_phase=phase_arr-shift(phase_arr, 1)
  middle_phase=phase_arr(dim0/2)

  ; take out the bounces
  bounce_count=0

  ; count up the positive bounces....
  mask=where(diff_phase gt bounce_thresh)
  mask_arr1=intarr(dim0)
  if (total(mask) gt -1) then mask_arr1(mask)=1

  ; count up the negative bounces....
  mask=where(diff_phase lt -1.0*bounce_thresh)
  mask_arr2=intarr(dim0)
  if (total(mask) gt -1) then mask_arr2(mask)=1

  bounce_arr=intarr(dim0)
  for k=0, dim0-1 do begin
    if (mask_arr1(k) eq 1) then bounce_count=bounce_count-1
    if (mask_arr2(k) eq 1) then bounce_count=bounce_count+1
    bounce_arr(k)=bounce_count
  endfor
 
  ; set the center point back to the right phase
  phase_arr=phase_arr+2.0*pi*bounce_arr
  phase_arr=phase_arr-phase_arr(dim0/2)+middle_phase

  out_phase(*,i)=phase_arr
endfor
 
return, out_phase  ; returns the fitted phase
 
end
