
;==================================================================
pro cos_fit, x, a, f, pder
;==================================================================

f=cos(a(0)*2*!pi*x+a(1))

if (n_params(0) le 3) then return

  pder=fltarr(n_elements(x),2)
  pder(*,0)=-2*!pi*x*sin(a(0)*2*!pi*x+a(1))
  pder(*,1)=-sin(a(0)*2*!pi*x+a(1))

end

;==================================================================
pro exp_fit, x, a, f, pder
;==================================================================

f=a(0)*exp(-x/a(1))

if (n_params(0) le 3) then return

  pder=fltarr(n_elements(x),2)
  pder(*,0)=exp(-x/a(1))
  pder(*,1)=a(0)*exp(-x/a(1))*x/a(1)^2

end

;==================================================================
function t2star_curve_fit, in_arr, tau_pe
;==================================================================

n_te=n_elements(in_arr)

tmp_y=reform(abs(in_arr),n_te)
x=float(indgen(n_te))

weights=fltarr(n_te)+1.0

a=fltarr(2)
a(0)=max(tmp_y)
old_a0=a(0)
a(1)=50.0/tau_pe  ; starting guess
old_a1=a(1)

fit_vals=curvefit(x,tmp_y,weights,a,FUNCTION_NAME='exp_fit')

t2star_resid=total((tmp_y-fit_vals)^2)
S0_val=a(0)
t2star_val=a(1)*tau_pe

if ((old_a0 eq a(0)) and (old_a1 eq a(1))) then begin
  a(0)=0
  a(1)=0
endif

osc_curve=in_arr/fit_vals

mask=indgen(n_te/2)*2
osc_curve=osc_curve(mask+1)
x=x(mask+1)

iii=complex(0,1)
pha=acos(float(osc_curve)/abs(osc_curve))
pha=pha*2.0*((float(-osc_curve*iii) gt 0.0)-0.5)
new_pha=phase_unwrap(pha)
pha_diff=new_pha-shift(new_pha, 1)

weights=fit_vals/max(fit_vals)

a(0)=median(pha_diff)/2.0/!pi/2  ; extra 2 because only every other point
a(1)=pha(0)

fit_vals=curvefit(x,float(osc_curve),weights,a,FUNCTION_NAME='cos_fit')

field_val=a(0)/tau_pe*1000.0
field_resid=total((float(osc_curve)-fit_vals)^2)

return, [S0_val, t2star_val, field_val, t2star_resid, field_resid]

end

;==================================================================
pro t2star_fit, dir_stem
;==================================================================

on_ioerror, cant_read_file

fname=dir_stem+'/fid_recon'

dims=intarr(8)
data_type=fix(0)

dummy_head=bytarr(40)
dummy_head2=bytarr(14)
header_fname=fname+'.hdr'

; read in the image size
get_lun, unit
openr, unit, header_fname
readu, unit, dummy_head
readu, unit, dims
readu, unit, dummy_head2
readu, unit, data_type
close, unit
free_lun, unit

n_dims=dims(0)
dim0=dims(1)
dim1=dims(2)
n_slices=dims(3)
n_pts=dims(4)

print, "number of dimensions = ", n_dims
if (n_dims gt 0) then print, "number of cols = ", dim0
if (n_dims gt 1) then print, "number of rows = ", dim1
if (n_dims gt 2) then print, "number of slices = ", n_slices
if (n_dims gt 3) then print, "number of volumes = ", n_pts

im=complexarr(dim0, dim1, n_slices, n_pts)

get_lun, unit
openr, unit, fname+'.img'
readu, unit, im
close, unit
free_lun, unit

im_thresh=0.05
max_abs=max(abs(im))

; read in the inter_echo time
np=get_varian_par(dir_stem,'np')
sw=get_varian_par(dir_stem,'sw')
grise=get_varian_par(dir_stem,'grise_copy')
gmax=get_varian_par(dir_stem,'gmax')
lro=get_varian_par(dir_stem,'lro')
; tau_pe is the inter-echo delay in ms
tau_pe=(np/(2.0*sw)+2.0*(sw*grise)/(lro*4257.5*gmax))*1000.0

S0_map=fltarr(dim0,dim1,n_slices)
t2star_map=fltarr(dim0,dim1,n_slices)
field_map=fltarr(dim0,dim1,n_slices)
t2star_resid_map=fltarr(dim0,dim1,n_slices)
field_resid_map=fltarr(dim0,dim1,n_slices)

for k=0, n_slices-1 do begin
  print, "processing slice", k+1
  for i=0, dim0-1 do begin
    for j=0, dim1-1 do begin
      prof=im(i,j,k,*)
      if (max(abs(prof)) gt im_thresh*max_abs) then begin
        tmp=t2star_curve_fit(reform(prof),tau_pe)
        S0_map(i,j,k)=tmp(0)
        t2star_map(i,j,k)=tmp(1)
        field_map(i,j,k)=tmp(2)
        t2star_resid_map(i,j,k)=tmp(3)
        field_resid_map(i,j,k)=tmp(4)
      endif
    endfor
  endfor
endfor

; spot outliers
mask=where(t2star_map gt 1000)
if (total(mask) gt -1) then begin
  t2star_map(mask)=0
  t2star_resid_map(mask)=0
  field_map(mask)=0
  field_resid_map(mask)=0
endif

get_lun, unit
openw, unit, dir_stem+'/fid_t2star.img'
writeu, unit, t2star_map
close, unit
free_lun, unit

; calcualte enough ANALYZE header info that Medx can read it
write_analyze_header, size(t2star_map), dir_stem, dir_stem+'/fid_t2star.hdr'

get_lun, unit
openw, unit, dir_stem+'/fid_field.img'
writeu, unit, field_map
close, unit
free_lun, unit

; calcualte enough ANALYZE header info that Medx can read it
write_analyze_header, size(field_map), dir_stem, dir_stem+'/fid_field.hdr'

goto, end_file

cant_read_file:

print, "Can't read file..."

end_file:

end
