
calculate_tag_inversion_gradients(double *t_taginv_b,double *t_taginv_a)
{
  char shape_name[MAXSTR];
  double t_cr;

  sprintf(shape_name,"gss5");
  pw_gss5 = create_trap_gradient(fabs(gss5),shape_name,
         p5+rof1+rof2,grate,ix);

  *t_taginv_b = fabs(gss5)*grate + rof1 + p5/2;
  *t_taginv_a = fabs(gss5)*grate + rof2 + p5/2 + pw_gss5;
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
    calculate_crusher_gradients(&t_cr);
    *t_taginv_a=*t_taginv_a + t_cr;
  }
}
calculate_tag_saturation_gradients(double *t_tagsat_b,double *t_tagsat_a)
{
  char shape_name[MAXSTR];
  double t_cr;

  sprintf(shape_name,"gss6");
  pw_gss6 = create_trap_gradient(fabs(gss6),shape_name,
         p6+rof1+rof2,grate,ix);

  *t_tagsat_b = fabs(gss6)*grate + rof1 + p6/2;
  *t_tagsat_a = fabs(gss6)*grate + rof2 + p6/2;
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
    calculate_crusher_gradients(&t_cr);
    *t_tagsat_a=*t_tagsat_a + t_cr;
  }
}
calculate_spatial_saturation_gradients(double *t_presat_b,double *t_presat_a)
{
  char shape_name[MAXSTR];
  double t_cr;

  sprintf(shape_name,"gsssat");
  pw_gsssat = create_trap_gradient(fabs(gsssat),shape_name,
         psat+rof1+rof2,grate,ix);

  *t_presat_b = fabs(gsssat)*grate + rof1 + psat/2;
  *t_presat_a = fabs(gsssat)*grate + rof2 + psat/2 + post_presat_del;
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
    calculate_crusher_gradients(&t_cr);
    *t_presat_a=*t_presat_a + t_cr;
  }
}
calculate_inversion_gradients(double *t_inv_b,double *t_inv_a)
{
  char shape_name[MAXSTR];

  sprintf(shape_name,"gss3");
  pw_gss3 = create_trap_gradient(fabs(gss3),shape_name,p3+rof1+rof2,grate,ix);

  *t_inv_b = fabs(gss3)*grate + rof1 + p3/2;
  *t_inv_a = fabs(gss3)*grate + rof2 + p3/2 + pw_gss3;
}
calculate_navigator_gradients(double *t_nav)
{
  char shape_name[MAXSTR];
  double nucgamma,area,ttrap,tft;

  nucgamma = 1e6*sfrq/B0;
  gnav = sw/lro/nucgamma;
  tft = np/2/sw;

  sprintf(shape_name,"gnav");
  ttrap = create_trap_gradient(fabs(gnav),shape_name,tft,grate,ix);
  pw_gnav = (ttrap-tft)/2;

  area = (pw_gnav + tft) * fabs(gnav) / 2;
  if (grise*gmax >= area) {
    pw_gnav_r = sqrt(area*grate);
    gnav_r = area/pw_gnav_r;
    tnav_r = area/gnav_r;
  }
  else {
    tnav_r = area/gmax;
    gnav_r = gmax;
  }

  sprintf(shape_name,"gnav_r");
  pw_gnav_r = create_ramp_gradient(fabs(gnav_r),shape_name,1,grate,ix);
  pw_gnav_r = create_ramp_gradient(fabs(gnav_r),shape_name,-1,grate,ix);
  
  if(tnav_r < pw_gnav_r) tnav_r = pw_gnav_r;
  if(pw_gnav_r<tep){
  	*t_nav = ttrap + 2*(pw_gnav_r + tnav_r) + tep - pw_gnav_r;
  }
  else {
  	*t_nav = ttrap + 2*(pw_gnav_r + tnav_r);  
  }

}
calculate_crusher_gradients(double *t_cr)
{
  char shape_name[MAXSTR];
  char err_string[MAXSTR];
  
  sprintf(shape_name,"gcrush");
  pw_gcrush = create_ramp_gradient(fabs(gcrush),shape_name,1,grate,ix);
  pw_gcrush = create_ramp_gradient(fabs(gcrush),shape_name,-1,grate,ix);
  if (pw_gcrush > tcrush) {
    sprintf(err_string,"%s: tcrush too small. Min tcrush = %f\n",
	    seqfil,pw_gcrush);
    printf(err_string);
    log_error(err_string);
    abort(1);
  }
  *t_cr = pw_gcrush + tcrush;
}
calculate_fatsat_gradients(double *t_fat)
{
  char shape_name[MAXSTR];

  sprintf(shape_name,"gfat");
  pw_gfat=create_trap_gradient(fabs(gmax)/2.0,shape_name,3.0e-3,grate,ix);
  *t_fat = rof1 + p4 + rof2 + pw_gfat;
}
calculate_excitation_gradients(double *t_ex_b,double *t_ex_a)
{
  double agss,trefmin,gssrint;
  char shape_name[MAXSTR];
  char err_string[MAXSTR];

  agss = fabs(gss); /* absolute slice select gradient strength */
  if (agss == 0.0) {
    sprintf(err_string,"%s: Zero slice select gradient\n",seqfil);
    printf(err_string);
    log_error(err_string);
    abort(1);
  }
  
  sprintf(shape_name,"gss");
  pw_gss=create_trap_gradient(agss,shape_name,rof1+p1+rof2,grate,ix);

  gssrint = agss*(p1/2.0 + rof2 + agss*grate/2.0);
  /* if the gssrint can be done as a triangular waveform then set it up */
  if (grise*gmax >= gssrint*(1.0+epigssf/100.0)) {
    trefmin = sqrt(grate*gssrint*(1.0+epigssf/100.0)); 
    /* since g_required=trefmin/grate */
  }
  /* otherwise set up waveform as a trapezoidal shape and hit the rail */
  else {
    trefmin = grate*gmax + gssrint*(1.0+epigssf/100.0)/gmax;
  }

  /* tref is the refocussing block length, not including gradient fall time */
  tref = trefmin;

  gssr = -gssrint*(1.0+epigssf/100.0)/tref;

  sprintf(shape_name,"gssr");
  pw_gssr=create_ramp_gradient(fabs(gssr),shape_name,1,grate,ix);
  pw_gssr=create_ramp_gradient(fabs(gssr),shape_name,-1,grate,ix);

  if (tref < pw_gssr) tref=pw_gssr; 

  *t_ex_b = agss*grate + tep + rof1 + p1/2;
  if(tep<fabs(gss)*grate) *t_ex_a = agss*grate - tep;
  else *t_ex_a = tep;
  *t_ex_a += p1/2 + rof2 + pw_gssr + tref;
}
calculate_spinecho_gradients(double *t_se_b,double *t_se_a)
{
  char shape_name[MAXSTR];
  char err_string[MAXSTR];
  double t_pulse;

  t_pulse = rof1+p2+rof2;

  sprintf(shape_name,"gss2");
  pw_gss2=create_spin_echo_gradient(shape_name,fabs(gss2),t_pulse,gspoil,
				    tspoil,tspoil,grate,ix);
  pw_gse_crush=(pw_gss2-t_pulse)/2.0;

  if(pw_gse_crush < tep) { 
    sprintf(err_string,"%s: tspoil too short. Min tspoil = %f\n",
	    seqfil, tep-pw_gse_crush);
    printf(err_string);
    log_error(err_string);
    abort(1);
  }

  *t_se_b = pw_gse_crush + tep + rof1 + p2/2;
  *t_se_a = pw_gse_crush - tep + rof2 + p2/2;
}
calculate_epi_gradients(double *t_epi_b,double *t_epi_a)
{
  double nucgamma,t_echo,pseudo_dwell_time,ramp;
  double grorint,gpeint,gbdint,maxgint,trefmin2,tbpe;
  int points;
  char err_string[MAXSTR];
  char shape_name[MAXSTR];
  unsigned long atdiv;

  nucgamma = 1e6*sfrq/B0;
  gro = sw/lro/nucgamma;

  if(nonlinsamp[0]=='y'){
    sampwait = getval("sampwait");
    sampwait/=1e6;
    points=np/2;
    
    /* first pass calculation */
    ramp=fabs(gro)*grate;
    at=(points/sw)-ramp+(sampwait*sampwait/ramp);
    atdiv=(unsigned long)(at/2.0e-6 + 0.5);
    at=atdiv*2.0e-6;
    pseudo_dwell_time=at/points;
    sprintf(shape_name,"epi_read");
    pw_gro=create_epi_read_gradients(fabs(gro),shape_name,grate,pseudo_dwell_time,
				    (int)np/2,(int)etl,epigroa,echo_del,ix);
    /* second pass calculation */
    ramp=pw_gro;
    at=(points/sw)-ramp+(sampwait*sampwait/ramp);
    atdiv=(unsigned long)(at/2.0e-6 + 0.5);
    at=atdiv*2.0e-6;
    pseudo_dwell_time=at/points;
  }
  else {
    pseudo_dwell_time=1.0/sw;
    at = (np/2)/sw;
  }

  sprintf(shape_name,"epi_read");
  pw_gro=create_epi_read_gradients(fabs(gro),shape_name,grate,pseudo_dwell_time,
				    (int)np/2,(int)etl,epigroa,echo_del,ix);
			
  sprintf(shape_name,"epi_read_rev");
  pw_gro=create_epi_read_gradients(fabs(gro),shape_name,grate,pseudo_dwell_time,
				    (int)np/2,(int)etl,-1*epigroa,echo_del,ix);
			
  sprintf(shape_name,"epi_phase");
  pw_gro=create_epi_phase_gradients(shape_name,pw_gro,pseudo_dwell_time,
				     (int)np/2,(int)etl,echo_del,ix);

  grorint = fabs(gro)*(at/2.0 + pw_gro/2.0);
  gpeint = B0 * (double)num_ints / (lpe*sfrq*1e6); /* int num_ints blips */
  gbdint = gpeint * eff_echo; /* phase encode dephaser g-t integral */
 
  /* Minimum refocussing block time for readout and phase rewinder */
  /* figure out the bigger of gbdint and grorint  giving the maxgint */
  maxgint = grorint*(1.0+epigrof/100.0) > gbdint*(1.0+gbdf/100.0) ?
    grorint*(1.0+epigrof/100.0) : gbdint*(1.0+gbdf/100.0);

  if (grise*gmax >= maxgint) {
    trefmin2 = sqrt(grate*maxgint);
    pw_gror = trefmin2;
  }
  else {
    trefmin2 = grate*gmax + maxgint/gmax;
    pw_gror = grise;
  }

  /* Create tref2 for readout and phase encode rewinder */
  tref2 = trefmin2;

  /* Make sure that pw_gror is not less than MAX_GRAD_RES*GRAD_UPDATE_TIME
     and if it is then generate a file that is close to within the nearest
     integral of GRAD_UPDATE_TIME */
  sprintf(shape_name,"gdephase");
  pw_gror=create_ramp_gradient(maxgint/tref2,shape_name,1,grate,ix);
  pw_gror=create_ramp_gradient(maxgint/tref2,shape_name,-1,grate,ix);

  if (tref2 < pw_gror) tref2=pw_gror;

  /* Calculate gbd and gror with new tref2 */
  gror = -grorint*(1.0+epigrof/100.0)/tref2;

  tbpe = pw_gro+echo_del;
  sgbpe = gpeint/tbpe;
  sgbpe = gmax/gradstepsz*floor(sgbpe/gmax*gradstepsz+0.5);

  gbd = -gbdint*(1.0+gbdf/100.0)/tref2;
  gbd = gmax/gradstepsz*floor(gbd/gmax*gradstepsz+0.5);

  sgbd = (sgbpe*tbpe)/((double) num_ints)/tref2;
  sgbd = gmax/gradstepsz*floor(sgbd/gmax*gradstepsz+0.5);

  if (grate*fabs(sgbpe) > pw_gro) {
    sprintf(err_string,"%s: Blips error. Increase sw, lpe or echo_del\n",
            seqfil);
    printf(err_string);
    log_error(err_string);
    sprintf(err_string,"%s: Could increase echo_del to %f\n",
            seqfil, echo_del*1.0e6+(grate*fabs(sgbpe)-pw_gro)*1000000.0);
    printf(err_string);
    log_error(err_string);
    abort(1);
  }
  
  /* Calculate timing */
  t_echo = pw_gro + episwing + at + pw_gro + echo_del;
  *t_epi_b = pw_gror + tref2 + t_echo*eff_echo + 2*pw_gro + episwing + at/2.0;
  *t_epi_a = t_echo*(etl-eff_echo-1.0) + at/2.0 + 2*pw_gro + echo_del;
}
calculate_nonlinear_sampling(void)
{
  int points,pt;
  double ramp,total,ns,prev,target,area,sum;
  char err_string[MAXSTR];
  
  points=np/2;
  if(points>SAMPPTS-1){
    sprintf(err_string,"%s: samptime array too small\n",
	    seqfil);
    printf(err_string);
    log_error(err_string);
    abort(1);
  }

  ramp=pw_gro;
  total=at+2*ramp;
  area=1.0/sw;
  ns=prev=sampwait;
  for(pt=0;pt<points;pt++){
    target=((pt+1)*area)+((sampwait*sampwait)/(2*ramp));
    do{
      if(ns==prev)ns+=area;
      else ns+=1.0e-9;
      if(ns<=ramp)sum=(ns*ns)/(2*ramp);
      else sum=ns-(ramp/2);
      if(ns>(ramp+at))sum=ramp+at-(((total-ns)*(total-ns))/(2*ramp));
      if(ns>total+1.0e-9){
	sprintf(err_string,"%s: Error in calculating sampling times\n",seqfil);
	printf(err_string);
	log_error(err_string);
	abort(1);
      }
    }while(sum<target);
    samptime[pt]=ns-prev;
    prev=ns;
  }
  samptime[points]=total-ns;
  if(samptime[points]<0)samptime[points]=0;
}

calculate_rf_modulation()
{
  double pss1,pss2,pwr1,pwr2,target,newtarg,dbi;
  int presat_off_pwr_mod,db_presat_off;
  int tag_off_pwr_mod,db_tag_off;
  char tpwr_file[MAXSTR],rfcoil[MAXSTR],full_fname[MAXSTR],*home_dir;
  int pwr_mod[MAXSTR],db_incr[MAXSTR],i,ret;
  FILE *fp,*table;
  char err_string[MAXSTR];

  getstr("rfcoil",rfcoil);

  home_dir=(char *)getenv("HOME");
  sprintf(tpwr_file,"%s/vnmrsys/shapelib/%s.SCALE",home_dir,rfcoil);

  fp=fopen(tpwr_file,"r");
  if(fp==NULL){
    sprintf(err_string,"%s: Unable to open file ~/vnmrsys/shapelib/%s.SCALE\n"
	    ,seqfil,rfcoil);
    printf(err_string);
    log_error(err_string);
    abort(1);
  }

  for(i=0;i<ns;i++){
    rewind(fp);
    ret=fscanf(fp,"%lf\t%lf\n",&pss1,&pwr1);
    rewind(fp);
    while(1){
      ret=fscanf(fp,"%lf\t%lf\n",&pss2,&pwr2);
      if(ret==EOF){
	sprintf(err_string,"%s: pss out of range of tpwr_file\n",seqfil);
	printf(err_string);
	log_error(err_string);
	abort(1);
      }
      if(pss2>pss[i]){
	target=pwr1;
	pwr2-=pwr1;
	pwr2*=((pss[i]-pss1)/(pss2-pss1));
	target+=pwr2;
	break;
      }
      pss1=pss2;
      pwr1=pwr2;
    }
    /* calculate db increase */
    dbi=20*log10(target);
    db_incr[i]=ceil(dbi);
    newtarg=pow(10,(double)db_incr[i]/20);
    newtarg=target/newtarg;
    pwr_mod[i]=(int)(newtarg*32767);
  }

    /* ********************************** */
    /* next work out any presat_off value */
    /* ********************************** */
    rewind(fp);
    ret=fscanf(fp,"%lf\t%lf\n",&pss1,&pwr1);
    rewind(fp);
    while(1){
      ret=fscanf(fp,"%lf\t%lf\n",&pss2,&pwr2);
      if(ret==EOF){
	sprintf(err_string,"%s: presat_off out of range of tpwr_file\n",seqfil);
	printf(err_string);
	log_error(err_string);
	abort(1);
      }
      if(pss2>presat_off){
	target=pwr1;
	pwr2-=pwr1;
	pwr2*=((presat_off-pss1)/(pss2-pss1));
	target+=pwr2;
	break;
      }
      pss1=pss2;
      pwr1=pwr2;
    }
    /* calculate db increase */
    dbi=20*log10(target);
    db_presat_off=ceil(dbi);
    newtarg=pow(10,(double)db_presat_off/20);
    newtarg=target/newtarg;
    presat_off_pwr_mod=(int)(newtarg*32767);

    /* ******************************* */
    /* next work out any tag_off value */
    /* ******************************* */
    rewind(fp);
    ret=fscanf(fp,"%lf\t%lf\n",&pss1,&pwr1);
    rewind(fp);
    while(1){
      ret=fscanf(fp,"%lf\t%lf\n",&pss2,&pwr2);
      if(ret==EOF){
	sprintf(err_string,"%s: tag_off out of range of tpwr_file\n",seqfil);
	printf(err_string);
	log_error(err_string);
	abort(1);
      }
      if(pss2>tag_off){
	target=pwr1;
	pwr2-=pwr1;
	pwr2*=((tag_off-pss1)/(pss2-pss1));
	target+=pwr2;
	break;
      }
      pss1=pss2;
      pwr1=pwr2;
    }
    /* calculate db increase */
    dbi=20*log10(target);
    db_tag_off=ceil(dbi);
    newtarg=pow(10,(double)db_tag_off/20);
    newtarg=target/newtarg;
    tag_off_pwr_mod=(int)(newtarg*32767);

  fclose(fp);

  sprintf(full_fname,"%s/vnmrsys/tablib/tpwr_table",home_dir);
  table=fopen(full_fname,"w");
  if(table==NULL){
    sprintf(err_string,"%s: Unable to open ~/vnmrsys/tablib/tpwr_table\n",
	    seqfil);
    printf(err_string);
    log_error(err_string);
    abort(1);
  }
  fprintf(table,"t1 = ");
  for(i=0;i<ns;i++){
    fprintf(table,"%d ",db_incr[i]+(int)tpwr1);
  }
  fprintf(table,"\nt2 = ");
  for(i=0;i<ns;i++){
    fprintf(table,"%d ",db_incr[i]+(int)tpwr2);
  }
  fprintf(table,"\nt3 = ");
  for(i=0;i<ns;i++){
    fprintf(table,"%d ",db_incr[i]+(int)tpwr3);
  }
  fprintf(table,"\nt4 = ");
  for(i=0;i<ns;i++){
    fprintf(table,"%d ",db_incr[i]+(int)tpwr4);
  }
  fprintf(table,"\nt5 = ");
  for(i=0;i<ns;i++){
    fprintf(table,"%d ",db_incr[i]+(int)tpwr5);
  }
  fprintf(table,"\nt58 = ");  /* correction to nearest dB */
    fprintf(table,"%d ",db_presat_off+(int)tpwr6);
    fprintf(table,"%d ",db_tag_off+(int)tpwr7);
  fprintf(table,"\nt59 = ");  /* fine correction using tpwrf */
    fprintf(table,"%d ",presat_off_pwr_mod);
    fprintf(table,"%d ",tag_off_pwr_mod);
  fprintf(table,"\nt60 = ");
  for(i=0;i<ns;i++){
    fprintf(table,"%d ",pwr_mod[i]);
  }
  fprintf(table,"\n");
  fclose(table);
  
  loadtable("tpwr_table");
}

