/*
  DANTE Tagging gradients and RF module.
  Requires the following parameters in the current experiment:

  create('dante_flag','flag')
  dante_flag='y'
  create('dante_pw90','pulse')
  create('dante_spacing','real')
  create('dante_direction','string')
  dante_direction='r'
  create('num_dante_pulses','real')
  num_dante_pulses=16
  create('dante_tpwr','integer')
  setlimit('dante_tpwr',17)
  create('dante_grad','real')
  setlimit('dante_grad',1000,-1000,0)
  dante_grad=1.0
  create('dante_gcrush','real')
  setlimit('dante_gcrush',1000,-1000,0)
  dante_gcrush=gmax/2.0
  create('dante_tcrush','delay')
  dante_tcrush=3.0e-3
*/

/* global variables */
double  dante_pw90,dante_del,dante_tpwr,dante_grad,dante_gcrush,dante_tcrush;
double  dante_spacing,num_dante_pulses;
double  pw_dante,pw_dante_cr,t_dante;
char    dante_direction[MAXSTR],dante_flag[MAXSTR];

calculate_dante_gradients(double *t_dante)
{
  char shape_name[MAXSTR];

  getstr("dante_direction",dante_direction);
  dante_spacing=getval("dante_spacing");
  dante_tpwr=getval("dante_tpwr");
  dante_grad=getval("dante_grad");
  dante_pw90=getval("dante_pw90");
  num_dante_pulses=getval("num_dante_pulses");

  sprintf(shape_name,"dante");
  pw_dante = create_ramp_gradient(fabs(dante_grad),shape_name,1,grate,ix);
  pw_dante = create_ramp_gradient(fabs(dante_grad),shape_name,-1,grate,ix);

  dante_del=B0/(dante_grad*sfrq*10000.0*dante_spacing*10.0) 
         - dante_pw90/num_dante_pulses -rof1 - rof2;
  if (dante_del < 0.0) {
    sprintf(err_string,"%s: dante_spacing too large\n",seqfil);
    printf(err_string);
    log_error(err_string);
    abort(1);
  }

 *t_dante = 2.0*pw_dante+num_dante_pulses*(dante_del+rof1+rof2)+dante_pw90;

  dante_gcrush=getval("dante_gcrush");
  dante_tcrush=getval("dante_tcrush");
  if ((fabs(dante_gcrush) > 0.0) && (dante_tcrush > 0.0)) {
    sprintf(shape_name,"dante_cr");
    pw_dante_cr = create_trap_gradient(fabs(dante_gcrush),shape_name,
         dante_tcrush,grate,ix);
    *t_dante = *t_dante + pw_dante_cr;
  }
}

dante_tagging()
{
  int i;

  obspower(dante_tpwr);
  if (dante_direction[0] == 'r') {
    obl_shapedgradient("dante_up","","",pw_dante,dante_grad,0.0,0.0,1,WAIT);
  } else {  /* assume phase encode */
    obl_shapedgradient("","dante_up","",pw_dante,0.0,dante_grad,0.0,1,WAIT);
  }
  for (i=0; i<(int)num_dante_pulses; i++) {
     sp4on();
     pulse(dante_pw90/num_dante_pulses,zero);
     sp4off();
     delay(dante_del);
  }
  if (dante_direction[0] == 'r') {
    obl_shapedgradient("dante_dn","","",pw_dante,dante_grad,0.0,0.0,1,WAIT);
    /* crusher */
    if ((fabs(dante_gcrush) > 0.0) && (dante_tcrush > 0.0)) {
      obl_shapedgradient("","dante_cr","dante_cr",pw_dante_cr,
             0.0,dante_gcrush,dante_gcrush,1,WAIT);
    }
  } else {  /* assume phase encode */
    obl_shapedgradient("","dante_dn","",pw_dante,0.0,dante_grad,0.0,1,WAIT);
    /* crusher */
    if ((fabs(dante_gcrush) > 0.0) && (dante_tcrush > 0.0)) {
      obl_shapedgradient("dante_cr","","dante_cr",pw_dante_cr,
             dante_gcrush,0.0,dante_gcrush,1,WAIT);
    }
  }
}
