#include <math.h>

logical_to_physical(double phi,double psi,double theta,
		    double logx,double logy,double logz,
		    double *phyx,double *phyy,double *phyz)
{
  float cosang1,sinang1,cosang2,sinang2,cosang3,sinang3;
  float m11,m12,m13,m21,m22,m23,m31,m32,m33,tol;
  float tm11,tm12,tm13,tm21,tm22,tm23,tm31,tm32,tm33;

  cosang1=cos(M_PI/180.0*psi);
  sinang1=sin(M_PI/180.0*psi);

  cosang2=cos(M_PI/180.0*phi);
  sinang2=sin(M_PI/180.0*phi);

  cosang3=cos(M_PI/180.0*theta);
  sinang3=sin(M_PI/180.0*theta);

  m11 = (sinang2*cosang1 - cosang2*cosang3*sinang1);
  m12 = (-1.0*sinang2*sinang1 - cosang2*cosang3*cosang1);
  m13 = (sinang3*cosang2);

  m21 = (-1.0*cosang2*cosang1 - sinang2*cosang3*sinang1);
  m22 = (cosang2*sinang1 - sinang2*cosang3*cosang1);
  m23 = (sinang3*sinang2);

  m31 = (sinang1*sinang3);
  m32 = (cosang1*sinang3);
  m33 = (cosang3);

  tol = 1.0e-14;

  if (fabs(m11) < tol) m11 = 0.0;
  if (fabs(m12) < tol) m12 = 0.0;
  if (fabs(m13) < tol) m13 = 0.0;
  if (fabs(m21) < tol) m21 = 0.0;
  if (fabs(m22) < tol) m22 = 0.0;
  if (fabs(m23) < tol) m23 = 0.0;
  if (fabs(m31) < tol) m31 = 0.0;
  if (fabs(m32) < tol) m32 = 0.0;
  if (fabs(m33) < tol) m33 = 0.0;

  /* Transpose matrix */

  tm11 = m11;
  tm21 = m12;
  tm31 = m13;
  tm12 = m21;
  tm22 = m22;
  tm32 = m23;
  tm13 = m31;
  tm23 = m32;
  tm33 = m33;

  /* Apply matrix elements to logical axes */

  *phyx = fabs(logx*tm11 + logy*tm12 + logz*tm13);
  *phyy = fabs(logx*tm21 + logy*tm22 + logz*tm23);
  *phyz = fabs(logx*tm31 + logy*tm32 + logz*tm33);
}

