int shapefile_exists(char* filename)
{
  FILE *fp;
  char file[MAXSTR];
  sprintf(file,"%s/shapelib/%s",userdir,filename);
  if((fp=fopen(file,"r"))==NULL){
    sprintf(file,"%s/shapelib/%s",systemdir,filename);
    if(fopen(file,"r")==NULL){
      return(0);
    }
    fclose(fp);
  }
  fclose(fp);
  return(1);
}

int check_rf_with_grad(char* filename)
{
  char file[MAXSTR];
  int n=0;

  /* Find RF file */
  sprintf(file,"%s.1.RF",filename);
  if(!shapefile_exists(file)){
    sprintf(file,"%s.RF",filename);
    if(!shapefile_exists(file)){
      printf("Cannot find RF pulse: %s\n",filename);
      return(0);
    }
    else n=1;
  }
  else {
    n=1;
    while(1){
      sprintf(file,"%s.%d.RF",filename,n+1);
      if(!shapefile_exists(file))break;
      n++;
    }
    /* Avoid confusion between file.RF and file.1.RF if n=1 */
    if(n==1)n=0;
  }

  /* Find GRD file */
  sprintf(file,"%s.GRD",filename);
  if(shapefile_exists(file)){
    return(n);
  } else {
    return(0);
  }
}

int calculate_rf_with_grad(double *t_ex_b,double *t_ex_a)
{
  char err_string[MAXSTR],shape_name[MAXSTR];
  char file[MAXSTR],string[MAXSTR];
  double trefmin,gssrint;
  double refoc;
  FILE *fp;
  
  sprintf(file,"%s/shapelib/%s.GRD",userdir,p1pat);
  if((fp=fopen(file,"r"))==NULL){
    sprintf(file,"%s/shapelib/%s.GRD",systemdir,p1pat);
    if((fp=fopen(file,"r"))==NULL){
      printf("Cannot find GRD file: %s\n",file);
      return(0);
    }
  }
  refoc=0.0;
  while(fgets(string,MAXSTR,fp)!=NULL){
    sscanf(string,"# REFOCUS %lf",&refoc);
  }
  fclose(fp);

  if(refoc==0.0){
    sprintf(err_string,"%s: REFOCUS tag not found in %s\n\n",seqfil,file);
    printf(err_string);
    log_error(err_string);
    abort(1);
  }

  gssrint = fabs(gss*refoc*p1);

  /* if the gssrint can be done as a triangular waveform then set it up */
  if (grise*gmax >= gssrint*(1.0+epigssf/100.0)) {
    trefmin = sqrt(grate*gssrint*(1.0+epigssf/100.0)); 
    /* since g_required=trefmin/grate */
  }
  /* otherwise set up waveform as a trapezoidal shape and hit the rail */
  else {
    trefmin = grate*gmax + gssrint*(1.0+epigssf/100.0)/gmax;
  }

  /* tref is the refocussing block length, not including gradient fall time */
  tref = trefmin;

  if(refoc<0.0){
    gssr = -gssrint*(1.0+epigssf/100.0)/tref;
  } else {
    gssr = gssrint*(1.0+epigssf/100.0)/tref;
  }

  sprintf(shape_name,"gssr");
  pw_gssr=create_ramp_gradient(fabs(gssr),shape_name,1,grate,ix);
  pw_gssr=create_ramp_gradient(fabs(gssr),shape_name,-1,grate,ix);

  if (tref < pw_gssr) tref=pw_gssr;

  *t_ex_b = rftep + rof1 + p1/2;
  *t_ex_a = p1/2 + rof2 + pw_gssr + tref;
}

void rf_with_grad_pulse(codeint vslice,int n)
{
  int i;
  char file[MAXSTR];

  if((ns>1)&&(exlist<0))exlist=curlist++;
  position_offset_list(pss,gss,ns,resto,OBSch,exlist,vslice);
  if(rfmod[0]=='y'){
    getelem(t1,vslice,v12);
    power(v12,OBSch);
    getelem(t60,vslice,v12);
  }
  else obspower(tpwr1);

  /* put out trigger on sp1 every spin excitation */
  sp1on();
  /* put out trigger on sp2 every volume */
  ifzero(v9);
    ifzero(v14);
       sp2on();
    endif(v14);
  endif(v9);

  obl_shapedgradient("","",p1pat,p1,0.0,0.0,gss,1,NOWAIT);
  delay(rftep);
  if(n==1){
    sp4on();
    if(rfmod[0]=='y') shapedvpulse(p1pat,p1,v12,v1,rof1,rof2);
    else shaped_pulse(p1pat,p1,v1,rof1,rof2);
    sp4off();
  } else {
    for(i=1;i<=n;i++){
      if(i%2)
	position_offset_list(pss,gss,ns,resto,OBSch,exlist,vslice);
      else
	position_offset_list(pss,-gss,ns,resto,OBSch,exlist,vslice);
      sprintf(file,"%s.%d",p1pat,i);
      sp4on();
      if(rfmod[0]=='y') shapedvpulse(file,(p1/n)-rof1-rof2,v12,v1,rof1,rof2);
      else shaped_pulse(file,(p1/n)-rof1-rof2,v1,rof1,rof2);
      sp4off();
    }
  }
  
  sp1off();
  sp2off();

  /* refocus pulse */
  obl_shapedgradient("","","gssr_up",pw_gssr,0.0,0.0,gssr,1,WAIT);
  delay(tref-pw_gssr);
  obl_shapedgradient("","","gssr_dn",pw_gssr,0.0,0.0,gssr,1,WAIT);
}
