/* Modified by Patricia, January 2000 */
/*
                       QUIPSS2 tags:

    uses p6    uses p5       uses p7
      90         inv        90     90
     slice       tag        tag    tag
    |                                            |
 rf |-oOo------o0X0X0o------o0o----o0o-----------| -
          _             _       _      _         |  \
    |  _ |*|   _______ |*|   _ |*|  _ |*|        |  |
    |_/ \|*|__/       \|*|__/ \|*|_/ \|*|________|  |
GSS |   crush         crush   crush  crush       |  |
    |             |<------TI1------>|            |  /
    |             |<-------------TI2------------>|  > EPI MODULE
    |                                            |  \
GPE |--------------------------------------------|  |
    |                                            |  |
    |                                            |  |
GRO |--------------------------------------------|  /
    |                                            | -
*/
/*
  RF Pulses
  p1 : see epi_defs.txt
  p2 : see epi_defs.txt
  p3 : see epi_defs.txt
  p4 : see epi_defs.txt
  p5 : tpwr5 : p5pat : gss5 - Inversion tagging slab (FAIR, QUIPSS)
  p6 : tpwr6 : p6pat : gss6 - Saturation of tagging slab (QUIPSS)
  psat : satpwr : satpat : gsssat - see epi_defs.txt
*/

/*
  Tables
  t1 : see epi_defs.txt
  t2 : see epi_defs.txt
  t3 : see epi_defs.txt
  t4 : see epi_defs.txt
  t5 : see epi_defs.txt
  t58 : presat_off tag_off    [dB correction]
  t59 : presat_off tag_off    [tpwrf correction]
  t60 : see epi_defs.txt
*/

/* global variables */
double mean_pss;
double tag_off=0.0,control_tag_off,tag_direction,tag_thk,tag_sign=1.0;
double tag_gap=1.0;
double tag_invert=0.0,tag_sat=0.0;    /* QUIPSS flags (presat in globals.h) */
double pw_flow, gflow, tflow, gflow_flag;
double foci_factor, pw_gss5_r;
char foci[MAXSTR];

/* Calculate */

calculate_macro_flow_suppress_gradients(double *t_macro_flow_suppress)
{
  char shape_name[MAXSTR];

  sprintf(shape_name,"gflow");
  pw_flow = create_trap_gradient(fabs(gflow),shape_name, tflow, grate, ix);

  *t_macro_flow_suppress = 2.0*pw_flow;
}

calculate_tag_inversion_gradients(double *t_taginv_b,double *t_taginv_a)
{
  char shape_name[MAXSTR];
  double t_cr;

  sprintf(shape_name,"gss5");
  pw_gss5 = create_trap_gradient(fabs(gss5),shape_name,
         p5+rof1+rof2,grate,ix);

  *t_taginv_b = fabs(gss5)*grate + rof1 + p5/2;
  *t_taginv_a = fabs(gss5)*grate + rof2 + p5/2 + pw_gss5;
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
    calculate_crusher_gradients(&t_cr);
    *t_taginv_a=*t_taginv_a + t_cr;
  }
}

calculate_foci_inversion_gradients(double *t_taginv_b,double *t_taginv_a)
{
  char shape_name[MAXSTR];
  double t_cr;

  sprintf(shape_name,"gss5r");
  pw_gss5_r = create_ramp_gradient(fabs(gss5),shape_name,1,grate,ix);
  pw_gss5_r = create_ramp_gradient(fabs(gss5),shape_name,-1,grate,ix);
  pw_gss5 = rof1 + p5 + rof2;
  *t_taginv_b = fabs(gss5)*grate + rof1 + p5/2;
  *t_taginv_a = fabs(gss5)*grate + rof2 + p5/2 + 2.0*fabs(gss5)*grate+pw_gss5;
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
    calculate_crusher_gradients(&t_cr);
    *t_taginv_a=*t_taginv_a + t_cr;
  }
}

calculate_tag_saturation_gradients(double *t_tagsat_b,double *t_tagsat_a)
{
  char shape_name[MAXSTR];
  double t_cr;

  sprintf(shape_name,"gss6");
  pw_gss6 = create_trap_gradient(fabs(gss6),shape_name,
         p6+rof1+rof2,grate,ix);

  *t_tagsat_b = fabs(gss6)*grate + rof1 + p6/2;
  *t_tagsat_a = fabs(gss6)*grate + rof2 + p6/2;
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
    calculate_crusher_gradients(&t_cr);
    *t_tagsat_a=*t_tagsat_a + t_cr;
  }
}

calculate_tag_rf_modulation()
{
  double pss1,pss2,pwr1,pwr2,target,newtarg,dbi;
  int presat_off_pwr_mod,db_presat_off;
  int tag_off_pwr_mod,db_tag_off;
  char tpwr_file[MAXSTR],rfcoil[MAXSTR],full_fname[MAXSTR],*home_dir;
  int ret;
  FILE *fp,*table;
  char err_string[MAXSTR];

  getstr("rfcoil",rfcoil);

  home_dir=(char *)getenv("HOME");
  sprintf(tpwr_file,"%s/vnmrsys/shapelib/%s.SCALE",home_dir,rfcoil);

  fp=fopen(tpwr_file,"r");
  if(fp==NULL){
    sprintf(tpwr_file,"/vnmr/shapelib/%s.SCALE",rfcoil);
    fp=fopen(tpwr_file,"r");
    if(fp==NULL){
      sprintf(err_string,"%s: Unable to open file ~/vnmrsys/shapelib/%s.SCALE\n"
	      ,seqfil,rfcoil);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
  }

  /* ***************************** */
  /* work out any presat_off value */
  /* ***************************** */
  rewind(fp);
  ret=fscanf(fp,"%lf\t%lf\n",&pss1,&pwr1);
  rewind(fp);
  while(1){
    ret=fscanf(fp,"%lf\t%lf\n",&pss2,&pwr2);
    if(ret==EOF){
      sprintf(err_string,"%s: presat_off out of range of tpwr_file\n",seqfil);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
    if(pss2>presat_off){
      target=pwr1;
      pwr2-=pwr1;
      pwr2*=((presat_off-pss1)/(pss2-pss1));
      target+=pwr2;
      break;
    }
    pss1=pss2;
    pwr1=pwr2;
  }
  /* calculate db increase */
  dbi=20*log10(target);
  db_presat_off=ceil(dbi);
  newtarg=pow(10,(double)db_presat_off/20);
  newtarg=target/newtarg;
  presat_off_pwr_mod=(int)(newtarg*32767);
  
  /* ******************************* */
  /* next work out any tag_off value */
  /* ******************************* */
  rewind(fp);
  ret=fscanf(fp,"%lf\t%lf\n",&pss1,&pwr1);
  rewind(fp);
  while(1){
    ret=fscanf(fp,"%lf\t%lf\n",&pss2,&pwr2);
    if(ret==EOF){
      sprintf(err_string,"%s: tag_off out of range of tpwr_file\n",seqfil);
      printf(err_string);
      log_error(err_string);
      abort(1);
    }
    if(pss2>tag_off){
      target=pwr1;
      pwr2-=pwr1;
      pwr2*=((tag_off-pss1)/(pss2-pss1));
      target+=pwr2;
      break;
    }
    pss1=pss2;
    pwr1=pwr2;
  }
  /* calculate db increase */
  dbi=20*log10(target);
  db_tag_off=ceil(dbi);
  newtarg=pow(10,(double)db_tag_off/20);
  newtarg=target/newtarg;
  tag_off_pwr_mod=(int)(newtarg*32767);
  
  fclose(fp);
  
  sprintf(full_fname,"%s/vnmrsys/tablib/tpwr_table",home_dir);
  table=fopen(full_fname,"w");
  if(table==NULL){
    sprintf(err_string,"%s: Unable to open ~/vnmrsys/tablib/tpwr_table\n",
	    seqfil);
    printf(err_string);
    log_error(err_string);
    abort(1);
  }
  fprintf(table,"\nt58 = ");  /* correction to nearest dB */
    fprintf(table,"%d ",db_presat_off+(int)tpwr6);
    fprintf(table,"%d ",db_tag_off+(int)tpwr7);
  fprintf(table,"\nt59 = ");  /* fine correction using tpwrf */
    fprintf(table,"%d ",presat_off_pwr_mod);
    fprintf(table,"%d ",tag_off_pwr_mod);
  fclose(table);
  
  loadtable("tpwr_table");
}

/* Playout */

macro_flow_suppress_gradients()
{
  obl_shapedgradient("","","gflow",pw_flow,0.0,0.0,gflow,1,WAIT);
  obl_shapedgradient("","","gflow",pw_flow,0.0,0.0,-gflow,1,WAIT);
}

macro_flow_suppress_gradients_se()
{
  obl_shapedgradient("","","gflow",pw_flow,0.0,0.0,gflow,1,WAIT);
}

tag_inversion_pulse()
{
  /* uses p5 : tpwr5 : p5pat : gss5 */
  if(!strncmp(p5pat,"sech",4))obspower(tpwr5);
  else{
    if(rfmod[0]=='y'){
      getelem(t58,two,v12);  /* t58 contains the coarse power settings */
      power(v12,OBSch);      /* element "two" is the tag_off val */
      getelem(t59,two,v12);  /* t59 contains the fine power settings */ 
    }
    else obspower(tpwr5);
  }

  /* 
     tag_direction=0: frequency=resto, non slice selective
     tag_direction=1: frequency=tag_off, slice selective
     tag_direction=-1: frequency=control_tag_off, slice selective
     tag_direction=2: frequency=tag_off - mean_pss, non slice selective
     tag_direction=-2: frequency=control_tag_off, non slice selective
  */
  
  if (fabs(tag_direction) > 0) {
    if (tag_direction == 1) {  /* tag in QUIPSSII and control in FAIR */
      position_offset(tag_off*tag_sign,gss5,resto,OBSch);
    } else if(tag_direction == 2) { /* control in QUIPSSII */
      position_offset((tag_off-mean_pss),gss5,resto,OBSch);
    } else if(tag_direction < 0) {  /* control in EPISTAR */
      position_offset(control_tag_off,gss5,resto,OBSch);
    }
  } else {
    poffset(0.0,0.0);  /* tag in FAIR: non slice selective */
  }
  
  if ((tag_direction == 0) || (tag_direction) == 2) { 
    /* non slice selective, dummy gradients */ 
    delay(fabs(gss5)*grate); 
    if(!strncmp(p5pat,"sech",4)) {
      sp4on();
      shaped_pulse(p5pat,p5,v1,rof1,rof2);
      sp4off();
    }
    else {
      if(rfmod[0]=='y') {
        sp4on();
        shapedvpulse(p5pat,p5,v12,v1,rof1,rof2);
        sp4off();
      }
      else {
        sp4on();
        shaped_pulse(p5pat,p5,v1,rof1,rof2);
        sp4off();
      }
    }
    delay(fabs(gss5)*grate);   
    if (tag_plane[0]=='r') {
      obl_shapedgradient("gss5","","",pw_gss5,gss5*tag_sign,0.0,0.0,1,WAIT);
    } else if (tag_plane[0]=='p') {
      obl_shapedgradient("","gss5","",pw_gss5,0.0,gss5*tag_sign,0.0,1,WAIT);
    } else {
      obl_shapedgradient("","","gss5",pw_gss5,0.0,0.0,gss5*tag_sign,1,WAIT);
    }

  } else {                            /* slice selective, gradients */
    if (tag_plane[0]=='r') {
      obl_shapedgradient("gss5","","",pw_gss5,gss5*tag_sign,0.0,0.0,1,NOWAIT);
    } else if (tag_plane[0]=='p') {
      obl_shapedgradient("","gss5","",pw_gss5,0.0,gss5*tag_sign,0.0,1,NOWAIT);
    } else {
      obl_shapedgradient("","","gss5",pw_gss5,0.0,0.0,gss5*tag_sign,1,NOWAIT);
    }
    delay(fabs(gss5)*grate+rftep);
    if(!strncmp(p5pat,"sech",4)) {
      sp4on();
      shaped_pulse(p5pat,p5,v1,rof1,rof2);
      sp4off();
    }
    else {
      if(rfmod[0]=='y') {
        sp4on();
        shapedvpulse(p5pat,p5,v12,v1,rof1,rof2);
        sp4off();
      }
      else {
        sp4on();
        shaped_pulse(p5pat,p5,v1,rof1,rof2);
        sp4off();
      }
    }
    if(rftep<fabs(gss5)*grate)delay(fabs(gss5)*grate-rftep);        
    delay(fabs(gss5)*grate+p5+rof1+rof2+fabs(gss5)*grate);
  }

  /* Crushers */
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
    crusher_gradients();
  }  
}

foci_inversion_pulse()
{
  /* uses p5 : tpwr5 : p5pat : gss5 */
  if(!strncmp(p5pat,"sech",4))obspower(tpwr5);
  else{
    if(rfmod[0]=='y'){
      getelem(t58,two,v12);  /* t58 contains the coarse power settings */
      power(v12,OBSch);      /* element "two" is the tag_off val */
      getelem(t59,two,v12);  /* t59 contains the fine power settings */
    }
    else obspower(tpwr5);
  }

  /*
     tag_direction=0: frequency=resto, non slice selective
     tag_direction=1: frequency=tag_off, slice selective
     tag_direction=-1: frequency=control_tag_off, slice selective
     tag_direction=2: frequency=tag_off - mean_pss, non slice selective
     tag_direction=-2: frequency=control_tag_off, non slice selective
  */
 
  if (fabs(tag_direction) > 0) {
    if (tag_direction == 1) {  /* tag in QUIPSSII and control in FAIR */
      position_offset(tag_off*tag_sign,gss5,resto,OBSch);
    } else if(tag_direction == 2) { /* control in QUIPSSII */
      position_offset(tag_off-(mean_pss),gss5,resto,OBSch);
    } else if(tag_direction < 0) {  /* control in EPISTAR */
      position_offset(control_tag_off,gss5,resto,OBSch);
    }
  } else {
    poffset(0.0,0.0);  /* tag in FAIR: non selective */
  }

  if ((tag_direction == 0) || (tag_direction) == 2) {
    /* non slice selective, dummy gradients */
    delay(fabs(gss5)*grate);
    if(!strncmp(p5pat,"sech",4)) {
      sp4on();
      shaped_pulse(p5pat,p5,v1,rof1,rof2);
      sp4off();
    }
    else {
      if(rfmod[0]=='y') {
        sp4on();
        shapedvpulse(p5pat,p5,v12,v1,rof1,rof2);
        sp4off();
      }
      else {
        sp4on();
        shaped_pulse(p5pat,p5,v1,rof1,rof2);
        sp4off();
      }
    }
    delay(fabs(gss5)*grate);

    if (tag_plane[0]=='r') {
      obl_shapedgradient("gss5r_up","","",pw_gss5_r,gss5,0.0,0.0,1,WAIT);
      obl_shapedgradient("focic","","",pw_gss5,gss5,0.0,0.0,1,WAIT);
      obl_shapedgradient("gss5r_dn","","",pw_gss5_r,gss5,0.0,0.0,1,WAIT);
    } else if (tag_plane[0]=='p') {
      obl_shapedgradient("","gss5r_up","",pw_gss5_r,gss5,0.0,0.0,1,WAIT);
      obl_shapedgradient("","focic","",pw_gss5,gss5,0.0,0.0,1,WAIT);
      obl_shapedgradient("","gss5r_up","",pw_gss5_r,gss5,0.0,0.0,1,WAIT);
    } else {
      obl_shapedgradient("","","gss5r_up",pw_gss5_r,gss5,0.0,0.0,1,WAIT);
      obl_shapedgradient("","","focic",pw_gss5,gss5,0.0,0.0,1,WAIT);
      obl_shapedgradient("","","gss5r_up",pw_gss5_r,gss5,0.0,0.0,1,WAIT);
    }
  } else {  
    /* slice selective, gradients */
    if (tag_plane[0]=='r') {
      obl_shapedgradient("gss5r_up","","",pw_gss5_r,gss5,0.0,0.0,1,WAIT);
      obl_shapedgradient("focic","","",pw_gss5,gss5,0.0,0.0,1,NOWAIT);
      if(!strncmp(p5pat,"sech",4)) {
        sp4on();
        shaped_pulse(p5pat,p5,v1,rof1,rof2);
        sp4off();
      }
      else {
        if(rfmod[0]=='y') {
          sp4on();
          shapedvpulse(p5pat,p5,v12,v1,rof1,rof2);
          sp4off();
        }
        else {
          sp4on();
          shaped_pulse(p5pat,p5,v1,rof1,rof2);
          sp4off();
        }
      }
      obl_shapedgradient("gss5r_dn","","",pw_gss5_r,gss5,0.0,0.0,1,WAIT);
   } else if (tag_plane[0]=='p') {
      obl_shapedgradient("","gss5r_up","",pw_gss5_r,gss5,0.0,0.0,1,WAIT);
      obl_shapedgradient("","focic","",pw_gss5,gss5,0.0,0.0,1,NOWAIT);
      if(!strncmp(p5pat,"sech",4)) {
        sp4on();
        shaped_pulse(p5pat,p5,v1,rof1,rof2);
        sp4off();
      }
      else {
        if(rfmod[0]=='y') {
          sp4on();
          shapedvpulse(p5pat,p5,v12,v1,rof1,rof2);
          sp4off();
        }
        else {
          sp4on();
          shaped_pulse(p5pat,p5,v1,rof1,rof2);
          sp4off();
        }
      }
      obl_shapedgradient("","gss5r_up","",pw_gss5_r,gss5,0.0,0.0,1,WAIT);
    } else {
      obl_shapedgradient("","","gss5r_up",pw_gss5_r,gss5,0.0,0.0,1,WAIT);
      obl_shapedgradient("","","focic",pw_gss5,gss5,0.0,0.0,1,NOWAIT);
      if(!strncmp(p5pat,"sech",4)) {
        sp4on();
        shaped_pulse(p5pat,p5,v1,rof1,rof2);
        sp4off();
      }
      else {
        if(rfmod[0]=='y') {
          sp4on();
          shapedvpulse(p5pat,p5,v12,v1,rof1,rof2);
          sp4off();
        }
        else {
          sp4on();
          shaped_pulse(p5pat,p5,v1,rof1,rof2);
          sp4off();
        }
      }
      obl_shapedgradient("","","gss5r_up",pw_gss5_r,gss5,0.0,0.0,1,WAIT);
    }              
    delay(fabs(gss5)*grate+p5+rof1+rof2+fabs(gss5)*grate);
  }

  /* Crushers */
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
    crusher_gradients();
  } 
}

tag_saturation_pulse()
{
  /* uses p6 : tpwr6 : p6pat : gss6 */
  if(!strncmp(p6pat,"sech",4))obspower(tpwr6);
  else{
    if(rfmod[0]=='y'){
      getelem(t58,two,v12);  /* t58 contains the coarse power settings */
      power(v12,OBSch);      /* element "two" is the tag_off val */
      getelem(t59,two,v12);  /* t59 contains the fine power settings */ 
    }
    else obspower(tpwr6);
  }
  
  position_offset(tag_off,gss6,resto,OBSch);

  if (tag_plane[0]=='r') {
    obl_shapedgradient("gss6","","",pw_gss6,gss6,0.0,0.0,1,NOWAIT);
  } else if (tag_plane[0]=='p') {
    obl_shapedgradient("","gss6","",pw_gss6,0.0,gss6,0.0,1,NOWAIT);
  } else {
    obl_shapedgradient("","","gss6",pw_gss6,0.0,0.0,gss6,1,NOWAIT);
  }
  delay(fabs(gss6)*grate+rftep);
  if(!strncmp(p6pat,"sech",4)) {
    sp4on();
    shaped_pulse(p6pat,p6,v1,rof1,rof2);
    sp4off();
  }
  else {
    if(rfmod[0]=='y') {
      sp4on();
      shapedvpulse(p6pat,p6,v12,v1,rof1,rof2);
      sp4off();
    }
    else {
      sp4on();
      shaped_pulse(p6pat,p6,v1,rof1,rof2);
      sp4off();
    }
  }
  if(rftep<fabs(gss6)*grate)delay(fabs(gss6)*grate-rftep);        

  /* Crushers */
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
    crusher_gradients();
  }  
}

tag_saturation_pulse1()
{
  /* uses p6 : tpwr6 : p6pat : gss6 */
  if(!strncmp(p6pat,"sech",4))obspower(tpwr6);
  else{
    if(rfmod[0]=='y'){
      getelem(t58,two,v12);  /* t58 contains the coarse power settings */
      power(v12,OBSch);      /* element "two" is the tag_off val */
      getelem(t59,two,v12);  /* t59 contains the fine power settings */ 
    }
    else obspower(tpwr6);
  }
  
  position_offset(tag_off,gss6,resto,OBSch);

  if (tag_plane[0]=='r') {
    obl_shapedgradient("gss6","","",pw_gss6,gss6,0.0,0.0,1,NOWAIT);
  } else if (tag_plane[0]=='p') {
    obl_shapedgradient("","gss6","",pw_gss6,0.0,gss6,0.0,1,NOWAIT);
  } else {
    obl_shapedgradient("","","gss6",pw_gss6,0.0,0.0,gss6,1,NOWAIT);
  }
  delay(fabs(gss6)*grate+rftep);
  if(!strncmp(p6pat,"sech",4)) {
    sp4on();
    shaped_pulse(p6pat,p6,v1,rof1,rof2);
    sp4off();
  }
  else {
    if(rfmod[0]=='y') {
      sp4on();
      shapedvpulse(p6pat,p6,v12,v1,rof1,rof2);
      sp4off();
    }
    else {
      sp4on();
      shaped_pulse(p6pat,p6,v1,rof1,rof2);
      sp4off();
    }
  }
  if(rftep<fabs(gss6)*grate)delay(fabs(gss6)*grate-rftep);        

  /* Crushers */
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
    obl_shapedgradient("gcr_up","","gcr_up",pw_gcrush,
                       gcrush,0.0,gcrush,1,WAIT);
    delay(tcrush-pw_gcrush);
    obl_shapedgradient("gcr_dn","","gcr_dn",
                       pw_gcrush,gcrush,0.0,gcrush,1,WAIT);
  }
}

tag_saturation_pulse2()
{
  /* uses p6 : tpwr6 : p6pat : gss6 */
  if(!strncmp(p6pat,"sech",4))obspower(tpwr6);
  else{
    if(rfmod[0]=='y'){
      getelem(t58,two,v12);  /* t58 contains the coarse power settings */
      power(v12,OBSch);      /* element "two" is the tag_off val */
      getelem(t59,two,v12);  /* t59 contains the fine power settings */ 
    }
    else obspower(tpwr6);
  }
  
  position_offset(tag_off,gss6,resto,OBSch);

  if (tag_plane[0]=='r') {
    obl_shapedgradient("gss6","","",pw_gss6,gss6,0.0,0.0,1,NOWAIT);
  } else if (tag_plane[0]=='p') {
    obl_shapedgradient("","gss6","",pw_gss6,0.0,gss6,0.0,1,NOWAIT);
  } else {
    obl_shapedgradient("","","gss6",pw_gss6,0.0,0.0,gss6,1,NOWAIT);
  }
  delay(fabs(gss6)*grate+rftep);
  if(!strncmp(p6pat,"sech",4)) { 
    sp4on();
    shaped_pulse(p6pat,p6,v1,rof1,rof2);
    sp4off();
  }
  else {
    if(rfmod[0]=='y') {
      sp4on();
      shapedvpulse(p6pat,p6,v12,v1,rof1,rof2);
      sp4off();
    }
    else {
      sp4on();
      shaped_pulse(p6pat,p6,v1,rof1,rof2);
      sp4off();
    }
  }
  if(rftep<fabs(gss6)*grate)delay(fabs(gss6)*grate-rftep);        

  /* Crushers */
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
    obl_shapedgradient("gcr_up","gcr_up","",pw_gcrush,
                       gcrush,gcrush,0.0,1,WAIT);
    delay(tcrush-pw_gcrush);
    obl_shapedgradient("gcr_dn","gcr_dn","",
                       pw_gcrush,gcrush,gcrush,0.0,1,WAIT);
  }
}

tag_saturation_pulse3()
{
  /* uses p6 : tpwr6 : p6pat : gss6 */
  if(!strncmp(p6pat,"sech",4))obspower(tpwr6);
  else{
    if(rfmod[0]=='y'){
      getelem(t58,two,v12);  /* t58 contains the coarse power settings */
      power(v12,OBSch);      /* element "two" is the tag_off val */
      getelem(t59,two,v12);  /* t59 contains the fine power settings */ 
    }
    else obspower(tpwr6);
  }
  
  position_offset(tag_off,gss6,resto,OBSch);

  if (tag_plane[0]=='r') {
    obl_shapedgradient("gss6","","",pw_gss6,gss6,0.0,0.0,1,NOWAIT);
  } else if (tag_plane[0]=='p') {
    obl_shapedgradient("","gss6","",pw_gss6,0.0,gss6,0.0,1,NOWAIT);
  } else {
    obl_shapedgradient("","","gss6",pw_gss6,0.0,0.0,gss6,1,NOWAIT);
  }
  delay(fabs(gss6)*grate+rftep);
  if(!strncmp(p6pat,"sech",4)) {
    sp4on();
    shaped_pulse(p6pat,p6,v1,rof1,rof2);
    sp4off();
  }
  else {
    if(rfmod[0]=='y') {
      sp4on();
      shapedvpulse(p6pat,p6,v12,v1,rof1,rof2);
      sp4off();
    }
    else {
      sp4on();
      shaped_pulse(p6pat,p6,v1,rof1,rof2);
      sp4off();
    }
  }
  if(rftep<fabs(gss6)*grate)delay(fabs(gss6)*grate-rftep);        

  /* Crushers */
  if ((fabs(gcrush) > 0.0) && (tcrush > 0.0)) {
    obl_shapedgradient("","gcr_up","gcr_up",pw_gcrush,
                       0.0,gcrush,gcrush,1,WAIT);
    delay(tcrush-pw_gcrush);
    obl_shapedgradient("","gcr_dn","gcr_dn",
                       pw_gcrush,0.0,gcrush,gcrush,1,WAIT);
  }
}
