#ifndef LINT
static char SCCSid[] = "@(#)episs.c 13.1 10/10/97 Copyright (c) 1991-1995 Varian Assoc.,Inc. All Rights Reserved";
#endif
/* 
 * Varian Assoc.,Inc. All Rights Reserved.
 * This software contains proprietary and confidential
 * information of Varian Assoc., Inc. and its contributors.
 * Use, disclosure and reproduction is prohibited without
 * prior consent.
 */
/***********************************************************************
    EPISS.C

    Blipped spin/gradient echo-planar imaging sequence.
    Simplified for single shot epi
    


*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
     _                       __                                 __     _
    |                       |                                     |     |
RF  | -----[alpha]----------|---------------- |** ACQ **|---------|     |
    |                       |                                     |     |
    |          < - - - - - -|- -esp - - - - - - - >               |     |
    |        ____           |                                     |     |
GSS | ______/////\     _____|_____________________________________|     |
    | <---->      \\\\/ gssr|                                     |     |
    |   predelay  <--> tref |                                     | etl | ns
    |                       |                                     |     |
    |                       |                                     |     |
GPE |  _____________________|______________/\_____________________|     |
    |                       |              \/                     |     |
    |                       |                                     |     |
    |         gror          |                  _________          |     |
GRO |  ___________     _____|_________________//////////\_________|     |
    |             \\\\/     |                                     |     |
    |_                      |__                                 __|    _|

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

#include "standard.h"
#include "group.h"

pulsesequence()
{
    /* INTERNAL VARIABLE DECLARATIONS *********************/
    int     image;
    double  sedelay,etl,predelay,seqtime;
    double  espmin,tblipc;
    double  agss,agro,gpes,grate,gssrint,grorint,gpeint;
    double  nucgamma;
    double  grora;
    double  eff_echo,gpeb,rgss,rgro,rgpes,rgpeb,rgssr,rgror;
    double  ted,ted1,ted2;
    double  tep,rftep,epigroa,epigrof;

    double tssr,trod,tpedc;
    double groa, gro,agror,agssr,agpes,agpeb;
    double invtime;
    double tpwrfat, pfat;
    char   spinecho[MAXSTR],fat_sat[MAXSTR],pfatpat[MAXSTR];

    initparms_sis();
    tpedc = getval("tped");     /* phase encode dephaser */
    tblipc = getval("tblip");   /* blipped gradient time, not used */
    eff_echo = getval("eff_echo"); /* echo train, maximum signal */
    getstr("spinecho",spinecho);  /* spin echo/gradient echo flag */
    image = getval("image");      /* reference scan/image flag */

    /* fat sat stuff */
    getstr("fat_sat",fat_sat);    /* fat saturation flag 'y'=on. 'n'=off. */
    getstr("pfatpat",pfatpat);    /* modulation pattern for fat sat pulse */
    tpwrfat = getval("tpwrfat");  /* fat sat power in dB */
    pfat = getval("pfat");        /* duration of fat sat pulse in us */

    grate = trise/gmax;
    agss = fabs(gss);
    trod = 0;   /* time for read out dephaser */
    etl = nv;   /* echo train length */

    nucgamma = 1e6*sfrq/B0;
    gro = np/2.0/at/lro/nucgamma;  /* note read gradient calculated here */
    gror = gro;    /* read refocussing(dephasing) lobe */
    agro = fabs(gro);
    agror = fabs(gror);

    rgro = grate*agro;
    rgror = grate*agror;
    rgss = grate*agss;

    /* get the global parameters */
    /*************************************************************************/
    P_getreal(GLOBAL,"tep",&tep,1); /* small tweaker time (us) to allow for
                            the gradient propagation time, causes echoes to
                            appear symmetrically about echo center line */
    tep=tep/1e6;
    P_getreal(GLOBAL,"rftep",&rftep,1); /* small tweaker time (us) to allow for
                            the gradient propagation time, w.r.t. RF pulses */
    rftep=rftep/1e6;
    epigroa = 0.0; /* default */
    P_getreal(GLOBAL,"epigroa",&epigroa,1); /* read dephase tweaker (percent) */
    groa=gro*epigroa/100.0;
    epigrof = 0.0; /* default */
    P_getreal(GLOBAL,"epigrof",&epigrof,1); /* grad risetime to full strength */
    grora=gror*epigrof/100.0;
    /*************************************************************************/


  /* GRADIENT*TIME INTEGRALS FOR SLICE SELECTION, PHASE AND READOUT */
    grorint = agro*(at/2.0 + rgro/2.0);
    gssrint = agss*(p1/2.0 + rof2 + rgss/2.0);
    gpeint = 1 / (lpe*nucgamma);

    trod = grorint/agror;   /* assume constant gradient */
    trod = trod - (rgror);  /* correct for ramps */

    gssr = gss;
    gssr = gssr*gssf;
    agssr = fabs(gssr);
    rgssr = grate*agssr;

    tssr = gssrint/agssr; /* slice refocussing time for constant pulse */
    tssr = tssr - rgssr;  /* correct for ramps */

    tblipc = rgro;        /* phase encode blip during readout ramp (not tblip */
    gpes = gpeint/tblipc;   /* for constant pulse */
    agpes = fabs(gpes);
    rgpes = grate*agpes;
    tblipc = tblipc - rgpes;  /* correct time for ramps */
    
    gpeb = gpeint*eff_echo/tpedc;  /* assume constant pulse */
    agpeb = fabs(gpeb);
    rgpeb = grate*agpeb;
    tpedc = tpedc-rgpeb;   /* correct timing for ramps */

  /* CALCULATE PHASE REFOCUSSING TO CORRECT FOR FREQUENCY JUMPS */

  /* RELAXATION DELAY */
    invtime = ti + rgss*2 + tspoil*2 + p2;
    espmin = rgro + at + tblipc + 2*rgpes;  
    seqtime = rgss + rof1 + p1 + rof2 + 
	        rgssr*2 + tssr + 
	        rgpeb*2 + tpedc +
	        rgror*2 + trod + 
		espmin*etl;
    if (spinecho[0] == 'y') 
    {
        /* second half echo time */
        ted2 = (espmin*eff_echo) + tspoil + p2/2.0 + rof2 + rgss;
        /* first half, event time */
        ted1 = p1/2.0 + rof2 + rgss*2 + tssr + rgssr*2 +
                  rgpeb*2 + tpedc + trod + rgro*2; 
        ted = ted2 - ted1;  /* buffer delay if spin echo */
	sedelay = ted + rof1 + p2 + rof2 + (rgss*2) + (tspoil*2); 
	seqtime = seqtime + sedelay;
    }
    else
	ted = 0;
    if (ir[0] == 'y') 
	seqtime = seqtime + invtime;
    if (fat_sat[0] == 'y')
        seqtime = seqtime + pfat + grate*fabs(gspoil)*2.0 + tspoil;

    if (tr - seqtime < 0.0) {
        printf("SS_EPI: Requested tr too short.  Min tr = %f\n",seqtime);
        abort(1);
    }
    predelay = (tr - ns*seqtime)/ns;

    if (image == 4)
    {
      gpeb = 0.0;
      gpes = 0.0;
    }
    if (spinecho[0] != 'y')
    {
	gpeb = -gpeb;
	gror = -gror;
        grora = -grora;
    }
    
/***
        printf("gro: %7.4f gss: %7.4f gpes: %7.4f\n",gro,gss,agpes);
        printf("gror: %7.4f gssr: %7.4f gpeb: %7.4f\n",gror,gssr,agpeb);
        printf("trod: %7.4f tssr: %7.4f tpe: %f \n",trod,tssr,tpe);
***/

  /* PULSE SEQUENCE *************************************/
    /* Relaxation delay ***********************************/       
    status(A);
    sp4off();

    /* Phase cycle: Part I ********************************/
    mod2(ct,v3);                    /*0101 0101*/
    dbl(v3,v3);                     /*0202 0202*/
    hlv(ct,v4);                     /*0011 2233*/
    mod2(v4,v4);                    /*0011 0011*/
    add(v3,v4,v1);                  /*0213 0213*/
    assign(v1,oph);
    add(one,v1,v2);

      assign(zero,v12);
      initval(etl,v11);

      msloop(seqcon[1],ns,v7,v8);
        assign(zero,v10);

        /* Gating ***************************************/
        xgate(ticks);

        delay(predelay);

        if (fat_sat[0] == 'y') {
          obspower(tpwrfat);
          /* move the centre freq to fat peak */
          obsoffset(resto-3.45*sfrq); 
          sp4on();
          shapedpulse(pfatpat,pfat,v1,rof1,rof2);
          sp4off();
          obl_gradient(gspoil,gspoil,0.0);
          delay(grate*fabs(gspoil)+tspoil);
          zero_all_gradients();
          delay(grate*fabs(gspoil));
          obsoffset(resto);  /* reset centre frequency back to water */
        }

        poffset_list(pss,gss,ns,v8);

	if (ir[0] == 'y')
	{
        	/* 180 degree RF pulse ********************************/
        	obspower(tpwr2);
        	obl_gradient(0.0,0.0,gss);
        	delay(rgss);
		delay(tspoil);
                sp4on();
        	shapedpulse(p2pat,p2,v2,rof1,rof2);
                sp4off();
        	delay(tspoil);
        	zero_all_gradients();
        	delay(rgss);
		delay(ti-tspoil-rgss-p2/2);
	}
       
        /* 90 degree pulse ************************************/ 
        observepower(tpwr1);
        obl_gradient(0.0,0.0,gss);
        delay(rgss);
        sp4on();
        shapedpulse(p1pat,p1,v1,rof1,rof2);
        sp4off();
        zero_all_gradients();
	delay(rgss);

        /* Read & Slice refocus and phase encode pulse ********/
        obl_gradient(0.0,0.0,-gssr);
        delay(rgssr);
	delay(tssr);
        zero_all_gradients();
        delay(rgssr);

        obl_gradient(0.0,-gpeb,0.0);
        delay(rgpeb);
	delay(tpedc);
        zero_all_gradients();
        delay(rgpeb);

        obl_gradient(gror+grora,0.0,0.0);
        delay(rgror);
	delay(trod);
        zero_all_gradients();
        delay(rgror);

	if (spinecho[0] == 'y')
	{
		delay(ted);
        	/* 180 degree RF pulse ********************************/
        	obspower(tpwr2);
        	obl_gradient(0.0,0.0,gss);
        	delay(rgss);
		delay(tspoil);
                sp4on();
        	shapedpulse(p2pat,p2,v2,rof1,rof2);
                sp4off();
        	delay(tspoil);
        	zero_all_gradients();
        	delay(rgss);
	}

        obsoffset(resto);

        peloop(seqcon[2],etl,v5,v6);
	  incr(v10);
          mod2(v10,v12);
         /* Acquire echo ***************************************/
          ifzero(v12);
	    /* poffset(pro,-gro); */
            obl_gradient(-gro,0.0,0.0);
	    delay(rgro);
          elsenz(v12);
	    /* poffset(pro,gro+groa); */
            obl_gradient(gro+groa,0.0,0.0);
	    delay(rgro);
          endif(v12);

          delay(tep);
          acquire(np,1.0/sw);

	  /* phase encode blip, triangular */
          obl_gradient(0.0,-gpes,0.0);
          delay(rgpes);
          delay(tblipc);    /* rgro = tblipc+rgpes */
          zero_all_gradients();
          delay(rgpes);

        endpeloop(seqcon[2],v6);
      endmsloop(seqcon[1],v8);
}
/***********************************************************************
                         MODIFICATION HISTORY

970320(ss): Single shot, spin echo or gradient echo version
970327(ss): tblip included in readout ramp. Timing corrected.
970327(ss): inversion pulse added
970509(ss): Renamed, episs.c. tev, tod replaced by tep.
            groa and grora adjusters added.
970717(ss): Blipped gradient moved to end of acquisition.
            d1 removed. tep,tev,tod removed.
            Bug in groa use corrected. Bugs in timing fixed.
            Position offset, pro, implemented. gpe/gro sign reversed.
970729(ss): tep delay included for gradient propagation delay.
            

***********************************************************************/
