char *ecc_setup_sccsID(){
    return "@(#)ecc_setup.c 1.1 04/17/98 Copyright(c)1997-98 Varian";
}
/* 
 * Varian Assoc.,Inc. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Varian Assoc., Inc. and its contributors.  Use, disclosure and
 * reproduction is prohibited without prior consent.
 */

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <sys/param.h>

#include "vnmrsys.h"
#include "acodes.h"
#include "acqparms.h"
#include "group.h"

/* These AP defines are also in oopc.h, but other stuff there messes us up */
#ifndef APSELECT
#define APSELECT 0xA000
#define APWRITE 0xB000
#endif

#ifndef TRUE
#define TRUE (0==0)
#define FALSE (!TRUE)
#endif

#define DECCLIB "imaging/decclib" /* DECC directory, relative to "/vnmr" */

/* Stuff about the DECC hardware */
#define DECC 0x968		/* Base AP address of DECC board */
#define DECCWRITE (APWRITE | (DECC & 0xf00))
#define DECC_LOOPTIME (3.92e-6)	/* Update rate of corrections */
/*#define DECC_LOOPTIME (3.6e-6)	/* OLD Update rate of corrections */
#define DSPADDR 0x87ffc0	/* Address inside DSP to load into */
#define DSPBUFLEN 64		/* Number of words to load into DSP */

#define MAXTAUS 10		/* Max ECC time constants per term */
#define N1 3			/* Number of axes affecting ECC;
				 * also, number of ECC DSP chips */
#define N2 4			/* Number of axes corrected;
				 * also, number of term types per chip */
/* SDAC hardware info */
#define SLEWTOP 5000.0		/* mV */
#define SLEWSLOPE 0.2		/* mV / us / dacunit */
#define SLEWOFFSET 1.5		/* mV / us */
#define MAX_ECC_SCALE 0.1	/* Fixed hdwe ECC gain on SDAC board */

#define X1_AXIS 0
#define Y1_AXIS 1
#define Z1_AXIS 2
#define B0_AXIS 3

/*
 * A N1xN2 matrix of Ecc structs holds the time constants and amplitudes
 * of all the ECC values.  The matrix element ecc[0][1] holds the
 * compensation terms for the y-axis (axis 1) due to the x-axis (axis 0).
 * Axes: 0=X1, 1=Y1, 2=Z1, 3=B0 (=Z0)
 */
typedef struct{
    int nterms;
    float tau[MAXTAUS];
    float amp[MAXTAUS];
} Ecc;

typedef struct{
    int src;
    int dst;
} Intpair;

#define POLARITY_ADR 0x0f
#define SDAC_DATAADR 0x960
#define SDAC_ADRADR (SDAC_DATAADR+2)

#define ECCSCALE 0
#define SHIMSCALE 1
#define TOTALSCALE 2
#define SLEWLIMIT 3
#define DUTYLIMIT 4
#define N_SDAC_VALS 5

typedef struct{
    char label[16];
    float max_val;
    float max_ival;
    float scale_ival;
    int funcadr;
    float values[3];
} Sdac;

static Sdac sdac_values[N_SDAC_VALS] = {
    {"eccscale", 0.1, 255, 256, 0x02},
    {"shimscale", 0.05, 255, 256, 0x12},
    {"totalscale", 1.0, 16383, 16384, 0x03},
    {"slewlimit", 0.0035, 255, 256, 0x04},
    {"dutylimit", 1.0, 255, 256, 0x24}
};

/*
 * Return the number of terms for the axis pair (srcaxis, dstaxis).
 */
static int
nterms(int srcaxis, int dstaxis)
{
    int rtn;

    if (srcaxis == dstaxis){
	rtn = 6;
	/*rtn = 5;		/* Old value */
    }else if (dstaxis == B0_AXIS) {
	rtn = 4;
	/*rtn = 3;		/* Old value */
    }else{
	rtn = 3;
    }
    /*fprintf(stderr,"nterms(%d,%d)=%d\n", srcaxis, dstaxis, rtn);/*DBG*/
    return rtn;
}

/*
 * Fill in the default values for an ecc matrix.
 */
static void
init_ecc_matrix(Ecc ecc_matrix[N1][N2])
{
    int i;
    int j;
    int k;

    for (i=0; i<N1; i++){
	for (j=0; j<N2; j++){
	    ecc_matrix[i][j].nterms = nterms(i, j);
	    for (k=0; k<ecc_matrix[i][j].nterms; k++){
		ecc_matrix[i][j].tau[k] = 1;
		ecc_matrix[i][j].amp[k] = 0;
	    }
	}
    }
}

static int
get_sdac_download_val(int type, int axis)
{
    int ival;
    float val;
    Sdac *pd;

    pd = &sdac_values[type];
    if (type == SLEWLIMIT){
	float trise;
	trise = 1e6 * pd->values[axis];
	val = (SLEWTOP - trise * SLEWOFFSET) / (SLEWSLOPE * trise);
    }else if (type == DUTYLIMIT){
	val = sqrt(pd->values[axis] / pd->max_val) * pd->scale_ival;
	}else{
	val = fabs(pd->values[axis] * pd->scale_ival / pd->max_val);
    }
    ival = (int)(val + 0.5);
    if (ival > pd->max_ival){
	ival = pd->max_ival;
    } else if (ival < 0) {
	ival = 0;
    }	
    return ival;
}

static float
get_quantized_sdac_value(int type, int axis)
{
    Sdac *pd;
    int download;
    float rtn;

    if (axis == B0_AXIS) {
	return 1.0;
    }
    if (axis < 0 || axis > B0_AXIS) {
	return 0;
    }
    pd = &sdac_values[type];
    download = get_sdac_download_val(type, axis);
    if (type == SLEWLIMIT) {
	rtn = SLEWTOP / (SLEWOFFSET + SLEWSLOPE * download);
    }else{
	rtn = download * pd->max_val / pd->scale_ival;
	if (pd->values[axis] < 0) {
	    rtn = -rtn;
	}
    }
    return rtn;
}

/*
 * Given the name of an input file, fill a matrix of Ecc structures
 * with the ECC values contained in the file.
 *
 * Format of input file is:
 * # Comment if line starts with "#" or """
 * # termtype	tau1 amp1	tau2 amp2 ...
 * xx		.025  -.03	*.011 .0137	.00278 .00741
 *
 * Termtype xy is ECC for y-axis due to x input
 * Possibilities are: xx xy xz xb0 yx yy yz yb0 zx zy zz zb0
 * where "b0" is the correction sent to the b0 coil.
 * Types are not case sensitive.
 * A "*" prefix to the tau value (as in tau2, above) means to leave
 * out that term from the ECC correction (tau2 and amp2 are both set
 * to zero).
 */
static int
get_decc_from_file(char *filepath, Ecc ecc_matrix[N1][N2])
{
    int i;
    int src;			/* Input axis number */
    int dst;			/* Output axis number */
    int nt;			/* Number of time constants of this type */
    char buf[512];
    char msg[512];
    FILE *fd;
    int disable;
    int rtn = TRUE;

    /* Init matrix values and open input file */
    init_ecc_matrix(ecc_matrix);
    fd = fopen(filepath, "r");
    if (!fd){
	rtn = FALSE;
    }

    if (rtn){
	/* Parse input file */
	while (fgets(buf, sizeof(buf), fd)){
	    /*char *lasts;*/
	    char *tok;
	    /*tok = strtok_r(buf, " \t\n", &lasts);*/
	    tok = strtok(buf, " \t\n");
	    if (tok && *tok != '#' && *tok != '"'){
		if (strcasecmp(tok,"xx") == 0){
		    src = 0; dst = 0;
		}else if (strcasecmp(tok,"xy") == 0){
		    src = 0; dst = 1;
		}else if (strcasecmp(tok,"xz") == 0){
		    src = 0; dst = 2;
		}else if (strcasecmp(tok,"xb0") == 0){
		    src = 0; dst = 3;
		}else if (strcasecmp(tok,"yx") == 0){
		    src = 1; dst = 0;
		}else if (strcasecmp(tok,"yy") == 0){
		    src = 1; dst = 1;
		}else if (strcasecmp(tok,"yz") == 0){
		    src = 1; dst = 2;
		}else if (strcasecmp(tok,"yb0") == 0){
		    src = 1; dst = 3;
		}else if (strcasecmp(tok,"zx") == 0){
		    src = 2; dst = 0;
		}else if (strcasecmp(tok,"zy") == 0){
		    src = 2; dst = 1;
		}else if (strcasecmp(tok,"zz") == 0){
		    src = 2; dst = 2;
		}else if (strcasecmp(tok,"zb0") == 0){
		    src = 2; dst = 3;
		}else{
		    continue;	/* Skip this line */
		}
		nt = ecc_matrix[src][dst].nterms;
		for (i=0; i<nt && (tok=strtok(NULL," \t\n")); i++) {
		    float amp;
		    float eccgain;
		    float srcgain;
		    float dstgain;
		    disable = FALSE;
		    if (*tok == '*') {
			disable = TRUE;
			ecc_matrix[src][dst].tau[i] = 0;
		    } else if (sscanf(tok,"%g",&ecc_matrix[src][dst].tau[i])
			       != 1)
		    {
			sprintf(msg,
				"Bad tau in DECC file \"%.300s\": \"%.99s\"",
				filepath, tok);
			text_error(msg);
			rtn = FALSE;
			break;
		    }
		    /*fprintf(stderr,"tau[%d][%d][%d]=%g, ",
			    src, dst, i, ecc_matrix[src][dst].tau[i]);/*DBG*/
		    tok = strtok(NULL," \t\n");
		    if (!tok){
			sprintf(msg,
				"Missing amplitude in DECC file \"%.300s\"",
				filepath);
			text_error(msg);
			rtn = FALSE;
			break;
		    }
		    if (disable){
			amp = 0;
		    } else if (sscanf(tok,"%g",&amp)
			       != 1)
		    {
			sprintf(msg,
				"Bad amp in DECC file \"%.300s\": \"%.99s\"",
				filepath, tok);
			text_error(msg);
			rtn = FALSE;
			break;
		    }
		    if (dst != B0_AXIS) {
			/* Correct amplitude for different gains on SDAC */
			eccgain = get_quantized_sdac_value(ECCSCALE, dst);
			srcgain = get_quantized_sdac_value(TOTALSCALE, src);
			dstgain = get_quantized_sdac_value(TOTALSCALE, dst);
			if (dstgain != 0 && eccgain != 0) {
			    amp *= srcgain / (dstgain * eccgain);
			}
		    }
		    ecc_matrix[src][dst].amp[i] = amp;
		    /*fprintf(stderr,"amp=%g\n", amp);/*DBG*/
		}
	    }
	}
    }
    return rtn;
}

/*
 * Convert a floating point number in native format to TMS32032
 * floating point single precision (32 bit) format.  Note that the
 * TMS floating point value is returned as an unsigned integer.
 */
static unsigned int
float2tms32(float x)
{
    unsigned int zero = 0x80000000; /* Zero value is special case */
    int nfracbits = 23;		/* Not including hidden bit / sign bit */
    int signbit = 1 << nfracbits;
    int fracmask = ~((~0)<<nfracbits);
    int iexp;
    int sign;
    int ifrac;
    unsigned int rtn;

    if (x == 0){
	rtn = zero;
    }else{
	iexp = ilogb(x);	/* Binary exponent if 1 <= |fraction| < 2 */
	ifrac = (int)scalbn(x, nfracbits-iexp); /* Frac part as integer */
	if (x<0 && (ifrac & signbit)){
	    /* Force top bit of negative fraction to be 0 */
	    ifrac <<= 1;
	    iexp--;
	}
	sign = x<0 ? signbit : 0;
	rtn = (iexp << (nfracbits+1)) | sign | (ifrac & fracmask);
    }
    return rtn;
}

/*
 * Take ECC values (for one output axis) from an ECC matrix, and form
 * a list of 32 bit words for the DSP bootloader.
 * Buffer organization:
 * +------------------------------------------------+ 
 * |        |            |                |         |
 * |        |      downloaded data        |         |
 * | header |            |                | trailer |
 * |        | amplitudes | time constants |         |
 * |        |            |                |         |
 * +------------------------------------------------+  
 * Returns a pointer to the list, and sets "nwords" to the length of
 * the list.
 * NB: The caller must free the memory for the list.
 */
static unsigned int *
make_tms_ecc_download(int axis,	/* Chip # (0-2) */
		      Ecc ecc_matrix[N1][N2],
		      int *nwords)
{
    unsigned int *rtn;
    unsigned int *pui;
    int i, j;
    int src, dst;
    int nt;			/* # terms of this type */
    int nw;			/* Cumulative number of words in block */
    float decay;
    float tau;

    /* The order ECC terms are loaded in each chip */
    Intpair order[N1][N2] = {{{0,0}, {1,0}, {2,0}, {0,3}},
			     {{1,1}, {2,1}, {0,1}, {1,3}},
			     {{2,2}, {0,2}, {1,2}, {2,3}}};
    unsigned int hdr[] = {0x8,	/* Byte-wide physical data */
			  0xe8,	/* 3 global control words */
			  0xf10f8,
			  0x310f8,
			  DSPBUFLEN, /* Nbr of words to download */
			  DSPADDR, /* DSP address of download */
			  0x0};	/* Block control word */
    unsigned int tail[] = {0x0};

    rtn = (unsigned int *)malloc(sizeof(hdr)
				 + DSPBUFLEN * sizeof(int)
				 + sizeof(tail));
    if (!rtn){
	text_error("Out of memory in make_tms_ecc_download()");
	return NULL;
    }

    /* Load header */
    for (i=0; i<sizeof(hdr)/sizeof(int); i++){
	rtn[i] = hdr[i];
    }
    pui = &rtn[i];		/* Next location to load */

    /* Load body of block; load amplitude and tau sections in parallel */
    for (i=nw=0; i<N2; i++){
	src = order[axis][i].src;	/* Source axis */
	dst = order[axis][i].dst;	/* Dest axis */
	nt = ecc_matrix[src][dst].nterms;
	nw += nt * 2;
	if (nw > DSPBUFLEN){
	    /* Block size too small to hold all the terms */
	    fprintf(stderr,
		    "make_tms_ecc_download(): i=%d, nw=%d, DSPBUFLEN=%d\n",
		    i, nw, DSPBUFLEN);
	    text_error("Internal error in make_tms_ecc_download()");
	    free(rtn);
	    rtn = NULL;
	    break;
	}
	for (j=0; j<nt; j++){
	    tau = ecc_matrix[src][dst].tau[j];
	    if (tau){
		decay = exp(-DECC_LOOPTIME / tau);
	    }else{
		decay = 0;
	    }
	    pui[j] = float2tms32(ecc_matrix[src][dst].amp[j]);
	    pui[j + DSPBUFLEN/2] = float2tms32(decay);
	    /*fprintf(stderr,"*tau[%d][%d][%d]=%g (%g), amp=%g\n",
		    src, dst, j,
		    ecc_matrix[src][dst].tau[j], decay,
		    ecc_matrix[src][dst].amp[j]);/*DBG*/
	}
	pui += nt;		/* Next location to load */
    }

    /* Pad block if necessary; pad amplitude and tau sections in parallel */
    for (i=0 ; nw < DSPBUFLEN; i++, nw += 2){
	/*if (!i) {fprintf(stderr,"Pad: ");}/*DBG*/
	/*fprintf(stderr,"%d ", i+1);/*DBG*/
	pui[i] = float2tms32(0.0);
	pui[i + DSPBUFLEN/2] = float2tms32(0.0);
    }
    /*fprintf(stderr,"\n");/*DBG*/
    pui += i + DSPBUFLEN/2;	/* Move pointer to end of tau section */

    /* Load trailer */
    for (i=0; rtn && i<sizeof(tail)/sizeof(int); i++){
	pui[i] = tail[i];
    }

    *nwords = (sizeof(hdr) / sizeof(hdr[0])
	       + DSPBUFLEN
	       + sizeof(tail) / sizeof(tail[0]));

    return rtn;
}

/*
 * Takes three DSP download blocks (one for each chip), and forms an APBOUT
 * string that loads all three chips.
 * Returns a pointer to the APBOUT string.  (The length of the string
 * is encoded into the string.)
 * NB: The caller must free the memory for the string.
 */
static codeint *
make_apbout_from_decc_dspcode(unsigned int *dspcode[], /* 3 DSP code lists */
			      int nwords) /* # words to each register */
{
    int i, j;
    int outlen;			/* Number of codeints to output */
    int headlen;
    int taillen;
    int regorder[] = {3, 2, 1};	/* First download to first reg # in list */
    codeint head[] = {APBOUT,
		      0,			/* Reserve for count-1 */
		      APSELECT | DECC+0,        /* Register 0 */
		      DECCWRITE | 0x00,		/* Reset FIFOs */
		      DECCWRITE | 0x40};	/* Enable FIFOs */
    codeint tail[] = {APSELECT | DECC+0,	/* Register 0 */
		      DECCWRITE | 0x47,		/* Interrupt all 3 DSPs */
		      DECCWRITE | 0x40};	/* DSPs to normal mode */
    codeint *pc;
    codeint *rtn;

    /* Allocate memory */
    headlen = sizeof(head) / sizeof(head[0]);
    taillen = sizeof(tail) / sizeof(tail[0]);
    outlen = headlen + N1 + N1 * nwords * 4 + taillen;
    rtn = (codeint *)malloc(outlen * sizeof(codeint));
    if (!rtn){
	text_error("Out of memory in make_apbout_from_decc_dspcode()");
	return NULL;
    }

    /* Write header */
    pc = rtn;
    head[1] = outlen - 3;
    for (i=0; i<headlen; i++){
	*pc++ = head[i];
    }

    /* Write to the FIFOs. Words go in bytewise, little-endian */
    for (i=0; i<N1; i++){
	*pc++ = APSELECT | DECC + regorder[i];
	for (j=0; j<nwords; j++){
	    *pc++ = DECCWRITE | (dspcode[i][j] & 0xff);
	    *pc++ = DECCWRITE | ((dspcode[i][j] >> 8) & 0xff);
	    *pc++ = DECCWRITE | ((dspcode[i][j] >> 16) & 0xff);
	    *pc++ = DECCWRITE | ((dspcode[i][j] >> 24) & 0xff);
	}
    }

    /* Write tail */
    for (i=0; i<taillen; i++){
	*pc++ = tail[i];
    }

    return rtn;
}

/*
 * Given a file containing ECC values, construct an APBOUT string
 * that loads those values into the DECC board.
 */
static codeint *
make_apbout_from_decc_file(char *filename)
{
    int i;
    int err = FALSE;
    Ecc ecc_matrix[N1][N2];
    unsigned int *dspcode[N1];
    int nwords[N1];
    codeint *apb_string = NULL;

    /* Read the ECC values from master file */
    if (!get_decc_from_file(filename, ecc_matrix)){
	return NULL;
    }

    /* Construct TMS32032 download blocks */
    for (i=0; i<N1; i++){
	dspcode[i] = make_tms_ecc_download(i, ecc_matrix, &nwords[i]);
    }
    if (!nwords[0] || nwords[0] != nwords[1] || nwords[1] != nwords[2]){
	fprintf(stderr,"make_apbout_from_decc_file(): nwords=%d, %d, %d\n",
		nwords[0], nwords[1], nwords[2]);
	text_error("Internal error in make_apbout_from_decc_file()");
	err = TRUE;
    }

    /* Make up an APBOUT string */
    if (!err){
	apb_string = make_apbout_from_decc_dspcode(dspcode, nwords[0]);
    }

    /* Free memory */
    for (i=0; i<N1; i++){
	if (dspcode[i]){
	    free(dspcode[i]);
	}
    }

    return apb_string;
}

/*
 * Load an APBOUT string into the Acode stream
 */
static void
send_apbout_string(codeint *apb)
{
    int len;
    codeint *apend;

    if (apb){
	len = apb[1] + 1;	/* Count word is (length - 1) */
	apend = apb + len + 2;	/* Include 2 hdr words (APBOUT + count) */
	while (apb < apend){
	    /*fprintf(stderr,"0x%04x\n", (int)*apb & 0xffff);/*DBG*/
	    putcode(*apb++);
	}
	curfifocount += len;
    }
}

static void
get_ecc_filename(char *fname, int len)
{
    char *pc;
    char pfx[] = ".";
    int n;

    sprintf(fname,"%s/%s/%s", systemdir, DECCLIB, pfx);
    n = strlen(fname);
    pc = fname + n;
    len -= n;

    if (P_getstring(GLOBAL,"sysgcoil", pc, 1, len))
    {
	strcpy(pc, "main");
    }
    /*fprintf(stderr,"filepath=%s, strlen=%d, name=%s\n", fname, n, pc);/*DBG*/
    return;
}

static void
get_sdac_setting_from_file(FILE *fd, char *label, float values[3])
{
    char buf[512];

    rewind(fd);
    while (fgets(buf, sizeof(buf), fd)){
	if (strncasecmp(buf, label, strlen(label)) == 0){
	    /* This line has the data */
	    sscanf(buf,"%*s %f %f %f", &values[0], &values[1], &values[2]);
	    break;
	}
    }
}

static void
set_sdac_reg(int type, int index, int val)
{
    int addr;

    addr = sdac_values[type].funcadr + 4 * index;
    apset(addr, SDAC_ADRADR);
    apset(val, SDAC_DATAADR);
    apset(0, SDAC_ADRADR);
    /*fprintf(stderr,"%s[%d]: addr=0x%x, val=%d=0x%x\n",
	    sdac_values[type].label, index, addr, val, val);/*DBG*/
}

static int
set_sdac_board(filename)
{
    int i, j;
    int rtn = TRUE;
    int polarity;
    float val;
    FILE *fd;
    Sdac *pd;

    fd = fopen(filename, "r");
    if (!fd){
	rtn = FALSE;
    }

    if (!rtn){
	/* If no file, just turn off gain for safety */
	i = TOTALSCALE;
	for (j=0; j<3; j++){
	    set_sdac_reg(TOTALSCALE, j, 0);
	}
    }else{
	for (i=0, pd=sdac_values; i<N_SDAC_VALS; pd++, i++){
	    for (j=0; j<3; j++){
		if (i == SLEWLIMIT) {
		    pd->values[j] = pd->max_val;
		}else{
		    pd->values[j] = 0;
		}
	    }
	    get_sdac_setting_from_file(fd, pd->label, pd->values);
	    /* Set values over APbus */
	    if (i == TOTALSCALE){
		/* Set appropriate polarity bits depending on signs */
		polarity = ((pd->values[2] >= 0)
			    | ((pd->values[1] >= 0) << 1)
			    | ((pd->values[0] >= 0) << 2));
		apset(POLARITY_ADR, SDAC_ADRADR);
		apset(polarity, SDAC_DATAADR);
		apset(0, SDAC_ADRADR);
		/*fprintf(stderr,"Polarity = 0x%x\n", polarity);/*DBG*/
	    }
	    for (j=0; j<3; j++){
		set_sdac_reg(i, j, get_sdac_download_val(i, j));
	    }
	}
	close(fd);
    }

    return rtn;
}

/*
 * Load the ECC values in the master ECC file into the DECC board
 * by putting the values directly into the Acode stream.
 * If there is no master file, the overall gains on the SDAC board
 * are set to zero.
 * Returns TRUE on success, FALSE on failure (probably because of
 * no master file).
 */
int
download_master_decc_values()
{
    char filename[MAXPATHLEN+1];
    codeint *apbout;
    int rtn = FALSE;

    get_ecc_filename(&filename, sizeof(filename));

    /* Set SDAC board */
    set_sdac_board(filename);

    /* Set DECC board */
    apbout = make_apbout_from_decc_file(filename);
    send_apbout_string(apbout);
    if (apbout){
	free(apbout);
	rtn = TRUE;
    }

    /*fprintf(stderr,"download_master_decc_values() V2: Done\n");/*DBG*/
    return rtn;
}
